// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class AccessControlSetupForm : Form
    {
        public AmtSecurityAdmin.AmtUserAcl[] aclArray;
        public string AdminName = null;
        private AmtSystem system;

        public AccessControlSetupForm(AmtSecurityAdmin.AmtUserAcl[] aclArray, AmtSystem system)
        {
            this.system = system;
            this.aclArray = aclArray;
            InitializeComponent();
        }

        private void removeButton_Click(object sender, EventArgs e)
        {
            if (accountsListView.SelectedItems.Count == 0) return;
            accountsListView.Items.Remove(accountsListView.SelectedItems[0]);
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            ArrayList acls = new ArrayList();
            foreach (ListViewItem l in accountsListView.Items)
            {
                acls.Add((AmtSecurityAdmin.AmtUserAcl)l.Tag);
            }
            //securityTemplate.UserAclEntries = (AmtSecurityAdmin.AmtUserAcl[])acls.ToArray(typeof(AmtSecurityAdmin.AmtUserAcl));
            aclArray = (AmtSecurityAdmin.AmtUserAcl[])acls.ToArray(typeof(AmtSecurityAdmin.AmtUserAcl));
            DialogResult = DialogResult.OK;
        }


        private void AccessControlTemplateSetupForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            
            if (aclArray != null)
            {
                foreach (AmtSecurityAdmin.AmtUserAcl acl in aclArray)
                {
                    ListViewItem l;
                    if (acl.Type == AmtSecurityAdmin.AmtUserAclType.Digest)
                    {
                        l = new ListViewItem(new string[] { acl.Username, "" }, 0);
                    }
                    else
                    {
                        l = new ListViewItem(new string[] { Properties.Resources.KerberosSid + Utils.BytesToHex(acl.KerberosSid), "" }, 0);
                    }
                    l.Tag = acl;
                    accountsListView.Items.Add(l);
                }
                UpdateListView();
            }
        }

        private void UpdateListView()
        {
            foreach (ListViewItem l in accountsListView.Items)
            {
                AmtSecurityAdmin.AmtUserAcl acl = (AmtSecurityAdmin.AmtUserAcl)l.Tag;
                string s = "";
                if (acl.Realms.Length == 0) s = ManageabilityCommanderTool.Properties.Resources.ACL_Realm_None;
                else if (acl.Realms.Length == 1) s = ManageabilityCommanderTool.Properties.Resources.ACL_Realm_One;
                else s = acl.Realms.Length.ToString() + ManageabilityCommanderTool.Properties.Resources.ACL_Realm_Multiple;
                if (acl.AccessPermission == AccessPermissionType.AnyAccessPermission) l.SubItems[1].Text = ManageabilityCommanderTool.Properties.Resources.ACL_Access_Any + s;
                if (acl.AccessPermission == AccessPermissionType.LocalAccessPermission) l.SubItems[1].Text = ManageabilityCommanderTool.Properties.Resources.ACL_Access_Local + s;
                if (acl.AccessPermission == AccessPermissionType.NetworkAccessPermission) l.SubItems[1].Text = ManageabilityCommanderTool.Properties.Resources.ACL_Access_Network + s;
            }
        }

        private void accountsListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (accountsListView.SelectedItems.Count == 0)
            {
                editButton.Enabled = removeButton.Enabled = false;
            }
            else
            {
                editButton.Enabled = true;
                AmtSecurityAdmin.AmtUserAcl acl = (AmtSecurityAdmin.AmtUserAcl)accountsListView.SelectedItems[0].Tag;
                if (acl.Username != null)
                    removeButton.Enabled = !(acl.Username.StartsWith("$$"));
                else
                    removeButton.Enabled = true; 
            }
        }

        private void addButton_Click(object sender, EventArgs e)
        {
            UserPermissionsForm form;
            form = new UserPermissionsForm(new AmtSecurityAdmin.AmtUserAcl("", "", AccessPermissionType.AnyAccessPermission, new AmtSecurityAdmin.AmtUserAclRealm[0]), system);
            if (form.ShowDialog(this) == DialogResult.OK)
            {
                AmtSecurityAdmin.AmtUserAcl acl = form.GetAcl();

                ListViewItem l;
                if (acl.Type == AmtSecurityAdmin.AmtUserAclType.Digest)
                {
                    l = new ListViewItem(new string[] { acl.Username, "" }, 0);
                }
                else
                {
                    l = new ListViewItem(new string[] { Properties.Resources.KerberosSid + Utils.BytesToHex(acl.KerberosSid), "" }, 0);
                }
                l.Tag = form.GetAcl();
                accountsListView.Items.Add(l);
                UpdateListView();
            }
        }

        private void editButton_Click(object sender, EventArgs e)
        {
            if (accountsListView.SelectedItems.Count == 0) return;

            UserPermissionsForm form;
            ListViewItem ll = accountsListView.SelectedItems[0];
            AmtSecurityAdmin.AmtUserAcl user = (AmtSecurityAdmin.AmtUserAcl)ll.Tag;
            form = new UserPermissionsForm(user, system);
            if (form.ShowDialog(this) == DialogResult.OK)
            {
                AmtSecurityAdmin.AmtUserAcl acl = form.GetAcl();

                ListViewItem l;
                if (acl.Type == AmtSecurityAdmin.AmtUserAclType.Digest)
                {
                    l = new ListViewItem(new string[] { acl.Username, "" }, 0);
                }
                else
                {
                    l = new ListViewItem(new string[] { Properties.Resources.KerberosSid + Utils.BytesToHex(acl.KerberosSid), "" }, 0);
                }
                l.Tag = form.GetAcl();
                accountsListView.Items.Remove(ll);
                accountsListView.Items.Add(l);
                UpdateListView();
            }
        }

    }
}
