// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class AddFilterForm : Form
    {
        private AmtSystem computer;

        public AddFilterForm(AmtSystem c)
        {
            computer = c;
            InitializeComponent();
        }

        private void AddFilterForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;

            typeComboBox.SelectedIndex = 0;
            directionComboBox.SelectedIndex = 0;
            profileComboBox.SelectedIndex = 0;
            ethernetComboBox.SelectedIndex = 0;

            udpDirectionComboBox.SelectedIndex = 0;
            ipDirectionComboBox.SelectedIndex = 0;
            tcpDirectionComboBox.SelectedIndex = 0;

            ipv4nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.NotSpecified, 0, null));
            ipv4nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Protocol_ICMP, 1, null));
            ipv4nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Protocol_IGMP, 2, null));
            ipv4nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Protocol_IPEncap, 4, null));
            ipv4nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Protocol_TCP, 6, null));
            ipv4nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Protocol_UDP, 17, null));
            ipv4nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Protocol_IPv6, 41, null));
            ipv4nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Protocol_IPsecESP, 50, null));
            ipv4nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Protocol_IPsecAH, 51, null));
            ipv4nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Protocol_IPComp, 108, null));
            ipv4nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Protocol_L2TP, 115, null));
            ipv4nextProtocolComboBox.SelectedIndex = 0;

            ipv6nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.NotSpecified, 0, null));
            ipv6nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Protocol_IGMP, 2, null));
            ipv6nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Protocol_IPEncap, 4, null));
            ipv6nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Protocol_TCP, 6, null));
            ipv6nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Protocol_UDP, 17, null));
            ipv6nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Protocol_IPv6, 41, null));
            ipv6nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Protocol_IPsecESP, 50, null));
            ipv6nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Protocol_IPsecAH, 51, null));
            ipv6nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Protocol_ICMPv6, 58, null));
            ipv6nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Protocol_IPComp, 108, null));
            ipv6nextProtocolComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Protocol_L2TP, 115, null));
            ipv6nextProtocolComboBox.SelectedIndex = 0;

            ipDirectionComboBox.Items.Insert(0, ManageabilityCommanderTool.Properties.Resources.NotSpecified);
            ip6DirectionComboBox.Items.Insert(0, ManageabilityCommanderTool.Properties.Resources.NotSpecified);
            tcpDirectionComboBox.Items.Insert(0, ManageabilityCommanderTool.Properties.Resources.NotSpecified);
            udpDirectionComboBox.Items.Insert(0, ManageabilityCommanderTool.Properties.Resources.NotSpecified);

            ipDirectionComboBox_SelectedIndexChanged(this, null);
            ip6DirectionComboBox_SelectedIndexChanged(this, null);
            tcpDirectionComboBox_SelectedIndexChanged(this, null);
            udpDirectionComboBox_SelectedIndexChanged(this, null);

            ip6DirectionComboBox.SelectedIndex = 0;
            ipv6nextProtocolComboBox.SelectedIndex = 0;
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            computer.CircuitBreaker.CircuitBreakerCreateFilter(GetFilter());
            this.DialogResult = DialogResult.OK;
        }

        public CircuitBreakerFilterType GetFilter()
        {
            CircuitBreakerFilterType filter;

            filter = new CircuitBreakerFilterType();
            filter.ActionEventOnMatch = actionCheckBox.Checked;
            filter.FilterDirection = (ushort)(directionComboBox.SelectedIndex);
            filter.FilterName = nameTextBox.Text;
            filter.FilterPacket = new CircuitBreakerPacketType();
            filter.FilterProfile = (ushort)(profileComboBox.SelectedIndex);
            uint rate;
            uint.TryParse(rateTextBox.Text, out rate);
            filter.FilterProfileData = rate;

            if (typeComboBox.SelectedIndex == 0) // Ethernet
            {
                CircuitBreakerPacketETHType packet = new CircuitBreakerPacketETHType();
                string ethHex = ethernetComboBox.SelectedItem.ToString().Substring(2, 4);
                packet.EthernetFrameType = (ushort)uint.Parse(ethHex, System.Globalization.NumberStyles.HexNumber);
                filter.FilterPacket.Item = packet;
            }
            
            if (typeComboBox.SelectedIndex == 1) // IPv4
            {
                CircuitBreakerIPv4Type ipv4 = new CircuitBreakerIPv4Type();
                if (ipDirectionComboBox.SelectedIndex == 0)
                {
                    ipv4.IPv4Desc = null;
                }
                else
                {
                    ipv4.IPv4Desc = new CircuitBreakerIPv4AddressAndMaskType();
                    ipv4.IPv4Desc.Address = ipAddressTextBox.Text;
                    ipv4.IPv4Desc.AddressMask = ipMaskTextBox.Text;
                    ipv4.IPv4Desc.IPAddressDirection = (ushort)(ipDirectionComboBox.SelectedIndex - 1); // 0 = Transmit, 1 = Receive
                }
                
                CircuitBreakerPacketIPType packet = new CircuitBreakerPacketIPType();
                packet.NextProtocol = (byte)((Utils.DropDownItem)ipv4nextProtocolComboBox.SelectedItem).Handle;
                packet.NextProtocolSpecified = (ipv4nextProtocolComboBox.SelectedIndex != 0);
                packet.IPPacket = new CircuitBreakerIPPacketType();
                packet.IPPacket.Item = ipv4;

                filter.FilterPacket.Item = packet;
            }

            if (typeComboBox.SelectedIndex == 2) // IPv4 + UDP
            {
                CircuitBreakerIPv4Type ipv4 = new CircuitBreakerIPv4Type();
                if (udpDirectionComboBox.SelectedIndex == 0)
                {
                    ipv4.IPv4Desc = null;
                }
                else
                {
                    ipv4.IPv4Desc = new CircuitBreakerIPv4AddressAndMaskType();
                    ipv4.IPv4Desc.Address = udpIpAddressTextBox.Text;
                    ipv4.IPv4Desc.AddressMask = udpMaskTextBox.Text;
                    ipv4.IPv4Desc.IPAddressDirection = (ushort)(udpDirectionComboBox.SelectedIndex - 1); // 0 = Transmit, 1 = Receive
                }

                CircuitBreakerPacketUDPType packet = new CircuitBreakerPacketUDPType();
                packet.IPPacket = new CircuitBreakerIPPacketType();
                packet.IPPacket.Item = ipv4;

                packet.IPLayeredPort = null;
                if (!radioButton4.Checked)
                {
                    packet.IPLayeredPort = new CircuitBreakerIPLayeredPortType();
                    packet.IPLayeredPort.Item = null;
                }

                if (radioButton1.Checked)
                {
                    CircuitBreakerIPLayeredPortSimpleType IPLayeredPortSimple = new CircuitBreakerIPLayeredPortSimpleType();
                    IPLayeredPortSimple.SourcePort = ushort.Parse(minPortTextBox.Text);
                    IPLayeredPortSimple.DestinationPort = ushort.Parse(maxPortTextBox.Text);
                    packet.IPLayeredPort.Item = IPLayeredPortSimple;
                    packet.IPLayeredPort.ItemElementName = ItemChoiceType.IPLayeredPortSimple;
                }

                if (radioButton2.Checked)
                {
                    CircuitBreakerIPLayeredPortRangeType IPLayeredPortRangeSource = new CircuitBreakerIPLayeredPortRangeType();
                    IPLayeredPortRangeSource.PortMin = ushort.Parse(minPortTextBox.Text);
                    IPLayeredPortRangeSource.PortMax = ushort.Parse(maxPortTextBox.Text);
                    packet.IPLayeredPort.Item = IPLayeredPortRangeSource;
                    packet.IPLayeredPort.ItemElementName = ItemChoiceType.IPLayeredPortRangeSource;
                }

                if (radioButton3.Checked)
                {
                    CircuitBreakerIPLayeredPortRangeType IPLayeredPortRangeDestination = new CircuitBreakerIPLayeredPortRangeType();
                    IPLayeredPortRangeDestination.PortMin = ushort.Parse(minPortTextBox.Text);
                    IPLayeredPortRangeDestination.PortMax = ushort.Parse(maxPortTextBox.Text);
                    packet.IPLayeredPort.Item = IPLayeredPortRangeDestination;
                    packet.IPLayeredPort.ItemElementName = ItemChoiceType.IPLayeredPortRangeDestination;
                }

                filter.FilterPacket.Item = packet;
            }

            if (typeComboBox.SelectedIndex == 3) // IPv4 + TCP
            {
                CircuitBreakerIPv4Type ipv4 = new CircuitBreakerIPv4Type();
                if (tcpDirectionComboBox.SelectedIndex == 0)
                {
                    ipv4.IPv4Desc = null;
                }
                else
                {
                    ipv4.IPv4Desc = new CircuitBreakerIPv4AddressAndMaskType();
                    ipv4.IPv4Desc.Address = tcpIpAddressTextBox.Text;
                    ipv4.IPv4Desc.AddressMask = tcpMaskTextBox.Text;
                    ipv4.IPv4Desc.IPAddressDirection = (ushort)(tcpDirectionComboBox.SelectedIndex - 1); // 0 = Transmit, 1 = Receive
                }

                CircuitBreakerPacketTCPType packet = new CircuitBreakerPacketTCPType();
                packet.IPPacket = new CircuitBreakerIPPacketType();
                packet.IPPacket.Item = ipv4;

                packet.IPLayeredPort = null;
                if (!radioButton5.Checked)
                {
                    packet.IPLayeredPort = new CircuitBreakerIPLayeredPortType();
                    packet.IPLayeredPort.Item = null;
                }

                if (radioButton8.Checked)
                {
                    CircuitBreakerIPLayeredPortSimpleType IPLayeredPortSimple = new CircuitBreakerIPLayeredPortSimpleType();
                    IPLayeredPortSimple.SourcePort = ushort.Parse(tcpMinPortTextBox.Text);
                    IPLayeredPortSimple.DestinationPort = ushort.Parse(tcpMaxPortTextBox.Text);
                    packet.IPLayeredPort.Item = IPLayeredPortSimple;
                    packet.IPLayeredPort.ItemElementName = ItemChoiceType.IPLayeredPortSimple;
                }

                if (radioButton7.Checked)
                {
                    CircuitBreakerIPLayeredPortRangeType IPLayeredPortRangeSource = new CircuitBreakerIPLayeredPortRangeType();
                    IPLayeredPortRangeSource.PortMin = ushort.Parse(tcpMinPortTextBox.Text);
                    IPLayeredPortRangeSource.PortMax = ushort.Parse(tcpMaxPortTextBox.Text);
                    packet.IPLayeredPort.Item = IPLayeredPortRangeSource;
                    packet.IPLayeredPort.ItemElementName = ItemChoiceType.IPLayeredPortRangeSource;
                }

                if (radioButton6.Checked)
                {
                    CircuitBreakerIPLayeredPortRangeType IPLayeredPortRangeDestination = new CircuitBreakerIPLayeredPortRangeType();
                    IPLayeredPortRangeDestination.PortMin = ushort.Parse(tcpMinPortTextBox.Text);
                    IPLayeredPortRangeDestination.PortMax = ushort.Parse(tcpMaxPortTextBox.Text);
                    packet.IPLayeredPort.Item = IPLayeredPortRangeDestination;
                    packet.IPLayeredPort.ItemElementName = ItemChoiceType.IPLayeredPortRangeDestination;
                }

                filter.FilterPacket.Item = packet;
            }

            if (typeComboBox.SelectedIndex == 4) // IPv6
            {
                CircuitBreakerIPv6Type ipv6 = new CircuitBreakerIPv6Type();
                if (ip6DirectionComboBox.SelectedIndex == 0)
                {
                    ipv6.IPv6Desc = null;
                }
                else
                {
                    ipv6.IPv6Desc = new CircuitBreakerIPv6AddressAndMaskType();
                    ipv6.IPv6Desc.Address = ip6AddressTextBox.Text;
                    ipv6.IPv6Desc.AddressMask = ip6MaskTextBox.Text;
                    ipv6.IPv6Desc.IPAddressDirection = (ushort)(ip6DirectionComboBox.SelectedIndex - 1); // 0 = Transmit, 1 = Receive
                }

                CircuitBreakerPacketIPType packet = new CircuitBreakerPacketIPType();
                packet.NextProtocol = (byte)((Utils.DropDownItem)ipv6nextProtocolComboBox.SelectedItem).Handle;
                packet.NextProtocolSpecified = (ipv6nextProtocolComboBox.SelectedIndex != 0);
                packet.IPPacket = new CircuitBreakerIPPacketType();
                packet.IPPacket.Item = ipv6;

                filter.FilterPacket.Item = packet;
            }

            return filter;
        }

        private void nameTextBox_TextChanged(object sender, EventArgs e)
        {
            int ss = nameTextBox.SelectionStart;
            int sl = nameTextBox.SelectionLength;
            nameTextBox.Text = Utils.FilterAmtName(nameTextBox.Text);
            nameTextBox.SelectionStart = ss;
            nameTextBox.SelectionLength = sl;
        }

        private void profileComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            rateTextBox.Enabled = (profileComboBox.SelectedIndex == 2);
        }

        private void rateTextBox_TextChanged(object sender, EventArgs e)
        {
            int ss = rateTextBox.SelectionStart;
            int sl = rateTextBox.SelectionLength;
            rateTextBox.Text = Utils.FilterNumber(rateTextBox.Text);
            rateTextBox.SelectionStart = ss;
            rateTextBox.SelectionLength = sl;
        }

        private void typeComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (typeComboBox.SelectedIndex)
            {
                case 0:
                    containerPanel.Controls.Add(this.ethernetPanel);
                    if (containerPanel.Controls.Count > 1) containerPanel.Controls.RemoveAt(0);
                    break;
                case 1:
                    containerPanel.Controls.Add(this.ipPanel);
                    if (containerPanel.Controls.Count > 1) containerPanel.Controls.RemoveAt(0);
                    break;
                case 2:
                    containerPanel.Controls.Add(this.udpPanel);
                    if (containerPanel.Controls.Count > 1) containerPanel.Controls.RemoveAt(0);
                    break;
                case 3:
                    containerPanel.Controls.Add(this.tcpPanel);
                    if (containerPanel.Controls.Count > 1) containerPanel.Controls.RemoveAt(0);
                    break;
                case 4:
                    containerPanel.Controls.Add(this.ip6panel);
                    if (containerPanel.Controls.Count > 1) containerPanel.Controls.RemoveAt(0);
                    break;
            }
        }

        private void radioButton4_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton4.Checked) // No Port Filtering
            {
                minPortTextBox.Visible = false;
                maxPortTextBox.Visible = false;
                toLabel.Visible = false;
                sourceLabel.Visible = false;
                destLabel.Visible = false;
            }
            else if (radioButton1.Checked) // Simple Filtering
            {
                minPortTextBox.Visible = true;
                maxPortTextBox.Visible = true;
                toLabel.Visible = false;
                sourceLabel.Visible = true;
                destLabel.Visible = true;
            }
            else if (radioButton2.Checked || radioButton3.Checked) // Source Range Filtering
            {
                minPortTextBox.Visible = true;
                maxPortTextBox.Visible = true;
                toLabel.Visible = true;
                sourceLabel.Visible = false;
                destLabel.Visible = false;
            }
        }

        private void radioButton5_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton5.Checked) // No Port Filtering
            {
                tcpMinPortTextBox.Visible = false;
                tcpMaxPortTextBox.Visible = false;
                tcpToLabel.Visible = false;
                tcpSourceLabel.Visible = false;
                tcpDestLabel.Visible = false;
            }
            else if (radioButton8.Checked) // Simple Filtering
            {
                tcpMinPortTextBox.Visible = true;
                tcpMaxPortTextBox.Visible = true;
                tcpToLabel.Visible = false;
                tcpSourceLabel.Visible = true;
                tcpDestLabel.Visible = true;
            }
            else if (radioButton7.Checked || radioButton6.Checked) // Source Range Filtering
            {
                tcpMinPortTextBox.Visible = true;
                tcpMaxPortTextBox.Visible = true;
                tcpToLabel.Visible = true;
                tcpSourceLabel.Visible = false;
                tcpDestLabel.Visible = false;
            }
        }

        private void ipDirectionComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            ipAddressTextBox.Enabled = (ipDirectionComboBox.SelectedIndex != 0);
            ipMaskTextBox.Enabled = (ipDirectionComboBox.SelectedIndex != 0);
        }

        private void tcpDirectionComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            tcpIpAddressTextBox.Enabled = (tcpDirectionComboBox.SelectedIndex != 0);
            tcpMaskTextBox.Enabled = (tcpDirectionComboBox.SelectedIndex != 0);
        }

        private void udpDirectionComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            udpIpAddressTextBox.Enabled = (udpDirectionComboBox.SelectedIndex != 0);
            udpMaskTextBox.Enabled = (udpDirectionComboBox.SelectedIndex != 0);
        }

        private void ip6DirectionComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            ip6AddressTextBox.Enabled = (ip6DirectionComboBox.SelectedIndex != 0);
            ip6MaskTextBox.Enabled = (ip6DirectionComboBox.SelectedIndex != 0);
        }

        private void ip6AddressTextBox_TextChanged(object sender, EventArgs e)
        {
            //
            // Validate the text against the IPv6 address regular expressions 
            // and enable the Add (OK) button as necessary.
            //
            // Complete IPv6 address (no compression)
            //    \A([0-9a-f]{1,4}:){7}[0-9a-f]{1,4}\Z
            // IPv6 address with zero compression at begin
            //    \A:(:[0-9a-f]{1,4}){1,7}\Z
            // IPv6 address with zero compression in middle
            //   \A([0-9a-f]{1,4}:){1,1}(:[0-9a-f]{1,4}){1,6}\Z
            //   \A([0-9a-f]{1,4}:){1,2}(:[0-9a-f]{1,4}){1,5}\Z
            //   \A([0-9a-f]{1,4}:){1,3}(:[0-9a-f]{1,4}){1,4}\Z
            //   \A([0-9a-f]{1,4}:){1,4}(:[0-9a-f]{1,4}){1,3}\Z
            //   \A([0-9a-f]{1,4}:){1,5}(:[0-9a-f]{1,4}){1,2}\Z
            //   \A([0-9a-f]{1,4}:){1,6}(:[0-9a-f]{1,4}){1,1}\Z
            // IPv6 address with zero compression at end
            //   \A(([0-9a-f]{1,4}:){1,7}|:):\Z
            //

            string ipv6Regex = "(\\A([0-9a-f]{1,4}:){7}[0-9a-f]{1,4}\\Z)|" +
                "(\\A:(:[0-9a-f]{1,4}){1,7}\\Z)|" +
                "(\\A([0-9a-f]{1,4}:){1,1}(:[0-9a-f]{1,4}){1,6}\\Z)|" +
                "(\\A([0-9a-f]{1,4}:){1,2}(:[0-9a-f]{1,4}){1,5}\\Z)|" +
                "(\\A([0-9a-f]{1,4}:){1,3}(:[0-9a-f]{1,4}){1,4}\\Z)|" +
                "(\\A([0-9a-f]{1,4}:){1,4}(:[0-9a-f]{1,4}){1,3}\\Z)|" +
                "(\\A([0-9a-f]{1,4}:){1,5}(:[0-9a-f]{1,4}){1,2}\\Z)|" +
                "(\\A([0-9a-f]{1,4}:){1,6}(:[0-9a-f]{1,4}){1,1}\\Z)|" +
                "(\\A(([0-9a-f]{1,4}:){1,7}|:):\\Z)";
            Match ipv6RegexMatch = Regex.Match(this.ip6AddressTextBox.Text, ipv6Regex, RegexOptions.IgnoreCase | RegexOptions.IgnorePatternWhitespace);
            this.okButton.Enabled = ipv6RegexMatch.Success;
        }

        private void ip6MaskTextBox_TextChanged(object sender, EventArgs e)
        {
            //
            // Validate the text against the IPv6 address regular expressions 
            // and enable the Add (OK) button as necessary.
            //
            // Complete IPv6 address (no compression)
            //    \A([0-9a-f]{1,4}:){7}[0-9a-f]{1,4}\Z
            // IPv6 address with zero compression at begin
            //    \A:(:[0-9a-f]{1,4}){1,7}\Z
            // IPv6 address with zero compression in middle
            //   \A([0-9a-f]{1,4}:){1,1}(:[0-9a-f]{1,4}){1,6}\Z
            //   \A([0-9a-f]{1,4}:){1,2}(:[0-9a-f]{1,4}){1,5}\Z
            //   \A([0-9a-f]{1,4}:){1,3}(:[0-9a-f]{1,4}){1,4}\Z
            //   \A([0-9a-f]{1,4}:){1,4}(:[0-9a-f]{1,4}){1,3}\Z
            //   \A([0-9a-f]{1,4}:){1,5}(:[0-9a-f]{1,4}){1,2}\Z
            //   \A([0-9a-f]{1,4}:){1,6}(:[0-9a-f]{1,4}){1,1}\Z
            // IPv6 address with zero compression at end
            //   \A(([0-9a-f]{1,4}:){1,7}|:):\Z
            //

            string ipv6Regex = "(\\A([0-9a-f]{1,4}:){7}[0-9a-f]{1,4}\\Z)|" +
                "(\\A:(:[0-9a-f]{1,4}){1,7}\\Z)|" +
                "(\\A([0-9a-f]{1,4}:){1,1}(:[0-9a-f]{1,4}){1,6}\\Z)|" +
                "(\\A([0-9a-f]{1,4}:){1,2}(:[0-9a-f]{1,4}){1,5}\\Z)|" +
                "(\\A([0-9a-f]{1,4}:){1,3}(:[0-9a-f]{1,4}){1,4}\\Z)|" +
                "(\\A([0-9a-f]{1,4}:){1,4}(:[0-9a-f]{1,4}){1,3}\\Z)|" +
                "(\\A([0-9a-f]{1,4}:){1,5}(:[0-9a-f]{1,4}){1,2}\\Z)|" +
                "(\\A([0-9a-f]{1,4}:){1,6}(:[0-9a-f]{1,4}){1,1}\\Z)|" +
                "(\\A(([0-9a-f]{1,4}:){1,7}|:):\\Z)";
            Match ipv6RegexMatch = Regex.Match(this.ip6MaskTextBox.Text, ipv6Regex, RegexOptions.IgnoreCase | RegexOptions.IgnorePatternWhitespace);
            this.okButton.Enabled = ipv6RegexMatch.Success;
        }

    }
}
