// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class AddPolicyForm : Form
    {
        private AmtSystem computer = null;

        private class filteritem
        {
            public string name;
            public uint handle;
            public override string ToString() { return name; }
        }

        public AddPolicyForm(AmtSystem computer)
        {
            this.computer = computer;
            InitializeComponent();
        }

        private void AddPolicyForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            
            Hashtable t = computer.CircuitBreaker.GetCircuitBreakerFiltersTable();
            if (t != null)
            {
                foreach (uint k in t.Keys)
                {
                    filteritem f = new filteritem();
                    f.handle = k;
                    f.name = t[k].ToString();
                    filterCheckedListBox.Items.Add(f);
                }
            }
            updateui();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
        }

        public CircuitBreakerPolicyType GetPolicy()
        {
            CircuitBreakerPolicyType policy = new CircuitBreakerPolicyType() ;

            if (antiSpoofingActiveCheckBox.Checked == true)
            {
                policy.AntiSpoofingFilter = new CircuitBreakerAntiSpoofingFilterType();
                policy.AntiSpoofingFilter.ActionCount = asActionCountCheckBox.Checked;
                policy.AntiSpoofingFilter.ActionEventOnMatch = asActionEventCheckBox.Checked;
            }
            policy.DefaultRXFilter = new CircuitBreakerDefaultFilterType();
            policy.DefaultRXFilter.ActionCount = rxActionCountCheckBox.Checked;
            policy.DefaultRXFilter.ActionDrop = rxActionDropCheckBox.Checked;
            policy.DefaultRXFilter.ActionEventOnMatch = rxActionEventCheckBox.Checked;
            policy.DefaultTXFilter = new CircuitBreakerDefaultFilterType();
            policy.DefaultTXFilter.ActionCount = txActionCountCheckBox.Checked;
            policy.DefaultTXFilter.ActionDrop = txActionDropCheckBox.Checked;
            policy.DefaultTXFilter.ActionEventOnMatch = txActionEventCheckBox.Checked;
            policy.PolicyName = nameTextBox.Text;
            policy.PolicyPrecedence = uint.Parse(precedenceTextBox.Text);

            policy.FilterCreationHandles = new uint[filterCheckedListBox.CheckedItems.Count];
            int i = 0;
            foreach (filteritem f in filterCheckedListBox.CheckedItems)
            {
                policy.FilterCreationHandles[i] = f.handle;
                i++;
            }

            return policy;
        }

        private void nameTextBox_TextChanged(object sender, EventArgs e)
        {
            int ss = nameTextBox.SelectionStart;
            int sl = nameTextBox.SelectionLength;
            nameTextBox.Text = Utils.FilterAmtName(nameTextBox.Text);
            nameTextBox.SelectionStart = ss;
            nameTextBox.SelectionLength = sl;
            updateui();
        }

        private void precedenceTextBox_TextChanged(object sender, EventArgs e)
        {
            int ss = precedenceTextBox.SelectionStart;
            int sl = precedenceTextBox.SelectionLength;
            precedenceTextBox.Text = Utils.FilterNumber(precedenceTextBox.Text);
            precedenceTextBox.SelectionStart = ss;
            precedenceTextBox.SelectionLength = sl;
            updateui();
        }

        private void updateui()
        {
            bool ok = true;
            try
            {
                uint.Parse(precedenceTextBox.Text);
            }
            catch (Exception) { ok = false; }

            okButton.Enabled = ok;
        }

        private void antiSpoofingActiveCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            asActionCountCheckBox.Enabled = antiSpoofingActiveCheckBox.Checked;
            asActionEventCheckBox.Enabled = antiSpoofingActiveCheckBox.Checked;
        }

    }
}
