// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class AddWatchdogForm : Form
    {
        public AddWatchdogForm()
        {
            InitializeComponent();
        }

        public Guid GetGuid()
        {
            try
            {
                string gs = identifierComboBox.Text;
                int i = gs.IndexOf(":");
                if (i >= 0) gs = gs.Substring(0, i);
                return new Guid(gs);
            }
            catch (Exception)
            {
                return Guid.Empty;
            }
        }

        private void addButton_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
            identifierComboBox_TextChanged(this, null);
        }

        private void generateIdButton_Click(object sender, EventArgs e)
        {
            identifierComboBox.Text = Guid.NewGuid().ToString().ToUpper();
            identifierComboBox_TextChanged(this, null);
        }

        public AmtWatchdogAgent GetAgent()
        {
            return new AmtWatchdogAgent(GetGuid(), descriptionTextBox.Text, ushort.Parse(timeoutTextBox.Text), ushort.Parse(startTimeTextBox.Text));
        }

        private void descriptionTextBox_TextChanged(object sender, EventArgs e)
        {
            int ss = descriptionTextBox.SelectionStart;
            int sl = descriptionTextBox.SelectionStart;
            descriptionTextBox.Text = Utils.FilterAmtName(descriptionTextBox.Text);
            descriptionTextBox.SelectionStart = ss;
            descriptionTextBox.SelectionStart = sl;
        }

        private void identifierComboBox_TextChanged(object sender, EventArgs e)
        {
            bool ok = true;
            try
            {
                string gs = identifierComboBox.Text;
                int i = gs.IndexOf(":");
                if (i >= 0) gs = gs.Substring(0, i);
                Guid g = new Guid(gs);
            }
            catch (Exception) { ok = false; }

            ushort r;
            if (!ushort.TryParse(timeoutTextBox.Text, out r)) ok = false;
            if (!ushort.TryParse(startTimeTextBox.Text, out r)) ok = false;

            addButton.Enabled = ok;
        }

        private void timeoutTextBox_TextChanged(object sender, EventArgs e)
        {
            int ss = timeoutTextBox.SelectionStart;
            int sl = timeoutTextBox.SelectionStart;
            timeoutTextBox.Text = Utils.FilterNumber(timeoutTextBox.Text);
            timeoutTextBox.SelectionStart = ss;
            timeoutTextBox.SelectionStart = sl;

            identifierComboBox_TextChanged(this, null);
        }

        private void startTimeTextBox_TextChanged(object sender, EventArgs e)
        {
            int ss = startTimeTextBox.SelectionStart;
            int sl = startTimeTextBox.SelectionStart;
            startTimeTextBox.Text = Utils.FilterNumber(startTimeTextBox.Text);
            startTimeTextBox.SelectionStart = ss;
            startTimeTextBox.SelectionStart = sl;

            identifierComboBox_TextChanged(this, null);
        }

        private void AddWatchdogForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
        }
    }
}
