// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;

namespace ManageabilityCommanderTool
{
    public partial class AddWatchdogTransitionForm : Form
    {
        public AddWatchdogTransitionForm()
        {
            InitializeComponent();

            checkBoxN6_CheckedChanged(this, null);
            checkBoxO6_CheckedChanged(this, null);
        }
        
        public AddWatchdogTransitionForm(ConsoleWatchdogActionType action)
        {
            InitializeComponent();

            logCheckBox.Checked = action.ActionEventOnTransition;

            checkBoxO6.Checked = (action.OldState == 0x00FF);
            checkBoxO1.Checked = (action.OldState & 0x0001) != 0;
            checkBoxO2.Checked = (action.OldState & 0x0002) != 0;
            checkBoxO3.Checked = (action.OldState & 0x0004) != 0;
            checkBoxO4.Checked = (action.OldState & 0x0008) != 0;
            checkBoxO5.Checked = (action.OldState & 0x0010) != 0;

            checkBoxN6.Checked = (action.NewState == 0x00FF);
            checkBoxN1.Checked = (action.NewState & 0x0001) != 0;
            checkBoxN2.Checked = (action.NewState & 0x0002) != 0;
            checkBoxN3.Checked = (action.NewState & 0x0004) != 0;
            checkBoxN4.Checked = (action.NewState & 0x0008) != 0;
            checkBoxN5.Checked = (action.NewState & 0x0010) != 0;

            if (action.ActionCbSpecified == false)
            {
                radioButtonT3.Checked = true;
            }
            else
            {
                if (action.ActionCb == 0) radioButtonT1.Checked = true;
                if (action.ActionCb == 1) radioButtonT2.Checked = true;
            }

            checkBoxN6_CheckedChanged(this, null);
            checkBoxO6_CheckedChanged(this, null);

            addButton.Text = ManageabilityCommanderTool.Properties.Resources.AddWDogTrans_Accept;
        }

        public ConsoleWatchdogActionType GetAction()
        {
            ConsoleWatchdogActionType action = new ConsoleWatchdogActionType();
            
            if (checkBoxO6.Checked)
            {
                action.OldState = 0x00FF;
            }
            else
            {
                if (checkBoxO1.Checked) action.OldState |= 0x0001;
                if (checkBoxO2.Checked) action.OldState |= 0x0002;
                if (checkBoxO3.Checked) action.OldState |= 0x0004;
                if (checkBoxO4.Checked) action.OldState |= 0x0008;
                if (checkBoxO5.Checked) action.OldState |= 0x0010;
            }

            if (checkBoxN6.Checked)
            {
                action.NewState = 0x00FF;
            }
            else
            {
                if (checkBoxN1.Checked) action.NewState |= 0x0001;
                if (checkBoxN2.Checked) action.NewState |= 0x0002;
                if (checkBoxN3.Checked) action.NewState |= 0x0004;
                if (checkBoxN4.Checked) action.NewState |= 0x0008;
                if (checkBoxN5.Checked) action.NewState |= 0x0010;
            }

            if (radioButtonT1.Checked)
            {
                action.ActionCbSpecified = true;
                action.ActionCb = 0;
            }
            else if (radioButtonT2.Checked)
            {
                action.ActionCbSpecified = true;
                action.ActionCb = 1;
            }
            else if (radioButtonT3.Checked)
            {
                action.ActionCbSpecified = false;
                action.ActionCb = 0;
            }

            action.ActionEventOnTransition = logCheckBox.Checked;

            return action;
        }

        private void addButton_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
        }

        private void checkBoxN6_CheckedChanged(object sender, EventArgs e)
        {
            checkBoxN1.Enabled = !checkBoxN6.Checked;
            checkBoxN2.Enabled = !checkBoxN6.Checked;
            checkBoxN3.Enabled = !checkBoxN6.Checked;
            checkBoxN4.Enabled = !checkBoxN6.Checked;
            checkBoxN5.Enabled = !checkBoxN6.Checked;
        }

        private void checkBoxO6_CheckedChanged(object sender, EventArgs e)
        {
            checkBoxO1.Enabled = !checkBoxO6.Checked;
            checkBoxO2.Enabled = !checkBoxO6.Checked;
            checkBoxO3.Enabled = !checkBoxO6.Checked;
            checkBoxO4.Enabled = !checkBoxO6.Checked;
            checkBoxO5.Enabled = !checkBoxO6.Checked;
        }

        private void AddWatchdogTransitionForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
        }

    }
}
