﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class AlarmClockForm : Form
    {
        private AmtSystem computer = null;
        private AmtSecurityAdmin.ProvisioningMode mode;
        private long delta = 0;
        private bool meread = false;
        private string currentPowerSetting;
        private bool enabled;
        private DateTime startTime;
        private DateTime nextTime;
        private TimeSpan interval;
        private bool modified = false;

        public AmtSystem Computer
        {
            get { return this.computer; }
            set { this.computer = value; }
        }

        public string CurrentPowerSetting
        {
            get { return this.currentPowerSetting; }
            set { this.currentPowerSetting = value; }
        }

        public new bool Enabled
        {
            get { return this.enabled; }
            set { this.enabled = value; }
        }

        public DateTime StartTime
        {
            get { return this.startTime; }
            set { this.startTime = value; }
        }

        public DateTime NextTime
        {
            get { return this.nextTime; }
            set { this.nextTime = value; }
        }

        public TimeSpan Interval
        {
            get { return this.interval; }
            set { this.interval = value; }
        }

        public bool Modified
        {
            get { return this.modified; }
            set { this.modified = value; }
        }

        public AlarmClockForm()
        {
            InitializeComponent();
        }

        private void AlarmClockForm_Load(object sender, EventArgs e)
        {
            if (this.Computer.NetworkTime != null)
            {
                mode = this.Computer.SecurityAdmin.GetProvisioningMode();
                DateTime metime;
                AmtCallStatus r = this.Computer.NetworkTime.GetLowAccuracyTimeSynch(out metime);
                if (r != AmtCallStatus.SUCCESS)
                {
                    Utils.ErrorMessageBox(this, "GetLowAccuracyTimeSynch", r);
                    meread = false;
                }
                else
                {
                    DateTime n;
                    if (mode == AmtSecurityAdmin.ProvisioningMode.Enterprise)
                        n = DateTime.UtcNow;
                    else
                        n = DateTime.Now;
                    delta = metime.Ticks - n.Ticks;
                    meread = true;
                }
            }

            this.mePowerSettingsLabel.Text = this.CurrentPowerSetting;
            this.enabledCheckBox.Checked = this.Enabled;

            if (this.Enabled)
            {
                if (mode == AmtSecurityAdmin.ProvisioningMode.Enterprise)
                    this.nextTimeLabel.Text = this.NextTime.ToUniversalTime().ToString("MM/dd/yyyy hh:mm tt");
                else
                    this.nextTimeLabel.Text = this.NextTime.ToString("MM/dd/yyyy hh:mm tt");
            }
            else
                this.nextTimeLabel.Text = "Unknown";

            if (this.NextTime == DateTime.MinValue)
            {
                if (mode == AmtSecurityAdmin.ProvisioningMode.Enterprise)
                    this.alarmDateTimePicker.Value = DateTime.UtcNow;
                else
                    this.alarmDateTimePicker.Value = DateTime.Now;
            }
            else
                this.alarmDateTimePicker.Value = this.NextTime;

            if (this.Interval != TimeSpan.MinValue)
            {
                this.daysNumericUpDown.Value = this.Interval.Days;
                this.hoursNumericUpDown.Value = this.Interval.Hours;
                this.minutesNumericUpDown.Value = this.Interval.Minutes;
                this.repeatCheckBox.Checked = true;
                this.daysNumericUpDown.Enabled = true;
                this.hoursNumericUpDown.Enabled = true;
                this.minutesNumericUpDown.Enabled = true;
            }
            else
            {
                this.repeatCheckBox.Checked = false;
                this.daysNumericUpDown.Enabled = false;
                this.hoursNumericUpDown.Enabled = false;
                this.minutesNumericUpDown.Enabled = false;
            }

            if (this.Enabled)
            {
                this.alarmDateTimePicker.Enabled = true;
                this.repeatCheckBox.Enabled = true;
            }
            else
            {
                this.alarmDateTimePicker.Enabled = false;
                this.repeatCheckBox.Enabled = false;
            }
        }

        private void enabledCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            this.Enabled = this.enabledCheckBox.Checked;
            this.Modified = true;

            if (this.Enabled)
            {
                this.alarmDateTimePicker.Enabled = true;
                this.repeatCheckBox.Enabled = true;
            }
            else
            {
                this.alarmDateTimePicker.Enabled = false;
                this.repeatCheckBox.Enabled = false;
            }

            if (this.Interval != TimeSpan.MinValue)
            {
                this.daysNumericUpDown.Enabled = true;
                this.hoursNumericUpDown.Enabled = true;
                this.minutesNumericUpDown.Enabled = true;
            }
            else
            {
                this.daysNumericUpDown.Enabled = false;
                this.hoursNumericUpDown.Enabled = false;
                this.minutesNumericUpDown.Enabled = false;
            }
        }

        private void alarmDateTimePicker_ValueChanged(object sender, EventArgs e)
        {
            this.startTime = this.alarmDateTimePicker.Value;
            this.Modified = true;
        }

        private void repeatCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.repeatCheckBox.Checked)
            {
                this.daysNumericUpDown.Enabled = true;
                this.hoursNumericUpDown.Enabled = true;
                this.minutesNumericUpDown.Enabled = true;
                this.Interval = new TimeSpan((int)this.daysNumericUpDown.Value,
                    (int)this.hoursNumericUpDown.Value, (int)this.minutesNumericUpDown.Value, 0);
            }
            else
            {
                this.daysNumericUpDown.Enabled = false;
                this.hoursNumericUpDown.Enabled = false;
                this.minutesNumericUpDown.Enabled = false;
                this.Interval = TimeSpan.MinValue;
            }
            this.Modified = true;
        }

        private void interval_ValueChanged(object sender, EventArgs e)
        {
            this.Interval = new TimeSpan((int)this.daysNumericUpDown.Value,
                (int)this.hoursNumericUpDown.Value, (int)this.minutesNumericUpDown.Value, 0);
            this.Modified = true;
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
            this.Close();
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            this.Modified = false;
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void refreshTimer_Tick(object sender, EventArgs e)
        {
            if (meread != true) return;

            DateTime n;
            if (mode == AmtSecurityAdmin.ProvisioningMode.Enterprise)
            {
                n = DateTime.UtcNow;
                this.meTimeLabel.Text = new DateTime(n.Ticks + delta, DateTimeKind.Utc).ToString() + " (UTC)";
            }
            else
            {
                n = DateTime.Now;
                this.meTimeLabel.Text = new DateTime(n.Ticks + delta, DateTimeKind.Local).ToString();
            }
        }
    }
}
