// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Net;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Net.NetworkInformation;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class AlertSubscriptionForm : Form
    {
        private AmtSystem computer;

        public AlertSubscriptionForm(AmtSystem computer)
        {
            this.computer = computer;

            InitializeComponent();
        }

        private void AlertSubscriptionForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            
            NetworkInterface[] netinterfaces = NetworkInterface.GetAllNetworkInterfaces();
            if (netinterfaces.Length > 0)
            {
                foreach (NetworkInterface n in netinterfaces)
                {
                    IPInterfaceProperties s = n.GetIPProperties();
                    foreach (UnicastIPAddressInformation addr in s.UnicastAddresses)
                    {
                        if ((addr.Address.GetAddressBytes().Length == 4) && (addr.Address.ToString().Equals("127.0.0.1") == false)) // Only IPv4 subscriptions are allowed and no loopback
                        {
                            if (addr.IPv4Mask != null && addr.IPv4Mask.ToString().Equals("255.255.255.255") == false)
                            {
                                ipAddressComboBox.Items.Add(new Utils.DropDownItem(addr.Address.ToString(), 0, addr.Address));
                            }
                        }
                        else
                        {
                            if ((addr.Address.AddressFamily == System.Net.Sockets.AddressFamily.InterNetworkV6) &&
                                (addr.Address.IsIPv6LinkLocal == false) &&
                                (addr.Address.Equals(IPAddress.IPv6Loopback) == false))
                            {
                                ipAddressComboBox.Items.Add(new Utils.DropDownItem(addr.Address.ToString(), 0, addr.Address));
                            }
                        }
                    }
                }
                ipAddressComboBox.SelectedIndex = 0;
            }

        //    System.Net.NetworkInformation.NetworkInterface ff;
            //ff.
      //      IPv6PortSettings[] ipv6PortSettings = 
    //        System.Diagnostics.Debug.Assert(ipv6PortSettings != null);

            // Populate the available IPv6 Interface names and select the first match.
  //          foreach (IPv6PortSettings ipv6PortSetting in ipv6PortSettings)
//            {
              //  if (String.Compare(ipv6PortSetting.InstanceID, selectedInterfaceID, true) == 0)
            //    {
          //          // Populate IPv6 addresses for the selected IPv6 interface.
        //            foreach (IPv6PortSettings.AddressInfo ipv6AddrInfo in ipv6PortSetting.CurrentAddressInfo)
      //                  this.ipv6AddrsListBox.Items.Add(ipv6AddrInfo.IPAddr.ToString());
    //                break;
  //              }
//            } // END foreach (IPv6PortSettings ipv6PortSetting in ipv6PortSettings)


            GetSubscriptions();

            AmtEventFilter[] filters = computer.Events.GetEventFilters();
            foreach (AmtEventFilter filter in filters)
            {
                eventFilterComboBox.Items.Add(new Utils.DropDownItem(string.Format("{0} - {1}", filter.Filter.AlertSubscriptionPolicyID.ToString(), filter.GetFilterString()), filter.Filter.AlertSubscriptionPolicyID, filter));
            }

            if (filters.Length > 0) eventFilterComboBox.SelectedIndex = 0;
        }

        private void subscriptionsListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void UpdateInfo()
        {
            removeButton.Enabled = (subscriptionsListView.SelectedItems.Count == 1);
            bool ok = true;
            IPAddress addr = null;
            IPAddress.TryParse(ipAddressComboBox.Text, out addr);
            if (addr == null)
                ok = false;
            addButton.Enabled = ok;
        }

        private void refreshButton_Click(object sender, EventArgs e)
        {
            computer.Events.ClearSubscriptionsCache();
            GetSubscriptions();
        }

        private void GetSubscriptions()
        {
            removeButton.Enabled = false;
            subscriptionsListView.BeginUpdate();
            subscriptionsListView.Items.Clear();
            AmtAlertSubscription[] subscriptions = computer.Events.GetAlertSubscriptions();
            foreach (AmtAlertSubscription subscription in subscriptions)
            {
                string hostname = subscription.Hostname;
                if (hostname == null) hostname = subscription.Address.ToString();
                ListViewItem l = new ListViewItem(new string[] { subscription.FilterPolicyID.ToString(), hostname });
                l.Tag = subscription;
                subscriptionsListView.Items.Add(l);
            }
            subscriptionsListView.EndUpdate();
        }

        private void ipAddressTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void addButton_Click(object sender, EventArgs e)
        {
            IPAddress addr = null;
            IPAddress.TryParse(ipAddressComboBox.Text, out addr);
            if (addr == null && computer.CoreVersion < 2.5) return;

            Utils.DropDownItem d = (Utils.DropDownItem)eventFilterComboBox.SelectedItem;
            computer.Events.SubscribeForAlerts(ipAddressComboBox.Text, (byte)d.Handle);
            GetSubscriptions();
        }

        private void removeButton_Click(object sender, EventArgs e)
        {
            if (subscriptionsListView.SelectedItems.Count != 1) return;
            AmtAlertSubscription sub = (AmtAlertSubscription)subscriptionsListView.SelectedItems[0].Tag;
            computer.Events.CancelAlertSubscription(sub);
            GetSubscriptions();
        }

        private void subscriptionsListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete) { removeButton_Click(this, null); e.Handled = true; }
        }

    }
}
