// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class AllocateStorageForm : Form
    {
        private AmtStorage storage;
        private string labelString;

        public AllocateStorageForm(AmtStorage storage, AmtEnterprise enterprise, AmtApplication application)
        {
            this.storage = storage;

            InitializeComponent();

            AmtEnterprise[] enterprises = storage.GetEnterpriseEntries();
            if (enterprises != null && enterprises.Length > 0)
            {
                foreach (AmtEnterprise e in enterprises)
                {
                    int i = enterpriseComboBox.Items.Add(e.Name);
                    if (enterprise != null && e.Handle == enterprise.Handle) enterpriseComboBox.SelectedIndex = i;
                }
                if (enterprise == null) enterpriseComboBox.SelectedIndex = 0;
            }

            AmtStorageAlloc[] allocs = storage.GetStorageAllocations();
            if (allocs != null & allocs.Length > 0)
            {
                foreach (AmtStorageAlloc a in allocs)
                {
                    vendorComboBox.Items.Add(a.SnrpEntryVendorName);
                    applicationComboBox.Items.Add(a.SnrpEntryApplicationName);
                    partnerComboBox.Items.Add(new Utils.DropDownItem(a.SnrpEntryVendorName + ", " + a.SnrpEntryApplicationName,0,a));
                }
                partnerComboBox.SelectedIndex = 0;
            }

            if (application != null)
            {
                vendorComboBox.Text = application.VendorName;
                applicationComboBox.Text = application.ApplicationName;
            }

            updateui();
        }

        private void updateui()
        {
            bool ok = true;

            if (enterpriseComboBox.Text == null || enterpriseComboBox.Text.Length == 0) ok = false;

            if (nonPartnerRadioButton.Checked)
            {
                if (vendorComboBox.Text == null || vendorComboBox.Text.Length == 0) ok = false;
                if (applicationComboBox.Text == null || applicationComboBox.Text.Length == 0) ok = false;
            }
            else
            {
                if (partnerComboBox.Text == null || partnerComboBox.Text.Length == 0) ok = false;
            }

            if (blockNameTextBox.Text == null || blockNameTextBox.Text.Length == 0) ok = false;

            okButton.Enabled = ok;
        }

        private void enterpriseComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            updateui();
        }

        private void vendorComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            updateui();
        }

        private void applicationComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            updateui();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            AmtStorageBlock block = null;
            AmtStorageWrapper.AmtStorageStatus r;

            if (nonPartnerRadioButton.Checked)
            {
                r = storage.AllocateBlock(enterpriseComboBox.Text, vendorComboBox.Text, applicationComboBox.Text, new Guid("00000000-0000-0000-0000-000000000000"), blockNameTextBox.Text, (uint)blockSizeNumericUpDown.Value * 4096, out block);
            }
            else
            {
                Utils.DropDownItem d = (Utils.DropDownItem)partnerComboBox.SelectedItem;
                AmtStorageAlloc a = (AmtStorageAlloc)d.Tag;
                r = storage.AllocateBlock(enterpriseComboBox.Text, a.SnrpEntryVendorName, a.SnrpEntryApplicationName, new Guid("00000000-0000-0000-0000-000000000000"), blockNameTextBox.Text, (uint)blockSizeNumericUpDown.Value * 4096, out block);
            }

            if (block == null || r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
            {
                MessageBox.Show(this, string.Format(Properties.Resources.ErrorAllocatingBlock, r.ToString()), Properties.Resources.AllocateBlockTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                r = block.SetVisibility(false);
                if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
                {
                    MessageBox.Show(this, string.Format(Properties.Resources.SetVisibilityFalseError, r.ToString()), Properties.Resources.AllocateBlockTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

                // Set Default Permissions
                AmtStorageWrapper.AmtStoragePermissionsGroup group1, group2;
                r = block.AddPermissionGroup(AmtStorageWrapper.AmtStorageGroupPermission.ReadWrite, "ReadWriteGroup", out group1);
                r = block.AddPermissionGroup(AmtStorageWrapper.AmtStorageGroupPermission.ReadOnly, "ReadOnlyGroup", out group2);
                block.AddPermissionsGroupMembers(group1, new uint[] { 0xFFFFFFF1 });
            }

            DialogResult = DialogResult.OK;
        }

        private void blockNameTextBox_TextChanged(object sender, EventArgs e)
        {
            updateui();
        }

        private void partnerRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            partnerComboBox.Enabled = partnerRadioButton.Checked;
            vendorComboBox.Enabled = nonPartnerRadioButton.Checked;
            applicationComboBox.Enabled = nonPartnerRadioButton.Checked;
        }

        private void AllocateStorageForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            labelString = label5.Text;
            blockSizeNumericUpDown_ValueChanged(this, null);
        }

        private void blockSizeNumericUpDown_ValueChanged(object sender, EventArgs e)
        {
            label5.Text = string.Format(labelString, (uint)blockSizeNumericUpDown.Value * 4);
        }

    }
}
