// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class AuditLogLockForm : Form
    {
        private AmtSystem system;

        public AuditLogLockForm(AmtSystem system)
        {
            this.system = system;
            InitializeComponent();
        }

        private void AuditLogLockForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            UpdateInfo();
        }

        void UpdateInfo()
        {
            lockTimeNumericUpDown.Enabled = radioButton1.Checked | radioButton2.Checked;
            unlockHandleNumericUpDown.Enabled = radioButton3.Checked;
        }

        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            if (radioButton1.Checked)
            {
                // Lock audit log
                uint handle;
                AmtCallStatus r = system.AuditLog.LockAuditLog((uint)(lockTimeNumericUpDown.Value), out handle);
                if (r != AmtCallStatus.SUCCESS)
                {
                    Utils.ErrorMessageBox(this, "LockAuditLog", r);
                }
                else
                {
                    MessageBox.Show(this, string.Format("Intel AMT audit log locked with handle {0}.", handle), "Audit Log", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    DialogResult = DialogResult.OK;
                }
            }

            if (radioButton2.Checked)
            {
                // Lock for un-provisioning
                uint handle;
                AmtCallStatus r = system.AuditLog.LockAuditLogUnprovisioning((uint)(lockTimeNumericUpDown.Value), out handle);
                if (r != AmtCallStatus.SUCCESS)
                {
                    Utils.ErrorMessageBox(this, "LockAuditLogUnprovisioning", r);
                }
                else
                {
                    MessageBox.Show(this, string.Format("Intel AMT audit log locked with handle {0}.", handle), "Audit Log", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    DialogResult = DialogResult.OK;
                }
            }

            if (radioButton3.Checked)
            {
                // Unlock
                AmtCallStatus r = system.AuditLog.UnLockAuditLog((uint)unlockHandleNumericUpDown.Value);
                if (r != AmtCallStatus.SUCCESS)
                {
                    Utils.ErrorMessageBox(this, "UnLockAuditLog", r);
                }
                else
                {
                    DialogResult = DialogResult.OK;
                }
            }
        }
    }
}
