// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Net;
using System.Text;
using System.Data;
using System.Drawing;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public class CertificateOperations
    {
        private static Process Run(string fileName, string args)
        {
            ProcessStartInfo info = new ProcessStartInfo(fileName, args);
            info.CreateNoWindow = true;
            info.UseShellExecute = false;
            Process processChild = Process.Start(info);
            return processChild;
        }

        public static string GetCertificateString(string certDataStr)
        {
            string s = "";

            int i = certDataStr.IndexOf("CN=");
            if (i >= 0)
            {
                s = certDataStr.Substring(i + 3);
                i = s.IndexOf(",");
                if (i > 0) s = s.Substring(0, i);
                return s;
            }

            i = certDataStr.IndexOf("O=");
            if (i >= 0)
            {
                s = certDataStr.Substring(i + 2);
                if (s[0] == '\"')
                {
                    s = s.Substring(1);
                    int j = s.IndexOf("\"");
                    if (j >= 0)
                    {
                        s = s.Substring(0, j);
                    }
                }
                else
                {
                    i = s.IndexOf(",");
                    if (i > 0) s = s.Substring(0, i);
                }
                return s;
            }

            return "Unknown";
        }

        public static bool IsTrustedCert(X509Certificate2 cert)
        {
            X509Chain chain = new X509Chain();
            chain.ChainPolicy.RevocationMode = X509RevocationMode.NoCheck;
            if (chain.Build(cert) == true) return chain.ChainElements.Count == 1;
            return false;
        }

        public static bool IsValidCert(X509Certificate2 cert)
        {
            X509Chain chain = new X509Chain();
            chain.ChainPolicy.RevocationMode = X509RevocationMode.NoCheck;
            return chain.Build(cert);
        }

        public static bool IsCaCertificate(X509Certificate2 cert)
        {
            // TODO - Not sure how to do this, but want to check for CA extensions.
            if (cert.Extensions.Count == 0) return true;
            foreach (X509Extension ex in cert.Extensions)
            {
                if (ex.GetType() == typeof(X509BasicConstraintsExtension))
                {
                    X509BasicConstraintsExtension ex2 = (X509BasicConstraintsExtension)ex;
                    return ex2.CertificateAuthority;
                }
            }
            return false;
        }

    }
}
