// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class EditHeuristicForm : Form
    {
        private AmtSystem system;

        public EditHeuristicForm(AmtSystem system)
        {
            this.system = system;
            InitializeComponent();

            CircuitBreakerPolicyInfoType[] cbp = system.CircuitBreaker.GetCircuitBreakerPolicies();
            if (cbp != null && cbp.Length > 0)
            {
                foreach (CircuitBreakerPolicyInfoType cp in cbp)
                {
                    policyComboBox.Items.Add(new Utils.DropDownItem(cp.Policy.PolicyName, 0, cp.PolicyCreationHandle));
                }
                policyComboBox.SelectedIndex = 0;
            }
            else
            {
                policyCheckBox.Enabled = false;
            }
        }

        public HcbOptionsType HcbOptions
        {
            get
            {
                if (slowCheckBox.Checked == false && fastCheckBox.Checked == false)
                {
                    return null;
                }
                HcbOptionsType o = new HcbOptionsType();
                if (fastCheckBox.Checked == false)
                {
                    o.FastConnectionRate = null;
                }
                else
                {
                    o.FastConnectionRate = new TimedCounterType();
                    o.FastConnectionRate.Threshold = (uint)fastRateNumericUpDown.Value;
                    o.FastConnectionRate.ClearTime = (uint)fastTimeNumericUpDown.Value;
                }
                if (slowCheckBox.Checked == false)
                {
                    o.SlowConnectionRate = null;
                }
                else
                {
                    o.SlowConnectionRate = new TimedCounterType();
                    o.SlowConnectionRate.Threshold = (uint)slowRateNumericUpDown.Value;
                    o.SlowConnectionRate.ClearTime = ((uint)slowTimeNumericUpDown.Value) * 1000;
                }
                o.BlockAll = blockAllCheckBox.Checked;
                o.BlockOffensivePort = blockPortCheckBox.Checked;
                o.PolicyCreationHandleSpecified = policyCheckBox.Checked;
                if (o.PolicyCreationHandleSpecified)
                {
                    o.PolicyCreationHandle = (uint)(((Utils.DropDownItem)policyComboBox.SelectedItem).Tag);
                }
                if (moveToStopCheckBox.Checked == true)
                {
                    o.EncounterTimeout = 0;
                }
                else
                {
                    o.EncounterTimeout = (ushort)encounterTimeoutNumericUpDown.Value;
                }
                return o;
            }
            set
            {
                if (value == null)
                {
                    slowCheckBox.Checked = false;
                    fastCheckBox.Checked = false;
                    return;
                }
                slowCheckBox.Checked = (value.SlowConnectionRate != null);
                if (value.SlowConnectionRate != null)
                {
                    slowRateNumericUpDown.Value = value.SlowConnectionRate.Threshold;
                    slowTimeNumericUpDown.Value = (value.SlowConnectionRate.ClearTime / 1000);
                }
                fastCheckBox.Checked = (value.FastConnectionRate != null);
                if (value.FastConnectionRate != null)
                {
                    fastRateNumericUpDown.Value = value.FastConnectionRate.Threshold;
                    fastTimeNumericUpDown.Value = value.FastConnectionRate.ClearTime;
                }
                blockAllCheckBox.Checked = value.BlockAll;
                blockPortCheckBox.Checked = value.BlockOffensivePort;
                policyCheckBox.Checked = value.PolicyCreationHandleSpecified;
                if (value.PolicyCreationHandleSpecified)
                {
                    foreach (Utils.DropDownItem i in policyComboBox.Items)
                    {
                        if (((uint)i.Tag) == value.PolicyCreationHandle)
                        {
                            policyComboBox.SelectedItem = i;
                            break;
                        }
                    }
                }
                if (value.EncounterTimeout == 0)
                {
                    moveToStopCheckBox.Checked = true;
                    encounterTimeoutNumericUpDown.Value = 3600;
                }
                else
                {
                    moveToStopCheckBox.Checked = false;
                    encounterTimeoutNumericUpDown.Value = value.EncounterTimeout;
                }
                UpdateInfo();
            }
        }

        private void EditHeuristicForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            UpdateInfo();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
        }

        private void UpdateInfo()
        {
            if (policyComboBox.Items.Count == 0) policyCheckBox.Checked = false;
            policyComboBox.Enabled = policyCheckBox.Checked;
            encounterTimeoutNumericUpDown.Enabled = !moveToStopCheckBox.Checked;
            fastRateNumericUpDown.Enabled = fastCheckBox.Checked;
            fastTimeNumericUpDown.Enabled = fastCheckBox.Checked;
            slowRateNumericUpDown.Enabled = slowCheckBox.Checked;
            slowTimeNumericUpDown.Enabled = slowCheckBox.Checked;
        }

        private void policyCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void moveToStopCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void slowCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void fastCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }
    }
}
