﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Windows.Forms;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class EditTLSCertForm : Form
    {
        #region Private Members

        private AmtSystem _computer = null;

        private AmtSecurityAdmin.AmtStoredCert[] _tlsCerts = null;

        private AmtSecurityAdmin.AmtStoredCert _tlsCert = null;

        #endregion

        #region Constructors

        public EditTLSCertForm()
        {
            InitializeComponent();
        }

        #endregion

        #region Public Properties

        public AmtSystem Computer
        {
            get { return _computer; }
            set { _computer = value; }
        }

        public AmtSecurityAdmin.AmtStoredCert[] TlsCerts
        {
            get { return _tlsCerts; }
            set { _tlsCerts = value; }
        }

        public AmtSecurityAdmin.AmtStoredCert TlsCert
        {
            get { return _tlsCert; }
            set { _tlsCert = value; }
        }

        #endregion

        #region Private Methods

        private void EditTLSCertForm_Load(object sender, EventArgs e)
        {
            if (TlsCerts == null)
                return; // Nothing to do.

            Utils.DropDownItem selectedCert = null;

            // Populate the ComboBox with the certificates in the Intel(R) AMT system.
            foreach (AmtSecurityAdmin.AmtStoredCert tlsCert in TlsCerts)
            {
                Utils.DropDownItem i = new Utils.DropDownItem(ManageabilityStack.CertificateOperations.GetCertificateLongString(tlsCert.Certificate),
                    (int)tlsCert.Handle, tlsCert);
                certsComboBox.Items.Add(i);

                if (TlsCert == tlsCert)
                    selectedCert = i;
            } // END foreach (AmtSecurityAdmin.AmtStoredCert tlsCert in TlsCerts)

            certsComboBox.SelectedItem = selectedCert;
        }

        private void certsComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            Utils.DropDownItem selectedItem = certsComboBox.SelectedItem as Utils.DropDownItem;
            if (selectedItem == null) return; // Nothing to do.
            TlsCert = selectedItem.Tag as AmtSecurityAdmin.AmtStoredCert;
        }

        private void newCertButton_Click(object sender, EventArgs e)
        {
            ManageabilityControls.NewCertificateForm newCertForm = new ManageabilityControls.NewCertificateForm();
            newCertForm.CertificateName = Computer.HostNameIdentity.DefaultName;
            newCertForm.SHA2Support = (Computer.CoreVersion >= 6.0 ? true : false);
            if (newCertForm.ShowDialog(this) == DialogResult.OK)
            {
                byte[] privateKey = ManageabilityStack.CertificateOperations.GetPkcs1PrivateKeyFromCert(newCertForm.Certificate);
                uint handle;
                AmtCallStatus r = Computer.SecurityAdmin.CertStoreAddCertificate(newCertForm.Certificate, privateKey, out handle);
                if (r == AmtCallStatus.SUCCESS)
                {
                    // Update the ComboBox with the certificates in the Intel(R) AMT system.
                    r = Computer.SecurityAdmin.CertStoreGetCertificates(out _tlsCerts);
                    if (r != AmtCallStatus.SUCCESS) return; // Nothing to do.
                    certsComboBox.Items.Clear();
                    Utils.DropDownItem selectedCert = null;
                    foreach (AmtSecurityAdmin.AmtStoredCert tlsCert in TlsCerts)
                    {
                        Utils.DropDownItem i = new Utils.DropDownItem(ManageabilityStack.CertificateOperations.GetCertificateLongString(tlsCert.Certificate),
                            (int) tlsCert.Handle, tlsCert);
                        certsComboBox.Items.Add(i);

                        if (newCertForm.Certificate == tlsCert.Certificate)
                            selectedCert = i;
                    } // END foreach (AmtSecurityAdmin.AmtStoredCert tlsCert in TlsCerts)

                    certsComboBox.SelectedItem = selectedCert;
                }
                else
                {
                    MessageBox.Show(this, string.Format(Properties.Resources.AddCertificateError, r.ToString()),
                        Properties.Resources.AddCertificate, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;
            Close();
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        #endregion
    }
}
