// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Xml.Serialization;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class EditWirelessProfileForm : Form
    {
        private AmtSystem system;
        private XProfileType xProfile = null;

        public EditWirelessProfileForm(AmtSystem system)
        {
            InitializeComponent();
            this.system = system;
        }

        private void EditWirelessProfileForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            string st = (string)Application.UserAppDataRegistry.GetValue("HideWirelessPassword", bool.TrueString);
            hideCheckBox.Checked = (st.CompareTo(bool.TrueString) == 0);

            if (securityModelComboBox.Items.Count == 0)
            {
                securityModelComboBox.Items.Add(new Utils.DropDownItem(ManageabilityCommanderTool.Properties.Resources.EditWirelessForm_SecMod_WPA, 0, null));
                securityModelComboBox.Items.Add(new Utils.DropDownItem(ManageabilityCommanderTool.Properties.Resources.EditWirelessForm_SecMod_RSN, 0, null));
                securityModelComboBox.SelectedIndex = 0;

                encryptionComboBox.Items.Add(new Utils.DropDownItem(ManageabilityCommanderTool.Properties.Resources.EditWirelessForm_Encry_TKIP, 0, null));
                encryptionComboBox.Items.Add(new Utils.DropDownItem(ManageabilityCommanderTool.Properties.Resources.EditWirelessForm_Encry_CCMP, 0, null));
                encryptionComboBox.SelectedIndex = 0;
            }

            UpdateInfo();
        }

        private void UpdateInfo()
        {
            bool ok = true;
            if (profileNameTextBox.Text.Length < 1) ok = false;
            if (ssidTextBox.Text.Length < 1) ok = false;
            if (radioButton1.Checked && Utils.CheckPassPhrase(passPhraseMaskedTextBox.Text) == false) ok = false;
            if (radioButton3.Checked && (xProfile == null || xProfile.Item == null)) ok = false;
            okButton.Enabled = ok;

            if (hideCheckBox.Checked)
            {
                passPhraseMaskedTextBox.PasswordChar = Properties.Resources.PasswordChar[0];
                passPhraseMaskedTextBox2.PasswordChar = Properties.Resources.PasswordChar[0];
            }
            else
            {
                passPhraseMaskedTextBox.PasswordChar = '\0';
                passPhraseMaskedTextBox2.PasswordChar = '\0';
            }

            passPhraseMaskedTextBox.Enabled = radioButton1.Checked;
            passPhraseMaskedTextBox2.Enabled = radioButton2.Checked;
            xProfileEditButton.Enabled = radioButton3.Checked;
            xProfileLabel.Enabled = radioButton3.Checked;

            xProfileLabel.Text = xProfileEditForm.GetProfileString(xProfile);
        }

        private void profileNameTextBox_TextChanged(object sender, EventArgs e)
        {
            int ss = profileNameTextBox.SelectionStart;
            int sl = profileNameTextBox.SelectionLength;
            profileNameTextBox.Text = Utils.FilterWirelessProfileName(profileNameTextBox.Text);
            profileNameTextBox.SelectionStart = ss;
            profileNameTextBox.SelectionLength = sl;
            UpdateInfo();
        }

        private void ssidTextBox_TextChanged(object sender, EventArgs e)
        {
            if (UTF8Encoding.UTF8.GetByteCount(ssidTextBox.Text) > 32)
            {
                int ss = ssidTextBox.SelectionStart;
                int sl = ssidTextBox.SelectionLength;
                ssidTextBox.Text = ssidTextBox.Text.Substring(0, ssidTextBox.Text.Length - 1);
                ssidTextBox.SelectionStart = ss;
                ssidTextBox.SelectionLength = sl;
            }
            UpdateInfo();
        }

        private void passPhraseMaskedTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }


        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        public void SetWirelessProfile(AmtWireless.AmtWirelessProfile profile)
        {
            if (securityModelComboBox.Items.Count == 0)
            {
                securityModelComboBox.Items.Add(new Utils.DropDownItem(ManageabilityCommanderTool.Properties.Resources.EditWirelessForm_SecMod_WPA, 0, null));
                securityModelComboBox.Items.Add(new Utils.DropDownItem(ManageabilityCommanderTool.Properties.Resources.EditWirelessForm_SecMod_RSN, 0, null));
                securityModelComboBox.SelectedIndex = 0;

                encryptionComboBox.Items.Add(new Utils.DropDownItem(ManageabilityCommanderTool.Properties.Resources.EditWirelessForm_Encry_TKIP, 0, null));
                encryptionComboBox.Items.Add(new Utils.DropDownItem(ManageabilityCommanderTool.Properties.Resources.EditWirelessForm_Encry_CCMP, 0, null));
                encryptionComboBox.SelectedIndex = 0;
            }

            profileNameTextBox.Text = profile.Name;
            precedenceNumericUpDown.Value = profile.Priority;
            ssidTextBox.Text = profile.SSID;
            securityModelComboBox.SelectedIndex = (int)profile.SecurityType;
            encryptionComboBox.SelectedIndex = (int)profile.EncryptionType;
            if (profile.AuthenticationType == AmtWireless.AmtWirelessAuthenticationType.PassPhrase) radioButton1.Checked = true;
            if (profile.AuthenticationType == AmtWireless.AmtWirelessAuthenticationType.RawKey) radioButton2.Checked = true;
            if (profile.AuthenticationType == AmtWireless.AmtWirelessAuthenticationType.XProfile) radioButton3.Checked = true;
            passPhraseMaskedTextBox.Text = profile.PassPhrase63;
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            AmtWireless.AmtWirelessProfile profile = new AmtWireless.AmtWirelessProfile();

            profile.Name = profileNameTextBox.Text;
            profile.Priority = (byte)precedenceNumericUpDown.Value;
            profile.SSID = ssidTextBox.Text;
            profile.SecurityType = (AmtWireless.AmtWirelessSecurityType)securityModelComboBox.SelectedIndex;
            profile.EncryptionType = (AmtWireless.AmtWirelessEncryptionType)encryptionComboBox.SelectedIndex;
            if (radioButton1.Checked) profile.AuthenticationType = AmtWireless.AmtWirelessAuthenticationType.PassPhrase;
            if (radioButton2.Checked) profile.AuthenticationType = AmtWireless.AmtWirelessAuthenticationType.RawKey;
            if (radioButton3.Checked) profile.AuthenticationType = AmtWireless.AmtWirelessAuthenticationType.XProfile;
            profile.PassPhrase63 = passPhraseMaskedTextBox.Text;
            profile.RawKey256 = Utils.HexToBytes(passPhraseMaskedTextBox2.Text);
            profile.xProfile = xProfile;

            AmtCallStatus s = system.Wireless.AddWirelessProfile(profile);
            if (s == AmtCallStatus.DUPLICATE)
            {
                s = system.Wireless.UpdateWirelessProfile(profile);
            }
            if (s != AmtCallStatus.SUCCESS)
            {
                MessageBox.Show(this,
                    ManageabilityCommanderTool.Properties.Resources.EditWirelessForm_Addprofile_ErrStr + s.ToString(),
                    ManageabilityCommanderTool.Properties.Resources.EditWirelessForm_Addprofile_IAMTErr,
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else DialogResult = DialogResult.OK;
        }

        private void EditWirelessProfileForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            Application.UserAppDataRegistry.SetValue("HideWirelessPassword", hideCheckBox.Checked.ToString());
        }

    }
}
