// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Xml.Serialization;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class EditWirelessProfilesForm : Form
    {
        private AmtSystem system;
        private XProfileType xProfile = null;

        public EditWirelessProfilesForm(AmtSystem system)
        {
            InitializeComponent();
            this.system = system;
        }

        private void EditWirelessProfilesForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 

            FetchState();
            UpdateInfo();
        }

        private void FetchState()
        {
            AmtWireless.AmtWirelessProfile[] profiles = system.Wireless.GetWirelessProfiles();

            profilesListView.BeginUpdate();
            profilesListView.Items.Clear();
            foreach (AmtWireless.AmtWirelessProfile profile in profiles)
            {
                ListViewItem l = new ListViewItem(new string[] { profile.Priority.ToString(), profile.Name, profile.SSID, profile.SecurityType.ToString() + ", " + profile.EncryptionType.ToString() + ", " + profile.AuthenticationType.ToString() });
                l.Tag = profile;
                profilesListView.Items.Add(l);
            }
            profilesListView.EndUpdate();
        }

        private void UpdateInfo()
        {
            upButton.Enabled = (profilesListView.SelectedItems.Count == 1) && (profilesListView.SelectedIndices[0] > 0);
            downButton.Enabled = (profilesListView.SelectedItems.Count == 1) && (profilesListView.SelectedIndices[0] < (profilesListView.Items.Count - 1));
            removeButton.Enabled = (profilesListView.SelectedItems.Count == 1);
        }

        private void refreshButton_Click(object sender, EventArgs e)
        {
            system.Wireless.ClearCache();
            FetchState();
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void addButton_Click(object sender, EventArgs e)
        {
            EditWirelessProfileForm form = new EditWirelessProfileForm(system);
            if (profilesListView.SelectedItems.Count != 0)
            {
                form.SetWirelessProfile((AmtWireless.AmtWirelessProfile)(profilesListView.SelectedItems[0].Tag));
            }
            if (form.ShowDialog(this) == DialogResult.OK) FetchState();
        }

        private void passPhraseMaskedTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void profilesListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (profilesListView.SelectedItems.Count == 1)
            {
                addButton.Text = "Edit...";
            }
            else
            {
                addButton.Text = "Add...";
            }
            UpdateInfo();
        }

        private void removeButton_Click(object sender, EventArgs e)
        {
            if (profilesListView.SelectedItems.Count != 1) return;
            ListViewItem l = profilesListView.SelectedItems[0];
            AmtWireless.AmtWirelessProfile profile = (AmtWireless.AmtWirelessProfile)l.Tag;
            system.Wireless.RemoveWirelessProfile(profile);
            FetchState();
            profilesListView_SelectedIndexChanged(this,null);
        }

        private void downButton_Click(object sender, EventArgs e)
        {
            if (profilesListView.SelectedItems.Count != 1) return;

            int s = profilesListView.SelectedIndices[0];

            AmtWireless.AmtWirelessProfile profile1 = (AmtWireless.AmtWirelessProfile)profilesListView.SelectedItems[0].Tag;
            AmtWireless.AmtWirelessProfile profile2 = (AmtWireless.AmtWirelessProfile)profilesListView.Items[profilesListView.SelectedIndices[0]+1].Tag;

            byte tempPri1 = profile1.Priority;
            byte tempPri2 = profile2.Priority;

            profile1.Priority = 255;
            system.Wireless.UpdateWirelessProfile(profile1);

            profile2.Priority = tempPri1;
            system.Wireless.UpdateWirelessProfile(profile2);
            profile1.Priority = tempPri2;
            system.Wireless.UpdateWirelessProfile(profile1);

            FetchState();
            profilesListView.SelectedIndices.Add(s+1);
        }

        private void upButton_Click(object sender, EventArgs e)
        {
            if (profilesListView.SelectedItems.Count != 1) return;

            int s = profilesListView.SelectedIndices[0];

            AmtWireless.AmtWirelessProfile profile1 = (AmtWireless.AmtWirelessProfile)profilesListView.SelectedItems[0].Tag;
            AmtWireless.AmtWirelessProfile profile2 = (AmtWireless.AmtWirelessProfile)profilesListView.Items[profilesListView.SelectedIndices[0] - 1].Tag;

            byte tempPri1 = profile1.Priority;
            byte tempPri2 = profile2.Priority;

            profile1.Priority = 255;
            system.Wireless.UpdateWirelessProfile(profile1);

            profile2.Priority = tempPri1;
            system.Wireless.UpdateWirelessProfile(profile2);
            profile1.Priority = tempPri2;
            system.Wireless.UpdateWirelessProfile(profile1);

            FetchState();
            profilesListView.SelectedIndices.Add(s-1);
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (profilesListView.SelectedItems.Count != 1) return;
            AmtWireless.AmtWirelessProfile profile = (AmtWireless.AmtWirelessProfile)profilesListView.SelectedItems[0].Tag;
            Utils.SerializeToClipboard(profile, typeof(AmtWireless.AmtWirelessProfile));
        }

        private void pasteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AmtWireless.AmtWirelessProfile profile = (AmtWireless.AmtWirelessProfile)Utils.UnSerializeFromClipboard(typeof(AmtWireless.AmtWirelessProfile));
            if (profile != null)
            {
                profile.Priority = 0;
                if (profilesListView.Items.Count != 0)
                {
                    AmtWireless.AmtWirelessProfile profile2 = (AmtWireless.AmtWirelessProfile)profilesListView.Items[profilesListView.Items.Count - 1].Tag;
                    profile.Priority = (byte)(profile2.Priority + 1);
                }

                AmtCallStatus s = system.Wireless.AddWirelessProfile(profile);
                if (s == AmtCallStatus.DUPLICATE)
                {
                    foreach (ListViewItem l in profilesListView.Items)
                    {
                        AmtWireless.AmtWirelessProfile p = (AmtWireless.AmtWirelessProfile)l.Tag;
                        if (profile.Name.CompareTo(p.Name) == 0)
                        {
                            profile.Priority = p.Priority;
                            break;
                        }
                    }
                    s = system.Wireless.UpdateWirelessProfile(profile);
                }
                if (s != AmtCallStatus.SUCCESS)
                {
                    MessageBox.Show(this,
                        ManageabilityCommanderTool.Properties.Resources.EditWirelessForm_Addprofile_ErrStr + s.ToString(),
                        ManageabilityCommanderTool.Properties.Resources.EditWirelessForm_Addprofile_IAMTErr,
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                FetchState();
            }
        }

        private void profileContextMenuStrip_Opening(object sender, CancelEventArgs e)
        {
            copyToolStripMenuItem.Enabled = (profilesListView.SelectedItems.Count == 1);
            pasteToolStripMenuItem.Enabled = (Utils.UnSerializeFromClipboard(typeof(AmtWireless.AmtWirelessProfile)) != null);
        }

        private void profilesListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control == true && e.KeyValue == 'C') { copyToolStripMenuItem_Click(this, null); e.Handled = true; }
            if (e.Control == true && e.KeyValue == 'V') { pasteToolStripMenuItem_Click(this, null); e.Handled = true; }
            if (e.KeyCode == Keys.Delete) { removeButton_Click(this, null); e.Handled = true; }
        }

        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void xProfileEditButton_Click(object sender, EventArgs e)
        {
            xProfileEditForm form = new xProfileEditForm(system, null);
            form.SetProfile(xProfile);
            if (form.ShowDialog(this) == DialogResult.OK)
            {
                xProfile = form.GetProfile();
                UpdateInfo();
            }
        }

    }
}
