// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class EndpointAccessControlEditForm : Form
    {
        private AmtSystem system;
        private bool enabled;

        public EndpointAccessControlEditForm(AmtSystem system)
        {
            this.system = system;
            InitializeComponent();
        }

        private void EndpointAccessControlEditForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            AmtCallStatus r = system.NetworkAccessControlAdmin.GetEACStatus(out enabled);

            uint signer;
            bool signerSpecified;
            r = system.NetworkAccessControlAdmin.GetPosture(out signer, out signerSpecified);

            clientCertificateComboBox.Items.Clear();
            clientCertificateComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Disabled, 0, null));
            clientCertificateComboBox.SelectedIndex = 0;

            // Add client certificate list
            AmtSecurityAdmin.AmtStoredCert[] certs;
            if (system.SecurityAdmin.CertStoreGetCertificates(out certs) == AmtCallStatus.SUCCESS) clientCertificateComboBox.Items.AddRange(certs);
            if (enabled && signerSpecified) SetClientCertificate(signer);
        }

        private void resetButton_Click(object sender, EventArgs e)
        {
            AmtCallStatus r = system.NetworkAccessControlAdmin.UpdatePostureState(PostureUpdateType.PostureUpdateBootCounters);
            if (r != AmtCallStatus.SUCCESS) Utils.ErrorMessageBox(this, "UpdatePostureState", r);
        }

        private void SetClientCertificate(uint handle)
        {
            for (int i = 0; i < clientCertificateComboBox.Items.Count; i++)
            {
                if (clientCertificateComboBox.Items[i].GetType() == typeof(AmtSecurityAdmin.AmtStoredCert))
                {
                    AmtSecurityAdmin.AmtStoredCert cert = (AmtSecurityAdmin.AmtStoredCert)clientCertificateComboBox.Items[i];
                    if (cert.Handle == handle) clientCertificateComboBox.SelectedIndex = i;
                }
            }
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            AmtCallStatus r;
            if (clientCertificateComboBox.SelectedIndex == 0)
            {
                r = system.NetworkAccessControlAdmin.EnableEAC(false);
                if (r != AmtCallStatus.SUCCESS) { Utils.ErrorMessageBox(this, "EnableEAC", r); return; }
                r = system.NetworkAccessControlAdmin.SetPostureSigner(0, false);
                if (r != AmtCallStatus.SUCCESS) { Utils.ErrorMessageBox(this, "SetPostureSigner", r); return; }
            }
            else
            {
                AmtSecurityAdmin.AmtStoredCert c = (AmtSecurityAdmin.AmtStoredCert)clientCertificateComboBox.SelectedItem;
                r = system.NetworkAccessControlAdmin.SetPostureSigner((uint)(c.Handle), true);
                if (r != AmtCallStatus.SUCCESS) { Utils.ErrorMessageBox(this, "SetPostureSigner", r); return; }
                r = system.NetworkAccessControlAdmin.EnableEAC(true);
                if (r != AmtCallStatus.SUCCESS) { Utils.ErrorMessageBox(this, "EnableEAC", r); return; }
            }

            DialogResult = DialogResult.OK;
        }

    }
}
