// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class EventFilterForm : Form
    {
        private AmtSystem system;

        public EventFilterForm(AmtSystem computer)
        {
            this.system = computer;

            InitializeComponent();

            UpdateInfo();
        }

        private void EventFilterForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            DisplayFilters();
        }

        private class EventFilterSorter : IComparer
        {
            #region IComparer Members

            public int Compare(object x, object y)
            {
                ListViewItem lvi1 = (ListViewItem)x;
                ListViewItem lvi2 = (ListViewItem)y;
                AmtEventFilter amtef1 = (AmtEventFilter) lvi1.Tag;
                AmtEventFilter amtef2 = (AmtEventFilter)lvi2.Tag;
                if (amtef1.Filter.AlertSubscriptionPolicyID < amtef2.Filter.AlertSubscriptionPolicyID)
                    return -1;
                else if (amtef1.Filter.AlertSubscriptionPolicyID == amtef2.Filter.AlertSubscriptionPolicyID)
                    return 0;
                else
                    return 1;
            }

            #endregion
        }

        private void DisplayFilters()
        {
            AmtEventFilter[] filters = system.Events.GetEventFilters();
            ArrayList ll = new ArrayList();

            foreach (AmtEventFilter amtfilter in filters)
            {
                EventFilterType filter = amtfilter.Filter;

                string fab = "";
                if ((filter.FilterConfiguration & 0x40) != 0) fab = "*";

                ListViewItem l = new ListViewItem(new string[] { filter.AlertSubscriptionPolicyID.ToString() + fab, amtfilter.GetFilterString(), amtfilter.GetActionString() });
                l.Tag = amtfilter;
                ll.Add(l);
            }
            filtersListView.BeginUpdate();
            filtersListView.Items.Clear();
            filtersListView.Items.AddRange((ListViewItem[])ll.ToArray(typeof(ListViewItem)));
            filtersListView.Sorting = SortOrder.Ascending;
            filtersListView.ListViewItemSorter = new EventFilterSorter();
            filtersListView.Sort();
            filtersListView.EndUpdate();
        }

        private void refreshButton_Click(object sender, EventArgs e)
        {
            system.Events.ClearFiltersCache();
            DisplayFilters();
        }

        private void removeButton_Click(object sender, EventArgs e)
        {
            if (filtersListView.SelectedItems.Count == 0) return;
            foreach (ListViewItem s in filtersListView.SelectedItems)
            {
                Application.DoEvents();
                AmtEventFilter amtFilter = (AmtEventFilter)s.Tag;
                system.Events.RemoveEventFilter(amtFilter);
            }
            refreshButton_Click(this, null);
        }

        private void filtersListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void UpdateInfo()
        {
            removeButton.Enabled = editButton.Enabled = (filtersListView.SelectedItems.Count != 0);
        }

        private void addButton_Click(object sender, EventArgs e)
        {
            AddEventFilterForm form = new AddEventFilterForm();
            if (form.ShowDialog(this) == DialogResult.OK)
            {
                EventFilterType filter = form.GetFilter();
                try
                {
                    AmtCallStatus r = system.Events.AddEventFilter(filter);
                    if (r != AmtCallStatus.SUCCESS)
                    {
                        MessageBox.Show(this,
                            ManageabilityCommanderTool.Properties.Resources.AddEvt_Ret_Str + r.ToString(), 
                            ManageabilityCommanderTool.Properties.Resources.EvtFltForm_EvtFlt_Err,
                            MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
                catch (System.Web.Services.Protocols.SoapHeaderException)
                {
                    MessageBox.Show(this,
                        ManageabilityCommanderTool.Properties.Resources.EvtFltForm_Add_ErrStr,
                        ManageabilityCommanderTool.Properties.Resources.EvtFltForm_EvtFlt_Err,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
                refreshButton_Click(this, null);
            }
        }

        private void editButton_Click(object sender, EventArgs e)
        {
            if (filtersListView.SelectedItems.Count == 0) return;
            AmtEventFilter amtFilter = (AmtEventFilter)filtersListView.SelectedItems[0].Tag;

            AddEventFilterForm form = new AddEventFilterForm();
            form.SetFilter(amtFilter.Filter);
            form.PolicyIdReadOnly = true;

            if (form.ShowDialog(this) == DialogResult.OK)
            {
                EventFilterType filter = form.GetFilter();
                
                // Remove old filter if needed
                if (filter.AlertSubscriptionPolicyID == amtFilter.Filter.AlertSubscriptionPolicyID)
                {
                    system.Events.RemoveEventFilter(amtFilter);
                }

                // Add new filter
                try
                {
                    AmtCallStatus r = system.Events.AddEventFilter(filter);
                    if (r != AmtCallStatus.SUCCESS)
                    {
                        MessageBox.Show(this,
                            ManageabilityCommanderTool.Properties.Resources.AddEvt_Ret_Str + r.ToString(),
                            ManageabilityCommanderTool.Properties.Resources.EvtFltForm_EvtFlt_Err,
                            MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
                catch (System.Web.Services.Protocols.SoapHeaderException)
                {
                    MessageBox.Show(this,
                        ManageabilityCommanderTool.Properties.Resources.EvtFltForm_Add_ErrStr,
                        ManageabilityCommanderTool.Properties.Resources.EvtFltForm_EvtFlt_Err,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }

                refreshButton_Click(this, null);
            }
        }

        private void filtersListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control == true && e.KeyValue == 'C') { copyToolStripMenuItem_Click(this, null); e.Handled = true; }
            if (e.Control == true && e.KeyValue == 'V') { pasteToolStripMenuItem_Click(this, null); e.Handled = true; }
            if (e.KeyCode == Keys.Delete) { removeButton_Click(this, null); e.Handled = true; }
            if (e.Control == true && e.KeyValue == 'A')
            {
                if (filtersListView.Items.Count > 0)
                {
                    filtersListView.SelectedIndices.Clear();
                    filtersListView.SelectedIndices.Add(0);
                    filtersListView.Focus();
                    SendKeys.Send("{HOME}");
                    SendKeys.Send("+{END}");
                    e.Handled = true;
                }
            }
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (filtersListView.SelectedItems.Count == 0) return;
            ArrayList ll = new ArrayList();
            foreach (ListViewItem l in filtersListView.SelectedItems)
            {
                AmtEventFilter amtFilter = (AmtEventFilter)l.Tag;
                ll.Add(amtFilter);
            }
            AmtEventFilter[] amtFilters = (AmtEventFilter[])ll.ToArray(typeof(AmtEventFilter));
            Utils.SerializeToClipboard(amtFilters, typeof(AmtEventFilter[]));
        }

        private void pasteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AmtEventFilter[] amtFilters = (AmtEventFilter[])Utils.UnSerializeFromClipboard(typeof(AmtEventFilter[]));
            if (amtFilters != null)
            {
                foreach (AmtEventFilter amtFilter in amtFilters)
                {
                    Application.DoEvents();
                    EventFilterType filter = amtFilter.Filter;
                    try
                    {
                        AmtCallStatus r = system.Events.AddEventFilter(filter);
                        if (r != AmtCallStatus.SUCCESS)
                        {
                            MessageBox.Show(this,
                                ManageabilityCommanderTool.Properties.Resources.AddEvt_Ret_Str + r.ToString(),
                                ManageabilityCommanderTool.Properties.Resources.EvtFltForm_EvtFlt_Err,
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }
                    catch (System.Web.Services.Protocols.SoapHeaderException)
                    {
                        MessageBox.Show(this,
                            ManageabilityCommanderTool.Properties.Resources.EvtFltForm_Add_ErrStr,
                            ManageabilityCommanderTool.Properties.Resources.EvtFltForm_EvtFlt_Err,
                            MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                }
                refreshButton_Click(this, null);
            }
        }

        private void filterContextMenuStrip_Opening(object sender, CancelEventArgs e)
        {
            copyToolStripMenuItem.Enabled = (filtersListView.SelectedItems.Count != 0);
            pasteToolStripMenuItem.Enabled = (Utils.UnSerializeFromClipboard(typeof(AmtEventFilter[])) != null);
        }

    }
}
