// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Printing;
using System.IO;
using System.Net;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Serialization;
using ManageabilityControls;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class MainForm : Form
    {
        public TreeNode rootNode = null;
        public TreeNode computerRootNode = null;
        private Panels.DiscoveryPanel discoverypanel = null;
        private Hashtable policyMonitorTable = new Hashtable();
        public BatchForm batchForm = null;
        public bool disposing = false;
        private Panels.PanelInterface CurrentPanelControl = null;
        private Thread FetchCacheThread = null;

        private string one_hostname = null;
        private string one_username = null;
        private string one_password = null;
        private bool one_tls = false;
        private bool one_tls_specified = false;
        public bool Simplified = false;
        public TreeNode DisplayedNode = null;
        public AmtTreeView GetTreeView()
        {
            return mainTreeView;
        }
        public void UpdateTreeView()
        {
            mainTreeView_AfterSelect(this, null);
        }

        public MainForm(string[] args)
        {
            InitializeComponent();

            // Delete the settings file if invalid, useful when developing for MONO in Visual Studio.
            try
            {
                Properties.Settings.Default.WsmanSupport.ToString();
            }
            catch (Exception)
            {
                File.Delete(Application.ExecutablePath + ".config");
            }

            string proxystring = null;
            foreach (string arg in args)
            {
                if (arg.Length > 3 && string.Compare(arg.Substring(0, 3), "-h:", true) == 0)
                {
                    one_hostname = arg.Substring(3);
                }
                if (arg.Length > 3 && string.Compare(arg.Substring(0, 3), "-u:", true) == 0)
                    one_username = arg.Substring(3);
                if (arg.Length > 3 && string.Compare(arg.Substring(0, 3), "-p:", true) == 0)
                    one_password = arg.Substring(3);
                if (arg.Length > 7 && string.Compare(arg.Substring(0, 7), "-proxy:", true) == 0)
                    proxystring = arg.Substring(7);
                if (string.Compare(arg, "-simple", true) == 0)
                    Simplified = true;
                if (string.Compare(arg, "-tls", true) == 0)
                {
                    one_tls_specified = true;
                    one_tls = true;
                }
                if (string.Compare(arg, "-notls", true) == 0)
                {
                    one_tls_specified = true;
                    one_tls = false;
                }
            }

            if (!Utils.IsMono())
            {
                mainToolTip.IsBalloon = true;
                if (proxystring == null)
                {
                    WebRequest.DefaultWebProxy = null; // We will not be using any web proxy.
                }
                else
                {
                    WebRequest.DefaultWebProxy = new WebProxy(proxystring);
                }
            }

            if (Utils.IsMono())
            {
                // Support only simplified UI in MONO
                displayToolTipsToolStripMenuItem.Visible = false;
                //Simplified = true;
                splitContainer1.SplitterDistance = 253;
                //manualAddButton.Text = Properties.Resources.AddComputer;
                //languageToolStripMenuItem.Visible = false;
                //toolStripMenuItem7.Visible = false;
                //aboutToolStripMenuItem.Visible = false;
                //viewToolStripMenuItem.Visible = false;
                //mainTreeView.ContextMenuStrip = null;
                //tabControl2.TabPages.RemoveAt(1); // Remove the "Remote Control" tab page from the computer panel.
            }

            if (one_hostname == null || one_username == null || one_password == null)
            {
                one_hostname = null;
                one_username = null;
                one_password = null;
                one_tls = false;
            }
            else
            {
                if (one_tls_specified == false)
                    one_tls = !one_hostname.EndsWith(":16992");
                if (one_hostname != null && one_hostname.Length >= 9 && string.Compare(one_hostname.Substring(0, 9), "localhost", true) == 0)
                {
                    one_hostname = one_hostname.Replace("localhost", Utils.GetLocalConnectFQDN());
                }
            }

            if (one_hostname == null)
            {
                rootNode = new TreeNode(ManageabilityCommanderTool.Properties.Resources.TreeNode_Network, 4, 5);                

                mainTreeView.Nodes.Add(rootNode);
                mainTreeView.SelectedNode = rootNode;

                //add a localhost node.  this may be replaced by a saved localhost
                //TODO: figure out if there is a good precheck to make for if system is not AMT
                //or does not support local host config.
                AmtSystem tempLocal = new AmtSystem(Utils.GetLocalConnectFQDN(), 16992, "admin", "", true, false, true);
                SetLocalComputer(tempLocal);

                byte[] systemSettingsBin = (byte[])Application.UserAppDataRegistry.GetValue("SystemsSettings2", null);

                if (systemSettingsBin != null)
                {
                    ArrayList settings = null;

                    try
                    {
                        settings = (ArrayList)Utils.UnserializeBinary(systemSettingsBin);
                    }
                    catch (Exception)
                    {
                    }

                    if (settings == null)
                    {
                        Application.UserAppDataRegistry.DeleteSubKey("SystemsSettings2", false);
                        MessageBox.Show(
                            ManageabilityCommanderTool.Properties.Resources.MainForm_Reg_ErrStr,
                            ManageabilityCommanderTool.Properties.Resources.MainForm_RegSetting,
                            MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    }
                    else
                    {
                        foreach (object o in settings)
                        {
                            if (o.GetType() != typeof(AmtSerialSystemSettings))
                                continue;
                            AmtSerialSystemSettings setting = (AmtSerialSystemSettings)o;
                            if (setting.MPS)
                            {
                                if (setting.Hostname.Equals("~~MPS~~"))
                                {
                                    // MPS
                                    AmtMpsServer m = setting.GetMPS();
                                    if (m != null)
                                        AddMps(m);
                                }
                            }
                            else
                            {
                                if (setting.MyComputer)
                                {
                                    //update the hostname for the local system, since this may 
                                    //have changed.  use the rest of the saved settings "myComputer"
                                    AmtSystem tempMyComputer = setting.GetSystem();                                   
                                    tempMyComputer.HostNameIdentity = tempLocal.HostNameIdentity;
                                    SetLocalComputer(tempMyComputer);
                                }
                                else
                                    AddComputer(setting.GetSystem());
                            }
                        }
                    }
                }

                rootNode.Expand();
            }

            if (ManageabilityStack.CertificateOperations.AllTrustedRootsToPemFile(Application.StartupPath + "\\Trusted Root Certificates.pem") == false)
            {
                MessageBox.Show(
                    ManageabilityCommanderTool.Properties.Resources.MainForm_Gen_Cert_ErrStr,
                    ManageabilityCommanderTool.Properties.Resources.MainForm_Trust_Cert,
                    MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }

            FileInfo vendorinfo = new FileInfo(Application.StartupPath + "\\vendors.txt");
            if (vendorinfo.Exists)
                Utils.LoadPciDeviceList(vendorinfo);
            else
            {
                vendorinfo = new FileInfo(Application.StartupPath + "\\docs\\vendors.txt");
                if (vendorinfo.Exists)
                    Utils.LoadPciDeviceList(vendorinfo);
            }

            // Preload some of the classes we will need.
            new Thread(new ThreadStart(AmtSystem.PreLoadModules)).Start();
        }

        public void AddComputer(AmtSystem computer)
        {
            int icon = GetIconIndex(computer);
            TreeNode systemNode = new TreeNode("", icon, icon);
            systemNode.Tag = computer;
            systemNode.Text = GetNodeName(systemNode);
            computer.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
            computer.OnFetchCompleted += new AmtSystem.FetchHandler(FetchHandler);
            if (rootNode != null)
            {
                //// IGNORE SYSTEM LIMIT FOR INTEL CONFIDENTIAL INTERNAL RELEASES.
                //// Limit Commander to 20 computers
                //if (Utils.GetNodeTagsofType(rootNode, typeof(AmtSystem)).Count >= 20)
                //{
                //    MessageBox.Show("No more than 20 computers can be added", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                //}
                //else
                //{
                    Utils.AddTreeNodeSorted(rootNode, systemNode);
                //}
            }
            else
            {
                mainTreeView.Nodes.Add(systemNode);
                computerRootNode = systemNode;
            }
            if (Utils.IsMono())
                mainTreeView.Invalidate();
        }

        public void AddComputer(AmtSystem computer, TreeNode node)
        {
            int icon = GetIconIndex(computer);
            TreeNode systemNode = new TreeNode("", icon, icon);
            systemNode.Tag = computer;
            systemNode.Text = GetNodeName(systemNode);
            computer.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
            computer.OnFetchCompleted += new AmtSystem.FetchHandler(FetchHandler);
            Utils.AddTreeNodeSorted(node, systemNode);
            if (Utils.IsMono())
                mainTreeView.Invalidate();
        }

        public void SetLocalComputer(AmtSystem computer)
        {
            int icon = GetIconIndex(computer);
            TreeNode systemNode = new TreeNode("", icon, icon);
            systemNode.Tag = computer;
            systemNode.Text = ManageabilityCommanderTool.Properties.Resources.TreeNode_MyComputer;
            computer.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
            computer.OnFetchCompleted += new AmtSystem.FetchHandler(FetchHandler);

            //second node should be the "My Computer" local computer node
            //remove node if it already exists
            if (mainTreeView.Nodes.Count >= 2)
                mainTreeView.Nodes.RemoveAt(1);

            //add local computer node
            mainTreeView.Nodes.Add(systemNode); 
            
            if (Utils.IsMono())
                mainTreeView.Invalidate();
        }

        private int GetIconIndex(AmtSystem computer)
        {
            int icon = 15;
            if (computer.State == AmtSystemObjState.Connected)
            {
                icon += 2;
                if (computer.Warnings != 0)
                    icon += 1;
            }
            else if (computer.State == AmtSystemObjState.Connecting)
                icon += 1;
            if (computer.IsMobile)
                icon += 4;
            return icon;
        }

        private void AddMps(AmtMpsServer mps)
        {
            TreeNode systemNode = new TreeNode("MPS: " + mps.NetworkNameIdentity.DisplayName, 27, 27);
            systemNode.Tag = mps;
            if (rootNode != null)
            {
                Utils.AddTreeNodeSorted(rootNode, systemNode);
            }
            else
            {
                mainTreeView.Nodes.Add(systemNode);
                computerRootNode = systemNode;
            }
            foreach (AmtSystem computer in mps.Computers)
            {
                AddComputer(computer, systemNode);
            }
            if (Utils.IsMono())
                mainTreeView.Invalidate();
        }

        /// <summary>
        /// This class is used to add items to DropDown combo boxes and still be able to reference the item quickly.
        /// </summary>
        private class DropDownItem
        {
            public string Text = null;
            public object Tag = null;
            public int Handle = 0;
            public override string ToString()
            {
                return Text;
            }

            public DropDownItem()
            {
            }

            public DropDownItem(string Text, int Handle, object Tag)
            {
                this.Text = Text;
                this.Handle = Handle;
                this.Tag = Tag;
            }
        }

        public string InfoTextStart()
        {
            return "";
        }

        public string InfoTextAdd(string title, string value)
        {
            if (value == null)
                value = Properties.Resources.Null;
            return string.Format("{0}\r\n    {1}\r\n", title, value.Replace("\r\n", "\r\n    "));
        }

        public string InfoTextEnd()
        {
            return "";
        }

        public void SaveSettings()
        {
            if (rootNode == null)
                return;
            ArrayList settings = new ArrayList();
            foreach (TreeNode n in rootNode.Nodes)
            {
                if (n.Tag.GetType() == typeof(AmtSystem))
                {
                    AmtSystem s = (AmtSystem)n.Tag;
                    settings.Add(new ManageabilityStack.AmtSerialSystemSettings(s));
                }
                else if (n.Tag.GetType() == typeof(AmtMpsServer))
                {
                    AmtMpsServer m = (AmtMpsServer)n.Tag;
                    settings.Add(new ManageabilityStack.AmtSerialSystemSettings(m));
                }
            }

            foreach (TreeNode n in mainTreeView.Nodes)
            {
                if (n.Tag != null && n.Tag.GetType() == typeof(AmtSystem))
                {
                    AmtSystem s = (AmtSystem)n.Tag;
                    settings.Add(new ManageabilityStack.AmtSerialSystemSettings(s));
                }
            }

            byte[] buf = Utils.SerializeBinary(settings);
            Application.UserAppDataRegistry.SetValue("SystemsSettings2", buf);
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void RightViewEx(Type newPanelType)
        {
            if (CurrentPanelControl == null || CurrentPanelControl.GetType() != newPanelType)
            {
                // Setup the new panel
                if (CurrentPanelControl != null)
                    CurrentPanelControl.Cleanup();
                CurrentPanelControl = (Panels.PanelInterface)Activator.CreateInstance(newPanelType);
                ((UserControl)CurrentPanelControl).Dock = DockStyle.Fill;
                ((UserControl)CurrentPanelControl).BorderStyle = BorderStyle.Fixed3D;
                splitContainer1.Panel2.Controls.Add((Control)CurrentPanelControl);
                splitContainer1.Panel2.Controls.RemoveAt(0);
            }
        }

        private void RightViewEx(Panels.PanelInterface newPanel)
        {
            if (CurrentPanelControl == null || CurrentPanelControl != newPanel)
            {
                // Setup the new panel
                if (CurrentPanelControl != null)
                    CurrentPanelControl.Cleanup();
                CurrentPanelControl = newPanel;
                ((UserControl)CurrentPanelControl).Dock = DockStyle.Fill;
                ((UserControl)CurrentPanelControl).BorderStyle = BorderStyle.Fixed3D;
                splitContainer1.Panel2.Controls.Add((Control)CurrentPanelControl);
                splitContainer1.Panel2.Controls.RemoveAt(0);
            }
        }

        private void mainTreeView_ArrangeContextMenu()
        {
            TreeNode node = (TreeNode)mainTreeView.SelectedNode;

            if (node == null)
            {
                foreach (object o in editToolStripMenuItem.DropDownItems)
                {
                    if (o.GetType() == typeof(ToolStripMenuItem))
                    {
                        ToolStripMenuItem mi = (ToolStripMenuItem)o;
                        mi.Visible = false;
                        mi.Tag = false;
                    }
                    else if (o.GetType() == typeof(ToolStripSeparator))
                    {
                        ToolStripSeparator mi = (ToolStripSeparator)o;
                        mi.Visible = false;
                        mi.Tag = false;
                    }
                }
                return;
            }

            if (node.Tag != null && node.Tag.GetType() == typeof(CircuitBreakerPolicyInfoType))
            {
                CircuitBreakerPolicyInfoType cbp = (CircuitBreakerPolicyInfoType)node.Tag;
                AmtSystem computer = (AmtSystem)node.Parent.Parent.Parent.Tag;

                uint MaxSupportedPolicies;
                uint MaxSupportedFilters;
                CircuitBreakerCapabilitiesType[] cbcs = computer.CircuitBreaker.GetCircuitBreakerCapabilities(out MaxSupportedPolicies, out MaxSupportedFilters);
                wiredNICToolStripMenuItem.Tag = (cbcs.Length > 0);
                wirelessNICToolStripMenuItem.Tag = (cbcs.Length > 1);

                wiredNICToolStripMenuItem.Checked = computer.CircuitBreaker.IsPolicyActive(cbp.PolicyCreationHandle, 0);
                wirelessNICToolStripMenuItem.Checked = computer.CircuitBreaker.IsPolicyActive(cbp.PolicyCreationHandle, 1);
            }

            addAMTSystemToolStripMenuItem.Tag = (node == rootNode || (node.Tag != null && node.Tag.GetType() == typeof(AmtMpsServer)) );           
            addMPSServerToolStripMenuItem1.Tag = (node == rootNode && Simplified == false);
            removeSystemToolStripMenuItem.Tag = (node.Tag != null && node.Tag.GetType() == typeof(AmtSystem));
            unprovisionToolStripMenuItem.Tag = false;
            if (node.Tag != null && node.Tag.GetType() == typeof(AmtSystem))
            {
                // Only show Unprovision option if system is connected (we need the SAS service)
                AmtSystem sys = (AmtSystem)node.Tag;
                if (sys.State == AmtSystemObjState.Connected)
                    unprovisionToolStripMenuItem.Tag = true;
            }
            removeMPSToolStripMenuItem.Tag = (node.Tag != null && node.Tag.GetType() == typeof(AmtMpsServer));            
            openWebInterfaceToolStripMenuItem.Tag = (node.Tag != null && node.Tag.GetType() == typeof(AmtSystem));
            if (!Utils.IsMono())
                takeControlToolStripMenuItem.Tag = (node.Tag != null && ((node.Tag.GetType() == typeof(AmtSystem) && (((AmtSystem)node.Tag).State == AmtSystemObjState.Connected))));
            else
                takeControlToolStripMenuItem.Tag = false;
            connectToolStripMenuItem.Tag = (node.Tag != null && ((node.Tag.GetType() == typeof(AmtSystem) && (((AmtSystem)node.Tag).State == AmtSystemObjState.Disconnected))));
            disconnectToolStripMenuItem.Tag = (node.Tag != null && ((node.Tag.GetType() == typeof(AmtSystem) && (((AmtSystem)node.Tag).State == AmtSystemObjState.Connected))));
            refreshFiltersToolStripMenuItem.Tag = (node.Tag == null && node.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Filters) == 0);
            addFilterToolStripMenuItem.Tag = (node.Tag == null && node.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Filters) == 0);
            removeFilterToolStripMenuItem.Tag = (node.Tag != null && node.Tag.GetType() == typeof(CircuitBreakerFilterInfoType));
            refreshPoliciesToolStripMenuItem.Tag = (node.Tag == null && node.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Policies) == 0);
            addPolicyToolStripMenuItem.Tag = (node.Tag == null && node.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Policies) == 0);
            removePolicyToolStripMenuItem.Tag = (node.Tag != null && node.Tag.GetType() == typeof(CircuitBreakerPolicyInfoType));
            activateOnToolStripMenuItem.Tag = (node.Tag != null && node.Tag.GetType() == typeof(CircuitBreakerPolicyInfoType));
            policyMonitorToolStripMenuItem.Tag = ((mainTreeView.SelectedNode.Tag != null) && (mainTreeView.SelectedNode.Tag.GetType() == typeof(CircuitBreakerPolicyInfoType)) || mainTreeView.SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Policies) == 0 || mainTreeView.SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Networking) == 0);
            refreshAssetsToolStripMenuItem.Tag = (node.Tag == null && node.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Hardware_Asset) == 0);
            refreshWatchdogsToolStripMenuItem.Tag = (node.Tag == null && node.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Watchdogs) == 0);
            addWatchdogToolStripMenuItem.Tag = (node.Tag == null && node.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Watchdogs) == 0);
            removeWatchdogToolStripMenuItem.Tag = (node.Tag != null && node.Tag.GetType() == typeof(AmtWatchdogAgent));
            refreshStorageToolStripMenuItem.Tag = (node.Tag != null && node.Tag.GetType() == typeof(AmtStorage));
            addEditEnterprisesToolStripMenuItem.Tag = (node.Tag != null && node.Tag.GetType() == typeof(AmtStorage));
            removeEnterpriseToolStripMenuItem.Tag = (node.Tag != null && node.Tag.GetType() == typeof(AmtEnterprise));
            removeApplicationToolStripMenuItem.Tag = (node.Tag != null && node.Tag.GetType() == typeof(AmtApplication));
            removeBlockToolStripMenuItem.Tag = (node.Tag != null && node.Tag.GetType() == typeof(AmtStorageBlock));
            allocateBlockToolStripMenuItem.Tag = (node.Tag != null && (node.Tag.GetType() == typeof(AmtEnterprise) || node.Tag.GetType() == typeof(AmtApplication)));
            refreshEventsToolStripMenuItem.Tag = (node.Tag == null && node.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Event_Log) == 0);

            // Disable the "Take Control" menu if TLS & connection warnings occured
            if (((bool)takeControlToolStripMenuItem.Tag) == true && node.Tag.GetType() == typeof(AmtSystem))
            {
                bool redirectionEnabled = true;
                AmtSystem s = (AmtSystem)node.Tag;
                if (s.State != AmtSystemObjState.Connected)
                    redirectionEnabled = false;
                else if (s.UseTls == true && s.Warnings != 0)
                    redirectionEnabled = false;
                takeControlToolStripMenuItem.Enabled = redirectionEnabled;
            }

            // Copy Menu
            bool copyMenu = false;
            copyMenu |= (node.Tag != null && node.Tag.GetType() == typeof(CircuitBreakerFilterInfoType));
            copyMenu |= (node.Tag == null && node.Text.Equals(ManageabilityCommanderTool.Properties.Resources.TreeNode_Filters));
            copyMenu |= (node.Tag != null && node.Tag.GetType() == typeof(AmtWatchdogAgent));
            copyMenu |= (node.Tag == null && node.Text.Equals(ManageabilityCommanderTool.Properties.Resources.TreeNode_Watchdogs));
            copyMenu |= (node.Tag != null && node.Tag.GetType() == typeof(CircuitBreakerPolicyInfoType));
            copyMenu |= (node.Tag == null && node.Text.Equals(ManageabilityCommanderTool.Properties.Resources.TreeNode_Policies));
            copyMenu |= (node == rootNode);
            copyMenu |= (node.Tag != null && node.Tag.GetType() == typeof(AmtSystem));
            copyToolStripMenuItem.Tag = copyMenu;

            // Paste Menu
            pasteToolStripMenuItem.Tag = false;
            if ((node.Tag == null && node.Text.Equals(ManageabilityCommanderTool.Properties.Resources.TreeNode_Filters)))
            {
                pasteToolStripMenuItem.Tag = true;
                pasteToolStripMenuItem.Enabled =
                    (Utils.UnSerializeFromClipboard(typeof(CircuitBreakerFilterType[])) != null) ||
                    (Utils.UnSerializeFromClipboard(typeof(CircuitBreakerFilterType)) != null) ||
                    Clipboard.GetDataObject().GetData(typeof(string)) != null;
                //string xml = (string)(Clipboard.GetDataObject().GetData(typeof(string)));
            }
            else if ((node.Tag == null && node.Text.Equals(ManageabilityCommanderTool.Properties.Resources.TreeNode_Watchdogs)))
            {
                pasteToolStripMenuItem.Tag = true;
                pasteToolStripMenuItem.Enabled =
                    (Utils.UnSerializeFromClipboard(typeof(AmtCompleteWatchdogAgent[])) != null) ||
                    (Utils.UnSerializeFromClipboard(typeof(AmtCompleteWatchdogAgent)) != null);
            }
            else if ((node.Tag == null && node.Text.Equals(ManageabilityCommanderTool.Properties.Resources.TreeNode_Policies)))
            {
                pasteToolStripMenuItem.Tag = true;
                pasteToolStripMenuItem.Enabled =
                    (Utils.UnSerializeFromClipboard(typeof(AmtCircuitBreaker.TotalCircuitBreakerPolicy[])) != null) ||
                    (Utils.UnSerializeFromClipboard(typeof(AmtCircuitBreaker.TotalCircuitBreakerPolicy)) != null) ||
                    Clipboard.GetDataObject().GetData(typeof(string)) != null;
            }
            else if (node == rootNode)
            {
                pasteToolStripMenuItem.Tag = true;
                pasteToolStripMenuItem.Enabled =
                    (Utils.UnSerializeFromClipboard(typeof(AmtSerialSystemSettings[])) != null) ||
                    (Utils.UnSerializeFromClipboard(typeof(AmtSerialSystemSettings)) != null);
            }

            //disable some menu items if node is the "my computer" local hostnode
            if (node.Tag != null && node.Tag.GetType() == typeof(AmtSystem))
            {
                AmtSystem s = (AmtSystem)node.Tag;

                if (s.MyComputer)
                {
                    removeSystemToolStripMenuItem.Tag = false;
                    takeControlToolStripMenuItem.Tag = false;
                    copyToolStripMenuItem.Tag = false;
                }
            }

            // Seperator
            toolStripMenuItem6.Tag = ((bool)copyToolStripMenuItem.Tag || (bool)pasteToolStripMenuItem.Tag);

            foreach (object o in editToolStripMenuItem.DropDownItems)
            {
                if (o.GetType() == typeof(ToolStripMenuItem))
                {
                    ToolStripMenuItem mi = (ToolStripMenuItem)o;
                    mi.Visible = (bool)mi.Tag;
                }
                else if (o.GetType() == typeof(ToolStripSeparator))
                {
                    ToolStripSeparator mi = (ToolStripSeparator)o;
                    mi.Visible = (bool)mi.Tag;
                }
            }
        }

        private void editToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            mainTreeView_ArrangeContextMenu();
        }

        private void mainTreeView_MouseDown(object sender, MouseEventArgs e)
        {
            TreeNode node = mainTreeView.GetNodeAt(e.X, e.Y);
            if (node == null)
            {
                treeContextMenuStrip.Items.Clear();
                return;
            }

            mainTreeView.SelectedNode = node;
            mainTreeView_ArrangeContextMenu();

            treeContextMenuStrip.Items.Clear();
            ArrayList temp = new ArrayList();
            temp.AddRange(editToolStripMenuItem.DropDownItems);
            foreach (object o in temp)
            {
                if (o.GetType() == typeof(ToolStripSeparator))
                {
                    ToolStripSeparator mi = (ToolStripSeparator)o;
                    if (mi.Tag != null && (bool)mi.Tag == true)
                    {
                        treeContextMenuStrip.Items.Add(new ToolStripSeparator());
                    }
                }
                else if (o.GetType() == typeof(ToolStripMenuItem))
                {
                    ToolStripMenuItem mi = (ToolStripMenuItem)o;
                    if (mi.Tag != null && (bool)mi.Tag == true)
                    {
                        ToolStripMenuItem m = new ToolStripMenuItem();
                        m.Text = mi.Text;
                        if (mi.Tag != null)
                            m.Visible = (bool)mi.Tag;
                        m.Enabled = mi.Enabled;
                        m.Checked = mi.Checked;
                        m.Tag = mi;
                        m.Click += new EventHandler(genericPassthruMenuItem_Click);

                        foreach (ToolStripMenuItem mi2 in mi.DropDownItems)
                        {
                            ToolStripMenuItem m2 = new ToolStripMenuItem();
                            m2.Text = mi2.Text;
                            m2.ToolTipText = mi2.ToolTipText;
                            m2.Enabled = mi2.Enabled;
                            if (mi2.Tag != null)
                                m2.Visible = (bool)mi2.Tag;
                            m2.Checked = mi2.Checked;
                            m2.Tag = mi2;
                            m2.Click += new EventHandler(genericPassthruMenuItem_Click);
                            m.DropDownItems.Add(m2);
                        }

                        treeContextMenuStrip.Items.Add(m);
                    }
                }
            }
        }

        private void genericPassthruMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem m1 = (ToolStripMenuItem)sender;
            ToolStripMenuItem m2 = (ToolStripMenuItem)m1.Tag;
            m2.PerformClick();
        }

        private void mainTreeView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            uiRefreshTimer.Enabled = false;

            TreeNode SelectedNode;
            if (e == null)
            {
                SelectedNode = DisplayedNode;
            }
            else
            {
                DisplayedNode = SelectedNode = e.Node;
            }

            #region Node - Root
            if (SelectedNode == null || SelectedNode == rootNode)
            {
                if (discoverypanel != null)
                {
                    RightViewEx(discoverypanel);
                    CurrentPanelControl.UpdatePanel();
                }
                return;
            }
            #endregion

            #region Node - AmtSystem
            if (SelectedNode.Tag != null && SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                AmtSystem computer = (AmtSystem)SelectedNode.Tag;
                RightViewEx(typeof(Panels.ComputerPanel));
                CurrentPanelControl.Setup(this, computer, null);
                CurrentPanelControl.UpdatePanel();
                return;
            }
            #endregion

            #region Nodes - Inventory
            if (SelectedNode.Tag == null && SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Memory_Banks) == 0)
            {
                RightViewEx(typeof(Panels.InformationPanel));
                CurrentPanelControl.Setup(this, new string[] { ManageabilityCommanderTool.Properties.Resources.TreeNode_Memory_Banks, "", "" }, ManageabilityControls.AboutForm.RebrandImageEx("Image-RAM.jpg", global::ManageabilityCommanderTool.Properties.Resources.Image_RAM));
                CurrentPanelControl.UpdatePanel();
                return;
            }

            if (SelectedNode.Tag == null && SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Hardware_Modules) == 0)
            {
                RightViewEx(typeof(Panels.InformationPanel));
                CurrentPanelControl.Setup(this, new string[] { ManageabilityCommanderTool.Properties.Resources.TreeNode_Hardware_Modules, "", "" }, ManageabilityControls.AboutForm.RebrandImageEx("Image-Card.jpg", global::ManageabilityCommanderTool.Properties.Resources.Image_Card));
                CurrentPanelControl.UpdatePanel();
                return;
            }

            if (SelectedNode.Tag != null && SelectedNode.Tag.GetType() == typeof(AmtHardwareComputerSystem))
            {
                AmtHardwareComputerSystem system = (AmtHardwareComputerSystem)SelectedNode.Tag;

                StringBuilder s = new StringBuilder(InfoTextStart());
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_CS_Manu, system.Manufacturer));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_CS_Prod, system.Product));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_CS_Ver, system.Version));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_CS_SN, system.SerialNumber));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_CS_UUID, system.UUID.ToString()));
                s.Append(InfoTextEnd());

                RightViewEx(typeof(Panels.InformationPanel));
                CurrentPanelControl.Setup(this, new string[] { ManageabilityCommanderTool.Properties.Resources.TreeNode_Computer_System, "", s.ToString() }, ManageabilityControls.AboutForm.RebrandImageEx("Image-Motherboard.jpg", global::ManageabilityCommanderTool.Properties.Resources.Image_Motherboard));
                CurrentPanelControl.UpdatePanel();
                return;
            }

            if (SelectedNode.Tag != null && SelectedNode.Tag.GetType() == typeof(AmtHardwareMemoryModule))
            {
                AmtHardwareMemoryModule module = (AmtHardwareMemoryModule)SelectedNode.Tag;

                StringBuilder s = new StringBuilder(InfoTextStart());
                if (module.Size != 0)
                {
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SM_Size_0, String.Format(ManageabilityCommanderTool.Properties.Resources.TreeNode_SM_Size_Format, module.Size.ToString(), (module.Size / 1024).ToString())));
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SM_Form_Factor, module.FormFactor.ToString()));
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SM_Type_Detail, module.TypeDetailStr));
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SM_Type, module.Type.ToString()));
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SM_Manu, module.Manufacturer));
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SM_Prod, module.PartNumber));
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SM_Serial, module.SerialNumber));
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SM_Speed_0, String.Format(ManageabilityCommanderTool.Properties.Resources.TreeNode_SM_Speed_Format, module.Speed.ToString())));
                }
                else
                {
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SM_Form_Factor, module.FormFactor.ToString()));
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SM_Type, module.Type.ToString()));
                }
                s.Append(InfoTextEnd());

                RightViewEx(typeof(Panels.InformationPanel));
                CurrentPanelControl.Setup(this, new string[] { ManageabilityCommanderTool.Properties.Resources.TreeNode_System_Memory, "", s.ToString() }, ManageabilityControls.AboutForm.RebrandImageEx("Image-RAM.jpg", global::ManageabilityCommanderTool.Properties.Resources.Image_RAM));
                CurrentPanelControl.UpdatePanel();
                return;
            }

            if (SelectedNode.Tag != null && SelectedNode.Tag.GetType() == typeof(AmtHardwareModule))
            {
                AmtHardwareModule module = (AmtHardwareModule)SelectedNode.Tag;

                StringBuilder s = new StringBuilder(InfoTextStart());
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_HD_Vendor, module.VendorIdStr));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_HD_Device, module.DeviceIdStr));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_HD_Device_ID, "0x" + module.DeviceID.ToString("X")));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_HD_Revision_ID, "0x" + module.RevisionID.ToString("X")));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_HD_Prog_Interface, "0x" + module.ProgIf.ToString("X")));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_HD_Subclass, "0x" + module.Subclass.ToString("X")));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_HD_Base_Class, "0x" + module.Baseclass.ToString("X")));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_HD_Subvendor_ID, "0x" + module.SubvendorID.ToString("X")));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_HD_Subsystem_ID, "0x" + module.SubsystemID.ToString("X")));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_HD_Device_Location, "0x" + module.DeviceLocation.ToString("X")));
                s.Append(InfoTextEnd());

                RightViewEx(typeof(Panels.InformationPanel));
                CurrentPanelControl.Setup(this, new string[] { ManageabilityCommanderTool.Properties.Resources.TreeNode_Hardware_Device, "", s.ToString() }, ManageabilityControls.AboutForm.RebrandImageEx("Image-Card.jpg", global::ManageabilityCommanderTool.Properties.Resources.Image_Card));
                CurrentPanelControl.UpdatePanel();
                return;
            }

            if (SelectedNode.Tag != null && SelectedNode.Tag.GetType() == typeof(AmtHardwareMediaDevice))
            {
                AmtHardwareMediaDevice media = (AmtHardwareMediaDevice)SelectedNode.Tag;

                StringBuilder s = new StringBuilder(InfoTextStart());
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_MD_Model_Number, media.ModelNumber));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_MD_SN, media.SerialNumber));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_MD_Max_MSize, media.MaxMediaSizeLStr));
                if ((media.CapabilitiesStr1.Length + media.CapabilitiesStr2.Length + media.CapabilitiesStr3.Length) > 0)
                {
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_MD_Support_Cap, media.CapabilitiesStr1.Replace(", ", "\r\n") + "\r\n" + media.CapabilitiesStr2.Replace(", ", "\r\n") + "\r\n" + media.CapabilitiesStr3.Replace(", ", "\r\n")));
                }
                s.Append(InfoTextEnd());

                RightViewEx(typeof(Panels.InformationPanel));
                CurrentPanelControl.Setup(this, new string[] { ManageabilityCommanderTool.Properties.Resources.TreeNode_Media_Device, "", s.ToString() }, ManageabilityControls.AboutForm.RebrandImageEx("Image-Harddisk.jpg", global::ManageabilityCommanderTool.Properties.Resources.Image_Harddisk));
                CurrentPanelControl.UpdatePanel();
                return;
            }

            if (SelectedNode.Tag != null && SelectedNode.Tag.GetType() == typeof(AmtHardwareBaseBoard))
            {
                AmtHardwareBaseBoard baseboard = (AmtHardwareBaseBoard)SelectedNode.Tag;

                StringBuilder s = new StringBuilder(InfoTextStart());
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_MB_Manu, baseboard.Manufacturer));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_MB_Prod, baseboard.Product));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_MB_Version, baseboard.Version));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_MB_SN, baseboard.SerialNumber));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_MB_Asset_Tag, baseboard.AssetTag));
                if (baseboard.ReplaceableSpecified)
                {
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_MB_Replace, baseboard.Replaceable.ToString()));
                }
                else
                {
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_MB_Replace, Properties.Resources.Unknown));
                }
                s.Append(InfoTextEnd());

                RightViewEx(typeof(Panels.InformationPanel));
                CurrentPanelControl.Setup(this, new string[] { ManageabilityCommanderTool.Properties.Resources.TreeNode_System_Motherboard, "", s.ToString() }, ManageabilityControls.AboutForm.RebrandImageEx("Image-Motherboard.jpg", global::ManageabilityCommanderTool.Properties.Resources.Image_Motherboard));
                CurrentPanelControl.UpdatePanel();
                return;
            }

            if (SelectedNode.Tag != null && SelectedNode.Tag.GetType() == typeof(AmtHardwareBios))
            {
                AmtHardwareBios bios = (AmtHardwareBios)SelectedNode.Tag;

                StringBuilder s = new StringBuilder(InfoTextStart());
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SB_Vendor, bios.Vendor));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SB_Version, bios.Version));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SB_Release_Date, bios.ReleaseDate));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SB_Charact, AmtHardwareBios.GetCharacteristicsString(bios.Characteristics)));
                s.Append(InfoTextEnd());

                RightViewEx(typeof(Panels.InformationPanel));
                CurrentPanelControl.Setup(this, new string[] { ManageabilityCommanderTool.Properties.Resources.TreeNode_System_BIOS, "", s.ToString() }, ManageabilityControls.AboutForm.RebrandImageEx("Image-Flash.jpg", global::ManageabilityCommanderTool.Properties.Resources.ImageFlash));
                CurrentPanelControl.UpdatePanel();
                return;
            }

            if (SelectedNode.Tag != null && SelectedNode.Tag.GetType() == typeof(AmtHardwareProcessor))
            {
                AmtHardwareProcessor proc = (AmtHardwareProcessor)SelectedNode.Tag;

                StringBuilder s = new StringBuilder(InfoTextStart());
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SP_Socket_Design, proc.SocketDesignation));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SP_Manu, proc.Manufacturer));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SP_Max_Speed_0, String.Format(ManageabilityCommanderTool.Properties.Resources.TreeNode_SP_Max_Speed_Format, proc.MaxSocketSpeed.ToString())));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SP_Cur_Speed_0, String.Format(ManageabilityCommanderTool.Properties.Resources.TreeNode_SP_Cur_Speed_Format, proc.CurrentSpeed.ToString())));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SP_ID, proc.ID.ToString("X")));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SP_Socket_Pop, proc.SocketPopulated.ToString()));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SP_Status, proc.Status.ToString()));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SP_Type, proc.Type.ToString()));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SP_Family, proc.Family.ToString()));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SP_Upgrade_Info, proc.UpgradeInformation.ToString()));
                if (proc.Version != null)
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_SP_Version, proc.Version.ToString()));
                s.Append(InfoTextEnd());

                RightViewEx(typeof(Panels.InformationPanel));
                CurrentPanelControl.Setup(this, new string[] { ManageabilityCommanderTool.Properties.Resources.TreeNode_System_Processor, "", s.ToString() }, ManageabilityControls.AboutForm.RebrandImageEx("Image-CPU.jpg", global::ManageabilityCommanderTool.Properties.Resources.Image_CPU));
                CurrentPanelControl.UpdatePanel();
                return;
            }

            if (SelectedNode.Tag != null && SelectedNode.Tag.GetType() == typeof(AmtHardwareBattery))
            {
                AmtHardwareBattery batt = (AmtHardwareBattery)SelectedNode.Tag;

                StringBuilder s = new StringBuilder(InfoTextStart());
                if (batt.SBDSManufactureDate != DateTime.MinValue)
                {
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_BT_Date_0, batt.SBDSManufactureDate.ToLongDateString()));
                }
                else
                {
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_BT_Date_0, ManageabilityCommanderTool.Properties.Resources.TreeNode_BT_Date_1));
                }

                if (batt.DesignCapacity != 0)
                {
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_BT_Cap, String.Format(ManageabilityCommanderTool.Properties.Resources.TreeNode_BT_mWh_Format, batt.DesignCapacity.ToString())));
                }
                else
                {
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_BT_Cap, ManageabilityCommanderTool.Properties.Resources.TreeNode_BT_Unknown));
                }

                if (batt.DesignVoltage != 0)
                {
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_BT_Voltage, String.Format(ManageabilityCommanderTool.Properties.Resources.TreeNode_BT_mVolt_Format, batt.DesignVoltage.ToString())));
                }
                else
                {
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_BT_Voltage, ManageabilityCommanderTool.Properties.Resources.TreeNode_BT_mVolt));
                }

                if (batt.SBDSSerialNumber != 0)
                {
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_BT_SN, batt.SBDSSerialNumber.ToString()));
                }
                else
                {
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_BT_SN, ManageabilityCommanderTool.Properties.Resources.TreeNode_BT_Unknown));
                }

                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_BT_Chem, batt.DeviceChemistry));

                if (batt.MaximumErrorInBatteryData != 0xFF)
                {
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_BT_Error, batt.MaximumErrorInBatteryData.ToString() + "%"));
                }
                else
                {
                    s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_BT_Error, ManageabilityCommanderTool.Properties.Resources.TreeNode_BT_Unknown));
                }

                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_BTR_SN, batt.SerialNumber));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_BTR_Version, batt.SBDSVersionNumber));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_BTR_Location, batt.Location));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_BTR_Manu, batt.Manufacturer));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_BTR_Date, batt.ManufactureDate));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_BTR_Name, batt.DeviceName));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.TreeNode_BTR_Chem, batt.SBDSDeviceChemistry));
                s.Append(InfoTextEnd());

                RightViewEx(typeof(Panels.InformationPanel));
                CurrentPanelControl.Setup(this, new string[] { ManageabilityCommanderTool.Properties.Resources.TreeNode_System_Battery, "", s.ToString() }, ManageabilityControls.AboutForm.RebrandImageEx("Image-Battery.jpg", global::ManageabilityCommanderTool.Properties.Resources.Image_Battery));
                CurrentPanelControl.UpdatePanel();
                return;
            }

            if (SelectedNode.Tag == null && SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Hardware_Asset) == 0)
            {
                AmtSystem computer = (AmtSystem)SelectedNode.Parent.Tag;
                RightViewEx(typeof(Panels.HardwareAssetPanel));
                CurrentPanelControl.Setup(this, computer, null);
                CurrentPanelControl.UpdatePanel();
            }
            #endregion

            #region Node - Networking Root
            if (SelectedNode.Tag == null && SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Networking) == 0)
            {
                AmtSystem computer = (AmtSystem)SelectedNode.Parent.Tag;
                RightViewEx(typeof(Panels.NetworkingPanel));
                CurrentPanelControl.Setup(this, computer, null);
                CurrentPanelControl.UpdatePanel();
                return;
            }
            #endregion

            #region Nodes - Watchdogs
            if (SelectedNode.Tag == null && SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Watchdogs) == 0)
            {
                AmtSystem computer = (AmtSystem)SelectedNode.Parent.Parent.Tag;
                RightViewEx(typeof(Panels.WatchdogsPanel));
                CurrentPanelControl.Setup(this, computer, null);
                CurrentPanelControl.UpdatePanel();
            }

            if (SelectedNode.Tag != null && SelectedNode.Tag.GetType() == typeof(AmtWatchdogAgent))
            {
                AmtSystem computer = (AmtSystem)SelectedNode.Parent.Parent.Parent.Tag;
                AmtWatchdogAgent wg = (AmtWatchdogAgent)SelectedNode.Tag;
                RightViewEx(typeof(Panels.WatchdogAgentPanel));
                CurrentPanelControl.Setup(this, computer, wg);
                CurrentPanelControl.UpdatePanel();
                uiRefreshTimer.Enabled = true;
                return;
            }
            #endregion

            #region Nodes - Filters
            if (SelectedNode.Tag != null && SelectedNode.Tag.GetType() == typeof(CircuitBreakerFilterInfoType))
            {
                AmtSystem computer = (AmtSystem)SelectedNode.Parent.Parent.Parent.Tag;
                CircuitBreakerFilterInfoType cb = (CircuitBreakerFilterInfoType)SelectedNode.Tag;
                RightViewEx(typeof(Panels.NetworkFilterPanel));
                CurrentPanelControl.Setup(this, computer, cb);
                CurrentPanelControl.UpdatePanel();
                return;
            }

            if (SelectedNode.Tag == null && SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Filters) == 0)
            {
                AmtSystem computer = (AmtSystem)SelectedNode.Parent.Parent.Tag;
                CircuitBreakerFilterInfoType[] cbs = computer.CircuitBreaker.GetCircuitBreakerFilters();
                RightViewEx(typeof(Panels.NetworkFiltersPanel));
                CurrentPanelControl.Setup(this, computer, cbs);
                CurrentPanelControl.UpdatePanel();
                return;
            }

            if (SelectedNode.Tag == null && SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Heuristics) == 0)
            {
                AmtSystem computer = (AmtSystem)SelectedNode.Parent.Parent.Tag;
                CircuitBreakerFilterInfoType cb = (CircuitBreakerFilterInfoType)SelectedNode.Tag;
                RightViewEx(typeof(Panels.HeuristicPanel));
                CurrentPanelControl.Setup(this, computer, null);
                CurrentPanelControl.UpdatePanel();
                uiRefreshTimer.Enabled = true;
                return;
            }
            #endregion

            #region Nodes - Policies
            if (SelectedNode.Tag == null && SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Policies) == 0)
            {
                AmtSystem computer = (AmtSystem)SelectedNode.Parent.Parent.Tag;
                CircuitBreakerFilterInfoType cb = (CircuitBreakerFilterInfoType)SelectedNode.Tag;
                RightViewEx(typeof(Panels.NetworkPolicies));
                CurrentPanelControl.Setup(this, computer, cb);
                CurrentPanelControl.UpdatePanel();
                return;
            }

            if (SelectedNode.Tag != null && SelectedNode.Tag.GetType() == typeof(CircuitBreakerPolicyInfoType))
            {
                AmtSystem computer = (AmtSystem)SelectedNode.Parent.Parent.Parent.Tag;
                CircuitBreakerPolicyInfoType cp = (CircuitBreakerPolicyInfoType)SelectedNode.Tag;
                RightViewEx(typeof(Panels.NetworkPolicyPanel));
                CurrentPanelControl.Setup(this, computer, cp);
                CurrentPanelControl.UpdatePanel();
                return;
            }
            #endregion

            #region Nodes - Events
            if (SelectedNode.Tag == null && SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Event_Log) == 0)
            {
                AmtSystem computer = (AmtSystem)SelectedNode.Parent.Tag;
                RightViewEx(typeof(Panels.EventPanel));
                CurrentPanelControl.Setup(this, computer, null);
                CurrentPanelControl.UpdatePanel();
                return;
            }
            #endregion

            #region Nodes - Audit Log
            if (SelectedNode.Tag == null && SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Audit_Log) == 0)
            {
                AmtSystem computer = (AmtSystem)SelectedNode.Parent.Tag;
                RightViewEx(typeof(Panels.AuditLogPanel));
                CurrentPanelControl.Setup(this, computer, null);
                CurrentPanelControl.UpdatePanel();
                return;
            }
            #endregion

            #region Nodes - Storage
            if (SelectedNode.Tag != null && SelectedNode.Tag.GetType() == typeof(AmtStorage))
            {
                AmtSystem computer = (AmtSystem)SelectedNode.Parent.Tag;
                AmtStorage storage = (AmtStorage)SelectedNode.Tag;
                RightViewEx(typeof(Panels.StoragePanel));
                CurrentPanelControl.Setup(this, computer, storage);
                CurrentPanelControl.UpdatePanel();
            }

            if (SelectedNode.Tag != null && SelectedNode.Tag.GetType() == typeof(AmtApplication))
            {
                AmtApplication app = (AmtApplication)SelectedNode.Tag;

                StringBuilder s = new StringBuilder(InfoTextStart());
                s.Append(InfoTextAdd(Properties.Resources.AppStore_EName, app.EnterpriseName));
                s.Append(InfoTextAdd(Properties.Resources.AppStore_AppName, app.ApplicationName));
                s.Append(InfoTextAdd(Properties.Resources.AppStore_VName, app.VendorName));
                s.Append(InfoTextAdd(Properties.Resources.AppStore_Par_App, app.PartnerApplication.ToString()));
                s.Append(InfoTextAdd(Properties.Resources.GUID, app.UUID.ToString()));
                s.Append(InfoTextEnd());

                RightViewEx(typeof(Panels.InformationPanel));
                CurrentPanelControl.Setup(this, new string[] { ManageabilityCommanderTool.Properties.Resources.AppStore, ManageabilityCommanderTool.Properties.Resources.AppStore_Info, s.ToString() }, ManageabilityControls.AboutForm.RebrandImageEx("Image-Flash.jpg", global::ManageabilityCommanderTool.Properties.Resources.ImageFlash));
                CurrentPanelControl.UpdatePanel();
                return;
            }

            if (SelectedNode.Tag != null && SelectedNode.Tag.GetType() == typeof(AmtStorageBlock))
            {
                AmtStorageBlock block = (AmtStorageBlock)SelectedNode.Tag;
                AmtApplication app = (AmtApplication)SelectedNode.Parent.Tag;
                AmtSystem system = (AmtSystem)SelectedNode.Parent.Parent.Parent.Parent.Tag;
                RightViewEx(typeof(Panels.StorageBlockPanel));
                CurrentPanelControl.Setup(this, system, new object[] { app, block });
                CurrentPanelControl.UpdatePanel();
                return;
            }

            if (SelectedNode.Tag != null && SelectedNode.Tag.GetType() == typeof(AmtEnterprise))
            {
                AmtEnterprise ent = (AmtEnterprise)SelectedNode.Tag;

                StringBuilder s = new StringBuilder(InfoTextStart());
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.EntStore_Name, ent.Name));
                s.Append(InfoTextAdd(ManageabilityCommanderTool.Properties.Resources.EntStore_Handle, ent.Handle.ToString()));
                s.Append(InfoTextEnd());

                RightViewEx(typeof(Panels.InformationPanel));
                CurrentPanelControl.Setup(this, new string[] { ManageabilityCommanderTool.Properties.Resources.EntStore, ManageabilityCommanderTool.Properties.Resources.EntStore_Info, s.ToString() }, ManageabilityControls.AboutForm.RebrandImageEx("Image-Flash.jpg", global::ManageabilityCommanderTool.Properties.Resources.ImageFlash));
                CurrentPanelControl.UpdatePanel();
                return;
            }

            #endregion

            #region Nodes - MPS
            if (SelectedNode.Tag != null && SelectedNode.Tag.GetType() == typeof(AmtMpsServer))
            {
                AmtMpsServer mps = (AmtMpsServer)SelectedNode.Tag;
                RightViewEx(typeof(Panels.MpsPanel));
                CurrentPanelControl.Setup(this, mps, null);
                CurrentPanelControl.UpdatePanel();
                return;
            }
            #endregion
        }

        private void uiRefreshTimer_Tick(object sender, EventArgs e)
        {
            if (CurrentPanelControl != null)
                CurrentPanelControl.ForceUpdatePanel();
        }

        private void removeSystemToolStripMenuItem_Click(object sender, EventArgs e)
        {
            TreeNode node = mainTreeView.SelectedNode;
            if (node == null)
                return;
            if (MessageBox.Show(this, ManageabilityCommanderTool.Properties.Resources.ConfirmRemoveComputer, ManageabilityCommanderTool.Properties.Resources.ConfirmRemoveComputerTitle, MessageBoxButtons.OKCancel, MessageBoxIcon.Asterisk) != DialogResult.OK)
                return;

            if (node.Tag != null && node.Tag.GetType() == typeof(AmtSystem))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                if (node.Parent != null && node.Parent.Tag != null && node.Parent.Tag.GetType() == typeof(AmtMpsServer))
                {
                    // Remove a computer within MPS
                    AmtMpsServer mps = (AmtMpsServer)node.Parent.Tag;
                    mps.Computers.Remove(computer);
                    ManageabilityControls.TerminalForm.DisposeRemotingForm(computer);
                    computer.Disconnect();
                    computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
                    computer.OnFetchCompleted -= new AmtSystem.FetchHandler(FetchHandler);
                    node.Remove();

                    SaveSettings();
                }
                else
                {
                    // Remove a standalone computer
                    ManageabilityControls.TerminalForm.DisposeRemotingForm(computer);
                    computer.Disconnect();
                    computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
                    computer.OnFetchCompleted -= new AmtSystem.FetchHandler(FetchHandler);
                    node.Remove();
                    SaveSettings();
                }
            }
        }

        private void openWebInterfaceToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                try
                {
                    System.Diagnostics.Process.Start(computer.WebURL);
                }
                catch (Exception)
                {
                }
            }
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new ManageabilityControls.AboutForm(ManageabilityCommanderTool.Properties.Resources.ProductName, ManageabilityCommanderTool.Properties.Resources.ProductSubName).ShowDialog(this);
        }

        public void UpdateTreeFilters(AmtSystem computer)
        {
            if (computer == null || computer.AmtRev < 2 || computer.CircuitBreaker == null)
                return;

            TreeNode node = null;

            if (computer.MyComputer)
                node = mainTreeView.Nodes[1];
            else
                node = Utils.GetNodeWithTagRec(mainTreeView.Nodes[0], computer);

            if (node == null || node.Nodes.Count == 0)
                return;
            node = node.Nodes[1].Nodes[0];

            CircuitBreakerFilterInfoType[] cbs = computer.CircuitBreaker.GetCircuitBreakerFilters();
            if (cbs == null || cbs.Length == 0)
            {
                node.Nodes.Clear();
                return;
            }

            ArrayList l = new ArrayList();
            foreach (CircuitBreakerFilterInfoType cb in cbs)
            {
                TreeNode nn = new TreeNode(cb.Filter.FilterName, 2, 2);
                nn.Tag = cb;
                l.Add(nn);
            }

            mainTreeView.BeginUpdate();
            node.Nodes.Clear();
            node.Nodes.AddRange((TreeNode[])l.ToArray(typeof(TreeNode)));
            mainTreeView.EndUpdate();

            mainTreeView_AfterSelect(this, null);
        }

        public void UpdateTreePolicies(AmtSystem computer)
        {
            if (computer == null || computer.AmtRev < 2 || computer.CircuitBreaker == null)
                return;

            uint currentlySelectedPolicyHandle = uint.MaxValue;
            if (mainTreeView.SelectedNode != null && mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(CircuitBreakerPolicyInfoType))
            {
                // The tree view is currently looking at a policy, let try to reselect it after the update
                CircuitBreakerPolicyInfoType cp = (CircuitBreakerPolicyInfoType)mainTreeView.SelectedNode.Tag;
                currentlySelectedPolicyHandle = cp.PolicyCreationHandle;
            }

            TreeNode node = null;

            if (computer.MyComputer)
                node = mainTreeView.Nodes[1];
            else
                node = Utils.GetNodeWithTagRec(mainTreeView.Nodes[0], computer);

            if (node == null || node.Nodes.Count == 0)
                return;
            node = node.Nodes[1].Nodes[1];

            CircuitBreakerPolicyInfoType[] cbp = computer.CircuitBreaker.GetCircuitBreakerPolicies();
            if (cbp == null || cbp.Length == 0)
            {
                node.Nodes.Clear();
                return;
            }

            ArrayList l = new ArrayList();
            TreeNode select = null;
            foreach (CircuitBreakerPolicyInfoType cp in cbp)
            {
                int active = 6;
                if (computer.CircuitBreaker.IsPolicyActive(cp.PolicyCreationHandle) != null)
                    active = 7;
                TreeNode nn = new TreeNode(cp.Policy.PolicyName, active, active);
                nn.Tag = cp;
                l.Add(nn);
                if (currentlySelectedPolicyHandle == cp.PolicyCreationHandle)
                    select = nn;
            }

            mainTreeView.BeginUpdate();
            node.Nodes.Clear();
            node.Nodes.AddRange((TreeNode[])l.ToArray(typeof(TreeNode)));
            if (select != null)
                mainTreeView.SelectedNode = select;
            mainTreeView.EndUpdate();

            mainTreeView_AfterSelect(this, null);
        }

        public void UpdateTreeWatchdogs(AmtSystem computer)
        {
            if (computer == null || computer.AmtRev < 2 || Simplified == true || computer.CircuitBreaker == null)
                return;

            TreeNode node = null;

            if (computer.MyComputer)
                node = mainTreeView.Nodes[1];
            else
                node = Utils.GetNodeWithTagRec(mainTreeView.Nodes[0], computer);

            if (node == null)
                return;

            if (node.Nodes.Count < 2)
                return;
            if (node.Nodes[1].Nodes.Count < 3)
                return;
            node = node.Nodes[1].Nodes[2];

            AmtWatchdogAgent[] watchdogs = computer.Watchdog.GetAgents();
            if (watchdogs == null || watchdogs.Length == 0)
            {
                node.Nodes.Clear();
                return;
            }

            ArrayList l = new ArrayList();
            foreach (AmtWatchdogAgent wg in watchdogs)
            {
                TreeNode nn = new TreeNode(wg.Description, 13, 13);
                nn.Tag = wg;
                l.Add(nn);
            }

            mainTreeView.BeginUpdate();
            node.Nodes.Clear();
            node.Nodes.AddRange((TreeNode[])l.ToArray(typeof(TreeNode)));
            mainTreeView.EndUpdate();

            mainTreeView_AfterSelect(this, null);
        }

        public void UpdateTreeAssets(AmtSystem computer)
        {
            if (computer == null)
                return;

            TreeNode node = null;

            if (computer.MyComputer)
                node = mainTreeView.Nodes[1];
            else
                node = Utils.GetNodeWithTagRec(mainTreeView.Nodes[0], computer);

            if (node == null || node.Nodes.Count == 0)
                return;
            node = node.Nodes[0];

            ArrayList l = new ArrayList();

            // Add computer to tree view
            AmtHardwareComputerSystem amtComputer = computer.Assets.GetComputerSystem();
            TreeNode amtComputerNode = null;
            if (amtComputer != null)
            {
                int icon = 15;
                if (computer.IsMobile)
                    icon += 4;
                amtComputerNode = new TreeNode(amtComputer.ToString(), icon, icon);
                amtComputerNode.Tag = amtComputer;
                l.Add(amtComputerNode);
            }

            // Add motherboard to tree view
            AmtHardwareBaseBoard amtBaseBoard = computer.Assets.GetBaseBoard();
            TreeNode amtBaseBoardNode = null;
            if (amtBaseBoard != null)
            {
                amtBaseBoardNode = new TreeNode(amtBaseBoard.ToString(), 10, 10);
                amtBaseBoardNode.Tag = amtBaseBoard;
                if (amtComputer != null)
                {
                    amtComputerNode.Nodes.Add(amtBaseBoardNode);
                }
                else
                {
                    l.Add(amtBaseBoardNode);
                }
            }

            // Add BIOS to tree view
            AmtHardwareBios b = computer.Assets.GetBios();
            if (b != null)
            {
                TreeNode nn = new TreeNode(b.ToString(), 8, 8);
                nn.Tag = b;
                if (amtBaseBoardNode != null)
                {
                    amtBaseBoardNode.Nodes.Add(nn);
                }
                else
                {
                    l.Add(nn);
                }
            }

            // Add processor to tree view
            AmtHardwareProcessor[] amtProcessors = computer.Assets.GetProcessors();
            if (amtProcessors != null)
            {
                foreach (AmtHardwareProcessor m in amtProcessors)
                {
                    TreeNode nn = new TreeNode(m.ToString(), 8, 8);
                    nn.Tag = m;
                    if (amtBaseBoardNode != null)
                    {
                        amtBaseBoardNode.Nodes.Add(nn);
                    }
                    else
                    {
                        l.Add(nn);
                    }
                }
            }

            // Add batteries to tree view
            AmtHardwareBattery[] batts = computer.Assets.GetBatteries();
            if (batts != null)
            {
                foreach (AmtHardwareBattery batt in batts)
                {
                    TreeNode nn = new TreeNode(batt.ToString(), 14, 14);
                    nn.Tag = batt;
                    if (amtComputer != null)
                    {
                        amtComputerNode.Nodes.Add(nn);
                    }
                    else
                    {
                        l.Add(nn);
                    }
                }
            }

            // Add media devices to tree view
            AmtHardwareMediaDevice[] md = computer.Assets.GetMediaDevices();
            if (md != null)
            {
                foreach (AmtHardwareMediaDevice m in md)
                {
                    TreeNode nn = new TreeNode(m.ToString(), 11, 11);
                    nn.Tag = m;
                    if (amtComputer != null)
                    {
                        amtComputerNode.Nodes.Add(nn);
                    }
                    else
                    {
                        l.Add(nn);
                    }
                }
            }

            // Add memory group to tree view
            TreeNode amtMemoryBanksNode = null;
            if (amtBaseBoardNode != null)
            {
                amtMemoryBanksNode = new TreeNode(ManageabilityCommanderTool.Properties.Resources.TreeNode_Memory_Banks, 9, 9);
                amtBaseBoardNode.Nodes.Add(amtMemoryBanksNode);
            }

            // Add memory to tree view
            AmtHardwareMemoryModule[] ms = computer.Assets.GetMemoryModules();
            if (ms != null)
            {
                foreach (AmtHardwareMemoryModule m in ms)
                {
                    TreeNode nn = new TreeNode(m.ToString(), 9, 9);
                    nn.Tag = m;
                    if (amtMemoryBanksNode != null)
                    {
                        amtMemoryBanksNode.Nodes.Add(nn);
                    }
                    else
                    {
                        l.Add(nn);
                    }
                }
            }

            // Add hardware group to tree view
            TreeNode amtModulesNode = null;
            if (amtBaseBoardNode != null)
            {
                amtModulesNode = new TreeNode(ManageabilityCommanderTool.Properties.Resources.TreeNode_Hardware_Modules, 10, 10);
                amtBaseBoardNode.Nodes.Add(amtModulesNode);
            }

            // Add hardware modules to tree view
            AmtHardwareModule[] hs = computer.Assets.GetHardwareModules();
            if (hs != null)
            {
                foreach (AmtHardwareModule m in hs)
                {
                    TreeNode nn = new TreeNode(m.ToString(), 10, 10);
                    nn.Tag = m;
                    if (amtModulesNode != null)
                    {
                        amtModulesNode.Nodes.Add(nn);
                    }
                    else
                    {
                        l.Add(nn);
                    }
                }
            }

            mainTreeView.BeginUpdate();
            node.Nodes.Clear();
            node.Nodes.AddRange((TreeNode[])l.ToArray(typeof(TreeNode)));
            mainTreeView.EndUpdate();

        }

        public void UpdateTreeStorage(AmtSystem computer)
        {
            if (computer == null)
                return;

            TreeNode node = null;

            if (computer.MyComputer)
                node = mainTreeView.Nodes[1];
            else
                node = Utils.GetNodeWithTagRec(mainTreeView.Nodes[0], computer);

            if (node == null)
                return;

            foreach (TreeNode n in node.Nodes)
            {
                if (n.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Data_Storage) == 0)
                {
                    node = n;
                    break;
                }
            }

            mainTreeView.BeginUpdate();

            // Update Enterprises
            Utils.CheckAllNodesRec(node);

            AmtEnterprise[] enterprises = computer.Storage.GetEnterpriseEntries();
            Hashtable eTable = new Hashtable();
            if (enterprises != null && enterprises.Length != 0)
            {
                foreach (AmtEnterprise enterprise in enterprises)
                {
                    // Look to see if the node already exists
                    TreeNode nn = Utils.GetNodeWithTag(node, enterprise);
                    if (nn == null)
                    {
                        nn = new TreeNode(enterprise.Name, 4, 5);
                        nn.Tag = enterprise;
                        node.Nodes.Add(nn);
                    }
                    nn.Checked = false;
                    eTable.Add(enterprise.Name, nn);
                }
            }

            // Update Registered Applications
            Guid selfid = Guid.Empty;
            if (computer.LimitedFeatures == true)
                selfid = new Guid("31313131-3131-3131-3131-313131313131");

            AmtApplication[] apps = computer.Storage.GetRegisteredApplications();
            if (apps != null && apps.Length != 0)
            {
                foreach (AmtApplication app in apps)
                {
                    TreeNode pn = (TreeNode)eTable[app.EnterpriseName];
                    TreeNode nn = Utils.GetNodeWithTag(pn, app);
                    if (nn == null)
                    {
                        string str = "";
                        if (app.UUID == selfid)
                            str = " " + Properties.Resources.Owner;
                        nn = new TreeNode(app.VendorName + ", " + app.ApplicationName + str, 4, 5);
                        nn.Tag = app;
                        pn.Nodes.Add(nn);
                    }
                    nn.Checked = false;

                    AmtStorageBlock[] blocks = computer.Storage.GetStorageBlocks(app);
                    if (blocks != null)
                    {
                        foreach (AmtStorageBlock block in blocks)
                        {
                            TreeNode nnn = Utils.GetNodeWithTag(nn, block);
                            if (nnn == null && block != null)
                            {
                                string hiddenStr = "";
                                if (block.Hidden == true)
                                    hiddenStr = " " + Properties.Resources.Hidden2;
                                nnn = new TreeNode(block.Name + hiddenStr, 1, 1);
                                nnn.Tag = block;
                                nn.Nodes.Add(nnn);
                            }
                            nnn.Checked = false;
                        }
                    }
                }
            }
            Utils.DeleteCheckedNodesRec(node);
            mainTreeView.EndUpdate();

            mainTreeView_AfterSelect(this, null);
        }

        private void refreshFiltersToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag == null && mainTreeView.SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Filters) == 0)
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Tag;
                computer.CircuitBreaker.ClearFiltersCache();
                UpdateTreeFilters(computer);
            }
        }

        private void wiredNICToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(CircuitBreakerPolicyInfoType))
            {
                CircuitBreakerPolicyInfoType cbp = (CircuitBreakerPolicyInfoType)mainTreeView.SelectedNode.Tag;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Parent.Tag;
                if (computer.CircuitBreaker.IsPolicyActive(cbp.PolicyCreationHandle, 0))
                {
                    computer.CircuitBreaker.CircuitBreakerDisablePolicy(0); // Wired NIC
                }
                else
                {
                    computer.CircuitBreaker.CircuitBreakerEnablePolicy(cbp.PolicyCreationHandle, 0); // Wired NIC
                }
                UpdateTreePolicies(computer);
            }
        }

        private void wirelessNICToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(CircuitBreakerPolicyInfoType))
            {
                CircuitBreakerPolicyInfoType cbp = (CircuitBreakerPolicyInfoType)mainTreeView.SelectedNode.Tag;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Parent.Tag;
                if (computer.CircuitBreaker.IsPolicyActive(cbp.PolicyCreationHandle, 1))
                {
                    computer.CircuitBreaker.CircuitBreakerDisablePolicy(1); // Wireless NIC
                }
                else
                {
                    computer.CircuitBreaker.CircuitBreakerEnablePolicy(cbp.PolicyCreationHandle, 1); // Wireless NIC
                }
                UpdateTreePolicies(computer);
            }
        }

        private void renameSystemToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                new RenameForm(computer).ShowDialog(this);
            }

        }

        private void sendCommandButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                new ManageabilityControls.RemoteControlForm(computer).ShowDialog(this);
            }
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information, "Manageability Commander Tool v" + ManageabilityControls.AboutForm.VersionString);

            Rebrand();
            mainToolStripStatusLabel.Text = mainToolStripStatusLabel.Text.Replace("0.00", ManageabilityControls.AboutForm.VersionString);
            mainToolStripStatusLabel.Text = Utils.FilterStrEx(mainToolStripStatusLabel.Text);
            this.Text = Utils.FilterStrEx(this.Text);

            discoverypanel = new Panels.DiscoveryPanel();
            discoverypanel.Setup(this, null, null);

            // Setup the panel
            CurrentPanelControl = discoverypanel;
            splitContainer1.Panel2.Controls.Add((Control)CurrentPanelControl);
            splitContainer1.Panel2.Controls[0].Dock = DockStyle.Fill;
            ((UserControl)splitContainer1.Panel2.Controls[0]).BorderStyle = BorderStyle.Fixed3D;

            //mainToolTip.Active = displayToolTipsToolStripMenuItem.Checked = Properties.Settings.Default.ToolTips;
            mainTreeView_AfterSelect(this, null);

            // View menu items
            showUsernamesToolStripMenuItem.Checked = ((string)Application.UserAppDataRegistry.GetValue("ShowUserNames", Boolean.TrueString)).CompareTo(Boolean.TrueString) == 0;
            showDomainNamesToolStripMenuItem.Checked = ((string)Application.UserAppDataRegistry.GetValue("ShowDomainNames", Boolean.TrueString)).CompareTo(Boolean.TrueString) == 0;
            showStatusBarToolStripMenuItem.Checked = ((string)Application.UserAppDataRegistry.GetValue("ShowStatusBar", Boolean.TrueString)).CompareTo(Boolean.TrueString) == 0;

            if (Simplified == false && !Utils.IsMono())
            {
                // Start listening to trap events if not in simplified mode.
                if (AmtTrapListener.TrapListener.Start() == false)
                {
                    MessageBox.Show(
                        ManageabilityCommanderTool.Properties.Resources.MainForm_SNMP_ErrStr,
                        ManageabilityCommanderTool.Properties.Resources.MainForm_Alert_Rec,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
                AmtTrapListener.TrapListener.AmtNetworkEvent += new AmtTrapListener.AmtNetworkEventHandler(AmtAllNetworkEventSink);
                AmtTrapListener.TrapListener.AmtNetworkDebugEvent += new AmtTrapListener.AmtNetworkEventDebugHandler(AlertViewer.AddAlert);
            }

            if (!Utils.IsMono())
            {
                ServicePointManager.ServerCertificateValidationCallback = new System.Net.Security.RemoteCertificateValidationCallback(RemoteCertificateValidationCallbackSink);
            }

            if (one_hostname != null)
            {
                // Setup and connect to a single computer.
                ushort port = 16992;
                int i = one_hostname.IndexOf(":");
                if (i >= 0)
                {
                    port = ushort.Parse(one_hostname.Substring(i + 1));
                    one_hostname = one_hostname.Substring(0, i);
                }

                AmtSystem computer = new AmtSystem(one_hostname, port, one_username, one_password, false, false);
                computer.UseTls = one_tls;
                AddComputer(computer);
                computer.WsManSupport = Properties.Settings.Default.WsmanSupport;
                computer.Connect();


                // Remove menu items that don't make sense.
                addAMTSystemToolStripMenuItem1.Visible = false;
                loadComputersToolStripMenuItem.Visible = false;
                saveComputersToolStripMenuItem.Visible = false;
                toolStripMenuItem5.Visible = false;
                batchRemoteControlToolStripMenuItem.Visible = false;
            }

            if (Simplified == true)
            {
                // The UI is in simplified mode, lets remove advanced things.
                toolStripMenuItem5.Visible = false;
                batchRemoteControlToolStripMenuItem.Visible = false;
                alertViewerToolStripMenuItem.Visible = false;
                certificateManagerToolStripMenuItem.Visible = false;
                toolStripMenuItem4.Visible = false;
                clearWebServicesCacheToolStripMenuItem.Visible = false;
                fetchWebServiceCacheToolStripMenuItem.Visible = false;
                if (!Utils.IsMono())
                    showManageabilityStackDebugInfoToolStripMenuItem.Visible = false;
            }
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            disposing = true;
            discoverypanel.Cleanup();
            discoverypanel = null;

            SaveSettings();
            Properties.Settings.Default.Save();

            // Remove the temp web file if exists
            FileInfo file = new FileInfo(Application.StartupPath + "\\StorageTemp.html");
            if (file.Exists == true)
                file.Delete();

            // Save view menu items
            Application.UserAppDataRegistry.SetValue("ShowStatusBar", showStatusBarToolStripMenuItem.Checked.ToString());
            Application.UserAppDataRegistry.SetValue("ShowUserNames", showUsernamesToolStripMenuItem.Checked.ToString());
            Application.UserAppDataRegistry.SetValue("ShowDomainNames", showDomainNamesToolStripMenuItem.Checked.ToString());

            if (Simplified == false)
            {
                AmtTrapListener.TrapListener.AmtNetworkEvent -= new AmtTrapListener.AmtNetworkEventHandler(AmtAllNetworkEventSink);
                AmtTrapListener.TrapListener.AmtNetworkDebugEvent -= new AmtTrapListener.AmtNetworkEventDebugHandler(AlertViewer.AddAlert);
                AmtTrapListener.TrapListener.Stop();
            }

            // Lets dispose all of the AmtSystem objects since they include threads that will not clean up.
            if (rootNode != null)
            {
                ArrayList l = Utils.GetNodeTagsofTypeRec(rootNode, typeof(AmtSystem));
                foreach (AmtSystem computer in l)
                {
                    ManageabilityControls.TerminalForm.DisposeRemotingForm(computer);
                    computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
                    computer.OnFetchCompleted -= new AmtSystem.FetchHandler(FetchHandler);
                    computer.Disconnect();
                    computer.Dispose();
                }
            }
            else
            {
                if (computerRootNode.Tag.GetType() == typeof(AmtSystem))
                {
                    AmtSystem computer = (AmtSystem)computerRootNode.Tag;
                    ManageabilityControls.TerminalForm.DisposeRemotingForm(computer);
                    computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
                    computer.OnFetchCompleted -= new AmtSystem.FetchHandler(FetchHandler);
                    computer.Disconnect();
                    computer.Dispose();
                }
            }
        }

        private void SystemStateChangedHandler(AmtSystem system)
        {
            if (this.disposing == true)
                return;

            Debug.Assert(system != null);

            AmtSystem.ObjectStateHandler method = new AmtSystem.ObjectStateHandler(SystemStateChangedHandlerEx);
            object[] args = new object[] { system };

            Debug.Assert(method != null);
            Debug.Assert(args != null);

            try
            {
                this.Invoke(method, args);
            }
            catch (Exception e)
            {
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Error,
                    "Error while changing system connection state",
                    String.Format("EXCEPTION: {0} (type {1})\r\nEXCEPTION STACK: {2}",
                    e.Message, e.GetType().ToString(), e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);

                Utils.ErrorMessageBox(this, "SystemStateChangedHandler",
                    "A error occurred while updating the connection state of the Intel(R) AMT system.");

                if ((system.State == AmtSystemObjState.Connected) ||
                    (system.State == AmtSystemObjState.Connecting))
                    system.Disconnect();
            }
        }

        private void SystemStateChangedHandlerEx(AmtSystem computer)
        {
            TreeNode systemNode = null; 

            if(computer.MyComputer)
                systemNode = mainTreeView.Nodes[1];
            else
                systemNode = Utils.GetNodeWithTagRec(mainTreeView.Nodes[0], computer);
            if (systemNode == null)
                return;

            if(!computer.MyComputer)
                systemNode.Text = GetNodeName(systemNode); 

            int icon = GetIconIndex(computer);
            systemNode.ImageIndex = icon;
            systemNode.SelectedImageIndex = icon;

            if (computer.State == AmtSystemObjState.Connected)
            {
                if (systemNode.Nodes.Count == 0)
                {
                    ArrayList l = new ArrayList();
                    if (computer.Assets != null)
                    {
                        TreeNode hardwareAssetNode = new TreeNode(ManageabilityCommanderTool.Properties.Resources.TreeNode_Hardware_Asset, 4, 5);
                        l.Add(hardwareAssetNode);
                    }

                    if (computer.CircuitBreaker != null)
                    {
                        TreeNode circuitBreakerNode = new TreeNode(ManageabilityCommanderTool.Properties.Resources.TreeNode_Networking, 4, 5);
                        l.Add(circuitBreakerNode);
                        TreeNode circuitBreakerFiltersNode = new TreeNode(ManageabilityCommanderTool.Properties.Resources.TreeNode_Filters, 4, 5);
                        circuitBreakerNode.Nodes.Add(circuitBreakerFiltersNode);
                        TreeNode circuitBreakerPoliciesNode = new TreeNode(ManageabilityCommanderTool.Properties.Resources.TreeNode_Policies, 4, 5);
                        circuitBreakerNode.Nodes.Add(circuitBreakerPoliciesNode);
                        if (Simplified == false && computer.Watchdog != null)
                        {
                            TreeNode watchDogNode = new TreeNode(ManageabilityCommanderTool.Properties.Resources.TreeNode_Watchdogs, 4, 5);
                            circuitBreakerNode.Nodes.Add(watchDogNode);
                        }
                        if (computer.CircuitBreaker.IsHeuristicSupported())
                        {
                            TreeNode heuristicsNode = new TreeNode(ManageabilityCommanderTool.Properties.Resources.TreeNode_Heuristics, 4, 5);
                            circuitBreakerNode.Nodes.Add(heuristicsNode);
                        }
                    }
                    if (Simplified == false && computer.Storage != null)
                    {
                        TreeNode dataStorageNode = new TreeNode(ManageabilityCommanderTool.Properties.Resources.TreeNode_Data_Storage, 4, 5);
                        dataStorageNode.Tag = computer.Storage;
                        l.Add(dataStorageNode);
                    }
                    if (computer.Events != null)
                    {
                        TreeNode eventLogNode = new TreeNode(ManageabilityCommanderTool.Properties.Resources.TreeNode_Event_Log, 4, 5);
                        l.Add(eventLogNode);
                        computer.Events.OnAlert += new AmtEventManager.AmtNetworkEventHandler(AmtNetworkEventSink);
                    }
                    if (computer.AuditLog != null)
                    {
                        TreeNode auditLogNode = new TreeNode(ManageabilityCommanderTool.Properties.Resources.TreeNode_Audit_Log, 4, 5);
                        l.Add(auditLogNode);
                    }

                    // Add all the nodes at once
                    systemNode.Nodes.AddRange((TreeNode[])l.ToArray(typeof(TreeNode)));

                    // We are in single computer mode, let expand the branches.
                    if (rootNode == null)
                        computerRootNode.ExpandAll();
                }
            }
            else
            {
                systemNode.Nodes.Clear();
                if (computer.State == AmtSystemObjState.Disconnected)
                    ManageabilityControls.TerminalForm.DisposeRemotingForm(computer);
            }

            if (DisplayedNode == systemNode)
            {
                mainTreeView_AfterSelect(this, null);
            }
        }



        public void FetchHandler(AmtSystem computer, object fetch)
        {
            if (this.disposing == true || fetch == null)
                return;
            Invoke(new AmtSystem.FetchHandler(FetchHandlerEx), new object[] { computer, fetch });
        }

        private void FetchHandlerEx(AmtSystem computer, object fetch)
        {
            try
            {
                if (fetch == computer.Assets)
                    UpdateTreeAssets(computer);
                if (Simplified == false)
                {
                    if (fetch == computer.Storage)
                        UpdateTreeStorage(computer);
                    if (fetch == computer.Watchdog)
                        UpdateTreeWatchdogs(computer);
                }
                if (fetch == computer.Events && mainTreeView.SelectedNode != null && mainTreeView.SelectedNode.Parent != null && mainTreeView.SelectedNode.Parent.Tag == computer)
                    mainTreeView_AfterSelect(this, null);
                if (fetch == computer.Info && mainTreeView.SelectedNode != null && mainTreeView.SelectedNode.Parent != null && mainTreeView.SelectedNode.Parent.Tag == computer)
                    mainTreeView_AfterSelect(this, null);
                if (fetch == computer.RemoteAccessAdmin && mainTreeView.SelectedNode != null && mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag == computer)
                {
                    mainTreeView_AfterSelect(this, null);
                }
                if (fetch == computer.CircuitBreaker)
                {
                    UpdateTreePolicies(computer);
                    UpdateTreeFilters(computer);
                }
                if (fetch.GetType() == typeof(AmtStorageBlock) && mainTreeView.SelectedNode != null && mainTreeView.SelectedNode.Tag == fetch)
                    mainTreeView_AfterSelect(this, null);
            }
            catch (NullReferenceException e)
            {
                string s = e.Message;
                System.Diagnostics.Debugger.Break();
            }
        }

        private void refreshWatchdogsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag == null && mainTreeView.SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Watchdogs) == 0)
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Tag;
                computer.Watchdog.ClearCache();
                UpdateTreeWatchdogs(computer);
            }
        }

        private void refreshStorageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtStorage))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
                computer.Storage.ClearCache();
                UpdateTreeStorage(computer);
            }
        }

        private void removeEnterpriseToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag == null || mainTreeView.SelectedNode.Tag.GetType() != typeof(AmtEnterprise))
                return;

            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Tag;
            AmtEnterprise enterprise = (AmtEnterprise)mainTreeView.SelectedNode.Tag;

            computer.Storage.RemoveEnterpriseEntry(enterprise);
            UpdateTreeStorage(computer);
        }

        private void removeApplicationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag == null || mainTreeView.SelectedNode.Tag.GetType() != typeof(AmtApplication))
                return;

            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Parent.Tag;
            AmtApplication application = (AmtApplication)mainTreeView.SelectedNode.Tag;

            computer.Storage.RemoveApplication(application);
            UpdateTreeStorage(computer);
        }

        private void removeBlockToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag == null || mainTreeView.SelectedNode.Tag.GetType() != typeof(AmtStorageBlock))
                return;

            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Parent.Parent.Tag;
            AmtStorageBlock block = (AmtStorageBlock)mainTreeView.SelectedNode.Tag;

            AmtStorageWrapper.AmtStorageStatus r;
            r = block.Remove();
            if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
            {
                MessageBox.Show(this, string.Format(Properties.Resources.BlockRemovalError, r.ToString()), Properties.Resources.StorageBlocksTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                UpdateTreeStorage(computer);
            }
        }

        private void addEditEnterprisesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtStorage))
            {
                AmtStorage storage = (AmtStorage)mainTreeView.SelectedNode.Tag;
                new StorageEnterpriseForm(storage).ShowDialog(this);

                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
                UpdateTreeStorage(computer);
            }
        }

        private void allocateBlockToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag == null)
                return;

            AmtSystem computer = null;
            AmtStorage storage = null;
            AmtEnterprise enterprise = null;
            AmtApplication application = null;

            if (mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtStorage))
            {
                storage = (AmtStorage)mainTreeView.SelectedNode.Tag;
                computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
            }
            if (mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtEnterprise))
            {
                enterprise = (AmtEnterprise)mainTreeView.SelectedNode.Tag;
                storage = (AmtStorage)mainTreeView.SelectedNode.Parent.Tag;
                computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Tag;
            }
            if (mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtApplication))
            {
                application = (AmtApplication)mainTreeView.SelectedNode.Tag;
                enterprise = (AmtEnterprise)mainTreeView.SelectedNode.Parent.Tag;
                storage = (AmtStorage)mainTreeView.SelectedNode.Parent.Parent.Tag;
                computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Parent.Tag;
            }

            if (storage == null)
                return;
            AmtEnterprise[] enterprises = storage.GetEnterpriseEntries();
            if (enterprises == null || enterprises.Length == 0)
                return;

            AllocateStorageForm form = new AllocateStorageForm(storage, enterprise, application);

            if (form.ShowDialog(this) == DialogResult.OK)
            {
                UpdateTreeStorage(computer);
            }
        }

        private void showManageabilityStackDebugInfoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AmtDebugForm.ShowForm();
        }

        private void clearWebServicesCacheToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ArrayList l = Utils.GetNodeTagsofTypeRec(mainTreeView.Nodes[0], typeof(AmtSystem));
            if (l != null && l.Count > 0)
            {
                foreach (AmtSystem s in l)
                    s.ClearCache();
            }
        }

        private void fetchWebServiceCacheToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (FetchCacheThread != null)
                return;

            ArrayList computers = Utils.GetNodeTagsofTypeRec(mainTreeView.Nodes[0], typeof(AmtSystem));
            if (computers != null && computers.Count > 0)
            {
                FetchCacheThread = new Thread(new ParameterizedThreadStart(FetchAllCache));
                FetchCacheThread.Start(computers);
            }
        }

        private void FetchAllCache(object computerlist)
        {
            ArrayList computers = (ArrayList)computerlist;
            foreach (AmtSystem computer in computers)
                computer.FetchCache();
            FetchCacheThread = null;
        }

        private void batchRemoteControlToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (rootNode == null)
                return;
            ArrayList l = Utils.GetNodeTagsofType(rootNode, typeof(AmtSystem));
            if (batchForm != null)
            {
                batchForm.SetSystems((AmtSystem[])l.ToArray(typeof(AmtSystem)));
                batchForm.Activate();
            }
            else
            {
                batchForm = new BatchForm(this);
                batchForm.SetSystems((AmtSystem[])l.ToArray(typeof(AmtSystem)));
                batchForm.Show();
            }
        }

        private void alertViewerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AlertViewer.ShowAlertViewer(this);
        }

        
        private void addMPSServerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (rootNode == null)
                return;

            ManageabilityControls.AddMpsForm f = new ManageabilityControls.AddMpsForm();
            if (f.ShowDialog(this) == DialogResult.OK)
            {
                AmtMpsServer mps = new AmtMpsServer(f.Hostname, f.HttpPort, f.SocksPort, f.ForceIPOption, f.IPAddress);
                AddMps(mps);
                SaveSettings();
            }
        }

        /// <summary>
        /// Network alert sink
        /// </summary>
        private void AmtNetworkEventSink(AmtEventManager source, AmtSystem system, AmtAlert alert)
        {
            object[] p = new object[3];
            p[0] = source;
            p[1] = system;
            p[2] = alert;
            this.Invoke(new AmtEventManager.AmtNetworkEventHandler(AmtNetworkEventSinkEx), p);
        }

        /// <summary>
        /// If we receive an alert while the user is looking at the event & alerts panel of this Intel AMT system, this, refresh the panel.
        /// </summary>
        private void AmtNetworkEventSinkEx(AmtEventManager source, AmtSystem system, AmtAlert alert)
        {
            // First, display the alert on the status bar
            eventToolStripStatusLabel.Visible = true;

            eventToolStripStatusLabel.Text = String.Format(ManageabilityCommanderTool.Properties.Resources.MainForm_Rec_Alert_Format, alert.TrapSource.ToString(), alert.SequenceNumber);

            // Check what the user is looking at
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Event_Log) != 0)
                return;
            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
            if (computer != system)
                return;

            // Refresh the panel
            mainTreeView_AfterSelect(this, null);
        }

        /// <summary>
        /// Network alert sink for all events, even with no computer is connected.
        /// </summary>
        private void AmtAllNetworkEventSink(IPEndPoint endpoint, AmtAlert alert)
        {
            object[] p = new object[2];
            p[0] = endpoint;
            p[1] = alert;
            this.Invoke(new AmtTrapListener.AmtNetworkEventHandler(AmtAllNetworkEventSinkEx), p);
        }

        /// <summary>
        /// Update the status bar when a new event is received.
        /// </summary>
        private void AmtAllNetworkEventSinkEx(IPEndPoint endpoint, AmtAlert alert)
        {
            // First, display the alert on the status bar
            eventToolStripStatusLabel.Visible = true;
            eventToolStripStatusLabel.Text = String.Format(ManageabilityCommanderTool.Properties.Resources.MainForm_Rec_Alert_Format, alert.TrapSource.ToString(), alert.SequenceNumber);
        }

        private void displayToolTipsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Properties.Settings.Default.ToolTips = mainToolTip.Active = displayToolTipsToolStripMenuItem.Checked;
        }

        private void userPanelUpdate_Click(object sender, EventArgs e)
        {
            mainTreeView_AfterSelect(this, null);
        }

        private bool RemoteCertificateValidationCallbackSink(object sender, X509Certificate certificate, X509Chain chain, System.Net.Security.SslPolicyErrors sslPolicyErrors)
        {
            if (sender.GetType() == typeof(System.Net.HttpWebRequest))
            {
                System.Net.HttpWebRequest r = (System.Net.HttpWebRequest)sender;
                X509Certificate2[] certArray = new X509Certificate2[chain.ChainElements.Count];
                for (int i = 0; i < chain.ChainElements.Count; i++)
                {
                    certArray[i] = chain.ChainElements[i].Certificate;
                }
                if (r.ConnectionGroupName != null)
                    AmtSystem.AddTlsCertificateCache(r.ConnectionGroupName, certArray);
            }
            return true; // Accept All Certificates
        }

        private void certificateManagerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Show the certificate manager form
            new ManageabilityStack.CertificateManagerForm().ShowDialog(this);

            // Regenerate the list of trusted root certs 
            ManageabilityStack.CertificateOperations.AllTrustedRootsToPemFile(Application.StartupPath + "\\Trusted Root Certificates.pem");
        }

        private void saveComputersToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (rootNode == null)
                return;
            saveFileDialog.Title = Properties.Resources.SaveIntelAmtComputers;
            saveFileDialog.DefaultExt = "xml";
            saveFileDialog.FileName = Properties.Resources.DefaultComputerFilename;
            saveFileDialog.Filter = Properties.Resources.XmlFileFilter;
            if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                ArrayList settings = new ArrayList();
                foreach (TreeNode n in rootNode.Nodes)
                {
                    if (n.Tag.GetType() == typeof(AmtSystem))
                    {
                        AmtSystem s = (AmtSystem)n.Tag;
                        settings.Add(new AmtSerialSystemSettings(s));
                    }
                    else if (n.Tag.GetType() == typeof(AmtMpsServer))
                    {
                        AmtMpsServer s = (AmtMpsServer)n.Tag;
                        settings.Add(new AmtSerialSystemSettings(s));
                    }
                }

                FileStream file = new FileStream(saveFileDialog.FileName, FileMode.Create);
                byte[] buf = UTF8Encoding.UTF8.GetBytes(Utils.Serialize(settings.ToArray(typeof(AmtSerialSystemSettings)), typeof(AmtSerialSystemSettings[])));
                file.Write(buf, 0, buf.Length);
                file.Close();
            }

        }

        private void loadComputersToolStripMenuItem_Click(object sender, EventArgs e)
        {
            openFileDialog.Title = Properties.Resources.OpenIntelAmtComputers;
            openFileDialog.DefaultExt = "xml";
            openFileDialog.FileName = Properties.Resources.DefaultComputerFilename;
            openFileDialog.Filter = Properties.Resources.XmlFileFilter;
            if (openFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                FileStream file;
                try
                {
                    file = new FileStream(openFileDialog.FileName, FileMode.Open);
                }
                catch (Exception ex)
                {
                    Utils.FileErrorMessageBox(this, ex);
                    return;
                }
                byte[] buf = new byte[file.Length];
                file.Read(buf, 0, (int)file.Length);
                file.Close();
                string text = UTF8Encoding.UTF8.GetString(buf);
                AmtSerialSystemSettings[] settings = (AmtSerialSystemSettings[])Utils.Unserialize(text, typeof(AmtSerialSystemSettings[]));

                foreach (AmtSerialSystemSettings setting in settings)
                {
                    if ((setting.MPS) && (setting.Hostname.Equals("~~MPS~~")))
                    {
                        AmtMpsServer m = setting.GetMPS();
                        if (m != null)
                            AddMps(m);
                    }
                    else
                    {
                        TreeNode systemNode = new TreeNode(setting.Hostname + Properties.Resources.SlashSeperator + setting.Username, 16, 16);
                        AddComputer(setting.GetSystem());
                    }
                }
            }
        }

        private void mainTreeView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                ToolStripMenuItem mi = null;
                TreeNode node = mainTreeView.SelectedNode;
                if (node.Tag != null && node.Tag.GetType() == typeof(AmtWatchdogAgent))
                    mi = removeWatchdogToolStripMenuItem;
                if (node.Tag != null && node.Tag.GetType() == typeof(AmtEnterprise))
                    mi = removeEnterpriseToolStripMenuItem;
                if (node.Tag != null && node.Tag.GetType() == typeof(AmtApplication))
                    mi = removeApplicationToolStripMenuItem;
                if (node.Tag != null && node.Tag.GetType() == typeof(AmtStorageBlock))
                    mi = removeBlockToolStripMenuItem;
                if (node.Tag != null && node.Tag.GetType() == typeof(CircuitBreakerPolicyInfoType))
                    mi = removePolicyToolStripMenuItem;

                if (mi != null && MessageBox.Show(this, Properties.Resources.RemoveSelectedItem, Properties.Resources.Remove, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    mi.PerformClick();
                    e.Handled = true;
                    return;
                }

                if (node.Tag != null && node.Tag.GetType() == typeof(AmtSystem))
                    mi = removeSystemToolStripMenuItem;
                if (node.Tag != null && node.Tag.GetType() == typeof(CircuitBreakerFilterInfoType))
                    mi = removeFilterToolStripMenuItem;
                if (mi != null)
                {
                    mi.PerformClick();
                    e.Handled = true;
                    return;
                }
            }

            if (e.KeyCode == (Keys.Control | Keys.C))
            {
                copyToolStripMenuItem_Click(this, null);
                e.Handled = true;
            }

            if (e.KeyCode == (Keys.Control | Keys.V))
            {
                pasteToolStripMenuItem_Click(this, null);
                e.Handled = true;
            }
        }

        private string ConvertToStdXml(string xml)
        {
            xml = xml.Replace("<CircuitBreakerFilterType xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">", "<CBFilter>");
            xml = xml.Replace("<CircuitBreakerFilterType>", "<CBFilter>");
            xml = xml.Replace("</CircuitBreakerFilterType>", "</CBFilter>");
            xml = xml.Replace(" xmlns=\"http://schemas.intel.com/2004/12/management/PC/CircuitBreaker\"", "");
            xml = xml.Replace("<TotalCircuitBreakerPolicy xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">", "<CBRuleSet>");
            xml = xml.Replace("</TotalCircuitBreakerPolicy>", "</CBRuleSet>");
            return xml;
        }

        private string ConvertFromStdXml(string xml)
        {
            if (xml.IndexOf("</ArrayOfCircuitBreakerFilterType>") >= 0)
                xml = xml.Replace("<CBFilter>", "<CircuitBreakerFilterType>");
            else
                xml = xml.Replace("<CBFilter>", "<CircuitBreakerFilterType xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">");
            xml = xml.Replace("</CBFilter>", "</CircuitBreakerFilterType>");
            xml = xml.Replace("<FilterName>", "<FilterName xmlns=\"http://schemas.intel.com/2004/12/management/PC/CircuitBreaker\">");
            xml = xml.Replace("<FilterDirection>", "<FilterDirection xmlns=\"http://schemas.intel.com/2004/12/management/PC/CircuitBreaker\">");
            xml = xml.Replace("<FilterProfile>", "<FilterProfile xmlns=\"http://schemas.intel.com/2004/12/management/PC/CircuitBreaker\">");
            xml = xml.Replace("<FilterProfileData>", "<FilterProfileData xmlns=\"http://schemas.intel.com/2004/12/management/PC/CircuitBreaker\">");
            xml = xml.Replace("<FilterPacket>", "<FilterPacket xmlns=\"http://schemas.intel.com/2004/12/management/PC/CircuitBreaker\">");
            xml = xml.Replace("<ActionEventOnMatch>", "<ActionEventOnMatch xmlns=\"http://schemas.intel.com/2004/12/management/PC/CircuitBreaker\">");
            xml = xml.Replace("<PolicyName>", "<PolicyName xmlns=\"http://schemas.intel.com/2004/12/management/PC/CircuitBreaker\">");
            xml = xml.Replace("<PolicyPrecedence>", "<PolicyPrecedence xmlns=\"http://schemas.intel.com/2004/12/management/PC/CircuitBreaker\">");
            xml = xml.Replace("<FilterCreationHandles>", "<FilterCreationHandles xmlns=\"http://schemas.intel.com/2004/12/management/PC/CircuitBreaker\">");
            xml = xml.Replace("<DefaultTXFilter>", "<DefaultTXFilter xmlns=\"http://schemas.intel.com/2004/12/management/PC/CircuitBreaker\">");
            xml = xml.Replace("<DefaultRXFilter>", "<DefaultRXFilter xmlns=\"http://schemas.intel.com/2004/12/management/PC/CircuitBreaker\">");
            xml = xml.Replace("<CBRuleSet>", "<TotalCircuitBreakerPolicy xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">");
            xml = xml.Replace("</CBRuleSet>", "</TotalCircuitBreakerPolicy>");
            return xml;
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            TreeNode node = mainTreeView.SelectedNode;

            if (node.Tag != null && node.Tag.GetType() == typeof(CircuitBreakerFilterInfoType))
            {
                // Copy the selected filter
                CircuitBreakerFilterInfoType filter = (CircuitBreakerFilterInfoType)node.Tag;
                string xml = Utils.Serialize(filter.Filter, typeof(CircuitBreakerFilterType));
                Clipboard.SetDataObject(ConvertToStdXml(xml), true);
            }
            else if (node.Tag == null && node.Text.Equals(ManageabilityCommanderTool.Properties.Resources.TreeNode_Filters))
            {
                // Copy all of the filters
                ArrayList l = new ArrayList();
                foreach (TreeNode n in node.Nodes)
                    l.Add(((CircuitBreakerFilterInfoType)n.Tag).Filter);
                CircuitBreakerFilterType[] filterArray = (CircuitBreakerFilterType[])l.ToArray(typeof(CircuitBreakerFilterType));
                string xml = Utils.Serialize(filterArray, typeof(CircuitBreakerFilterType[]));
                Clipboard.SetDataObject(ConvertToStdXml(xml), true);
            }
            else if (node.Tag != null && node.Tag.GetType() == typeof(AmtWatchdogAgent))
            {
                // Copy watchdog
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Parent.Tag;
                AmtCompleteWatchdogAgent a = new AmtCompleteWatchdogAgent();
                a.agent = (AmtWatchdogAgent)node.Tag;
                a.actions = computer.Watchdog.GetActions(a.agent);
                Utils.SerializeToClipboard(a, typeof(AmtCompleteWatchdogAgent));
            }
            else if (node.Tag == null && node.Text.Equals(ManageabilityCommanderTool.Properties.Resources.TreeNode_Watchdogs))
            {
                // Copy watchdogs
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Tag;
                ArrayList l = new ArrayList();
                foreach (TreeNode n in node.Nodes)
                {
                    AmtCompleteWatchdogAgent a = new AmtCompleteWatchdogAgent();
                    a.agent = (AmtWatchdogAgent)n.Tag;
                    a.actions = computer.Watchdog.GetActions(a.agent);
                    l.Add(a);
                }
                AmtCompleteWatchdogAgent[] wgArray = (AmtCompleteWatchdogAgent[])l.ToArray(typeof(AmtCompleteWatchdogAgent));
                Utils.SerializeToClipboard(wgArray, typeof(AmtCompleteWatchdogAgent[]));
            }
            else if (node.Tag != null && node.Tag.GetType() == typeof(CircuitBreakerPolicyInfoType))
            {
                // Copy policy
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Parent.Tag;
                CircuitBreakerPolicyType policy = ((CircuitBreakerPolicyInfoType)node.Tag).Policy;
                AmtCircuitBreaker.TotalCircuitBreakerPolicy p = new AmtCircuitBreaker.TotalCircuitBreakerPolicy(computer, policy);
                //Utils.SerializeToClipboard(p, typeof(AmtCircuitBreaker.TotalCircuitBreakerPolicy));
                string xml = Utils.Serialize(p, typeof(AmtCircuitBreaker.TotalCircuitBreakerPolicy));
                Clipboard.SetDataObject(ConvertToStdXml(xml), true);
            }
            else if (node.Tag == null && node.Text.Equals(ManageabilityCommanderTool.Properties.Resources.TreeNode_Policies))
            {
                // Copy policies
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Tag;
                ArrayList l = new ArrayList();
                foreach (TreeNode n in node.Nodes)
                {
                    CircuitBreakerPolicyType policy = ((CircuitBreakerPolicyInfoType)n.Tag).Policy;
                    AmtCircuitBreaker.TotalCircuitBreakerPolicy p = new AmtCircuitBreaker.TotalCircuitBreakerPolicy(computer, policy);
                    l.Add(p);
                }
                AmtCircuitBreaker.TotalCircuitBreakerPolicy[] pArray = (AmtCircuitBreaker.TotalCircuitBreakerPolicy[])l.ToArray(typeof(AmtCircuitBreaker.TotalCircuitBreakerPolicy));
                //Utils.SerializeToClipboard(pArray, typeof(AmtCircuitBreaker.TotalCircuitBreakerPolicy[]));
                string xml = Utils.Serialize(pArray, typeof(AmtCircuitBreaker.TotalCircuitBreakerPolicy[]));
                Clipboard.SetDataObject(ConvertToStdXml(xml), true);
            }
            else if (node.Tag != null && node.Tag.GetType() == typeof(AmtSystem))
            {
                // Copy system settings
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                AmtSerialSystemSettings settings = new AmtSerialSystemSettings(computer);
                Utils.SerializeToClipboard(settings, typeof(AmtSerialSystemSettings));
            }
            else if (node == rootNode)
            {
                // Copy system settings
                ArrayList l = new ArrayList();
                foreach (TreeNode n in rootNode.Nodes)
                {
                    if (n.Tag.GetType() == typeof(AmtSystem))
                    {
                        l.Add(new AmtSerialSystemSettings((AmtSystem)(n.Tag)));
                    }
                }
                AmtSerialSystemSettings[] sa = (AmtSerialSystemSettings[])l.ToArray(typeof(AmtSerialSystemSettings));
                Utils.SerializeToClipboard(sa, typeof(AmtSerialSystemSettings[]));
            }
        }

        private void pasteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            TreeNode node = mainTreeView.SelectedNode;

            if (node.Tag == null && node.Text.Equals(ManageabilityCommanderTool.Properties.Resources.TreeNode_Filters))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Tag;
                string xml = (string)(Clipboard.GetDataObject().GetData(typeof(string)));
                xml = ConvertFromStdXml(xml);

                // Paste filters
                CircuitBreakerFilterType[] filters = (CircuitBreakerFilterType[])Utils.Unserialize(xml, typeof(CircuitBreakerFilterType[]));
                if (filters != null)
                {
                    foreach (CircuitBreakerFilterType f in filters)
                    {
                        computer.CircuitBreaker.CircuitBreakerCreateFilter(f);
                    }
                    UpdateTreeFilters(computer);
                    return;
                }

                // Paste filter
                CircuitBreakerFilterType filter = (CircuitBreakerFilterType)Utils.Unserialize(xml, typeof(CircuitBreakerFilterType));
                if (filter != null)
                {
                    computer.CircuitBreaker.CircuitBreakerCreateFilter(filter);
                    UpdateTreeFilters(computer);
                    return;
                }
            }

            if (node.Tag == null && node.Text.Equals(ManageabilityCommanderTool.Properties.Resources.TreeNode_Watchdogs))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Tag;

                // Paste watchdogs
                AmtCompleteWatchdogAgent[] wgs = (AmtCompleteWatchdogAgent[])Utils.UnSerializeFromClipboard(typeof(AmtCompleteWatchdogAgent[]));
                if (wgs != null)
                {
                    foreach (AmtCompleteWatchdogAgent a in wgs)
                    {
                        computer.Watchdog.Add(a.agent);
                        if (a.actions.Length > 0)
                            computer.Watchdog.SetActions(a.agent, a.actions);
                    }
                    UpdateTreeWatchdogs(computer);
                    return;
                }

                // Paste watchdog
                AmtCompleteWatchdogAgent wg = (AmtCompleteWatchdogAgent)Utils.UnSerializeFromClipboard(typeof(AmtCompleteWatchdogAgent));
                if (wg != null)
                {
                    computer.Watchdog.Add(wg.agent);
                    if (wg.actions.Length > 0)
                        computer.Watchdog.SetActions(wg.agent, wg.actions);
                    UpdateTreeWatchdogs(computer);
                    return;
                }
            }

            if (node.Tag == null && node.Text.Equals(ManageabilityCommanderTool.Properties.Resources.TreeNode_Policies))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Tag;
                string xml = (string)(Clipboard.GetDataObject().GetData(typeof(string)));
                xml = ConvertFromStdXml(xml);

                // Paste policies
                AmtCircuitBreaker.TotalCircuitBreakerPolicy[] policies = (AmtCircuitBreaker.TotalCircuitBreakerPolicy[])Utils.Unserialize(xml, typeof(AmtCircuitBreaker.TotalCircuitBreakerPolicy[]));
                if (policies != null)
                {
                    foreach (AmtCircuitBreaker.TotalCircuitBreakerPolicy p in policies)
                    {
                        p.Policy.FilterCreationHandles = p.GetFilterhandles(computer);
                        computer.CircuitBreaker.CircuitBreakerCreatePolicy(p.Policy);
                    }
                    UpdateTreePolicies(computer);
                    return;
                }

                // Paste policy
                AmtCircuitBreaker.TotalCircuitBreakerPolicy totalPolicy = (AmtCircuitBreaker.TotalCircuitBreakerPolicy)Utils.Unserialize(xml, typeof(AmtCircuitBreaker.TotalCircuitBreakerPolicy));
                if (totalPolicy != null)
                {
                    totalPolicy.Policy.FilterCreationHandles = totalPolicy.GetFilterhandles(computer);
                    computer.CircuitBreaker.CircuitBreakerCreatePolicy(totalPolicy.Policy);
                    UpdateTreePolicies(computer);
                    return;
                }
            }

            if (node == rootNode)
            {
                AmtSerialSystemSettings settings = (AmtSerialSystemSettings)Utils.UnSerializeFromClipboard(typeof(AmtSerialSystemSettings));
                if (settings != null)
                {
                    AddComputer(settings.GetSystem());
                    return;
                }

                AmtSerialSystemSettings[] settingsArray = (AmtSerialSystemSettings[])Utils.UnSerializeFromClipboard(typeof(AmtSerialSystemSettings[]));
                if (settingsArray != null)
                {
                    foreach (AmtSerialSystemSettings setting in settingsArray)
                    {
                        AddComputer(setting.GetSystem());
                    }
                    return;
                }
            }

        }

        private void ManageabilityDTKLatestNewsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ManageabilityControls.AboutForm.GotoNews();
        }

        private void ManageabilityDTKDownloadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ManageabilityControls.AboutForm.GotoDownloads();
        }

        private void ManageabilityDTKSupportForumsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ManageabilityControls.AboutForm.GotoForums();
        }

        private void showStatusBarToolStripMenuItem_CheckedChanged(object sender, EventArgs e)
        {
            statusStrip.Visible = showStatusBarToolStripMenuItem.Checked;
        }

        private void showUsernamesToolStripMenuItem_CheckedChanged(object sender, EventArgs e)
        {
            UpdateTreeViewNamesRec(rootNode);
        }

        private void showDomainNamesToolStripMenuItem_CheckedChanged(object sender, EventArgs e)
        {
            UpdateTreeViewNamesRec(rootNode);
        }

        private void UpdateTreeViewNamesRec(TreeNode n)
        {
            foreach (TreeNode nn in n.Nodes)
            {
                string s = GetNodeName(nn);
                if (s != null)
                    nn.Text = s;
                UpdateTreeViewNamesRec(nn);
            }
            if (Utils.IsMono())
                mainTreeView.Invalidate();
        }

        private string GetNodeName(TreeNode n)
        {
            if (n.Tag == null)
                return null;
            if (n.Tag.GetType() == typeof(AmtSystem))
            {
                string name = "";
                AmtSystem computer = (AmtSystem)n.Tag;
                if (showDomainNamesToolStripMenuItem.Checked == true)
                {
                    name += computer.HostNameIdentity.DisplayName;
                }
                else
                {
                    name += GetShortHostName(computer.HostNameIdentity.DisplayName);
                }
                if (showUsernamesToolStripMenuItem.Checked == true)
                {
                    if (computer.Username.Length == 0 && computer.Password.Length == 0)
                    {
                        name += Properties.Resources.SlashSeperator + Properties.Resources.KerberosLocalUser;
                    }
                    else
                    {
                        name += Properties.Resources.SlashSeperator + computer.Username;
                    }
                }
                return name;
            }           
            return null;
        }

        private static string GetShortHostName(string longHostName)
        {
            IPAddress addr = null;
            if (System.Net.IPAddress.TryParse(longHostName, out addr) == false)
            {
                int i = longHostName.IndexOf(".");
                if (i >= 0)
                    return longHostName.Substring(0, i);
            }
            return longHostName;
        }

        private void mainTreeView_DoubleClick(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                if (computer.State == AmtSystemObjState.Connected)
                {
                    computer.Disconnect();
                }
                else
                {
                    connectToolStripMenuItem_Click(this, null);
                }
            }
        }

        private void advancedPropertiesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new AdvancedPropertiesForm().ShowDialog(this);
        }


        private void clearAllEntriesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (rootNode.Nodes.Count == 0)
                return;
            if (MessageBox.Show(this, Properties.Resources.ConfirmRemoveAllComputers, Properties.Resources.ConfirmRemoveAllEntires, MessageBoxButtons.OKCancel, MessageBoxIcon.Asterisk) != DialogResult.OK)
                return;

            foreach (TreeNode node in rootNode.Nodes)
            {
                if (node.Tag != null && node.Tag.GetType() == typeof(AmtSystem))
                {
                    AmtSystem computer = (AmtSystem)node.Tag;
                    ManageabilityControls.TerminalForm.DisposeRemotingForm(computer);
                    computer.Disconnect();
                    computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
                    computer.OnFetchCompleted -= new AmtSystem.FetchHandler(FetchHandler);

                }
            }
            rootNode.Nodes.Clear();
            mainTreeView.SelectedNode = rootNode;
            SaveSettings();
        }

        /// <summary>
        /// Rebrand the UI using other bitmaps and string if the Branding directory is present
        /// </summary>
        private void Rebrand()
        {
            if (Directory.Exists(Application.StartupPath + "\\Branding") == false)
                return;

            // Change the form icon
            if (File.Exists(Application.StartupPath + "\\Branding\\Console.ico"))
            {
                this.Icon = new Icon(Application.StartupPath + "\\Branding\\Console.ico");
            }

            if (File.Exists(Application.StartupPath + "\\Branding\\TopBanner.jpg"))
            {
                Image i = Image.FromFile(Application.StartupPath + "\\Branding\\TopBanner.jpg");
                PictureBox p = new PictureBox();
                p.BackColor = Color.Black;
                p.Image = i;
                p.Dock = DockStyle.Top;
                p.Height = i.Height;
                this.Controls.Add(p);
                this.Controls.Remove(mainMenuStrip);
                this.Controls.Add(mainMenuStrip);
                this.Height += i.Height;
                this.MinimumSize = new Size(this.MinimumSize.Width, this.MinimumSize.Height);
            }

            if (File.Exists(Application.StartupPath + "\\Branding\\Branding.txt"))
            {
                FileStream f = new FileStream(Application.StartupPath + "\\Branding\\Branding.txt", FileMode.Open);
                byte[] buf = new byte[f.Length];
                f.Read(buf, 0, (int)(f.Length));
                f.Close();
                string data = UTF8Encoding.UTF8.GetString(buf);
                string[] brandStrings = data.Replace("\r\n", "\r").Split('\r');
                if (brandStrings.Length > 0)
                    Text = brandStrings[0];
                if (brandStrings.Length > 1)
                    mainToolStripStatusLabel.Text = brandStrings[1];
                if (brandStrings.Length > 2)
                    aboutToolStripMenuItem.Text = Utils.FilterStr(brandStrings[2]);

                ManageabilityDTKLatestNewsToolStripMenuItem.Visible = false;
                ManageabilityDTKDownloadToolStripMenuItem.Visible = false;
                ManageabilityDTKSupportForumsToolStripMenuItem.Visible = false;
                toolStripMenuItem7.Visible = false;
            }
        }

        public void ShowPolicyMonitor()
        {
            AmtSystem computer = null;

            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(CircuitBreakerPolicyInfoType))
            {
                computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Parent.Tag;
            }

            if (mainTreeView.SelectedNode != null && mainTreeView.SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Policies) == 0)
            {
                computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Tag;
            }
            else if (mainTreeView.SelectedNode != null && mainTreeView.SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Networking) == 0)
            {
                computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
            }

            if (computer != null)
            {
                PolicyMonitorForm f = (PolicyMonitorForm)policyMonitorTable[computer];
                if (f == null)
                {
                    f = new PolicyMonitorForm(computer);
                    policyMonitorTable[computer] = f;
                    f.Show(this);
                }
                else
                {
                    try
                    {
                        f.Visible = true;
                        f.Activate();
                    }
                    catch (ObjectDisposedException)
                    {
                        policyMonitorTable.Remove(computer);
                        f = new PolicyMonitorForm(computer);
                        policyMonitorTable[computer] = f;
                        f.Show(this);
                    }
                }
            }
        }

        private void connectToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                if (computer.State != AmtSystemObjState.Connected)
                {
                    computer.WsManSupport = Properties.Settings.Default.WsmanSupport;
                    computer.Connect();
                }
            }
        }

        private void disconnectToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                if (computer.State == AmtSystemObjState.Connected)
                {
                    computer.Disconnect();
                }
            }
        }

        private void policyMonitorToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ShowPolicyMonitor();
        }

        private void addAMTSystemToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null || mainTreeView.SelectedNode == rootNode)
            {
                // Add a computer to Commander
                ManageabilityControls.AddSystemForm f = new ManageabilityControls.AddSystemForm(false);
                if (f.ShowDialog(this) == DialogResult.OK)
                {
                    AmtSystem computer = new AmtSystem(f.Hostname, 16992, f.Username, f.Password, f.RememberPassword, false, f.ForceIPOption, f.IPAddress);
                    AddComputer(computer);
                    if (rootNode != null)
                        rootNode.Expand();
                    SaveSettings();
                }
            }
            else if (mainTreeView.SelectedNode != null && mainTreeView.SelectedNode.Tag != null && (mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtMpsServer)))
            {
                // Add a computer to MPS
                AmtMpsServer mps = (AmtMpsServer)mainTreeView.SelectedNode.Tag;
                ManageabilityControls.AddSystemForm f = new ManageabilityControls.AddSystemForm(true, true);
                if (f.ShowDialog(this) == DialogResult.OK)
                {
                    AmtSystem computer = new AmtSystem(f.Hostname, (f.UseTls ? 16993 : 16992), f.Username, f.Password, f.RememberPassword, false);
                    mps.AddComputer(computer);
                    AddComputer(computer, mainTreeView.SelectedNode);
                    mainTreeView.SelectedNode.Expand();
                    SaveSettings();
                }
            }
        }

        private void takeControlButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;

                if (ManageabilityStack.AmtRedirectorWrapper.CheckRedirectionWrapper() == false)
                {
                    MessageBox.Show("IMRSDK.dll file not found.", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                if (computer.Redirection != null && computer.Redirection.GetRedirectionListenerState() == false)
                {
                    MessageBox.Show(this,
                        ManageabilityCommanderTool.Properties.Resources.MainForm_PortDisable_ErrStr,
                        ManageabilityCommanderTool.Properties.Resources.MainForm_Terminal,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
                else
                {
                    ManageabilityControls.TerminalForm.ShowRemotingForm(computer, this, Simplified);
                }
            }
        }

        private void refreshEventsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CurrentPanelControl.ForceUpdatePanel();
        }

        private void addFilterToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CurrentPanelControl.PerformAction("ADDFILTER", null);
        }

        private void refreshAssetsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CurrentPanelControl.ForceUpdatePanel();
        }

        private void removeFilterToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CurrentPanelControl.PerformAction("REMOVE", null);
        }

        private void addPolicyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CurrentPanelControl.PerformAction("ADD", null);
        }

        private void removePolicyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CurrentPanelControl.PerformAction("REMOVE", null);
        }

        private void addWatchdogToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CurrentPanelControl.PerformAction("ADD", null);
        }

        private void removeWatchdogToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CurrentPanelControl.PerformAction("REMOVE", null);
        }

        private void setCredentialsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            TreeNode node = mainTreeView.SelectedNode;
            if (node == null)
                return;
            if (node.Tag == null || node.Tag.GetType() != typeof(AmtSystem))
                return;
        }

        private void removeMPSToolStripMenuItem_Click(object sender, EventArgs e)
        {
            TreeNode node = mainTreeView.SelectedNode;
            if (node == null)
                return;
            if (MessageBox.Show(this, "Remove selected MPS?", "Remove MPS", MessageBoxButtons.OKCancel, MessageBoxIcon.Asterisk) != DialogResult.OK)
                return;

            if (node.Tag != null && node.Tag.GetType() == typeof(AmtMpsServer))
            {
                foreach (TreeNode n in node.Nodes)
                {
                    AmtSystem computer = (AmtSystem)n.Tag;
                    computer.Disconnect();
                    computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
                    computer.OnFetchCompleted -= new AmtSystem.FetchHandler(FetchHandler);
                    n.Remove();
                }

                AmtMpsServer mps = (AmtMpsServer)mainTreeView.SelectedNode.Tag;
                node.Remove();
                SaveSettings();
            }
        }

        private void unprovisionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            TreeNode node = mainTreeView.SelectedNode;
            if (node == null)
                return;
            System.Diagnostics.Debug.Assert(node.Tag.GetType() == typeof(AmtSystem));
            AmtSystem system = (AmtSystem)node.Tag;
            UnprovisionForm unprovisionForm = new UnprovisionForm(system);
            DialogResult dr = unprovisionForm.ShowDialog();
            if (dr == DialogResult.OK)
            {
                ///////////////////////////////////
                // assess our provisioning mode
                AmtSecurityAdmin.ProvisioningMode ProvMode = AmtSecurityAdmin.ProvisioningMode.Current;
                if (unprovisionForm.changeToEnterpriseRadioButton.Checked == true)
                    ProvMode = AmtSecurityAdmin.ProvisioningMode.Enterprise;
                else if (unprovisionForm.changeToSmbRadioButton.Checked == true)
                    ProvMode = AmtSecurityAdmin.ProvisioningMode.SmallBusiness;

                ///////////////////////////////
                // Perform the unprovision
                AmtCallStatus result;
                if (unprovisionForm.fullUnprovisionCheckBox.Checked == true)
                    result = system.SecurityAdmin.Unprovision(ProvMode);
                else
                    result = system.SecurityAdmin.PartialUnprovision();

                if (result != AmtCallStatus.SUCCESS)
                {
                    MessageBox.Show("An error occurred during Unprovision (" + result.ToString() + ")",
                        "Unprovision Error",
                        MessageBoxButtons.OK);
                }
                else
                {
                    system.Disconnect();
                    MessageBox.Show("Successfully unprovisioned", "Success", MessageBoxButtons.OK);
                }
            }
        }

        public void Unprovision(object sender, EventArgs e)
        {
            unprovisionToolStripMenuItem_Click(sender, e);
        }

        private void flushDNSCacheToolStripMenuItem_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process proc;
            string path = Environment.GetFolderPath(Environment.SpecialFolder.System) + "\\ipconfig.exe";
            proc = System.Diagnostics.Process.Start(path, "/flushdns");
            if (proc.WaitForExit(1000) == false)
            {
                MessageBox.Show("Flush DNS command did not complete within 1 second", "Error");
                proc.Kill();
            }

            System.Diagnostics.Debug.Print("'IPConfig /flushdns' returned exit code " + proc.ExitCode);
        }
    }
}
