// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Net;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class ManagedInterfaceEditForm : Form
    {
        private AmtSystem computer;
        private AmtNetworkAdminInterface inf;

        public ManagedInterfaceEditForm(AmtSystem computer, AmtNetworkAdminInterface inf)
        {
            this.computer = computer;
            this.inf = inf;
            InitializeComponent();
        }

        private void ManagedInterfaceEditForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            InterfaceDescriptorGetType t = inf.GetInterfaceSettings();

            if (t.DhcpEnabled == true)
            {
                radioButton1.Checked = true;
            }
            else
            {
                radioButton2.Checked = true;
            }

            if (t.IPv4Parameters != null)
            {
                ipAddrTextBox.Text = new IPAddress((uint)System.Net.IPAddress.NetworkToHostOrder((int)t.IPv4Parameters.LocalAddress)).ToString();
                subnetTextBox.Text = new IPAddress((uint)System.Net.IPAddress.NetworkToHostOrder((int)t.IPv4Parameters.SubnetMask)).ToString();
                gatewayTextBox.Text = new IPAddress((uint)System.Net.IPAddress.NetworkToHostOrder((int)t.IPv4Parameters.DefaultGatewayAddress)).ToString();
                dns1TextBox.Text = new IPAddress((uint)System.Net.IPAddress.NetworkToHostOrder((int)t.IPv4Parameters.PrimaryDnsAddress)).ToString();
                dns2TextBox.Text = new IPAddress((uint)System.Net.IPAddress.NetworkToHostOrder((int)t.IPv4Parameters.SecondaryDnsAddress)).ToString();
            }

            bool vlanmode;
            ushort vlantag;
            if (inf.GetVlanParameters(out vlanmode, out vlantag) == AmtCallStatus.SUCCESS)
            {
                vlanCheckBox.Checked = vlanmode;
                vlanNumericUpDown.Value = vlantag;
            }

            UpdateInfo();
        }

        private void UpdateInfo()
        {
            ipAddrTextBox.Enabled = (radioButton2.Checked == true);
            subnetTextBox.Enabled = (radioButton2.Checked == true);
            gatewayTextBox.Enabled = (radioButton2.Checked == true);
            dns1TextBox.Enabled = (radioButton2.Checked == true);
            dns2TextBox.Enabled = (radioButton2.Checked == true);
            vlanNumericUpDown.Enabled = (vlanCheckBox.Checked == true);

            bool ok = true;
            if (radioButton2.Checked == true)
            {
                IPAddress addr;
                if (IPAddress.TryParse(ipAddrTextBox.Text, out addr) == false) ok = false;
                if (IPAddress.TryParse(subnetTextBox.Text, out addr) == false) ok = false;
                if (IPAddress.TryParse(gatewayTextBox.Text, out addr) == false) ok = false;
                if (IPAddress.TryParse(dns1TextBox.Text, out addr) == false) ok = false;
                if (IPAddress.TryParse(dns2TextBox.Text, out addr) == false) ok = false;
            }
            okButton.Enabled = ok;
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            AmtCallStatus r = inf.SetVlanParameters(vlanCheckBox.Checked, (ushort)vlanNumericUpDown.Value);
            if (r != AmtCallStatus.SUCCESS) Utils.ErrorMessageBox(this, "SetVlanParameters", r);

            InterfaceDescriptorSetType descriptor = new InterfaceDescriptorSetType();
            if (radioButton2.Checked == false)
            {
                // DHCP Mode
                descriptor.InterfaceMode = InterfaceModeType.SHARED_MAC_ADDRESS;
                descriptor.IPv4Parameters = null;
                if (computer.CoreVersion >= (float)4.0)
                {
                    descriptor.LinkPolicy = 31;
                }
                else
                {
                    descriptor.LinkPolicy = 0x0F; // TODO **************
                }
            }
            else
            {
                // Static Mode
                if (computer.CoreVersion >= (float)4.0)
                {
                    descriptor.InterfaceMode = InterfaceModeType.SHARED_MAC_ADDRESS;
                    descriptor.LinkPolicy = 31;
                }
                else
                {
                    descriptor.InterfaceMode = InterfaceModeType.SEPARATE_MAC_ADDRESS;
                    descriptor.LinkPolicy = 0x0F; // TODO **************
                }
                descriptor.IPv4Parameters = new IPv4ParametersType();
                IPAddress addr;
                IPAddress.TryParse(ipAddrTextBox.Text, out addr);
                descriptor.IPv4Parameters.LocalAddress = Utils.GetIpAddr(addr);
                IPAddress.TryParse(subnetTextBox.Text, out addr);
                descriptor.IPv4Parameters.SubnetMask = Utils.GetIpAddr(addr);
                IPAddress.TryParse(gatewayTextBox.Text, out addr);
                descriptor.IPv4Parameters.DefaultGatewayAddress = Utils.GetIpAddr(addr);
                IPAddress.TryParse(dns1TextBox.Text, out addr);
                descriptor.IPv4Parameters.PrimaryDnsAddress = Utils.GetIpAddr(addr);
                IPAddress.TryParse(dns2TextBox.Text, out addr);
                descriptor.IPv4Parameters.SecondaryDnsAddress = Utils.GetIpAddr(addr);
            }
            r = inf.SetInterfaceSettings(descriptor);
            if (r != AmtCallStatus.SUCCESS) Utils.ErrorMessageBox(this, "SetInterfaceSettings", r);

            DialogResult = DialogResult.OK;
        }

        private void radioButton2_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void vlanCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void ipAddrTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

    }
}
