// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class Me2PowerSaverForm : Form
    {
        private AmtSystem system;

        public Me2PowerSaverForm(AmtSystem system)
        {
            InitializeComponent();
            this.system = system;
        }

        private void MePowerSaverForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            FetchState();
            UpdateInfo();
        }

        private void FetchState()
        {
            int activeStateAC;
            int wakeOnNetAccessThresholdAC;
            int wakeOnNetAccessSleepTimer;
            system.SecurityAdmin.GetPowerSavingOptions(out activeStateAC, out wakeOnNetAccessThresholdAC, out wakeOnNetAccessSleepTimer);

            activeTrackBar.Value = activeStateAC;
            if (wakeOnNetAccessThresholdAC < 0)
            {
                sleepCheckBox.Checked = false;
                //sleepTrackBar.Enabled = false;
                sleepTrackBar.Value = activeStateAC;
            }
            else
            {
                sleepCheckBox.Checked = true;
                //sleepTrackBar.Enabled = true;
                sleepTrackBar.Value = wakeOnNetAccessThresholdAC;
            }

            if (wakeOnNetAccessSleepTimer < 0)
            {
                idleCheckBox.Checked = false;
                //timeoutNumericUpDown.Enabled = false;
                timeoutNumericUpDown.Value = 0;
            }
            else
            {
                idleCheckBox.Checked = true;
                //timeoutNumericUpDown.Enabled = true;
                timeoutNumericUpDown.Value = wakeOnNetAccessSleepTimer;
            }
        }

        private void UpdateInfo()
        {
            sleepTrackBar.Enabled = sleepCheckBox.Checked;
            if (sleepTrackBar.Enabled == false)
            {
                sleepTrackBar.Value = activeTrackBar.Value;
            }
            else
            {
                //if (sleepTrackBar.Value < activeTrackBar.Value) sleepTrackBar.Value = activeTrackBar.Value;
            }

            timeoutNumericUpDown.Enabled = idleCheckBox.Checked;

            SetLabelState(s0Label, activeTrackBar.Value >= 0, sleepTrackBar.Value >= 0);
            SetLabelState(s1Label, activeTrackBar.Value >= 1, sleepTrackBar.Value >= 1);
            SetLabelState(s2Label, activeTrackBar.Value >= 2, sleepTrackBar.Value >= 2);
            SetLabelState(s3Label, activeTrackBar.Value >= 3, sleepTrackBar.Value >= 3);
            SetLabelState(s4Label, activeTrackBar.Value >= 4, sleepTrackBar.Value >= 4);
            SetLabelState(s5Label, activeTrackBar.Value >= 5, sleepTrackBar.Value >= 5);
        }

        private void SetLabelState(Label label, bool active, bool sleep)
        {
            if (active == true)
            {
                label.Text = ManageabilityCommanderTool.Properties.Resources.Me2PwrSaverForm_ON;
                label.ForeColor = Color.ForestGreen;
            }
            else if (sleep == true)
            {
                label.Text = ManageabilityCommanderTool.Properties.Resources.Me2PwrSaverForm_Sleep;
                label.ForeColor = Color.Goldenrod;
            }
            else
            {
                label.Text = ManageabilityCommanderTool.Properties.Resources.Me2PwrSaverForm_OFF;
                label.ForeColor = Color.Red;
            }
        }

        private void activeTrackBar_Scroll(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void sleepTrackBar_Scroll(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            int xActiveTrackBar = activeTrackBar.Value;
            int xSleepTrackBar = sleepTrackBar.Value;
            if (sleepCheckBox.Checked == false) { xSleepTrackBar = -1; }
            int xTimeoutNumericUpDown = (int)timeoutNumericUpDown.Value;
            if (idleCheckBox.Checked == false) { xTimeoutNumericUpDown = -1; }

            AmtCallStatus r = system.SecurityAdmin.SetPowerSavingOptions(xActiveTrackBar, xSleepTrackBar, xTimeoutNumericUpDown);
            if (r == AmtCallStatus.SUCCESS)
            {
                DialogResult = DialogResult.OK;
            }
            else
            {
                MessageBox.Show(this, 
                    ManageabilityCommanderTool.Properties.Resources.Me2PwrSaverForm_Err + r.ToString(), 
                    ManageabilityCommanderTool.Properties.Resources.Me2PwrSaverForm_Opt,
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void sleepCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void idleCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void applyButton_Click(object sender, EventArgs e)
        {
            int xActiveTrackBar = activeTrackBar.Value;
            int xSleepTrackBar = sleepTrackBar.Value;
            if (sleepCheckBox.Checked == false) { xSleepTrackBar = -1; }
            int xTimeoutNumericUpDown = (int)timeoutNumericUpDown.Value;
            if (idleCheckBox.Checked == false) { xTimeoutNumericUpDown = -1; }

            AmtCallStatus r = system.SecurityAdmin.SetPowerSavingOptions(xActiveTrackBar, xSleepTrackBar, xTimeoutNumericUpDown);
            if (r != AmtCallStatus.SUCCESS)
            {
                MessageBox.Show(this,
                    ManageabilityCommanderTool.Properties.Resources.Me2PwrSaverForm_Err + r.ToString(),
                    ManageabilityCommanderTool.Properties.Resources.Me2PwrSaverForm_Opt, 
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                FetchState();
                UpdateInfo();
            }
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
        }

        private void refreshButton_Click_1(object sender, EventArgs e)
        {
            FetchState();
            UpdateInfo();
        }
    }
}
