// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class Me3PowerSaverForm : Form
    {
        private AmtSystem system;

        public Me3PowerSaverForm(AmtSystem system)
        {
            InitializeComponent();
            this.system = system;
        }

        private void Me3PowerSaverForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            FetchState();
        }

        private void FetchState()
        {
            AmtSecurityAdmin.AmtPowerPackage[] packages = system.SecurityAdmin.GetPowerPackages();

            if (packages != null)
            {
                radioButton1.Visible = (packages.Length > 0);
                radioButton2.Visible = (packages.Length > 1);
                radioButton3.Visible = (packages.Length > 2);
                radioButton4.Visible = (packages.Length > 3);
                radioButton5.Visible = (packages.Length > 4);
                radioButton6.Visible = (packages.Length > 5);
                radioButton7.Visible = (packages.Length > 6);

                if (packages.Length > 0) radioButton1.Text = packages[0].name;
                if (packages.Length > 1) radioButton2.Text = packages[1].name;
                if (packages.Length > 2) radioButton3.Text = packages[2].name;
                if (packages.Length > 3) radioButton4.Text = packages[3].name;
                if (packages.Length > 4) radioButton5.Text = packages[4].name;
                if (packages.Length > 5) radioButton6.Text = packages[5].name;
                if (packages.Length > 6) radioButton7.Text = packages[6].name;
            }

            AmtSecurityAdmin.AmtPowerPackage ap = system.SecurityAdmin.GetActivePowerPackage();

            if (ap != null)
            {
                radioButton1.Checked = radioButton1.Text.Equals(ap.name);
                radioButton2.Checked = radioButton2.Text.Equals(ap.name);
                radioButton3.Checked = radioButton3.Text.Equals(ap.name);
                radioButton4.Checked = radioButton4.Text.Equals(ap.name);
                radioButton5.Checked = radioButton5.Text.Equals(ap.name);
                radioButton6.Checked = radioButton6.Text.Equals(ap.name);
                radioButton7.Checked = radioButton7.Text.Equals(ap.name);
            }

            timeoutNumericUpDown.Minimum = 0;
            timeoutNumericUpDown.Maximum = uint.MaxValue;
            timeoutNumericUpDown.Value = system.SecurityAdmin.GetIdleWakeTimeout();
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void refreshButton_Click(object sender, EventArgs e)
        {
            system.SecurityAdmin.ClearPowerPolicyCache();
            FetchState();
        }

        private void applyButton_Click(object sender, EventArgs e)
        {
            AmtSecurityAdmin.AmtPowerPackage[] packages = system.SecurityAdmin.GetPowerPackages();

            int i = -1;
            if (radioButton1.Checked) i = 0;
            if (radioButton2.Checked) i = 1;
            if (radioButton3.Checked) i = 2;
            if (radioButton4.Checked) i = 3;
            if (radioButton5.Checked) i = 4;
            if (radioButton6.Checked) i = 5;
            if (radioButton7.Checked) i = 6;

            system.SecurityAdmin.SetActivePowerPackage(packages[i]);
            system.SecurityAdmin.SetIdleWakeTimeout((uint)(timeoutNumericUpDown.Value));

            system.SecurityAdmin.ClearPowerPolicyCache();
            FetchState();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            applyButton_Click(this, null);
            Close();
        }
    }
}
