// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class NvramSettingsForm : Form
    {
        private AmtSystem computer;
        private uint totalBlocks;

        public NvramSettingsForm(AmtSystem computer)
        {
            this.computer = computer;
            InitializeComponent();
        }

        private void NvramSettingsForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            GlobalStorageAttributesType attribs = computer.Storage.GetStorageAttributes();
            totalStorageLabel.Text = String.Format(ManageabilityCommanderTool.Properties.Resources.Block_Str,(attribs.TotalStorage / 1024).ToString(), (attribs.TotalStorage / 4096).ToString());
            totalBlocks = attribs.TotalStorage / 4096;

            if (attribs.TotalAllocatedStorage == 4096)
            {
                allocatedStorageLabel.Text = String.Format(ManageabilityCommanderTool.Properties.Resources.Block_Alloc_Str,(attribs.TotalAllocatedStorage / 1024).ToString() ,(attribs.TotalAllocatedStorage / 4096).ToString() );
            }
            else
            {
                allocatedStorageLabel.Text = String.Format(ManageabilityCommanderTool.Properties.Resources.Blocks_Alloc_Str,(attribs.TotalAllocatedStorage / 1024).ToString() ,(attribs.TotalAllocatedStorage / 4096).ToString() );
            }
            textBox1.Text = (attribs.MaxNonPartnerTotalAllocationSize / 4096).ToString();
            textBox2.Text = (attribs.MaxPartnerStorage / 4096).ToString();
            UpdateInfo();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            ushort maxNonPartnerTotalAllocationSize;
            if (ushort.TryParse(textBox1.Text, out maxNonPartnerTotalAllocationSize) == false) return;
            uint maxPartnerStorage;
            if (uint.TryParse(textBox2.Text, out maxPartnerStorage) == false) return;

            AmtCallStatus r = computer.Storage.SetStorageAttributes((uint)(maxPartnerStorage * 4096), (ushort)(maxNonPartnerTotalAllocationSize * 4096));
            if (r != AmtCallStatus.SUCCESS)
            {
                MessageBox.Show(this,
                    r.ToString(),
                    ManageabilityCommanderTool.Properties.Resources.NVRamSet_Error,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            else
            {
                this.DialogResult = DialogResult.OK;
            }
        }

        private void UpdateInfo()
        {
            bool ok = true;
            
            try
            {
                ushort x1 = ushort.Parse(textBox1.Text);
                if (x1 > totalBlocks) ok = false;
            }
            catch (Exception)
            {
                ok = false;
            }
            

            try
            {
                uint x2 = uint.Parse(textBox2.Text);
                if (x2 > totalBlocks) ok = false;
            }
            catch (Exception)
            {
                ok = false;
            }

            okButton.Enabled = ok;
        }

        private void textBox1_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

    }
}
