// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool.Panels
{
    public partial class AuditLogPanel : UserControl, PanelInterface
    {
        private MainForm mainform;
        private TreeView mainTreeView;
        private AmtSystem computer;
        private Thread FetchCacheThread = null;

        public AuditLogPanel()
        {
            InitializeComponent();
        }

        public void Setup(MainForm mainform, object obj1, object obj2)
        {
            this.mainform = mainform;
            this.mainTreeView = mainform.GetTreeView();
            this.computer = (AmtSystem)obj1;
        }

        public void Cleanup()
        {
            computer = null;
        }

        public void PerformAction(string command, object[] args) { }

        public void UpdatePanel()
        {
            if (computer == null) return;

            clearAuditLogButton.Enabled = (computer.AuditLog != null);
            auditPolicyButton.Enabled = (computer.AuditLog != null);

            if (computer.Info.IsAuditLogCached() == false)
            {
                auditLogListView.Items.Clear();
                auditLogListView.Groups.Clear();
                auditLogListView.Items.Add(new ListViewItem(Properties.Resources.Loading, 3));
                auditLogProgressBar.Value = 0;
                return;
            }

            auditLogListView.BeginUpdate();
            auditLogListView.Groups.Clear();
            auditLogListView.Items.Clear();

            AmtGeneralInfo.AmtAuditLogStatus auditLogStatus;
            AmtCallStatus r = computer.Info.GetAuditLogStatus(out auditLogStatus);
            AmtGeneralInfo.AmtAuditLogRecord[] records;
            r = computer.Info.ReadAuditLog(out records);

            if (auditLogStatus != null)
            {
                auditLogProgressBar.Value = (int)(100 - auditLogStatus.PercentageFree);
            }
            else
            {
                auditLogProgressBar.Value = 0;
            }

            int select = -1;
            if (auditLogListView.SelectedIndices.Count == 1)
            {
                select = auditLogListView.SelectedIndices[0];
            }

            if (records == null)
            {
                auditLogListView.EndUpdate();
            }
            else
            {
                string dateStr = "";
                ListViewGroup currentGroup = null;
                foreach (AmtGeneralInfo.AmtAuditLogRecord entry in records)
                {
                    if (entry.TimeStamp.ToLongDateString().CompareTo(dateStr) != 0)
                    {
                        dateStr = entry.TimeStamp.ToLongDateString();
                        currentGroup = new ListViewGroup(dateStr);
                        auditLogListView.Groups.Add(currentGroup);
                    }

                    ListViewItem ll = new ListViewItem(new String[] { entry.TimeStamp.ToLongTimeString() + " " + entry.Address, entry.AuditAppID.ToString() + ", " + entry.EventID.ToString() }, 0);
                    ll.Group = currentGroup;
                    ll.Tag = entry;
                    auditLogListView.Items.Add(ll);
                }
                if (select != -1) auditLogListView.SelectedIndices.Add(select);
                auditLogListView.EndUpdate();
            }

            auditLogListView_SelectedIndexChanged(this, null);
        }

        public void ForceUpdatePanel()
        {
        }

        private void fetchAuditLogEx(object param)
        {
            AmtSystem computer = (AmtSystem)param;
            AmtGeneralInfo.AmtAuditLogRecord[] records;
            computer.Info.ReadAuditLog(out records);
            AmtGeneralInfo.AmtAuditLogStatus auditLogStatus;
            computer.Info.GetAuditLogStatus(out auditLogStatus);
            mainform.FetchHandler(computer, computer.Info);
        }

        private void refreshAuditLogButton_Click(object sender, EventArgs e)
        {
            if (FetchCacheThread != null) return;
            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
            computer.Info.ClearAuditLogCache();
            mainform.UpdateTreeView();
            new Thread(new ParameterizedThreadStart(fetchAuditLogEx)).Start(computer);
        }

        private void auditPolicyButton_Click(object sender, EventArgs e)
        {
            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
            AuditLogPolicyForm form = new AuditLogPolicyForm(computer);
            form.ShowDialog(this);
        }

        private void clearAuditLogButton_Click(object sender, EventArgs e)
        {
            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
            if (MessageBox.Show(this, Properties.Resources.ClearAuditLog, Properties.Resources.AuditLogTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk) == DialogResult.Yes)
            {
                AmtCallStatus r;
                uint lockhandle;

                // Lock & clear the audit log
                r = computer.AuditLog.LockAuditLog(10, out lockhandle);
                if (r != AmtCallStatus.SUCCESS) { Utils.ErrorMessageBox(this, "LockAuditLog", r); return; }
                r = computer.AuditLog.ClearAuditLog();
                if (r != AmtCallStatus.SUCCESS) { Utils.ErrorMessageBox(this, "ClearAuditLog", r); return; }
                r = computer.AuditLog.UnLockAuditLog(lockhandle);
                if (r != AmtCallStatus.SUCCESS) { Utils.ErrorMessageBox(this, "UnLockAuditLog", r); return; }

                // Refresh the audit log
                computer.Info.ClearAuditLogCache();
                mainform.UpdateTreeView();
                new Thread(new ParameterizedThreadStart(fetchAuditLogEx)).Start(computer);
            }
        }

        private void auditLogListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (auditLogListView.SelectedItems.Count != 1 || auditLogListView.SelectedItems[0].Tag == null)
            {
                auditLogLabel1.Text = "";
                auditLogLabel2.Text = "";
                auditLogLabel3.Text = "";
                auditLogLabel4.Text = "";
                return;
            }

            AmtGeneralInfo.AmtAuditLogRecord entry = (AmtGeneralInfo.AmtAuditLogRecord)auditLogListView.SelectedItems[0].Tag;
            auditLogLabel1.Text = entry.AuditAppID.ToString() + ", " + entry.EventID.ToString();

            string inititiator = "";
            if (entry.Initiator == AmtGeneralInfo.AmtAuditlogInitiator.HttpDigest) inititiator = entry.Initiator.ToString() + ", " + entry.HttpDigestUsername;
            else if (entry.Initiator == AmtGeneralInfo.AmtAuditlogInitiator.Kerberos) inititiator = entry.Initiator.ToString() + ", " + entry.KerberosDomain + "/" + entry.KerberosUserInDomain;
            else if (entry.Initiator == AmtGeneralInfo.AmtAuditlogInitiator.Local) inititiator = entry.Initiator.ToString();

            auditLogLabel2.Text = inititiator;
            auditLogLabel3.Text = entry.Address.ToString();
            if (entry.ExtendedData != null && entry.ExtendedData.Length > 0) auditLogLabel4.Text = "0x" + Utils.BytesToHex(entry.ExtendedData); else auditLogLabel4.Text = "";
        }
    }
}
