// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Net;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool.Panels
{
    public partial class DiscoveryPanel : UserControl, PanelInterface
    {
        private TreeView mainTreeView;
        private MainForm mainform;
        private AmtDiscovery discovery = new AmtDiscovery();
        private IPAddress a1 = null;
        private IPAddress a2 = null;

        public DiscoveryPanel()
        {
            InitializeComponent();
        }

        public void Setup(MainForm mainform, object obj1, object obj2)
        {
            this.mainform = mainform;
            mainTreeView = mainform.GetTreeView();

            discovery.OnDiscoveryEvent += new AmtDiscovery.AmtDiscoveryEventHandler(AmtDiscoveryEventSink);

            // Load State
            discoveryStartAddrTextBox.Text = (string)Application.UserAppDataRegistry.GetValue("DiscoveryStartAddr", "192.168.1.1");
            discoveryEndAddrTextBox.Text = (string)Application.UserAppDataRegistry.GetValue("DiscoveryEndAddr", "192.168.1.254");
        }

        public void Cleanup()
        {
            // Save State
            Application.UserAppDataRegistry.SetValue("DiscoveryStartAddr", discoveryStartAddrTextBox.Text);
            Application.UserAppDataRegistry.SetValue("DiscoveryEndAddr", discoveryEndAddrTextBox.Text);

            discovery.Stop();
        }

        public void PerformAction(string command, object[] args) { }

        public void UpdatePanel()
        {
            if (discovery.IsRunning == true)
            {
                startDiscoveryButton.Text = ManageabilityCommanderTool.Properties.Resources.MainForm_DiscoveryBtn_Stop;
            }
            else
            {
                startDiscoveryButton.Text = ManageabilityCommanderTool.Properties.Resources.MainForm_DiscoveryBtn_Start;
            }
        }

        public void ForceUpdatePanel()
        {
        }

        private void startDiscoveryButton_Click(object sender, EventArgs e)
        {
            if (discovery.IsRunning == false)
            {
                if (updateDiscoveryInfo() == false) return;
                if (IPAddress.TryParse(discoveryStartAddrTextBox.Text, out a1) == false) return;
                if (IPAddress.TryParse(discoveryEndAddrTextBox.Text, out a2) == false) return;

                discoveryProgressBar.Maximum = AmtDiscovery.PingCount;
                discoveryProgressBar.Value = 0;
                lock (discoveryListView)
                {
                    discoveryListView.Items.Clear();
                }

                discovery.Start(a1, a2);
                startDiscoveryButton.Text = ManageabilityCommanderTool.Properties.Resources.MainForm_DiscoveryBtn_Stop;
            }
            else
            {
                discovery.Stop();
                discoveryProgressBar.Visible = false;
                startDiscoveryButton.Text = ManageabilityCommanderTool.Properties.Resources.MainForm_DiscoveryBtn_Start;
            }
        }

        private void manualAddButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null || mainTreeView.SelectedNode == mainform.rootNode)
            {
                // Add a computer to Commander
                ManageabilityControls.AddSystemForm f = new ManageabilityControls.AddSystemForm(false);
                if (f.ShowDialog(this) == DialogResult.OK)
                {
                    AmtSystem computer = new AmtSystem(f.Hostname, 16992, f.Username, f.Password, f.RememberPassword, false, f.ForceIPOption, f.IPAddress);
                    mainform.AddComputer(computer);
                    if (mainform.rootNode != null) mainform.rootNode.Expand();
                    mainform.SaveSettings();
                }
            }
            else if (mainTreeView.SelectedNode != null && mainTreeView.SelectedNode.Tag != null && (mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtMpsServer)))
            {
                // Add a computer to MPS
                AmtMpsServer mps = (AmtMpsServer)mainTreeView.SelectedNode.Tag;
                ManageabilityControls.AddSystemForm f = new ManageabilityControls.AddSystemForm(true);
                if (f.ShowDialog(this) == DialogResult.OK)
                {
                    AmtSystem computer = new AmtSystem(f.Hostname, 16992, f.Username, f.Password, f.RememberPassword, false, f.ForceIPOption, f.IPAddress);
                    mps.AddComputer(computer);
                    mainform.AddComputer(computer, mainTreeView.SelectedNode);
                    mainform.SaveSettings();
                }
            }
        }

        private void addDiscoveredButton_Click(object sender, EventArgs e)
        {
            if (discoveryListView.SelectedItems.Count != 1) return;
            ListViewItem l = discoveryListView.SelectedItems[0];
            if (l.Tag == null) return;
            AmtDiscovery.AmtDiscoveredComputer discComputer = (AmtDiscovery.AmtDiscoveredComputer)l.Tag;
            if (l.ImageIndex == 16 || l.ImageIndex == 19)
            {
                // Add a Intel AMT Computer
                string hostname = discComputer.EndPoint.Address.ToString();
                if (resolveNamesToolStripMenuItem.Checked) hostname = discComputer.Hostname;
                ManageabilityControls.AddSystemForm f = new ManageabilityControls.AddSystemForm(hostname);
                // if (discComputer.Entry != null) f.DnsHostEntry = discComputer.Entry;
                if (f.ShowDialog(this) == DialogResult.OK)
                {
                    AmtSystem computer = new AmtSystem(f.Hostname, 16992, f.Username, f.Password, f.RememberPassword, false, f.ForceIPOption, f.IPAddress);
                    computer.UseTls = (discComputer.State == AmtDiscovery.ProvisionState.TLS);
                    computer.AmtRev = discComputer.AmtRev;
                    computer.AmtRevMin = discComputer.AmtRevMin;
                    // if (f.DnsHostEntry != null) computer.SetDnsEntryCache(f.DnsHostEntry);
                    mainform.AddComputer(computer);
                    if (mainform.rootNode != null) mainform.rootNode.Expand();
                    mainform.SaveSettings();
                }
            }
 
        }

        private void discoveryStartAddrTextBox_TextChanged(object sender, EventArgs e)
        {
            updateDiscoveryInfo();
        }

        private void discoveryEndAddrTextBox_TextChanged(object sender, EventArgs e)
        {
            updateDiscoveryInfo();
        }

        private bool updateDiscoveryInfo()
        {
            Utils.FilterIpAddress(discoveryStartAddrTextBox);
            Utils.FilterIpAddress(discoveryEndAddrTextBox);

            IPAddress x1 = null;
            IPAddress x2 = null;
            try
            {
                x1 = IPAddress.Parse(discoveryStartAddrTextBox.Text);
                x2 = IPAddress.Parse(discoveryEndAddrTextBox.Text);
            }
            catch (Exception)
            {
                startDiscoveryButton.Enabled = false;
                return false;
            }

            long diff = AmtDiscovery.GetAddressDiff(x1, x2);
            if (diff < 0 || diff > 65536)
            {
                startDiscoveryButton.Enabled = false;
                return false;
            }

            startDiscoveryButton.Enabled = true;
            return true;
        }

        private void resolveNamesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            resolveNamesToolStripMenuItem.Checked = !resolveNamesToolStripMenuItem.Checked;

            lock (discoveryListView)
            {
                foreach (ListViewItem l in discoveryListView.Items)
                {
                    AmtDiscovery.AmtDiscoveredComputer discComputer = (AmtDiscovery.AmtDiscoveredComputer)l.Tag;
                    string hostname = discComputer.EndPoint.Address.ToString();
                    if (resolveNamesToolStripMenuItem.Checked) hostname = discComputer.Hostname;
                    l.SubItems[0].Text = hostname + " - " + discComputer.Info;
                }
            }
        }

        private void discoveryListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            addComputerToolStripMenuItem.Enabled = addDiscoveredButton.Enabled = ((discoveryListView.SelectedItems.Count == 1) && (discoveryListView.SelectedItems[0].Tag != null));
        }

        private void AmtDiscoveryEventSink(AmtDiscovery amtDiscovery, IPEndPoint ep, AmtDiscovery.AmtDiscoveryEvent e, AmtDiscovery.AmtDiscoveredComputer computer)
        {
            if (mainform.disposing == true) return;
            object[] p = new object[4];
            p[0] = amtDiscovery;
            p[1] = ep;
            p[2] = e;
            p[3] = computer;
            this.Invoke(new AmtDiscovery.AmtDiscoveryEventHandler(AmtDiscoveryEventSinkEx), p);
        }

        private void AmtDiscoveryEventSinkEx(AmtDiscovery amtDiscovery, IPEndPoint ep, AmtDiscovery.AmtDiscoveryEvent e, AmtDiscovery.AmtDiscoveredComputer computer)
        {
            ListViewItem l;
            if (e == AmtDiscovery.AmtDiscoveryEvent.DiscoveryFinished)
            {
                startDiscoveryButton.Text = ManageabilityCommanderTool.Properties.Resources.MainForm_DiscoveryBtn_Start;
                discoveryProgressBar.Visible = false;
            }
            else if (e == AmtDiscovery.AmtDiscoveryEvent.LaunchingDiscovery)
            {
                discoveryProgressBar.Visible = true;
            }
            else if (e == AmtDiscovery.AmtDiscoveryEvent.MachineFound ||
                e == AmtDiscovery.AmtDiscoveryEvent.RmcpPong)
            {
                int icon = 16;
                if (computer.IsMobile) icon += 3;
                string hostname = computer.EndPoint.Address.ToString();
                if (resolveNamesToolStripMenuItem.Checked) hostname = computer.Hostname;
                l = new ListViewItem(new string[] { hostname + " - " + computer.Info, computer.Method.ToString(), computer.State.ToString(), computer.State.ToString() }, icon);
                l.Tag = computer;
                lock (discoveryListView)
                {
                    discoveryListView.Items.Add(l);
                }
                if (e == AmtDiscovery.AmtDiscoveryEvent.MachineFound)
                {
                    discoveryProgressBar.Increment(1);
                }
            }
            else if (e == AmtDiscovery.AmtDiscoveryEvent.MachineNotFound)
            {
                discoveryProgressBar.Increment(1);
            }
            else if (e == AmtDiscovery.AmtDiscoveryEvent.RmcpPing)
            {
                discoveryProgressBar.Increment(1);
            }
            else if (e == AmtDiscovery.AmtDiscoveryEvent.StartingTcpSweep)
            {
                discoveryProgressBar.Value = 0;
                discoveryProgressBar.Maximum = (int)AmtDiscovery.GetAddressDiff(a1, a2);
            }
        }
    }
}
