// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Diagnostics;
using ManageabilityStack;

namespace ManageabilityCommanderTool.Panels
{
    public partial class EventPanel : UserControl, PanelInterface
    {
        private MainForm mainform;
        private TreeView mainTreeView;
        private AmtSystem computer;
        private Thread FetchCacheThread = null;

        public EventPanel()
        {
            InitializeComponent();
        }

        public void Setup(MainForm mainform, object obj1, object obj2)
        {
            this.mainform = mainform;
            this.mainTreeView = mainform.GetTreeView();
            this.computer = (AmtSystem)obj1;
        }

        public void Cleanup()
        {
            computer = null;
        }

        public void PerformAction(string command, object[] args) { }

        public void UpdatePanel()
        {
            if (computer == null) return;

            systemSensorsButton.Enabled = !computer.Events.LocalAccessOnly;
            eventFilterButton.Enabled = !computer.Events.LocalAccessOnly;
            alertSubscriptionButton.Enabled = !computer.Events.LocalAccessOnly;
            clearEventLogButton.Enabled = !computer.Events.LocalAccessOnly;

            #region Update the Alert Log
            if (mainform.Simplified == false)
            {
                AmtAlert[] alerts = computer.Events.GetAlerts();
                networkAlertListView.BeginUpdate();
                networkAlertListView.Groups.Clear();
                networkAlertListView.Items.Clear();

                int select = -1;
                if (networkAlertListView.SelectedIndices.Count == 1)
                {
                    select = networkAlertListView.SelectedIndices[0];
                }

                if (alerts == null)
                {
                    networkAlertListView.EndUpdate();
                }
                else
                {
                    string dateStr = "";
                    ListViewGroup CurrentGroup = null;
                    foreach (AmtAlert alert in alerts)
                    {
                        AmtEvent entry = alert.amtEvent;
                        if (entry.TimeStamp.ToLongDateString().CompareTo(dateStr) != 0)
                        {
                            CurrentGroup = new ListViewGroup(entry.TimeStamp.ToLongDateString());
                            dateStr = entry.TimeStamp.ToLongDateString();
                            ListViewItem l = new ListViewItem(new String[] { dateStr, "" }, 3);
                            //networkAlertListView.Items.Add(l);
                            networkAlertListView.Groups.Add(CurrentGroup);
                        }
                        int severity = 1;
                        if (entry.EventSeverity < 5) severity = 0;
                        if (entry.EventSeverity > 10) severity = 2;
                        ListViewItem ll = new ListViewItem(
                            new String[] { entry.TimeStamp.ToLongTimeString() + " " + entry.EntityTypeStr,
                                           entry.EventDetailStr },
                            severity);
                        ll.Tag = entry;
                        ll.Group = CurrentGroup;
                        networkAlertListView.Items.Add(ll);
                    }
                    if (select != -1) networkAlertListView.SelectedIndices.Add(select);
                    networkAlertListView.EndUpdate();
                }
            }
            #endregion

            #region Update the Event Log
            if (computer.Events.IsCached() == false)
            {
                eventLogListView.Items.Clear();
                eventLogListView.Groups.Clear();
                eventLogListView.Items.Add(new ListViewItem(Properties.Resources.Loading, 3));
            }
            else
            {
                AmtEvent[] log = computer.Events.GetEventLog();

                eventLogListView.BeginUpdate();
                eventLogListView.Groups.Clear();
                eventLogListView.Items.Clear();

                int select = -1;
                if (eventLogListView.SelectedIndices.Count == 1)
                {
                    select = eventLogListView.SelectedIndices[0];
                }

                if (log == null)
                {
                    eventLogListView.EndUpdate();
                }
                else
                {
                    string dateStr = "";
                    ListViewGroup currentGroup = null;
                    foreach (AmtEvent entry in log)
                    {
                        if (entry.TimeStamp.ToLongDateString().CompareTo(dateStr) != 0)
                        {
                            dateStr = entry.TimeStamp.ToLongDateString();
                            currentGroup = new ListViewGroup(dateStr);
                            eventLogListView.Groups.Add(currentGroup);
                        }
                        int severity = 1;
                        if (entry.EventSeverity < 5) severity = 0;
                        if (entry.EventSeverity > 10) severity = 2;
                        ListViewItem l = new ListViewItem(new String[] { entry.TimeStamp.ToLongTimeString() + " " + entry.EntityTypeStr, entry.EventDetailStr }, severity);
                        l.Group = currentGroup;
                        l.Tag = entry;
                        eventLogListView.Items.Add(l);
                    }
                    if (select != -1) eventLogListView.SelectedIndices.Add(select);
                    eventLogListView.EndUpdate();
                }
            }
            #endregion
        }

        public void ForceUpdatePanel()
        {
            refreshLogButton_Click(this, null);
        }

        private void refreshLogButton_Click(object sender, EventArgs e)
        {
            if (FetchCacheThread != null) return;
            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
            computer.Events.ClearCache();
            mainform.UpdateTreeView();
            new Thread(new ParameterizedThreadStart(fetchEventsEx)).Start(computer);
        }

        private void fetchEventsEx(object param)
        {
            AmtSystem computer = (AmtSystem)param;
            computer.Events.FetchCache();
            mainform.FetchHandler(computer, computer.Events);
        }

        private void systemSensorsButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null || mainTreeView.SelectedNode.Parent.Tag == null)
                return;
            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
            new SensorsForm(computer).ShowDialog(this);
        }

        private void eventFilterButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null || mainTreeView.SelectedNode.Parent.Tag == null) return;
            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
            new EventFilterForm(computer).ShowDialog(this);
        }

        private void alertSubscriptionButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null || mainTreeView.SelectedNode.Parent.Tag == null) return;
            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
            new AlertSubscriptionForm(computer).ShowDialog(this);
        }

        private void clearEventLogButton_Click(object sender, EventArgs e)
        {
            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
            if (eventTabControl.SelectedIndex == 0)
            {
                if (MessageBox.Show(this, Properties.Resources.ClearEventLog, Properties.Resources.EventLogTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk) == DialogResult.Yes)
                {
                    computer.Events.ClearLog();
                    mainform.UpdateTreeView();
                    new Thread(new ParameterizedThreadStart(fetchEventsEx)).Start(computer);
                }
            }
            else
            {
                if (MessageBox.Show(this, Properties.Resources.ClearAlertLog, Properties.Resources.AlertLogTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk) == DialogResult.Yes)
                {
                    computer.Events.ClearAlerts();
                    networkAlertListView.Items.Clear();
                }
            }
        }

        private void copyToClipboardToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ListView listView;
            if (eventTabControl.SelectedIndex == 0) { listView = eventLogListView; } else { listView = networkAlertListView; }
            if (listView.SelectedItems.Count != 1 || listView.SelectedItems[0].Tag == null) return;

            StringBuilder s = new StringBuilder();
            ListViewItem item = listView.SelectedItems[0];
            if (item.Tag.GetType() == typeof(AmtEvent))
            {
                AmtEvent evt = (AmtEvent)item.Tag;
                s.Append(String.Format(ManageabilityCommanderTool.Properties.Resources.MainForm_copyTo_0_Format, evt.TimeStamp.ToString()));
                s.Append(String.Format(ManageabilityCommanderTool.Properties.Resources.MainForm_copyTo_1_Format, evt.EventSensorTypeStr));
                s.Append(String.Format(ManageabilityCommanderTool.Properties.Resources.MainForm_copyTo_2_Format, evt.EventOffsetStr));
                s.Append(String.Format(ManageabilityCommanderTool.Properties.Resources.MainForm_copyTo_3_Format, evt.EventTypesStr));
                s.Append(String.Format(ManageabilityCommanderTool.Properties.Resources.MainForm_copyTo_4_Format, evt.EventSourceStr));
                s.Append(String.Format(ManageabilityCommanderTool.Properties.Resources.MainForm_copyTo_5_Format, evt.EventSeverityStr));
                s.Append(String.Format(ManageabilityCommanderTool.Properties.Resources.MainForm_copyTo_6_Format, evt.SensorDevice.ToString()));
                s.Append(String.Format(ManageabilityCommanderTool.Properties.Resources.MainForm_copyTo_7_Format, evt.EntityTypeStr));
                Clipboard.SetDataObject(s.ToString());
            }
        }

        private void eventLogListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            ListView listView;
            if (eventTabControl.SelectedIndex == 0)
            {
                listView = eventLogListView;
            }
            else
            {
                listView = networkAlertListView;
            }

            if (listView.SelectedItems.Count != 1 || listView.SelectedItems[0].Tag == null)
            {
                sensorTypeLabel.Text = "";
                eventTypeLabel.Text = "";
                eventSourceLabel.Text = "";
                eventSeverityLabel.Text = "";
                sensorDeviceLabel.Text = "";
                entityLabel.Text = "";
                eventDataLabel.Text = "";

                copyToClipboardToolStripMenuItem.Visible = false;
                return;
            }

            ListViewItem item = listView.SelectedItems[0];
            if (item.Tag.GetType() == typeof(AmtEvent))
            {
                AmtEvent evt = (AmtEvent)item.Tag;
                sensorTypeLabel.Text = evt.EventSensorTypeStr;
                Debug.Print("AMT Event selected");

                string offset = evt.EventOffsetStr;
                if (offset != null && offset.Length > 0) offset = ", " + offset;

                eventTypeLabel.Text = evt.EventTypesStr + offset;
                eventSourceLabel.Text = evt.EventSourceStr;
                eventSeverityLabel.Text = evt.EventSeverityStr;
                sensorDeviceLabel.Text = ManageabilityCommanderTool.Properties.Resources.Sensor_Label + evt.SensorDevice.ToString(); ;
                entityLabel.Text = evt.EntityTypeStr;
                eventDataLabel.Text = evt.EventDataStr;

                copyToClipboardToolStripMenuItem.Visible = true;
                return;
            }
            else if (item.Tag.GetType() == typeof(AmtAlert))
            {
                AmtAlert alert = (AmtAlert)item.Tag;
                Debug.Print("AMT Alert selected");
            }

            copyToClipboardToolStripMenuItem.Visible = false;
        }

        private void eventTabControl_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (eventTabControl.SelectedIndex == 0) clearEventLogButton.Text = Properties.Resources.ClearEventLogButton;
            else clearEventLogButton.Text = Properties.Resources.ClearAlertLogButton;
        }
    }
}
