// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Data;
using System.Text;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool.Panels
{
    public partial class HardwareAssetPanel : UserControl, PanelInterface
    {
        private MainForm mainform;
        private TreeView mainTreeView;
        private AmtSystem computer = null;

        public HardwareAssetPanel()
        {
            InitializeComponent();           
        }

        public void Setup(MainForm mainform, object obj1, object obj2)
        {
            this.mainform = mainform;
            this.mainTreeView = mainform.GetTreeView();
            this.computer = (AmtSystem)obj1;

            printAssetSummaryButton.Visible = !mainform.Simplified;
            saveAssetSummaryButton.Visible = !mainform.Simplified;
            refreshHardwareButton.Visible = !mainform.Simplified;
        }

        public void Cleanup()
        {
            computer = null;
        }

        public void PerformAction(string command, object[] args) { }

        public void UpdatePanel()
        {
        }

        public void ForceUpdatePanel()
        {
            refreshHardwareButton_Click(this, null);
        }

        private void refreshHardwareButton_Click(object sender, EventArgs e)
        {
            if (computer == null) return;

            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag == null && mainTreeView.SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Hardware_Asset) == 0)
            {
                computer.Assets.ClearCache();
                mainform.UpdateTreeAssets(computer);
            }
        }

        private void printAssetSummaryButton_Click(object sender, EventArgs e)
        {
            // The the computer node
            if (mainTreeView.SelectedNode == null || mainTreeView.SelectedNode.Parent == null) return;
            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
            if (computer == null) return;
            PrintExportClass.SaveHardwareAssetToFile(computer, "HardwareAsset.htm", new FileInfo(System.Reflection.Assembly.GetEntryAssembly().Location).Directory.ToString());

            System.Diagnostics.ProcessStartInfo info = new System.Diagnostics.ProcessStartInfo("HardwareAsset.htm");
            info.Verb = "print";
            System.Diagnostics.Process processChild = System.Diagnostics.Process.Start(info);
        }

        private void saveAssetSummaryButton_Click(object sender, EventArgs e)
        {
            // The the computer node
            if (mainTreeView.SelectedNode == null || mainTreeView.SelectedNode.Parent == null) return;
            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
            if (computer == null) return;

            // Get the save filename
            saveFileDialog.Title = ManageabilityCommanderTool.Properties.Resources.MainForm_Save_Asset;
            saveFileDialog.FileName = computer.HostNameIdentity.DefaultName + ".htm";
            saveFileDialog.DefaultExt = ".htm";
            saveFileDialog.Filter = ManageabilityCommanderTool.Properties.Resources.MainForm_FD_SaveML;
            if (saveFileDialog.ShowDialog(this) != DialogResult.OK) return;

            // Save the asset to file
            PrintExportClass.SaveHardwareAssetToFile(computer, saveFileDialog.FileName, new FileInfo(System.Reflection.Assembly.GetEntryAssembly().Location).Directory.ToString());
        }
    }
}
