// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool.Panels
{
    public partial class HeuristicPanel : UserControl, PanelInterface
    {
        private MainForm mainform;
        private TreeView mainTreeView;
        private AmtSystem computer = null;

        public HeuristicPanel()
        {
            InitializeComponent();           
        }

        public void Setup(MainForm mainform, object obj1, object obj2)
        {
            this.mainform = mainform;
            this.mainTreeView = mainform.GetTreeView();
            this.computer = (AmtSystem)obj1;
        }

        public void Cleanup()
        {
            computer = null;
        }

        public void PerformAction(string command, object[] args) { }

        public void UpdatePanel()
        {
            if (computer == null) return;

            CircuitBreakerFilterInfoType[] cbs = computer.CircuitBreaker.GetCircuitBreakerFilters();
            heuristicInterfaceComboBox.SelectedIndex = 0;
            string s;
            HcbOptionsType options = computer.CircuitBreaker.GetHcbOptions(0);
            string Label_NOTSET = ManageabilityCommanderTool.Properties.Resources.Label_NOTSET;
            if (options == null)
            {
                heuristicSlowLabel.Text = Label_NOTSET;
                heuristicFastLabel.Text = Label_NOTSET;
                heuristicActionLabel.Text = ManageabilityCommanderTool.Properties.Resources.Label_NOACTION;
                heuristicEncounterResolutionLabel.Text = Label_NOTSET;
                heuristicPictureBox.Image = Properties.Resources.ShieldGray;
            }
            else
            {
                if (options.SlowConnectionRate != null)
                {
                    heuristicSlowLabel.Text = string.Format(ManageabilityCommanderTool.Properties.Resources.Evt_In_Sec, options.SlowConnectionRate.Threshold, (options.SlowConnectionRate.ClearTime / 1000));
                }
                else
                {
                    heuristicSlowLabel.Text = Label_NOTSET;
                }
                if (options.FastConnectionRate != null)
                {
                    heuristicFastLabel.Text = string.Format(ManageabilityCommanderTool.Properties.Resources.Evt_In_MSec, options.FastConnectionRate.Threshold, options.FastConnectionRate.ClearTime);
                }
                else
                {
                    heuristicFastLabel.Text = Label_NOTSET;
                }
                s = "";
                if (options.BlockAll) s += ManageabilityCommanderTool.Properties.Resources.Opt_Block_All;
                if (options.BlockOffensivePort) s += ManageabilityCommanderTool.Properties.Resources.Opt_Block_Offensive;

                if (options.PolicyCreationHandleSpecified)
                {
                    CircuitBreakerPolicyInfoType policy = computer.CircuitBreaker.GetCircuitBreakerPolicy(options.PolicyCreationHandle);
                    if (policy != null)
                    {
                        s += string.Format(ManageabilityCommanderTool.Properties.Resources.Act_Policy_Name, policy.Policy.PolicyName);
                    }
                    else
                    {
                        s += ManageabilityCommanderTool.Properties.Resources.Act_Policy_Num + options.PolicyCreationHandle.ToString();
                    }
                }

                if (s.Length > 2) s = s.Substring(2); else s = ManageabilityCommanderTool.Properties.Resources.Label_NOACTION;
                heuristicActionLabel.Text = s;



                if (options.EncounterTimeout == 0)
                {
                    heuristicEncounterResolutionLabel.Text = ManageabilityCommanderTool.Properties.Resources.Heu_Res_Stop;
                }
                else
                {
                    heuristicEncounterResolutionLabel.Text = string.Format(ManageabilityCommanderTool.Properties.Resources.Heu_Res_Clear, options.EncounterTimeout);
                }
            }

            AmtCircuitBreaker.AmtHcbState state = computer.CircuitBreaker.GetHcbState(0);

            if (state.HcbState == 0) heuristicMonitoringLabel.Text = ManageabilityCommanderTool.Properties.Resources.Heu_Mon_Run;
            if (state.HcbState == 1) heuristicMonitoringLabel.Text = ManageabilityCommanderTool.Properties.Resources.Heu_Mon_Timed;
            if (state.HcbState == 2) heuristicMonitoringLabel.Text = ManageabilityCommanderTool.Properties.Resources.Heu_Mon_Perm;
            if (state.HcbState == 3) heuristicMonitoringLabel.Text = ManageabilityCommanderTool.Properties.Resources.Heu_Mon_Disable;

            if (state.HcbTriggerReasonSpecified == true)
            {
                if (state.HcbTriggerReason == 0) heuristicTriggerReasonLabel.Text = ManageabilityCommanderTool.Properties.Resources.Heu_Tri_SlowWorm;
                if (state.HcbTriggerReason == 1) heuristicTriggerReasonLabel.Text = ManageabilityCommanderTool.Properties.Resources.Heu_Tri_FastWorm;
                if (state.HcbTriggerReason == 2) heuristicTriggerReasonLabel.Text = ManageabilityCommanderTool.Properties.Resources.Heu_Tri_DOS;
            }
            else
            {
                heuristicTriggerReasonLabel.Text = ManageabilityCommanderTool.Properties.Resources.Heu_Not_Specify;
            }

            if (state.TimeStampSpecified == true)
            {
                heuristicTimeStampLabel.Text = Utils.ClockToDate(state.TimeStamp).ToString();
            }
            else
            {
                heuristicTimeStampLabel.Text = ManageabilityCommanderTool.Properties.Resources.Heu_Not_Specify;
            }

            s = "";
            if (state.BlockedAll) s += ManageabilityCommanderTool.Properties.Resources.Block_Info_All;
            if (state.BlockedPortInfo != null)
            {
                if (state.BlockedPortInfo.PortBlocked != 0) s += ManageabilityCommanderTool.Properties.Resources.Block_Info_Port + state.BlockedPortInfo.PortBlocked.ToString();
                if (state.BlockedPortInfo.ProtocolBlocked != 0) s += ManageabilityCommanderTool.Properties.Resources.Block_Info_Proto + state.BlockedPortInfo.ProtocolBlocked.ToString();
            }
            if (s.Length > 2) s = ManageabilityCommanderTool.Properties.Resources.Block_Info + s.Substring(2); else s = "";
            if (state.EnabledCbPolicySpecified)
            {
                if (s.Length > 0) s += ", ";

                CircuitBreakerPolicyInfoType policy = computer.CircuitBreaker.GetCircuitBreakerPolicy(state.EnabledCbPolicy);
                if (policy != null)
                {
                    s += ManageabilityCommanderTool.Properties.Resources.Act_Info + "\"" + policy.Policy.PolicyName + "\"";
                }
                else
                {
                    s += ManageabilityCommanderTool.Properties.Resources.Act_Info + state.EnabledCbPolicy.ToString();
                }
            }
            if (s.Length == 0) s = ManageabilityCommanderTool.Properties.Resources.Label_NOACTION;
            heuristicActionTakenLabel.Text = s;

            // Set image
            if (state.HcbState == 0) heuristicPictureBox.Image = Properties.Resources.ShieldGreen; // Running
            else if (state.HcbState == 1) heuristicPictureBox.Image = Properties.Resources.ShieldYellow; // Timed Encounter
            else if (state.HcbState == 2) heuristicPictureBox.Image = Properties.Resources.ShieldRed; // Permanent Encounter
            else if (state.HcbState == 3) heuristicPictureBox.Image = Properties.Resources.ShieldGray; // Disabled
        }

        public void ForceUpdatePanel()
        {
            heuristicRefreshButton_Click(this, null);
        }

        private void heuristicRefreshButton_Click(object sender, System.EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag == null && mainTreeView.SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Heuristics) == 0)
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Tag;
                computer.CircuitBreaker.ClearHcbCache();
                mainform.UpdateTreeView();
            }
        }

        private void clearHeuristicButton_Click(object sender, System.EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag == null && mainTreeView.SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Heuristics) == 0)
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Tag;
                if (MessageBox.Show(this,
                    ManageabilityCommanderTool.Properties.Resources.MainForm_MsgBox_Heu_0,
                    ManageabilityCommanderTool.Properties.Resources.MainForm_MsgBox_Heu_1,
                    MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    AmtCallStatus r = computer.CircuitBreaker.ClearHcbState(0);
                    if (r != AmtCallStatus.SUCCESS) MessageBox.Show(this,
                        String.Format(ManageabilityCommanderTool.Properties.Resources.MainForm_MsgBox_Heu_2, r.ToString()),
                        ManageabilityCommanderTool.Properties.Resources.MainForm_MsgBox_Heu_3,
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                    mainform.UpdateTreeView();
                }
            }
        }

        private void setHeuristicButton_Click(object sender, System.EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag == null && mainTreeView.SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Heuristics) == 0)
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Tag;
                EditHeuristicForm f = new EditHeuristicForm(computer);
                f.HcbOptions = computer.CircuitBreaker.GetHcbOptions(0);
                if (f.ShowDialog(this) == DialogResult.OK)
                {
                    AmtCallStatus r = computer.CircuitBreaker.SetHcbOptions(0, f.HcbOptions);
                    if (r != AmtCallStatus.SUCCESS) MessageBox.Show(this,
                        String.Format(ManageabilityCommanderTool.Properties.Resources.MainForm_MsgBox_Heu_4, r.ToString()),
                        ManageabilityCommanderTool.Properties.Resources.MainForm_MsgBox_Heu_3,
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                    mainform.UpdateTreeView();
                }
            }
        }
    }
}
