// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Data;
using System.Text;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool.Panels
{
    public partial class MpsPanel : UserControl, PanelInterface
    {
        private MainForm mainform;
        private TreeView mainTreeView;
        private AmtMpsServer server = null;

        public MpsPanel()
        {
            InitializeComponent();           
        }

        public void Setup(MainForm mainform, object obj1, object obj2)
        {
            this.mainform = mainform;
            this.mainTreeView = mainform.GetTreeView();
            this.server = (AmtMpsServer)obj1;
        }

        public void Cleanup()
        {
            server = null;
        }

        public void PerformAction(string command, object[] args) { }

        public void UpdatePanel()
        {
            hostnameLabel.Text = server.NetworkNameIdentity.DisplayName;
            httpPortLabel.Text = server.HttpPort.ToString();
            socksPortLabel.Text = server.SocksPort.ToString();
        }

        public void ForceUpdatePanel()
        {
            hostnameLabel.Text = server.NetworkNameIdentity.DisplayName;
            httpPortLabel.Text = server.HttpPort.ToString();
            socksPortLabel.Text = server.SocksPort.ToString();
        }

        private void addComputerButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null || mainTreeView.SelectedNode == mainform.rootNode)
            {
                // Add a computer to Commander
                ManageabilityControls.AddSystemForm f = new ManageabilityControls.AddSystemForm(false);
                if (f.ShowDialog(this) == DialogResult.OK)
                {
                    AmtSystem computer = new AmtSystem(f.Hostname, 16992, f.Username, f.Password, f.RememberPassword, false, f.ForceIPOption, f.IPAddress);
                    mainform.AddComputer(computer);
                    if (mainform.rootNode != null) mainform.rootNode.Expand();
                    mainform.SaveSettings();
                }
            }
            else if (mainTreeView.SelectedNode != null && mainTreeView.SelectedNode.Tag != null && (mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtMpsServer)))
            {
                // Add a computer to MPS
                AmtMpsServer mps = (AmtMpsServer)mainTreeView.SelectedNode.Tag;
                ManageabilityControls.AddSystemForm f = new ManageabilityControls.AddSystemForm(true);
                if (f.ShowDialog(this) == DialogResult.OK)
                {
                    AmtSystem computer = new AmtSystem(f.Hostname, 16992, f.Username, f.Password, f.RememberPassword, false, f.ForceIPOption, f.IPAddress);
                    mps.AddComputer(computer);
                    mainform.AddComputer(computer, mainTreeView.SelectedNode);
                    mainTreeView.SelectedNode.Expand();
                    mainform.SaveSettings();
                }
            }
        }

        private void hardwarePanel_Paint(object sender, PaintEventArgs e)
        {

        }

    }
}
