// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System.Data;
using System.Text;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool.Panels
{
    public partial class NetworkFiltersPanel : UserControl, PanelInterface
    {
        private MainForm mainform;
        private TreeView mainTreeView;
        private AmtSystem computer = null;
        private CircuitBreakerFilterInfoType[] cbs;

        public NetworkFiltersPanel()
        {
            InitializeComponent();            
        }

        public void Setup(MainForm mainform, object obj1, object obj2)
        {
            this.mainform = mainform;
            this.mainTreeView = mainform.GetTreeView();
            this.computer = (AmtSystem)obj1;
            this.cbs = (CircuitBreakerFilterInfoType[])obj2;
        }

        public void Cleanup()
        {
            computer = null;
            cbs = null;
        }

        public void PerformAction(string command, object[] args)
        {
            if (command.Equals("ADDFILTER")) createNewFilterButton_Click(this, null);
        }

        public void UpdatePanel()
        {
            if (computer == null) return;

            uint MaxSupportedPolicies;
            uint MaxSupportedFilters;
            CircuitBreakerCapabilitiesType[] cbcs = computer.CircuitBreaker.GetCircuitBreakerCapabilities(out MaxSupportedPolicies, out MaxSupportedFilters);

            filtersCountLabel.Text = cbs.Length.ToString() + Properties.Resources.SlashSeperator + MaxSupportedFilters.ToString();

            filtersInfoListView.Items.Clear();
            int i = 0;
            foreach (CircuitBreakerCapabilitiesType cbc in cbcs)
            {
                i++;
                filtersInfoListView.Items.Add(new ListViewItem(new string[] { cbc.HardwareDescription, "" }));

                string s = ManageabilityCommanderTool.Properties.Resources.LV_AntiSpoof_None;
                if (cbc.IPv4_AntiSpoofingCapable && cbc.IPv6_AntiSpoofingCapable) s = ManageabilityCommanderTool.Properties.Resources.LV_AntiSpoof_Both;
                if (!cbc.IPv4_AntiSpoofingCapable && cbc.IPv6_AntiSpoofingCapable) s = ManageabilityCommanderTool.Properties.Resources.LV_AntiSpoof_V6;
                if (cbc.IPv4_AntiSpoofingCapable && !cbc.IPv6_AntiSpoofingCapable) s = ManageabilityCommanderTool.Properties.Resources.LV_AntiSpoof_V4;

                filtersInfoListView.Items.Add(new ListViewItem(new string[] { 
                        ManageabilityCommanderTool.Properties.Resources.LV_AntiSpoof_Str, s }));
                filtersInfoListView.Items.Add(new ListViewItem(new string[] { 
                        ManageabilityCommanderTool.Properties.Resources.LV_Info_V4_Counter, 
                        cbc.IPv4_MaxRxCounters.ToString() + Properties.Resources.SlashSeperator + cbc.IPv4_MaxTxCounters.ToString() }));
                filtersInfoListView.Items.Add(new ListViewItem(new string[] { 
                        ManageabilityCommanderTool.Properties.Resources.LV_Info_V4_Flt,
                        cbc.IPv4_MaxRxFilters.ToString() + Properties.Resources.SlashSeperator + cbc.IPv4_MaxTxFilters.ToString() }));
                filtersInfoListView.Items.Add(new ListViewItem(new string[] { 
                        ManageabilityCommanderTool.Properties.Resources.LV_Info_V6_Counter, 
                        cbc.IPv6_MaxRxCounters.ToString() + Properties.Resources.SlashSeperator + cbc.IPv6_MaxTxCounters.ToString() }));
                filtersInfoListView.Items.Add(new ListViewItem(new string[] { 
                        ManageabilityCommanderTool.Properties.Resources.LV_Info_V6_Flt,
                        cbc.IPv6_MaxRxFilters.ToString() + Properties.Resources.SlashSeperator + cbc.IPv6_MaxTxFilters.ToString() }));
                filtersInfoListView.Items.Add(new ListViewItem(new string[] { 
                        ManageabilityCommanderTool.Properties.Resources.LV_Info_V6_V4, 
                        cbc.IPv6_To_IPv4_Ratio.ToString() }));
            }
        }

        public void ForceUpdatePanel()
        {
        }

        private void createNewFilterButton_Click(object sender, System.EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag == null && mainTreeView.SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Filters) == 0)
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Tag;
                DialogResult r = new AddFilterForm(computer).ShowDialog(this);
                if (r == DialogResult.OK) mainform.UpdateTreeFilters(computer);
            }
        }

    }
}
