// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System.Data;
using System.Text;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool.Panels
{
    public partial class NetworkPolicies : UserControl, PanelInterface
    {
        private MainForm mainform;
        private TreeView mainTreeView;
        private AmtSystem computer = null;

        public NetworkPolicies()
        {
            InitializeComponent();           
        }

        public void Setup(MainForm mainform, object obj1, object obj2)
        {
            this.mainform = mainform;
            this.mainTreeView = mainform.GetTreeView();
            this.computer = (AmtSystem)obj1;
        }

        public void Cleanup()
        {
            computer = null;
        }

        public void PerformAction(string command, object[] args)
        {
            if (command.Equals("ADD")) createNewPolicyButton_Click(this, null);
        }

        public void UpdatePanel()
        {
            if (computer == null) return;

            uint MaxSupportedPolicies;
            uint MaxSupportedFilters;
            CircuitBreakerCapabilitiesType[] cbcs = computer.CircuitBreaker.GetCircuitBreakerCapabilities(out MaxSupportedPolicies, out MaxSupportedFilters);
            CircuitBreakerPolicyInfoType[] cbp = computer.CircuitBreaker.GetCircuitBreakerPolicies();

            policyCountLabel.Text = cbp.Length.ToString() + Properties.Resources.SlashSeperator + MaxSupportedPolicies.ToString();
            wirelessPolicyComboBox.Enabled = (cbcs.Length > 1);

            wiredPolicyComboBox.BeginUpdate();
            wirelessPolicyComboBox.BeginUpdate();

            wiredPolicyComboBox.Items.Clear();
            Utils.DropDownItem d1 = new Utils.DropDownItem(ManageabilityCommanderTool.Properties.Resources.No_Act_Policy, -1, null);
            wiredPolicyComboBox.Items.Add(d1);
            wiredPolicyComboBox.SelectedIndex = 0;

            wirelessPolicyComboBox.Items.Clear();
            Utils.DropDownItem d2 = new Utils.DropDownItem(ManageabilityCommanderTool.Properties.Resources.No_Act_Policy, -1, null);
            wirelessPolicyComboBox.Items.Add(d1);
            wirelessPolicyComboBox.SelectedIndex = 0;

            CircuitBreakerPolicyInfoType[] policies = computer.CircuitBreaker.GetCircuitBreakerPolicies();
            foreach (CircuitBreakerPolicyInfoType policy in policies)
            {
                Utils.DropDownItem dd = new Utils.DropDownItem(policy.Policy.PolicyName, (int)policy.PolicyCreationHandle, policy);
                wiredPolicyComboBox.Items.Add(dd);
                wirelessPolicyComboBox.Items.Add(dd);
                if (computer.CircuitBreaker.IsPolicyActive(policy.PolicyCreationHandle, 0)) wiredPolicyComboBox.SelectedItem = dd;
                if (computer.CircuitBreaker.IsPolicyActive(policy.PolicyCreationHandle, 1)) wirelessPolicyComboBox.SelectedItem = dd;
            }

            wiredPolicyComboBox.EndUpdate();
            wirelessPolicyComboBox.EndUpdate();
            applyPolicyButton.Enabled = false;
        }

        public void ForceUpdatePanel()
        {
            refrechButton_Click(this, null);
        }

        private void createNewPolicyButton_Click(object sender, System.EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag == null && mainTreeView.SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Policies) == 0)
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Tag;

                AddPolicyForm f = new AddPolicyForm(computer);
                DialogResult r = f.ShowDialog(this);
                if (r == DialogResult.OK)
                {
                    computer.CircuitBreaker.CircuitBreakerCreatePolicy(f.GetPolicy());
                    mainform.UpdateTreePolicies(computer);
                }
            }
        }

        private void refrechButton_Click(object sender, System.EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag == null && mainTreeView.SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Policies) == 0)
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Tag;
                computer.CircuitBreaker.ClearPoliciesCache();
                mainform.UpdateTreePolicies(computer);
            }
        }

        private void applyPolicyButton_Click(object sender, System.EventArgs e)
        {
            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Tag;

            Utils.DropDownItem dd1 = (Utils.DropDownItem)wiredPolicyComboBox.SelectedItem;
            Utils.DropDownItem dd2 = (Utils.DropDownItem)wirelessPolicyComboBox.SelectedItem;

            int c = 0;
            if (dd1.Tag != null) c++;
            if (dd2.Tag != null) c++;

            CircuitBreakerHardwarePolicyType[] policies = new CircuitBreakerHardwarePolicyType[c];
            c = 0;
            if (dd1.Tag != null)
            {
                policies[c] = new CircuitBreakerHardwarePolicyType();
                policies[c].HardwareID = 0;
                policies[c].PolicyCreationHandle = (uint)dd1.Handle;
                c++;
            }
            if (dd2.Tag != null)
            {
                policies[c] = new CircuitBreakerHardwarePolicyType();
                policies[c].HardwareID = 1;
                policies[c].PolicyCreationHandle = (uint)dd2.Handle;
                c++;
            }

            if (policies.Length == 0)
            {
                computer.CircuitBreaker.CircuitBreakerDisablePolicy();
            }
            else
            {
                computer.CircuitBreaker.CircuitBreakerEnablePolicies(policies);
            }

            mainform.UpdateTreePolicies(computer);
            mainform.UpdateTreeView();
            applyPolicyButton.Enabled = false;
        }

        private void showPolicyMonitorButton_Click(object sender, System.EventArgs e)
        {
            mainform.ShowPolicyMonitor();
        }

        private void wiredPolicyComboBox_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            applyPolicyButton.Enabled = true;
        }

        private void wirelessPolicyComboBox_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            applyPolicyButton.Enabled = true;
        }
    }
}
