// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Threading;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool.Panels
{
    public partial class NetworkPolicyPanel : UserControl, PanelInterface
    {
        private MainForm mainform;
        private TreeView mainTreeView;
        private AmtSystem computer = null;
        private CircuitBreakerPolicyInfoType cp;

        public NetworkPolicyPanel()
        {
            InitializeComponent();         
        }

        public void Setup(MainForm mainform, object obj1, object obj2)
        {
            this.mainform = mainform;
            this.mainTreeView = mainform.GetTreeView();
            this.computer = (AmtSystem)obj1;
            this.cp = (CircuitBreakerPolicyInfoType)obj2;
        }

        public void Cleanup()
        {
            computer = null;
        }

        public void PerformAction(string command, object[] args)
        {
            if (command.Equals("REMOVE")) removePolicyButton_Click(this, null);
        }

        public void UpdatePanel()
        {
            if (computer == null) return;

            policyNameLabel.Text = cp.Policy.PolicyName;
            policyHandleLabel.Text = cp.PolicyCreationHandle.ToString();
            policyPrecedenceLabel.Text = cp.Policy.PolicyPrecedence.ToString();

            policyFilterListView.Items.Clear();
            uint[] filterHandles = cp.Policy.FilterCreationHandles;
            if (filterHandles != null)
            {
                Hashtable filterTable = computer.CircuitBreaker.GetCircuitBreakerFiltersTable();
                foreach (uint filterHandle in filterHandles)
                {
                    ListViewItem l = new ListViewItem(new string[] { (string)filterTable[filterHandle] } , 2);
                    policyFilterListView.Items.Add(l);
                }
            }

            uint MaxSupportedPolicies;
            uint MaxSupportedFilters;
            CircuitBreakerCapabilitiesType[] cbcs = computer.CircuitBreaker.GetCircuitBreakerCapabilities(out MaxSupportedPolicies, out MaxSupportedFilters);

            // Display all hardware ID on which this policy is active
            string activity = "";
            uint[] activePolicy = computer.CircuitBreaker.IsPolicyActive(cp.PolicyCreationHandle);
            if (activePolicy == null) { activity = ManageabilityCommanderTool.Properties.Resources.Policy_Not_Active; }
            else
            {
                activity = ManageabilityCommanderTool.Properties.Resources.Policy_Active_On;
                for (int i = 0; i < activePolicy.Length; i++)
                {
                    if (i != 0) activity += ", ";
                    activity += cbcs[activePolicy[i]].HardwareDescription;
                }
            }
            policyActiveOnLabel.Text = activity;

            string s = ManageabilityCommanderTool.Properties.Resources.MatchAction_None;
            if (cp.Policy.AntiSpoofingFilter != null)
            {
                if (cp.Policy.AntiSpoofingFilter.ActionCount && cp.Policy.AntiSpoofingFilter.ActionEventOnMatch) s = ManageabilityCommanderTool.Properties.Resources.MatchAction_Both;
                if (!cp.Policy.AntiSpoofingFilter.ActionCount && cp.Policy.AntiSpoofingFilter.ActionEventOnMatch) s = ManageabilityCommanderTool.Properties.Resources.MatchAction_Log;
                if (cp.Policy.AntiSpoofingFilter.ActionCount && !cp.Policy.AntiSpoofingFilter.ActionEventOnMatch) s = ManageabilityCommanderTool.Properties.Resources.MatchAction_Count;
            }
            policyAntiSpoofLabel.Text = s;

            s = "";
            if (cp.Policy.DefaultRXFilter.ActionCount) s += ManageabilityCommanderTool.Properties.Resources.FltAction_Count;
            if (cp.Policy.DefaultRXFilter.ActionDrop) s += ManageabilityCommanderTool.Properties.Resources.FltAction_Drop;
            if (cp.Policy.DefaultRXFilter.ActionEventOnMatch) s += ManageabilityCommanderTool.Properties.Resources.FltAction_Log;
            if (s.Length > 2) s = s.Substring(2); else s = ManageabilityCommanderTool.Properties.Resources.FltAction_None;
            policyDefaultRxLabel.Text = s;

            s = "";
            if (cp.Policy.DefaultTXFilter.ActionCount) s += ManageabilityCommanderTool.Properties.Resources.FltAction_Count;
            if (cp.Policy.DefaultTXFilter.ActionDrop) s += ManageabilityCommanderTool.Properties.Resources.FltAction_Drop;
            if (cp.Policy.DefaultTXFilter.ActionEventOnMatch) s += ManageabilityCommanderTool.Properties.Resources.FltAction_Log;
            if (s.Length > 2) s = s.Substring(2); else s = ManageabilityCommanderTool.Properties.Resources.FltAction_None;
            policyDefaultTxLabel.Text = s;
        }

        public void ForceUpdatePanel()
        {
        }

        private void removePolicyButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(CircuitBreakerPolicyInfoType))
            {
                CircuitBreakerPolicyInfoType cbp = (CircuitBreakerPolicyInfoType)mainTreeView.SelectedNode.Tag;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Parent.Tag;
                mainTreeView.SelectedNode = mainTreeView.SelectedNode.Parent;
                computer.CircuitBreaker.CircuitBreakerRemovePolicy(cbp.PolicyCreationHandle);
                mainform.UpdateTreePolicies(computer);
            }
        }

        private void policyActivateAllButton_Click(object sender, EventArgs e)
        {
            CircuitBreakerPolicyInfoType cp = (CircuitBreakerPolicyInfoType)mainTreeView.SelectedNode.Tag;
            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Parent.Tag;

            uint MaxSupportedPolicies;
            uint MaxSupportedFilters;
            CircuitBreakerCapabilitiesType[] cbcs = computer.CircuitBreaker.GetCircuitBreakerCapabilities(out MaxSupportedPolicies, out MaxSupportedFilters);

            CircuitBreakerHardwarePolicyType[] policies = new CircuitBreakerHardwarePolicyType[cbcs.Length];
            int i = 0;
            foreach (CircuitBreakerCapabilitiesType c in cbcs)
            {
                policies[i] = new CircuitBreakerHardwarePolicyType();
                policies[i].HardwareID = c.HardwareID;
                policies[i].PolicyCreationHandle = cp.PolicyCreationHandle;
                i++;
            }

            computer.CircuitBreaker.CircuitBreakerEnablePolicies(policies);

            mainform.UpdateTreePolicies(computer);
            mainform.UpdateTreeView();
        }

    }
}
