// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool.Panels
{
    public partial class StoragePanel : UserControl, PanelInterface
    {
        private MainForm mainform;
        private TreeView mainTreeView;
        private AmtSystem computer = null;
        private AmtStorage storage;

        public StoragePanel()
        {
            InitializeComponent();           
        }

        public void Setup(MainForm mainform, object obj1, object obj2)
        {
            this.mainform = mainform;
            this.mainTreeView = mainform.GetTreeView();
            this.computer = (AmtSystem)obj1;
            this.storage = (AmtStorage)obj2;
        }

        public void Cleanup()
        {
            computer = null;
            storage = null;
        }

        public void PerformAction(string command, object[] args) { }

        public void UpdatePanel()
        {
            if (computer == null) return;

            GlobalStorageAttributesType attribs = storage.GetStorageAttributes();
            totalStorageLabel.Text = String.Format(ManageabilityCommanderTool.Properties.Resources.Node_Store_Used_Total_Format, (attribs.TotalAllocatedStorage / 1024).ToString(), (attribs.TotalStorage / 1024).ToString());
            storageProgressBar.Maximum = (int)attribs.TotalStorage;
            storageProgressBar.Value = (int)attribs.TotalAllocatedStorage;

            AmtEnterprise[] enterprises = storage.GetEnterpriseEntries();
            if (enterprises != null)
            {
                regEnterprisesLabel.Text = String.Format("{0} / {1}", enterprises.Length.ToString(), attribs.MaxEaclEntries.ToString());
            }
            else
            {
                regEnterprisesLabel.Text = String.Format(ManageabilityCommanderTool.Properties.Resources.None_Max_Str, attribs.MaxEaclEntries.ToString());
            }

            //AmtApplication[] applications = storage.GetRegisteredApplications();
            AmtStorageAlloc[] allocations = storage.GetStorageAllocations();
            if (allocations != null)
            {
                regPartnerLabel.Text = String.Format("{0} / {1}", allocations.Length.ToString(), attribs.MaxFpaclEntries.ToString());
            }
            else
            {
                regPartnerLabel.Text = String.Format(ManageabilityCommanderTool.Properties.Resources.None_Max_Str, attribs.MaxFpaclEntries.ToString());
            }
        }

        public void ForceUpdatePanel()
        {
        }

        private void allocateBlockButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtStorage))
            {
                AmtStorage storage = (AmtStorage)mainTreeView.SelectedNode.Tag;
                if (new AllocateStorageForm(storage, null, null).ShowDialog(this) == DialogResult.OK)
                {
                    AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
                    mainform.UpdateTreeStorage(computer);
                }
            }
        }

        private void editApplicationsButton_Click(object sender, EventArgs e)
        {
            AmtStorage storage = (AmtStorage)mainTreeView.SelectedNode.Tag;
            new StorageApplicationForm(storage).ShowDialog(this);
        }

        private void editEnterprisesButton_Click(object sender, EventArgs e)
        {
            AmtStorage storage = (AmtStorage)mainTreeView.SelectedNode.Tag;
            new StorageEnterpriseForm(storage).ShowDialog(this);

            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
            mainform.UpdateTreeStorage(computer);
        }

        private void nvRamSettingsButton_Click(object sender, EventArgs e)
        {
            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
            new NvramSettingsForm(computer).ShowDialog(this);
            mainform.UpdateTreeStorage(computer);
        }
    }
}
