// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool.Panels
{
    public partial class WatchdogAgentPanel : UserControl, PanelInterface
    {
        private MainForm mainform;
        private TreeView mainTreeView;
        private AmtSystem computer;
        private AmtWatchdogAgent wg;

        public WatchdogAgentPanel()
        {
            InitializeComponent();           
        }

        public void Setup(MainForm mainform, object obj1, object obj2)
        {
            this.mainform = mainform;
            this.mainTreeView = mainform.GetTreeView();
            this.computer = (AmtSystem)obj1;
            this.wg = (AmtWatchdogAgent)obj2;
        }

        public void Cleanup()
        {
            computer = null;
            wg = null;
        }

        public void PerformAction(string command, object[] args)
        {
            if (command.Equals("REMOVE")) removeWatchdogButton_Click(this, null);
        }

        public void UpdatePanel()
        {
            if (computer == null) return;

            // Update the watchdog
            computer.Watchdog.UpdateAgent(ref wg);
            mainTreeView.SelectedNode.Tag = wg;

            // Show the transitions
            watchdogNameLabel.Text = wg.Description;
            watchdogIdLabel.Text = wg.AgentId.ToString().ToUpper();
            watchdogTimeoutsLabel.Text = String.Format(ManageabilityCommanderTool.Properties.Resources.Node_Dog_ShowTrans_Format, wg.HeartbeatTime.ToString(), wg.StartupTime.ToString());
            watchdogStateLabel.Text = ToWatchdogStateString(wg.State);

            // Update the image
            if ((wg.State & 0x0001) != 0) watchdogPictureBox.Image = Properties.Resources.RoundInfo;  // Not Started
            else if ((wg.State & 0x0002) != 0) watchdogPictureBox.Image = Properties.Resources.RoundInfo;  // Stopped
            else if ((wg.State & 0x0004) != 0) watchdogPictureBox.Image = Properties.Resources.RoundGreen; // Running
            else if ((wg.State & 0x0008) != 0) watchdogPictureBox.Image = Properties.Resources.RoundRed;   // Expired
            else if ((wg.State & 0x0010) != 0) watchdogPictureBox.Image = Properties.Resources.RoundGray;  // Suspended

            ConsoleWatchdogActionType[] actions = computer.Watchdog.GetActions(wg);
            watchdogTransitionListView.Items.Clear();
            if (actions != null)
            {
                int i = 0;
                foreach (ConsoleWatchdogActionType action in actions)
                {
                    i++;
                    string PolicyAction = "";
                    if (action.ActionCbSpecified == false) PolicyAction = ManageabilityCommanderTool.Properties.Resources.Node_Dog_PolicyAct_0;
                    else if (action.ActionCb == 1) PolicyAction = ManageabilityCommanderTool.Properties.Resources.Node_Dog_PolicyAct_1;
                    else if (action.ActionCb == 0) PolicyAction = ManageabilityCommanderTool.Properties.Resources.Node_Dog_PolicyAct_2;
                    ListViewItem l = new ListViewItem(new String[] { AmtWatchdogRemote.TransitionStateToString(action.OldState), AmtWatchdogRemote.TransitionStateToString(action.NewState), action.ActionEventOnTransition.ToString(), PolicyAction });
                    l.Tag = action;
                    watchdogTransitionListView.Items.Add(l);
                }
            }
            watchdogRemoveTransitionButton.Enabled = (watchdogTransitionListView.SelectedItems.Count == 1);
            watchdogEditButton.Enabled = (watchdogTransitionListView.SelectedItems.Count == 1);
        }

        public void ForceUpdatePanel()
        {
            updateWatchdogStateButton_Click(this, null);
        }

        private void updateWatchdogStateButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode.Tag == null || mainTreeView.SelectedNode.Tag.GetType() != typeof(AmtWatchdogAgent)) return;

            // Update the state of the watchdog
            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Parent.Tag;
            AmtWatchdogAgent wg = (AmtWatchdogAgent)mainTreeView.SelectedNode.Tag;
            computer.Watchdog.UpdateAgent(ref wg);
            if (wg != null)
            {
                mainTreeView.SelectedNode.Tag = wg;
                string s = ToWatchdogStateString(wg.State);
                if (watchdogStateLabel.Text.CompareTo(s) != 0) watchdogStateLabel.Text = s;

                // Update the image
                if ((wg.State & 0x0001) != 0) watchdogPictureBox.Image = Properties.Resources.RoundInfo;  // Not Started
                else if ((wg.State & 0x0002) != 0) watchdogPictureBox.Image = Properties.Resources.RoundInfo;  // Stopped
                else if ((wg.State & 0x0004) != 0) watchdogPictureBox.Image = Properties.Resources.RoundGreen; // Running
                else if ((wg.State & 0x0008) != 0) watchdogPictureBox.Image = Properties.Resources.RoundRed;   // Expired
                else if ((wg.State & 0x0010) != 0) watchdogPictureBox.Image = Properties.Resources.RoundGray;  // Suspended
            }
            else
            {
                string s = Properties.Resources.WatchdogRemovedState;
                if (watchdogStateLabel.Text.CompareTo(s) != 0) watchdogStateLabel.Text = s;
            }
        }

        private void watchdogRefreshButton_Click(object sender, EventArgs e)
        {
            mainform.UpdateTreeView();
        }

        private void watchdogEditButton_Click(object sender, EventArgs e)
        {
            if (watchdogTransitionListView.SelectedItems.Count != 1) return;
            ListViewItem l = watchdogTransitionListView.SelectedItems[0];

            if (mainTreeView.SelectedNode.Tag == null || mainTreeView.SelectedNode.Tag.GetType() != typeof(AmtWatchdogAgent)) return;
            AmtWatchdogAgent agent = (AmtWatchdogAgent)mainTreeView.SelectedNode.Tag;
            ConsoleWatchdogActionType editaction = (ConsoleWatchdogActionType)l.Tag;

            AddWatchdogTransitionForm f = new AddWatchdogTransitionForm(editaction);
            if (f.ShowDialog(this) == DialogResult.OK)
            {
                ConsoleWatchdogActionType action = f.GetAction();
                // Save the new transit states
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Parent.Tag;
                ConsoleWatchdogActionType[] actions = new ConsoleWatchdogActionType[watchdogTransitionListView.Items.Count];
                int i = 0;
                foreach (ListViewItem ll in watchdogTransitionListView.Items)
                {
                    if ((ConsoleWatchdogActionType)ll.Tag == editaction)
                    {
                        actions[i] = action;
                    }
                    else
                    {
                        actions[i] = (ConsoleWatchdogActionType)ll.Tag;
                    }
                    i++;
                }
                computer.Watchdog.SetActions(agent, actions);

                mainform.UpdateTreeView();
            }
        }

        private void watchdogRemoveTransitionButton_Click(object sender, EventArgs e)
        {
            if (watchdogTransitionListView.SelectedItems.Count != 1) return;
            ListViewItem l = watchdogTransitionListView.SelectedItems[0];
            TreeNode n = mainTreeView.SelectedNode;
            if (n == null || n.Tag.GetType() != typeof(AmtWatchdogAgent)) return;
            AmtWatchdogAgent wga = (AmtWatchdogAgent)n.Tag;
            watchdogTransitionListView.Items.Remove(l);

            // Save the new transit states
            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Parent.Tag;
            ConsoleWatchdogActionType[] actions = new ConsoleWatchdogActionType[watchdogTransitionListView.Items.Count];
            int i = 0;
            foreach (ListViewItem ll in watchdogTransitionListView.Items)
            {
                actions[i] = (ConsoleWatchdogActionType)ll.Tag;
                i++;
            }
            if (actions.Length != 0)
            {
                computer.Watchdog.SetActions(wga, actions);
            }
            else
            {
                //computer.Watchdog.SetActions(wga, null);
            }
        }

        private void watchdogAddTransitionButton_Click(object sender, EventArgs e)
        {
            AddWatchdogTransitionForm f = new AddWatchdogTransitionForm();
            if (f.ShowDialog(this) == DialogResult.OK)
            {
                ConsoleWatchdogActionType action = f.GetAction();

                ListViewItem l = new ListViewItem(new string[] { "" });
                l.Tag = action;
                watchdogTransitionListView.Items.Add(l);

                TreeNode n = mainTreeView.SelectedNode;
                if (n == null || n.Tag.GetType() != typeof(AmtWatchdogAgent)) return;
                AmtWatchdogAgent wga = (AmtWatchdogAgent)n.Tag;

                // Save the new transit states
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Parent.Tag;
                ConsoleWatchdogActionType[] actions = new ConsoleWatchdogActionType[watchdogTransitionListView.Items.Count];
                int i = 0;
                foreach (ListViewItem ll in watchdogTransitionListView.Items)
                {
                    actions[i] = (ConsoleWatchdogActionType)ll.Tag;
                    i++;
                }
                computer.Watchdog.SetActions(wga, actions);

                mainform.UpdateTreeView();
            }
        }

        private void removeWatchdogButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtWatchdogAgent))
            {
                AmtWatchdogAgent wg = (AmtWatchdogAgent)mainTreeView.SelectedNode.Tag;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Parent.Tag;
                computer.Watchdog.Delete(wg);
                mainTreeView.SelectedNode = mainTreeView.SelectedNode.Parent;
                mainform.UpdateTreeWatchdogs(computer);
            }
        }

        private void watchdogTransitionListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            watchdogRemoveTransitionButton.Enabled = (watchdogTransitionListView.SelectedItems.Count == 1);
            watchdogEditButton.Enabled = (watchdogTransitionListView.SelectedItems.Count == 1);
        }

        private string ToWatchdogStateString(byte state)
        {
            string sts = "";
            if (state == 0x00FF)
            {
                sts = ManageabilityCommanderTool.Properties.Resources.MainForm_WDog_ANY;
            }
            else
            {
                if ((state & 0x0001) != 0) sts += ManageabilityCommanderTool.Properties.Resources.MainForm_WDog_NSTART;
                if ((state & 0x0002) != 0) sts += ManageabilityCommanderTool.Properties.Resources.MainForm_WDog_STOP;
                if ((state & 0x0004) != 0) sts += ManageabilityCommanderTool.Properties.Resources.MainForm_WDog_RUN;
                if ((state & 0x0008) != 0) sts += ManageabilityCommanderTool.Properties.Resources.MainForm_WDog_EXPIRE;
                if ((state & 0x0010) != 0) sts += ManageabilityCommanderTool.Properties.Resources.MainForm_WDog_SUSPEND;
                if (sts.Length > 2) sts = sts.Substring(2);
            }
            return sts;
        }

    }
}
