// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool.Panels
{
    public partial class WatchdogsPanel : UserControl, PanelInterface
    {
        private MainForm mainform;
        private TreeView mainTreeView;
        private AmtSystem computer;

        public WatchdogsPanel()
        {
            InitializeComponent();           
        }

        public void Setup(MainForm mainform, object obj1, object obj2)
        {
            this.mainform = mainform;
            this.mainTreeView = mainform.GetTreeView();
            this.computer = (AmtSystem)obj1;
        }

        public void Cleanup()
        {
            computer = null;
        }

        public void PerformAction(string command, object[] args)
        {
            if (command.Equals("ADD")) createWatchdogButton_Click(this, null);
        }

        public void UpdatePanel()
        {
            if (computer == null) return;

            AgentPresenceCapabilitiesType wgc = computer.Watchdog.QueryCapabilities();
            if (wgc != null)
            {
                watchdogTotalAgentsLabel.Text = wgc.MaxTotalAgents.ToString();
                watchdogTotalActionsLabel.Text = wgc.MaxTotalActions.ToString();
                watchdogMinActionsLabel.Text = wgc.MinGuaranteedActionListSize.ToString();
            }

            // Show network interfaces
            uint MaxSupportedPolicies;
            uint MaxSupportedFilters;
            CircuitBreakerCapabilitiesType[] cbcs = computer.CircuitBreaker.GetCircuitBreakerCapabilities(out MaxSupportedPolicies, out MaxSupportedFilters);
            wirelessWatchdogPolicyComboBox.Enabled = (cbcs.Length > 1);
            bool wirelessInterface = (cbcs.Length > 1);

            // Add No Associations
            wiredWatchdogPolicyComboBox.BeginUpdate();
            wirelessWatchdogPolicyComboBox.BeginUpdate();

            wiredWatchdogPolicyComboBox.Items.Clear();
            wirelessWatchdogPolicyComboBox.Items.Clear();
            Utils.DropDownItem dd = new Utils.DropDownItem();

            dd.Text = ManageabilityCommanderTool.Properties.Resources.Node_Dog_NoPoAss;
            dd.Handle = int.MaxValue;
            wiredWatchdogPolicyComboBox.Items.Add(dd);
            wiredWatchdogPolicyComboBox.SelectedItem = dd;
            if (wirelessInterface) wirelessWatchdogPolicyComboBox.Items.Add(dd);
            if (wirelessInterface) wirelessWatchdogPolicyComboBox.SelectedItem = dd;

            // Add the policies
            CircuitBreakerPolicyInfoType[] policies = computer.CircuitBreaker.GetCircuitBreakerPolicies();
            foreach (CircuitBreakerPolicyInfoType policy in policies)
            {
                Utils.DropDownItem d = new Utils.DropDownItem();
                d.Text = policy.Policy.PolicyName;
                d.Tag = policy;
                d.Handle = (int)policy.PolicyCreationHandle;
                wiredWatchdogPolicyComboBox.Items.Add(d);
                if (wirelessInterface) wirelessWatchdogPolicyComboBox.Items.Add(d);
            }

            CircuitBreakerWGHardwarePolicyType[] CbPolicys = computer.Watchdog.GetCbPolicy();
            if (CbPolicys != null)
            {
                foreach (CircuitBreakerWGHardwarePolicyType policy in CbPolicys)
                {
                    if (policy.HardwareID == 0)
                    {
                        bool found = false;
                        foreach (Utils.DropDownItem d in wiredWatchdogPolicyComboBox.Items)
                        {
                            if (d.Handle == policy.PolicyCreationHandle)
                            {
                                wiredWatchdogPolicyComboBox.SelectedItem = d;
                                found = true;
                                break;
                            }
                        }
                        if (found == false)
                        {
                            Utils.DropDownItem d = new Utils.DropDownItem();
                            d.Text = String.Format(ManageabilityCommanderTool.Properties.Resources.Node_Dog_UnknownHdl_Format, policy.PolicyCreationHandle.ToString());
                            d.Handle = (int)policy.PolicyCreationHandle;
                            wiredWatchdogPolicyComboBox.Items.Add(d);
                            wiredWatchdogPolicyComboBox.SelectedItem = d;
                        }
                    }
                    else if (wirelessInterface && policy.HardwareID == 1)
                    {
                        bool found = false;
                        foreach (Utils.DropDownItem d in wirelessWatchdogPolicyComboBox.Items)
                        {
                            if (d.Handle == policy.PolicyCreationHandle)
                            {
                                wirelessWatchdogPolicyComboBox.SelectedItem = d;
                                found = true;
                                break;
                            }
                        }
                        if (found == false)
                        {
                            Utils.DropDownItem d = new Utils.DropDownItem();
                            d.Text = String.Format(ManageabilityCommanderTool.Properties.Resources.Node_Dog_UnknownHdl_Format, policy.PolicyCreationHandle.ToString());
                            d.Handle = (int)policy.PolicyCreationHandle;
                            wirelessWatchdogPolicyComboBox.Items.Add(d);
                            wirelessWatchdogPolicyComboBox.SelectedItem = d;
                        }
                    }
                }
            }
            wiredWatchdogPolicyComboBox.EndUpdate();
            wirelessWatchdogPolicyComboBox.EndUpdate();
        }

        public void ForceUpdatePanel()
        {
        }

        private void createWatchdogButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag == null && mainTreeView.SelectedNode.Text.CompareTo(ManageabilityCommanderTool.Properties.Resources.TreeNode_Watchdogs) == 0)
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Tag;

                AddWatchdogForm f = new AddWatchdogForm();
                DialogResult r = f.ShowDialog(this);
                if (r == DialogResult.OK)
                {
                    computer.Watchdog.Add(f.GetAgent());
                    mainform.UpdateTreeWatchdogs(computer);
                }
            }
        }

        private void applyWatchdogPolicyButton_Click(object sender, EventArgs e)
        {
            CircuitBreakerWGHardwarePolicyType p1 = null;
            Utils.DropDownItem d1 = (Utils.DropDownItem)wiredWatchdogPolicyComboBox.SelectedItem;
            if (d1 != null && d1.Handle != int.MaxValue)
            {
                p1 = new CircuitBreakerWGHardwarePolicyType();
                p1.HardwareID = 0;
                p1.PolicyCreationHandle = (uint)d1.Handle;
            }

            CircuitBreakerWGHardwarePolicyType p2 = null;
            Utils.DropDownItem d2 = (Utils.DropDownItem)wirelessWatchdogPolicyComboBox.SelectedItem;
            if (d2 != null && d2.Handle != int.MaxValue)
            {
                p2 = new CircuitBreakerWGHardwarePolicyType();
                p2.HardwareID = 1;
                p2.PolicyCreationHandle = (uint)d2.Handle;
            }

            int c = 0;
            if (p1 != null) c++;
            if (p2 != null) c++;

            CircuitBreakerWGHardwarePolicyType[] policies = null;
            if (c > 0)
            {
                policies = new CircuitBreakerWGHardwarePolicyType[c];
                c = 0;
                if (p1 != null) { policies[c] = p1; c++; }
                if (p2 != null) { policies[c] = p2; c++; }
            }

            AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Parent.Tag;
            computer.Watchdog.SetCbPolicy(policies);

            mainform.UpdateTreeView();
        }
    }
}
