// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class PolicyMonitorForm : Form
    {
        private AmtSystem computer;

        public PolicyMonitorForm(AmtSystem c)
        {
            computer = c;
            InitializeComponent();

            hostnameTextBox.Text = computer.HostNameIdentity.DefaultName + " / " + computer.Username;
            refreshButton_Click(this, null);
            Refresh(false);

            if (computer != null) computer.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }

        private void SystemStateChangedHandler(AmtSystem computer) { try { this.Invoke(new AmtSystem.ObjectStateHandler(SystemStateChangedHandlerEx), new object[] { computer }); } catch (Exception) { } }
        private void SystemStateChangedHandlerEx(AmtSystem computer) { if (computer.State != AmtSystemObjState.Connected) Close(); }

        private void refreshButton_Click(object sender, EventArgs e)
        {
            Refresh(false);
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void resetButton_Click(object sender, EventArgs e)
        {
            Refresh(true);
            Refresh(false);
        }

        private void Refresh(bool reset)
        {
            try
            {
                CircuitBreakerFilterStatisticsType[] stats;
                uint PolicyCreationHandle;
                uint ActivationTime;
                uint LastResetTime;

                stats = computer.CircuitBreaker.CircuitBreakerGetActiveStatistics((uint)(nicComboBox.SelectedIndex), reset, out PolicyCreationHandle, out ActivationTime, out LastResetTime);

                if (stats == null || stats.Length == 0)
                {
                    filterStatusListView.Items.Clear();
                }
                else
                {
                    filterStatusListView.BeginUpdate();

                    ArrayList a = new ArrayList();
                    foreach (ListViewItem l in filterStatusListView.Items) { a.Add(l); }
                    foreach (CircuitBreakerFilterStatisticsType s in stats)
                    {
                        ListViewItem l = Utils.GetItemWithTag(filterStatusListView, s.FilterCreationHandle);
                        if (l == null)
                        {
                            string filtername = computer.CircuitBreaker.GetFilterName(s.FilterCreationHandle);
                            l = new ListViewItem(new string[] { filtername, s.FilterMatched.ToString(), s.ReadCount.ToString() });
                            l.Tag = s.FilterCreationHandle;
                            filterStatusListView.Items.Add(l);
                        }
                        else
                        {
                            if (l.SubItems[1].Text.CompareTo(s.FilterMatched.ToString()) != 0) l.SubItems[1].Text = s.FilterMatched.ToString();
                            if (l.SubItems[2].Text.CompareTo(s.ReadCount.ToString()) != 0) l.SubItems[2].Text = s.ReadCount.ToString();
                            a.Remove(l);
                        }
                    }
                    foreach (ListViewItem l in a) { filterStatusListView.Items.Remove(l); }

                    filterStatusListView.EndUpdate();
                }
            }
            catch (Exception)
            {
                computer.Disconnect();
            }
        }

        private void refreshTimer_Tick(object sender, EventArgs e)
        {
            Refresh(false);
        }

        private void autoRefreshCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            refreshTimer.Enabled = autoRefreshCheckBox.Checked;
        }

        private void PolicyMonitorForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            refreshTimer.Enabled = autoRefreshCheckBox.Checked = true;

            uint MaxSupportedPolicies;
            uint MaxSupportedFilters;
            CircuitBreakerCapabilitiesType[] cbcs = computer.CircuitBreaker.GetCircuitBreakerCapabilities(out MaxSupportedPolicies, out MaxSupportedFilters);
            if (cbcs.Length > 0) nicComboBox.Items.Add(new Utils.DropDownItem(ManageabilityCommanderTool.Properties.Resources.PolicyMonitor_Wired_NIC, 0 , null));
            if (cbcs.Length > 1) nicComboBox.Items.Add(new Utils.DropDownItem(ManageabilityCommanderTool.Properties.Resources.PolicyMonitor_Wireless_NIC, 0 , null));
            nicComboBox.SelectedIndex = 0;
        }

        private void nicComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            Refresh(false);
        }

        private void PolicyMonitorForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (computer != null) computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }
    }
}
