// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Xml;
using System.Text;
using System.Web.UI;
using System.Drawing;
using System.Drawing.Printing;
using System.Collections.Generic;
using System.Web.UI.HtmlControls;
using System.Runtime.InteropServices;
using ManageabilityStack;

using System.Text.RegularExpressions;

namespace ManageabilityCommanderTool
{
    class PrintExportClass
    {
        private static void AddText(FileStream fs, string value)
        {
            byte[] info = UTF8Encoding.UTF8.GetBytes(value);
            fs.Write(info, 0, info.Length);
        }

        private static XmlDocument HardwareAssetToXml(AmtSystem computer)
        {
            AmtHardwareComputerSystem amtComputer = computer.Assets.GetComputerSystem();

            // Define the xml doc for saving hardware data and create the xml header;
            System.Xml.XmlDocument doc = new System.Xml.XmlDocument();
            doc.AppendChild(doc.CreateXmlDeclaration("1.0", "UTF-16", "yes"));

            // Create the XML
            XmlElement elem_computer = doc.CreateElement("Computer");

            XmlAttribute Title = doc.CreateAttribute("Title");
            Title.Value = "Computer System";
            elem_computer.Attributes.Append(Title);

            XmlAttribute Manufacturer = doc.CreateAttribute("Manufacturer");
            Manufacturer.Value = amtComputer.Manufacturer;
            elem_computer.Attributes.Append(Manufacturer);

            XmlAttribute Product = doc.CreateAttribute("Product");
            Product.Value = amtComputer.Product;
            elem_computer.Attributes.Append(Product);

            XmlAttribute Version = doc.CreateAttribute("Version");
            Version.Value = amtComputer.Version;
            elem_computer.Attributes.Append(Version);

            XmlAttribute SerialNumber = doc.CreateAttribute("SerialNumber");
            SerialNumber.Value = amtComputer.SerialNumber;
            elem_computer.Attributes.Append(SerialNumber);

            XmlAttribute UUID = doc.CreateAttribute("UUID");
            UUID.Value = amtComputer.UUID.ToString();
            elem_computer.Attributes.Append(UUID);

            doc.AppendChild(elem_computer);

            // Motherboard
            AmtHardwareBaseBoard baseboard = computer.Assets.GetBaseBoard();

            XmlElement elem_motherboard = doc.CreateElement("Motherboard");

            Title = doc.CreateAttribute("Title");
            Title.Value = "System Motherboard";
            elem_motherboard.Attributes.Append(Title);

            Manufacturer = doc.CreateAttribute("Manufacturer");
            Manufacturer.Value = baseboard.Manufacturer;
            elem_motherboard.Attributes.Append(Manufacturer);

            Product = doc.CreateAttribute("Product");
            Product.Value = baseboard.Product;
            elem_motherboard.Attributes.Append(Product);

            Version = doc.CreateAttribute("Version");
            Version.Value = baseboard.Version;
            elem_motherboard.Attributes.Append(Version);

            SerialNumber = doc.CreateAttribute("SerialNumber");
            SerialNumber.Value = baseboard.SerialNumber;
            elem_motherboard.Attributes.Append(SerialNumber);

            XmlAttribute AssetTag = doc.CreateAttribute("AssetTag");
            AssetTag.Value = baseboard.AssetTag;
            elem_motherboard.Attributes.Append(AssetTag);

            XmlAttribute Replaceable = doc.CreateAttribute("Replaceable");
            Replaceable.Value = baseboard.Replaceable.ToString();
            elem_motherboard.Attributes.Append(Replaceable);

            elem_computer.AppendChild(elem_motherboard);

            // Bios
            AmtHardwareBios bios = computer.Assets.GetBios();

            XmlElement elem_BIOS = doc.CreateElement("BIOS");

            Title = doc.CreateAttribute("Title");
            Title.Value = "System BIOS";
            elem_BIOS.Attributes.Append(Title);

            XmlAttribute Vendor = doc.CreateAttribute("Vendor");
            Vendor.Value = bios.Vendor;
            elem_BIOS.Attributes.Append(Vendor);

            Version = doc.CreateAttribute("Version");
            Version.Value = bios.Version;
            elem_BIOS.Attributes.Append(Version);

            XmlAttribute ReleaseDate = doc.CreateAttribute("ReleaseDate");
            ReleaseDate.Value = bios.ReleaseDate;
            elem_BIOS.Attributes.Append(ReleaseDate);

            XmlAttribute Characteristics = doc.CreateAttribute("Characteristics");
            Characteristics.Value = AmtHardwareBios.GetCharacteristicsString(bios.Characteristics);
            elem_BIOS.Attributes.Append(Characteristics);

            elem_motherboard.AppendChild(elem_BIOS);

            // Processor
            AmtHardwareProcessor[] proc = computer.Assets.GetProcessors();

            for (int i = 0; i < proc.Length; i++)
            {
                XmlElement elem_processor = doc.CreateElement("Processor" + i.ToString());

                Title = doc.CreateAttribute("Title");
                Title.Value = "System Processor " + i.ToString();
                elem_processor.Attributes.Append(Title);

                XmlAttribute SocketDesignation = doc.CreateAttribute("SocketDesignation");
                SocketDesignation.Value = proc[i].SocketDesignation;
                elem_processor.Attributes.Append(SocketDesignation);

                Manufacturer = doc.CreateAttribute("Manufacturer");
                Manufacturer.Value = proc[i].Manufacturer;
                elem_processor.Attributes.Append(Manufacturer);

                XmlAttribute MaxSocketSpeed = doc.CreateAttribute("MaxSocketSpeed");
                MaxSocketSpeed.Value = proc[i].MaxSocketSpeed.ToString() + " Megahertz";
                elem_processor.Attributes.Append(MaxSocketSpeed);

                XmlAttribute CurrentSpeed = doc.CreateAttribute("CurrentSpeed");
                CurrentSpeed.Value = proc[i].CurrentSpeed.ToString() + " Megahertz";
                elem_processor.Attributes.Append(CurrentSpeed);

                XmlAttribute Identifier = doc.CreateAttribute("Identifier");
                Identifier.Value = "0x" + proc[i].ID.ToString("X");
                elem_processor.Attributes.Append(Identifier);

                XmlAttribute SocketPopulated = doc.CreateAttribute("SocketPopulated");
                SocketPopulated.Value = proc[i].SocketPopulated.ToString();
                elem_processor.Attributes.Append(SocketPopulated);

                XmlAttribute Status = doc.CreateAttribute("Status");
                Status.Value = proc[i].Status.ToString();
                elem_processor.Attributes.Append(Status);

                XmlAttribute Type = doc.CreateAttribute("Type");
                Type.Value = proc[i].Type.ToString();
                elem_processor.Attributes.Append(Type);

                XmlAttribute Family = doc.CreateAttribute("Family");
                Family.Value = proc[i].Family.ToString();
                elem_processor.Attributes.Append(Family);

                XmlAttribute UpgradeInformation = doc.CreateAttribute("UpgradeInformation");
                UpgradeInformation.Value = proc[i].UpgradeInformation.ToString();
                elem_processor.Attributes.Append(UpgradeInformation);

                Version = doc.CreateAttribute("Version");
                Version.Value = proc[i].Version.ToString();
                elem_processor.Attributes.Append(Version);

                elem_motherboard.AppendChild(elem_processor);
            }

            // Memory banks
            XmlElement elem_memorybanks = doc.CreateElement("MemoryBanks");

            Title = doc.CreateAttribute("Title");
            Title.Value = "Memory Banks";
            elem_memorybanks.Attributes.Append(Title);

            elem_motherboard.AppendChild(elem_memorybanks);

            // System memory
            AmtHardwareMemoryModule[] module = computer.Assets.GetMemoryModules();

            for (int i = 0; i < module.Length; i++)
            {
                XmlElement elem_memory = doc.CreateElement("SystemMemory" + i.ToString());

                Title = doc.CreateAttribute("Title");
                Title.Value = "System Memory " + i.ToString();
                elem_memory.Attributes.Append(Title);

                XmlAttribute Size = doc.CreateAttribute("Size");
                Size.Value = module[i].Size.ToString() + " Megabytes";
                elem_memory.Attributes.Append(Size);

                XmlAttribute FormFactor = doc.CreateAttribute("FormFactor");
                FormFactor.Value = module[i].FormFactor.ToString();
                elem_memory.Attributes.Append(FormFactor);

                XmlAttribute Type = doc.CreateAttribute("Type");
                Type.Value = module[i].Type.ToString();
                elem_memory.Attributes.Append(Type);

                XmlAttribute TypeDetail = doc.CreateAttribute("TypeDetail");
                TypeDetail.Value = module[i].TypeDetailStr;
                elem_memory.Attributes.Append(TypeDetail);

                Manufacturer = doc.CreateAttribute("Manufacturer");
                Manufacturer.Value = module[i].Manufacturer;
                elem_memory.Attributes.Append(Manufacturer);

                Product = doc.CreateAttribute("Product");
                Product.Value = module[i].PartNumber;
                elem_memory.Attributes.Append(Product);

                XmlAttribute Serial = doc.CreateAttribute("Serial");
                Serial.Value = module[i].SerialNumber;
                elem_memory.Attributes.Append(Serial);

                XmlAttribute Speed = doc.CreateAttribute("Speed");
                Speed.Value = module[i].Speed.ToString() + " Megahertz";
                elem_memory.Attributes.Append(Speed);

                elem_memorybanks.AppendChild(elem_memory);
            }

            // Hardware modules
            XmlElement elem_hardwaremodules = doc.CreateElement("HardwareModules");

            Title = doc.CreateAttribute("Title");
            Title.Value = "Hardware Modules";
            elem_memorybanks.Attributes.Append(Title);

            elem_motherboard.AppendChild(elem_hardwaremodules);

            // Hardware devices
            AmtHardwareModule[] hdmodule = computer.Assets.GetHardwareModules();

            for (int i = 0; i < hdmodule.Length; i++)
            {
                XmlElement elem_hardwaredevices = doc.CreateElement("HardwareModules" + i.ToString());

                Title = doc.CreateAttribute("Title");
                Title.Value = "Hardware Device " + i.ToString();
                elem_hardwaredevices.Attributes.Append(Title);

                Vendor = doc.CreateAttribute("Vendor");
                Vendor.Value = hdmodule[i].VendorIdStr;
                elem_hardwaredevices.Attributes.Append(Vendor);

                XmlAttribute DeviceID = doc.CreateAttribute("DeviceID");
                DeviceID.Value = "0x" + hdmodule[i].DeviceID.ToString("X");
                elem_hardwaredevices.Attributes.Append(DeviceID);

                XmlAttribute RevisionID = doc.CreateAttribute("RevisionID");
                RevisionID.Value = "0x" + hdmodule[i].RevisionID.ToString("X");
                elem_hardwaredevices.Attributes.Append(RevisionID);

                XmlAttribute ProgInterface = doc.CreateAttribute("ProgInterface");
                ProgInterface.Value = "0x" + hdmodule[i].ProgIf.ToString("X");
                elem_hardwaredevices.Attributes.Append(ProgInterface);

                XmlAttribute Subclass = doc.CreateAttribute("Subclass");
                Subclass.Value = "0x" + hdmodule[i].Subclass.ToString("X");
                elem_hardwaredevices.Attributes.Append(Subclass);

                XmlAttribute Baseclass = doc.CreateAttribute("Baseclass");
                Baseclass.Value = "0x" + hdmodule[i].Baseclass.ToString("X");
                elem_hardwaredevices.Attributes.Append(Baseclass);

                XmlAttribute SubvendorID = doc.CreateAttribute("SubvendorID");
                SubvendorID.Value = "0x" + hdmodule[i].SubvendorID.ToString("X");
                elem_hardwaredevices.Attributes.Append(SubvendorID);

                XmlAttribute SubsystemID = doc.CreateAttribute("SubsystemID");
                SubsystemID.Value = "0x" + hdmodule[i].SubsystemID.ToString("X");
                elem_hardwaredevices.Attributes.Append(SubsystemID);

                XmlAttribute DeviceLocation = doc.CreateAttribute("DeviceLocation");
                DeviceLocation.Value = "0x" + hdmodule[i].DeviceLocation.ToString("X");
                elem_hardwaredevices.Attributes.Append(DeviceLocation);

                elem_hardwaremodules.AppendChild(elem_hardwaredevices);
            }

            // Media
            AmtHardwareMediaDevice[] media = computer.Assets.GetMediaDevices();

            for (int i = 0; i < media.Length; i++)
            {
                XmlElement elem_mediadevices = doc.CreateElement("MediaDevice" + i.ToString());

                Title = doc.CreateAttribute("Title");
                Title.Value = "Media Device " + i.ToString();
                elem_mediadevices.Attributes.Append(Title);


                XmlAttribute ModelNumber = doc.CreateAttribute("ModelNumber");
                ModelNumber.Value = media[i].ModelNumber;
                elem_mediadevices.Attributes.Append(ModelNumber);

                SerialNumber = doc.CreateAttribute("SerialNumber");
                SerialNumber.Value = media[i].SerialNumber;
                elem_mediadevices.Attributes.Append(SerialNumber);

                XmlAttribute MaxMediaSize = doc.CreateAttribute("MaxMediaSize");
                MaxMediaSize.Value = media[i].MaxMediaSizeLStr;
                elem_mediadevices.Attributes.Append(MaxMediaSize);

                // This is too much detail, not needed.
                /*
                XmlAttribute Capabilities1 = doc.CreateAttribute("Capabilities1");
                Capabilities1.Value = "0x" + media[i].Capabilities1.ToString("X") + " - " + media[i].CapabilitiesStr1;
                elem_mediadevices.Attributes.Append(Capabilities1);

                XmlAttribute Capabilities2 = doc.CreateAttribute("Capabilities2");
                Capabilities2.Value = "0x" + media[i].Capabilities2.ToString("X") + " - " + media[i].CapabilitiesStr2;
                elem_mediadevices.Attributes.Append(Capabilities2);

                XmlAttribute Capabilities3 = doc.CreateAttribute("Capabilities3");
                Capabilities3.Value = "0x" + media[i].Capabilities3.ToString("X") + " - " + media[i].CapabilitiesStr1;
                elem_mediadevices.Attributes.Append(Capabilities3);
                */

                elem_motherboard.AppendChild(elem_mediadevices);
            }

            return doc;
        }

        private static string InsertBookmarkText(string strFullText, Match matchedBookmark, string strBookmarkText)
        {
            int TagStartofBookmark = matchedBookmark.Index;
            int TagEndofBookmark = strFullText.IndexOf("</a>", matchedBookmark.Index) + 4;
            return (strFullText.Replace(strFullText.Substring(TagStartofBookmark, TagEndofBookmark - TagStartofBookmark), strBookmarkText));//"<a name=\"" + matchedBookmark.Groups["BookmarkName"].Value + "\">" + strBookmarkText)); 

        }

        private static string HardwareAssetToHtml(AmtSystem computer, string TemplateFileName)
        {
            XmlDocument doc = HardwareAssetToXml(computer);
            XmlNode node_computer = null;
            XmlNode node_motherboard = null;
            XmlNode node_BIOS = null;
            XmlNode node_memorybank = null;
            XmlNode node_hardwaremodules = null;

            XmlNode[] ProcessorArray = new XmlNode[16];
            ProcessorArray.Initialize();
            XmlNode[] StorageArray = new XmlNode[16];
            StorageArray.Initialize();

            int count_Processor = 0;
            int count_Storage = 0;
            int tableFirstTR = 0;
            int tableTREnd = 0;
            int StartValuePosofBookmarkTableStart = 0;
            int EndValuePosofBookmarkTableStart = 0;

            Encoding fileEncoding = HTMLFileEncoding.GetEncoding(TemplateFileName);
            StreamReader TemplateFile = new StreamReader(TemplateFileName, fileEncoding);
            string ReportString = TemplateFile.ReadToEnd();
            TemplateFile.Close();


            foreach (XmlNode node in doc.ChildNodes)
            {
                if (node.Name.Equals("Computer"))
                {
                    node_computer = node;
                }
            }
            foreach (XmlNode node in node_computer.ChildNodes)
            {
                if (node.Name.Equals("Motherboard"))
                {
                    node_motherboard = node;
                }
            }
            for (int i = 0; i < node_motherboard.ChildNodes.Count; i++)
            {
                if (node_motherboard.ChildNodes[i].Name.Equals("BIOS"))
                {
                    node_BIOS = node_motherboard.ChildNodes[i];
                }

                if (node_motherboard.ChildNodes[i].Name.StartsWith("Processor"))
                {
                    ProcessorArray.SetValue(node_motherboard.ChildNodes[i], count_Processor);
                    count_Processor++;
                }

                if (node_motherboard.ChildNodes[i].Name.Equals("MemoryBanks"))
                {
                    node_memorybank = node_motherboard.ChildNodes[i];
                }

                if (node_motherboard.ChildNodes[i].Name.StartsWith("MediaDevice"))
                {
                    if (!node_motherboard.ChildNodes[i].Attributes["MaxMediaSize"].Value.Equals("No Media"))
                    {
                        StorageArray.SetValue(node_motherboard.ChildNodes[i], count_Storage);
                        count_Storage++;
                    }
                }

                if (node_motherboard.ChildNodes[i].Name.Equals("HardwareModules"))
                {
                    node_hardwaremodules = node_motherboard.ChildNodes[i];
                }
            }

            string patternAllBookmarks = @"\B<a name=""(?<BookmarkName>\S*)"">";//define the pattern of bookmark: <a name="BookmarkName">BookmarkText</a>  

            Match Bookmark = Regex.Match(ReportString, patternAllBookmarks);
            string strInsert = null;
            string strBookmarkTableStartText = "";

            //fill in the report with the values
            while (Bookmark.Success != false)
            {
                bool skip = false;
                switch (Bookmark.Groups["BookmarkName"].Value)
                {
                    //title
                    case "Title.ReportTime.Value":
                        ReportString = InsertBookmarkText(ReportString, Bookmark, System.DateTime.Now.ToString());
                        break;
                    case "Title.Hostname.Value":
                        ReportString = InsertBookmarkText(ReportString, Bookmark, computer.HostNameIdentity.DefaultName);
                        break;

                    //computer
                    case "Computer.Manufacturer.Value":
                        if (node_computer != null) ReportString = InsertBookmarkText(ReportString, Bookmark, node_computer.Attributes["Manufacturer"].Value); else skip = true;
                        break;
                    case "Computer.Version.Value":
                        if (node_computer != null) ReportString = InsertBookmarkText(ReportString, Bookmark, node_computer.Attributes["Version"].Value); else skip = true;
                        break;
                    case "Computer.Product.Value":
                        if (node_computer != null) ReportString = InsertBookmarkText(ReportString, Bookmark, node_computer.Attributes["Product"].Value); else skip = true;
                        break;
                    case "Computer.SerialNumber.Value":
                        if (node_computer != null) ReportString = InsertBookmarkText(ReportString, Bookmark, node_computer.Attributes["SerialNumber"].Value); else skip = true;
                        break;
                    case "Computer.UUID.Value":
                        if (node_computer != null) ReportString = InsertBookmarkText(ReportString, Bookmark, node_computer.Attributes["UUID"].Value); else skip = true;
                        break;

                    //motherboard
                    case "Motherboard.Manufacturer.Value":
                        if (node_motherboard != null) ReportString = InsertBookmarkText(ReportString, Bookmark, node_motherboard.Attributes["Manufacturer"].Value); else skip = true;
                        break;
                    case "Motherboard.SerialNumber.Value":
                        if (node_motherboard != null) ReportString = InsertBookmarkText(ReportString, Bookmark, node_motherboard.Attributes["SerialNumber"].Value); else skip = true;
                        break;
                    case "Motherboard.Product.Value":
                        if (node_motherboard != null) ReportString = InsertBookmarkText(ReportString, Bookmark, node_motherboard.Attributes["Product"].Value); else skip = true;
                        break;
                    case "Motherboard.AssetTag.Value":
                        if (node_motherboard != null) ReportString = InsertBookmarkText(ReportString, Bookmark, node_motherboard.Attributes["AssetTag"].Value); else skip = true;
                        break;
                    case "Motherboard.Version.Value":
                        if (node_motherboard != null) ReportString = InsertBookmarkText(ReportString, Bookmark, node_motherboard.Attributes["Version"].Value); else skip = true;
                        break;

                    //BIOS
                    case "BIOS.Vendor.Value":
                        if (node_BIOS != null) ReportString = InsertBookmarkText(ReportString, Bookmark, node_BIOS.Attributes["Vendor"].Value); else skip = true;
                        break;
                    case "BIOS.ReleaseDate.Value":
                        if (node_BIOS != null) ReportString = InsertBookmarkText(ReportString, Bookmark, node_BIOS.Attributes["ReleaseDate"].Value); else skip = true;
                        break;
                    case "BIOS.Version.Value":
                        if (node_BIOS != null) ReportString = InsertBookmarkText(ReportString, Bookmark, node_BIOS.Attributes["Version"].Value); else skip = true;
                        break;
                    case "BIOS.Characteristics.Value":
                        if (node_BIOS != null) ReportString = InsertBookmarkText(ReportString, Bookmark, node_BIOS.Attributes["Characteristics"].Value); else skip = true;
                        break;


                    case "Table.Processor.Start":
                        tableFirstTR = ReportString.IndexOf("<tr>", Bookmark.Index);
                        tableTREnd = ReportString.IndexOf("</table>", Bookmark.Index);
                        strInsert = ReportString.Substring(tableFirstTR, tableTREnd - tableFirstTR);

                        StartValuePosofBookmarkTableStart = ReportString.IndexOf(">", Bookmark.Index) + 1;
                        EndValuePosofBookmarkTableStart = ReportString.IndexOf("<", Bookmark.Index + 1);
                        strBookmarkTableStartText = ReportString.Substring(StartValuePosofBookmarkTableStart, EndValuePosofBookmarkTableStart - StartValuePosofBookmarkTableStart);

                        if (count_Processor > 1)
                        {
                            for (int j = count_Processor - 1; j >= 0; j--)
                            {
                                if (j > 0)
                                {
                                    string NewTableRows = strInsert;
                                    Match BookmarksinInsertString = Regex.Match(NewTableRows, patternAllBookmarks);

                                    while (BookmarksinInsertString.Success != false)
                                    {
                                        switch (BookmarksinInsertString.Groups["BookmarkName"].Value)
                                        {
                                            case "Processor.Manufacturer.Value":
                                                if (ProcessorArray.Length > 0) NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, ProcessorArray[j].Attributes["Manufacturer"].Value); else skip = true;
                                                break;
                                            case "Processor.MaxSpeed.Value":
                                                if (ProcessorArray.Length > 0) NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, ProcessorArray[j].Attributes["MaxSocketSpeed"].Value); else skip = true;
                                                break;
                                            case "Processor.Identifier.Value":
                                                if (ProcessorArray.Length > 0) NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, ProcessorArray[j].Attributes["Identifier"].Value); else skip = true;
                                                break;
                                            case "Processor.CurrentSpeed.Value":
                                                if (ProcessorArray.Length > 0) NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, ProcessorArray[j].Attributes["CurrentSpeed"].Value); else skip = true;
                                                break;
                                            case "Processor.Version.Value":
                                                if (ProcessorArray.Length > 0) NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, ProcessorArray[j].Attributes["Version"].Value); else skip = true;
                                                break;
                                            default:
                                                NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, "");
                                                break;
                                        }
                                        BookmarksinInsertString = Regex.Match(NewTableRows, patternAllBookmarks);
                                    }
                                    ReportString = ReportString.Insert(tableTREnd, NewTableRows);
                                }
                            }
                        }
                        ReportString = InsertBookmarkText(ReportString, Bookmark, strBookmarkTableStartText);
                        break;

                    case "Table.Memory.Start":
                        tableFirstTR = ReportString.IndexOf("<tr>", Bookmark.Index);
                        tableTREnd = ReportString.IndexOf("</table>", Bookmark.Index);
                        strInsert = ReportString.Substring(tableFirstTR, tableTREnd - tableFirstTR);

                        StartValuePosofBookmarkTableStart = ReportString.IndexOf(">", Bookmark.Index) + 1;
                        EndValuePosofBookmarkTableStart = ReportString.IndexOf("<", Bookmark.Index + 1);
                        strBookmarkTableStartText = ReportString.Substring(StartValuePosofBookmarkTableStart, EndValuePosofBookmarkTableStart - StartValuePosofBookmarkTableStart);

                        for (int j = node_memorybank.ChildNodes.Count - 1; j >= 0; j--)
                        {
                            if (j > 0)
                            {
                                string NewTableRows = strInsert;
                                Match BookmarksinInsertString = Regex.Match(NewTableRows, patternAllBookmarks);

                                while (BookmarksinInsertString.Success != false)
                                {
                                    switch (BookmarksinInsertString.Groups["BookmarkName"].Value)
                                    {
                                        case "Memory.Index.Value":
                                            int MemoryBankIndex = j + 1;
                                            NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, MemoryBankIndex.ToString());
                                            break;
                                        case "Memory.Size.Value":
                                            if (node_memorybank.ChildNodes.Count > 0) NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, node_memorybank.ChildNodes[j].Attributes["Size"].Value); else skip = true;
                                            break;
                                        case "Memory.Type.Value":
                                            if (node_memorybank.ChildNodes.Count > 0) NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, node_memorybank.ChildNodes[j].Attributes["Type"].Value); else skip = true;
                                            break;
                                        case "Memory.FormFactor.Value":
                                            if (node_memorybank.ChildNodes.Count > 0) NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, node_memorybank.ChildNodes[j].Attributes["FormFactor"].Value); else skip = true;
                                            break;
                                        case "Memory.Speed.Value":
                                            if (node_memorybank.ChildNodes.Count > 0) NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, node_memorybank.ChildNodes[j].Attributes["Speed"].Value); else skip = true;
                                            break;
                                        default:
                                            NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, "");
                                            break;
                                    }
                                    BookmarksinInsertString = Regex.Match(NewTableRows, patternAllBookmarks);
                                }

                                ReportString = ReportString.Insert(tableTREnd, NewTableRows);
                            }
                        }
                        ReportString = InsertBookmarkText(ReportString, Bookmark, strBookmarkTableStartText);
                        break;
                    //Extend the Storage table to fit the count of Storages;
                    case "Table.Storage.Start":
                        tableFirstTR = ReportString.IndexOf("<tr>", Bookmark.Index);
                        tableTREnd = ReportString.IndexOf("</table>", Bookmark.Index);
                        strInsert = ReportString.Substring(tableFirstTR, tableTREnd - tableFirstTR);

                        StartValuePosofBookmarkTableStart = ReportString.IndexOf(">", Bookmark.Index) + 1;
                        EndValuePosofBookmarkTableStart = ReportString.IndexOf("<", Bookmark.Index + 1);
                        strBookmarkTableStartText = ReportString.Substring(StartValuePosofBookmarkTableStart, EndValuePosofBookmarkTableStart - StartValuePosofBookmarkTableStart);

                        if (count_Storage > 1)
                        {
                            for (int j = count_Storage - 1; j >= 0; j--)
                            {
                                if (j > 0)
                                {
                                    string NewTableRows = strInsert;
                                    Match BookmarksinInsertString = Regex.Match(NewTableRows, patternAllBookmarks);

                                    while (BookmarksinInsertString.Success != false)
                                    {
                                        switch (BookmarksinInsertString.Groups["BookmarkName"].Value)
                                        {
                                            case "Storage.Index.Value":
                                                int StorageIndex = j + 1;
                                                NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, StorageIndex.ToString());
                                                break;
                                            case "Storage.ModelNumber.Value":
                                                if (StorageArray.Length > 0) NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, StorageArray[j].Attributes["ModelNumber"].Value); else skip = true;
                                                break;
                                            case "Storage.SerialNumber.Value":
                                                if (StorageArray.Length > 0) NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, StorageArray[j].Attributes["SerialNumber"].Value); else skip = true;
                                                break;
                                            case "Storage.MaxMediaSize.Value":
                                                if (StorageArray.Length > 0) NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, StorageArray[j].Attributes["MaxMediaSize"].Value); else skip = true;
                                                break;
                                            default:
                                                NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, "");
                                                break;
                                        }
                                        BookmarksinInsertString = Regex.Match(NewTableRows, patternAllBookmarks);
                                    }

                                    ReportString = ReportString.Insert(tableTREnd, NewTableRows);
                                }
                            }
                        }
                        ReportString = InsertBookmarkText(ReportString, Bookmark, strBookmarkTableStartText);
                        break;
                    //Extend the hardware modules table to fit the count of hardware modules;
                    case "Table.Hardware.Start":
                        tableFirstTR = ReportString.IndexOf("<tr>", Bookmark.Index);
                        tableTREnd = ReportString.IndexOf("</table>", Bookmark.Index);
                        strInsert = ReportString.Substring(tableFirstTR, tableTREnd - tableFirstTR);

                        StartValuePosofBookmarkTableStart = ReportString.IndexOf(">", Bookmark.Index) + 1;
                        EndValuePosofBookmarkTableStart = ReportString.IndexOf("<", Bookmark.Index + 1);
                        strBookmarkTableStartText = ReportString.Substring(StartValuePosofBookmarkTableStart, EndValuePosofBookmarkTableStart - StartValuePosofBookmarkTableStart);

                        for (int j = node_hardwaremodules.ChildNodes.Count - 1; j >= 0; j--)
                        {
                            if (j > 0)
                            {
                                string NewTableRows = strInsert;
                                Match BookmarksinInsertString = Regex.Match(NewTableRows, patternAllBookmarks);

                                while (BookmarksinInsertString.Success != false)
                                {
                                    switch (BookmarksinInsertString.Groups["BookmarkName"].Value)
                                    {
                                        case "Hardware.Index.Value":
                                            int HardwareModulesIndex = j + 1;
                                            NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, HardwareModulesIndex.ToString());
                                            break;
                                        case "Hardware.Vendor.Value":
                                            if (node_hardwaremodules.ChildNodes.Count > 0) NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, node_hardwaremodules.ChildNodes[j].Attributes["Vendor"].Value); else skip = true;
                                            break;
                                        case "Hardware.DeviceID.value":
                                            if (node_hardwaremodules.ChildNodes.Count > 0) NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, node_hardwaremodules.ChildNodes[j].Attributes["DeviceID"].Value); else skip = true;
                                            break;
                                        case "Hardware.SubvendorID.value":
                                            if (node_hardwaremodules.ChildNodes.Count > 0) NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, node_hardwaremodules.ChildNodes[j].Attributes["SubvendorID"].Value); else skip = true;
                                            break;
                                        case "Hardware.SubsystemID.value":
                                            if (node_hardwaremodules.ChildNodes.Count > 0) NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, node_hardwaremodules.ChildNodes[j].Attributes["SubsystemID"].Value); else skip = true;
                                            break;
                                        default:
                                            NewTableRows = InsertBookmarkText(NewTableRows, BookmarksinInsertString, "");
                                            break;
                                    }
                                    BookmarksinInsertString = Regex.Match(NewTableRows, patternAllBookmarks);
                                }

                                ReportString = ReportString.Insert(tableTREnd, NewTableRows);
                            }
                        }
                        ReportString = InsertBookmarkText(ReportString, Bookmark, strBookmarkTableStartText);
                        break;
                    //Processor (0)
                    case "Processor.Manufacturer.Value":
                        if (ProcessorArray.Length > 0) ReportString = InsertBookmarkText(ReportString, Bookmark, ProcessorArray[0].Attributes["Manufacturer"].Value); else skip = true;
                        break;
                    case "Processor.MaxSpeed.Value":
                        if (ProcessorArray.Length > 0) ReportString = InsertBookmarkText(ReportString, Bookmark, ProcessorArray[0].Attributes["MaxSocketSpeed"].Value); else skip = true;
                        break;
                    case "Processor.Identifier.Value":
                        if (ProcessorArray.Length > 0) ReportString = InsertBookmarkText(ReportString, Bookmark, ProcessorArray[0].Attributes["Identifier"].Value); else skip = true;
                        break;
                    case "Processor.CurrentSpeed.Value":
                        if (ProcessorArray.Length > 0) ReportString = InsertBookmarkText(ReportString, Bookmark, ProcessorArray[0].Attributes["CurrentSpeed"].Value); else skip = true;
                        break;
                    case "Processor.Version.Value":
                        if (ProcessorArray.Length > 0) ReportString = InsertBookmarkText(ReportString, Bookmark, ProcessorArray[0].Attributes["Version"].Value); else skip = true;
                        break;

                    //Memory (0)
                    case "Memory.Index.Value":
                        ReportString = InsertBookmarkText(ReportString, Bookmark, "1");
                        break;
                    case "Memory.Size.Value":
                        if (node_memorybank.ChildNodes.Count > 0) ReportString = InsertBookmarkText(ReportString, Bookmark, node_memorybank.ChildNodes[0].Attributes["Size"].Value); else skip = true;
                        break;
                    case "Memory.Type.Value":
                        if (node_memorybank.ChildNodes.Count > 0) ReportString = InsertBookmarkText(ReportString, Bookmark, node_memorybank.ChildNodes[0].Attributes["Type"].Value); else skip = true;
                        break;
                    case "Memory.FormFactor.Value":
                        if (node_memorybank.ChildNodes.Count > 0) ReportString = InsertBookmarkText(ReportString, Bookmark, node_memorybank.ChildNodes[0].Attributes["FormFactor"].Value); else skip = true;
                        break;
                    case "Memory.Speed.Value":
                        if (node_memorybank.ChildNodes.Count > 0) ReportString = InsertBookmarkText(ReportString, Bookmark, node_memorybank.ChildNodes[0].Attributes["Speed"].Value); else skip = true;
                        break;

                    //Storage (0)
                    case "Storage.Index.Value":
                        ReportString = InsertBookmarkText(ReportString, Bookmark, "1");
                        break;
                    case "Storage.ModelNumber.Value":
                        if (StorageArray.Length > 0) ReportString = InsertBookmarkText(ReportString, Bookmark, StorageArray[0].Attributes["ModelNumber"].Value); else skip = true;
                        break;
                    case "Storage.SerialNumber.Value":
                        if (StorageArray.Length > 0) ReportString = InsertBookmarkText(ReportString, Bookmark, StorageArray[0].Attributes["SerialNumber"].Value); else skip = true;
                        break;
                    case "Storage.MaxMediaSize.Value":
                        if (StorageArray.Length > 0) ReportString = InsertBookmarkText(ReportString, Bookmark, StorageArray[0].Attributes["MaxMediaSize"].Value); else skip = true;
                        break;

                    //Hardware Modules (0)
                    case "Hardware.Index.Value":
                        ReportString = InsertBookmarkText(ReportString, Bookmark, "1");
                        break;
                    case "Hardware.Vendor.Value":
                        if (node_hardwaremodules.ChildNodes.Count > 0) ReportString = InsertBookmarkText(ReportString, Bookmark, node_hardwaremodules.ChildNodes[0].Attributes["Vendor"].Value); else skip = true;
                        break;
                    case "Hardware.DeviceID.value":
                        if (node_hardwaremodules.ChildNodes.Count > 0) ReportString = InsertBookmarkText(ReportString, Bookmark, node_hardwaremodules.ChildNodes[0].Attributes["DeviceID"].Value); else skip = true;
                        break;
                    case "Hardware.SubvendorID.value":
                        if (node_hardwaremodules.ChildNodes.Count > 0) ReportString = InsertBookmarkText(ReportString, Bookmark, node_hardwaremodules.ChildNodes[0].Attributes["SubvendorID"].Value); else skip = true;
                        break;
                    case "Hardware.SubsystemID.value":
                        if (node_hardwaremodules.ChildNodes.Count > 0) ReportString = InsertBookmarkText(ReportString, Bookmark, node_hardwaremodules.ChildNodes[0].Attributes["SubsystemID"].Value); else skip = true;
                        break;

                    default:
                        ReportString = InsertBookmarkText(ReportString, Bookmark, "");
                        break;
                }

                if (skip == true) ReportString = InsertBookmarkText(ReportString, Bookmark, "---");
                Bookmark = Regex.Match(ReportString, patternAllBookmarks);
            }

            return (ReportString);
        }

        public static void SaveHardwareAssetToFile(AmtSystem computer, string filename, string currentWorkingDirectory)
        {
            if (filename.EndsWith(".xml"))
            {
                XmlTextWriter writer = new XmlTextWriter(filename, null);
                writer.Formatting = Formatting.Indented;
                HardwareAssetToXml(computer).Save(writer);
                writer.Close();
            }
            else if (filename.EndsWith(".csv"))
            {
                AmtHardwareComputerSystem amtComputer = computer.Assets.GetComputerSystem();
                AmtHardwareProcessor[] proc = computer.Assets.GetProcessors();
                AmtHardwareBios bios = computer.Assets.GetBios();
                AmtHardwareMediaDevice[] media = computer.Assets.GetMediaDevices();
                AmtHardwareModule[] hdmodule = computer.Assets.GetHardwareModules();
                AmtHardwareMemoryModule[] module = computer.Assets.GetMemoryModules();

                //<Computer Name>, <System GUID>, <CPU Type>, <CPU Speed>, <BIOS Version>, <Total Memory>, <Disk1 Name>, <Disk1 Size>, <Disk2 Name>, <Disk2 Size>, <Disk3 Name>, <Disk3 Size>, <Number of hardware Modules>, etc.
                FileStream csvfile = File.Open(filename, FileMode.Append);
                if (csvfile.Length == 0)
                {
                    AddText(csvfile, "<Computer Name>, <System GUID>, <CPU Type>, <CPU Max Speed>, <BIOS Version>, <Total Memory>, <Disk1 Name>, <Disk1 Size>, <Disk2 Name>, <Disk2 Size>, <Disk3 Name>, <Disk3 Size>, <Number of hardware Modules>\n");
                }

                AddText(csvfile, computer.HostNameIdentity.DefaultName + ", ");
                AddText(csvfile, amtComputer.UUID.ToString() + ", ");
                AddText(csvfile, proc[0].Type.ToString() + ", ");
                AddText(csvfile, proc[0].MaxSocketSpeed.ToString() + "MHZ" + ", ");
                AddText(csvfile, bios.Version + ", ");

                uint TotalMemory = 0;
                for (int i = 0; i < module.Length; i++) TotalMemory += (uint)(module[i].Size);
                AddText(csvfile, TotalMemory.ToString() + "MB" + ", ");

                for (int i = 0; i < 3; i++)
                {
                    AddText(csvfile, media[i].ModelNumber + ", ");
                    AddText(csvfile, media[i].MaxMediaSizeLStr + ", ");
                }

                AddText(csvfile, hdmodule.Length.ToString() + "\n");
                csvfile.Close();
            }
            else if (filename.EndsWith(".htm") || filename.EndsWith(".html"))
            {
                string TemplateFileName = null;
                if (File.Exists(currentWorkingDirectory + "\\AssetReportTemplate-" + System.Threading.Thread.CurrentThread.CurrentUICulture + ".htm") == true)
                {
                    // Use culture template
                    TemplateFileName = currentWorkingDirectory + "\\AssetReportTemplate-" + System.Threading.Thread.CurrentThread.CurrentUICulture + ".htm";
                }
                else if (File.Exists(currentWorkingDirectory + "\\AssetReportTemplate.htm") == true)
                {
                    // Use default template
                    TemplateFileName = currentWorkingDirectory + "\\AssetReportTemplate.htm";
                }

                if (TemplateFileName != null)
                {
                    Encoding fileEncoding = HTMLFileEncoding.GetEncoding(TemplateFileName);
                    StreamWriter streamWriter = new StreamWriter(filename, false, fileEncoding);
                    streamWriter.Write(HardwareAssetToHtml(computer, TemplateFileName));
                    streamWriter.Flush();
                    streamWriter.Close();
                }
            }
        }

        public static string FilterStrHtml(string s)
        {
            if (s == null) return null;
            /*
            if (System.Threading.Thread.CurrentThread.CurrentUICulture.ToString().StartsWith("en"))
            {
                s = s.Replace("?", "(TM)").Replace("?", "&#174;").Replace("?", "&#169;").Trim();
            }
            */
            return s;
        }

    }
}
