// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Net;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class RemoteAccessServerForm : Form
    {
        private AmtSystem system;
        private AmtRemoteAccessAdmin.AmtRemoteAccessServer EditServer = null;

        public RemoteAccessServerForm(AmtSystem system)
        {
            this.system = system;
            InitializeComponent();
        }

        private void UpdateInfo()
        {
            serverNameTextBox.Enabled = radioButton1.Checked;
            serverNameTextBox.Text = serverNameTextBox.Text.Trim();
            serverIpTextBox.Enabled = radioButton2.Checked;
            serverIpTextBox.Text = serverIpTextBox.Text.Trim();
            serverCommonNameTextBox.Enabled = radioButton2.Checked;
            serverCommonNameTextBox.Text = serverCommonNameTextBox.Text.Trim();
            usernameTextBox.Enabled = radioButton3.Checked;
            passwordControl.Enabled = radioButton3.Checked;
            clientCertificateComboBox.Enabled = radioButton4.Checked;

            bool ok = true;

            if (radioButton1.Checked)
            {
                if (serverNameTextBox.Text.Length == 0) ok = false;
            }
            else
            {
                if (serverIpTextBox.Text.Length == 0) ok = false;
                if (serverCommonNameTextBox.Text.Length == 0) ok = false;
                IPAddress addr;
                if (IPAddress.TryParse(serverIpTextBox.Text, out addr) == false) ok = false;
            }

            if (radioButton3.Checked)
            {
                if (usernameTextBox.Text.Length == 0) ok = false;
                if (Utils.CheckPassPhrase(passwordControl.Text) == false) ok = false;
            }
            else
            {

            }

            okButton.Enabled = ok;
        }

        private void RemoteAccessServerForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            system.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);

            // Add client certificate list
            AmtSecurityAdmin.AmtStoredCert[] certs = null;
            if (system.SecurityAdmin != null && system.SecurityAdmin.CertStoreGetCertificates(out certs) == AmtCallStatus.SUCCESS) clientCertificateComboBox.Items.AddRange(certs);
            if (clientCertificateComboBox.Items.Count > 0)
            {
                clientCertificateComboBox.Enabled = true;
                clientCertificateComboBox.SelectedIndex = 0;
                radioButton4.Enabled = true;
            }
            else
            {
                clientCertificateComboBox.Enabled = false;
                radioButton4.Enabled = false;
            }

            // Select the correct existing certificate
            if (certs != null && EditServer != null)
            {
                foreach (AmtSecurityAdmin.AmtStoredCert c in certs)
                {
                    if (c.Handle == EditServer.MutualAuthCertHandle) clientCertificateComboBox.SelectedItem = c;
                }
            }

            UpdateInfo();
        }

        private void SystemStateChangedHandler(AmtSystem computer)
        {
            if (computer.State != AmtSystemObjState.Connected) DialogResult = DialogResult.Cancel;
        }

        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        public void SetServer(AmtRemoteAccessAdmin.AmtRemoteAccessServer server)
        {
            EditServer = server;

            // Setup the server address
            if (server.ServerName != null)
            {
                radioButton1.Checked = true;
                radioButton2.Checked = false;
                serverNameTextBox.Text = server.ServerName;
            }
            else if (server.ServerAddress != null)
            {
                radioButton1.Checked = false;
                radioButton2.Checked = true;
                serverCommonNameTextBox.Text = server.ServerCommonName;
                serverIpTextBox.Text = server.ServerAddress.ToString();
            }

            // Setup server port
            portNumericUpDown.Value = server.Port;

            // Select the correct certificate or username/password
            if (server.Username == null)
            {
                radioButton3.Checked = false;
                radioButton4.Checked = true;
                foreach (AmtSecurityAdmin.AmtStoredCert cert in clientCertificateComboBox.Items)
                {
                    if (cert.Handle == server.MutualAuthCertHandle)
                    {
                        clientCertificateComboBox.SelectedItem = cert;
                        break;
                    }
                }
            }
            else
            {
                radioButton3.Checked = true;
                radioButton4.Checked = false;
                usernameTextBox.Text = server.Username;
                if (server.Password != null) passwordControl.Text = server.Password;
            }
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            AmtRemoteAccessAdmin.AmtRemoteAccessServer server = new AmtRemoteAccessAdmin.AmtRemoteAccessServer();
            
            if (radioButton1.Checked)
            {
                server.ServerName = serverNameTextBox.Text;
            }
            else
            {
                server.ServerCommonName = serverCommonNameTextBox.Text;
                server.ServerAddress = System.Net.IPAddress.Parse(serverIpTextBox.Text);
            }
            
            if (radioButton3.Checked)
            {
                server.Username = usernameTextBox.Text;
                server.Password = passwordControl.Text;
            }
            else
            {
                server.MutualAuthCert = (AmtSecurityAdmin.AmtStoredCert)clientCertificateComboBox.SelectedItem;
                server.MutualAuthCertHandle = server.MutualAuthCert.Handle;
            }

            server.Port = (ushort)(portNumericUpDown.Value);

            AmtCallStatus r;
            if (EditServer == null)
            {
                r = system.RemoteAccessAdmin.AddRemoteServer(server);
            }
            else
            {
                server.Handle = EditServer.Handle;

                if (system.WsMan != null)
                {
                    // Copy the corresponding WSMAN references.
                    server.wsmanContext = EditServer.wsmanContext;
                }

                r = system.RemoteAccessAdmin.UpdateRemoteServer(server);
            }

            if (r == AmtCallStatus.SUCCESS) DialogResult = DialogResult.OK; else Utils.ErrorMessageBox(this, "AddRemoteServer", r);
        }

        private void serverNameTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void RemoteAccessServerForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            system.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }
    }
}
