// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class StorageApplicationForm : Form
    {
        private AmtStorage storage;

        public StorageApplicationForm(AmtStorage storage)
        {
            InitializeComponent();

            this.storage = storage;
            UpdateView();
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
        }

        private void UpdateView()
        {
            appsListView.Items.Clear();
            AmtStorageAlloc[] allocations = storage.GetStorageAllocations();
            if (allocations == null || allocations.Length == 0) return;

            foreach (AmtStorageAlloc a in allocations)
            {
                ListViewItem l = new ListViewItem(new string[] { a.SnrpEntryVendorName, a.SnrpEntryApplicationName, string.Format(Properties.Resources.KiloBytes, (a.TotalAllocationSize / 1024).ToString()) });
                l.Tag = a;
                appsListView.Items.Add(l);
            }
        }

        private void addButton_Click(object sender, EventArgs e)
        {
            uint a;
            if (uint.TryParse(allocTextBox.Text, out a) == false) return;
            storage.AddStorageFpaclEntry(vendorTextBox.Text, applicationTextBox.Text, a * 4096);
            UpdateView();
        }

        private void appsListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            removeButton.Enabled = (appsListView.SelectedItems.Count == 1);
        }

        private void removeButton_Click(object sender, EventArgs e)
        {
            if (appsListView.SelectedItems.Count != 1) return;
            ListViewItem l = appsListView.SelectedItems[0];
            AmtStorageAlloc a = (AmtStorageAlloc)l.Tag;
            storage.RemoveStorageFpaclEntry(a);
            UpdateView();
        }

        private void StorageApplicationForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
        }
    }
}
