// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class StorageEnterpriseForm : Form
    {
        private AmtStorage storage;

        public StorageEnterpriseForm(AmtStorage storage)
        {
            InitializeComponent();

            this.storage = storage;
            UpdateView();
        }

        private void addButton_Click(object sender, EventArgs e)
        {
            storage.AddEnterpriseEntry(enterpriseTextBox.Text);
            UpdateView();
        }

        private void UpdateView()
        {
            enterprisesListView.Items.Clear();
            AmtEnterprise[] enterprises = storage.GetEnterpriseEntries();
            if (enterprises == null || enterprises.Length == 0) return;

            foreach (AmtEnterprise e in enterprises)
            {
                ListViewItem l = new ListViewItem(new string[] { e.Name });
                l.Tag = e;
                enterprisesListView.Items.Add(l);
            }
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;
        }

        private void enterprisesListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            removeButton.Enabled = (enterprisesListView.SelectedItems.Count == 1);
        }

        private void removeButton_Click(object sender, EventArgs e)
        {
            if (enterprisesListView.SelectedItems.Count != 1) return;
            ListViewItem l = enterprisesListView.SelectedItems[0];
            AmtEnterprise enterprise = (AmtEnterprise)l.Tag;
            storage.RemoveEnterpriseEntry(enterprise);
            UpdateView();
        }

        private void StorageEnterpriseForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
        }

        private void enterprisesListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete) { removeButton_Click(this, null); e.Handled = true; }
        }
    }
}
