// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class UserPermissionsForm : Form
    {
        private bool ReadOnly = false;
        private AmtSecurityAdmin.AmtUserAcl User;
        private string UserAccountString;
        private AmtSystem system;
        public string AdminName = null;

        public UserPermissionsForm(AmtSecurityAdmin.AmtUserAcl user, AmtSystem system)
        {
            this.system = system;
            if (user.Username != null && user.Username.StartsWith("$$"))
                ReadOnly = true;
            else
                ReadOnly = false; 
            User = user;
            InitializeComponent();
            UserAccountString = label4.Text;
        }

        private void AddItem(string name, int tag)
        {
            ListViewItem l = new ListViewItem(name);
            l.Tag = tag;
            permissionsListView.Items.Add(l);
        }

        private void UpdateInfo()
        {
            bool addOk = true;
            if (accountUsernameTextBox.Text.StartsWith("$$")) addOk = false;
            if (string.Compare(accountUsernameTextBox.Text, "admin", true) == 0) addOk = false;
            if (AdminName != null && string.Compare(accountUsernameTextBox.Text, AdminName, true) == 0) addOk = false;
            if (kerberosCheckBox.Checked == false && passwordControl.Text.Length == 0) addOk = false; 
            if (kerberosCheckBox.Checked == true)
            {
                byte[] b = Utils.HexToBytes(accountUsernameTextBox.Text);
                if (b == null) addOk = false;
                else if (b.Length == 0 || b.Length > 28 || b.Length % 4 != 0) addOk = false;
            }

            // One permission must be set
            bool perm = false;
            foreach (ListViewItem l in permissionsListView.Items)
            {
                if (l.Checked == true) { perm = true; break; }
            }
            if (perm == false) addOk = false;

            okButton.Enabled = addOk;
        }

        private void SetAccessPrivilage(AmtSecurityAdmin.AmtUserAclRealm r, bool access)
        {
            foreach (ListViewItem l in permissionsListView.Items)
            {
                if ((int)(l.Tag) == (int)r) {l.Checked = access;return;}
            }
        }

        private void UserPermissionsForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            
            if (ReadOnly)
            {
                accountUsernameTextBox.ReadOnly = true;
                passwordControl.Enabled = false;
                accessPermissionComboBox.Enabled = false;
                kerberosCheckBox.Enabled = false;
                selectAllButton.Visible = false;
                selectNoneButton.Visible = false;
                inverseButton.Visible = false;
                permissionsListView.Height += selectAllButton.Height;
                Height -= selectAllButton.Height;
                okButton.Visible = false;
                cancelButton.Text = Properties.Resources.Close;
            }
            bool ro = ReadOnly;
            ReadOnly = false;
            selectNoneButton_Click(this, null);
            SetAccessPermissionType(User.AccessPermission);
            accessPermissionComboBox_SelectedIndexChanged(this, null);
            foreach (AmtSecurityAdmin.AmtUserAclRealm r in User.Realms) SetAccessPrivilage(r, true);
            ReadOnly = ro;

            if (User.Type == AmtSecurityAdmin.AmtUserAclType.Digest)
            {
                // Digest
                kerberosCheckBox.Checked = false;
                accountUsernameTextBox.Text = User.Username;
                passwordControl.Text = User.Password;
            }
            else
            {
                // Kerberos*
                kerberosCheckBox.Checked = true;
                accountUsernameTextBox.Text = Utils.BytesToHex(User.KerberosSid);
                passwordControl.Text = "";
            }

            if (system != null && accountUsernameTextBox.Text.Length != 0)
            {
                kerberosCheckBox.Enabled = false;
                accountUsernameTextBox.ReadOnly = true;
            }
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;
        }

        private void selectAllButton_Click(object sender, EventArgs e)
        {
            permissionsListView.BeginUpdate();
            foreach (ListViewItem l in permissionsListView.Items)
            {
                if ((int)(l.Tag) != 20) l.Checked = true;
            }
            permissionsListView.EndUpdate();
        }

        private void selectNoneButton_Click(object sender, EventArgs e)
        {
            permissionsListView.BeginUpdate();
            foreach (ListViewItem l in permissionsListView.Items) l.Checked = false;
            permissionsListView.EndUpdate();
        }

        private void inverseButton_Click(object sender, EventArgs e)
        {
            permissionsListView.BeginUpdate();
            foreach (ListViewItem l in permissionsListView.Items)
            {
                if ((int)(l.Tag) != 20) l.Checked = !l.Checked;
            }
            permissionsListView.EndUpdate();
        }

        private void kerberosCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (kerberosCheckBox.Checked)
            {
                label4.Text = Properties.Resources.KerberosSidHex;
                passwordControl.Enabled = false;
            }
            else
            {
                label4.Text = UserAccountString;
                passwordControl.Enabled = true;
            }
            UpdateInfo();
        }

        public AmtSecurityAdmin.AmtUserAcl GetAcl()
        {
            ArrayList userAclRealm = new ArrayList();
            foreach (ListViewItem l in permissionsListView.Items) { if (l.Checked == true) userAclRealm.Add((AmtSecurityAdmin.AmtUserAclRealm)l.Tag); }
            AmtSecurityAdmin.AmtUserAclRealm[] realms = (AmtSecurityAdmin.AmtUserAclRealm[])userAclRealm.ToArray(typeof(AmtSecurityAdmin.AmtUserAclRealm));

            AmtSecurityAdmin.AmtUserAcl user;
            if (kerberosCheckBox.Checked == false)
            {
                // Digest
                user = new AmtSecurityAdmin.AmtUserAcl(accountUsernameTextBox.Text, passwordControl.Text, GetAccessPermissionType(), realms);
            }
            else
            {
                // Kerberos*
                user = new AmtSecurityAdmin.AmtUserAcl(Utils.HexToBytes(accountUsernameTextBox.Text), GetAccessPermissionType(), realms);
            }

            if (User != null)
            {
                user.Handle = User.Handle;
                user.HandleSet = User.HandleSet;
                if (user.Password != null && user.Password.Length == 0) user.Password = null;
            }

            return user;
        }

        private AccessPermissionType GetAccessPermissionType()
        {
            if (accessPermissionComboBox.SelectedIndex == 0) return AccessPermissionType.AnyAccessPermission;
            if (accessPermissionComboBox.SelectedIndex == 1) return AccessPermissionType.LocalAccessPermission;
            return AccessPermissionType.NetworkAccessPermission;
        }

        private void SetAccessPermissionType(AccessPermissionType ap)
        {
            if (ap == AccessPermissionType.AnyAccessPermission) accessPermissionComboBox.SelectedIndex = 0;
            if (ap == AccessPermissionType.LocalAccessPermission) accessPermissionComboBox.SelectedIndex = 1;
            if (ap == AccessPermissionType.NetworkAccessPermission) accessPermissionComboBox.SelectedIndex = 2;
        }

        private void accountUsernameTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private bool permissionsListViewUpdate = false;
        private void permissionsListView_ItemChecked(object sender, ItemCheckedEventArgs e)
        {
            if (ReadOnly == true)
            {
                if (permissionsListViewUpdate == false)
                {
                    permissionsListViewUpdate = true;
                    e.Item.Checked = !e.Item.Checked;
                    permissionsListViewUpdate = false;
                }
            }
            else
            {
                UpdateInfo();
            }
        }

        private void accessPermissionComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Save Realms
            ArrayList userAclRealm = new ArrayList();
            foreach (ListViewItem l in permissionsListView.Items) { if (l.Checked == true) userAclRealm.Add((AmtSecurityAdmin.AmtUserAclRealm)l.Tag); }
            AmtSecurityAdmin.AmtUserAclRealm[] realms = (AmtSecurityAdmin.AmtUserAclRealm[])userAclRealm.ToArray(typeof(AmtSecurityAdmin.AmtUserAclRealm));

            // Update the list
            AccessPermissionType ap = GetAccessPermissionType();
            permissionsListView.BeginUpdate();
            permissionsListView.Items.Clear();
            if (ap == AccessPermissionType.LocalAccessPermission || ap == AccessPermissionType.AnyAccessPermission)
            {
                // Local only realms
                AddItem("Agent Presence Local", 9);
                AddItem("Firmware Update", 14);
                //AddItem("Embedded IT", 15);
                AddItem("Local User Notification", 16);
                AddItem("Endpoint Access Control", 17);
                //AddItem("Wake-on-Event", 22);
            }
            if (ap == AccessPermissionType.NetworkAccessPermission || ap == AccessPermissionType.AnyAccessPermission)
            {
                // Remote only realms
                AddItem("Redirection", 2);
                AddItem("User Access Administration", 3);
                AddItem("Hardware Asset", 4);
                AddItem("Remote Control", 5);
                AddItem("Event Manager", 7);
                AddItem("Storage Admin", 8);
                AddItem("Agent Presence Remote", 10);
                AddItem("Circuit Breaker", 11);
                AddItem("Network Time", 12);
                AddItem("Endpoint Access Control Admin", 18);
                AddItem("Security Audit Log", 20);
            }
            // Local and remote realms
            AddItem("Storage", 6);
            AddItem("General Info", 13);
            AddItem("Event Log Reader", 19);
            AddItem("User Access Control", 21);
            //AddItem("Disk Encryption", 23);

            // Load realms
            foreach (AmtSecurityAdmin.AmtUserAclRealm r in realms) SetAccessPrivilage(r, true);
            permissionsListView.EndUpdate();
        }

    }
}
