// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class WirelessInterfaceManagementForm : Form
    {
        private AmtSystem system;
        private InterfaceModeType ifmode;

        public WirelessInterfaceManagementForm(AmtSystem system)
        {
            this.system = system;
            InitializeComponent();
        }

        private void WirelessInterfaceManagementForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            InterfaceDescriptorGetType interfaceDescriptor = system.Network.Wireless.GetInterfaceSettings();
            if (interfaceDescriptor != null)
            {
                ifmode = interfaceDescriptor.InterfaceMode;
                checkBox1.Checked = ((interfaceDescriptor.LinkPolicy & 0x01) == 0x01);
                checkBox2.Checked = ((interfaceDescriptor.LinkPolicy & 0x0E) == 0x0E);
                checkBox3.Checked = ((interfaceDescriptor.LinkPolicy & 0x10) == 0x10);
                checkBox4.Checked = ((interfaceDescriptor.LinkPolicy & 0xE0) == 0xE0);
            }
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            InterfaceDescriptorSetType st = new InterfaceDescriptorSetType();
            byte link = 0;
            if (checkBox1.Checked) link |= 0x01;
            if (checkBox2.Checked) link |= 0x0E;
            if (checkBox3.Checked) link |= 0x10;
            if (checkBox4.Checked) link |= 0xE0;
            st.LinkPolicy = link;
            st.IPv4Parameters = null;
            st.InterfaceMode = ifmode;
            AmtCallStatus r = system.Network.Wireless.SetInterfaceSettings(st);
            if (r != AmtCallStatus.SUCCESS)
            {
                MessageBox.Show(this, string.Format(Properties.Resources.SetInterfaceSettingsError, r.ToString()), Properties.Resources.InterfaceError, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                DialogResult = DialogResult.OK;
            }
        }
    }
}
