// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class WsmanSettingForm : Form
    {
        private AmtSystem system;
        private bool wsmanonly;

        public WsmanSettingForm(AmtSystem system)
        {
            this.system = system;
            InitializeComponent();
        }

        private void WsmanSettingForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            system.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);

            wsmanonly = system.SecurityAdmin.GetWsmanOnly();
            if (wsmanonly)
            {
                wsmanRadioButton.Checked = true;
            }
            else
            {
                dualRadioButton.Checked = true;
            }
            UpdateInfo();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            if (system.SecurityAdmin.SetWsmanOnly(wsmanRadioButton.Checked) == true)
            {
                system.Disconnect();
                system.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
                DialogResult = DialogResult.OK;
            }
        }

        private void UpdateInfo()
        {
            okButton.Enabled = (wsmanonly && dualRadioButton.Checked) || (!wsmanonly && wsmanRadioButton.Checked);
        }

        private void wsmanRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void dualRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void SystemStateChangedHandler(AmtSystem system)
        {
            if (system.State != AmtSystemObjState.Connected)
            {
                system.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
                DialogResult = DialogResult.Cancel;
            }
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            system.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
            DialogResult = DialogResult.Cancel;
        }

        private void WsmanSettingForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            system.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }
    }
}
