// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class xProfileEditForm : Form
    {
        private AmtSystem computer;
        private AmtNetworkAdminInterface inf;
        private uint selectedClientCertificateHandle = 0;
        private uint selectedIssuerCertificateHandle = 0;

        public xProfileEditForm(AmtSystem computer, AmtNetworkAdminInterface inf)
        {
            this.computer = computer;
            this.inf = inf;
            InitializeComponent();
            profileTypeComboBox.SelectedIndex = 0;
            issuerCertificateComboBox.SelectedIndex = 0;
            serverIdentityCheckComboBox.SelectedIndex = 0;
        }

        private void xProfileEditForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
        }

        private void profileTypeComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            clientCertificateComboBox.Items.Clear();
            if (profileTypeComboBox.SelectedIndex != 1 && profileTypeComboBox.SelectedIndex != 7)
            {
                clientCertificateComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.NotSpecified, 0, null));
                clientCertificateComboBox.SelectedIndex = 0;
            }
            
            // Add client certificate list
            AmtSecurityAdmin.AmtStoredCert[] certs;
            if (computer.SecurityAdmin.CertStoreGetCertificates(out certs) == AmtCallStatus.SUCCESS)
            {
                clientCertificateComboBox.Items.AddRange(certs);
            }
            SetClientCertificate(selectedClientCertificateHandle, true);
            
            // Add issuer certificate list
            issuerCertificateComboBox.Items.Clear();
            issuerCertificateComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.NotSpecified, 0, null));
            issuerCertificateComboBox.SelectedIndex = 0;
            AmtSecurityAdmin.AmtStoredCert[] tcerts;
            if (computer.SecurityAdmin.CertStoreGetTrustedCertificates(out tcerts) == AmtCallStatus.SUCCESS)
            {
                issuerCertificateComboBox.Items.AddRange(tcerts);
            }
            SetIssuerCertificate(selectedIssuerCertificateHandle, true);

            roamingIdentityPanel.Visible = false;
            usernamePanel.Visible = false;
            clientIdentityPanel.Visible = false;
            serverIdentityPanel.Visible = false;
            userCredentialsPanel.Visible = false;
            pacPanel.Visible = false;

            switch (profileTypeComboBox.SelectedIndex)
            {
                case 1:
                {
                    roamingIdentityPanel.Visible = false;
                    usernamePanel.Visible = true;
                    clientIdentityPanel.Visible = true;
                    serverIdentityPanel.Visible = true;
                    userCredentialsPanel.Visible = false;
                    pacPanel.Visible = false;
                    break;
                }
                case 2:
                case 3:
                {
                    roamingIdentityPanel.Visible = true;
                    usernamePanel.Visible = false;
                    clientIdentityPanel.Visible = true;
                    serverIdentityPanel.Visible = true;
                    userCredentialsPanel.Visible = true;
                    pacPanel.Visible = false;
                    break;
                }
                case 4:
                {
                    roamingIdentityPanel.Visible = false;
                    usernamePanel.Visible = false;
                    clientIdentityPanel.Visible = false;
                    serverIdentityPanel.Visible = false;
                    userCredentialsPanel.Visible = true;
                    pacPanel.Visible = false;
                    break;
                }
                case 5:
                {
                    roamingIdentityPanel.Visible = true;
                    usernamePanel.Visible = false;
                    clientIdentityPanel.Visible = true;
                    serverIdentityPanel.Visible = true;
                    userCredentialsPanel.Visible = true;
                    pacPanel.Visible = true;
                    break;
                }
                case 6:
                {
                    roamingIdentityPanel.Visible = true;
                    usernamePanel.Visible = false;
                    clientIdentityPanel.Visible = true;
                    serverIdentityPanel.Visible = true;
                    userCredentialsPanel.Visible = true;
                    pacPanel.Visible = true;
                    break;
                }
                case 7:
                {
                    roamingIdentityPanel.Visible = true;
                    usernamePanel.Visible = false;
                    clientIdentityPanel.Visible = true;
                    serverIdentityPanel.Visible = true;
                    userCredentialsPanel.Visible = true;
                    pacPanel.Visible = true;
                    break;
                }
            }
        }

        public XProfileType GetProfile()
        {
            XProfileType xProfile = null;

            switch (profileTypeComboBox.SelectedIndex)
            {
                case 0:
                {
                    break;
                }
                case 1:
                {
                    XProfileTLSType t = new XProfileTLSType();

                    t.Username = usernameTextBox.Text;

                    // Set client certificate
                    if (clientCertificateComboBox.SelectedIndex >= 0)
                    {
                        AmtSecurityAdmin.AmtStoredCert cert = (AmtSecurityAdmin.AmtStoredCert)clientCertificateComboBox.SelectedItem;
                        if (cert != null) t.ClientCertificate = cert.Handle;
                    }

                    t.ServerIdentity = new ServerIdentityType();

                    // Set issuer certificate
                    t.ServerIdentity.CertificateIssuer = 0;
                    t.ServerIdentity.CertificateIssuerSpecified = false;
                    if (issuerCertificateComboBox.SelectedIndex > 0)
                    {
                        AmtSecurityAdmin.AmtStoredCert tcert = (AmtSecurityAdmin.AmtStoredCert)issuerCertificateComboBox.SelectedItem;
                        if (tcert != null)
                        {
                            t.ServerIdentity.CertificateIssuer = tcert.Handle;
                            t.ServerIdentity.CertificateIssuerSpecified = true;
                        }
                    }

                    t.ServerIdentity.ServerNameOptions = null;
                    if (serverIdentityCheckComboBox.SelectedIndex > 0)
                    {
                        t.ServerIdentity.ServerNameOptions = new CertificateNameType();
                        if (serverIdentityCheckComboBox.SelectedIndex == 1) t.ServerIdentity.ServerNameOptions.Options = CertNameOptionsType.CertNameMatch;
                        if (serverIdentityCheckComboBox.SelectedIndex == 2) t.ServerIdentity.ServerNameOptions.Options = CertNameOptionsType.DomainSuffixMatch;
                        t.ServerIdentity.ServerNameOptions.ServerOrCertName = serverCheckTextBox.Text;
                    }
                    
                    xProfile = new XProfileType();
                    xProfile.Item = (object)t;
                    break;
                }
                case 2:
                {
                    XProfileTTLS_MSCHAPv2Type t = new XProfileTTLS_MSCHAPv2Type();

                    t.RoamingIdentity = null;
                    if (roamingIdentityTextBox.Text.Length > 0) t.RoamingIdentity = roamingIdentityTextBox.Text;

                    if (clientCertificateComboBox.SelectedIndex > 0)
                    {
                        AmtSecurityAdmin.AmtStoredCert cert = (AmtSecurityAdmin.AmtStoredCert)clientCertificateComboBox.SelectedItem;
                        if (cert != null)
                        {
                            t.ClientCertificate = cert.Handle;
                            t.ClientCertificateSpecified = true;
                        }
                    }

                    t.UserCredentials = new UserCredentialsType();
                    t.UserCredentials.Username = ucUsernameTextBox.Text;
                    t.UserCredentials.Password = ucPasswordMaskedTextBox.Text;
                    if (ucDomainTextBox.Text.Length > 0) t.UserCredentials.Domain = ucDomainTextBox.Text;

                    t.ServerIdentity = new ServerIdentityType();

                    // Set issuer certificate
                    t.ServerIdentity.CertificateIssuer = 0;
                    t.ServerIdentity.CertificateIssuerSpecified = false;
                    if (issuerCertificateComboBox.SelectedIndex > 0)
                    {
                        AmtSecurityAdmin.AmtStoredCert tcert = (AmtSecurityAdmin.AmtStoredCert)issuerCertificateComboBox.SelectedItem;
                        if (tcert != null)
                        {
                            t.ServerIdentity.CertificateIssuer = tcert.Handle;
                            t.ServerIdentity.CertificateIssuerSpecified = true;
                        }
                    }

                    t.ServerIdentity.ServerNameOptions = null;
                    if (serverIdentityCheckComboBox.SelectedIndex > 0)
                    {
                        t.ServerIdentity.ServerNameOptions = new CertificateNameType();
                        if (serverIdentityCheckComboBox.SelectedIndex == 1) t.ServerIdentity.ServerNameOptions.Options = CertNameOptionsType.CertNameMatch;
                        if (serverIdentityCheckComboBox.SelectedIndex == 2) t.ServerIdentity.ServerNameOptions.Options = CertNameOptionsType.DomainSuffixMatch;
                        t.ServerIdentity.ServerNameOptions.ServerOrCertName = serverCheckTextBox.Text;
                    }

                    xProfile = new XProfileType();
                    xProfile.Item = (object)t;
                    break;
                }
                case 3:
                {
                    XProfilePEAP_MSCHAPv2Type t = new XProfilePEAP_MSCHAPv2Type();

                    t.RoamingIdentity = null;
                    if (roamingIdentityTextBox.Text.Length > 0) t.RoamingIdentity = roamingIdentityTextBox.Text;

                    if (clientCertificateComboBox.SelectedIndex > 0)
                    {
                        AmtSecurityAdmin.AmtStoredCert cert = (AmtSecurityAdmin.AmtStoredCert)clientCertificateComboBox.SelectedItem;
                        if (cert != null)
                        {
                            t.ClientCertificate = cert.Handle;
                            t.ClientCertificateSpecified = true;
                        }
                    }

                    t.UserCredentials = new UserCredentialsType();
                    t.UserCredentials.Username = ucUsernameTextBox.Text;
                    t.UserCredentials.Password = ucPasswordMaskedTextBox.Text;
                    if (ucDomainTextBox.Text.Length > 0) t.UserCredentials.Domain = ucDomainTextBox.Text;

                    t.ServerIdentity = new ServerIdentityType();

                    // Set issuer certificate
                    t.ServerIdentity.CertificateIssuer = 0;
                    t.ServerIdentity.CertificateIssuerSpecified = false;
                    if (issuerCertificateComboBox.SelectedIndex > 0)
                    {
                        AmtSecurityAdmin.AmtStoredCert tcert = (AmtSecurityAdmin.AmtStoredCert)issuerCertificateComboBox.SelectedItem;
                        if (tcert != null)
                        {
                            t.ServerIdentity.CertificateIssuer = tcert.Handle;
                            t.ServerIdentity.CertificateIssuerSpecified = true;
                        }
                    }

                    t.ServerIdentity.ServerNameOptions = null;
                    if (serverIdentityCheckComboBox.SelectedIndex > 0)
                    {
                        t.ServerIdentity.ServerNameOptions = new CertificateNameType();
                        if (serverIdentityCheckComboBox.SelectedIndex == 1) t.ServerIdentity.ServerNameOptions.Options = CertNameOptionsType.CertNameMatch;
                        if (serverIdentityCheckComboBox.SelectedIndex == 2) t.ServerIdentity.ServerNameOptions.Options = CertNameOptionsType.DomainSuffixMatch;
                        t.ServerIdentity.ServerNameOptions.ServerOrCertName = serverCheckTextBox.Text;
                    }

                    xProfile = new XProfileType();
                    xProfile.Item = (object)t;
                    break;
                }
                case 4:
                {
                    XProfileEAP_GTCType t = new XProfileEAP_GTCType();

                    t.UserCredentials = new UserCredentialsType();
                    t.UserCredentials.Username = ucUsernameTextBox.Text;
                    t.UserCredentials.Password = ucPasswordMaskedTextBox.Text;
                    if (ucDomainTextBox.Text.Length > 0) t.UserCredentials.Domain = ucDomainTextBox.Text;

                    xProfile = new XProfileType();
                    xProfile.Item = (object)t;
                    break;
                }
                case 5:
                {
                    XProfileEAPFAST_MSCHAPv2Type t = new XProfileEAPFAST_MSCHAPv2Type();

                    t.RoamingIdentity = null;
                    if (roamingIdentityTextBox.Text.Length > 0) t.RoamingIdentity = roamingIdentityTextBox.Text;

                    if (clientCertificateComboBox.SelectedIndex > 0)
                    {
                        AmtSecurityAdmin.AmtStoredCert cert = (AmtSecurityAdmin.AmtStoredCert)clientCertificateComboBox.SelectedItem;
                        if (cert != null)
                        {
                            t.ClientCertificate = cert.Handle;
                            t.ClientCertificateSpecified = true;
                        }
                    }

                    t.UserCredentials = new UserCredentialsType();
                    t.UserCredentials.Username = ucUsernameTextBox.Text;
                    t.UserCredentials.Password = ucPasswordMaskedTextBox.Text;
                    if (ucDomainTextBox.Text.Length > 0) t.UserCredentials.Domain = ucDomainTextBox.Text;

                    t.ServerIdentity = new ServerIdentityType();

                    // Set issuer certificate
                    t.ServerIdentity.CertificateIssuer = 0;
                    t.ServerIdentity.CertificateIssuerSpecified = false;
                    if (issuerCertificateComboBox.SelectedIndex > 0)
                    {
                        AmtSecurityAdmin.AmtStoredCert tcert = (AmtSecurityAdmin.AmtStoredCert)issuerCertificateComboBox.SelectedItem;
                        if (tcert != null)
                        {
                            t.ServerIdentity.CertificateIssuer = tcert.Handle;
                            t.ServerIdentity.CertificateIssuerSpecified = true;
                        }
                    }

                    t.ServerIdentity.ServerNameOptions = null;
                    if (serverIdentityCheckComboBox.SelectedIndex > 0)
                    {
                        t.ServerIdentity.ServerNameOptions = new CertificateNameType();
                        if (serverIdentityCheckComboBox.SelectedIndex == 1) t.ServerIdentity.ServerNameOptions.Options = CertNameOptionsType.CertNameMatch;
                        if (serverIdentityCheckComboBox.SelectedIndex == 2) t.ServerIdentity.ServerNameOptions.Options = CertNameOptionsType.DomainSuffixMatch;
                        t.ServerIdentity.ServerNameOptions.ServerOrCertName = serverCheckTextBox.Text;
                    }

                    if (pacDataTextBox.Text.Length > 0 || pacMaskedTextBox.Text.Length > 0)
                    {
                        t.ProtectedAccessCredentials = new ManualPACType();
                        t.ProtectedAccessCredentials.PACData = Utils.HexToBytes(pacDataTextBox.Text);
                        t.ProtectedAccessCredentials.Password = pacMaskedTextBox.Text;
                    }

                    xProfile = new XProfileType();
                    xProfile.Item = (object)t;
                    break;
                }
                case 6:
                {
                    XProfileEAPFAST_GTCType t = new XProfileEAPFAST_GTCType();

                    t.RoamingIdentity = null;
                    if (roamingIdentityTextBox.Text.Length > 0) t.RoamingIdentity = roamingIdentityTextBox.Text;

                    if (clientCertificateComboBox.SelectedIndex > 0)
                    {
                        AmtSecurityAdmin.AmtStoredCert cert = (AmtSecurityAdmin.AmtStoredCert)clientCertificateComboBox.SelectedItem;
                        if (cert != null)
                        {
                            t.ClientCertificate = cert.Handle;
                            t.ClientCertificateSpecified = true;
                        }
                    }

                    t.UserCredentials = new UserCredentialsType();
                    t.UserCredentials.Username = ucUsernameTextBox.Text;
                    t.UserCredentials.Password = ucPasswordMaskedTextBox.Text;
                    if (ucDomainTextBox.Text.Length > 0) t.UserCredentials.Domain = ucDomainTextBox.Text;

                    t.ServerIdentity = new ServerIdentityType();

                    // Set issuer certificate
                    t.ServerIdentity.CertificateIssuer = 0;
                    t.ServerIdentity.CertificateIssuerSpecified = false;
                    if (issuerCertificateComboBox.SelectedIndex > 0)
                    {
                        AmtSecurityAdmin.AmtStoredCert tcert = (AmtSecurityAdmin.AmtStoredCert)issuerCertificateComboBox.SelectedItem;
                        if (tcert != null)
                        {
                            t.ServerIdentity.CertificateIssuer = tcert.Handle;
                            t.ServerIdentity.CertificateIssuerSpecified = true;
                        }
                    }

                    t.ServerIdentity.ServerNameOptions = null;
                    if (serverIdentityCheckComboBox.SelectedIndex > 0)
                    {
                        t.ServerIdentity.ServerNameOptions = new CertificateNameType();
                        if (serverIdentityCheckComboBox.SelectedIndex == 1) t.ServerIdentity.ServerNameOptions.Options = CertNameOptionsType.CertNameMatch;
                        if (serverIdentityCheckComboBox.SelectedIndex == 2) t.ServerIdentity.ServerNameOptions.Options = CertNameOptionsType.DomainSuffixMatch;
                        t.ServerIdentity.ServerNameOptions.ServerOrCertName = serverCheckTextBox.Text;
                    }

                    if (pacDataTextBox.Text.Length > 0 || pacMaskedTextBox.Text.Length > 0)
                    {
                        t.ProtectedAccessCredentials = new ManualPACType();
                        t.ProtectedAccessCredentials.PACData = Utils.HexToBytes(pacDataTextBox.Text);
                        t.ProtectedAccessCredentials.Password = pacMaskedTextBox.Text;
                    }

                    xProfile = new XProfileType();
                    xProfile.Item = (object)t;
                    break;
                }
                case 7:
                {
                    XProfileEAPFAST_TLSType t = new XProfileEAPFAST_TLSType();

                    t.RoamingIdentity = null;
                    if (roamingIdentityTextBox.Text.Length > 0) t.RoamingIdentity = roamingIdentityTextBox.Text;

                    if (clientCertificateComboBox.SelectedIndex >= 0)
                    {
                        AmtSecurityAdmin.AmtStoredCert cert = (AmtSecurityAdmin.AmtStoredCert)clientCertificateComboBox.SelectedItem;
                        if (cert != null) t.ClientCertificate = cert.Handle;
                    }

                    t.UserCredentials = new UserCredentialsType();
                    t.UserCredentials.Username = ucUsernameTextBox.Text;
                    t.UserCredentials.Password = ucPasswordMaskedTextBox.Text;
                    if (ucDomainTextBox.Text.Length > 0) t.UserCredentials.Domain = ucDomainTextBox.Text;

                    t.ServerIdentity = new ServerIdentityType();

                    // Set issuer certificate
                    t.ServerIdentity.CertificateIssuer = 0;
                    t.ServerIdentity.CertificateIssuerSpecified = false;
                    if (issuerCertificateComboBox.SelectedIndex > 0)
                    {
                        AmtSecurityAdmin.AmtStoredCert tcert = (AmtSecurityAdmin.AmtStoredCert)issuerCertificateComboBox.SelectedItem;
                        if (tcert != null)
                        {
                            t.ServerIdentity.CertificateIssuer = tcert.Handle;
                            t.ServerIdentity.CertificateIssuerSpecified = true;
                        }
                    }

                    t.ServerIdentity.ServerNameOptions = null;
                    if (serverIdentityCheckComboBox.SelectedIndex > 0)
                    {
                        t.ServerIdentity.ServerNameOptions = new CertificateNameType();
                        if (serverIdentityCheckComboBox.SelectedIndex == 1) t.ServerIdentity.ServerNameOptions.Options = CertNameOptionsType.CertNameMatch;
                        if (serverIdentityCheckComboBox.SelectedIndex == 2) t.ServerIdentity.ServerNameOptions.Options = CertNameOptionsType.DomainSuffixMatch;
                        t.ServerIdentity.ServerNameOptions.ServerOrCertName = serverCheckTextBox.Text;
                    }

                    if (pacDataTextBox.Text.Length > 0 || pacMaskedTextBox.Text.Length > 0)
                    {
                        t.ProtectedAccessCredentials = new ManualPACType();
                        t.ProtectedAccessCredentials.PACData = Utils.HexToBytes(pacDataTextBox.Text);
                        t.ProtectedAccessCredentials.Password = pacMaskedTextBox.Text;
                    }

                    xProfile = new XProfileType();
                    xProfile.Item = (object)t;
                    break;
                }
            }

            return xProfile;
        }

        private void SetServerIdentity(ServerIdentityType id)
        {
            if (id.CertificateIssuerSpecified == false)
            {
                issuerCertificateComboBox.SelectedIndex = 0;
            }
            else
            {
                SetIssuerCertificate(id.CertificateIssuer, true);
            }

            if (id.ServerNameOptions == null)
            {
                serverIdentityCheckComboBox.SelectedIndex = 0;
                serverCheckTextBox.Text = "";
            }
            else
            {
                if (id.ServerNameOptions.Options == CertNameOptionsType.CertNameMatch) serverIdentityCheckComboBox.SelectedIndex = 1;
                if (id.ServerNameOptions.Options == CertNameOptionsType.DomainSuffixMatch) serverIdentityCheckComboBox.SelectedIndex = 2;
                serverCheckTextBox.Text = id.ServerNameOptions.ServerOrCertName;
            }
        }

        private void SetClientCertificate(uint handle, bool specified)
        {
            if (specified == true)
            {
                selectedClientCertificateHandle = handle;
                for (int i = 0; i < clientCertificateComboBox.Items.Count; i++)
                {
                    if (clientCertificateComboBox.Items[i].GetType() == typeof(AmtSecurityAdmin.AmtStoredCert))
                    {
                        AmtSecurityAdmin.AmtStoredCert cert = (AmtSecurityAdmin.AmtStoredCert)clientCertificateComboBox.Items[i];
                        if (cert.Handle == handle) clientCertificateComboBox.SelectedIndex = i;
                    }
                }
            }
            else
            {
                if (clientCertificateComboBox.Items.Count > 0) clientCertificateComboBox.SelectedIndex = 0;
            }
        }


        private void SetIssuerCertificate(uint handle, bool specified)
        {
            if (specified == true)
            {
                selectedIssuerCertificateHandle = handle;
                for (int i = 0; i < issuerCertificateComboBox.Items.Count; i++)
                {
                    if (issuerCertificateComboBox.Items[i].GetType() == typeof(AmtSecurityAdmin.AmtStoredCert))
                    {
                        AmtSecurityAdmin.AmtStoredCert cert = (AmtSecurityAdmin.AmtStoredCert)issuerCertificateComboBox.Items[i];
                        if (cert.Handle == handle) issuerCertificateComboBox.SelectedIndex = i;
                    }
                }
            }
            else
            {
                if (issuerCertificateComboBox.Items.Count > 0) issuerCertificateComboBox.SelectedIndex = 0;
            }
        }

        private void SetUserCredentials(UserCredentialsType c)
        {
            if (c != null)
            {
                ucUsernameTextBox.Text = c.Username;
                ucPasswordMaskedTextBox.Text = c.Password;
                if (c.Domain != null) ucDomainTextBox.Text = c.Domain; else ucDomainTextBox.Text = "";
            }
            else
            {
                ucUsernameTextBox.Text = "";
                ucPasswordMaskedTextBox.Text = "";
                ucDomainTextBox.Text = "";
            }
        }

        private void SetProtectedAccess(ManualPACType pa)
        {
            if (pa != null)
            {
                pacDataTextBox.Text = Utils.BytesToHex(pa.PACData);
                pacMaskedTextBox.Text = pa.Password;
            }
            else
            {
                pacDataTextBox.Text = "";
                pacMaskedTextBox.Text = "";
            }
        }

        static public string GetProfileString(XProfileType xProfile)
        {
            string[] xProfilesNames = Properties.Resources.xProfileNames.Split(',');

            if (xProfile == null || xProfile.Item == null) return xProfilesNames[0];
            if (xProfile.Item.GetType() == typeof(XProfileTLSType)) return xProfilesNames[1];
            if (xProfile.Item.GetType() == typeof(XProfileTTLS_MSCHAPv2Type)) return xProfilesNames[2];
            if (xProfile.Item.GetType() == typeof(XProfilePEAP_MSCHAPv2Type)) return xProfilesNames[3];
            if (xProfile.Item.GetType() == typeof(XProfileEAP_GTCType)) return xProfilesNames[4];
            if (xProfile.Item.GetType() == typeof(XProfileEAPFAST_MSCHAPv2Type)) return xProfilesNames[5];
            if (xProfile.Item.GetType() == typeof(XProfileEAPFAST_GTCType)) return xProfilesNames[6];
            if (xProfile.Item.GetType() == typeof(XProfileEAPFAST_TLSType)) return xProfilesNames[7];
            return Properties.Resources.Unknown;
        }

        public void SetProfile(XProfileType xProfile)
        {
            if (xProfile == null || xProfile.Item == null)
            {
                profileTypeComboBox.SelectedIndex = 0;
                return;
            }
            if (xProfile.Item.GetType() == typeof(XProfileTLSType))
            {
                XProfileTLSType t = (XProfileTLSType)xProfile.Item;
                profileTypeComboBox.SelectedIndex = 1;
                usernameTextBox.Text = t.Username;
                SetServerIdentity(t.ServerIdentity);
                SetClientCertificate(t.ClientCertificate, true);
                return;
            }
            if (xProfile.Item.GetType() == typeof(XProfileTTLS_MSCHAPv2Type))
            {
                XProfileTTLS_MSCHAPv2Type t = (XProfileTTLS_MSCHAPv2Type)xProfile.Item;
                profileTypeComboBox.SelectedIndex = 2;
                roamingIdentityTextBox.Text = t.RoamingIdentity;
                SetServerIdentity(t.ServerIdentity);
                SetClientCertificate(t.ClientCertificate, t.ClientCertificateSpecified);
                SetUserCredentials(t.UserCredentials);
                return;
            }
            if (xProfile.Item.GetType() == typeof(XProfilePEAP_MSCHAPv2Type))
            {
                XProfilePEAP_MSCHAPv2Type t = (XProfilePEAP_MSCHAPv2Type)xProfile.Item;
                profileTypeComboBox.SelectedIndex = 3;
                roamingIdentityTextBox.Text = t.RoamingIdentity;
                SetServerIdentity(t.ServerIdentity);
                SetClientCertificate(t.ClientCertificate, t.ClientCertificateSpecified);
                SetUserCredentials(t.UserCredentials);
                return;
            }
            if (xProfile.Item.GetType() == typeof(XProfileEAP_GTCType))
            {
                XProfileEAP_GTCType t = (XProfileEAP_GTCType)xProfile.Item;
                profileTypeComboBox.SelectedIndex = 4;
                SetUserCredentials(t.UserCredentials);
                return;
            }
            if (xProfile.Item.GetType() == typeof(XProfileEAPFAST_MSCHAPv2Type))
            {
                XProfileEAPFAST_MSCHAPv2Type t = (XProfileEAPFAST_MSCHAPv2Type)xProfile.Item;
                profileTypeComboBox.SelectedIndex = 5;
                roamingIdentityTextBox.Text = t.RoamingIdentity;
                SetServerIdentity(t.ServerIdentity);
                SetClientCertificate(t.ClientCertificate, t.ClientCertificateSpecified);
                SetUserCredentials(t.UserCredentials);
                SetProtectedAccess(t.ProtectedAccessCredentials);
                return;
            }
            if (xProfile.Item.GetType() == typeof(XProfileEAPFAST_GTCType))
            {
                XProfileEAPFAST_GTCType t = (XProfileEAPFAST_GTCType)xProfile.Item;
                profileTypeComboBox.SelectedIndex = 6;
                roamingIdentityTextBox.Text = t.RoamingIdentity;
                SetServerIdentity(t.ServerIdentity);
                SetClientCertificate(t.ClientCertificate, t.ClientCertificateSpecified);
                SetUserCredentials(t.UserCredentials);
                SetProtectedAccess(t.ProtectedAccessCredentials);
                return;
            }
            if (xProfile.Item.GetType() == typeof(XProfileEAPFAST_TLSType))
            {
                XProfileEAPFAST_TLSType t = (XProfileEAPFAST_TLSType)xProfile.Item;
                profileTypeComboBox.SelectedIndex = 7;
                roamingIdentityTextBox.Text = t.RoamingIdentity;
                SetServerIdentity(t.ServerIdentity);
                SetClientCertificate(t.ClientCertificate, true);
                SetUserCredentials(t.UserCredentials);
                SetProtectedAccess(t.ProtectedAccessCredentials);
                return;
            }
        }

        private void serverIdentityCheckComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            serverCheckTextBox.Enabled = (serverIdentityCheckComboBox.SelectedIndex > 0);
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            if (inf != null)
            {
                AmtCallStatus r = inf.Set8021XWiredProfile(GetProfile());
                if (r != AmtCallStatus.SUCCESS)
                {
                    MessageBox.Show(this, string.Format(Properties.Resources.xProfileError, r.ToString()), Properties.Resources.xProfileTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    DialogResult = DialogResult.OK;
                }
            }
            else
            {
                DialogResult = DialogResult.OK;
            }
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Utils.SerializeToClipboard(GetProfile(), typeof(XProfileType));
        }

        private void pasteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            XProfileType p = (XProfileType)Utils.UnSerializeFromClipboard(typeof(XProfileType));
            SetProfile(p);
        }
    }
}
