// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityConnectorTool
{
    public partial class EnrollementForm : Form
    {
        private MspProfile[] profiles;

        public EnrollementForm(MspProfile[] profiles)
        {
            this.profiles = profiles;
            InitializeComponent();
        }

        private void EnrollementForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            foreach (MspProfile profile in profiles)
            {
                providerComboBox.Items.Add(new Utils.DropDownItem(profile.FriendlyName, 0, profile));
            }
            providerComboBox.SelectedIndex = 0;
        }

        public MspProfile GetSelectedProfile()
        {
            return (MspProfile)(((Utils.DropDownItem)providerComboBox.SelectedItem).Tag);
        }

        public string GetUsername()
        {
            return accountNumberTextBox.Text;
        }

        public string GetPassword()
        {
            return passwordControl.Text;
        }

        private void accountNumberTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void UpdateInfo()
        {
            okButton.Enabled = (accountNumberTextBox.Text.Length > 0) && (passwordControl.Text.Length > 0);
        }

        private void SaveSoftSettings(MspProfile profile)
        {
            Properties.Settings.Default.LocalhostName = "aid-" + accountNumberTextBox.Text + ".soft.cp.intel.com";
            Properties.Settings.Default.ManagementServer = profile.GatewayFqdn;
            Properties.Settings.Default.ManagementServerPort = (int)profile.Port;
            Properties.Settings.Default.ManagementAccount = accountNumberTextBox.Text;
            Properties.Settings.Default.ManagementPassword = passwordControl.Text;
            Properties.Settings.Default.MeName = "3PDSAF";
            Properties.Settings.Default.MePassword = "3PDSPass";
            Properties.Settings.Default.ManagementEnabled = true;
            Properties.Settings.Default.Save();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            SaveSoftSettings(GetSelectedProfile());
            DialogResult = DialogResult.OK;
        }

    }
}
