// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityConnectorTool
{
    public partial class SoftwareManagementConfigForm : Form
    {
        private string Host;
        private int Port;

        public SoftwareManagementConfigForm()
        {
            InitializeComponent();
        }

        public static bool IsConfigured()
        {
            if (Properties.Settings.Default.ManagementEnabled == false) return false;
            if (Properties.Settings.Default.LocalhostName == null || Properties.Settings.Default.LocalhostName.Length == 0) return false;
            if (Properties.Settings.Default.MeName == null || Properties.Settings.Default.MeName.Length == 0) return false;
            if (Properties.Settings.Default.MePassword == null || Properties.Settings.Default.MePassword.Length == 0) return false;
            if (Properties.Settings.Default.ManagementServer == null || Properties.Settings.Default.ManagementServer.Length == 0) return false;
            if (Properties.Settings.Default.ManagementServerPort == 0) return false;
            if (Properties.Settings.Default.ManagementAccount == null || Properties.Settings.Default.ManagementAccount.Length == 0) return false;
            if (Properties.Settings.Default.ManagementPassword == null || Properties.Settings.Default.ManagementPassword.Length == 0) return false;
            return true;
        }

        private void SoftwareManagementConfigForm_Load(object sender, EventArgs e)
        {
            enabledCheckBox.Checked = Properties.Settings.Default.ManagementEnabled;
            localhostTextBox.Text = Properties.Settings.Default.LocalhostName;
            if (localhostTextBox.Text.Length == 0)
            {
                localhostTextBox.Text = Utils.GetLocalFqdn();
            }
            meNameTextBox.Text = Properties.Settings.Default.MeName;
            mePasswordControl.Text = Properties.Settings.Default.MePassword;
            serverNameTextBox.Text = Properties.Settings.Default.ManagementServer;
            if (Properties.Settings.Default.ManagementServerPort != 0) serverNameTextBox.Text += ":" + Properties.Settings.Default.ManagementServerPort.ToString();
            serverAccountNameTextBox.Text = Properties.Settings.Default.ManagementAccount;
            serverPasswordControl.Text = Properties.Settings.Default.ManagementPassword;
            tlsCheckBox.Checked = Properties.Settings.Default.ManagementTls;
            UpdateInfo();
        }

        private void UpdateInfo()
        {
            bool ok = true;

            int i = serverNameTextBox.Text.IndexOf(':');
            if (i < 1) ok = false;
            else
            {
                string hoststr = serverNameTextBox.Text.Substring(0, i);
                string portstr = serverNameTextBox.Text.Substring(i + 1);
                if (hoststr.Length == 0 || portstr.Length == 0) ok = false;
                else
                {
                    if (int.TryParse(portstr, out Port) == false) ok = false;
                    else
                    {
                        if (Port < 1 || Port > 65534)
                        {
                            Port = 0;
                            ok = false;
                        }
                    }
                }
                Host = hoststr;
            }

            if (localhostTextBox.Text.Length == 0) ok = false;
            if (meNameTextBox.Text.Length == 0) ok = false;
            if (mePasswordControl.Text.Length == 0) ok = false;
            if (serverNameTextBox.Text.Length == 0) ok = false;
            if (serverAccountNameTextBox.Text.Length == 0) ok = false;
            if (serverPasswordControl.Text.Length == 0) ok = false;

            okButton.Enabled = ok;
        }

        private void localhostTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            Properties.Settings.Default.ManagementEnabled = enabledCheckBox.Checked;
            Properties.Settings.Default.LocalhostName = localhostTextBox.Text;
            Properties.Settings.Default.MeName = meNameTextBox.Text;
            Properties.Settings.Default.MePassword = mePasswordControl.Text;
            Properties.Settings.Default.ManagementServer = Host;
            Properties.Settings.Default.ManagementServerPort = Port;
            Properties.Settings.Default.ManagementAccount = serverAccountNameTextBox.Text;
            Properties.Settings.Default.ManagementPassword = serverPasswordControl.Text;
            Properties.Settings.Default.ManagementTls = tlsCheckBox.Checked;
            Properties.Settings.Default.Save();
            DialogResult = DialogResult.OK;
        }

        private void enabledCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            localhostTextBox.Enabled = enabledCheckBox.Checked;
            meNameTextBox.Enabled = enabledCheckBox.Checked;
            mePasswordControl.Enabled = enabledCheckBox.Checked;
            serverNameTextBox.Enabled = enabledCheckBox.Checked;
            serverAccountNameTextBox.Enabled = enabledCheckBox.Checked;
            serverPasswordControl.Enabled = enabledCheckBox.Checked;
            tlsCheckBox.Enabled = enabledCheckBox.Checked;
        }
    }
}
