// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Text;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Reflection;
using System.Text.RegularExpressions;

namespace ManageabilityControls
{
    public partial class AboutForm : Form
    {
        public static string VersionString
        {
            get
            {
                string infoVersion;

                object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyInformationalVersionAttribute), false);
                if (attributes.Length == 0)
                {
                    return "";
                }
                infoVersion = ((AssemblyInformationalVersionAttribute)attributes[0]).InformationalVersion;
                if (String.IsNullOrEmpty(infoVersion))
                    return infoVersion;

                //
                // Determine if this is a test build by examining the revision number.
                //
                Regex verRegEx = new Regex("(?<majorVersion>(\\d+|\\*))\\.(?<minorVersion>(\\d+|\\*))\\.?(?<buildNumber>(\\d+|\\*)?)\\.?(?<revision>(\\d+|\\*)?)");
                Match match = verRegEx.Match(infoVersion);
                if (match.Success)
                {
                    int revision = Convert.ToInt32(match.Groups["revision"].Value);
                    if (revision == 0)
                    {
                        // Intel internal only build.
                        infoVersion = String.Format("{0} - {1} - {2}", infoVersion,
                            Properties.Resources.IntelConfNotification,
                            Properties.Resources.NoDistNotification);
                    }
                    else if (revision == 1)
                    {
                        // Intel internal only test build.
                        infoVersion = String.Format("{0} - {1} - {2} - {3}", infoVersion,
                            Properties.Resources.IntelConfNotification,
                            Properties.Resources.TestBuildNotification,
                            Properties.Resources.NoDistNotification);
                    }
                    else if (revision == 3)
                    {
                        // Test build.
                        infoVersion = String.Format("{0} - {1} - {2}", infoVersion,
                            Properties.Resources.TestBuildNotification,
                            Properties.Resources.NoDistNotification);
                    }
                }

                return infoVersion;
            }
        }

        private string ProdName;
        private string ProdSubname;

        public AboutForm(string name, string subname)
        {
            ProdName = name;
            ProdSubname = subname;
            InitializeComponent();
        }

        private void AboutForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            Rebrand();
            this.Text = productNameLabel.Text = ManageabilityStack.Utils.FilterStrEx(ProdName);
            productSubnameLabel.Text = ManageabilityStack.Utils.FilterStrEx(ProdSubname);
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                System.Diagnostics.Process.Start("http://www.intel.com/software/manageability/");
            }
            catch (System.ComponentModel.Win32Exception) { }
        }

        public static void GotoNews()
        {
            try
            {
                System.Diagnostics.Process.Start("http://www.intel.com/software/amt-dtk");
            }
            catch (System.ComponentModel.Win32Exception) { }
        }

        public static void GotoDownloads()
        {
            try
            {
                System.Diagnostics.Process.Start("http://www.intel.com/software/amt-dtk");
            }
            catch (System.ComponentModel.Win32Exception) { }
        }

        public static void GotoForums()
        {
            try
            {
                System.Diagnostics.Process.Start("http://softwarecommunity.intel.com/isn/Community/en-US/forums/2436/ShowForum.aspx");
            }
            catch (System.ComponentModel.Win32Exception) { }
        }

        /// <summary>
        /// Rebrand the UI using other bitmaps and string if the Branding directory is present
        /// </summary>
        private void Rebrand()
        {
            if (Directory.Exists(Application.StartupPath + "\\Branding") == false) return;
            if (File.Exists(Application.StartupPath + "\\Branding\\AboutBox.png")) BackgroundImage = Image.FromFile(Application.StartupPath + "\\Branding\\AboutBox.png");

            // Change the form icon
            //if (File.Exists(Application.StartupPath + "\\Branding\\Console.ico")) { this.Icon = new Icon(Application.StartupPath + "\\Branding\\Console.ico"); }

            // Update the form's text
            if (File.Exists(Application.StartupPath + "\\Branding\\Branding.txt"))
            {
                FileStream f = new FileStream(Application.StartupPath + "\\Branding\\Branding.txt", FileMode.Open);
                byte[] buf = new byte[f.Length];
                f.Read(buf, 0, (int)(f.Length));
                f.Close();
                string data = UTF8Encoding.UTF8.GetString(buf);
                string[] brandStrings = data.Replace("\r\n", "\r").Split('\r');
                if (brandStrings.Length > 3) ProdName = brandStrings[3];
                if (brandStrings.Length > 4) ProdSubname = brandStrings[4];
            }
        }

        public static void RebrandImage(PictureBox box, string file)
        {
            if (File.Exists(Application.StartupPath + "\\Branding\\" + file)) box.Image = Image.FromFile(Application.StartupPath + "\\Branding\\" + file);
        }

        public static void RebrandImage(PictureBox box, string file, Bitmap defaultBitmap)
        {
            if (File.Exists(Application.StartupPath + "\\Branding\\" + file))
            {
                box.Image = Image.FromFile(Application.StartupPath + "\\Branding\\" + file);
            }
            else
            {
                box.Image = defaultBitmap;
            }
        }

        public static Image RebrandImageEx(string file, Bitmap defaultBitmap)
        {
            if (File.Exists(Application.StartupPath + "\\Branding\\" + file))
            {
                return Image.FromFile(Application.StartupPath + "\\Branding\\" + file);
            }
            else
            {
                return defaultBitmap;
            }
        }

    }
}
