// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Text;
using System.Data;
using System.Drawing;
using System.Diagnostics;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;

namespace ManageabilityControls
{
    public partial class AddWatchdogForm : Form
    {
        public AddWatchdogForm()
        {
            InitializeComponent();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
        }

        public Guid AgentID
        {
            get
            {
                return new Guid(agentIdTextBox.Text);
            }
            set
            {
                agentIdTextBox.Text = value.ToString().ToUpper();
            }
        }

        public string AgentName
        {
            get
            {
                return agentNameTextBox.Text;
            }
            set
            {
                agentNameTextBox.Text = value;
            }
        }

        public string AgentProcessCondition
        {
            get
            {
                if (processCheckBox.Checked == false) return null;
                return processComboBox.Text;
            }
            set
            {
                processCheckBox.Checked = (value != null);
                if (value != null) processComboBox.Text = value;
            }
        }

        private void generateButton_Click(object sender, EventArgs e)
        {
            agentIdTextBox.Text = Guid.NewGuid().ToString().ToUpper();
        }

        private void agentIdTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void UpdateInfo()
        {
            bool ok = true;
            try
            {
                new Guid(agentIdTextBox.Text);
            }
            catch (Exception)
            {
                ok = false;
            }

            if (agentNameTextBox.Text.Length == 0) ok = false;

            okButton.Enabled = ok;
            processComboBox.Enabled = processCheckBox.Checked;
        }

        private void processCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void AddWatchdogForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            UpdateInfo();
            UpdateProcessList();
        }

        private void UpdateProcessList()
        {
            processComboBox.Items.Clear();
            Process[] procList = Process.GetProcesses();
            foreach (Process p in procList)
            {
                try
                {
                    processComboBox.Items.Add(new DropDownItem(p.ProcessName,0,null));
                }
                catch (Exception) { }
            }
        }

        /// <summary>
        /// This class is used to add items to DropDown combo boxes and still be able to reference the item quickly.
        /// </summary>
        public class DropDownItem
        {
            public string Text = null;
            public object Tag = null;
            public int Handle = 0;
            public override string ToString() { return Text; }

            public DropDownItem()
            {
            }

            public DropDownItem(string Text, int Handle, object Tag)
            {
                this.Text = Text;
                this.Handle = Handle;
                this.Tag = Tag;
            }
        }

    }
}
