// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class AgentNotificationForm : Form
    {
        private AmtSystem computer;
        private Guid WatchdogNotificationGuid = new Guid("C0770F68-9174-479b-87A5-A821FDFEF3C7");
        private int mode = 0;
        private AmtWatchdogAgent watchdog;
        private ushort PerformCommand = 0;

        public enum AgentNotificationCommands
        {
            Sleep = 100,
            Shutdown = 101,
            Logoff = 102,
            Reset = 103
        }

        public AgentNotificationForm(AmtSystem computer)
        {
            this.computer = computer;
            InitializeComponent();

            // If the computer is disconnected, close this form
            if (computer != null) computer.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }

        public AgentNotificationForm(AmtSystem computer, AgentNotificationCommands command)
        {
            PerformCommand = (ushort)command;
            this.computer = computer;
            InitializeComponent();

            // If the computer is disconnected, close this form
            if (computer != null) computer.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }

        private void SystemStateChangedHandler(AmtSystem computer) { try { this.Invoke(new AmtSystem.ObjectStateHandler(SystemStateChangedHandlerEx), new object[] { computer }); } catch (Exception) { } }
        private void SystemStateChangedHandlerEx(AmtSystem computer) { if (computer.State != AmtSystemObjState.Connected) Close(); }

        private void AgentNotificationForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            actionComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Sleep, 100, null));
            actionComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Shutdown, 101, null));
            actionComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Logoff, 102, null));
            actionComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.Reset, 103, null));
            actionComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.TestNotification, 99, null));
            actionComboBox.SelectedIndex = 0;

            if (PerformCommand != 0)
            {
                foreach (Utils.DropDownItem i in actionComboBox.Items)
                {
                    if (i.Handle == PerformCommand)
                    {
                        actionComboBox.SelectedItem = i;
                        break;
                    }
                }
            }

            AmtWatchdogAgent[] agents = computer.Watchdog.GetAgents();
            AmtWatchdogAgent w = null;
            if (agents != null && agents.Length > 0)
            {
                foreach (AmtWatchdogAgent agent in agents)
                {
                    if (agent.AgentId.Equals(WatchdogNotificationGuid) == true)
                    {
                        w = agent;
                        break;
                    }
                }
            }
            if (w != null)
            {
                computer.Watchdog.Delete(w);
                statusLabel.Text = Properties.Resources.RemovingOldWatchdog;
                mode = 1;
                mainTimer.Enabled = true;
            }
            else
            {
                if (PerformCommand != 0) sendButton_Click(this, null);
            }
        }

        private void sendButton_Click(object sender, EventArgs e)
        {
            int code = ((Utils.DropDownItem)actionComboBox.SelectedItem).Handle;
            watchdog = new AmtWatchdogAgent(WatchdogNotificationGuid, Properties.Resources.NotificationWatchdogName, (ushort)code, 1);
            computer.Watchdog.Add(watchdog);
            sendButton.Enabled = false;
            mode = 2;
            statusLabel.Text = Properties.Resources.SendingWatchdogCommand;
            sendButton.Enabled = false;
            mainTimer.Enabled = true;
            closeButton.Enabled = false;
        }

        private void mainTimer_Tick(object sender, EventArgs e)
        {
            switch (mode)
            {
                case 1:
                    mode = 0;
                    mainTimer.Enabled = false;
                    sendButton.Enabled = true;
                    if (PerformCommand != 0)
                    {
                        sendButton_Click(this, null);
                    }
                    else
                    {
                        statusLabel.Text = Properties.Resources.WaitingForCommand;
                    }
                    break;
                case 2:
                    computer.Watchdog.UpdateAgent(ref watchdog);
                    if ((watchdog.State != 255) && ((watchdog.State & 0x0004) != 0))
                    {
                        statusLabel.Text = Properties.Resources.Success;
                        if (PerformCommand != 0) DialogResult = DialogResult.OK;
                    }
                    else
                    {
                        PerformCommand = 0;
                        statusLabel.Text = Properties.Resources.ReverseWatchdogFailed;
                    }
                    closeButton.Enabled = true;
                    mode = 0;
                    mainTimer.Enabled = false;
                    sendButton.Enabled = true;
                    computer.Watchdog.Delete(watchdog);
                    watchdog = null;
                    break;
            }
        }

        private void AgentNotificationForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (computer != null) computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }
    }
}
