﻿// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intel’s prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class AmtTreeView : TreeView
    {
        #region Constructor
        public AmtTreeView()
        {
            InitializeComponent();
        }
        #endregion

        #region Event Handlers
        protected override void OnPaint(PaintEventArgs pe)
        {
            base.OnPaint(pe);
        }

        private void TreeViewEx_DrawNode(object sender, DrawTreeNodeEventArgs e)
        {
            if (e.Node.Tag == null)
            {
                e.DrawDefault = true;
                return;
            }

            // Dual Icon 
            #region AmtSystem
            if (e.Node.Tag.GetType() == typeof(AmtSystem))
            {                
                AmtTreeView tv = sender as AmtTreeView;
                AmtSystem system = (AmtSystem) e.Node.Tag;

                //for the local system "my computer" dont draw additional icons
                if (system.MyComputer)
                {
                    e.DrawDefault = true; 
                    return;
                }

                Icon SecondIcon = null;
                Debug.Assert(system.HostNameIdentity != null);
                switch (system.HostNameIdentity.ForceIPOption)
                {
                    case ForceIPOption.UseDefault:
                        SecondIcon = ManageabilityControls.Properties.Resources.IPUseAny;
                        break;
                    case ForceIPOption.UseIPv4:
                        SecondIcon = ManageabilityControls.Properties.Resources.IPv4Icon;
                        break;
                    case ForceIPOption.UseIPv6:
                        SecondIcon = ManageabilityControls.Properties.Resources.IPv6Icon;
                        break;
                    default:
                        Debug.Assert(false, "Invalid ForceIPOption");
                        break;
                }

                Font nodeFont = e.Node.NodeFont;
                if (nodeFont == null)
                    nodeFont = tv.Font;
                Rectangle textRect = new Rectangle(e.Bounds.X + 18, e.Bounds.Y,
                                                   e.Bounds.Width + 18, e.Bounds.Height);
                Rectangle fullRect = new Rectangle(e.Node.Bounds.X, e.Node.Bounds.Y,
                                                   e.Node.Bounds.Width + 36, e.Node.Bounds.Height);
                Rectangle iconRect = new Rectangle(e.Node.Bounds.X, e.Node.Bounds.Y, 16, 16);

                // Ensure we erase current contents
                e.Graphics.FillRectangle(Brushes.White, fullRect);

                // Draw 2nd Icon
                if (SecondIcon != null)
                    e.Graphics.DrawIcon(SecondIcon, iconRect);

                // If Not FOCUSed
                if ((e.State & TreeNodeStates.Focused) == 0)
                {
                    // Draw the node text normally
                    e.Graphics.DrawString(e.Node.Text, nodeFont, SystemBrushes.ControlText, textRect);
                }
                else // FOCUSed
                {
                    // Draw the node text highlighted
                    e.Graphics.FillRectangle(SystemBrushes.Highlight, textRect);

                    using (Pen focusPen = new Pen(Color.Black))
                    {
                        focusPen.DashStyle = System.Drawing.Drawing2D.DashStyle.Dot;
                        Rectangle focusBounds = textRect;
                        focusBounds.Size = new Size(focusBounds.Width - 1, focusBounds.Height - 1);
                        e.Graphics.DrawRectangle(focusPen, focusBounds);
                        e.Graphics.DrawString(e.Node.Text, nodeFont, SystemBrushes.HighlightText, textRect);
                    }
                }
            } // END if (e.Node.Tag.GetType() == typeof(AmtSystem))
            #endregion
            #region AmtMpsServer
            else if (e.Node.Tag.GetType() == typeof(AmtMpsServer))
            {
                AmtTreeView tv = sender as AmtTreeView;
                AmtMpsServer mps = (AmtMpsServer)e.Node.Tag;
                Icon SecondIcon = null;
                Debug.Assert(mps.NetworkNameIdentity != null);
                switch (mps.NetworkNameIdentity.ForceIPOption)
                {
                    case ForceIPOption.UseDefault:
                        SecondIcon = ManageabilityControls.Properties.Resources.IPUseAny;
                        break;
                    case ForceIPOption.UseIPv4:
                        SecondIcon = ManageabilityControls.Properties.Resources.IPv4Icon;
                        break;
                    case ForceIPOption.UseIPv6:
                        SecondIcon = ManageabilityControls.Properties.Resources.IPv6Icon;
                        break;
                    default:
                        Debug.Assert(false, "Invalid ForceIPOption");
                        break;
                }

                Font nodeFont = e.Node.NodeFont;
                if (nodeFont == null)
                    nodeFont = tv.Font;
                Rectangle textRect = new Rectangle(e.Bounds.X + 18, e.Bounds.Y,
                                                   e.Bounds.Width + 18, e.Bounds.Height);
                Rectangle fullRect = new Rectangle(e.Node.Bounds.X, e.Node.Bounds.Y,
                                                   e.Node.Bounds.Width + 36, e.Node.Bounds.Height);
                Rectangle iconRect = new Rectangle(e.Node.Bounds.X, e.Node.Bounds.Y, 16, 16);

                // Ensure we erase current contents
                e.Graphics.FillRectangle(Brushes.White, fullRect);

                // Draw 2nd Icon
                if (SecondIcon != null)
                    e.Graphics.DrawIcon(SecondIcon, iconRect);

                // If Not FOCUSed
                if ((e.State & TreeNodeStates.Focused) == 0)
                {
                    // Draw the node text normally
                    e.Graphics.DrawString(e.Node.Text, nodeFont, SystemBrushes.ControlText, textRect);
                }
                else // FOCUSed
                {
                    // Draw the node text highlighted
                    e.Graphics.FillRectangle(SystemBrushes.Highlight, textRect);

                    using (Pen focusPen = new Pen(Color.Black))
                    {
                        focusPen.DashStyle = System.Drawing.Drawing2D.DashStyle.Dot;
                        Rectangle focusBounds = textRect;
                        focusBounds.Size = new Size(focusBounds.Width - 1, focusBounds.Height - 1);
                        e.Graphics.DrawRectangle(focusPen, focusBounds);
                        e.Graphics.DrawString(e.Node.Text, nodeFont, SystemBrushes.HighlightText, textRect);
                    }
                }
            } // END if (e.Node.Tag.GetType() == typeof(AmtSystem))
            #endregion
            else
            {
                e.DrawDefault = true;
            }
        }
        #endregion
    }
}
