﻿// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intel’s prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class ControlModeForm : Form
    {
        private AmtSystem _system; 

        public ControlModeForm(AmtSystem System)
        {
            _system = System; 
            InitializeComponent();
            UpdateFields(); 
        }

        private void UpdateFields()
        {
            AmtUserConsent.ControlModes CurrentMode = _system.UserConsent.CurrentControlMode.Value;            

            if (CurrentMode == AmtUserConsent.ControlModes.CLIENT_CONTROL_MODE)
            {
                CurrentControlModeStateLabel.Text = ManageabilityControls.Properties.Resources.ControlMode_Client;
            }
            else if (CurrentMode == AmtUserConsent.ControlModes.ADMIN_CONTROL_MODE)
            {
                CurrentControlModeStateLabel.Text = ManageabilityControls.Properties.Resources.ControlMode_Admin;
            }
            else
            {
                CurrentControlModeStateLabel.Text = ManageabilityControls.Properties.Resources.NotSupported;
            }


            AmtUserConsent.AllowedModes AllowedMode = _system.UserConsent.SupportedControlModes.Value;

            if (AllowedMode == AmtUserConsent.AllowedModes.CLIENT_AND_ADMIN)
            {
                SupportedControlModesStateLabel.Text = ManageabilityControls.Properties.Resources.ControlMode_Client + ", " +
                    ManageabilityControls.Properties.Resources.ControlMode_Admin;

                DisableClientControlButton.Enabled = true; 
            }
            else if (AllowedMode == AmtUserConsent.AllowedModes.ADMIN_ONLY)
            {
                SupportedControlModesStateLabel.Text = ManageabilityControls.Properties.Resources.ControlMode_Admin;
                DisableClientControlButton.Enabled = false; 
            }
            else
            {
                SupportedControlModesStateLabel.Text = ManageabilityControls.Properties.Resources.NotSupported;
                DisableClientControlButton.Enabled = false; 
            }

        }

        private void okButton_Click(object sender, EventArgs e)
        {
            this.Close(); 
        }

        private void DisableClientControlButton_Click(object sender, EventArgs e)
        {
            DialogResult result = MessageBox.Show(this,
                        ManageabilityControls.Properties.Resources.ControlMode_Warning,
                        ManageabilityControls.Properties.Resources.ControlMode,
                        MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation);

            if (result == DialogResult.OK)
            {
                _system.UserConsent.DisableClientControlMode();
            }

            UpdateFields(); 
        }
    }
}
