// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Net;
using System.Text;
using System.Data;
using System.Drawing;
using System.Diagnostics;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Net.NetworkInformation;
using System.Security.Cryptography.X509Certificates;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class CreateCertificateForm : Form
    {
        private bool sha2Support = false;
        private int keysize = 512;
        public X509Certificate2 RootCertificate = null;
        public X509Certificate2 Certificate = null;

        public bool SHA2Support
        {
            get { return this.sha2Support; }
            set { this.sha2Support = value; }
        }

        public CreateCertificateForm(X509Certificate2 RootCertificate)
        {
            InitializeComponent();

            this.RootCertificate = RootCertificate;
        }

        public bool TrustedRoot()
        {
            return trustedCertCheckBox.Checked;
        }

        private void CreateCertificateForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            UpdateInfo();

            if (RootCertificate == null)
            {
                while (certTypeComboBox.Items.Count > 1) certTypeComboBox.Items.RemoveAt(1);
                certTypeComboBox.Enabled = false;
                certTypeComboBox.SelectedIndex = 0;
                commonNameTextBox.Text = Properties.Resources.SampleRootCertificate;
            }
            else
            {
                certTypeComboBox.Items.RemoveAt(0);
                certTypeComboBox.SelectedIndex = 0;
                commonNameTextBox.Text = Properties.Resources.SampleSubCaCertificate;
                trustedCertCheckBox.Visible = false;
                Height -= certTypeComboBox.Height;
            }
            UpdateInfo();

            this.digestComboBox.Enabled = this.sha2Support;
            this.digestComboBox.SelectedIndex = 0;
        }

        private void keySizeActiveTrackBar_Scroll(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void UpdateInfo()
        {
            if (RootCertificate == null || certTypeComboBox.SelectedIndex == 0 || certTypeComboBox.SelectedIndex == 3 || certTypeComboBox.SelectedIndex == 4)
            {
                keySizeActiveTrackBar.Value = 2;
                keySizeActiveTrackBar.Enabled = false;
                keySizeLabel.Enabled = false;
            }
            else
            {
                keySizeActiveTrackBar.Enabled = true;
                keySizeLabel.Enabled = true;
            }

            switch (keySizeActiveTrackBar.Value)
            {
                case 0:
                    keysize = 1024;
                    break;
                case 1:
                    keysize = 1536;
                    break;
                case 2:
                    keysize = 2048;
                    break;
            }
            keySizeLabel.Text = keysize.ToString() + ManageabilityControls.Properties.Resources.CertEdit_Key_Size;
        }

        private void generateButton_Click(object sender, EventArgs e)
        {
            generateButton.Text = Properties.Resources.Generating;
            generateButton.Enabled = false;
            closeButton.Enabled = false;
            certTypeComboBox.Enabled = false;
            commonNameTextBox.Enabled = false;
            orgNameTextBox.Enabled = false;
            countryCodeTextBox.Enabled = false;
            keySizeActiveTrackBar.Enabled = false;

            CertificateOperations.SignatureAlgorithm digestAlgorithm = CertificateOperations.SignatureAlgorithm.SHA1;
            if (this.sha2Support)
                digestAlgorithm = (CertificateOperations.SignatureAlgorithm)Enum.Parse(typeof(CertificateOperations.SignatureAlgorithm),
                    this.digestComboBox.SelectedItem.ToString());

            if (RootCertificate == null)
            {
                // Create a self-signed certificate
                    Certificate = CertificateOperations.CreateSelfSignedCert(commonNameTextBox.Text,
                        orgNameTextBox.Text, countryCodeTextBox.Text, keysize, digestAlgorithm);
            }
            else
            {
                if (certTypeComboBox.SelectedIndex == 0)
                {
                    // Sub-CA Certificate
                    Certificate = ManageabilityStack.CertificateOperations.IssueSignedCert(RootCertificate, 
                        commonNameTextBox.Text, orgNameTextBox.Text, countryCodeTextBox.Text, keysize, 
                        ManageabilityStack.CertificateOperations.IssueCertType.SubCA, digestAlgorithm);
                }
                else if (certTypeComboBox.SelectedIndex == 1)
                {
                    // Intel AMT Remote Console Certificate
                    Certificate = ManageabilityStack.CertificateOperations.IssueSignedCert(RootCertificate, 
                        commonNameTextBox.Text, orgNameTextBox.Text, countryCodeTextBox.Text, keysize, 
                        ManageabilityStack.CertificateOperations.IssueCertType.TlsClientRemote, digestAlgorithm);
                }
                else if (certTypeComboBox.SelectedIndex == 2)
                {
                    // Intel AMT Local Agent Certificate
                    Certificate = ManageabilityStack.CertificateOperations.IssueSignedCert(RootCertificate, 
                        commonNameTextBox.Text, orgNameTextBox.Text, countryCodeTextBox.Text, keysize, 
                        ManageabilityStack.CertificateOperations.IssueCertType.TlsClientLocal, digestAlgorithm);
                }
                else if (certTypeComboBox.SelectedIndex == 3)
                {
                    // Intel AMT Remote Configuration Setup Certificate
                    Certificate = ManageabilityStack.CertificateOperations.IssueSignedCert(RootCertificate, 
                        commonNameTextBox.Text, orgNameTextBox.Text, countryCodeTextBox.Text, keysize, 
                        ManageabilityStack.CertificateOperations.IssueCertType.TlsConfig, digestAlgorithm);
                }
                else if (certTypeComboBox.SelectedIndex == 4)
                {
                    // All Permissions Certificate
                    Certificate = ManageabilityStack.CertificateOperations.IssueSignedCert(RootCertificate, 
                        commonNameTextBox.Text, orgNameTextBox.Text, countryCodeTextBox.Text, keysize, 
                        ManageabilityStack.CertificateOperations.IssueCertType.AllPermissions, digestAlgorithm);
                }
                else if (certTypeComboBox.SelectedIndex == 5)
                {
                    // Open AMT Switchbox certificate
                    Certificate = ManageabilityStack.CertificateOperations.IssueSignedCert(RootCertificate, 
                        commonNameTextBox.Text, orgNameTextBox.Text, countryCodeTextBox.Text, keysize, 
                        ManageabilityStack.CertificateOperations.IssueCertType.TlsServer,digestAlgorithm);
                }
            }
            DialogResult = DialogResult.OK;
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
        }

        private void certTypeComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (certTypeComboBox.SelectedIndex == 0)
            {
                // Sub-CA Certificate
                commonNameTextBox.Text = Properties.Resources.SampleSubCaCertificate;
            }
            else
            {
                if (Utils.GetLocalFqdn() != null)
                {
                    commonNameTextBox.Text = Utils.GetLocalFqdn();
                }
                else
                {
                    commonNameTextBox.Text = Properties.Resources.SampleComputerName;
                }
            }
            UpdateInfo();
        }

    }
}
