// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class DeviceManagerForm : Form
    {
        private SerialHighDriver serial;
        private AmtSystem computer;

        public DeviceManagerForm(SerialHighDriver serial, AmtSystem computer)
        {
            this.serial = serial;
            this.computer = computer;
            InitializeComponent();

            // If the computer is disconnected, close this form
            this.computer = computer;
            if (computer != null) computer.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }

        private void DeviceManagerForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;

            // Get the device list
            serial.ProcessDataBlockEvent += new SerialHighDriver.ProcessDataBlockHandler(ProcessDataBlockHandler);
            serial.WriteDataBlock("DEVC", null);
            toolStripStatusLabel.Text = ManageabilityControls.Properties.Resources.Dev_Mgr_Loading;
        }

        private void SystemStateChangedHandler(AmtSystem computer) { try { this.Invoke(new AmtSystem.ObjectStateHandler(SystemStateChangedHandlerEx), new object[] { computer }); } catch (Exception) { } }
        private void SystemStateChangedHandlerEx(AmtSystem computer) { if (computer.State != AmtSystemObjState.Connected) Close(); }

        private void ProcessDataBlockHandler(SerialHighDriver sender, string name, byte[] data)
        {
            // Speed up by context switching only if this is a message we process.
            if (name.Length == 4 && ((name.CompareTo("DENY") == 0) || (name.CompareTo("DEVC") == 0) || (name.CompareTo("UPDT") == 0)))
            {
                this.Invoke(new SerialHighDriver.ProcessDataBlockHandler(ProcessDataBlockHandlerEx), new object[] { sender, name, data });
            }
        }

        private void ProcessDataBlockHandlerEx(SerialHighDriver sender, string name, byte[] data)
        {
            if (name.CompareTo("DENY") == 0)
            {
                MessageBox.Show(this, Properties.Resources.AccessDenied);
            }
            else if (name.CompareTo("DEVC") == 0)
            {
                toolStripStatusLabel.Text = "";
                DeviceTreeView.BeginUpdate();
                DeviceTreeView.Nodes.Clear();
                Fill_DeviceTreeView(data);
                DeviceTreeView.EndUpdate();
            }
            else if (name.CompareTo("UPDT") == 0)
            {
                DeviceTreeView.Nodes.Clear();
                serial.WriteDataBlock("DEVC", null);
                toolStripStatusLabel.Text = Properties.Resources.Loading;
            }
        }

        private void DeviceManagerForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            serial.ProcessDataBlockEvent -= new SerialHighDriver.ProcessDataBlockHandler(ProcessDataBlockHandler);
            serial = null;
            if (computer != null) computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }

        /*[DllImport("setupapi.dll")]
        static extern bool SetupDiLoadClassIcon(
            ref Guid ClassGuid, ref IntPtr LargeIcon, ref int MiniIconIndex);*/

        private const int DN_STARTED = (0x00000008);
        private const int DN_HAS_PROBLEM = (0x00000400);
        private const int DN_PRIVATE_PROBLEM = (0x00008000);

        private const int CM_PROB_DISABLED = (0x00000016);

        private void Fill_DeviceTreeView(byte[] data)
        {
            string s = UTF8Encoding.UTF8.GetString(data);
            string[] devc = s.Split('\r');

            int iconIndex = 54; // Get root icon

            foreach (string devd in devc)
            {
                string[] pp = devd.Split('|');

                if (pp.Length == 1 && pp[0] != "")
                {
                    TreeNode rootNode = new TreeNode(pp[0], iconIndex, iconIndex);
                    rootNode.ContextMenuStrip = hostContextMenu;
                    DeviceTreeView.Nodes.Add(rootNode);
                    continue;
                }

                if (pp.Length != 1)
                {
                    iconIndex = deviceIconList2.Images.IndexOfKey(pp[3]);

                    TreeNode treeNode = new TreeNode(pp[1], iconIndex, iconIndex);
                    treeNode.Tag = pp[2];
                    treeNode.Name = pp[3];
                    DeviceTreeView.Nodes[0].Nodes.Add(treeNode);

                    int devNum = int.Parse(pp[0]);
                    for (int idd = 4; idd < (devNum * 4) + 4; idd += 4)
                    {
                        if (pp[1].Equals("Network adapters") &&
                            (pp[idd + 3].Contains("Miniport") || pp[idd + 3].Contains("Direct Parallel")))
                        {
                            continue;
                        }
                        if (pp[1].Equals("Sound, video and game controllers") && pp[idd + 3].Contains("Microsoft"))
                        {
                            continue;
                        }

                        UInt32 status = UInt32.Parse(pp[idd + 1]);
                        UInt32 problem = UInt32.Parse(pp[idd + 2]);
                        string nodeNamesuffix = "";
                        bool expand = false;
                        int iconIndex2 = iconIndex;

                        //
                        // handle off the status/problem codes
                        //
                        if (((status & DN_HAS_PROBLEM) != 0) && problem == CM_PROB_DISABLED)
                        {
                            nodeNamesuffix = Properties.Resources.DriverDisabled;
                            iconIndex2 = 53;
                            expand = true;
                        }
                        else if ((status & DN_HAS_PROBLEM) != 0)
                        {
                            nodeNamesuffix = Properties.Resources.DriverProblem;
                            iconIndex2 = 52;
                            expand = true;
                        }
                        //if (status & DN_PRIVATE_PROBLEM)
                        //{ }
                        if ((status & DN_STARTED) != 0)
                        {
                            // Do nothing
                        }

                        TreeNode subTreeNode = new TreeNode(Utils.FilterStrEx(pp[idd + 3]) + nodeNamesuffix, iconIndex2, iconIndex2);
                        subTreeNode.Tag = pp[idd];
                        subTreeNode.ContextMenuStrip = deviceContextMenu;
                        int k = DeviceTreeView.Nodes[0].Nodes.Count;
                        DeviceTreeView.Nodes[0].Nodes[k - 1].Nodes.Add(subTreeNode);

                        if (expand)
                        {
                            DeviceTreeView.Nodes[0].Nodes[k - 1].Expand();
                        }
                    }
                }
            }
            DeviceTreeView.Nodes[0].Expand();
        }

        private void DeviceTreeView_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                TreeNode node = DeviceTreeView.GetNodeAt(e.X, e.Y);
                if (node == null)
                {
                    return;
                }

                DeviceTreeView.SelectedNode = node;

                if (DeviceTreeView.SelectedNode.Text.Contains(" - DISABLED"))
                {
                    enableMenuItem.Enabled = true;
                    disableMenuItem.Enabled = false;
                }
                else
                {
                    enableMenuItem.Enabled = false;
                    disableMenuItem.Enabled = true;
                }
            }

            return;
        }

        private void hwrescanMenuItem_Click(object sender, EventArgs e)
        {
            serial.WriteDataBlock("HWRS", null);
        }

        private void uninstallMenuItem_Click(object sender, EventArgs e)
        {
            string s = "";

            s += DeviceTreeView.SelectedNode.Parent.Tag.ToString() + "|";
            s += DeviceTreeView.SelectedNode.Tag.ToString();

            serial.WriteDataBlock("RMVD", UTF8Encoding.UTF8.GetBytes(s));
        }

        private void enableMenuItem_Click(object sender, EventArgs e)
        {
            string s = "";

            s += DeviceTreeView.SelectedNode.Parent.Tag.ToString() + "|";
            s += DeviceTreeView.SelectedNode.Tag.ToString();

            serial.WriteDataBlock("ENAB", UTF8Encoding.UTF8.GetBytes(s));
        }

        private void disableMenuItem_Click(object sender, EventArgs e)
        {
            string s = "";

            s += DeviceTreeView.SelectedNode.Parent.Tag.ToString() + "|";
            s += DeviceTreeView.SelectedNode.Tag.ToString();

            serial.WriteDataBlock("DSAB", UTF8Encoding.UTF8.GetBytes(s));
        }

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void refreshToolStripMenuItem_Click(object sender, EventArgs e)
        {
            serial.WriteDataBlock("DEVC", null);
            toolStripStatusLabel.Text = ManageabilityControls.Properties.Resources.Dev_Mgr_Loading;
        }
    }
}
