// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Xml;
using System.IO;
using System.Text;
using System.Data;
using System.Drawing;
using System.Threading;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Xml.Serialization;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class DiagnosticViewerForm : Form
    {
        private AmtSystem computer = null;
        private SerialHighDriver serial = null;
        private bool disposed = true;
        private bool local = false;
        private int PendingRequests = 0;
        private ManagementData[] offLineCache = null;
        private ArrayList possibleQueries = null;
        private string[] AF_Providers = null;
        private Hashtable AF_ProvidersQueries = new Hashtable();
        private string customQuery;

        /*
        IntelDP.OSInformation|Get All OS Information
        IntelDP.OSInformation|Get Current OS Time
        IntelDP.Win32ServiceInfo|Get All Win32 Service Information
        IntelDP.Win32ProcessInfo|Get All Win32 Process Information
        IntelDP.Win32DeviceManager|Get All Currently Present Win32 Devices
        IntelDP.WMIProvider|Get All Restore Points
        IntelDP.WMIProvider|Get Last 50 Windows Event Log System Error Events
        IntelDP.WMIProvider|Get Last 50 Application Crash or Hang Events
        IntelDP.WMIProvider|Get Last 50 System Crash Events
        IntelDP.SoftwareInfo|Get All AutoRun Programs
        IntelDP.SoftwareInfo|Get All Installed Software

        DiagnosticAgent|GetNames
        IntelDP.OSInformation|GetQueries
        */

        public DiagnosticViewerForm()
        {
            local = true;
            InitializeComponent();
            possibleQueries = new ArrayList(customRequestComboBox.Items);
            UpdateStatusLabel();
        }

        public DiagnosticViewerForm(byte[] buf)
        {
            local = false;
            InitializeComponent();
            possibleQueries = new ArrayList(customRequestComboBox.Items);

            offLineCache = ManagementData.GetFromBinary(buf);
            UpdateComboBox();
            UpdateStatusLabel();
        }

        public DiagnosticViewerForm(string xml)
        {
            local = false;
            InitializeComponent();
            possibleQueries = new ArrayList(customRequestComboBox.Items);

            offLineCache = ManagementData.GetFromXml(xml);
            UpdateComboBox();
            UpdateStatusLabel();
        }

        public DiagnosticViewerForm(SerialHighDriver serial, string title, AmtSystem computer)
        {
            local = false;
            if (title != null) this.Text += " - " + title;
            InitializeComponent();
            possibleQueries = new ArrayList(customRequestComboBox.Items);

            newToolStripMenuItem.Visible = false;
            openToolStripMenuItem.Visible = false;

            // If the computer is disconnected, close this form
            this.computer = computer;
            this.serial = serial;
            if (computer != null) computer.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
            if (serial != null) serial.ProcessDataBlockEvent += new SerialHighDriver.ProcessDataBlockHandler(ProcessDataBlockHandler);
            UpdateStatusLabel();
        }

        private void SystemStateChangedHandler(AmtSystem computer) { try { this.Invoke(new AmtSystem.ObjectStateHandler(SystemStateChangedHandlerEx), new object[] { computer }); } catch (Exception) { } }
        private void SystemStateChangedHandlerEx(AmtSystem computer) {if (computer.State != AmtSystemObjState.Connected) Close();}

        private void WindowsManagementForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            UpdateStatusLabel();
            disposed = false;

            if (serial != null)
            {
                SOLQuery q = new SOLQuery("DiagnosticAgent", "GetNames", null);
                q.Sequence = 10000;
                serial.WriteDataBlock("DIAG", q.Serialize());
            }
        }


        private void ProcessDataBlockHandler(SerialHighDriver sender, string name, byte[] data)
        {
            // Speed up by context switching only if this is a message we process.
            if (name.Length == 4 && (name.Equals("WMIQ") || name.Equals("DIAG")) && disposed == false)
            {
                this.Invoke(new SerialHighDriver.ProcessDataBlockHandler(ProcessDataBlockHandlerEx), new object[] { sender, name, data });
            }
        }

        private void ProcessDataBlockHandlerEx(SerialHighDriver sender, string name, byte[] data)
        {
            if (name.Equals("WMIQ"))
            {
                DisplayManagementData(ManagementData.GetFromBinary(data)[0]);
            }
            else if (name.Equals("DIAG"))
            {
                SOLResponse r = new SOLResponse(data);
                if (r.HRESULT == 0)
                {
                    if (r.XmlData != null && r.XmlData.Length > 0) DisplayManagementData(r.XmlData[0]);
                    else if (r.Sequence == 10000 && r.StringData != null && r.StringData.Length > 0)
                    {
                        AF_ProvidersQueries.Clear();
                        AF_Providers = r.StringData;
                        for (int i = 0; i < AF_Providers.Length; i++)
                        {
                            SOLQuery q = new SOLQuery(AF_Providers[i], "GetQueries", null);
                            q.Sequence = (ushort)(10001 + i);
                            serial.WriteDataBlock("DIAG", q.Serialize());
                        }
                    }
                    else if (r.Sequence > 10000 && r.Sequence < 11000 && AF_Providers != null)
                    {
                        AF_ProvidersQueries[AF_Providers[r.Sequence - 10001]] = r.StringData;
                        if ((r.Sequence - 10001) == (AF_Providers.Length - 1)) UpdateComboBox();
                    }
                }
            }
        }

        private void DisplayManagementData(ManagementData mdata)
        {
            this.Invoke(new DisplayManagementDataHandler(DisplayManagementDataEx), new object[] { mdata });
        }

        private delegate void DisplayManagementDataHandler(ManagementData mdata);
        private void DisplayManagementDataEx(ManagementData mdata)
        {
            if (mdata == null || mdata.Data == null || mdata.Data.Length == 0)
            {
                customListView.Columns.Clear();
                customListView.Items.Clear();
            }
            else if (mdata.Data.Length == 1)
            {
                // Single column view
                customListView.Columns.Clear();
                customListView.Items.Clear();

                ColumnHeader c1 = new ColumnHeader();
                c1.Text = "Name";
                c1.Width = 150;
                customListView.Columns.Add(c1);

                ColumnHeader c2 = new ColumnHeader();
                c2.Text = "Value";
                customListView.Columns.Add(c2);
                c2.Width = 350;

                Hashtable t = mdata.Data[0];
                foreach (string key in t.Keys)
                {
                    ListViewItem l = new ListViewItem(new string[] { key, (string)t[key] });
                    customListView.Items.Add(l);
                }
            }
            else
            {
                // Multi column view
                customListView.Columns.Clear();
                customListView.Items.Clear();

                // Add columns
                Hashtable t = mdata.Data[0];
                foreach (string key in t.Keys)
                {
                    ColumnHeader c = new ColumnHeader();
                    c.Text = key;
                    c.Width = 150;
                    customListView.Columns.Add(c);
                }

                // Add items
                ArrayList items = new ArrayList();
                foreach (Hashtable h in mdata.Data)
                {
                    ListViewItem l = new ListViewItem();
                    int idx = 0;
                    foreach (ColumnHeader c in customListView.Columns)
                    {
                        if (idx == 0) l.Text = (string)h[c.Text]; else l.SubItems.Add((string)h[c.Text]);
                        idx++;
                    }
                    items.Add(l);
                }
                customListView.Items.AddRange((ListViewItem[])items.ToArray(typeof(ListViewItem)));
            }

            PendingRequests--;
            UpdateStatusLabel();
        }

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void UpdateStatusLabel()
        {
            string status = "";

            if (local == true)
            {
                status = "Local Mode";
            }
            else
            {
                if (serial != null) status = "Interactive Mode"; else status = "Offline Mode";
            }

            if (PendingRequests == 1) status += ", 1 pending request.";
            if (PendingRequests > 1) status += string.Format(", {0} pending requests.", PendingRequests);

            mainToolStripStatusLabel.Text = status;
            Application.DoEvents();
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                // Open the file
                ManagementData[] cache = null;
                FileInfo f = new FileInfo(openFileDialog.FileName);
                if (f.Extension.Equals(".xml"))
                {
                    // Open the XML file
                    FileStream file;
                    try
                    {
                        file = new FileStream(openFileDialog.FileName, FileMode.Open);
                    }
                    catch (Exception ex) { Utils.FileErrorMessageBox(this, ex); return; }
                    byte[] buf = new byte[file.Length];
                    file.Read(buf, 0, buf.Length);
                    file.Close();
                    cache = ManagementData.GetFromXml(UTF8Encoding.UTF8.GetString(buf));
                }
                else if (f.Extension.Equals(".bin"))
                {
                    // Open the BIN file
                    FileStream file;
                    try
                    {
                        file = new FileStream(openFileDialog.FileName, FileMode.Open);
                    }
                    catch (Exception ex) { Utils.FileErrorMessageBox(this, ex); return; }
                    byte[] buf = new byte[file.Length];
                    file.Read(buf, 0, buf.Length);
                    file.Close();
                    cache = ManagementData.GetFromBinary(buf);
                }

                if (cache != null)
                {
                    offLineCache = cache;
                    
                    local = false;
                    serial = null;
                    UpdateStatusLabel();
                    UpdateComboBox();
                }
            }
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            offLineCache = null;
            local = true;
            customRequestComboBox.Items.Clear();
            customRequestComboBox.Items.AddRange((string[])possibleQueries.ToArray(typeof(string)));
            customRequestComboBox.Enabled = true;
            UpdateStatusLabel();
            DisplayManagementDataEx(null);
        }

        private void UpdateComboBox()
        {
            if (AF_ProvidersQueries.Count > 0)
            {
                // Agent Foundations
                ArrayList l = new ArrayList();
                foreach (string provider in AF_ProvidersQueries.Keys)
                {
                    foreach (string query in (string[])(AF_ProvidersQueries[provider]))
                    {
                        if (query.StartsWith("Get")) l.Add(new Utils.DropDownItem(query, 1, provider));
                    }
                }
                customRequestComboBox.Items.Clear();
                customRequestComboBox.Items.AddRange((Utils.DropDownItem[])l.ToArray(typeof(Utils.DropDownItem)));
                customRequestComboBox.Enabled = (customRequestComboBox.Items.Count > 0);
            }
            else
            {
                // Outpost/Guardpost or Offline
                ArrayList l = new ArrayList();
                if (offLineCache != null) foreach (ManagementData datablock in offLineCache)
                {
                    if (datablock.WmiQuery != null)
                    {
                        l.Add(new Utils.DropDownItem(datablock.WmiQuery, 0, datablock));
                    }
                    else
                    {
                        l.Add(new Utils.DropDownItem(datablock.QueryDetail, 0, datablock));
                    }
                }
                customRequestComboBox.Items.Clear();
                customRequestComboBox.Items.AddRange((Utils.DropDownItem[])l.ToArray(typeof(Utils.DropDownItem)));
                customRequestComboBox.Enabled = (customRequestComboBox.Items.Count > 0);
                if (customRequestComboBox.Items.Count > 0) customRequestComboBox.SelectedIndex = 0;
            }
        }

        private void customRequestComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (customRequestComboBox.SelectedItem == null)
            {
                customListView.Columns.Clear();
                customListView.Items.Clear();
                return;
            }

            if (local == true)
            {
                // Local Read
                string query = customRequestComboBox.Text;
                int i = query.IndexOf('|');
                if (i < 0) query = "|" + query;
                customQuery = "WMI|" + query;
                PendingRequests++;
                UpdateStatusLabel();
                new Thread(new ThreadStart(WmiLocalRead)).Start();
            }
            else if (serial != null)
            {
                if (customRequestComboBox.SelectedItem.GetType() == typeof(Utils.DropDownItem))
                {
                    Utils.DropDownItem item = (Utils.DropDownItem)customRequestComboBox.SelectedItem;
                    if (item.Handle == 1)
                    {
                        // Agent Fundation
                        PendingRequests++;
                        UpdateStatusLabel();
                        SOLQuery q = new SOLQuery((string)item.Tag, item.Text, null);
                        serial.WriteDataBlock("DIAG", q.Serialize());
                    }
                }
                else
                {
                    // Serial-over-LAN query
                    PendingRequests++;
                    UpdateStatusLabel();
                    int i = customRequestComboBox.Text.IndexOf("|");
                    if (i >= 0)
                    {
                        customQuery = "WMI|" + customRequestComboBox.Text;
                    }
                    else
                    {
                        customQuery = "WMI||" + customRequestComboBox.Text;
                    }
                    serial.WriteDataBlock("WMIQ|" + customQuery, null);
                }
            }
            else
            {
                // Offline display
                ManagementData data = (ManagementData)((Utils.DropDownItem)customRequestComboBox.SelectedItem).Tag;
                PendingRequests++;
                UpdateStatusLabel();
                DisplayManagementDataEx(data);
            }
        }

        private void WmiLocalRead()
        {
            ManagementData res = ManagementData.GetWmiQuery(customQuery);
            if (res != null) DisplayManagementData(res);
            return;
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                // Save the file
                FileInfo f = new FileInfo(saveFileDialog.FileName);
                if (f.Extension.Equals(".xml"))
                {
                    FileStream file;
                    try
                    {
                        file = new FileStream(saveFileDialog.FileName, FileMode.Create);
                    }
                    catch (Exception ex) { Utils.FileErrorMessageBox(this, ex); return; }
                    byte[] buf = UTF8Encoding.UTF8.GetBytes(ManagementData.ToXml(offLineCache));
                    file.Write(buf, 0, buf.Length);
                    file.Close();
                }
                else if (f.Extension.Equals(".bin"))
                {
                    FileStream file;
                    try
                    {
                        file = new FileStream(saveFileDialog.FileName, FileMode.Create);
                    }
                    catch (Exception ex) { Utils.FileErrorMessageBox(this, ex); return; }
                    byte[] buf = ManagementData.ToBinary(offLineCache);
                    file.Write(buf, 0, buf.Length);
                    file.Close();
                }
            }
        }

        private void DiagnosticViewerForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (computer != null) computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
            if (serial != null) serial.ProcessDataBlockEvent -= new SerialHighDriver.ProcessDataBlockHandler(ProcessDataBlockHandler);
        }

        private void ManageabilityOutpostToolGuardpostToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ManageabilityOutpostToolGuardpostToolStripMenuItem.Checked = true;
            intelAgentFondationToolStripMenuItem.Checked = false;
        }

        private void intelAgentFondationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ManageabilityOutpostToolGuardpostToolStripMenuItem.Checked = false;
            intelAgentFondationToolStripMenuItem.Checked = true;
        }

        private void xToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (serial != null)
            {
                SOLQuery q = new SOLQuery("IntelDP.Win32ServiceInfo", "Get All Win32 Service Information", null);
                serial.WriteDataBlock("DIAG", q.Serialize());
            }
        }

    }

    class SOLQuery
    {
        public SOLQuery(string provider, string query, string[] arguments)
        {
            this.provider = provider;
            this.query = query;
            this.arguments = arguments;
        }

        public byte[] Serialize()
        {
            return FormatMiddleMessage("UTXT", FormatHighMessage());
        }

        private string FormatHighMessage()
        {
            // Format the high protocol query message
            string highMessage = provider + "\r\n";
            highMessage += query + "\r\n";
            if (arguments != null)
            {
                foreach (string arg in arguments)
                {
                    highMessage += arg + "\r\n";
                }
            }
            highMessage += "\r\n";

            return highMessage;
        }

        private byte[] FormatMiddleMessage(string type, string highMessage)
        {
            MemoryStream middleMessage = new MemoryStream();

            // Write message type (query)
            middleMessage.WriteByte((byte)'Q');

            // Write reserved byte
            middleMessage.WriteByte((byte)0);

            // Write sequence number
            middleMessage.Write(LEtoBEs(sequence), 0, 2);

            // Write payload type
            middleMessage.Write(UTF8Encoding.UTF8.GetBytes(type.ToCharArray()), 0, 4);

            // Write payload length and payload
            byte[] payload = null;
            switch (type)
            {
                case "UTXT":
                case "UXML":
                    // Uncompressed payload
                    payload = UTF8Encoding.UTF8.GetBytes(highMessage.ToCharArray());
                    break;
                case "GTXT":
                case "GXML":
                    // GZIP compressed payload
                    int uncompressedLength = UTF8Encoding.UTF8.GetByteCount(highMessage);
                    MemoryStream compressedPayload = new MemoryStream();
                    compressedPayload.Write(LEtoBEl((uint)uncompressedLength), 0, 4);
                    compressedPayload.Write(UTF8Encoding.UTF8.GetBytes(highMessage.ToCharArray()), 0, uncompressedLength);
                    payload = compressedPayload.ToArray();
                    break;
                case "HRES":
                    // HRESULT encoded as an 8 character string
                    payload = UTF8Encoding.UTF8.GetBytes(highMessage.ToCharArray());
                    break;
                default:
                    throw new Exception("FormatMiddleSOLQuery: unknown type");
            }

            // Write payload length
            middleMessage.Write(LEtoBEl((uint)payload.Length), 0, 4);

            // Write payload
            middleMessage.Write(payload, 0, payload.Length);

            // Return formatted message
            return middleMessage.ToArray();
        }

        private byte[] LEtoBEs(ushort le)
        {
            byte[] be = new byte[2];
            be[0] = (byte)(le >> 8);
            be[1] = (byte)(le & 0xFF);
            return be;
        }

        private byte[] LEtoBEl(uint le)
        {
            byte[] be = new byte[4];
            be[0] = (byte)(le >> 24);
            be[1] = (byte)((le >> 16) & 0xFF);
            be[2] = (byte)((le >> 8) & 0xFF);
            be[3] = (byte)(le & 0xFF);
            return be;
        }

        public ushort Sequence
        {
            get { return this.sequence; }
            set { this.sequence = value; }
        }

        private ushort sequence;
        private string provider;
        private string query;
        private string[] arguments;
    }

    class SOLResponse
    {
        public SOLResponse(byte[] message)
        {
            hresult = 0;
            xmlData = null;
            stringData = null;
            ParseMiddleMessage(message);
        }

        public void ParseMiddleMessage(byte[] middleMessage)
        {
            // Parse as a memory stream
            MemoryStream messageBuffer = new MemoryStream(middleMessage);

            // Parse message ID
            id = (byte)messageBuffer.ReadByte();
            if (id != (byte)'R') { throw new Exception("ParseMiddleMessage: invalid message type"); }

            // Parse reserved field
            int reserved = messageBuffer.ReadByte();

            // Parse sequence number
            byte[] seqField = new byte[2];
            messageBuffer.Read(seqField, 0, 2);
            sequence = BEtoLEs(seqField);

            // Parse type field
            byte[] typeField = new byte[4];
            messageBuffer.Read(typeField, 0, 4);
            type = UTF8Encoding.UTF8.GetString(typeField);

            // Parse payload length field
            byte[] lengthField = new byte[4];
            messageBuffer.Read(lengthField, 0, 4);
            uint payloadLength = BEtoLEl(lengthField);

            // Parse payload field
            byte[] payloadField = new byte[payloadLength];
            messageBuffer.Read(payloadField, 0, (int)payloadLength);
            string[] delimiter = new string[] { "\r\n" };
            switch (type)
            {
                case "UTXT":
                    // Convert payload to an array of strings
                    stringData = UTF8Encoding.UTF8.GetString(payloadField).Split(delimiter, StringSplitOptions.RemoveEmptyEntries);
                    break;
                case "UXML":
                    // Convert payload to XML string to Management data
                    xmlData = ManagementData.GetFromXml(UTF8Encoding.UTF8.GetString(payloadField));
                    break;
                case "GTXT":
                    // Uncompress payload and convert to an array of strings
                    string uncompressedPayload = UTF8Encoding.UTF8.GetString(Utils.Decompress(payloadField));
                    uncompressedPayload.Split(delimiter, StringSplitOptions.RemoveEmptyEntries);
                    break;
                case "GXML":
                    // Uncompress payload and convert to XML data
                    xmlData = ManagementData.GetFromBinary(payloadField);
                    break;
                case "HRES":
                    // Convert HRESULT to integer
                    hresult = ConvertPayloadToHRESULT(payloadField);
                    break;
                default:
                    throw new Exception("ParseMiddleMessage: invalid type");
            }
        }

        private ushort BEtoLEs(byte[] be)
        {
            uint le = ((uint)be[0]) << 8;
            le += (uint)be[1];
            return (ushort)le;
        }

        private uint BEtoLEl(byte[] be)
        {
            uint le = ((uint)be[0]) << 24;
            le += ((uint)be[1]) << 16;
            le += ((uint)be[2]) << 8;
            le += (uint)be[3];
            return le;
        }

        private UInt32 ConvertPayloadToHRESULT(byte[] payload)
        {
            if (payload.Length > 8) throw new Exception("ConvertPayloadToHRESULT: payload too long for an HRESULT");

            UInt32 hresult = 0;
            foreach (byte b in payload)
            {
                hresult = (hresult << 4) + (UInt32)(b - '0');
            }

            return hresult;
        }

        public ushort Sequence { get { return sequence; } }
        public string[] StringData { get { return stringData; } }
        public ManagementData[] XmlData { get {return xmlData;} }
        public uint HRESULT { get {return hresult;} }

        private ushort sequence;
        private byte id;
        private string type;
        private uint hresult;
        private ManagementData[] xmlData;
        private string[] stringData;
    }

}
