// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class FileExplorerForm : Form
    {
        private SerialHighDriver serial;
        private AmtSystem computer;

        public FileExplorerForm(SerialHighDriver serial, AmtSystem computer)
        {
            this.serial = serial;
            this.computer = computer;
            InitializeComponent();

            // If the computer is disconnected, close this form
            this.computer = computer;
            if (computer != null) computer.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }

        private void FileExplorerForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            serial.ProcessDataBlockEvent += new SerialHighDriver.ProcessDataBlockHandler(ProcessDataBlockHandler);
            serial.WriteDataBlock("DRIV", null);
        }

        private void SystemStateChangedHandler(AmtSystem computer) { try { this.Invoke(new AmtSystem.ObjectStateHandler(SystemStateChangedHandlerEx), new object[] { computer }); } catch (Exception) { } }
        private void SystemStateChangedHandlerEx(AmtSystem computer) { if (computer.State != AmtSystemObjState.Connected) Close(); }

        private void ProcessDataBlockHandler(SerialHighDriver sender, string name, byte[] data)
        {
            // Speed up by context switching only if this is a message we process.
            if (name.Length == 4 && ((name.CompareTo("DENY") == 0) || (name.CompareTo("EROR") == 0) || (name.CompareTo("DRIV") == 0) || (name.CompareTo("FLLT") == 0)))
            {
                object[] p = new object[3];
                p[0] = sender;
                p[1] = name;
                p[2] = data;
                this.Invoke(new SerialHighDriver.ProcessDataBlockHandler(ProcessDataBlockHandlerEx), p);
            }
        }

        private void ProcessDataBlockHandlerEx(SerialHighDriver sender, string name, byte[] data)
        {
            if (name.CompareTo("DENY") == 0)
            {
                MessageBox.Show(Properties.Resources.PermissionError);
            }
            else if (name.CompareTo("EROR") == 0)
            {
                MessageBox.Show(Properties.Resources.NoPathAccessError);
                fileListView.Items.Clear();
            }
            else if (name.CompareTo("DRIV") == 0 && data != null)
            {
                GetDrive(data);
            }
            else if (name.CompareTo("FLLT") == 0 && data != null)
            {
                Fill_FileListView(data);
                Fill_FileTreeView(data);
            }
        }

        public void GetDrive(byte[] data)
        {
            folderTreeView.BeginUpdate();
            folderTreeView.Nodes.Clear();

            TreeNode RootNode = new TreeNode(Properties.Resources.MyComputerrRootNode, 0, 0);
            folderTreeView.Nodes.Add(RootNode);

            string d = UTF8Encoding.UTF8.GetString(data);
            string[] ds = d.Split('|');

            for (int num = 0; num <= ds.Length - 2; num++)
            {
                TreeNode tnDrive = new TreeNode(ds[num], 1, 1);
                folderTreeView.Nodes[0].Nodes.Add(tnDrive);
            }
            folderTreeView.EndUpdate();
        }

        private void FileExplorerForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            serial.ProcessDataBlockEvent -= new SerialHighDriver.ProcessDataBlockHandler(ProcessDataBlockHandler);
            serial = null;
            if (computer != null) computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }

        private void Fill_FileListView(byte[] data)
        {
            fileListView.BeginUpdate();
            fileListView.Items.Clear();

            string[] infos = UTF8Encoding.UTF8.GetString(data).Split('|');
            for (int i = 0; i < infos.Length - 1; i += 5)
            {
                if (infos[i] != null)
                {
                    if (infos[i + 1].ToString() == "Recycled" || infos[i + 1].ToString() == "RECYCLED" || infos[i + 1].ToString() == "RECYCLER")
                    {
                        continue;
                    }
                    else
                    {
                        string fullname = infos[i];
                        ListViewItem LiItem;
                        if (infos[i + 2].CompareTo("-1") == 0)
                        {
                            LiItem = new ListViewItem(new string[] { infos[i + 1], "", infos[i + 3], infos[i + 4] }, 4);
                        }
                        else
                        {
                            LiItem = new ListViewItem(new string[] { infos[i + 1], infos[i + 2], infos[i + 3], infos[i + 4] }, 2);
                        }
                        LiItem.Tag = fullname;
                        fileListView.Items.Insert(0, LiItem);
                    }
                }
            }
            fileListView.EndUpdate();
        }

        private void Fill_FileTreeView(byte[] data)
        {
            folderTreeView.BeginUpdate();
            string[] ts = UTF8Encoding.UTF8.GetString(data).Split('|');
            folderTreeView.SelectedNode.Nodes.Clear();
            for (int i = 0; i < ts.Length - 1; i += 5)
            {
                if (ts[i] != null)
                {
                    if (ts[i + 2].CompareTo("-1") == 0)
                    {
                        if (ts[i + 1].ToString() == "Recycled" || ts[i + 1].ToString() == "RECYCLED" || ts[i + 1].ToString() == "RECYCLER") continue;
                        TreeNode tn = new TreeNode(ts[i+1], 4, 5);
                        folderTreeView.SelectedNode.Nodes.Insert(0, tn);
                    }
                }
            }
            folderTreeView.EndUpdate();
        }

        private void treeView1_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                TreeNode node = folderTreeView.GetNodeAt(e.X, e.Y);
                folderTreeView.SelectedNode = node;
            }
        }

        private void after_select(object sender, System.Windows.Forms.TreeViewEventArgs e)
        {
            if (e.Node.Text == Properties.Resources.MyComputerrRootNode)
            {
                e.Node.Expand();
            }
            else
            {
                currentPathTextBox.Text = folderTreeView.SelectedNode.FullPath.Remove(0, 12).Replace("\\\\","\\");
                string m = folderTreeView.SelectedNode.FullPath;
                m = m.Remove(0, 12);
                serial.WriteDataBlock("FLLT", UTF8Encoding.UTF8.GetBytes(m));
            }
        }
    }
}
