// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Text;
using System.Drawing;
using System.Diagnostics;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class IAmtTerminalControl : Component
    {
        private TerminalForm form = null;

        public IAmtTerminalControl()
        {
            InitializeComponent();
        }

        public IAmtTerminalControl(IContainer container)
        {
            container.Add(this);
            InitializeComponent();
        }

        /// <summary>
        /// Enable support for Manageability Outpost Tool features
        /// </summary>
        public bool EnableOutpostFeatures
        {
            get { return enableOutpostFeatures; }
            set { enableOutpostFeatures = value; }
        }
        private bool enableOutpostFeatures = false;

        /// <summary>
        /// Intel AMT machine hostname or IP Address
        /// </summary>
        public string Hostname
        {
            get { return hostname; }
            set { hostname = value; }
        }
        private string hostname = null;

        /// <summary>
        /// Intel AMT port number
        /// </summary>
        public ushort Port
        {
            get { return port; }
            set { port = value; }
        }
        private ushort port = 16992;

        /// <summary>
        /// Intel AMT machine login username
        /// </summary>
        public string Username
        {
            get { return username; }
            set { username = value; }
        }
        private string username = null;

        /// <summary>
        /// Intel AMT machine login password
        /// </summary>
        public string Password
        {
            get { return password; }
            set { password = value; }
        }
        private string password = null;

        /// <summary>
        /// Terminal window title
        /// </summary>
        public string Title
        {
            get { return title; }
            set { title = value; }
        }
        private string title = null;

        /// <summary>
        /// The terminal window icon.
        /// </summary>
        public Icon Icon
        {
            get { return icon; }
            set { icon = value; }
        }
        private Icon icon = null;

        public void Show()
        {
            Show(null);
        }

        public void Show(System.Windows.Forms.IWin32Window owner)
        {
            if (form == null || form.TerminalDisposed == true)
            {
                form = null;
                form = new TerminalForm(hostname, port, username, password, title);
                form.EnableOutpostFeatures = enableOutpostFeatures;
                if (icon != null) form.Icon = icon;
                form.Show(owner);
            }
            else
            {
                form.Activate();
            }
        }

        public void Hide()
        {
            if (form != null) form.Hide();
        }

        public void DisposeTerminal()
        {
            if (form != null)
            {
                form.DisposeTerminal();
                form.Dispose();
                form = null;
            }
        }

    }
}
