// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class KerberosConfigForm : Form
    {
        private AmtSystem system;
        private bool readOnly;

        public KerberosConfigForm(AmtSystem system, bool readOnly)
        {
            this.system = system;
            this.readOnly = readOnly;
            InitializeComponent();
        }

        private void UpdateInfo()
        {
            if (readOnly == false)
            {
                realmNameTextBox.Enabled = kerberosEnabledCheckBox.Checked;
                keyVersionNumericUpDown.Enabled = kerberosEnabledCheckBox.Checked;
                tolerenceNumericUpDown.Enabled = kerberosEnabledCheckBox.Checked;
                masterKeyTextBox.Enabled = kerberosEnabledCheckBox.Checked;
                spnListView.Enabled = kerberosEnabledCheckBox.Checked;
                spnOptionTextBox.Enabled = (kerberosEnabledCheckBox.Checked && spnListView.SelectedItems.Count > 0);
                updateOptionButton.Enabled = (kerberosEnabledCheckBox.Checked && spnListView.SelectedItems.Count > 0);
            }
            else
            {
                kerberosEnabledCheckBox.Enabled = false;
                realmNameTextBox.Enabled = false;
                keyVersionNumericUpDown.Enabled = false;
                tolerenceNumericUpDown.Enabled = false;
                masterKeyTextBox.Enabled = false;
                spnListView.Enabled = false;
                spnOptionTextBox.Enabled = false;
                updateOptionButton.Enabled = false;
                okButton.Visible = false;
                cancelButton.Text = Properties.Resources.Close;
                editSpnPanel.Visible = false;
                this.Height -= editSpnPanel.Height;
            }

            bool ok = true;
            if (kerberosEnabledCheckBox.Checked == true && Utils.HexToBytes(masterKeyTextBox.Text).Length != 16) ok = false;
            //if (kerberosEnabledCheckBox.Checked == true && spnListView.Items[0].Checked == false && spnListView.Items[1].Checked == false && spnListView.Items[2].Checked == false && spnListView.Items[3].Checked == false) ok = false;
            okButton.Enabled = ok;
        }

        private void KerberosConfigForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            LoadSettings();
        }

        private void LoadSettings()
        {
            KerberosOptionsType k;
            AmtCallStatus r = system.SecurityAdmin.GetKerberosOptions(out k);
            if (k == null)
            {
                kerberosEnabledCheckBox.Checked = false;
            }
            else
            {
                kerberosEnabledCheckBox.Checked = true;
                realmNameTextBox.Text = k.KerberosRealmName;
                if (k.KerberosKeyVersion >= keyVersionNumericUpDown.Minimum && k.KerberosKeyVersion <= keyVersionNumericUpDown.Maximum) keyVersionNumericUpDown.Value = (decimal)(k.KerberosKeyVersion);
                if (k.KerberosMaximumClockTolerance >= tolerenceNumericUpDown.Minimum && k.KerberosMaximumClockTolerance <= tolerenceNumericUpDown.Maximum) tolerenceNumericUpDown.Value = (decimal)(k.KerberosMaximumClockTolerance);
                masterKeyTextBox.Text = Utils.BytesToHex(k.KerberosMasterKey);

                spnListView.Items[0].Checked = false;
                spnListView.Items[1].Checked = false;
                spnListView.Items[2].Checked = false;
                spnListView.Items[3].Checked = false;

                foreach (KerberosSpnType spi in k.KerberosSpn)
                {
                    if (spi.SpnProtocolIndex == KerberosSpnProtocolType.HTTP) { spnListView.Items[0].SubItems[1].Text = spi.SpnString; spnListView.Items[0].Checked = true; }
                    if (spi.SpnProtocolIndex == KerberosSpnProtocolType.HTTPS) { spnListView.Items[1].SubItems[1].Text = spi.SpnString; spnListView.Items[1].Checked = true; }
                    if (spi.SpnProtocolIndex == KerberosSpnProtocolType.SOL_IDER) { spnListView.Items[2].SubItems[1].Text = spi.SpnString; spnListView.Items[2].Checked = true; }
                    if (spi.SpnProtocolIndex == KerberosSpnProtocolType.SOL_IDER_SSL) { spnListView.Items[3].SubItems[1].Text = spi.SpnString; spnListView.Items[3].Checked = true; }
                }
            }
            UpdateInfo();
        }

        private void kerberosEnabledCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void spnListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateInfo();
            if (spnListView.SelectedItems.Count > 0)
            {
                spnOptionTextBox.Text = spnListView.SelectedItems[0].SubItems[1].Text;
            }
        }

        private void updateOptionButton_Click(object sender, EventArgs e)
        {
            if (spnListView.SelectedItems.Count > 0)
            {
                spnListView.SelectedItems[0].SubItems[1].Text = spnOptionTextBox.Text;
            }
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            KerberosOptionsType k = null;

            if (kerberosEnabledCheckBox.Checked == true)
            {
                k = new KerberosOptionsType();
                k.KerberosRealmName = realmNameTextBox.Text;
                k.KerberosKeyVersion = (uint)(keyVersionNumericUpDown.Value);
                k.KerberosMasterKey = Utils.HexToBytes(masterKeyTextBox.Text);
                k.KerberosEncryption = KerberosEncryptionType.HMAC_RC4;
                k.KerberosMaximumClockTolerance = (uint)(tolerenceNumericUpDown.Value);

                int spicount = 0;
                if (spnListView.Items[0].Checked == true) spicount++;
                if (spnListView.Items[1].Checked == true) spicount++;
                if (spnListView.Items[2].Checked == true) spicount++;
                if (spnListView.Items[3].Checked == true) spicount++;

                int ptr = 0;
                k.KerberosSpn = new KerberosSpnType[spicount];
                if (spnListView.Items[0].Checked == true)
                {
                    k.KerberosSpn[ptr] = new KerberosSpnType();
                    k.KerberosSpn[ptr].SpnProtocolIndex = KerberosSpnProtocolType.HTTP;
                    k.KerberosSpn[ptr].SpnString = spnListView.Items[0].SubItems[1].Text;
                    ptr++;
                }
                if (spnListView.Items[1].Checked == true)
                {
                    k.KerberosSpn[ptr] = new KerberosSpnType();
                    k.KerberosSpn[ptr].SpnProtocolIndex = KerberosSpnProtocolType.HTTPS;
                    k.KerberosSpn[ptr].SpnString = spnListView.Items[1].SubItems[1].Text;
                    ptr++;
                }
                if (spnListView.Items[2].Checked == true)
                {
                    k.KerberosSpn[ptr] = new KerberosSpnType();
                    k.KerberosSpn[ptr].SpnProtocolIndex = KerberosSpnProtocolType.SOL_IDER;
                    k.KerberosSpn[ptr].SpnString = spnListView.Items[2].SubItems[1].Text;
                    ptr++;
                }
                if (spnListView.Items[3].Checked == true)
                {
                    k.KerberosSpn[ptr] = new KerberosSpnType();
                    k.KerberosSpn[ptr].SpnProtocolIndex = KerberosSpnProtocolType.SOL_IDER_SSL;
                    k.KerberosSpn[ptr].SpnString = spnListView.Items[3].SubItems[1].Text;
                    ptr++;
                }
            }

            AmtCallStatus r = system.SecurityAdmin.SetKerberosOptions(k);
            if (r != AmtCallStatus.SUCCESS)
            {
                Utils.ErrorMessageBox(this, "SetKerberosOptions", r);
            }
            else
            {
                DialogResult = DialogResult.OK;
            }
        }

        private void masterKeyTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }
    }
}
