// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class MeInfoForm : Form
    {
        private HeciWrapper ManagementEngine;

        public MeInfoForm()
        {
            ManagementEngine = new HeciWrapper();
            InitializeComponent();
        }

        public MeInfoForm(HeciWrapper ManagementEngine)
        {
            this.ManagementEngine = ManagementEngine;
            InitializeComponent();
        }

        private void MeInfoForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            if (ManagementEngine == null)
            {
                AddValue(Properties.Resources.NotSupported, "", null);
                return;
            }

            MeCoreVersions versions = ManagementEngine.Versions;
            if (versions == null)
            {
                AddValue(Properties.Resources.NotSupported, "", null);
                return;
            }

            if (ManagementEngine.MeInfo != null) ManagementEngine.MeInfo.GetAmtState();

            ListViewGroup g1 = new ListViewGroup(Properties.Resources.Versions);
            mainListView.Groups.Add(g1);
            AddValue(Properties.Resources.HeciVersion, ManagementEngine.major.ToString() + "." + ManagementEngine.minor.ToString() + "." + ManagementEngine.hotfix, g1);

            // Versions
            AddValue(Properties.Resources.BiosVersion, versions.BiosVersion, g1);
            foreach (string key in versions.Versions.Keys) AddValue(key, (string)(versions.Versions[key]), g1);

            // Provisioning Info
            if (ManagementEngine.MeInfo == null) return;
            MeProvisioningMode pm = ManagementEngine.MeInfo.GetProvisioningMode();
            ListViewGroup g2 = new ListViewGroup(Properties.Resources.Provisioning);
            mainListView.Groups.Add(g2);
            AddValue(Properties.Resources.ProvisioningMode, pm.ProvisioningMode.ToString(), g2);
            HeciMeInfo.ProvisioningStateEnum ps = ManagementEngine.MeInfo.GetProvisioningState();
            AddValue(Properties.Resources.ProvisioningState, ps.ToString(), g2);

            MeMacAddresses macs = ManagementEngine.MeInfo.GetMacAddresses();
            if (macs.Host != null) AddValue(Properties.Resources.HostMac, Utils.BytesToHex(macs.Host), g2);
            if (macs.Dedicated != null) AddValue(Properties.Resources.DedicatedMac, Utils.BytesToHex(macs.Dedicated), g2);

            if (ManagementEngine.GetVersion().CompareTo((decimal)2.5) <= 0) return;

            bool ztc = ManagementEngine.MeInfo.GetZeroTouchEnabled();
            AddValue(Properties.Resources.RemoteConfiguration, (ztc ? Properties.Resources.Enabled : Properties.Resources.Disabled), g2);
            string PkiFqdnSuffix = ManagementEngine.MeInfo.GetPkiFqdnSuffix();
            if (PkiFqdnSuffix == null) PkiFqdnSuffix = Properties.Resources.NoElement;
            AddValue(Properties.Resources.PkiFqdnSuffix, PkiFqdnSuffix.ToString(), g2);

            HeciMeInfo.RandomGeneratorStatus gn = ManagementEngine.MeInfo.GetRandomGeneratorStatus();
            AddValue(Properties.Resources.RandomGenerator, gn.ToString(), g2);

            //if (ManagementEngine.GetVersion().CompareTo((decimal)3.0) >= 0)
            //{
                string cs = ManagementEngine.MeInfo.GetConfigurationServerFqdn();
                if (cs == null) cs = Properties.Resources.NoElement;
                AddValue(Properties.Resources.ConfigurationServer, cs, g2);
            //}

            // Audit Record Info
            MeAuditRecord par = ManagementEngine.MeInfo.GetAuditRecord();
            if (par != null)
            {
                ListViewGroup g3 = new ListViewGroup(Properties.Resources.ProvisioningAuditRecord);
                mainListView.Groups.Add(g3);
                AddValue(Properties.Resources.TlsMode, par.ProvisioningTLSMode.ToString(), g3);
                AddValue(Properties.Resources.TlsStartTime, par.TlsStartTime.ToString(), g3);
                AddValue(Properties.Resources.ProvisioningServer, par.ProvServerIP.ToString(), g3);
                AddValue(Properties.Resources.SecureDNS, par.SecureDNS.ToString(), g3);
                AddValue(Properties.Resources.ServerFQDN, par.ProvServerFQDN, g3);
                AddValue(Properties.Resources.OemDefault, par.IsOemDefault.ToString(), g3);
                AddValue(Properties.Resources.HostInitiated, par.HostInitiated.ToString(), g3);
            }

            // Provisioning Hashes
            int h = 1;
            MeHashEntry[] l = ManagementEngine.MeInfo.EnumerateHashHandles();
            foreach (MeHashEntry he in l)
            {
                ListViewGroup g = new ListViewGroup(string.Format(Properties.Resources.TrustedConfigurationCertificateTitle, h.ToString()));
                mainListView.Groups.Add(g);
                AddValue(Properties.Resources.Name, he.Name, g);
                AddValue(string.Format("{0} ({1})", Properties.Resources.Hash, he.HashAlgorithm.ToString()), Utils.BytesToHex(he.Hash), g);
                AddValue(Properties.Resources.Default, he.IsDefault.ToString(), g);
                AddValue(Properties.Resources.Active, he.IsActive.ToString(), g);
                h++;
            }

            mainListView.Focus();
        }

        private void AddValue(string name, string value, ListViewGroup group)
        {
            ListViewItem l = new ListViewItem(new string[] { name, value });
            l.Group = group;
            mainListView.Items.Add(l);
        }

        private void selectAllToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            if (mainListView.Items.Count != 0)
            {
                mainListView.Focus();
                mainListView.Items[0].Selected = true;
                SendKeys.Send("{HOME}");
                SendKeys.Send("+{END}");
            }
        }

        private void copySelectedNamesValuesToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            if (mainListView.SelectedItems.Count == 0) return;
            string clip = "";
            foreach (ListViewItem l in mainListView.SelectedItems)
            {
                clip += l.Text + "\t" + l.SubItems[1].Text + "\r\n";
            }
            Clipboard.SetDataObject(clip);
        }

        private void copySelectedValuesOnlyToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            if (mainListView.SelectedItems.Count == 0) return;
            string clip = "";
            foreach (ListViewItem l in mainListView.SelectedItems)
            {
                clip += l.SubItems[1].Text + "\r\n";
            }
            Clipboard.SetDataObject(clip);
        }

        private void contextMenuStrip_Opening(object sender, CancelEventArgs e)
        {
            copySelectedValuesOnlyToolStripMenuItem2.Enabled = (mainListView.SelectedItems.Count > 0);
            copySelectedNamesValuesToolStripMenuItem2.Enabled = (mainListView.SelectedItems.Count > 0);
            selectAllToolStripMenuItem2.Enabled = (mainListView.Items.Count > 0);
        }

        private void mainListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.A)
            {
                if (mainListView.Items.Count != 0)
                {
                    e.Handled = true;
                    mainListView.Focus();
                    mainListView.Items[0].Selected = true;
                    SendKeys.Send("{HOME}");
                    SendKeys.Send("+{END}");
                }
            }
        }

    }
}
