// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Net;
using System.Data;
using System.Text;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Net.NetworkInformation;
using System.Security.Cryptography.X509Certificates;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class NewCertificateForm : Form
    {
        private bool sha2Support = false;
        private X509Store CertificateStore;
        private X509Store CertificateRootStore;
        private int keysize = 1024;
        private bool keySizeEnabled = true;

        public bool SHA2Support
        {
            get { return this.sha2Support; }
            set { this.sha2Support = value; }
        }

        public X509Certificate2 Certificate;
        public string CertificateName
        {
            get { return certificateNameComboBox.Text; }
            set { certificateNameComboBox.Text = value; }
        }

        public NewCertificateForm()
        {
            InitializeComponent();
        }

        public void LockKeySize(int keysize)
        {
            this.keysize = keysize;
            keySizeEnabled = false;
            keySizeActiveTrackBar.Enabled = keySizeEnabled;

            switch (keysize)
            {
                case 1024:
                    keySizeActiveTrackBar.Value = 0;
                    break;
                case 1536:
                    keySizeActiveTrackBar.Value = 1;
                    break;
                case 2048:
                    keySizeActiveTrackBar.Value = 2;
                    break;
            }
        }

        private void SwitchboxCertificateForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;

            // Open the certificate store
            CertificateStore = new X509Store(StoreName.My, StoreLocation.CurrentUser);
            CertificateStore.Open(OpenFlags.ReadWrite);

            // Open the certificate root store
            CertificateRootStore = new X509Store(StoreName.Root, StoreLocation.CurrentUser);
            CertificateRootStore.Open(OpenFlags.ReadWrite);

            // Add this computer's name to the combo box
            IPHostEntry entry = null;
            try
            {
                entry = System.Net.Dns.GetHostEntry("");
            }
            catch (Exception) { }
            if (entry != null && entry.HostName.IndexOf(".") > 0)
            {
                // This computer is probably part of a domain.
                if (!certificateNameComboBox.Items.Contains(entry.HostName)) certificateNameComboBox.Items.Add(entry.HostName);
            }

            // Fill in the certificate name combo box with other possible names
            NetworkInterface[] netinterfaces = NetworkInterface.GetAllNetworkInterfaces();
            if (netinterfaces != null && netinterfaces.Length > 0)
            {
                // Set all the names
                foreach (NetworkInterface n in netinterfaces)
                {
                    string suffix = n.GetIPProperties().DnsSuffix;
                    if (suffix != null && suffix.Length > 0)
                    {
                        string s = System.Net.Dns.GetHostName() + "." + suffix;
                        if (!certificateNameComboBox.Items.Contains(s)) certificateNameComboBox.Items.Add(s);
                    }
                }
            }

            // Update the certificate list on screen
            UpdateCertificateList();

            switch (keysize)
            {
                case 1024:
                    keySizeActiveTrackBar.Value = 0;
                    break;
                case 1536:
                    keySizeActiveTrackBar.Value = 1;
                    break;
                case 2048:
                    keySizeActiveTrackBar.Value = 2;
                    break;
            }

            keySizeActiveTrackBar.Enabled = keySizeEnabled;
            UpdateInfo();

            this.digestComboBox.Enabled = this.sha2Support;
            this.digestComboBox.SelectedIndex = 0;
        }

        private void UpdateCertificateList()
        {
            // Load the list is issuer certificates
            issuerComboBox.BeginUpdate();
            issuerComboBox.Items.Clear();
            issuerComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.IssueNoCertificate, 0, null));
            issuerComboBox.SelectedIndex = 0;
            foreach (X509Certificate2 cert in CertificateStore.Certificates)
            {
                if (cert.HasPrivateKey && CertificateOperations.IsTrustedCert(cert) && CertificateOperations.IsCaCertificate(cert) && CertificateOperations.IsKeyExportable(cert))
                {
                    Utils.DropDownItem i = new Utils.DropDownItem(CertificateOperations.GetCertificateString(cert), 0, cert);
                    issuerComboBox.Items.Add(i);
                }
            }
            foreach (X509Certificate2 cert in CertificateRootStore.Certificates)
            {
                if (cert.HasPrivateKey && CertificateOperations.IsTrustedCert(cert) && CertificateOperations.IsCaCertificate(cert))
                {
                    Utils.DropDownItem i = new Utils.DropDownItem(CertificateOperations.GetCertificateString(cert), 0, cert);
                    issuerComboBox.Items.Add(i);
                }
            }
            if (issuerComboBox.Items.Count > 1) issuerComboBox.SelectedIndex = 1;
            issuerComboBox.EndUpdate();
        }

        private void SwitchboxCertificateForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            // Close the certificate stores
            CertificateStore.Close();
            CertificateStore = null;
            CertificateRootStore.Close();
            CertificateRootStore = null;
        }

        private void keySizeActiveTrackBar_Scroll(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void UpdateInfo()
        {
            switch (keySizeActiveTrackBar.Value)
            {
                case 0:
                    keysize = 1024;
                    break;
                case 1:
                    keysize = 1536;
                    break;
                case 2:
                    keysize = 2048;
                    break;
            }
            keySizeLabel.Text = string.Format(Properties.Resources.NumBytes, keysize.ToString());

            orgNameTextBox.Enabled = (issuerComboBox.SelectedIndex != 0);
            countryCodeTextBox.Enabled = (issuerComboBox.SelectedIndex != 0);
            keySizeActiveTrackBar.Enabled = ((issuerComboBox.SelectedIndex != 0) && keySizeEnabled);
            keySizeLabel.Enabled = (issuerComboBox.SelectedIndex != 0);
            okButton.Enabled = (issuerComboBox.SelectedIndex != 0);
        }

        private void issuerComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            string ok = okButton.Text;
            okButton.Text = Properties.Resources.WaitButton;
            okButton.Enabled = false;
            cancelButton.Enabled = false;
            X509Certificate2 rootcert = (X509Certificate2)(((Utils.DropDownItem)issuerComboBox.SelectedItem).Tag);

            if (this.sha2Support)
            {
                Certificate = ManageabilityStack.CertificateOperations.IssueSignedCert(rootcert, 
                    certificateNameComboBox.Text, orgNameTextBox.Text, countryCodeTextBox.Text, 
                    keysize, ManageabilityStack.CertificateOperations.IssueCertType.AllPermissions,
                    (CertificateOperations.SignatureAlgorithm)Enum.Parse(typeof(CertificateOperations.SignatureAlgorithm),
                        this.digestComboBox.SelectedItem.ToString()));
            }
            else
            {
                Certificate = ManageabilityStack.CertificateOperations.IssueSignedCert(rootcert, 
                    certificateNameComboBox.Text, orgNameTextBox.Text, countryCodeTextBox.Text, 
                    keysize, ManageabilityStack.CertificateOperations.IssueCertType.AllPermissions);
            }
            if (Certificate != null) DialogResult = DialogResult.OK;

            okButton.Text = ok;
            okButton.Enabled = true;
            cancelButton.Enabled = true;
            UpdateInfo();
        }

        private void newRootCertificateButton_Click(object sender, EventArgs e)
        {
            ManageabilityControls.CreateCertificateForm form = new CreateCertificateForm(null);
            form.SHA2Support = this.sha2Support;
            if (form.ShowDialog(this) == DialogResult.OK)
            {
                if (form.Certificate != null)
                {
                    CertificateStore.Add(form.Certificate);

                    if (form.TrustedRoot() == true)
                    {
                        // Add this certificate to trusted root store.
                        try
                        {
                            CertificateRootStore.Add(form.Certificate);
                        }
                        catch (Exception) { }
                    }

                    UpdateCertificateList();
                }
            }
        }

    }
}
