// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class ProcessMonitorForm : Form
    {
        private SerialHighDriver serial;
        private AmtSystem computer;

        public ProcessMonitorForm(SerialHighDriver serial, AmtSystem computer)
        {
            this.serial = serial;
            this.computer = computer;
            InitializeComponent();
        }

        private void ProcessMonitorForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;

            // If the computer is disconnected, close this form
            if (computer != null) computer.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);

            // Get the proc list
            serial.ProcessDataBlockEvent += new SerialHighDriver.ProcessDataBlockHandler(ProcessDataBlockHandler);
            serial.WriteDataBlock("PROC", null);
        }

        private void SystemStateChangedHandler(AmtSystem computer) { try { this.Invoke(new AmtSystem.ObjectStateHandler(SystemStateChangedHandlerEx), new object[] { computer }); } catch (Exception) { } }
        private void SystemStateChangedHandlerEx(AmtSystem computer) { if (computer.State != AmtSystemObjState.Connected) Close(); }

        private void ProcessDataBlockHandler(SerialHighDriver sender, string name, byte[] data)
        {
            // Speed up by context switching only if this is a message we process.
            if (name.Length == 4 && name.CompareTo("PROC") == 0)
            {
                object[] p = new object[3];
                p[0] = sender;
                p[1] = name;
                p[2] = data;
                this.Invoke(new SerialHighDriver.ProcessDataBlockHandler(ProcessDataBlockHandlerEx), p);
            }
        }

        private void ProcessDataBlockHandlerEx(SerialHighDriver sender, string name, byte[] data)
        {
            if (name.CompareTo("PROC") == 0)
            {
                string s = UTF8Encoding.UTF8.GetString(data);
                string[] ps = s.Split('\r');
                Hashtable pt = new Hashtable();
                foreach (string pps in ps)
                {
                    string[] pp = pps.Split('|');
                    if (pp.Length == 11) pt.Add(int.Parse(pp[0]), pp);
                }

                ArrayList removeList = new ArrayList();
                foreach (ListViewItem l in processListView.Items)
                {
                    int id = int.Parse(l.SubItems[1].Text);
                    if (pt.Contains(id))
                    {
                        string[] pp = (string[])pt[id];
                        bool responding = bool.Parse(pp[8]);
                        int memory = int.Parse(pp[10]) / 1024;
                        string nr = "";
                        if (responding == false) nr = "[NR] ";
                        if (!(l.SubItems[0].Text.Equals(nr + pp[1]))) l.SubItems[0].Text = nr + pp[1];
                        if (!(l.SubItems[2].Text.Equals(memory.ToString() + "k"))) l.SubItems[2].Text = memory.ToString() + "k";
                        if (!(l.SubItems[3].Text.Equals(pp[9]))) l.SubItems[3].Text = pp[9];
                        pt.Remove(id);
                    }
                    else
                    {
                        removeList.Add(l);
                    }
                }

                ArrayList addList = new ArrayList();
                foreach (int id in pt.Keys)
                {
                    string[] pp = (string[])pt[id];
                    bool responding = bool.Parse(pp[8]);
                    int memory = int.Parse(pp[10]) / 1024;
                    string nr = "";
                    if (responding == false) nr = "[NR] ";
                    ListViewItem l = new ListViewItem(new string[] { nr + pp[1], pp[0], memory.ToString() + "k", pp[9] });
                    addList.Add(l);
                }

                if (removeList.Count > 0 || addList.Count > 0)
                {
                    processListView.BeginUpdate();
                    foreach (ListViewItem l in removeList) processListView.Items.Remove(l);
                    processListView.Items.AddRange((ListViewItem[])addList.ToArray(typeof(ListViewItem)));
                    processListView.EndUpdate();
                }

                refreshTimer.Enabled = autoRefreshToolStripMenuItem.Checked;
            }
        }

        private void ProcessMonitorForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            serial.ProcessDataBlockEvent -= new SerialHighDriver.ProcessDataBlockHandler(ProcessDataBlockHandler);
            serial = null;
            if (computer != null) computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void refreshToolStripMenuItem_Click(object sender, EventArgs e)
        {
            serial.WriteDataBlock("PROC", null);
        }

        private void refreshTimer_Tick(object sender, EventArgs e)
        {
            refreshTimer.Enabled = false;
            serial.WriteDataBlock("PROC", null);
        }

        private void startButton_Click(object sender, EventArgs e)
        {
            refreshTimer.Enabled = false;
            serial.WriteDataBlock("EXEC", UTF8Encoding.UTF8.GetBytes(execTextBox.Text));
            execTextBox.Text = "";
            serial.WriteDataBlock("PROC", null);
        }

        private void execTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == 13) startButton_Click(this, null);
        }

        private void stopButton_Click(object sender, EventArgs e)
        {
           
            if (processListView.SelectedItems.Count != 1) return;
            ListViewItem l = processListView.SelectedItems[0];
            if (MessageBox.Show(this, 
                String.Format(ManageabilityControls.Properties.Resources.ProcessMonitor_Kill_Proc_Format ,l.SubItems[0].Text ), 
                ManageabilityControls.Properties.Resources.ProcessMonitor_Killproc, 
                MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
            {
                serial.WriteDataBlock("KILL", UTF8Encoding.UTF8.GetBytes(l.SubItems[1].Text));
                refreshTimer.Enabled = false;
                serial.WriteDataBlock("PROC", null);
            }
        }

        private void processListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            stopButton.Enabled = (processListView.SelectedItems.Count == 1);
        }

        private void autoRefreshToolStripMenuItem_Click(object sender, EventArgs e)
        {
            autoRefreshToolStripMenuItem.Checked = !autoRefreshToolStripMenuItem.Checked;
            if (autoRefreshToolStripMenuItem.Checked == false) refreshTimer.Enabled = false;
        }

        private void processListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                stopButton_Click(this, null);
                e.Handled = true;
            }
        }

    }
}
