// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class RedirectStatsForm : Form
    {
        private AmtRedirectorWrapper redirector;
        private AmtSystem computer;

        public RedirectStatsForm(AmtRedirectorWrapper redirector, AmtSystem computer)
        {
            InitializeComponent();
            this.redirector = redirector;

            // If the computer is disconnected, close this form
            this.computer = computer;
            if (computer != null) computer.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }

        private void SystemStateChangedHandler(AmtSystem computer) { try { this.Invoke(new AmtSystem.ObjectStateHandler(SystemStateChangedHandlerEx), new object[] { computer }); } catch (Exception) { } }
        private void SystemStateChangedHandlerEx(AmtSystem computer) { if (computer.State != AmtSystemObjState.Connected) Close(); }

        private void statsTimer_Tick(object sender, EventArgs e)
        {
            GetData();
        }

        private void GetData()
        {
            
            AmtRedirectorWrapper.AmtIdeStats stats = redirector.GetRedirectorStats();
            if (stats != null)
            {
                label11.Text = stats.error_state.ToString();
                label12.Text = stats.data_transfer.ToString();
                label13.Text = stats.num_reopen.ToString("###,###,###,##0");
                label14.Text = stats.num_error.ToString("###,###,###,##0");
                label15.Text = stats.num_reset.ToString("###,###,###,##0");
                label16.Text = stats.last_cmd_length.ToString("###,###,###,##0");
                label17.Text = stats.data_sent.ToString("###,###,###,##0");
                label18.Text = stats.data_received.ToString("###,###,###,##0");
                label19.Text = stats.packets_sent.ToString("###,###,###,##0");
                label20.Text = stats.packets_received.ToString("###,###,###,##0");

            }
            else
            {

                label11.Text = ManageabilityControls.Properties.Resources.RedirectStatsForm_Disable;
                label12.Text = "---";
                label13.Text = "---";
                label14.Text = "---";
                label15.Text = "---";
                label16.Text = "---";
                label17.Text = "---";
                label18.Text = "---";
                label19.Text = "---";
                label20.Text = "---";
            }
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void RedirectStatsForm_Load(object sender, EventArgs e)
        {
            GetData();
        }

        private void RedirectStatsForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (computer != null) computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }
    }
}
