// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.WSManagement;
using Intel.Manageability.Exceptions;

namespace ManageabilityControls
{
    public partial class RemoteControlForm : Form
    {
        private AmtSystem computer;
        public static System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(RemoteControlForm));
        private bool LoadCompleted = false;

        public RemoteControlForm(AmtSystem computer)
        {
            this.computer = computer;
            InitializeComponent();
        }

        private void RemoteControlForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            AmtRemoteControl.PowerManagementCapabilities c = computer.Remote.GetPowerManagementCapabilities();

            if (c.HWResetSupported) commandComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_0, 0x0010, null));
            if (c.PowerStateSettable) commandComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_1, 0x0011, null));
            if (c.PowerStateSettable) commandComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_2, 0x0012, null));
            if (c.PowerCyclingSupported) commandComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_3, 0x0013, null));
            commandComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_4, 0x0021, null));
            commandComboBox.SelectedIndex = 0;

            ianaComboBox.Items.Add(new DropDownItem("Intel", 0x0157, null));
            ianaComboBox.Items.Add(new DropDownItem("ASF", 0x11BE, null));
            ianaComboBox.SelectedIndex = 0;

            UpdateControls();
            LoadInfo();
            LoadCompleted = true;
            UpdateInfo();
        }

        private void UpdateControls()
        {
            AMT_BootCapabilities c = computer.Remote.GetBootCapabilities();

            specialCommandComboBox.Items.Clear();
            specialCommandComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_5, 0x0000, null));
            if (ianaComboBox.SelectedIndex == 1)
            {
                // ASF
                specialCommandComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_6, 0x0001, null));
                specialCommandComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_7, 0x0002, null));
                specialCommandComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_8, 0x0003, null));
                specialCommandComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_9, 0x0004, null));
                specialCommandComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_10, 0x0005, null));
            }
            else
            {
                // Intel
                if (c.ForcePXEBootExist && c.ForcePXEBoot) specialCommandComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_6, 0x0001, null));
                if (c.ForceHardDriveBootExist && c.ForceHardDriveBoot) specialCommandComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_7, 0x0002, null));
                if (c.ForceHardDriveSafeModeBootExist && c.ForceHardDriveSafeModeBoot) specialCommandComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_8, 0x0003, null));
                if (c.ForceDiagnosticBootExist && c.ForceDiagnosticBoot) specialCommandComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_9, 0x0004, null));
                if (c.ForceCDorDVDBootExist && c.ForceCDorDVDBoot) specialCommandComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_10, 0x0005, null));
                specialCommandComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_11, 0x00C1, null));
            }
            specialCommandComboBox.SelectedIndex = 0;

            specialParamsComboBox.Items.Clear();
            specialParamsComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_12,0x0000,null));
            if (c.IDERExist && c.IDER) specialParamsComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_13, 0x0001, null));
            if (c.IDERExist && c.IDER) specialParamsComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_17, 0x0101, null));
            if (c.BIOSReflashExist && c.BIOSReflash) specialParamsComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_14, 0x0004, null));
            if (c.BIOSSetupExist && c.BIOSSetup) specialParamsComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_15, 0x0008, null));
            if (c.BIOSPauseExist && c.BIOSPause) specialParamsComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_16, 0x0010, null));
            specialParamsComboBox.SelectedIndex = 0;
            specialParamsComboBox.Enabled = (((DropDownItem)specialCommandComboBox.SelectedItem).Handle == 0x00C1);

            eomParamsComboBox.Items.Clear();
            eomParamsComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_18, 0x0000, null));
            if (c.SOLExist && c.SOL) eomParamsComboBox.Items.Add(new DropDownItem(ManageabilityControls.Properties.Resources.RemoteCtlForm_19, 0x0001, null));
            eomParamsComboBox.SelectedIndex = 0;
            eomParamsComboBox.Enabled = true;

            screenRadioButton.Enabled = (c.VerbosityScreenBlankExist && c.VerbosityScreenBlank); // VerbosityScreenBlank
            lockPowerCheckBox.Enabled = (c.PowerButtonLockExist && c.PowerButtonLock); // PowerButtonLock
            lockResetCheckBox.Enabled = (c.ResetButtonLockExist && c.ResetButtonLock); // ResetButtonLock
            // 0x0008
            // 0x0010
            lockKeyboardCheckBox.Enabled = (c.KeyboardLockExist && c.KeyboardLock); // KeyboardLock
            lockSleepCheckBox.Enabled = (c.SleepButtonLockExist && c.SleepButtonLock); // SleepButtonLock
            // 0x0080
            // 0x0100
            // 0x0200
            // 0x0400
            passwordBypassCheckBox.Enabled = (c.UserPasswordBypassExist && c.UserPasswordBypass); // UserPasswordBypass
            progressEventCheckBox.Enabled = (c.ForcedProgressEventsExist && c.ForcedProgressEvents); // ForcedProgressEvents
            verboseRadioButton.Enabled = (c.VerbosityVerboseExist && c.VerbosityVerbose); // VerbosityVerbose
            quietRadioButton.Enabled = (c.VerbosityQuietExist && c.VerbosityQuiet); // VerbosityQuiet
            configResetCheckBox.Enabled = (c.ConfigurationDataResetExist && c.ConfigurationDataReset); // ConfigurationDataReset
        }

        private void sendButton_Click(object sender, EventArgs e)
        {
            byte   Command;
            uint   IanaOemNumber;
            byte   SpecialCommand;
            ushort SpecialCommandParameter;
            ushort OEMparameters;

            Command = (byte)((DropDownItem)commandComboBox.SelectedItem).Handle;
            IanaOemNumber = (uint)((DropDownItem)ianaComboBox.SelectedItem).Handle;
            SpecialCommand = (byte)((DropDownItem)specialCommandComboBox.SelectedItem).Handle;
            SpecialCommandParameter = (ushort)((DropDownItem)specialParamsComboBox.SelectedItem).Handle;
            OEMparameters = (ushort)((DropDownItem)eomParamsComboBox.SelectedItem).Handle;

            ushort BootOptions = 0;
            if (lockPowerCheckBox.Checked      == true) BootOptions |= (1 << 1);
            if (lockResetCheckBox.Checked      == true) BootOptions |= (1 << 2);
            if (lockKeyboardCheckBox.Checked   == true) BootOptions |= (1 << 5);
            if (lockSleepCheckBox.Checked      == true) BootOptions |= (1 << 6);
            if (passwordBypassCheckBox.Checked == true) BootOptions |= (1 << 11);
            if (progressEventCheckBox.Checked  == true) BootOptions |= (1 << 12);
            if (configResetCheckBox.Checked    == true) BootOptions |= (1 << 14);

            if (quietRadioButton.Checked       == true) BootOptions |= (1 << 13);
            if (verboseRadioButton.Checked     == true) BootOptions |= (1 << 14);
            if (screenRadioButton.Checked      == true) BootOptions |= (3 << 13);

            AmtCallStatus r = computer.Remote.SendRemoteControl(Command, IanaOemNumber, SpecialCommand, SpecialCommandParameter, BootOptions, OEMparameters);
            if (r == AmtCallStatus.SUCCESS)
            {
                MessageBox.Show(this, 
                    ManageabilityControls.Properties.Resources.RemoteCtlForm_ROP_Succ,
                    ManageabilityControls.Properties.Resources.RemoteCtlForm_RCMD,
                    MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show(this,
                    ManageabilityControls.Properties.Resources.RemoteCtlForm_ROP_Fail + r.ToString(),
                    ManageabilityControls.Properties.Resources.RemoteCtlForm_RCMD,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        private void LoadInfo()
        {
            int x;
            
            x = int.Parse((string)Application.UserAppDataRegistry.GetValue("RC-Command", "0"));
            if (x < commandComboBox.Items.Count) commandComboBox.SelectedIndex = x;

            x = int.Parse((string)Application.UserAppDataRegistry.GetValue("RC-IanaOemNumber", "0"));
            if (x < ianaComboBox.Items.Count) ianaComboBox.SelectedIndex = x;

            UpdateControls();

            x = int.Parse((string)Application.UserAppDataRegistry.GetValue("RC-SpecialCommand", "0"));
            if (x < specialCommandComboBox.Items.Count) specialCommandComboBox.SelectedIndex = x;

            x = int.Parse((string)Application.UserAppDataRegistry.GetValue("RC-SpecialCommandParameter", "0"));
            if (x < specialParamsComboBox.Items.Count) specialParamsComboBox.SelectedIndex = x;

            x = int.Parse((string)Application.UserAppDataRegistry.GetValue("RC-OEMparameters", "0"));
            if (x < eomParamsComboBox.Items.Count) eomParamsComboBox.SelectedIndex = x;

            lockPowerCheckBox.Checked = bool.Parse((string)Application.UserAppDataRegistry.GetValue("RC-lockPower", bool.FalseString));
            lockResetCheckBox.Checked = bool.Parse((string)Application.UserAppDataRegistry.GetValue("RC-lockReset", bool.FalseString));
            lockKeyboardCheckBox.Checked = bool.Parse((string)Application.UserAppDataRegistry.GetValue("RC-lockKeyboard", bool.FalseString));
            lockSleepCheckBox.Checked = bool.Parse((string)Application.UserAppDataRegistry.GetValue("RC-lockSleep", bool.FalseString));
            passwordBypassCheckBox.Checked = bool.Parse((string)Application.UserAppDataRegistry.GetValue("RC-passwordBypass", bool.FalseString));
            progressEventCheckBox.Checked = bool.Parse((string)Application.UserAppDataRegistry.GetValue("RC-progressEvent", bool.FalseString));
            configResetCheckBox.Checked = bool.Parse((string)Application.UserAppDataRegistry.GetValue("RC-configReset", bool.FalseString));
            quietRadioButton.Checked = bool.Parse((string)Application.UserAppDataRegistry.GetValue("RC-quiet", bool.FalseString));
            verboseRadioButton.Checked = bool.Parse((string)Application.UserAppDataRegistry.GetValue("RC-verbose", bool.FalseString));
            screenRadioButton.Checked = bool.Parse((string)Application.UserAppDataRegistry.GetValue("RC-screen", bool.FalseString));
        }

        private void UpdateInfo()
        {
            if (LoadCompleted == false) return;

            byte Command;
            uint IanaOemNumber;
            byte SpecialCommand;
            ushort SpecialCommandParameter;
            ushort OEMparameters;

            try
            {
                Command = (byte)((DropDownItem)commandComboBox.SelectedItem).Handle;
                IanaOemNumber = (uint)((DropDownItem)ianaComboBox.SelectedItem).Handle;
                SpecialCommand = (byte)((DropDownItem)specialCommandComboBox.SelectedItem).Handle;
                SpecialCommandParameter = (ushort)((DropDownItem)specialParamsComboBox.SelectedItem).Handle;
                OEMparameters = (ushort)((DropDownItem)eomParamsComboBox.SelectedItem).Handle;
            }
            catch (Exception) { return; }

            Application.UserAppDataRegistry.SetValue("RC-Command", commandComboBox.SelectedIndex.ToString());
            Application.UserAppDataRegistry.SetValue("RC-IanaOemNumber", ianaComboBox.SelectedIndex.ToString());
            Application.UserAppDataRegistry.SetValue("RC-SpecialCommand", specialCommandComboBox.SelectedIndex.ToString());
            Application.UserAppDataRegistry.SetValue("RC-SpecialCommandParameter", specialParamsComboBox.SelectedIndex.ToString());
            Application.UserAppDataRegistry.SetValue("RC-OEMparameters", eomParamsComboBox.SelectedIndex.ToString());

            Application.UserAppDataRegistry.SetValue("RC-lockPower", lockPowerCheckBox.Checked);
            Application.UserAppDataRegistry.SetValue("RC-lockReset", lockResetCheckBox.Checked);
            Application.UserAppDataRegistry.SetValue("RC-lockKeyboard", lockKeyboardCheckBox.Checked);
            Application.UserAppDataRegistry.SetValue("RC-lockSleep", lockSleepCheckBox.Checked);
            Application.UserAppDataRegistry.SetValue("RC-passwordBypass", passwordBypassCheckBox.Checked);
            Application.UserAppDataRegistry.SetValue("RC-progressEvent", progressEventCheckBox.Checked);
            Application.UserAppDataRegistry.SetValue("RC-configReset", configResetCheckBox.Checked);
            Application.UserAppDataRegistry.SetValue("RC-quiet", quietRadioButton.Checked);
            Application.UserAppDataRegistry.SetValue("RC-verbose", verboseRadioButton.Checked);
            Application.UserAppDataRegistry.SetValue("RC-screen", screenRadioButton.Checked);

            ushort BootOptions = 0;
            if (lockPowerCheckBox.Checked == true) BootOptions |= (1 << 1);
            if (lockResetCheckBox.Checked == true) BootOptions |= (1 << 2);
            if (lockKeyboardCheckBox.Checked == true) BootOptions |= (1 << 5);
            if (lockSleepCheckBox.Checked == true) BootOptions |= (1 << 6);
            if (passwordBypassCheckBox.Checked == true) BootOptions |= (1 << 11);
            if (progressEventCheckBox.Checked == true) BootOptions |= (1 << 12);
            if (configResetCheckBox.Checked == true) BootOptions |= (1 << 14);

            if (quietRadioButton.Checked == true) BootOptions |= (1 << 13);
            if (verboseRadioButton.Checked == true) BootOptions |= (1 << 14);
            if (screenRadioButton.Checked == true) BootOptions |= (3 << 13);

            statusLabel.Text = String.Format(ManageabilityControls.Properties.Resources.RemoteCtlForm_SendingCMD_Format,
                Command.ToString("X"),
                IanaOemNumber.ToString("X"),
                SpecialCommand.ToString("X"),
                SpecialCommandParameter.ToString("X"),
                BootOptions.ToString("X"),
                OEMparameters.ToString("X"));
        }

        /// <summary>
        /// This class is used to add items to DropDown combo boxes and still be able to reference the item quickly.
        /// </summary>
        private class DropDownItem
        {
            public string Text = null;
            public object Tag = null;
            public int Handle = 0;
            public override string ToString() { return Text; }

            public DropDownItem()
            {
            }

            public DropDownItem(string Text, int Handle, object Tag)
            {
                this.Text = Text;
                this.Handle = Handle;
                this.Tag = Tag;
            }
        }

        private void commandComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void ianaComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateControls();
            UpdateInfo();
        }

        private void specialCommandComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            specialParamsComboBox.Enabled = (((DropDownItem)specialCommandComboBox.SelectedItem).Handle == 0x00C1);
            //eomParamsComboBox.Enabled = (((DropDownItem)specialCommandComboBox.SelectedItem).Handle == 0x00C1);
            if (((DropDownItem)specialCommandComboBox.SelectedItem).Handle != 0x00C1)
            {
                if (specialParamsComboBox.Items.Count > 0) specialParamsComboBox.SelectedIndex = 0;
                //if (eomParamsComboBox.Items.Count > 0) eomParamsComboBox.SelectedIndex = 0;
            }
            UpdateInfo();
        }

        private void specialParamsComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void eomParamsComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void lockPowerCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void statusLabel_Click(object sender, EventArgs e)
        {
            UpdateInfo();
        }

    }
}
