// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class RemoteManagementTargetForm : Form
    {
        public string Host = null;
        public int Port = 0;
        private AmtSystem system = null;
        public string Username = null;
        public string Password = null;
        public string HostIdentity = null;

        public RemoteManagementTargetForm(AmtSystem system)
        {
            this.system = system;
            InitializeComponent();
        }

        private void RemoteManagementTargetForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            clientTextBox.Text = system.HostNameIdentity.DefaultName;
            hostTextBox.Text = Properties.Settings.Default.RemoteManagementTargetHost + ":" + Properties.Settings.Default.RemoteManagementTargetPort;
            
            if (Properties.Settings.Default.RemoteManagementUsername != null && Properties.Settings.Default.RemoteManagementUsername.Length > 0)
            {
                certRadioButton.Checked = false;
                userRadioButton.Checked = true;
                usernameTextBox.Text = Properties.Settings.Default.RemoteManagementUsername;
                passwordControl.Text = Properties.Settings.Default.RemoteManagementPassword;
            }
            else
            {
                certRadioButton.Checked = true;
                userRadioButton.Checked = false;
            }

            UpdateInfo();
        }

        private bool UpdateInfo()
        {
            bool ok = true;
            int i = hostTextBox.Text.IndexOf(':');
            if (i < 1) ok = false;
            else
            {
                string hoststr = hostTextBox.Text.Substring(0, i);
                string portstr = hostTextBox.Text.Substring(i + 1);
                if (hoststr.Length == 0 || portstr.Length == 0) ok = false;
                else
                {
                    if (int.TryParse(portstr, out Port) == false) ok = false;
                    else
                    {
                        if (Port < 1 || Port > 65534)
                        {
                            Port = 0;
                            ok = false;
                        }
                    }
                }
                Host = hoststr;
            }

            if (certRadioButton.Checked == true)
            {
                Username = "";
                Password = "";
                usernameTextBox.Enabled = false;
                passwordControl.Enabled = false;
            }
            else
            {
                Username = usernameTextBox.Text;
                Password = passwordControl.Text;
                usernameTextBox.Enabled = true;
                passwordControl.Enabled = true;
                if (Username.Length == 0 || Password.Length == 0) ok = false;
            }

            if (clientTextBox.Text.Length == 0) ok = false;
            HostIdentity = clientTextBox.Text;

            okButton.Enabled = ok;
            return ok;
        }

        private void hostTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            if (UpdateInfo() == false) return;
            Properties.Settings.Default.RemoteManagementTargetHost = Host;
            Properties.Settings.Default.RemoteManagementTargetPort = Port;
            Properties.Settings.Default.RemoteManagementUsername = Username;
            Properties.Settings.Default.RemoteManagementPassword = Password;
            Properties.Settings.Default.Save();
            DialogResult = DialogResult.OK;
        }

        private void usernameTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void passwordControl_PasswordChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void certRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

    }
}
