// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class ScriptCommandForm : Form
    {
        public ScriptingForm.ScriptCommand command = null;

        public ScriptCommandForm()
        {
            InitializeComponent();
        }

        private void ScriptCommandForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            commandComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.CMDNop, (int)ScriptingForm.ScriptCommands.Nop, null));
            commandComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.CMDlabel, (int)ScriptingForm.ScriptCommands.Label, null));
            commandComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.CMDWait, (int)ScriptingForm.ScriptCommands.Wait, null));
            commandComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.CMDJump, (int)ScriptingForm.ScriptCommands.Jump, null));
            commandComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.CMDReset, (int)ScriptingForm.ScriptCommands.Reset, null));
            commandComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.CMDIfOnScreen, (int)ScriptingForm.ScriptCommands.IfOnScreen, null));
            commandComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.CMDWaitFor, (int)ScriptingForm.ScriptCommands.WaitFor, null));
            commandComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.CMDSend, (int)ScriptingForm.ScriptCommands.Send, null));
            commandComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.CMDExit, (int)ScriptingForm.ScriptCommands.ExitScript, null));
            commandComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.CMDMessage, (int)ScriptingForm.ScriptCommands.Message, null));

            if (command == null)
            {
                commandComboBox.SelectedIndex = 0;
            }
            else
            {
                SelectCommand(command.command);
                switch (command.command)
                {
                    case ScriptingForm.ScriptCommands.Reset:
                        arg1TextBox.Text = command.arg1;
                        arg2TextBox.Text = command.arg2;
                        resetComboBox.SelectedIndex = command.arg3;
                        arg4NumericUpDown.Value = command.arg4;
                        break;
                    default:
                        arg1TextBox.Text = command.arg1;
                        arg2TextBox.Text = command.arg2;
                        if (command.arg3 != int.MinValue) arg3NumericUpDown.Value = command.arg3;
                        if (command.arg4 != int.MinValue) arg4NumericUpDown.Value = command.arg4;
                        break;
                }
            }
            UpdateInfo();
        }

        private void SelectCommand(ScriptingForm.ScriptCommands cmd)
        {
            foreach (Utils.DropDownItem i in commandComboBox.Items)
            {
                if (i.Handle == (int)cmd)
                {
                    commandComboBox.SelectedItem = i;
                    return;
                }
            }
        }

        private void UpdateInfo()
        {
            Utils.DropDownItem i = (Utils.DropDownItem)commandComboBox.SelectedItem;
            switch ((ScriptingForm.ScriptCommands)(i.Handle))
            {
                case ScriptingForm.ScriptCommands.Nop:
                    panel1.Visible = false;
                    panel2.Visible = false;
                    panel3.Visible = false;
                    panel4.Visible = false;
                    break;
                case ScriptingForm.ScriptCommands.Label:
                    panel1.Visible = true;
                    label1.Text = Properties.Resources.LabelName;
                    panel2.Visible = false;
                    panel3.Visible = false;
                    panel4.Visible = false;
                    break;
                case ScriptingForm.ScriptCommands.Wait:
                    panel1.Visible = false;
                    panel2.Visible = false;
                    panel3.Visible = true;
                    label3.Text = Properties.Resources.TimeSeconds;
                    resetComboBox.Visible = false;
                    arg3NumericUpDown.Visible = true;
                    panel4.Visible = false;
                    break;
                case ScriptingForm.ScriptCommands.Jump:
                    panel1.Visible = true;
                    label1.Text = Properties.Resources.LabelName;
                    panel2.Visible = false;
                    label3.Text = "Jump count (0 = always)";
                    resetComboBox.Visible = false;
                    arg3NumericUpDown.Visible = true;
                    panel3.Visible = true;
                    panel4.Visible = false;
                    break;
                case ScriptingForm.ScriptCommands.Reset:
                    panel1.Visible = false;
                    panel2.Visible = false;
                    panel3.Visible = true;
                    resetComboBox.Visible = true;
                    arg3NumericUpDown.Visible = false;
                    panel4.Visible = false;
                    break;
                case ScriptingForm.ScriptCommands.IfOnScreen:
                    panel1.Visible = true;
                    label1.Text = Properties.Resources.OnScreenText;
                    panel2.Visible = true;
                    label2.Text = Properties.Resources.JumpToLabel;
                    panel3.Visible = false;
                    panel4.Visible = false;
                    break;
                case ScriptingForm.ScriptCommands.WaitFor:
                    panel1.Visible = true;
                    label1.Text = Properties.Resources.OnScreenText;
                    panel2.Visible = false;
                    panel3.Visible = false;
                    panel4.Visible = false;
                    break;
                case ScriptingForm.ScriptCommands.Send:
                    panel1.Visible = true;
                    label1.Text = Properties.Resources.SendString;
                    panel2.Visible = false;
                    panel3.Visible = false;
                    panel4.Visible = false;
                    break;
                case ScriptingForm.ScriptCommands.ExitScript:
                    panel1.Visible = false;
                    panel2.Visible = false;
                    panel3.Visible = false;
                    panel4.Visible = false;
                    break;
                case ScriptingForm.ScriptCommands.Message:
                    panel1.Visible = true;
                    label1.Text = Properties.Resources.DisplayString;
                    panel2.Visible = false;
                    panel3.Visible = false;
                    panel4.Visible = false;
                    break;
            }

            if (((ScriptingForm.ScriptCommands)(i.Handle)) == ScriptingForm.ScriptCommands.Send)
            {
                addSpecialKeyButton.Visible = true;
                arg1TextBox.Width = arg2TextBox.Width - (addSpecialKeyButton.Width + 2);
            }
            else
            {
                addSpecialKeyButton.Visible = false;
                arg1TextBox.Width = arg2TextBox.Width;
            }
        }

        private void commandComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            arg1TextBox.Text = "";
            arg2TextBox.Text = "";
            arg3NumericUpDown.Value = 0;
            arg4NumericUpDown.Value = 0;
            resetComboBox.SelectedIndex = 0;
            UpdateInfo();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            int arg3 = int.MinValue;
            int arg4 = int.MinValue;
            if (panel3.Visible == true && arg3NumericUpDown.Visible == true) arg3 = (int)arg3NumericUpDown.Value;
            if (panel3.Visible == true && resetComboBox.Visible == true) arg3 = (int)resetComboBox.SelectedIndex;
            if (panel4.Visible == true) arg4 = (int)arg4NumericUpDown.Value;
            command = new ScriptingForm.ScriptCommand((ScriptingForm.ScriptCommands)(commandComboBox.SelectedIndex), arg1TextBox.Text, arg2TextBox.Text, arg3, arg4);
            DialogResult = DialogResult.OK;
        }

        private void addSpecialKeyButton_Click(object sender, EventArgs e)
        {
            addKeyContextMenuStrip.Show(addSpecialKeyButton, addSpecialKeyButton.Width / 2, addSpecialKeyButton.Height / 2);
        }

        private void leftqToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (sender == leftqToolStripMenuItem) arg1TextBox.AppendText("\\q");
            if (sender == rightwToolStripMenuItem) arg1TextBox.AppendText("\\w");
            if (sender == upaToolStripMenuItem) arg1TextBox.AppendText("\\a");
            if (sender == downzToolStripMenuItem) arg1TextBox.AppendText("\\z");
            if (sender == f11ToolStripMenuItem) arg1TextBox.AppendText("\\1");
            if (sender == f22ToolStripMenuItem) arg1TextBox.AppendText("\\2");
            if (sender == f33ToolStripMenuItem) arg1TextBox.AppendText("\\3");
            if (sender == f44ToolStripMenuItem) arg1TextBox.AppendText("\\4");
            if (sender == f55ToolStripMenuItem) arg1TextBox.AppendText("\\5");
            if (sender == f66ToolStripMenuItem) arg1TextBox.AppendText("\\6");
            if (sender == f77ToolStripMenuItem) arg1TextBox.AppendText("\\7");
            if (sender == f88ToolStripMenuItem) arg1TextBox.AppendText("\\8");
            if (sender == f99ToolStripMenuItem) arg1TextBox.AppendText("\\9");
            if (sender == f100ToolStripMenuItem) arg1TextBox.AppendText("\\0");
            if (sender == f11ToolStripMenuItem1) arg1TextBox.AppendText("\\-");
            if (sender == f12ToolStripMenuItem) arg1TextBox.AppendText("\\=");
            if (sender == backslashToolStripMenuItem) arg1TextBox.AppendText("\\\\");
            if (sender == returnrToolStripMenuItem) arg1TextBox.AppendText("\\r");
            if (sender == tabtToolStripMenuItem) arg1TextBox.AppendText("\\t");
            if (sender == escapexToolStripMenuItem) arg1TextBox.AppendText("\\x");
            if (sender == ctrlIiToolStripMenuItem) arg1TextBox.AppendText("\\i");
        }
    }
}
