// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class SetAmtTimeForm : Form
    {
        private AmtSystem system;
        private long delta = 0;
        private bool meread = false;
        private AmtSecurityAdmin.ProvisioningMode mode;

        public SetAmtTimeForm(AmtSystem system)
        {
            this.system = system;
            if (system.SecurityAdmin != null) mode = system.SecurityAdmin.GetProvisioningMode(); else mode = AmtSecurityAdmin.ProvisioningMode.SmallBusiness;
            InitializeComponent();
        }

        private void SetAmtTimeForm_Load(object sender, EventArgs e)
        {
            if (mode == AmtSecurityAdmin.ProvisioningMode.Enterprise)
            {
                label1.Text += " (UTC)";
                label2.Text += " (UTC)";
            }
            GetDelta();
        }

        private void GetDelta()
        {
            if (system.NetworkTime == null) return;
            DateTime met;
            AmtCallStatus r = system.NetworkTime.GetLowAccuracyTimeSynch(out met);
            if (r != AmtCallStatus.SUCCESS) { Utils.ErrorMessageBox(this, "GetLowAccuracyTimeSynch", r); meread = false; return; }
            DateTime n;
            if (mode == AmtSecurityAdmin.ProvisioningMode.Enterprise) n = DateTime.UtcNow; else n = DateTime.Now;
            delta = met.Ticks - n.Ticks;
            meread = true;
        }

        private void refreshTimer_Tick(object sender, EventArgs e)
        {
            DateTime n;
            if (mode == AmtSecurityAdmin.ProvisioningMode.Enterprise) n = DateTime.UtcNow; else n = DateTime.Now;
            currentTimeLabel.Text = n.ToString();
            if (meread == true) engineTimeLabel.Text = new DateTime(n.Ticks + delta).ToString();
        }

        private void refreshButton_Click(object sender, EventArgs e)
        {
            GetDelta();
        }

        private void syncButton_Click(object sender, EventArgs e)
        {
            DateTime n;
            if (mode == AmtSecurityAdmin.ProvisioningMode.Enterprise) n = DateTime.UtcNow; else n = DateTime.Now;
            AmtCallStatus r = system.NetworkTime.SetHighAccuracyTimeSynch(n);
            if (r != AmtCallStatus.SUCCESS) Utils.ErrorMessageBox(this, "SetHighAccuracyTimeSynch", r); else GetDelta();
        }

        private void setButton_Click(object sender, EventArgs e)
        {
            DateTime n;
            if (mode == AmtSecurityAdmin.ProvisioningMode.Enterprise) n = DateTime.UtcNow; else n = DateTime.Now;
            SelectTimeForm f = new SelectTimeForm();
            f.SetTime(n);
            if (f.ShowDialog(this) == DialogResult.OK)
            {
                n = f.GetTime();
                AmtCallStatus r = system.NetworkTime.SetHighAccuracyTimeSynch(n);
                if (r != AmtCallStatus.SUCCESS) Utils.ErrorMessageBox(this, "SetHighAccuracyTimeSynch", r); else GetDelta();
            }
        }
    }
}
