// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class StoragePermissionsEditForm : Form
    {
        private LocalStorageInfo Store;
        private AmtStorageApplication App;
        private AmtStorageBlock Block;

        public StoragePermissionsEditForm(LocalStorageInfo store, AmtStorageApplication app, AmtStorageBlock block)
        {
            Store = store;
            App = app;
            Block = block;
            InitializeComponent();
            this.Text += " - " + Block.Name;
        }

        private void StoragePermissionsEditForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            DisplayTree();
        }

        private void DisplayTree()
        {
            mainTreeView.BeginUpdate();
            mainTreeView.Nodes.Clear();

            TreeNode root = new TreeNode(Block.Name, 1, 1);
            root.Tag = Block;
            mainTreeView.Nodes.Add(root);

            AmtStorageWrapper.AmtStoragePermissionsGroup[] groups;
            AmtStorageWrapper.AmtStorageStatus r;
            r = Block.GetPermissionsGroups(out groups);
            if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
            {
                Utils.ErrorMessageBox(this, "GetPermissionsGroups", r.ToString());
                mainTreeView.EndUpdate();
                return;
            }

            foreach (AmtStorageWrapper.AmtStoragePermissionsGroup group in groups)
            {
                string p = Properties.Resources.PermissionUnknown;
                if (group.Permission == AmtStorageWrapper.AmtStorageGroupPermission.ReadOnly) p = Properties.Resources.PermissionReadOnly;
                if (group.Permission == AmtStorageWrapper.AmtStorageGroupPermission.ReadWrite) p = Properties.Resources.PermissionReadWrite;
                TreeNode groupNode = new TreeNode(group.Name + p, 4, 5);
                groupNode.Tag = group;
                root.Nodes.Add(groupNode);

                uint[] members;
                r = Block.GetPermissionsGroupMembers(group, out members);
                if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
                {
                    Utils.ErrorMessageBox(this, "GetPermissionsGroupMembers", r.ToString());
                    mainTreeView.EndUpdate();
                    return;
                }

                foreach (uint member in members)
                {
                    string str = string.Format(Properties.Resources.StorageApplicationHandle, member);
                    if (member == 0xFFFFFFF0) str = Properties.Resources.SameEnterpriseVendorApplication;
                    if (member == 0xFFFFFFF1) str = Properties.Resources.SameEnterpriseVendor;
                    TreeNode memberNode = new TreeNode(str, 2, 2);
                    memberNode.Tag = member;
                    groupNode.Nodes.Add(memberNode);
                }
            }

            mainTreeView.ExpandAll();
            mainTreeView.EndUpdate();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void mainTreeView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtStorageWrapper.AmtStoragePermissionsGroup))
            {
                addGroupButton.Enabled = false;
                addMemberButton.Enabled = true;
                removeButton.Enabled = true;
            }
            else if (mainTreeView.SelectedNode.Tag.GetType() == typeof(uint))
            {
                addGroupButton.Enabled = false;
                addMemberButton.Enabled = false;
                removeButton.Enabled = true;
            }
            else if (mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtStorageBlock))
            {
                addGroupButton.Enabled = true;
                addMemberButton.Enabled = false;
                removeButton.Enabled = false;
            }
            else
            {
                addGroupButton.Enabled = false;
                addMemberButton.Enabled = false;
                removeButton.Enabled = false;
            }
        }

        private void removeButton_Click(object sender, EventArgs e)
        {
            AmtStorageWrapper.AmtStorageStatus r;

            if (mainTreeView.SelectedNode == null)
            {
                addGroupButton.Enabled = false;
                addMemberButton.Enabled = false;
                removeButton.Enabled = false;
            }
            else if (mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtStorageWrapper.AmtStoragePermissionsGroup))
            {
                AmtStorageWrapper.AmtStoragePermissionsGroup group = (AmtStorageWrapper.AmtStoragePermissionsGroup)mainTreeView.SelectedNode.Tag;
                r = Block.RemovePermissionGroup(group);
                if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
                {
                    Utils.ErrorMessageBox(this, "RemoveAllPermissionGroups", r.ToString());
                }
                else
                {
                    DisplayTree();
                }
            }
            else if (mainTreeView.SelectedNode.Tag.GetType() == typeof(uint))
            {
                uint member = (uint)mainTreeView.SelectedNode.Tag;
                AmtStorageWrapper.AmtStoragePermissionsGroup group = (AmtStorageWrapper.AmtStoragePermissionsGroup)mainTreeView.SelectedNode.Parent.Tag;
                r = Block.RemovePermissionsGroupMembers(group, new uint[] { member });
                if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
                {
                    Utils.ErrorMessageBox(this, "RemovePermissionsGroupMembers", r.ToString());
                }
                else
                {
                    DisplayTree();
                }
            }
        }

        private void addGroupButton_Click(object sender, EventArgs e)
        {
            StorageAddGroupForm form = new StorageAddGroupForm();
            if (form.ShowDialog(this) == DialogResult.OK)
            {
                AmtStorageWrapper.AmtStorageStatus r;
                AmtStorageWrapper.AmtStoragePermissionsGroup groupHandle;
                r = Block.AddPermissionGroup(form.GroupPermission, form.GroupName, out groupHandle);
                if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
                {
                    Utils.ErrorMessageBox(this, "AddPermissionGroup", r.ToString());
                }
                else
                {
                    DisplayTree();
                }
            }
        }

        private void addMemberButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null || mainTreeView.SelectedNode.Tag.GetType() != typeof(AmtStorageWrapper.AmtStoragePermissionsGroup)) return;
            AmtStorageWrapper.AmtStoragePermissionsGroup group = (AmtStorageWrapper.AmtStoragePermissionsGroup)mainTreeView.SelectedNode.Tag;

            StorageAddGroupMemberForm form = new StorageAddGroupMemberForm(Store, App);
            if (form.ShowDialog(this) == DialogResult.OK)
            {
                AmtStorageWrapper.AmtStorageStatus r;
                r = Block.AddPermissionsGroupMembers(group, new uint[] { form.Member });
                if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
                {
                    Utils.ErrorMessageBox(this, "AddPermissionsGroupMembers", r.ToString());
                }
                else
                {
                    DisplayTree();
                }
            }
        }
    }
}
