// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class StorageViewerForm : Form
    {
        private AmtStorageBlock Block;
        private AmtStorageApplication App;
        private byte[] buffer = null;

        public StorageViewerForm(AmtStorageApplication app, AmtStorageBlock block)
        {
            App = app;
            Block = block;
            InitializeComponent();
        }

        private void StorageViewerForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            storagePictureBox.AllowDrop = true;
            blockOwnerLabel.Text = string.Format(Properties.Resources.AppVendorEnterpriseSeperator, App.Enterprise, App.Vendor, App.Application);
            blockNameLabel.Text = Block.Name;
            blockSizeLabel.Text = string.Format(Properties.Resources.NumberOfBytes, Block.Size);
            buffer = Block.ReadBlock();
            DisplayBuffer();
        }

        private void DisplayBuffer()
        {
            openInWMIViewerToolStripMenuItem1.Enabled = openInWMIViewerToolStripMenuItem.Enabled = (blockNameLabel.Text.StartsWith("IntelComDiag"));

            if (uTF8StringToolStripMenuItem.Checked)
            {
                storageTextBox.Visible = true;
                storagePictureBox.Visible = false;
                if (buffer != null)
                {
                    storageTextBox.Text = UTF8Encoding.UTF8.GetString(buffer);
                }
                else
                {
                    storageTextBox.Clear();
                }
                storageTextBox.SelectionStart = 0;
                storageTextBox.SelectionLength = 0;
            }
            else if (hEXBinaryToolStripMenuItem.Checked)
            {
                storageTextBox.Visible = true;
                storagePictureBox.Visible = false;
                if (buffer != null)
                {
                    storageTextBox.Text = Utils.BytesToHex(buffer, 4);
                }
                else
                {
                    storageTextBox.Clear();
                }
                storageTextBox.SelectionStart = 0;
                storageTextBox.SelectionLength = 0;
            }
            else if (viewAsImageToolStripMenuItem1.Checked)
            {
                try
                {
                    storageTextBox.Visible = false;
                    storagePictureBox.Visible = true;
                    storagePictureBox.Image = Bitmap.FromStream(new MemoryStream(buffer));
                }
                catch (Exception)
                {
                    storageTextBox.Visible = true;
                    storagePictureBox.Visible = false;
                    storageTextBox.Text = Properties.Resources.InvalidImageData;
                }
            }
        }

        private void clearBlockToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(this, Properties.Resources.ClearStorageBlockQ, Properties.Resources.FlashStorage, MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk) == DialogResult.Yes)
            {
                byte[] buf = new byte[Block.Size];
                AmtStorageWrapper.AmtStorageStatus r = Block.WriteBlock(buf);
                if (r == AmtStorageWrapper.AmtStorageStatus.SUCCESS)
                {
                    buffer = buf;
                    DisplayBuffer();
                }
            }
        }

        private void loadBlockToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                FileStream file;
                try
                {
                    file = new FileStream(openFileDialog.FileName, FileMode.Open);
                }
                catch (Exception ex) { Utils.FileErrorMessageBox(this, ex); return; }
                byte[] buf = new byte[file.Length];
                int len = buf.Length;
                file.Read(buf, 0, buf.Length);
                file.Close();
                Block.WriteBlock(buf);
                storageTextBox.Text = UTF8Encoding.UTF8.GetString(buf);
            }
        }

        private void saveBlockToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                byte[] buf = Block.ReadBlock();
                FileStream file;
                try
                {
                    file = new FileStream(saveFileDialog.FileName, FileMode.Create);
                }
                catch (Exception ex) { Utils.FileErrorMessageBox(this, ex); return; }
                file.Write(buf, 0, buf.Length);
                file.Close();
            }
        }

        private void uTF8StringToolStripMenuItem_Click(object sender, EventArgs e)
        {
            uTF8StringToolStripMenuItem.Checked = true;
            hEXBinaryToolStripMenuItem.Checked = false;
            DisplayBuffer();
        }

        private void hEXBinaryToolStripMenuItem_Click(object sender, EventArgs e)
        {
            uTF8StringToolStripMenuItem.Checked = false;
            hEXBinaryToolStripMenuItem.Checked = true;
            DisplayBuffer();
        }

        private void viewHEXToolStripMenuItem_Click(object sender, EventArgs e)
        {
            viewHEXToolStripMenuItem.Checked = true;
            hEXBinaryToolStripMenuItem.Checked = true;
            viewUTF8ToolStripMenuItem.Checked = false;
            uTF8StringToolStripMenuItem.Checked = false;
            viewAsImageToolStripMenuItem.Checked = false;
            viewAsImageToolStripMenuItem1.Checked = false;
            DisplayBuffer();
        }

        private void viewUTF8ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            viewHEXToolStripMenuItem.Checked = false;
            hEXBinaryToolStripMenuItem.Checked = false;
            viewUTF8ToolStripMenuItem.Checked = true;
            uTF8StringToolStripMenuItem.Checked = true;
            viewAsImageToolStripMenuItem.Checked = false;
            viewAsImageToolStripMenuItem1.Checked = false;
            DisplayBuffer();
        }

        private void viewAsImageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            viewHEXToolStripMenuItem.Checked = false;
            hEXBinaryToolStripMenuItem.Checked = false;
            viewUTF8ToolStripMenuItem.Checked = false;
            uTF8StringToolStripMenuItem.Checked = false;
            viewAsImageToolStripMenuItem.Checked = true;
            viewAsImageToolStripMenuItem1.Checked = true;
            DisplayBuffer();
        }

        private void openInWMIViewerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            byte[] buf = Block.ReadBlock();
            if (buf != null && buf.Length > 0) new ManageabilityControls.DiagnosticViewerForm(buf).Show();
        }

        private void storageDataTextBox_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop, false) == true) e.Effect = DragDropEffects.All;
        }

        private void storageDataTextBox_DragDrop(object sender, DragEventArgs e)
        {
            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);

            FileStream f;
            try
            {
                f = new FileStream(files[0], FileMode.Open);
            }
            catch (Exception ex) { Utils.FileErrorMessageBox(this, ex); return; }
            if (f.Length > Block.Size)
            {
                MessageBox.Show(this,
                    Properties.Resources.MainForm_FSize_Exceed_ErrStr,
                    Properties.Resources.MainForm_NVRAM_Storage,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            else
            {
                byte[] buf = new byte[f.Length];
                f.Read(buf, 0, (int)f.Length);
                f.Close();
                Block.WriteBlock(buf);
                buffer = Block.ReadBlock();
                DisplayBuffer();
            }
        }

        private void refreshToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Block.ClearCache();
            buffer = Block.ReadBlock();
            DisplayBuffer();
        }
    }
}
