// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Net;
using System.Data;
using System.Text;
using System.Drawing;
using System.Threading;
using System.Collections;
using System.Net.Sockets;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class TcpRedirectForm : Form
    {
        private TcpRedirector redirector;
        private AmtSystem computer;

        public TcpRedirectForm(TcpRedirector redirector, AmtSystem computer)
        {
            this.computer = computer;
            this.redirector = redirector;
            InitializeComponent();
        }

        private void addButton_Click(object sender, EventArgs e)
        {
            AddTcpRedirectPortsForm f = new AddTcpRedirectPortsForm();
            if (f.ShowDialog(this) == DialogResult.OK)
            {
                if (!redirector.IsMapped(f.LocalPort))
                {
                    redirector.AddMapping(f.LocalPort, f.RemotePort);
                    UpdateData();
                }
            }
        }

        /// <summary>
        /// Sync up the list view with the redirector states. This is a bit tricky since I don't want to just delete everything
        /// and start over, but rather, update the list view control.
        /// </summary>
        private void UpdateData()
        {
            foreach (TcpRedirector.TcpRedirect t in redirector.redirectorList.Values)
            {
                t.Tag = false;
            }
            ArrayList removelist = new ArrayList();
            foreach (ListViewItem l in tcpListView.Items)
            {
                int localPort = int.Parse(l.SubItems[0].Text);
                TcpRedirector.TcpRedirect t = (TcpRedirector.TcpRedirect)redirector.redirectorList[localPort];
                if (t != null)
                {
                    l.SubItems[1].Text = t.remoteport.ToString();

                    l.SubItems[2].Text = String.Format("{0} ({1} bytes in/ {2} bytes out)", t.state, t.datain.ToString(), t.dataout.ToString());
                    
                    l.Tag = true;
                    t.Tag = true;
                }
                else
                {
                    removelist.Add(l);
                }
            }
            foreach (ListViewItem l in removelist)
            {
                tcpListView.Items.Remove(l);
            }
            foreach (TcpRedirector.TcpRedirect t in redirector.redirectorList.Values)
            {
                if (((bool)t.Tag) == false)
                {
                    ListViewItem ll = new ListViewItem(new string[3] { t.localport.ToString(), t.remoteport.ToString(), String.Format("{0} ({1} bytes in/ {2} bytes out)", t.state, t.datain.ToString(), t.dataout.ToString()) });
                    ll.Tag = true;
                    tcpListView.Items.Add(ll);
                }
            }
        }

        private void tcpListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            removeButton.Enabled = (tcpListView.SelectedItems.Count == 1);
        }

        private void removeButton_Click(object sender, EventArgs e)
        {
            if (tcpListView.SelectedItems.Count != 1) return;
            ListViewItem l = tcpListView.SelectedItems[0];
            int localPort;
            if (int.TryParse(l.SubItems[0].Text, out localPort) == false) return;
            redirector.RemoteMapping(localPort);
        }

        private void TcpRedirectForm_Load(object sender, EventArgs e)
        {
            UpdateData();
        }

        private void updateTimer_Tick(object sender, EventArgs e)
        {
            UpdateData();
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void TcpRedirectForm_Load_1(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            UpdateData();

            // If the computer is disconnected, close this form
            if (computer != null) computer.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }

        private void SystemStateChangedHandler(AmtSystem computer) { try { this.Invoke(new AmtSystem.ObjectStateHandler(SystemStateChangedHandlerEx), new object[] { computer }); } catch (Exception) { } }
        private void SystemStateChangedHandlerEx(AmtSystem computer) { if (computer.State != AmtSystemObjState.Connected) Close(); }

        private void tcpListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                removeButton_Click(this, null);
                e.Handled = true;
            }
        }

        private void TcpRedirectForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (computer != null) computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }

    }
}
