// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class TerminalAnalyzer : Form
    {
        private TerminalForm parent;
        private AmtSystem computer;

        public TerminalAnalyzer(TerminalForm parent, AmtSystem computer)
        {
            this.parent = parent;
            this.computer = computer;
            InitializeComponent();

            // If the computer is disconnected, close this form
            this.computer = computer;
            if (computer != null) computer.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }

        private void clearButton_Click(object sender, EventArgs e)
        {
            escListView.Items.Clear();
        }

        public void AddSequence(string current, string sequence, int code)
        {
            object[] p = new object[3];
            p[0] = current;
            p[1] = sequence;
            p[2] = code;
            try
            {
                Invoke(new AddSequenceHandle(AddSequenceEx), p);
            }
            catch (Exception) { }
        }

        private delegate void AddSequenceHandle(string current, string sequence, int code);
        public void AddSequenceEx(string current, string sequence, int code)
        {
            if (freezeCheckBox.Checked == false) escListView.Items.Insert(0, new ListViewItem(new string[] { current, sequence }, code));
        }

        private void TerminalAnalyser_FormClosed(object sender, FormClosedEventArgs e)
        {
            parent.analyser = null;
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void saveToFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                FileStream f;
                try
                {
                    f = new FileStream(saveFileDialog.FileName, FileMode.Create);
                }
                catch (Exception ex) { ManageabilityStack.Utils.FileErrorMessageBox(this, ex); return; }
                foreach (ListViewItem l in escListView.Items)
                {
                    byte[] b = UTF8Encoding.UTF8.GetBytes(l.SubItems[0].Text + "\t" + l.SubItems[1].Text + "\r\n");
                    f.Write(b, 0, b.Length);
                }
                f.Close();
            }
        }

        private void copyAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string s = "";
            foreach (ListViewItem l in escListView.Items)
            {
                s += (l.SubItems[0].Text + "\t" + l.SubItems[1].Text + "\r\n");
            }
            Clipboard.SetDataObject(s, true);
        }

        private void TerminalAnalyzer_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
        }

        private void SystemStateChangedHandler(AmtSystem computer) { try { this.Invoke(new AmtSystem.ObjectStateHandler(SystemStateChangedHandlerEx), new object[] { computer }); } catch (Exception) { } }
        private void SystemStateChangedHandlerEx(AmtSystem computer) { if (computer.State != AmtSystemObjState.Connected) Close(); }

        private void TerminalAnalyzer_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (computer != null) computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }

    }
}
