﻿// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intel’s prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using ManageabilityStack;
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.WSManagement;
using Intel.Manageability.Exceptions;


namespace ManageabilityControls
{
    public partial class TerminalBootCapabilities : Form
    {
        private AmtSystem _computer; 

        public TerminalBootCapabilities(AmtSystem computer)
        {
            _computer = computer; 

            InitializeComponent();
            PopulateListView(); 
        }

        private void PopulateListView()
        {
            AMT_BootCapabilities bootCapabilites = _computer.Remote.GetBootCapabilities(); 

            ListViewItem lviTemp = BootCapabilitesListView.Items.Add("IDER");
            if(bootCapabilites.IDERExist)
                lviTemp.SubItems.Add(bootCapabilites.IDER.ToString());
            else
                lviTemp.SubItems.Add("false");

            lviTemp = BootCapabilitesListView.Items.Add("SOL");
            if (bootCapabilites.SOLExist)
                lviTemp.SubItems.Add(bootCapabilites.SOL.ToString());
            else
                lviTemp.SubItems.Add("false"); 

            lviTemp = BootCapabilitesListView.Items.Add("BIOS Reflash");
            if (bootCapabilites.BIOSReflashExist)
                lviTemp.SubItems.Add(bootCapabilites.BIOSReflash.ToString());
            else
                lviTemp.SubItems.Add("false"); 

            lviTemp = BootCapabilitesListView.Items.Add("BIOS Setup");
            if (bootCapabilites.BIOSSetupExist)
                lviTemp.SubItems.Add(bootCapabilites.BIOSSetup.ToString());
            else
                lviTemp.SubItems.Add("false"); 


            lviTemp = BootCapabilitesListView.Items.Add("BIOS Pause");
            if (bootCapabilites.BIOSPauseExist)
                lviTemp.SubItems.Add(bootCapabilites.BIOSPause.ToString());
            else
                lviTemp.SubItems.Add("false");

            lviTemp = BootCapabilitesListView.Items.Add("Force PXE Boot");
            if (bootCapabilites.ForcePXEBootExist)
                lviTemp.SubItems.Add(bootCapabilites.ForcePXEBoot.ToString());
            else
                lviTemp.SubItems.Add("false");

            lviTemp = BootCapabilitesListView.Items.Add("Force Hard Drive Boot");
            if (bootCapabilites.ForceHardDriveBootExist)
                lviTemp.SubItems.Add(bootCapabilites.ForceHardDriveBoot.ToString());
            else
                lviTemp.SubItems.Add("false");

            lviTemp = BootCapabilitesListView.Items.Add("Force Hard Drive Safe Mode Boot");
            if (bootCapabilites.ForceHardDriveSafeModeBootExist)
                lviTemp.SubItems.Add(bootCapabilites.ForceHardDriveSafeModeBoot.ToString()); 
            else
                lviTemp.SubItems.Add("false");

            lviTemp = BootCapabilitesListView.Items.Add("Force Diagnostic Boot");
            if (bootCapabilites.ForceDiagnosticBootExist)
                lviTemp.SubItems.Add(bootCapabilites.ForceDiagnosticBoot.ToString()); 
            else
                lviTemp.SubItems.Add("false");
                                    
            lviTemp = BootCapabilitesListView.Items.Add("Force CD or DVD Boot");
            if (bootCapabilites.ForceCDorDVDBootExist)
                lviTemp.SubItems.Add(bootCapabilites.ForceCDorDVDBoot.ToString()); 
            else
                lviTemp.SubItems.Add("false");

            lviTemp = BootCapabilitesListView.Items.Add("Verbosity Screen Blank");
            if (bootCapabilites.VerbosityScreenBlankExist)
                lviTemp.SubItems.Add(bootCapabilites.VerbosityScreenBlank.ToString()); 
            else
                lviTemp.SubItems.Add("false");
            
            lviTemp = BootCapabilitesListView.Items.Add("Power Button Lock");
            if (bootCapabilites.PowerButtonLockExist)
                lviTemp.SubItems.Add(bootCapabilites.PowerButtonLock.ToString()); 
            else
                lviTemp.SubItems.Add("false");

            lviTemp = BootCapabilitesListView.Items.Add("Reset Button Lock");
            if (bootCapabilites.ResetButtonLockExist)
                lviTemp.SubItems.Add(bootCapabilites.ResetButtonLock.ToString());
            else
                lviTemp.SubItems.Add("false");

            lviTemp = BootCapabilitesListView.Items.Add("Keyboard Lock");
            if (bootCapabilites.KeyboardLockExist)
                lviTemp.SubItems.Add(bootCapabilites.KeyboardLock.ToString()); 
            else
                lviTemp.SubItems.Add("false");

            lviTemp = BootCapabilitesListView.Items.Add("Sleep Button Lock");
            if (bootCapabilites.SleepButtonLockExist)
                lviTemp.SubItems.Add(bootCapabilites.SleepButtonLock.ToString()); 
            else
                lviTemp.SubItems.Add("false");

            lviTemp = BootCapabilitesListView.Items.Add("User Password Bypass");
            if (bootCapabilites.UserPasswordBypassExist)
                lviTemp.SubItems.Add(bootCapabilites.UserPasswordBypass.ToString()); 
            else
                lviTemp.SubItems.Add("false");

            lviTemp = BootCapabilitesListView.Items.Add("Forced Progress Events");
            if (bootCapabilites.ForcedProgressEventsExist)
                lviTemp.SubItems.Add(bootCapabilites.ForcedProgressEvents.ToString()); 
            else
                lviTemp.SubItems.Add("false");

            lviTemp = BootCapabilitesListView.Items.Add("Verbosity Verbose");
            if (bootCapabilites.VerbosityVerboseExist)
                lviTemp.SubItems.Add(bootCapabilites.VerbosityVerbose.ToString()); 
            else
                lviTemp.SubItems.Add("false");

            lviTemp = BootCapabilitesListView.Items.Add("Verbosity Quiet");
            if (bootCapabilites.VerbosityQuietExist)
                lviTemp.SubItems.Add(bootCapabilites.VerbosityQuiet.ToString()); 
            else
                lviTemp.SubItems.Add("false");

            lviTemp = BootCapabilitesListView.Items.Add("Configuration Data Reset");
            if (bootCapabilites.ConfigurationDataResetExist)
                lviTemp.SubItems.Add(bootCapabilites.ConfigurationDataReset.ToString()); 
            else
                lviTemp.SubItems.Add("false");
            
        }

        private void CloseButton_Click(object sender, EventArgs e)
        {
            this.Close(); 
        }
    }
}
