// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class TerminalControl : UserControl
    {
        public char[] screen = new char[80 * 25];
        private byte[] screenFColor = new byte[80 * 25];
        private byte[] screenBColor = new byte[80 * 25];
        private bool currentReverse = false;
        private byte currentBColor = 0;
        private byte currentFColor = 7;
        private bool lineWrap = true;
        private SolidBrush[] colors = new SolidBrush[16];
        private int termx = 0;
        private int termy = 0;
        private int fontx;
        private int fonty;
        private bool displayChanges = false;

        private Font TermFont = new Font("Courier New", 13);
        public override Font Font
        {
            get { return TermFont; }
            set
            {
                TermFont = value;
                OnFontChanged(null);
            }
        }

        /*
        The following table "AsciiToUnicode" is used under following license:
         
        Copyright (C) 1993/94 Markus Kuhn, Institut fuer Physiologie und
        Biokybernetik (IPB), Universitaet Erlangen, Deutschland
         
        Permission to use, copy, modify, distribute, and sell this software
        and its documentation for any purpose is hereby granted without fee,
        provided that the above copyright notice appear in all copies and
        related publications and that both that copyright notice and this
        permission notice appear in supporting documentation. The authors make
        no representations about the suitability of this software for any
        purpose. It is provided "as is" without express or implied warranty.
        Send your comments, suggestions or bug reports to
                ftpebs@uni-erlangen.de
        Or mail to
                Institut fuer Physiologie und Biokybernetik
                Markus Prosch
                Universitaetsstrasse 17
                D-91054 Erlangen
                Germany
        $Id: ebs.c,v 1.21 1994/10/18 14:28:44 msprosch Exp $
        */

        private char[] AsciiToUnicode = new char[] 
        {
              (char)0x00c7, (char)0x00fc, (char)0x00e9, (char)0x00e2, (char)0x00e4, (char)0x00e0, (char)0x00e5, (char)0x00e7,
              (char)0x00ea, (char)0x00eb, (char)0x00e8, (char)0x00ef, (char)0x00ee, (char)0x00ec, (char)0x00c4, (char)0x00c5,
              (char)0x00c9, (char)0x00e6, (char)0x00c6, (char)0x00f4, (char)0x00f6, (char)0x00f2, (char)0x00fb, (char)0x00f9,
              (char)0x00ff, (char)0x00d6, (char)0x00dc, (char)0x00a2, (char)0x00a3, (char)0x00a5, (char)0x20a7, (char)0x0192,
              (char)0x00e1, (char)0x00ed, (char)0x00f3, (char)0x00fa, (char)0x00f1, (char)0x00d1, (char)0x00aa, (char)0x00da,
              (char)0x00bf, (char)0x2310, (char)0x00ac, (char)0x00bd, (char)0x00bc, (char)0x00a1, (char)0x00ab, (char)0x00bb,
              (char)0x2593, (char)0x2592, (char)0x2591, (char)0x2502, (char)0x2524, (char)0x2561, (char)0x2562, (char)0x2556,
              (char)0x2555, (char)0x2563, (char)0x2551, (char)0x2557, (char)0x255d, (char)0x255c, (char)0x255b, (char)0x2510,
              (char)0x2514, (char)0x2534, (char)0x252c, (char)0x251c, (char)0x2500, (char)0x253c, (char)0x255e, (char)0x255f,
              (char)0x255a, (char)0x2554, (char)0x2569, (char)0x2566, (char)0x2560, (char)0x2550, (char)0x256c, (char)0x2567,
              (char)0x2568, (char)0x2564, (char)0x2565, (char)0x2568, (char)0x2558, (char)0x2552, (char)0x2553, (char)0x256b,
              (char)0x256a, (char)0x2518, (char)0x250c, (char)0x2588, (char)0x2584, (char)0x258b, (char)0x2590, (char)0x2580,
              (char)0x03b1, (char)0x00df, (char)0x0393, (char)0x03c0, (char)0x03a3, (char)0x03c3, (char)0x00b5, (char)0x03c4,
              (char)0x03c6, (char)0x03b8, (char)0x2126, (char)0x03b4, (char)0x221e, (char)0x00f8, (char)0x03b5, (char)0x220f,
              (char)0x2261, (char)0x00b1, (char)0x2265, (char)0x2266, (char)0x2320, (char)0x2321, (char)0x00f7, (char)0x2248,
              (char)0x00b0, (char)0x2022, (char)0x00b7, (char)0x221a, (char)0x207f, (char)0x00b2, (char)0x220e, (char)0x00a0
        };

        public int TermFontWidth { get { return fontx; } }
        public int TermFontHeight { get { return fonty; } }
        public int TermCursorX { get { return termx; } }
        public int TermCursorY { get { return termy; } }
        public bool UTF8TerminalEmulation = false;

        public TerminalControl()
        {
            InitializeComponent();
            //OnFontChanged();
        }

        private void TerminalControl_Load(object sender, EventArgs e)
        {
            Setup();
        }

        private void Setup()
        {
            // Normal Colors
            colors[0] = new SolidBrush(Color.FromArgb(0, 0, 0)); // Black
            colors[1] = new SolidBrush(Color.FromArgb(187, 0, 0)); // Red
            colors[2] = new SolidBrush(Color.FromArgb(0, 187, 0)); // Green
            colors[3] = new SolidBrush(Color.FromArgb(187, 187, 0)); // Yellow
            colors[4] = new SolidBrush(Color.FromArgb(0, 0, 187)); // Blue
            colors[5] = new SolidBrush(Color.FromArgb(187, 0, 187)); // Magenta
            colors[6] = new SolidBrush(Color.FromArgb(0, 187, 187)); // Cyan
            colors[7] = new SolidBrush(Color.FromArgb(187, 187, 187)); // White

            // Bold Colors
            colors[08] = new SolidBrush(Color.FromArgb(85, 85, 85)); // Black
            colors[09] = new SolidBrush(Color.FromArgb(255, 85, 85)); // Red
            colors[10] = new SolidBrush(Color.FromArgb(85, 255, 85)); // Green
            colors[11] = new SolidBrush(Color.FromArgb(255, 255, 85)); // Yellow
            colors[12] = new SolidBrush(Color.FromArgb(85, 85, 255)); // Blue
            colors[13] = new SolidBrush(Color.FromArgb(255, 85, 255)); // Magenta
            colors[14] = new SolidBrush(Color.FromArgb(85, 255, 255)); // Cyan
            colors[15] = new SolidBrush(Color.FromArgb(255, 255, 255)); // White

            ClearScreen();
        }

        public void ClearScreen()
        {
            Array.Clear(screen, 0, 25 * 80);
            // Set the first line
            for (int z = 0; z < 80; z++)
            {
                screenFColor[z] = currentFColor;
                screenBColor[z] = currentBColor;
            }
            // Copy to the other lines
            for (int z = 0; z < 25; z++)
            {
                Array.Copy(screenFColor, 0, screenFColor, z * 80, 80);
                Array.Copy(screenBColor, 0, screenBColor, z * 80, 80);
            }
            termx = 0;
            termy = 0;
            this.Invalidate();
            displayChanges = false;
        }

        public void ResetScreen()
        {
            currentReverse = false;
            currentBColor = 0;
            currentFColor = 7;
            ClearScreen();
        }

        public void ProcessEscapeCode(SerialHighDriver.EscCode code, int[] args)
        {
            switch (code)
            {
                case SerialHighDriver.EscCode.ResetDevice:
                    // Reset
                    ResetScreen();
                    break;
                case SerialHighDriver.EscCode.ClearScreen:
                    ClearScreen();
                    break;
                case SerialHighDriver.EscCode.MoveCursor:
                    if (args.Length == 2)
                    {
                        if (args[0] < 1) args[0] = 1;
                        if (args[1] < 1) args[1] = 1;
                        termy = Math.Min(args[0] - 1, 24);
                        termx = Math.Min(args[1] - 1, 79);
                        displayChanges = true;
                    }
                    break;
                case SerialHighDriver.EscCode.ScreenAttribs:
                    // Change attributes
                    for (int i = 0; i < args.Length; i++)
                    {
                        if (args[i] == 0)
                        {
                            // Reset Attributes
                            currentBColor = 0;
                            currentFColor = 7;
                            currentReverse = false;
                        }
                        else if (args[i] == 1)
                        {
                            // Bright
                            if (currentFColor < 8) currentFColor += 8;
                        }
                        else if (args[i] == 2 || args[i] == 22)
                        {
                            // Dim
                            if (currentFColor >= 8) currentFColor -= 8;
                        }
                        else if (args[i] == 7)
                        {
                            // Set Reverse attribute true
                            currentReverse = true;
                        }
                        else if (args[i] == 27)
                        {
                            // Set Reverse attribute false
                            currentReverse = false;
                        }
                        else if (args[i] >= 30 && args[i] <= 39)
                        {
                            // Set Foreground Color
                            bool bright = (currentFColor >= 8);
                            currentFColor = (byte)(args[i] - 30);
                            if (bright && currentFColor <= 8) currentFColor += 8;
                        }
                        else if (args[i] >= 40 && args[i] <= 49)
                        {
                            // Set Background Color
                            currentBColor = (byte)(args[i] - 40);
                        }
                        else if (args[i] >= 90 && args[i] <= 99)
                        {
                            // Set Bright Foreground Color
                            currentFColor = (byte)(args[i] - 82);
                        }
                        else if (args[i] >= 100 && args[i] <= 109)
                        {
                            // Set Bright Background Color
                            currentBColor = (byte)(args[i] - 92);
                        }
                    }
                    break;
                case SerialHighDriver.EscCode.EraseLine:
                    if (args.Length == 0 || (args.Length == 1 && args[0] == 0))
                    {
                        if (args[0] == 0) // Erase from the cursor to the end of the line
                        {
                            if (currentReverse == false)
                            {
                                for (int i = termx; i < 80; i++)
                                {
                                    int j = i + (termy * 80);
                                    screen[j] = '\0';
                                    screenFColor[j] = currentFColor;
                                    screenBColor[j] = currentBColor;
                                }
                            }
                            else
                            {
                                for (int i = termx; i < 80; i++)
                                {
                                    int j = i + (termy * 80);
                                    screen[j] = '\0';
                                    screenFColor[j] = currentBColor;
                                    screenBColor[j] = currentFColor;
                                }
                            }
                            displayChanges = true;
                        }
                    }
                    else if (args.Length == 1)
                    {
                        if (args[0] == 1) // Erase from the beginning of the line to the cursor
                        {
                            if (currentReverse == false)
                            {
                                for (int i = 0; i < termx; i++)
                                {
                                    int j = i + (termy * 80);
                                    screen[j] = '\0';
                                    screenFColor[j] = currentFColor;
                                    screenBColor[j] = currentBColor;
                                }
                            }
                            else
                            {
                                for (int i = 0; i < termx; i++)
                                {
                                    int j = i + (termy * 80);
                                    screen[j] = '\0';
                                    screenFColor[j] = currentBColor;
                                    screenBColor[j] = currentFColor;
                                }
                            }
                            displayChanges = true;
                        }
                        else if (args[0] == 2) // Erase the line
                        {
                            if (currentReverse == false)
                            {
                                for (int i = 0; i < 80; i++)
                                {
                                    int j = i + (termy * 80);
                                    screen[j] = '\0';
                                    screenFColor[j] = currentFColor;
                                    screenBColor[j] = currentBColor;
                                }
                            }
                            else
                            {
                                for (int i = 0; i < 80; i++)
                                {
                                    int j = i + (termy * 80);
                                    screen[j] = '\0';
                                    screenFColor[j] = currentBColor;
                                    screenBColor[j] = currentFColor;
                                }
                            }
                            displayChanges = true;
                        }
                    }
                    break;
                case SerialHighDriver.EscCode.EnableLineWrap:
                    lineWrap = true;
                    break;
                case SerialHighDriver.EscCode.DisableLineWrap:
                    lineWrap = false;
                    break;
                default:
                    break;
            }
        }

        protected override void WndProc(ref Message messg)
        {
            // Prevent the background from being drawn
            if (messg.Msg == (int)0x0014) messg.Msg = (int)0x0000;
            base.WndProc(ref messg);
        }

        /// <summary>
        /// Force the terminal to be redrawn. This must be called periodicaly because characters will not be displayed until this is called.
        /// We do this to boost performance, repainting on each character would take too long.
        /// </summary>
        public void InvalidateTerminal()
        {
            if (displayChanges == true)
            {
                this.Invalidate();
                displayChanges = false;
            }
        }

        //public void TerminalControl_Paint(object sender, PaintEventArgs e)
        protected override void OnPaint(PaintEventArgs e)
        {
            StringFormat f = new StringFormat();
            f.Alignment = StringAlignment.Center;
            f.LineAlignment = StringAlignment.Center;
            int skipdraw;
            int skipdrawcolor;
            Rectangle r;
            Rectangle r2;

            // Starting position
            int h = 0;
            //if (mainMenuStrip.Visible) h += mainMenuStrip.Height;
            //if (serialPanel.Visible) h += serialPanel.Height;
            r2 = new Rectangle(0, h, fontx, fonty);

            for (int y = 0; y < 25; y++)
            {
                skipdraw = 0;

                r2.Y = h + fonty * y;

                for (int x = 0; x < 80; x++)
                {
                    // Background Drawing Optimization
                    if (skipdraw == 0)
                    {
                        // See if we can optimize the background drawing
                        skipdraw = 1;
                        skipdrawcolor = screenBColor[x + 80 * y];
                        int z = x + 1;
                        while (z < 80 && skipdrawcolor == screenBColor[z + 80 * y])
                        {
                            skipdraw++;
                            z++;
                        }
                        r = new Rectangle((x * fontx), h + (y * fonty), fontx * skipdraw, fonty);
                        if (e.ClipRectangle.IntersectsWith(r)) e.Graphics.FillRectangle(colors[skipdrawcolor], r);
                    }

                    // Draw foreground letters
                    if (screen[x + 80 * y] > 32 || (x == termx && y == termy))
                    {
                        r2.X = fontx * x;
                        if (e.ClipRectangle.IntersectsWith(r2))
                        {
                            if (screen[x + 80 * y] == '_')
                            {
                                // Draw an underline (which is special)
                                e.Graphics.DrawString(screen[x + 80 * y].ToString(), this.Font, colors[screenFColor[x + 80 * y]], new Rectangle(r2.X, r2.Y - 5, r2.Width, r2.Height + 8), f);
                            }
                            else if (screen[x + 80 * y] > 32)
                            {
                                // Draw a normal character
                                e.Graphics.DrawString(screen[x + 80 * y].ToString(), this.Font, colors[screenFColor[x + 80 * y]], r2, f);
                            }
                            else
                            {
                                // Erace the cursor
                                e.Graphics.FillRectangle(colors[screenBColor[x + 80 * y]], r2.X, r2.Y, r2.Width, r2.Height);
                            }
                        }

                        // Draw the new cursor
                        if (x == termx && y == termy)
                        {
                            int delta = (r2.Bottom - r2.Y);
                            delta -= delta / 5;
                            e.Graphics.FillRectangle(colors[7], r2.X, r2.Y + delta, r2.Width, r2.Height - delta);
                        }
                    }

                    skipdraw--;
                }
            }
        }

        public Size SetSize(int s)
        {
            Font termFont = null;
            String fontType = "Courier New";
            System.OperatingSystem OSInfo = System.Environment.OSVersion;

            if (System.Threading.Thread.CurrentThread.CurrentUICulture.Equals(new System.Globalization.CultureInfo("zh-CHS")))
            {
                if (OSInfo.Version.Major == 6) //Windows Vista
                {
                    fontType = "MSYH";
                }
                else if (OSInfo.Version.Major == 5)  //Windows 2000/XP/2003 Server
                {
                    fontType = "SimSun";
                }
                else fontType = "Courier New";
            }
            else fontType = "Courier New";

            switch (s)
            {
                case 1:
                    termFont = new Font(fontType, 8);
                    //fontx = 6;
                    //fonty = 12;
                    break;
                case 2:
                    termFont = new Font(fontType, 11);
                    //fontx = 8;
                    //fonty = 14;
                    break;
                case 3:
                    termFont = new Font(fontType, 13);
                    //fontx = 10;
                    //fonty = 16;
                    break;
                case 4:
                    termFont = new Font(fontType, 15, FontStyle.Bold);
                    //fontx = 12;
                    //fonty = 18;
                    break;
                case 5:
                    termFont = new Font(fontType, 18, FontStyle.Bold);
                    //fontx = 14;
                    //fonty = 20;
                    break;
            }

            this.Font = termFont;
            //OnFontChanged(null);
            Size TermSize = TextRenderer.MeasureText("M", termFont);
            return new Size((TermSize.Width / 2) * 80, (TermSize.Height) * 25);
        }

        protected override void OnFontChanged(EventArgs e)
        {
            Size TermSize = TextRenderer.MeasureText("M", this.Font);
            fontx = (TermSize.Width / 2);
            fonty = TermSize.Height;
            this.Size = new Size(fontx * 80, fonty * 25);
            Invalidate();
        }

        public Size SetFontForSize(Size targetsize)
        {
            if (this.Font == null || targetsize == null) return new Size(1,1);

            int sx = (targetsize.Width / 80);
            int sy = (targetsize.Height / 25);
            Font termFont;
            Size box;

            // TODO: Find a better way to do this.
            float fs = 0;
            do
            {
                fs += (float)0.5;
                termFont = new Font(this.Font.FontFamily, fs, this.Font.Style);
                box = TextRenderer.MeasureText("M", termFont);
            }
            while ((box.Width / 2) <= sx && box.Height <= sy);
            if (fs == 0.5) fs = (float)1.0;

            this.Font = new Font(this.Font.FontFamily, fs - (float)0.5, this.Font.Style);

            //box = TextRenderer.MeasureText("M", Font);
            //return new Size((box.Width / 2) * 80, box.Height * 25);
            return this.Size;
        }

        private char utf8accumulator = (char)0;
        private int utf8accumulatorCount = (char)0;
        public void WriteChar(char c)
        {
            if (c == 0) return;
            displayChanges = true;

            if (UTF8TerminalEmulation == true)
            {
                bool fallout = true;
                // VT100 - UTF-8 emulation
                if ((c & 0x80) == 0)
                {
                    // Sub 127 char.
                    utf8accumulator = c;
                    utf8accumulatorCount = 0;
                    fallout = false;
                }
                else if ((c & 0xE0) == 0xC0)
                {
                    // 2 byte char
                    utf8accumulator = (char)(c & 0x1F);
                    utf8accumulatorCount = 1;
                    fallout = true;
                }
                else if ((c & 0xF0) == 0xE0)
                {
                    // 3 byte char
                    utf8accumulator = (char)(c & 0x0F);
                    utf8accumulatorCount = 2;
                    fallout = true;
                }
                else if ((c & 0xC0) == 0x80)
                {
                    if (utf8accumulatorCount > 0)
                    {
                        utf8accumulator = (char)(utf8accumulator << 6);
                        utf8accumulator += (char)(c & 0x3F);
                        utf8accumulatorCount--;
                        fallout = (utf8accumulatorCount != 0);
                    }
                    else
                    {
                        utf8accumulator = (char)0;
                        utf8accumulatorCount = 0;
                        fallout = true;
                    }
                }
                if (fallout) return;
                c = utf8accumulator;
            }
            else
            {
                // ANSI - Extended ASCII emulation.
                if ((c & 0x80) != 0) c = AsciiToUnicode[c & 0x7F];
            }

            switch ((int)c)
            {
                case 024: { c = '\u2191'; break; } // Up arrow character
                case 025: { c = '\u2193'; break; } // Down arrow character
            }

            if (termx > 79 || termy > 24)
            {
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Event, String.Format(Properties.Resources.WriteCharOutOfScreen, termx, termy));
                if (termx > 79) termx = 79;
                if (termy > 24) termy = 24;
            }

            switch ((int)c)
            {
                case 8: // Backspace
                    if (termx > 0)
                    {
                        termx = termx - 1;
                        screen[termx + (80 * termy)] = (char)0;
                    }
                    break;
                case 10: // Linefeed
                    termy++;
                    if (termy == 25)
                    {
                        // Move everything up one line
                        Array.Copy(screen, 80, screen, 0, 80 * 24);
                        Array.Copy(screenFColor, 80, screenFColor, 0, 80 * 24);
                        Array.Copy(screenBColor, 80, screenBColor, 0, 80 * 24);
                        Array.Clear(screen, 24 * 80, 80);
                        Array.Clear(screenFColor, 24 * 80, 80);
                        Array.Clear(screenBColor, 24 * 80, 80);
                        termy = 24;
                    }
                    break;
                case 13: // Carriage Return
                    termx = 0;
                    break;
                default:
                    int termz = termx + (80 * termy);
                    screen[termz] = c;
                    if (currentReverse == false)
                    {
                        screenFColor[termz] = currentFColor;
                        screenBColor[termz] = currentBColor;
                    }
                    else
                    {
                        screenFColor[termz] = currentBColor;
                        screenBColor[termz] = currentFColor;
                    }
                    termx++;
                    if (termx >= 80)
                    {
                        termx = 0;
                        if (lineWrap) termy++;
                        if (termy >= 25) termy = 0;
                    }
                    break;
            }
        }

        protected override bool IsInputKey(System.Windows.Forms.Keys keyData)
        {
            switch (keyData & Keys.KeyCode)
            {
                case Keys.Up:
                    return true;
                case Keys.Down:
                    return true;
                case Keys.Right:
                    return true;
                case Keys.Left:
                    return true;
                default:
                    return base.IsInputKey(keyData);
            }
        }

    }
}