// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Net;
using System.Text;
using System.Data;
using System.Drawing;
using System.Threading;
using System.Diagnostics;
using System.Collections;
using System.Net.Sockets;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Security.Cryptography.X509Certificates;
using ManageabilityStack;
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.WSManagement;
using Intel.Manageability.Exceptions;

namespace ManageabilityControls
{
    public partial class TerminalForm : Form
    {
        public  AmtSystem computer = null;
        private bool computerOwner = false;
        private SerialHighDriver serial;
        public bool Simplified = false;
        public string AutoStartScript = null;
        private string floppyImagePath = null;
        private string cdromImagePath = null;
        private TcpRedirector tcpRedirect;
        private Thread powerPollThread = null;
        private bool powerPollThreadExit = false;
        private bool powerPollState = true;
        public bool g_BatteryPower = false;
        public bool g_OsLockup = false;
        private AmtRedirectorWrapper.AmtIdeStats RedirectorStats = null;
        public bool TerminalDisposed = false;
        private NetworkStream NetStream = null;
        public TerminalAnalyzer analyser;
        private TcpListener relayListener = null;
        private Hashtable relayClientsBuffers = new Hashtable();
        private ScriptingForm scriptingForm = null;
        private bool LocalTerminalMode = false;
        private FileStream CaptureStream = null;
        public AmtRemoteControl.AmtPowerStates CurrentPowerState;
        private string SerialAgentSupportedFeatures = "";
        public bool Silent = false;
        public char[] screen { get { return terminalControl.screen; } }

        public TerminalControl terminalControl { get { return terminalAutoSizeControl.Terminal; } }
        public delegate void TerminalInputHandler(TerminalForm form, byte[] buf);
        public event TerminalInputHandler UserInputEvent;
        public bool TcpMode { get { return NetStream != null; } }

        private DateTime LastRedirectionStatisticsTimestamp = DateTime.Now;


        public static void ShowRemotingForm(AmtSystem system, System.Windows.Forms.IWin32Window owner, bool simplified)
        {
            if (system.ControlForm == null)
            {
                system.ControlForm = new TerminalForm(system, null, simplified);
                ((TerminalForm)system.ControlForm).Show();
            }
            else
            {
                try
                {
                    ((TerminalForm)system.ControlForm).Show();
                }
                catch (ObjectDisposedException)
                {
                    system.ControlForm = new TerminalForm(system, null, simplified);
                    ((TerminalForm)system.ControlForm).Show();
                }
                ((TerminalForm)system.ControlForm).Activate();
            }
        }

        public static TerminalForm GetRemotingForm(AmtSystem system, System.Windows.Forms.IWin32Window owner, bool simplified)
        {
            if (system.ControlForm == null) system.ControlForm = new TerminalForm(system, null, simplified);
            return (TerminalForm)(system.ControlForm);
        }

        public static void DisposeRemotingForm(AmtSystem system)
        {
            if (system.ControlForm != null)
            {
                ((TerminalForm)system.ControlForm).Close();
                ((TerminalForm)system.ControlForm).DisposeTerminal();
                system.ControlForm = null;
            }
        }

        // Run the terminal as just a window.
        public TerminalForm(string title)
        {
            InitializeComponent();
            toolStripStatusLabel.Text = string.Format(Properties.Resources.AddVToVersion, AboutForm.VersionString);
            LocalTerminalMode = true;
            if (title != null) this.Text = Utils.FilterStr(title);
        }

        // Run the terminal as Telnet application.
        public TerminalForm(NetworkStream netStream, string title)
        {
            InitializeComponent();
            toolStripStatusLabel.Text = string.Format(Properties.Resources.AddVToVersion, AboutForm.VersionString);
            if (title != null) this.Text = Utils.FilterStr(title);
            NetStream = netStream;
        }

        // Run the terminal as Intel? AMT stand-alone.
        public TerminalForm(string hostname, ushort port, string username, string password, string title)
        {
            InitializeComponent();
            toolStripStatusLabel.Text = string.Format(Properties.Resources.AddVToVersion, AboutForm.VersionString);
            this.computer = new AmtSystem(hostname, port, username, password, false, false, ForceIPOption.UseDefault, null);
            computerOwner = true;
            if (title == null || title.Length == 0) this.Text = this.Text + " - " + computer.HostNameIdentity.DefaultName; else this.Text = Utils.FilterStr(title) + " - " + computer.HostNameIdentity.DefaultName; 
        }
        
        // Run the terminal as slave to an existing Manageability Stack.
        public TerminalForm(AmtSystem computer, string title, bool simplified)
        {
            InitializeComponent();
            toolStripStatusLabel.Text = string.Format(Properties.Resources.AddVToVersion, AboutForm.VersionString);
            this.computer = computer;
            computerOwner = false;
            Simplified = simplified;
            SystemStateChangedHandlerEx(computer);
            if (title == null || title.Length == 0)
            {

                 this.Text += " - " + computer.HostNameIdentity.DefaultName;

            }
            else
            {
                this.Text = title;
            }
        }

        // Run the terminal using a know serial driver object
        public TerminalForm(AmtSystem computer, string title, SerialHighDriver serial)
        {
            InitializeComponent();

            toolStripStatusLabel.Text = string.Format(Properties.Resources.AddVToVersion, AboutForm.VersionString);
            this.computer = computer;
            this.serial = serial;
            computerOwner = false;
            Simplified = false;
            SystemStateChangedHandlerEx(computer);
            if (title == null || title.Length == 0)
            {

                 this.Text += " - " + computer.HostNameIdentity.DefaultName;

            }
            else
            {
                this.Text = title;
            }
        }

        public void DisposeTerminal()
        {
            TerminalDisposed = true;
            Close();
            StopPowerPolling();
            StopSerial();
            if (tcpRedirect != null)
            {
                tcpRedirect.Dispose();
                tcpRedirect = null;
            }
            if (computer != null) computer.ControlForm = null;
            if (computerOwner)
            {
                computer.Disconnect();
                computer.Dispose();
                computer = null;
            }
            SerialAgentSupportedFeatures = "";
            Dispose();
        }

        public void ProcessChar(byte[] buf, int offset, int length)
        {
            for (int i = offset; i < (offset + length); i++) ProcessCharEx(buf[i]);
            terminalControl.InvalidateTerminal();
        }

        public void InvalidateTerminal()
        {
            terminalControl.InvalidateTerminal();
        }

        private int termstate = 0;
        private int[] escNumber = new int[32];
        private int escNumberPtr = 0;
        private byte[] packetAccumulator = null;
        private int packetAccumulatorPtr = 0;
        private void ProcessCharEx(byte b)
        {
            switch (termstate)
            {
                case 0: // Normal Term State
                    switch (b)
                    {
                        case 27: // ESC
                            termstate = 1;
                            break;
                        default:
                            // Process a single char
                            ProcessCharHandler(null, b);
                            break;
                    }
                    break;
                case 1:
                    switch (b)
                    {
                        case 91:
                            escNumberPtr = 0;
                            Array.Clear(escNumber, 0, 32);
                            termstate = 2;
                            break;
                        default:
                            termstate = 0;
                            break;
                    }
                    break;
                case 2:
                    if (b >= '0' && b <= '9')
                    {
                        // This is a number
                        escNumber[escNumberPtr] = (escNumber[escNumberPtr] * 10) + (b - '0');
                        break;
                    }
                    else if (b == ';')
                    {
                        // New number
                        escNumberPtr++;
                        break;
                    }
                    else if (b == '@')
                    {
                        // New Packet
                        if (escNumber[1] == 0)
                        {
                            // Empty Command
                            termstate = 0;
                        }
                        else
                        {
                            packetAccumulatorPtr = 0;
                            packetAccumulator = new byte[escNumber[1]];
                            termstate = 3;
                        }
                        break;
                    }
                    else
                    {
                        // Process Escape Sequence
                        int[] vals = new int[escNumberPtr + 1];
                        Array.Copy(escNumber, vals, escNumberPtr + 1);
                        ProcessEscapeHandler(null, (ManageabilityStack.SerialHighDriver.EscCode)b, vals);
                        termstate = 0;
                    }
                    break;
                case 3: // Packet Accumulator
                    packetAccumulator[packetAccumulatorPtr] = b;
                    if (packetAccumulatorPtr == packetAccumulator.Length - 1)
                    {
                        packetAccumulator = null;
                        termstate = 0;
                    }
                    else
                    {
                        // More to come
                        packetAccumulatorPtr++;
                    }
                    break;
            }
        }

        private string analyser1 = "";
        private string analyser2 = "";
        private StringBuilder analyser3 = new StringBuilder();
        private int analyser4 = 0;
        private int analyser5 = 0;
        private void ProcessCharHandler(SerialHighDriver sender, byte c)
        {
            if (CaptureStream != null) CaptureStream.WriteByte(c);

            if (analyser != null)
            {
                if (c == 13) analyser3.Append("\r");
                else if (c == 10) analyser3.Append("\n");
                else if (c == 0x20) analyser3.Append("~");
                else if (c == '\\') analyser3.Append("\\\\");
                else analyser3.Append(UTF8Encoding.UTF8.GetString(new byte[] { c }));
            }

            lock (relayClientsBuffers)
            {
                foreach (TcpClient client in relayClientsBuffers.Keys)
                {
                    try
                    {
                        client.GetStream().WriteByte(c);
                    }
                    catch (Exception) { continue; }
                }
            }

            terminalControl.WriteChar((char)c);
        }

        private void ProcessEscapeHandler(SerialHighDriver sender, SerialHighDriver.EscCode code, int[] args)
        {
            if (CaptureStream != null)
            {
                string s = "[ESC: " + code.ToString();
                if (args != null) { foreach (int arg in args) s += ", " + arg.ToString(); }
                s += "]";
                byte[] buf = UTF8Encoding.UTF8.GetBytes(s);
                CaptureStream.Write(buf, 0, buf.Length);
            }

            if (analyser != null)
            {
                string s = "";
                if (args != null) foreach (int a in args) s += a + ";";
                if (s.Length > 1) s = s.Substring(0, s.Length - 1);

                if (analyser3.Length == 0)
                {
                    analyser2 += " + " + code.ToString() + "(" + s + ")";
                    analyser5 = Math.Max(analyser4, analyser5);
                }
                else
                {
                    analyser3 = analyser3.Replace("\r", "\\r").Replace("\n", "\\n");
                    analyser.AddSequence(analyser1, analyser2 + " + \"" + analyser3.ToString() + "\"", Math.Max(analyser4, analyser5));
                    analyser1 = "(" + (terminalControl.TermCursorX + 1) + "," + (terminalControl.TermCursorY + 1) + ")";
                    analyser2 = code.ToString() + "(" + s + ")";
                    analyser3 = new StringBuilder();
                    analyser4 = 0;
                    analyser5 = 0;
                }
            }

            foreach (TcpClient client in relayClientsBuffers.Keys)
            {
                NetworkStream ns;
                try
                {
                    ns = client.GetStream();
                }
                catch (Exception)
                {
                    continue;
                }
                ns.WriteByte(27);
                ns.WriteByte(91);
                if (args != null)
                {
                    bool first = true;
                    foreach (int a in args)
                    {
                        if (first == false) ns.WriteByte((byte)';');
                        byte[] b = UTF8Encoding.UTF8.GetBytes(a.ToString());
                        ns.Write(b, 0, b.Length);
                        first = false;
                    }
                }
                ns.WriteByte((byte)code);
            }

            switch (code)
            {
                case SerialHighDriver.EscCode.MoveCursor:
                    if (args.Length == 2 && (args[0] == 0 || args[1] == 0)) analyser4 = 2;
                    //if (termy == args[0] - 1 && termx == args[1] - 1) analyser4 = 1;
                    break;
                /*
                default:
                    string s = "";
                    if (args != null) foreach (int a in args) s += a + ";";
                    if (s.Length > 1) s = s.Substring(0, s.Length - 1);
                    analyser4 = 2;
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Event, string.Format(Properties.Resources.UnknownEscapeCode, ((int)code).ToString(), s));
                    break;
                */
            }

            // Draw this escape code to screen
            terminalControl.ProcessEscapeCode(code, args);
        }

        private void ProcessDataBlockProgressHandler(SerialHighDriver sender, bool inbound, string name, int total, int current)
        {
            if (TerminalDisposed == true) return;
            this.Invoke(new SerialHighDriver.ProcessDataBlockProgressHandler(ProcessDataBlockProgressHandlerEx), new object[] { sender, inbound, name, total, current });
        }

        private void ProcessDataBlockProgressHandlerEx(SerialHighDriver sender, bool inbound, string name, int total, int current)
        {
            if (name.Length > 6 && name.Substring(0, 4).CompareTo("FILE") == 0)
            {
                if (inbound)
                {
                    toolStripFileStatusLabel1.Visible = (total != current);
                    toolStripFileProgressBar1.Visible = (total != current);
                    toolStripFileStatusLabel1.Text = Properties.Resources.Term_Download;
                    toolStripFileProgressBar1.Maximum = total;
                    toolStripFileProgressBar1.Value = current;
                }
                else
                {
                    toolStripFileStatusLabel2.Visible = (total != current);
                    toolStripFileProgressBar2.Visible = (total != current);
                    toolStripFileStatusLabel2.Text = Properties.Resources.Term_Upload;
                    toolStripFileProgressBar2.Maximum = total;
                    toolStripFileProgressBar2.Value = current;
                }
            }
        }

        private void ProcessDataBlockHandler(SerialHighDriver sender, string name, byte[] data)
        {
            if (this.Visible == false) return;

            object[] p = new object[3];
            p[0] = sender;
            p[1] = name;
            p[2] = data;
            this.Invoke(new SerialHighDriver.ProcessDataBlockHandler(ProcessDataBlockHandlerEx), p);
        }

        private void ProcessDataBlockHandlerEx(SerialHighDriver sender, string name, byte[] data)
        {
            // First, try to get supported features of remote serial agent
            if (packetCodesActiveToolStripMenuItem.Checked == false) serial.WriteDataBlock("INFO", null);

            if (name.Length < 4) return;

            // If we get remote information, save it and use it.
            if (name.Equals("INFO") == true && data.Length > 0)
            {
                SerialAgentSupportedFeatures = UTF8Encoding.UTF8.GetString(data);
            }

            if (name.Substring(0, 4).CompareTo("FILE") == 0)
            {
                saveFileDialog.FileName = name.Substring(5);
                if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
                {
                    FileStream f;
                    try
                    {
                        f = new FileStream(saveFileDialog.FileName, FileMode.Create);
                    }
                    catch (Exception ex) { Utils.FileErrorMessageBox(this, ex); return; }
                    f.Write(data,0,data.Length);
                    f.Close();
                }
            }

            if (name.Substring(0, 4).CompareTo("UPLD") == 0)
            {
                if (openFileDialog.ShowDialog(this) == DialogResult.OK)
                {
                    FileStream f;
                    try
                    {
                        f = new FileStream(openFileDialog.FileName, FileMode.Open);
                    }
                    catch (Exception ex) { Utils.FileErrorMessageBox(this, ex); return; }
                    byte[] buf = new byte[f.Length];
                    f.Read(buf,0,(int)f.Length);
                    f.Close();
                    serial.WriteDataBlock("FILE|" + openFileDialog.FileName, buf);
                }
            }

            if (name.Substring(0, 4).CompareTo("EXIT") == 0)
            {
                packetCodesActiveToolStripMenuItem.Checked = false;
            }

            if (name.Substring(0, 4).CompareTo("PING") == 0)
            {
                packetCodesActiveToolStripMenuItem.Checked = true;
                serialPacketsToolStripMenuItem.Visible = true;
                tcprPanel.Visible = true;
            }

            if (name.Substring(0, 4).CompareTo("PONG") == 0)
            {
                packetCodesActiveToolStripMenuItem.Checked = true;
                serialPacketsToolStripMenuItem.Visible = true;
                tcprPanel.Visible = true;
            }

            if (name.Substring(0, 4).CompareTo("QUIT") == 0)
            {
                Close();
                StopSerial();
            }
        }

        private void ProcessSpecialEventSink(SerialHighDriver sender, SerialHighDriver.SpecialCommand command)
        {
            switch(command)
            {
                case SerialHighDriver.SpecialCommand.REPAINT:
                    // Invalidate command
                    terminalControl.InvalidateTerminal();
                    break;
                case SerialHighDriver.SpecialCommand.DISCONNECT:
                    // Disconnection
                    Invoke(new System.EventHandler(disconnectSOLToolStripMenuItem_Click), new object[] {this, null});
                    break;
            }
        }

        private void ResetRedirector()
        {
            StopSerial();
            computer.RedirectorWrapper.Reset();
            StartSerial();
        }

        private void SetSize(int s)
        {
            Size displaySize = terminalControl.SetSize(s);

            int h = displaySize.Height;
            if (mainMenuStrip.Visible) h += mainMenuStrip.Height;
            if (serialPanel.Visible) h += serialPanel.Height;
            if (bottomPanel.Visible) h += bottomPanel.Height;
            if (statusStrip.Visible) h += statusStrip.Height;
            this.SetClientSizeCore(displaySize.Width, h);

            terminalControl.Invalidate();
        }

        public void SendKeyCode(byte[] key)
        {
            computer.RedirectorWrapper.Write(key);
        }

        public void TerminalForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (packetCodesActiveToolStripMenuItem.Checked == true && sendSpecialKeysToolStripMenuItem.Checked == true)
            {
                if (e.KeyValue == 0x0012) { serial.WriteAltDown(); serial.WriteAltUp(); e.Handled = true; return; }
                //if (e.KeyValue == 0x0011) { serial.WriteCtrlDown(); serial.WriteCtrlUp();  e.Handled = true; return; }
            }

            if (e.Alt == true && e.Control == true && e.KeyCode == Keys.F1)
            {
                // Reactivate debugging features
                debugFeaturesToolStripMenuItem.Visible = true;
                captureToolStripMenuItem.Visible = true;
                runScriptToolStripMenuItem.Visible = true;
                scriptingToolStripMenuItem.Visible = true;
                return;
            }

            /*
            if (e.Control == true && e.Alt == false && e.KeyCode >= Keys.A && e.KeyCode <= Keys.Z)
            {
                if (serial != null) serial.Write((byte)(e.KeyCode - 64));
                UserInputEvent(this, new byte[] { (byte)(e.KeyCode - 64) });
                e.Handled = true;
                return;
            }
            */

            if (UserInputEvent != null)
            {
                switch (e.KeyCode)
                {
                    case Keys.Return:
                        if (Properties.Settings.Default.TerminalReturnSendLF) UserInputEvent(this, new byte[] { 0x0d, 0x0a }); else UserInputEvent(this, new byte[] { 0x0d });
                        e.Handled = true;
                        break;
                    case Keys.Escape:
                        UserInputEvent(this, new byte[] { 0x1b });
                        e.Handled = true;
                        break;
                    case Keys.Up:
                        UserInputEvent(this, new byte[] { 27, 91, 65 });
                        e.Handled = true;
                        break;
                    case Keys.Down:
                        UserInputEvent(this, new byte[] { 27, 91, 66 });
                        e.Handled = true;
                        break;
                    case Keys.Right:
                        UserInputEvent(this, new byte[] { 27, 91, 67 });
                        e.Handled = true;
                        break;
                    case Keys.Left:
                        UserInputEvent(this, new byte[] { 27, 91, 68 });
                        e.Handled = true;
                        break;    
                }
            }
            
            if (serial == null) return;

            if (vT100IntelToolStripMenuItem.Checked == true)
            {
                switch (e.KeyCode)
                {
                    case Keys.Return:
                        serial.Write(0x0D);
                        if (Properties.Settings.Default.TerminalReturnSendLF) serial.Write(0x0A);
                        e.Handled = true;
                        break;
                    case Keys.I:
                        if (e.Modifiers == Keys.Control)
                        {
                            serial.Write(new byte[]{0x09});
                            e.Handled = true;
                        }
                        break;
                    case Keys.Escape:
                        serial.Write(0x1B);
                        e.Handled = true;
                        break;
                    case Keys.Up:
                        serial.WriteKeyUp();
                        e.Handled = true;
                        break;
                    case Keys.Down:
                        serial.WriteKeyDown();
                        e.Handled = true;
                        break;
                    case Keys.Right:
                        serial.WriteKeyRight();
                        e.Handled = true;
                        break;
                    case Keys.Left:
                        serial.WriteKeyLeft();
                        e.Handled = true;
                        break;
                    case Keys.Home:
                        serial.WriteKeyHome();
                        e.Handled = true;
                        break;
                    case Keys.End:
                        serial.WriteKeyEnd();
                        e.Handled = true;
                        break;
                    case Keys.Insert:
                        serial.WriteKeyInsert();
                        e.Handled = true;
                        break;
                    case Keys.Delete:
                        serial.WriteKeyDelete();
                        e.Handled = true;
                        break;
                    case Keys.PageUp:
                        serial.WriteKeyPageUp();
                        e.Handled = true;
                        break;
                    case Keys.PageDown:
                        serial.WriteKeyPageDown();
                        e.Handled = true;
                        break;
                    case Keys.F1:
                        serial.WriteKeyF1();
                        e.Handled = true;
                        break;
                    case Keys.F2:
                        serial.WriteKeyF2();
                        e.Handled = true;
                        break;
                    case Keys.F3:
                        serial.WriteKeyF3();
                        e.Handled = true;
                        break;
                    case Keys.F4:
                        serial.WriteKeyF4();
                        e.Handled = true;
                        break;
                    case Keys.F5:
                        serial.WriteKeyF5();
                        e.Handled = true;
                        break;
                    case Keys.F6:
                        serial.WriteKeyF6();
                        e.Handled = true;
                        break;
                    case Keys.F7:
                        serial.WriteKeyF7();
                        e.Handled = true;
                        break;
                    case Keys.F8:
                        serial.WriteKeyF8();
                        e.Handled = true;
                        break;
                    case Keys.F9:
                        serial.WriteKeyF9();
                        e.Handled = true;
                        break;
                    case Keys.F10:
                        serial.WriteKeyF10();
                        e.Handled = true;
                        break;
                }
            }


            if (alternateToolStripMenuItem.Checked == true)
            {
                switch (e.KeyCode)
                {
                    case Keys.Return:
                        serial.Write(0x0D);
                        if (Properties.Settings.Default.TerminalReturnSendLF) serial.Write(0x0A);
                        e.Handled = true;
                        break;
                    case Keys.I:
                        if (e.Modifiers == Keys.Control)
                        {
                            serial.Write(0x09);
                            e.Handled = true;
                        }
                        break;
                    case Keys.Escape:
                        serial.Write(0x1B);
                        e.Handled = true;
                        break;
                    case Keys.Up:
                        serial.WriteKeyUp();
                        e.Handled = true;
                        break;
                    case Keys.Down:
                        serial.WriteKeyDown();
                        e.Handled = true;
                        break;
                    case Keys.Right:
                        serial.WriteKeyRight();
                        e.Handled = true;
                        break;
                    case Keys.Left:
                        serial.WriteKeyLeft();
                        e.Handled = true;
                        break;
                    case Keys.Home:
                        serial.WriteKeyHome();
                        e.Handled = true;
                        break;
                    case Keys.End:
                        serial.WriteKeyEnd();
                        e.Handled = true;
                        break;
                    case Keys.Insert:
                        serial.WriteKeyInsert();
                        e.Handled = true;
                        break;
                    case Keys.Delete:
                        serial.WriteKeyDelete();
                        e.Handled = true;
                        break;
                    case Keys.PageUp:
                        serial.Write(27);
                        serial.Write((byte)'?');
                        e.Handled = true;
                        break;
                    case Keys.PageDown:
                        serial.Write(27);
                        serial.Write((byte)'/');
                        e.Handled = true;
                        break;
                    case Keys.F1:
                        serial.Write(27);
                        serial.Write((byte)'1');
                        e.Handled = true;
                        break;
                    case Keys.F2:
                        serial.Write(27);
                        serial.Write((byte)'2');
                        e.Handled = true;
                        break;
                    case Keys.F3:
                        serial.Write(27);
                        serial.Write((byte)'3');
                        e.Handled = true;
                        break;
                    case Keys.F4:
                        serial.Write(27);
                        serial.Write((byte)'4');
                        e.Handled = true;
                        break;
                    case Keys.F5:
                        serial.Write(27);
                        serial.Write((byte)'5');
                        e.Handled = true;
                        break;
                    case Keys.F6:
                        serial.Write(27);
                        serial.Write((byte)'6');
                        e.Handled = true;
                        break;
                    case Keys.F7:
                        serial.Write(27);
                        serial.Write((byte)'7');
                        e.Handled = true;
                        break;
                    case Keys.F8:
                        serial.Write(27);
                        serial.Write((byte)'8');
                        e.Handled = true;
                        break;
                    case Keys.F9:
                        serial.Write(27);
                        serial.Write((byte)'9');
                        e.Handled = true;
                        break;
                    case Keys.F10:
                        serial.Write(27);
                        serial.Write((byte)'0');
                        e.Handled = true;
                        break;
                    case Keys.F11:
                        serial.Write(27);
                        serial.Write((byte)'!');
                        e.Handled = true;
                        break;
                    case Keys.F12:
                        serial.Write(27);
                        serial.Write((byte)'@');
                        e.Handled = true;
                        break;
                }
            }

            if (vT100F1ESCOPToolStripMenuItem.Checked == true)
            {
                switch (e.KeyCode)
                {
                    case Keys.Return:
                        serial.Write(0x0D);
                        if (Properties.Settings.Default.TerminalReturnSendLF) serial.Write(0x0A);
                        e.Handled = true;
                        break;
                    case Keys.I:
                        if (e.Modifiers == Keys.Control)
                        {
                            serial.Write(0x09);
                            e.Handled = true;
                        }
                        break;
                    case Keys.Escape:
                        serial.Write(0x1B);
                        e.Handled = true;
                        break;
                    case Keys.Up:
                        serial.WriteKeyUp();
                        e.Handled = true;
                        break;
                    case Keys.Down:
                        serial.WriteKeyDown();
                        e.Handled = true;
                        break;
                    case Keys.Right:
                        serial.WriteKeyRight();
                        e.Handled = true;
                        break;
                    case Keys.Left:
                        serial.WriteKeyLeft();
                        e.Handled = true;
                        break;
                    case Keys.Home:
                        serial.WriteKeyHome();
                        e.Handled = true;
                        break;
                    case Keys.End:
                        serial.WriteKeyEnd();
                        e.Handled = true;
                        break;
                    case Keys.Insert:
                        serial.WriteKeyInsert();
                        e.Handled = true;
                        break;
                    case Keys.Delete:
                        serial.WriteKeyDelete();
                        e.Handled = true;
                        break;
                    case Keys.PageUp:
                        serial.Write(27);
                        serial.Write((byte)'?');
                        e.Handled = true;
                        break;
                    case Keys.PageDown:
                        serial.Write(27);
                        serial.Write((byte)'/');
                        e.Handled = true;
                        break;
                    case Keys.F1:
                        serial.Write(new byte[] { 27, 79, 80 });
                        e.Handled = true;
                        break;
                    case Keys.F2:
                        serial.Write(new byte[] { 27, 79, 81 });
                        e.Handled = true;
                        break;
                    case Keys.F3:
                        serial.Write(new byte[] { 27, 79, 82 });
                        e.Handled = true;
                        break;
                    case Keys.F4:
                        serial.Write(new byte[] { 27, 79, 83 });
                        e.Handled = true;
                        break;
                    case Keys.F5:
                        serial.Write(new byte[] { 27, 79, 84 });
                        e.Handled = true;
                        break;
                    case Keys.F6:
                        serial.Write(new byte[] { 27, 79, 85 });
                        e.Handled = true;
                        break;
                    case Keys.F7:
                        serial.Write(new byte[] { 27, 79, 86 });
                        e.Handled = true;
                        break;
                    case Keys.F8:
                        serial.Write(new byte[] { 27, 79, 87 });
                        e.Handled = true;
                        break;
                    case Keys.F9:
                        serial.Write(new byte[] { 27, 79, 88 });
                        e.Handled = true;
                        break;
                    case Keys.F10:
                        serial.Write(new byte[] { 27, 79, 89 });
                        e.Handled = true;
                        break;
                    case Keys.F11:
                        serial.Write(new byte[] { 27, 79, 90 });
                        e.Handled = true;
                        break;
                    case Keys.F12:
                        serial.Write(new byte[] { 27, 79, 91 });
                        e.Handled = true;
                        break;
                }
            }

        }

        public void TerminalForm_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (UserInputEvent != null && e.KeyChar != 27 && e.KeyChar != 13) UserInputEvent(this, new byte[] { (byte)e.KeyChar });

            if (serial == null) return;

            //toolStripStatusLabel.Text = "Sending " + ((byte)e.KeyChar).ToString();
            if (e.KeyChar != 27 && e.KeyChar != 13)
            {
                serial.Write((byte)e.KeyChar);
            }
            e.Handled = true;
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void verySmallToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SetSize(1);
        }

        private void smallToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SetSize(2);
        }

        private void standardToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SetSize(3);
        }

        private void largeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SetSize(4);
        }

        private void veryLargeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SetSize(5);
        }

        private ushort GetKeyLock()
        {
            AMT_BootCapabilities c = computer.Remote.GetBootCapabilities();
            ushort keylock = 0;
            if (Properties.Settings.Default.LockLocalKeyboard == true) keylock = 0x20;
            if (c.KeyboardLockExist == true && c.KeyboardLock == false) keylock = 0;
            return keylock;
        }

        public void powerUpToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (computer == null || computer.State != AmtSystemObjState.Connected || computer.Remote == null) return;

            if (e == null || MessageBox.Show(this, 
                Properties.Resources.Term_MsgBox_PwrUp_0,
                Properties.Resources.Term_MsgBox_PwrUp_1,
                MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.Yes)
            {
                powerStateLabel.Text = Properties.Resources.Term_PwrTransit;
                if (computer.AmtRev == 1) { StopSerial(); Thread.Sleep(100); }
                AmtCallStatus s = computer.Remote.SendRemoteControl(0x11, 0x157, 0x0, 0x0, 0x0, 0x0);
                if (s != AmtCallStatus.SUCCESS && s != AmtCallStatus.FAILED_WEB_CALL) MessageBox.Show(this,
                    Properties.Resources.Term_MsgBox_PwrUp_2 + s.ToString(),
                    Properties.Resources.Term_MsgBox_PwrUp_1,
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                else if (terminalControl != null) terminalControl.ResetScreen();
            }
        }

        public void powerDownToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (computer == null || computer.State != AmtSystemObjState.Connected || computer.Remote == null) return;

            if (e == null || MessageBox.Show(this,
                Properties.Resources.Term_MsgBox_PwrDown_0,
                Properties.Resources.Term_MsgBox_PwrDown_1,
                MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.Yes)
            {
                powerStateLabel.Text = Properties.Resources.Term_PwrTransit;

                if (computer.RedirectorWrapper.RedirectActive == true) computer.RedirectorWrapper.StopRedirect();
                StopSerial();
                Thread.Sleep(100);
                AmtCallStatus s = computer.Remote.SendRemoteControl(0x12, 0x157, 0x0, 0x0, 0x0, 0x0);
                if (s != AmtCallStatus.SUCCESS && s != AmtCallStatus.FAILED_WEB_CALL) MessageBox.Show(this,
                    Properties.Resources.Term_MsgBox_PwrDown_2 + s.ToString(),
                    Properties.Resources.Term_MsgBox_PwrDown_1, 
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                else if (terminalControl != null) terminalControl.ResetScreen();

                if (computer.AmtRev > 1)
                {
                    // Turn SOL back on
                    System.Threading.Thread.Sleep(50);
                    StartSerial();
                }
            }
        }

        public void normalBootToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (computer == null || computer.State != AmtSystemObjState.Connected || computer.Remote == null) return;

            byte command = 0x11; // Powerup
            if (CurrentPowerState == AmtRemoteControl.AmtPowerStates.S0_POWERED || CurrentPowerState == AmtRemoteControl.AmtPowerStates.S1_SLEEPING_WITH_CONTEXT) command = 0x10; // Reset

            if (e == null || MessageBox.Show(this,
                Properties.Resources.Term_MsgBox_NormReb_0,
                Properties.Resources.Term_MsgBox_NormReb_1,
                MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.Yes)
            {
                powerStateLabel.Text = Properties.Resources.Term_PwrTransit;
                if (computer.AmtRev == 1) { StopSerial(); Thread.Sleep(100); }
                AmtCallStatus s = computer.Remote.SendRemoteControl(command, 0x157, 0x0, 0x0, 0x0, 0x0);
                if (s != AmtCallStatus.SUCCESS && s != AmtCallStatus.FAILED_WEB_CALL) MessageBox.Show(this,
                    Properties.Resources.Term_MsgBox_NormReb_2 + s.ToString(),
                    Properties.Resources.Term_MsgBox_NormReb_1,
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                else
                {
                    packetCodesActiveToolStripMenuItem.Checked = false;
                    if (terminalControl != null) terminalControl.ResetScreen();
                }
            }
        }

        public void remotedNormalBootToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (computer == null || computer.State != AmtSystemObjState.Connected || computer.Remote == null) return;

            if (!UserConsentRequestForm.RequestUserConsentIfNeeded(this, computer, AmtUserConsent.OptInRequiredValues.All))
            {
                return;
            }

            byte command = 0x11; // Powerup
            if (CurrentPowerState == AmtRemoteControl.AmtPowerStates.S0_POWERED || CurrentPowerState == AmtRemoteControl.AmtPowerStates.S1_SLEEPING_WITH_CONTEXT) command = 0x10; // Reset

            if (e == null || MessageBox.Show(this,
                Properties.Resources.RebootComputerWithRemotingQ,
                Properties.Resources.Term_MsgBox_NormReb_1,
                MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.Yes)
            {
                powerStateLabel.Text = Properties.Resources.Term_PwrTransit;
                if (computer.AmtRev == 1) { StopSerial(); Thread.Sleep(100); }
                AmtCallStatus s = computer.Remote.SendRemoteControl(command, 0x157, 0xc1, 0x0, GetKeyLock(), 0x1);
                if (s != AmtCallStatus.SUCCESS && s != AmtCallStatus.FAILED_WEB_CALL) MessageBox.Show(this,
                    Properties.Resources.Term_MsgBox_NormReb_2 + s.ToString(),
                    Properties.Resources.Term_MsgBox_NormReb_1,
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                else
                {
                    packetCodesActiveToolStripMenuItem.Checked = false;
                    if (terminalControl != null) terminalControl.ResetScreen();
                }
            }
        }

        public void remoteBIOSSetupToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (computer == null || computer.State != AmtSystemObjState.Connected || computer.Remote == null) return;

            if (!UserConsentRequestForm.RequestUserConsentIfNeeded(this, computer, AmtUserConsent.OptInRequiredValues.All))
            {
                return;
            }

            byte command = 0x11; // Powerup
            if (CurrentPowerState == AmtRemoteControl.AmtPowerStates.S0_POWERED || CurrentPowerState == AmtRemoteControl.AmtPowerStates.S1_SLEEPING_WITH_CONTEXT) command = 0x10; // Reset

            if (e == null || MessageBox.Show(this,
                Properties.Resources.Term_MsgBox_BIOSReb_0,
                Properties.Resources.Term_MsgBox_BIOSReb_1,
                MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.Yes)
            {
                powerStateLabel.Text = Properties.Resources.Term_PwrTransit;
                if (computer.AmtRev == 1) { StopSerial(); Thread.Sleep(100); }
                AmtCallStatus s = computer.Remote.SendRemoteControl(command, 0x157, 0xc1, 0x8, GetKeyLock(), 0x1);
                if (s != AmtCallStatus.SUCCESS && s != AmtCallStatus.FAILED_WEB_CALL) MessageBox.Show(this,
                    Properties.Resources.Term_MsgBox_RCMDRet + s.ToString(),
                    Properties.Resources.Term_MsgBox_BIOSReb_2,
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                else
                {
                    packetCodesActiveToolStripMenuItem.Checked = false;
                    if (terminalControl != null) terminalControl.ResetScreen();
                }
            }
        }

        public void remoteBootToRedirectFloppyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (computer == null || computer.State != AmtSystemObjState.Connected || computer.Remote == null) return;

            if (!UserConsentRequestForm.RequestUserConsentIfNeeded(this, computer, AmtUserConsent.OptInRequiredValues.All))
            {
                return;
            }

            byte command = 0x11; // Powerup
            if (CurrentPowerState == AmtRemoteControl.AmtPowerStates.S0_POWERED || CurrentPowerState == AmtRemoteControl.AmtPowerStates.S1_SLEEPING_WITH_CONTEXT) command = 0x10; // Reset

            if (e == null || MessageBox.Show(this, 
                Properties.Resources.Term_MsgBox_FlopReb_0,
                Properties.Resources.Term_MsgBox_FlopReb_1,
                MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.Yes)
            {
                powerStateLabel.Text = Properties.Resources.Term_PwrTransit;
                if (computer.AmtRev == 1) { StopSerial(); Thread.Sleep(100); }
                AmtCallStatus s = computer.Remote.SendRemoteControl(command, 0x157, 0xc1, 0x1, GetKeyLock(), 0x1);
                if (s != AmtCallStatus.SUCCESS && s != AmtCallStatus.FAILED_WEB_CALL) MessageBox.Show(this,
                    Properties.Resources.Term_MsgBox_RCMDRet + s.ToString(),
                    Properties.Resources.Term_MsgBox_FlopReb_2,
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                else
                {
                    packetCodesActiveToolStripMenuItem.Checked = false;
                    if (terminalControl != null) terminalControl.ResetScreen();
                }
            }
        }

        public void remoteBootToRedirectCDToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (computer == null || computer.State != AmtSystemObjState.Connected || computer.Remote == null) return;

            if (!UserConsentRequestForm.RequestUserConsentIfNeeded(this, computer, AmtUserConsent.OptInRequiredValues.All))
            {
                return;
            }

            byte command = 0x11; // Powerup
            if (CurrentPowerState == AmtRemoteControl.AmtPowerStates.S0_POWERED || CurrentPowerState == AmtRemoteControl.AmtPowerStates.S1_SLEEPING_WITH_CONTEXT) command = 0x10; // Reset

            if (e == null || MessageBox.Show(this,
                Properties.Resources.Term_MsgBox_CDReb_0,
                Properties.Resources.Term_MsgBox_CDReb_1,
                MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.Yes)
            {
                powerStateLabel.Text = Properties.Resources.Term_PwrTransit;
                if (computer.AmtRev == 1) { StopSerial(); Thread.Sleep(100); }
                AmtCallStatus s = computer.Remote.SendRemoteControl(command, 0x157, 0xc1, 0x101, GetKeyLock(), 0x1);
                if (s != AmtCallStatus.SUCCESS && s != AmtCallStatus.FAILED_WEB_CALL) MessageBox.Show(this,
                    Properties.Resources.Term_MsgBox_RCMDRet + s.ToString(),
                    Properties.Resources.Term_MsgBox_CDReb_2,
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                else
                {
                    packetCodesActiveToolStripMenuItem.Checked = false;
                    if (terminalControl != null) terminalControl.ResetScreen();
                }
            }
        }

        private void remoteRebootToPXEToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (computer == null || computer.State != AmtSystemObjState.Connected || computer.Remote == null) return;

            if (!UserConsentRequestForm.RequestUserConsentIfNeeded(this, computer, AmtUserConsent.OptInRequiredValues.All))
            {
                return;
            }

            byte command = 0x11; // Powerup
            if (CurrentPowerState == AmtRemoteControl.AmtPowerStates.S0_POWERED || CurrentPowerState == AmtRemoteControl.AmtPowerStates.S1_SLEEPING_WITH_CONTEXT) command = 0x10; // Reset

            if (e == null || MessageBox.Show(this,
                Properties.Resources.Term_MsgBox_PXE_0,
                Properties.Resources.Term_MsgBox_PXE_1,
                MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.Yes)
            {
                powerStateLabel.Text = Properties.Resources.Term_PwrTransit;
                if (computer.AmtRev == 1) { StopSerial(); Thread.Sleep(100); }
                AmtCallStatus s = computer.Remote.SendRemoteControl(command, 0x157, 0x01, 0x0, GetKeyLock(), 0x0);
                if (s != AmtCallStatus.SUCCESS && s != AmtCallStatus.FAILED_WEB_CALL) MessageBox.Show(this,
                    Properties.Resources.Term_MsgBox_RCMDRet + s.ToString(),
                    Properties.Resources.Term_MsgBox_CDReb_2,
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                else
                {
                    packetCodesActiveToolStripMenuItem.Checked = false;
                    if (terminalControl != null) terminalControl.ResetScreen();
                }
            }
        }

        private void remoteRebootToDiagnosticCDToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (computer == null || computer.State != AmtSystemObjState.Connected || computer.Remote == null) return;

            if (!UserConsentRequestForm.RequestUserConsentIfNeeded(this, computer, AmtUserConsent.OptInRequiredValues.All))
            {
                return;
            }

            byte command = 0x11; // Powerup
            if (CurrentPowerState == AmtRemoteControl.AmtPowerStates.S0_POWERED || CurrentPowerState == AmtRemoteControl.AmtPowerStates.S1_SLEEPING_WITH_CONTEXT) command = 0x10; // Reset

            if (e == null || MessageBox.Show(this,
                Properties.Resources.Term_MsgBox_Diag_0,
                Properties.Resources.Term_MsgBox_Diag_1,
                MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.Yes)
            {
                powerStateLabel.Text = Properties.Resources.Term_PwrTransit;
                if (computer.AmtRev == 1) { StopSerial(); Thread.Sleep(100); }
                AmtCallStatus s = computer.Remote.SendRemoteControl(command, 0x157, 0x04, 0x0, GetKeyLock(), 0x0);
                if (s != AmtCallStatus.SUCCESS && s != AmtCallStatus.FAILED_WEB_CALL) MessageBox.Show(this,
                    Properties.Resources.Term_MsgBox_RCMDRet + s.ToString(),
                    Properties.Resources.Term_MsgBox_CDReb_2,
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                else
                {
                    packetCodesActiveToolStripMenuItem.Checked = false;
                    if (terminalControl != null) terminalControl.ResetScreen();
                }
            }
        }

        private string[] PowerStates = new String[]
        {
            Properties.Resources.Term_PwrState_0,
            Properties.Resources.Term_PwrState_1,
            Properties.Resources.Term_PwrState_2,
            Properties.Resources.Term_PwrState_3,
            Properties.Resources.Term_PwrState_4,
            Properties.Resources.Term_PwrState_5,
            Properties.Resources.Term_PwrState_6,
            Properties.Resources.Term_PwrState_7,
            Properties.Resources.Term_PwrState_8,
            Properties.Resources.Term_PwrState_9,
            Properties.Resources.Term_PwrState_10,
            Properties.Resources.Term_PwrState_11,
            Properties.Resources.Term_PwrState_12,
            Properties.Resources.Term_PwrState_13,
            Properties.Resources.Term_PwrState_14,
            Properties.Resources.Term_PwrState_15,
            Properties.Resources.Term_PwrState_16

/*

            "Full power (S0)",
            "Sleeping with full context (S1)",
            "Sleeping without context (S2)",
            "Sleeping with memory context only (S3)",
            "Hibernating (S4)",
            "Soft off (S5)",
            "Hibernating / Soft Off (S4 or S5)",
            "Mechanical off",
            "Sleep state (S1,S2 or S3)",
            "Sleep state (S1,S2,S3 or S4)",
            "Softoff override",
            "Powered on (Legacy on)",
            "Powered off (Legacy off)",
            "Unknown Power State (Code 13)",
            "Unknown Power State (Code 14)",
            "Unknown Power State (Code 15)",
            "Unable to obtain power state",*/
        };

        private void customCommandToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (new RemoteControlForm(computer).ShowDialog(this) == DialogResult.OK)
            {
                packetCodesActiveToolStripMenuItem.Checked = false; ;
            }
        }

        private void resetTerminalToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ResetRedirector();
            terminalControl.ResetScreen();
        }

        private void redirectActiveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AmtRedirectorWrapper.IMRResult r;
            if (!UserConsentRequestForm.RequestUserConsentIfNeeded(this, computer, AmtUserConsent.OptInRequiredValues.All))
            {
                return;
            }

            if (computer.RedirectorWrapper.RedirectActive == true)
            {
                r = computer.RedirectorWrapper.StopRedirect();
                if (r != AmtRedirectorWrapper.IMRResult.IMR_RES_OK) MessageBox.Show(this, string.Format("IDE redirection error: {0}", r.ToString()), "Serial-over-LAN", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                if (floppyImagePath == null || cdromImagePath == null)
                {
                    MessageBox.Show(this,
                        Properties.Resources.Term_MsgBox_RAct_FLCD,
                        Properties.Resources.Term_MsgBox_RAct_Redirect,
                        MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }
                else
                {
                    bool floppyExists = false;
                    bool cdromExists = false;
                    try {floppyExists = new FileInfo(floppyImagePath).Exists;} catch (Exception) {}
                    try {cdromExists = new FileInfo(cdromImagePath).Exists;} catch (Exception) {}

                    if (floppyImagePath.Length != 2 && floppyExists == false)
                    {
                        MessageBox.Show(this,
                            Properties.Resources.Term_MsgBox_RAct_FL,
                            Properties.Resources.Term_MsgBox_RAct_Redirect,
                            MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    }
                    else if (cdromImagePath.Length != 2 && cdromExists == false)
                    {
                        MessageBox.Show(this,
                            Properties.Resources.Term_MsgBox_RAct_CD,
                            Properties.Resources.Term_MsgBox_RAct_Redirect,
                            MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    }
                    else
                    {
                        AmtRedirectorWrapper.RedirectState rs = AmtRedirectorWrapper.RedirectState.IDER_SET_ONRESET;
                        if (gracefullyToolStripMenuItem.Checked == true) rs = AmtRedirectorWrapper.RedirectState.IDER_SET_GRACEFULLY;
                        if (immediatelyToolStripMenuItem.Checked == true) rs = AmtRedirectorWrapper.RedirectState.IDER_SET_IMMEDIATELY;
                        if (onResetToolStripMenuItem.Checked == true) rs = AmtRedirectorWrapper.RedirectState.IDER_SET_ONRESET;
                        r = computer.RedirectorWrapper.StartRedirect(floppyImagePath, cdromImagePath, rs);
                        if (r != AmtRedirectorWrapper.IMRResult.IMR_RES_OK) MessageBox.Show(this, string.Format("IDE redirection error: {0}", r.ToString()), "Serial-over-LAN", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
        }

        private void redirectToImageFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (computer.RedirectorWrapper.RedirectActive == true) return;
            if (floppyImagePath != null && floppyImagePath.Length > 2) imageFileDialog.FileName = floppyImagePath;
            imageFileDialog.Filter = Properties.Resources.Term_FD_DImg;
            if (imageFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                FileInfo fi = null;
                try { fi = new FileInfo(imageFileDialog.FileName); } catch (Exception) { }
                if (fi == null || fi.Exists == false)
                {
                    MessageBox.Show(this,
                        Properties.Resources.Term_MsgBox_InvalidFile,
                        Properties.Resources.Term_MsgBox_RAct_Redirect,
                        MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }
                else if (fi.FullName.StartsWith("\\\\"))
                {
                    MessageBox.Show(this,
                        Properties.Resources.IderNoNetworkDrive,
                        Properties.Resources.Term_MsgBox_RAct_Redirect,
                        MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }
                else
                {
                    floppyPathLabel.Text = fi.Name;
                    floppyImagePath = imageFileDialog.FileName;
                    Application.UserAppDataRegistry.SetValue("FloppyRedirect", floppyImagePath);
                    foreach (object z in changeTargetFloppyToolStripMenuItem.DropDownItems)
                    {
                        if (z.GetType() == typeof(ToolStripMenuItem))
                        {
                            ToolStripMenuItem x = (ToolStripMenuItem)z;
                            x.Checked = (sender == x);
                        }
                    }
                }
            }
        }

        private void redirectToImageFileToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            if (computer.RedirectorWrapper.RedirectActive == true) return;
            if (cdromImagePath != null && cdromImagePath.Length > 2) imageFileDialog.FileName = cdromImagePath;
            imageFileDialog.Filter = Properties.Resources.Term_FD_CDImg;
            if (imageFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                FileInfo fi = null;
                try { fi = new FileInfo(imageFileDialog.FileName); } catch (Exception) { }
                if (fi == null || fi.Exists == false)
                {
                    MessageBox.Show(this,
                        Properties.Resources.Term_MsgBox_InvalidFile,
                        Properties.Resources.Term_MsgBox_RAct_Redirect,
                        MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }
                else if (fi.FullName.StartsWith("\\\\"))
                {
                    MessageBox.Show(this,
                        Properties.Resources.IderNoNetworkDrive,
                        Properties.Resources.Term_MsgBox_RAct_Redirect,
                        MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }
                else
                {
                    cdromPathLabel.Text = fi.Name;
                    cdromImagePath = imageFileDialog.FileName;
                    Application.UserAppDataRegistry.SetValue("CdromRedirect", cdromImagePath);
                    foreach (object z in changeTargetCDROMToolStripMenuItem.DropDownItems)
                    {
                        if (z.GetType() == typeof(ToolStripMenuItem))
                        {
                            ToolStripMenuItem x = (ToolStripMenuItem)z;
                            x.Checked = (sender == x);
                        }
                    }
                }
            }
        }

        private void activeteRedirectButton_Click(object sender, EventArgs e)
        {
            redirectActiveToolStripMenuItem_Click(this, null);
        }

        private void redirectionToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            if (computer == null) return;

            // Check if IDE-R is available
            redirectActiveToolStripMenuItem.Enabled = (computer.Info != null && computer.Info.IdeRedirectEnabled == true);

            // Check if IDE-R is active
            bool ider = ((computer.RedirectorWrapper != null) && (computer.RedirectorWrapper.RedirectActive));

            redirectActiveToolStripMenuItem.Checked = ider;
            immediatelyToolStripMenuItem.Enabled = !ider;
            gracefullyToolStripMenuItem.Enabled = !ider;
            onResetToolStripMenuItem.Enabled = !ider;

            foreach (object z in changeTargetFloppyToolStripMenuItem.DropDownItems)
            {
                if (z.GetType() == typeof(ToolStripMenuItem))
                {
                    ToolStripMenuItem x = (ToolStripMenuItem)z;
                    x.Enabled = !ider;
                }
            }

            foreach (object z in changeTargetCDROMToolStripMenuItem.DropDownItems)
            {
                if (z.GetType() == typeof(ToolStripMenuItem))
                {
                    ToolStripMenuItem x = (ToolStripMenuItem)z;
                    x.Enabled = !ider;
                }
            }
        }

        private void immediatelyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.UserAppDataRegistry.SetValue("RedirectStart", 1);
            immediatelyToolStripMenuItem.Checked = true;
            gracefullyToolStripMenuItem.Checked = false;
            onResetToolStripMenuItem.Checked = false;
        }

        private void gracefullyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.UserAppDataRegistry.SetValue("RedirectStart", 2);
            immediatelyToolStripMenuItem.Checked = false;
            gracefullyToolStripMenuItem.Checked = true;
            onResetToolStripMenuItem.Checked = false;
        }

        private void onResetToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.UserAppDataRegistry.SetValue("RedirectStart", 3);
            immediatelyToolStripMenuItem.Checked = false;
            gracefullyToolStripMenuItem.Checked = false;
            onResetToolStripMenuItem.Checked = true;
        }

        private void showRedirectionStatsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RedirectStatsForm f = new RedirectStatsForm(computer.RedirectorWrapper, computer);
            f.Show(this);
        }

        private void TerminalForm_Load(object sender, EventArgs e)
        {
            // If silent, don't display terminal
            if (Silent == true)
            {
                this.Left = 20000;
                this.ShowInTaskbar = false;
            }

            // Set the terminal emulation
            string emul = (string)Application.UserAppDataRegistry.GetValue("Emulation", "ANSI");
            bool vt100 = (emul.CompareTo("VT100") == 0);
            aNSIExtendedASCIIToolStripMenuItem.Checked = (vt100 == false);
            vT100UTF8ToolStripMenuItem.Checked = (vt100 == true);
            terminalControl.UTF8TerminalEmulation = vT100UTF8ToolStripMenuItem.Checked;

            // Set the key translation
            emul = (string)Application.UserAppDataRegistry.GetValue("KeyTranslation", "Intel");
            vT100IntelToolStripMenuItem.Checked = emul.Equals("Intel");
            alternateToolStripMenuItem.Checked = emul.Equals("Alternate");
            vT100F1ESCOPToolStripMenuItem.Checked = emul.Equals("VT100+");

            switch ((int)Application.UserAppDataRegistry.GetValue("RedirectStart", 0))
            {
                case 1:
                    immediatelyToolStripMenuItem.Checked = true;
                    gracefullyToolStripMenuItem.Checked = false;
                    onResetToolStripMenuItem.Checked = false;
                    break;
                case 2:
                    immediatelyToolStripMenuItem.Checked = false;
                    gracefullyToolStripMenuItem.Checked = true;
                    onResetToolStripMenuItem.Checked = false;
                    break;
                case 3:
                    immediatelyToolStripMenuItem.Checked = false;
                    gracefullyToolStripMenuItem.Checked = false;
                    onResetToolStripMenuItem.Checked = true;
                    break;
            }

            floppyImagePath = (string)Application.UserAppDataRegistry.GetValue("FloppyRedirect", "dummy.img");
            if (floppyImagePath != null)
            {
                if (floppyImagePath.Length > 2)
                {
                    try
                    {
                        floppyPathLabel.Text = new FileInfo(floppyImagePath).Name;
                    }
                    catch (Exception)
                    {
                        floppyImagePath = null;
                    }
                }
                else
                {
                    floppyPathLabel.Text = floppyImagePath;
                }
                redirectToImageFileToolStripMenuItem.Checked = floppyImagePath.Length > 2;
            }
            cdromImagePath = (string)Application.UserAppDataRegistry.GetValue("CdromRedirect", "dummy.iso");
            if (cdromImagePath != null)
            {
                if (cdromImagePath.Length > 2)
                {
                    try
                    {
                        cdromPathLabel.Text = new FileInfo(cdromImagePath).Name;
                    }
                    catch (Exception)
                    {
                        cdromImagePath = null;
                    }
                }
                else
                {
                    cdromPathLabel.Text = cdromImagePath;
                }
                redirectToImageFileToolStripMenuItem1.Checked = cdromImagePath.Length > 2;
            }
            
            toolStripMenuItem4.Visible = false;
            toolStripMenuItem5.Visible = false;
            for (char d = 'A'; d <= 'Z'; d++)
            {
                DriveInfo drive = new DriveInfo(d.ToString());
                if (drive.DriveType == DriveType.Removable && (d == 'A' || d == 'B'))
                {
                    ToolStripMenuItem x = new ToolStripMenuItem(string.Format(Properties.Resources.Term_Tool_Redirect, (d.ToString() + ":")));
                    x.Tag = d.ToString();
                    x.Click += new EventHandler(redirectDriveMenuItem1_Click);
                    changeTargetFloppyToolStripMenuItem.DropDownItems.Insert(0, x);
                    toolStripMenuItem4.Visible = true;
                    if (floppyImagePath == null)
                    {
                        floppyImagePath = d.ToString() + ":";
                        floppyPathLabel.Text = floppyImagePath;
                        x.Checked = true;
                    }
                    else
                    {
                        if (floppyImagePath != null && floppyImagePath.Length == 2 && floppyImagePath[0] == d) x.Checked = true;
                    }
                }
                if (drive.DriveType == DriveType.CDRom)
                {
                    ToolStripMenuItem x = new ToolStripMenuItem(string.Format(Properties.Resources.Term_Tool_Redirect, (d.ToString() + ":")));
                    x.Tag = d.ToString();
                    x.Click += new EventHandler(redirectDriveMenuItem2_Click);
                    changeTargetCDROMToolStripMenuItem.DropDownItems.Insert(0, x);
                    toolStripMenuItem5.Visible = true;
                    if (cdromImagePath == null)
                    {
                        cdromImagePath = d.ToString() + ":";
                        cdromPathLabel.Text = cdromImagePath;
                        x.Checked = true;
                    }
                    else
                    {
                        if (cdromImagePath != null && cdromImagePath.Length == 2 && cdromImagePath[0] == d) x.Checked = true;
                    }
                }
            }

            if (computer != null)
            {
                // Check the state of the system object and launch a connect if required.
                if (computerOwner)
                {
                    computer.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
                    SystemStateChangedHandler(computer);
                    if (computer.State != AmtSystemObjState.Connected) computer.Connect();
                }

                StartPowerPolling();
            }
            else
            {
                if (NetStream != null)
                {
                    // This is a Telnet Session, de-feature the terminal
                    runScriptToolStripMenuItem.Visible = false;
                    scriptingToolStripMenuItem.Visible = false;
                    captureToolStripMenuItem.Visible = false;
                    serialPacketsToolStripMenuItem.Visible = false;
                    redirectionToolStripMenuItem.Visible = false;
                    resetToolStripMenuItem.Visible = false;
                    connectSOLToolStripMenuItem.Visible = false;
                    disconnectSOLToolStripMenuItem.Visible = false;
                    toolStripMenuItem9.Visible = false;
                    debugFeaturesToolStripMenuItem.Visible = false;
                    serialPanel.Visible = false;
                    bottomPanel.Visible = false;
                    toolStripMenuItem11.Visible = false;
                    advancedPropertiesToolStripMenuItem.Visible = false;
                    StartSerial();
                }
            }

            // Start the relay listener
            relayListener = new TcpListener(IPAddress.Any, 0);
            relayListener.Start();
            relayListener.BeginAcceptTcpClient(new AsyncCallback(ListenerAsyncCallback), null);

            // Simplify the terminal if needed
            if (Simplified == true)
            {
                serialPacketsToolStripMenuItem.Visible = false;
                debugFeaturesToolStripMenuItem.Visible = false;
                tcprPanel.Visible = false;
                toolStripMenuItem2.Visible = false;
                customCommandToolStripMenuItem.Visible = false;
                toolStripMenuItem7.Visible = false;
                showRedirectionStatsToolStripMenuItem.Visible = false;
                immediatelyToolStripMenuItem.Visible = false;
                scriptingToolStripMenuItem.Visible = false;
                runScriptToolStripMenuItem.Visible = false;
                toolStripMenuItem11.Visible = false;
                advancedPropertiesToolStripMenuItem.Visible = false;
            }

            // Terminal used in local mode, get rid of everything.
            if (LocalTerminalMode == true)
            {
                serialPacketsToolStripMenuItem.Visible = false;
                redirectionToolStripMenuItem.Visible = false;
                resetToolStripMenuItem.Visible = false;
                debugFeaturesToolStripMenuItem.Visible = false;
                bottomPanel.Visible = false;
                serialPanel.Visible = false;
                statusStrip.Visible = false;
                connectSOLToolStripMenuItem.Visible = false;
                disconnectSOLToolStripMenuItem.Visible = false;
                toolStripMenuItem9.Visible = false;
                scriptingToolStripMenuItem.Visible = false;
                runScriptToolStripMenuItem.Visible = false;
                toolStripMenuItem11.Visible = false;
                advancedPropertiesToolStripMenuItem.Visible = false;
            }

            // Set the terminal font size
            terminalAutoSizeControl.Font = Properties.Settings.Default.TerminalFont;
        }

        public void StartPowerPolling()
        {
            // Start the polling thread. It only really does anything is the system object is connected.
            if (powerPollThread == null)
            {
                powerPollThread = new Thread(new ThreadStart(PowerStatePollingLoop));
                powerPollThread.Start();
            }
        }

        public void StopPowerPolling()
        {
            if (powerPollThread != null)
            {
                powerPollThreadExit = true;
                //while (powerPollThread.IsAlive) Thread.Sleep(10);
                Thread.Sleep(100);
                powerPollThread = null;
            }
        }

        public void ListenerAsyncCallback(IAsyncResult ar)
        {
            if (relayListener == null) return;
            try
            {
                TcpClient client = relayListener.EndAcceptTcpClient(ar);
                NetworkStream ns = client.GetStream();
                byte[] buf = new byte[256];
                relayClientsBuffers[client] = buf;
                ns.BeginRead(buf, 0, 256, new AsyncCallback(ReceiveAsyncCallback), client);
                relayListener.BeginAcceptTcpClient(new AsyncCallback(ListenerAsyncCallback), null);
            }
            catch (Exception) { }
        }

        public void ReceiveAsyncCallback(IAsyncResult ar)
        {
            TcpClient client = (TcpClient)ar.AsyncState;
            try
            {
                NetworkStream ns = client.GetStream();
                byte[] buf = (byte[])relayClientsBuffers[client];
                int len = ns.EndRead(ar);
                if (len == 0)
                {
                    // Close everything and return
                    relayClientsBuffers.Remove(client);
                    return;
                }

                byte[] buf2 = new byte[len];
                Array.Copy(buf, buf2, len);
                serial.Write(buf2);

                ns.BeginRead(buf, 0, 256, new AsyncCallback(ReceiveAsyncCallback), client);
            }
            catch (Exception)
            {
                relayClientsBuffers.Remove(client);
            }
        }

        private uint old_data_sent;
        private uint old_data_received;
        private void PowerStatePollingLoop()
        {
            while (powerPollThreadExit == false)
            {
                if (computer.State == AmtSystemObjState.Connected)
                {
                    // Get Power
                    if (powerPollThreadExit == true) break;

                    if (powerPollState == true && computer.Remote != null)
                    {
                        AmtCallStatus r = computer.Remote.GetSystemPowerStateEx(out CurrentPowerState, out g_BatteryPower, out g_OsLockup);
                        if (r != AmtCallStatus.SUCCESS) CurrentPowerState = AmtRemoteControl.AmtPowerStates.ERROR_GETTING_STATE;
                    }

                    // Get Redirector
                    if (powerPollThreadExit == true) break;
                    RedirectorStats = computer.RedirectorWrapper.GetRedirectorStats();

                    // Update
                    if (powerPollThreadExit == true) break;
                    try
                    {
                        this.Invoke(new ThreadStart(SetPowerStateLabel));
                    }
                    catch (ObjectDisposedException) { break; }
                    catch (InvalidOperationException) { break; }
                }

                if (powerPollThreadExit == true) break;
                Thread.Sleep(1000);
            }
        }

        private void SetPowerStateLabel()
        {
            // Set power State
            if (powerPollState == true)
            {
                string ss = PowerStates[(int)CurrentPowerState];
                if (CurrentPowerState != AmtRemoteControl.AmtPowerStates.ERROR_GETTING_STATE)
                {
                    if (computer.IsMobile == true)
                    {
                        if (g_BatteryPower == true)
                        {
                            ss += Properties.Resources.Term_BatteryPower;
                        }
                        else
                        {
                            ss += Properties.Resources.Term_PluggedIn;
                        }
                    }
                    if (g_OsLockup == true) ss += Properties.Resources.Term_OsLockup;
                }
                powerStateLabel.Text = ss;
                powerUpToolStripMenuItem.Enabled = (CurrentPowerState != AmtRemoteControl.AmtPowerStates.S0_POWERED);
            }
            else
            {
                powerStateLabel.Text = Properties.Resources.Term_PowerIndicatorDisabled;
                powerUpToolStripMenuItem.Enabled = true;
            }

            // Set the TCP redirector Stats
            int Connections;
            int InData;
            int OutData;
            if (tcpRedirect != null)
            {
                tcpRedirect.GetStats(out Connections, out InData, out OutData);
                if (Connections != 0)
                {
                    tcpLabel1.Text = Connections.ToString() + Properties.Resources.Term_TCP;
                    tcpLabel1.BackColor = Color.LightGreen;
                    tcpLabel2.BackColor = Color.LightGreen;
                }
                else
                {
                    tcpLabel1.BackColor = Color.Transparent;
                    tcpLabel2.BackColor = Color.Transparent;
                    if (tcpRedirect.redirectorList.Count == 0)
                    {
                        tcpLabel1.Text = Properties.Resources.Term_SetPwrState_NoMapping;
                    }
                    else
                    {
                        tcpLabel1.Text = Properties.Resources.Term_SetPwrState_NoTCP;
                    }
                }
                tcpLabel2.Text = String.Format(Properties.Resources.Term_TcpRedirectTransfer, (InData / 1024).ToString(), (OutData / 1024).ToString());
            }

            if (RedirectorStats == null)
            {
                redirectStatusLabel.Text = Properties.Resources.Term_SetPwrState_IDER_Dis;
                if (harddiskPictureBox.Image != hdPictureBox1.Image) harddiskPictureBox.Image = hdPictureBox1.Image;
                return;
            }

            // Set IDE Redirect
            string s = "";
            if (RedirectorStats.data_transfer == true || RedirectorStats.data_sent > 0)
            {
                double SecondsElapsed = (DateTime.Now - LastRedirectionStatisticsTimestamp).TotalMilliseconds / 1000;
                double kBIn = (RedirectorStats.data_received - old_data_received) / 1024 / SecondsElapsed;
                double kBOut = (RedirectorStats.data_sent - old_data_sent) / 1024 / SecondsElapsed;

                s += String.Format(Properties.Resources.Term_SetPwrState_IDER_Act_Format,
                    RedirectorStats.data_sent.ToString("###,###,###,##0"),
                    kBOut.ToString("0.0"),
                    RedirectorStats.data_received.ToString("###,###,###,##0"),
                    kBIn.ToString("0.0"));

                LastRedirectionStatisticsTimestamp = DateTime.Now;

                if (old_data_received < RedirectorStats.data_received)
                {
                    if (harddiskPictureBox.Image != hdPictureBox4.Image) harddiskPictureBox.Image = hdPictureBox4.Image;
                }
                else if (old_data_sent < RedirectorStats.data_sent)
                {
                    if (harddiskPictureBox.Image != hdPictureBox3.Image) harddiskPictureBox.Image = hdPictureBox3.Image;
                }
                else
                {
                    if (harddiskPictureBox.Image != hdPictureBox2.Image) harddiskPictureBox.Image = hdPictureBox2.Image;
                }
            }
            else
            {
                s += Properties.Resources.Term_SetPwrState_IDER_DT_Dis;
                if (harddiskPictureBox.Image != hdPictureBox1.Image) harddiskPictureBox.Image = hdPictureBox1.Image;
            }
            redirectStatusLabel.Text = s;

            old_data_sent = RedirectorStats.data_sent;
            old_data_received = RedirectorStats.data_received;
        }

        private void redirectDriveMenuItem1_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem menuitem = (ToolStripMenuItem)sender;
            floppyPathLabel.Text = ((string)menuitem.Tag) + ":";
            floppyImagePath = floppyPathLabel.Text;
            Application.UserAppDataRegistry.SetValue("FloppyRedirect", floppyImagePath);
            foreach (object z in changeTargetFloppyToolStripMenuItem.DropDownItems)
            {
                if (z.GetType() == typeof(ToolStripMenuItem))
                {
                    ToolStripMenuItem x = (ToolStripMenuItem)z;
                    x.Checked = (menuitem == x);
                }
            }
        }

        private void redirectDriveMenuItem2_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem menuitem = (ToolStripMenuItem)sender;
            cdromPathLabel.Text = ((string)menuitem.Tag) + ":";
            cdromImagePath = cdromPathLabel.Text;
            Application.UserAppDataRegistry.SetValue("CdromRedirect", cdromImagePath);
            foreach (object z in changeTargetCDROMToolStripMenuItem.DropDownItems)
            {
                if (z.GetType() == typeof(ToolStripMenuItem))
                {
                    ToolStripMenuItem x = (ToolStripMenuItem)z;
                    x.Checked = (menuitem == x);
                }
            }
        }

        private void sendKeyCodeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new sendKeyCodeForm(this).Show(this);
        }

        private void packetCodesActiveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (packetCodesActiveToolStripMenuItem.Checked == false)
            {
                serial.Ping();
                serial.WriteDataBlock("INFO", null);
            }
            else
            {
                packetCodesActiveToolStripMenuItem.Checked = false;
                SerialAgentSupportedFeatures = "";
            }
        }

        private void tCPPortRedirectorToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (serial != null)
            {
                TcpRedirectForm tf = new TcpRedirectForm(tcpRedirect, computer);
                tf.Show(this);
            }
        }

        private void processMonitorToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (serial != null)
            {
                ProcessMonitorForm form = new ProcessMonitorForm(serial, computer);
                form.Show();
            }
        }

        private void TerminalForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            TerminalDisposed = true;

            if (CaptureStream != null)
            {
                CaptureStream.Close();
                CaptureStream = null;
                captureTerminalDataToolStripMenuItem.Checked = false;
            }

            // Close all relay clients
            foreach (TcpClient client in relayClientsBuffers.Keys)
            {
                client.Close();
            }
            relayClientsBuffers.Clear();
            relayListener.Stop();
            relayListener = null;

            // Save the TCP Mappings to the registry
            if (tcpRedirect != null)
            {
                string tcpmappings = tcpRedirect.GetMappingString();
                Application.UserAppDataRegistry.SetValue("TcpMappings", tcpmappings);
            }
            if (powerPollThread != null)
            {
                powerPollThreadExit = true;
                //while (powerPollThread.IsAlive) Thread.Sleep(10);
                Thread.Sleep(100);
                powerPollThread = null;
            }
            StopSerial();

            if (tcpRedirect != null)
            {
                tcpRedirect.Dispose(); 
                tcpRedirect = null;
            }
            
            if (computerOwner)
            {
                computer.Disconnect();
                computer.Dispose();
                computer = null;
            }

            // If this is a direct TCP terminal, close the connection
            if (NetStream != null)
            {
                NetStream.Close();
                NetStream = null;
            }
        }

        private void resetToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            if (computer == null || computer.State != AmtSystemObjState.Connected || computer.Remote == null)
            {
                foreach (ToolStripItem i in resetToolStripMenuItem.DropDownItems) i.Enabled = false;
            }
            else
            {
                remoteBootToRedirectFloppyToolStripMenuItem.Enabled = (RedirectorStats != null);
                remoteBootToRedirectCDToolStripMenuItem.Enabled = (RedirectorStats != null);
                foreach (ToolStripItem i in resetToolStripMenuItem.DropDownItems) i.Enabled = true;
                powerUpToolStripMenuItem.Enabled = (CurrentPowerState != AmtRemoteControl.AmtPowerStates.S0_POWERED);

                AMT_BootCapabilities bootCapabilites = computer.Remote.GetBootCapabilities();

                if (!bootCapabilites.ForceDiagnosticBootExist || !bootCapabilites.ForceDiagnosticBoot)
                    remoteRebootToDiagnosticCDToolStripMenuItem.Enabled = false;

                if (!bootCapabilites.ForcePXEBootExist || !bootCapabilites.ForcePXEBoot)
                    remoteRebootToPXEToolStripMenuItem.Enabled = false; 
            }
        }

        private void SystemStateChangedHandler(AmtSystem system)
        {
            if (TerminalDisposed == false && system != null) this.Invoke(new AmtSystem.ObjectStateHandler(SystemStateChangedHandlerEx), new object[] { system });
        }

        private void SystemStateChangedHandlerEx(AmtSystem computer)
        {
            if (TerminalDisposed == true) return;
            switch (computer.State)
            {
                case AmtSystemObjState.Connected:
                    // First check that if in TLS mode, everything is ok.
                    if (computer.UseTls == true && computer.Warnings != 0)
                    {
                        // Problem detected, we can't connect to this computer.
                        WriteSilentStatus("Connection problem, TLS certificate error.");
                        if (Silent == false) MessageBox.Show(this, Properties.Resources.TlsCertificateError, Properties.Resources.Terminal, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        DisposeTerminal();
                    }
                    else
                    {
                        // Ok, connect normally.
                        WriteSilentStatus("Connected");
                        padlockPictureBox.Visible = computer.UseTls;

                        string tls = "";
                        if (computer.UseTls) tls = Properties.Resources.Term_StateChgH_TLS;
                        solLabel.Text = tls + Properties.Resources.Term_MsgBox_StartSerial_1;
                        if (computer.AmtRev != 1) StartSerial();
                        if (serial != null)
                        {
                            solLabel.Text = tls + Properties.Resources.Term_SOL_Conn;
                        }
                        else
                        {
                            solLabel.Text = tls + Properties.Resources.Term_SOL_Disconn;
                        }

                        // Auto-launch the script if needed
                        if (AutoStartScript != null)
                        {
                            scriptingToolStripMenuItem_Click(this, null);
                            scriptingForm.RunScript(AutoStartScript);
                        }
                    }

                    break;
                case AmtSystemObjState.Connecting:
                    WriteSilentStatus("Connecting...");
                    solLabel.Text = Properties.Resources.Term_Web_Conn;
                    StopSerial();
                    break;
                case AmtSystemObjState.Disconnected:
                    solLabel.Text = Properties.Resources.Term_Web_Disconn;
                    StopSerial();
                    if (Silent == true) DisposeTerminal();
                    break;
                case AmtSystemObjState.Error:
                    solLabel.Text = Properties.Resources.Term_Web_Err;
                    StopSerial();
                    break;
            }
        }

        public void WriteSilentStatus(string status)
        {
            if (Silent == false) return;
            string txt = string.Format("{0} - {1}: {2}.\r\n", System.DateTime.Now.ToString(), computer.HostNameIdentity.DefaultName, status);
            FileStream f = new FileStream("script.log", FileMode.Append, FileAccess.Write, FileShare.ReadWrite);
            byte[] buf = UTF8Encoding.UTF8.GetBytes(txt);
            f.Write(buf, 0, buf.Length);
            f.Close();
        }

        /// <summary>
        /// Enable support for Manageability Outpost Tool features
        /// </summary>
        public bool EnableOutpostFeatures
        {
            get { return enableOutpostFeatures; }
            set
            {
                enableOutpostFeatures = value;
                serialPacketsToolStripMenuItem.Visible = value;
                tcprPanel.Visible = value;
            }
        }
        private bool enableOutpostFeatures = false;

        private void connectSOLToolStripMenuItem_Click(object sender, EventArgs e)
        {
            StartSerial();
        }

        private void disconnectSOLToolStripMenuItem_Click(object sender, EventArgs e)
        {
            StopSerial();
        }

        private void fileToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            connectSOLToolStripMenuItem.Enabled = (serial == null);
            disconnectSOLToolStripMenuItem.Enabled = (serial != null);
        }

        private void StartSerial()
        {
            if (serial == null)
            {
                try
                {
                    if (NetStream != null)
                    {
                        serial = new SerialHighDriver(NetStream);
                    }
                    else
                    {
                        serial = new SerialHighDriver(computer.RedirectorWrapper);
                    }
                }
                catch (Exception)
                {
                    serial = null;
                    MessageBox.Show(this, 
                        Properties.Resources.Term_MsgBox_StartSerial_0,
                        Properties.Resources.Term_MsgBox_StartSerial_1,
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                if (serial != null)
                {
                    serial.ProcessCharEvent += new SerialHighDriver.ProcessCharHandler(ProcessCharHandler);
                    serial.ProcessDataBlockEvent += new SerialHighDriver.ProcessDataBlockHandler(ProcessDataBlockHandler);
                    serial.ProcessDataBlockProgressEvent += new SerialHighDriver.ProcessDataBlockProgressHandler(ProcessDataBlockProgressHandler);
                    serial.ProcessEscapeEvent += new SerialHighDriver.ProcessEscapeHandler(ProcessEscapeHandler);
                    serial.ProcessSpecialEvent += new SerialHighDriver.ProcessSpecialHandler(ProcessSpecialEventSink);
                    if (TerminalDisposed == false)
                    {
                        if (tcpRedirect == null)
                        {
                            string tcpmappings = (string)Application.UserAppDataRegistry.GetValue("TcpMappings", null);
                            tcpRedirect = new TcpRedirector(serial, tcpmappings);
                        }
                        terminalControl.ResetScreen();
                    }
                    AmtRedirectorWrapper.IMRResult r = serial.Start();
                    if (r != AmtRedirectorWrapper.IMRResult.IMR_RES_OK)
                    {
                        // We got an error starting serial-over-LAN
                        MessageBox.Show(this, string.Format("Serial-over-LAN error: {0}", r.ToString()), "Serial-over-LAN", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        StopSerial();
                    }
                    else
                    {
                        // Connection success
                        solLabel.Text = Properties.Resources.Term_SOL_Conn;
                    }
                }
            }
        }

        private void StopSerial()
        {
            if (serial != null)
            {
                serial.ProcessCharEvent -= new SerialHighDriver.ProcessCharHandler(ProcessCharHandler);
                serial.ProcessDataBlockEvent -= new SerialHighDriver.ProcessDataBlockHandler(ProcessDataBlockHandler);
                serial.ProcessDataBlockProgressEvent -= new SerialHighDriver.ProcessDataBlockProgressHandler(ProcessDataBlockProgressHandler);
                serial.ProcessEscapeEvent -= new SerialHighDriver.ProcessEscapeHandler(ProcessEscapeHandler);
                serial.ProcessSpecialEvent -= new SerialHighDriver.ProcessSpecialHandler(ProcessSpecialEventSink);
                serial.Dispose();
                serial = null;
                if (TerminalDisposed == false)
                {
                    if (tcpRedirect != null)
                    {
                        tcpRedirect.Dispose();
                        tcpRedirect = null;
                    }
                    terminalControl.ResetScreen();
                    solLabel.Text = Properties.Resources.Term_SOL_Disconn;
                }
            }
        }

        private void vT100UTF8ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            vT100UTF8ToolStripMenuItem.Checked = true;
            aNSIExtendedASCIIToolStripMenuItem.Checked = false;
            Application.UserAppDataRegistry.SetValue("Emulation", "VT100");
            terminalControl.UTF8TerminalEmulation = true;
        }

        private void aNSIExtendedASCIIToolStripMenuItem_Click(object sender, EventArgs e)
        {
            vT100UTF8ToolStripMenuItem.Checked = false;
            aNSIExtendedASCIIToolStripMenuItem.Checked = true;
            Application.UserAppDataRegistry.SetValue("Emulation", "ANSI");
            terminalControl.UTF8TerminalEmulation = false;
        }

        private void vT100IntelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            vT100IntelToolStripMenuItem.Checked = true;
            alternateToolStripMenuItem.Checked = false;
            vT100F1ESCOPToolStripMenuItem.Checked = false;
            Application.UserAppDataRegistry.SetValue("KeyTranslation", "Intel");
        }

        private void microsoftHPToolStripMenuItem_Click(object sender, EventArgs e)
        {
            vT100IntelToolStripMenuItem.Checked = false;
            alternateToolStripMenuItem.Checked = true;
            vT100F1ESCOPToolStripMenuItem.Checked = false;
            Application.UserAppDataRegistry.SetValue("KeyTranslation", "Alternate");
        }

        private void vT100F1ESCOPToolStripMenuItem_Click(object sender, EventArgs e)
        {
            vT100IntelToolStripMenuItem.Checked = false;
            alternateToolStripMenuItem.Checked = false;
            vT100F1ESCOPToolStripMenuItem.Checked = true;
            Application.UserAppDataRegistry.SetValue("KeyTranslation", "VT100+");
        }

        private void deviceManagerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (serial != null) new DeviceManagerForm(serial, computer).Show(this);
        }

        private void fileManagerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (serial != null) new FileExplorerForm(serial, computer).Show(this);
        }

        private void showDebugInformationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AmtDebugForm.ShowForm();
        }

        private void terminalAnalyzerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (analyser == null) analyser = new TerminalAnalyzer(this, computer);
            analyser.Show();
        }

        private void launchPuttyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Directory.SetCurrentDirectory(Application.StartupPath);

            int p = ((IPEndPoint)relayListener.LocalEndpoint).Port;
            try
            {
                Process.Start("putty.exe", "telnet://127.0.0.1:" + p.ToString());
            }
            catch (Exception)
            {
                MessageBox.Show(this, Properties.Resources.ErrorLaunchingApp, Properties.Resources.ApplicationLaunchTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void launchDefaultTelnetToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Directory.SetCurrentDirectory(Application.StartupPath);

            int p = ((IPEndPoint)relayListener.LocalEndpoint).Port;
            try
            {
                Process.Start("telnet://127.0.0.1:" + p.ToString());
            }
            catch (Exception)
            {
                MessageBox.Show(this, Properties.Resources.ErrorLaunchingTelnet, Properties.Resources.ApplicationLaunchTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void launchIAmtTermToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Directory.SetCurrentDirectory(Application.StartupPath);

            int p = ((IPEndPoint)relayListener.LocalEndpoint).Port;
            try
            {
                Process.Start("IAmtTerm.exe", "127.0.0.1 " + p.ToString() + " \"" + string.Format(Properties.Resources.IAmtTermLaunchTitle, p.ToString()) + "\"");
            }
            catch (Exception)
            {
                MessageBox.Show(this, Properties.Resources.ErrorLaunchingApp, Properties.Resources.ApplicationLaunchTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void watchdogsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (serial != null) new WatchdogMonitorForm(serial, computer).Show(this);
        }

        private void powerStateLabel_Click(object sender, EventArgs e)
        {
            powerPollState = !powerPollState;
        }

        private void scriptingToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (scriptingForm != null)
            {
                try
                {
                    scriptingForm.Show();
                    scriptingForm.Activate();
                    return;
                }
                catch (Exception)
                {
                    scriptingForm = null;
                }
            }

            if (scriptingForm == null)
            {
                scriptingForm = new ScriptingForm(this);
                scriptingForm.Show(this);
            }
        }

        private void runScriptToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openScriptFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                scriptingToolStripMenuItem_Click(this, null);
                scriptingForm.RunScript(openScriptFileDialog.FileName);
            }
        }

        private void serialPacketsToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            foreach (ToolStripItem i in serialPacketsToolStripMenuItem.DropDownItems)
            {
                i.Enabled = (serial != null);
            }
        }

        private void padlockPictureBox_Click(object sender, EventArgs e)
        {
            if (computer.UseTls == false) return;
            X509Certificate2[] certs = computer.GetTlsCertificates();
            if (certs != null && certs.Length > 0) X509Certificate2UI.DisplayCertificate(certs[0], Handle);
        }

        private void agentCommandToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AgentNotificationForm f = new AgentNotificationForm(computer);
            f.ShowDialog(this);
        }

        private void sleepToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AgentNotificationForm f = new AgentNotificationForm(computer, AgentNotificationForm.AgentNotificationCommands.Sleep);
            f.ShowDialog(this);
        }

        private void shutdownToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AgentNotificationForm f = new AgentNotificationForm(computer, AgentNotificationForm.AgentNotificationCommands.Shutdown);
            f.ShowDialog(this);
        }

        private void restartToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AgentNotificationForm f = new AgentNotificationForm(computer, AgentNotificationForm.AgentNotificationCommands.Reset);
            f.ShowDialog(this);
        }

        private void logoffToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AgentNotificationForm f = new AgentNotificationForm(computer, AgentNotificationForm.AgentNotificationCommands.Logoff);
            f.ShowDialog(this);
        }

        private void captureTerminalDataToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (CaptureStream != null)
            {
                CaptureStream.Close();
                CaptureStream = null;
                captureTerminalDataToolStripMenuItem.Checked = false;
                toolStripStatusLabel.Text = Properties.Resources.CaptureTextStopped;
            }
            else
            {
                captureFileDialog.DefaultExt = ".txt";
                captureFileDialog.FileName = Properties.Resources.TextCaptureFilename;
                captureFileDialog.Filter = Properties.Resources.TextCaptureFilter;
                if (captureFileDialog.ShowDialog(this) == DialogResult.OK)
                {
                    try
                    {
                        CaptureStream = new FileStream(captureFileDialog.FileName, FileMode.Create);
                    }
                    catch (Exception ex) { Utils.FileErrorMessageBox(this, ex); return; }
                    captureTerminalDataToolStripMenuItem.Checked = true;
                    toolStripStatusLabel.Text = Properties.Resources.CaptureTextStarted;
                }
            }
        }

        private void binaryCaptureToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (serial.RawCaptureStream != null)
            {
                serial.RawCaptureStream.Close();
                serial.RawCaptureStream = null;
                binaryCaptureToolStripMenuItem.Checked = false;
                toolStripStatusLabel.Text = Properties.Resources.CaptureBinaryStopped;
            }
            else
            {
                captureFileDialog.DefaultExt = ".bin";
                captureFileDialog.FileName = Properties.Resources.BinaryCaptureFilename;
                captureFileDialog.Filter = Properties.Resources.BinaryCaptureFilter;
                if (captureFileDialog.ShowDialog(this) == DialogResult.OK)
                {
                    try
                    {
                        serial.RawCaptureStream = new FileStream(captureFileDialog.FileName, FileMode.Create);
                    }
                    catch (Exception ex) { Utils.FileErrorMessageBox(this, ex); return; }
                    binaryCaptureToolStripMenuItem.Checked = true;
                    toolStripStatusLabel.Text = Properties.Resources.CaptureBinaryStarted;
                }
            }
        }

        private void replayFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (replayFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                FileStream f;
                try
                {
                    f = new FileStream(replayFileDialog.FileName, FileMode.Open);
                }
                catch (Exception ex) { Utils.FileErrorMessageBox(this, ex); return; }
                serial.ReplayStream(f);
                f.Close();
            }
        }

        private void TerminalForm_MouseDown(object sender, MouseEventArgs e)
        {
            if (serial != null && packetCodesActiveToolStripMenuItem.Checked == true)
            {
                byte sx = (byte)((e.X / terminalControl.TermFontWidth) + 1);
                byte sy = (byte)((e.Y / terminalControl.TermFontHeight) + 1);
                byte button = 0;
                if ((e.Button & MouseButtons.Left) != 0) button += 1;
                if ((e.Button & MouseButtons.Right) != 0) button += 2;
                serial.WriteDataBlock("MSEV", new byte[] { button, (byte)sx, (byte)sy });
                //if (sx < 80 && sy < 25) toolStripStatusLabel.Text = "Mouse Click on char value = " + ((int)screen[sx + 80 * sy]).ToString();
            }
        }

        private void TerminalForm_MouseUp(object sender, MouseEventArgs e)
        {
            if (serial != null && packetCodesActiveToolStripMenuItem.Checked == true)
            {
                byte sx = (byte)((e.X / terminalControl.TermFontWidth) + 1);
                byte sy = (byte)((e.Y / terminalControl.TermFontHeight) + 1);
                serial.WriteDataBlock("MSEV", new byte[] { 0, (byte)sx, (byte)sy });
            }
        }

        private byte lastMouseX = 0xFF;
        private byte lastMouseY = 0xFF;
        private void TerminalForm_MouseMove(object sender, MouseEventArgs e)
        {
            if (serial != null && packetCodesActiveToolStripMenuItem.Checked == true)
            {
                byte sx = (byte)((e.X / terminalControl.TermFontWidth) + 1);
                byte sy = (byte)((e.Y / terminalControl.TermFontHeight) + 1);
                if (sx != lastMouseX || sy != lastMouseY)
                {
                    byte button = 0;
                    if ((e.Button & MouseButtons.Left) != 0) button += 1;
                    if ((e.Button & MouseButtons.Right) != 0) button += 2;
                    serial.WriteDataBlock("MSEV", new byte[] { button, sx, sy });
                    lastMouseX = sx;
                    lastMouseY = sy;
                }
            }
        }

        private void refreshConsoleToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (serial != null) serial.WriteDataBlock("RDRW", null);
        }

        private void exitConsoleToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (serial != null) serial.WriteDataBlock("REST", null);
        }

        private void windowsManagementToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (serial != null)
            {
                DiagnosticViewerForm form;
                if (computer != null) form = new DiagnosticViewerForm(serial, computer.HostNameIdentity.DefaultName, computer);
                else form = new DiagnosticViewerForm(serial, null, computer);
                form.Show();
            }
        }

        private void copyScreenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            StringBuilder sb = new StringBuilder(25*82);
            for (int y = 0; y < 25; y++)
            {
                for (int x = 0; x < 80; x++)
                {
                    char c = screen[(80 * y) + x];
                    if (c >= 32) sb.Append(c); else sb.Append(" ");
                }
                sb.Append("\r\n");
            }
            Clipboard.SetDataObject(sb.ToString(), true);
        }

        private void pasteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string cb = (string)Utils.UnSerializeFromClipboard(typeof(string));
            if (cb != null && cb.Length > 0)
            {
                if (cb.Length > 300 && MessageBox.Show(this, string.Format(Properties.Resources.TerminalPasteLarge, cb.Length), Properties.Resources.Terminal, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No) return;
                serial.Write(cb);
            }
        }

        private void editToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            string cb = (string)Utils.UnSerializeFromClipboard(typeof(string));
            pasteToolStripMenuItem.Enabled = (cb != null && cb.Length > 0);
        }

        private void advancedPropertiesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new TerminalPropertiesForm(computer).ShowDialog(this);
        }

        private void setFontToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Font backup = fontDialog.Font = terminalAutoSizeControl.Font;
            if (fontDialog.ShowDialog(this) == DialogResult.OK)
            {
                terminalAutoSizeControl.Font = fontDialog.Font;
            }
            else
            {
                terminalAutoSizeControl.Font = backup;
            }
            terminalAutoSizeControl.Invalidate();
        }

        private void fontDialog_Apply(object sender, EventArgs e)
        {
            terminalAutoSizeControl.Font = fontDialog.Font;
            terminalAutoSizeControl.Invalidate();
        }

        private void TerminalForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Save the terminal font size
            Properties.Settings.Default.TerminalFont = terminalAutoSizeControl.Font;
            Properties.Settings.Default.Save();
        }

        private void requestUserConsentToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new UserConsentRequestForm(computer).ShowDialog(this);
        }

        private void viewBootCapabilitiesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new TerminalBootCapabilities(computer).Show(this); 
        }

        private void fileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (computer.UserConsent == null || computer.UserConsent.OptInRequired == null)
            {
                requestUserConsentToolStripMenuItem.Enabled = false;
            }
        }

    }
}
