﻿// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intel’s prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class UserConsentForm : Form
    {
        private AmtSystem system;

        public UserConsentForm(AmtSystem System)
        {
            this.system = System;
            InitializeComponent();
            UpdateFields();
        }

        private void UpdateFields()
        {
            //put in logic to get the current user consent policy
            switch (system.UserConsent.OptInRequired)
            {
                case AmtUserConsent.OptInRequiredValues.None:
                    //if policy == none
                    NoneRadioButton.Checked = true;
                    break;

                //else if policy == KVM Only
                case AmtUserConsent.OptInRequiredValues.KVM:
                    KVMOnlyRadioButton.Checked = true;
                    break;

                //else if policy == all redir apps
                case AmtUserConsent.OptInRequiredValues.All:
                    AllRedirRadioButton.Checked = true;
                    break;
            }
            groupBox1.Enabled = system.UserConsent.CanModifyOptInPolicy;

            if (system.UserConsent.CurrentControlMode.Value == AmtUserConsent.ControlModes.CLIENT_CONTROL_MODE)
            {
                ClientControlLable.Visible = true;
                ClientControlPictureBox.Visible = true;
            }
            else
            {
                ClientControlLable.Visible = false;
                ClientControlPictureBox.Visible = false;
            }

        }

        private void okButton_Click(object sender, EventArgs e)
        {
            //put in logic to change the user consent policy
            if (NoneRadioButton.Checked == true)
            {
                system.UserConsent.OptInRequired = AmtUserConsent.OptInRequiredValues.None;
            }
            else if (KVMOnlyRadioButton.Checked == true)
            {
                system.UserConsent.OptInRequired = AmtUserConsent.OptInRequiredValues.KVM;
            }
            else if (AllRedirRadioButton.Checked == true)
            {
                system.UserConsent.OptInRequired = AmtUserConsent.OptInRequiredValues.All;
            }

            if (true) //if successful
            {
                this.Close();
            }
            else //error changing state
            {
                MessageBox.Show(this,
                    ManageabilityControls.Properties.Resources.UserConsentForm_Error,
                    ManageabilityControls.Properties.Resources.UserConsentForm,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                //don't close form
            }
        }
    }
}
