﻿// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intel’s prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class UserConsentReboot : Form
    {
        AmtSystem _system = null;

        private UserConsentReboot()
        {
            InitializeComponent();
        }

        public UserConsentReboot(AmtSystem system)
            : this()
        {
            _system = system;
            instructionLabel.Text = "The client in question may not support sprites.  A reboot or\npower-up may be in order.  You may try this below, or\nask the user to attempt it.";
            AmtRemoteControl.AmtPowerStates pState;
            bool batPower, osLockup;
            AmtCallStatus r = system.Remote.GetSystemPowerStateEx(out pState, out batPower, out osLockup);
            if (r != AmtCallStatus.SUCCESS)
            {
                pState = AmtRemoteControl.AmtPowerStates.ERROR_GETTING_STATE;
            }
            powerUpButton.Enabled = (pState != AmtRemoteControl.AmtPowerStates.S0_POWERED);
            rebootButton.Enabled = (pState == AmtRemoteControl.AmtPowerStates.S0_POWERED);
        }

        private void powerUpButton_Click(object sender, EventArgs e)
        {
            AmtCallStatus s = _system.Remote.SendRemoteControl(0x11, 0x157, 0x0, 0x0, 0x0, 0x0);
            if (s != AmtCallStatus.SUCCESS && s != AmtCallStatus.FAILED_WEB_CALL)
                MessageBox.Show(this,
                    Properties.Resources.Term_MsgBox_PwrUp_2 + s.ToString(),
                    Properties.Resources.Term_MsgBox_PwrUp_1,
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
            DialogResult = DialogResult.OK;
        }

        private void rebootButton_Click(object sender, EventArgs e)
        {
            AmtCallStatus s = _system.Remote.SendRemoteControl(0x10, 0x157, 0x0, 0x0, 0x0, 0x0);
            if (s != AmtCallStatus.SUCCESS && s != AmtCallStatus.FAILED_WEB_CALL)
                MessageBox.Show(this,
                    Properties.Resources.Term_MsgBox_NormReb_2 + s.ToString(),
                    Properties.Resources.Term_MsgBox_NormReb_1,
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                DialogResult = DialogResult.OK;
        }
    }
}
