﻿// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intel’s prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ManageabilityControls
{
    public partial class WaitForm : Form
    {
        #region Private
        private delegate void UiCallback();
        #endregion

        #region Constructor
        public WaitForm(Form parent, string text)
        {
            InitializeComponent();
            textLabel.Text = text;
        }
        #endregion

        #region Private EventHandlers
        /// <summary>
        /// Initializes form to center the window appropriately
        /// </summary>
        /// <param name="sender">N/A</param>
        /// <param name="e">N/A</param>
        private void ResolvingDnsForm_Load(object sender, EventArgs e)
        {
            this.CenterToParent();
        }
        #endregion

        #region Public methods
        /// <summary>
        /// Allows the window to be hidden from a non-GUI thread
        /// </summary>
        public new void Hide()
        {
            if (this.InvokeRequired == true)
            {
                UiCallback HidingCallback = new UiCallback(Hide);
                this.Invoke(HidingCallback);
                return;
            }

            base.Hide();
        }

        /// <summary>
        /// Allows the window to be Show()n from a non-GUI thread
        /// </summary>
        public new void Show()
        {
            if (this.InvokeRequired == true)
            {
                UiCallback ShowingCallback = new UiCallback(Show);
                this.Invoke(ShowingCallback);
                return;
            }

            base.Show();
        }
        #endregion
    }
}
