// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using ManageabilityStack;


namespace ManageabilityDirectorTool
{
    public partial class AddCertificateHashForm : Form
    {
        private AmtSystem system;

        public AddCertificateHashForm(AmtSystem system)
        {
            this.system = system;
            InitializeComponent();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            if (radioButton1.Checked == true)
            {
                Utils.DropDownItem i = (Utils.DropDownItem)certComboBox.SelectedItem;
                X509Certificate2 cert = (X509Certificate2)i.Tag;

                X509Certificate2 rootCert = ManageabilityStack.CertificateOperations.GetTrustedRootCert(cert);
                if (rootCert == null)
                {
                    MessageBox.Show(this,
                        ManageabilityDirectorTool.Properties.Resources.MsgBox_AddCertHash_Fail + "No root certificate found.",
                        ManageabilityDirectorTool.Properties.Resources.MsgBox_AddCertHash_Title,
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                CertHashEntryType entry = new CertHashEntryType();
                entry.Active = false;
                entry.Default = false;
                string n = CertificateOperations.GetCertificateString(cert.Subject);
                if (n.Length > 32) n = n.Substring(0, 32);
                entry.FriendlyName = n;
                entry.HashType = 1; // SHA-1
                entry.Hash = Utils.HexToBytes(rootCert.Thumbprint);

                AmtCallStatus r = system.SecurityAdmin.AddCertificateHashEntry(entry);
                if (r != AmtCallStatus.SUCCESS)
                {
                    MessageBox.Show(this,
                        ManageabilityDirectorTool.Properties.Resources.MsgBox_AddCertHash_Fail + r.ToString(),
                        ManageabilityDirectorTool.Properties.Resources.MsgBox_AddCertHash_Title,
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                DialogResult = DialogResult.OK;
            }
            else
            {
                CertHashEntryType entry = new CertHashEntryType();
                entry.Active = false;
                entry.Default = false;
                entry.FriendlyName = certNameTextBox.Text;
                entry.HashType = 1; // SHA-1
                entry.Hash = Utils.HexToBytes(hashTextBox.Text);

                AmtCallStatus r = system.SecurityAdmin.AddCertificateHashEntry(entry);
                if (r != AmtCallStatus.SUCCESS)
                {
                    MessageBox.Show(this,
                        ManageabilityDirectorTool.Properties.Resources.MsgBox_AddCertHash_Fail + r.ToString(),
                        ManageabilityDirectorTool.Properties.Resources.MsgBox_AddCertHash_Title,
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                DialogResult = DialogResult.OK;
            }
        }

        private void UpdateInfo()
        {
            bool ok = true;
            if (radioButton1.Checked == true)
            {
                certComboBox.Enabled = true;
                certNameTextBox.Enabled = false;
                hashTextBox.Enabled = false;
            }
            else
            {
                certComboBox.Enabled = false;
                certNameTextBox.Enabled = true;
                hashTextBox.Enabled = true;
                if (certNameTextBox.Text.Length == 0) ok = false;
                if (Utils.CountHexChars(hashTextBox.Text) != 40) ok = false;
            }
            okButton.Enabled = ok;
        }

        private void certNameTextBox_TextChanged(object sender, EventArgs e)
        {
            if (certNameTextBox.Text.Length > 32) certNameTextBox.Text = certNameTextBox.Text.Substring(0, 32);
            UpdateInfo();
        }

        private void AddCertificateHashForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            X509Store CertificateStore = new X509Store(StoreName.My, StoreLocation.CurrentUser);
            CertificateStore.Open(OpenFlags.ReadOnly);
            foreach (X509Certificate2 cert in CertificateStore.Certificates)
            {
                if (CertificateOperations.IsAmtRemoteConfig(cert) && 
                    ManageabilityStack.CertificateOperations.IsKeyExportable(cert) &&
                    ManageabilityStack.CertificateOperations.IsTrustedCert(cert))
                {
                    certComboBox.Items.Add(new Utils.DropDownItem(CertificateOperations.GetCertificateString(cert.Subject), 0, cert));
                }
            }
            if (certComboBox.Items.Count > 0)
            {
                certComboBox.SelectedIndex = 0;
            }
            else
            {
                radioButton1.Checked = false;
                radioButton1.Enabled = false;
                radioButton2.Checked = true;
                certComboBox.Enabled = false;
            }
            CertificateStore.Close();

            UpdateInfo();
        }

        private void hashTextBox_TextChanged(object sender, EventArgs e)
        {
            Utils.FilterHexStr(hashTextBox);
            UpdateInfo();
        }

        private void radioButton2_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

    }
}
