// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;

namespace ManageabilityDirectorTool
{
    public partial class AdminAccessControlSetupForm : Form
    {
        public SecurityTemplate securityTemplate;

        public AdminAccessControlSetupForm(SecurityTemplate securityTemplate)
        {
            this.securityTemplate = securityTemplate;

            InitializeComponent();
        }

        private void AdminAccessControlSetupForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            if (securityTemplate.AdminAclPassword == null)
            {
                doNotChangeCheckBox.Checked = true;
                if (securityTemplate.AdminAclEntry != null) adminUsernameTextBox.Text = securityTemplate.AdminAclEntry;
            }
            else
            {
                doNotChangeCheckBox.Checked = false;
                if (securityTemplate.AdminAclEntry != null) adminUsernameTextBox.Text = securityTemplate.AdminAclEntry;
                adminPasswordMaskedTextBox1.Text = securityTemplate.AdminAclPassword;
                adminPasswordMaskedTextBox2.Text = securityTemplate.AdminAclPassword;
            }

            UpdateInfo();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            if (doNotChangeCheckBox.Checked)
            {
                securityTemplate.AdminAclPassword = null;
            }
            else
            {
                // Double check everything.
                if (adminUsernameTextBox.Text.Length == 0) return;
                if (ManageabilityStack.Utils.CheckPassPhrase(adminPasswordMaskedTextBox1.Text) == false) return;
                if (adminPasswordMaskedTextBox1.Text.Equals(adminPasswordMaskedTextBox2.Text) == false) return;

                securityTemplate.AdminAclEntry = adminUsernameTextBox.Text;
                securityTemplate.AdminAclPassword = adminPasswordMaskedTextBox1.Text;
            }
            DialogResult = DialogResult.OK;
        }

        private void UpdateInfo()
        {
            if (doNotChangeCheckBox.Checked)
            {
                adminUsernameTextBox.Enabled = false;
                adminPasswordMaskedTextBox1.Enabled = false;
                adminPasswordMaskedTextBox2.Enabled = false;
                hideCheckBox.Enabled = false;
                okButton.Enabled = true;
            }
            else
            {
                adminUsernameTextBox.Enabled = true;
                adminPasswordMaskedTextBox1.Enabled = true;
                adminPasswordMaskedTextBox2.Enabled = true;
                hideCheckBox.Enabled = true;
                bool ok = true;
                if (adminUsernameTextBox.Text.Length == 0) ok = false;
                if (ManageabilityStack.Utils.CheckPassPhrase(adminPasswordMaskedTextBox1.Text) == false) ok = false;
                if (adminPasswordMaskedTextBox1.Text.CompareTo(adminPasswordMaskedTextBox2.Text) != 0) ok = false;
                okButton.Enabled = ok;
            }

            if (hideCheckBox.Checked)
            {
                adminPasswordMaskedTextBox1.PasswordChar = (char) 0x25CF; // Bullet character
                adminPasswordMaskedTextBox2.PasswordChar = (char) 0x25CF; // Bullet character
            }
            else
            {
                adminPasswordMaskedTextBox1.PasswordChar = '\0';
                adminPasswordMaskedTextBox2.PasswordChar = '\0';
            }
        }

        private void hideCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void adminUsernameTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void adminPasswordMaskedTextBox1_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void doNotChangeCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void label1_Click(object sender, EventArgs e)
        {

        }

        private void groupBox1_Enter(object sender, EventArgs e)
        {

        }

        private void cancelButton_Click(object sender, EventArgs e)
        {

        }

        private void adminPasswordMaskedTextBox1_MaskInputRejected(object sender, MaskInputRejectedEventArgs e)
        {

        }
    }
}
