// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using ManageabilityStack;

namespace ManageabilityDirectorTool
{
    public partial class CertificateEditForm : Form
    {
        private SecurityTemplate securityTemplate;
        private X509Store CertificateStore;
        private int keysize = 1024;

        public CertificateEditForm(SecurityTemplate securityTemplate)
        {
            this.securityTemplate = securityTemplate;

            InitializeComponent();
        }

        private void CertificateEditForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            
            // Open the certificate stores
            CertificateStore = new X509Store(StoreName.My, StoreLocation.CurrentUser);
            CertificateStore.Open(OpenFlags.ReadOnly);

            // Load the list is issuer certificates
            issuerComboBox.Items.Add(new Utils.DropDownItem(ManageabilityDirectorTool.Properties.Resources.CertEdit_No_Issue, 0, null));
            foreach (X509Certificate2 cert in CertificateStore.Certificates)
            {
                if (cert.HasPrivateKey && ManageabilityStack.CertificateOperations.IsTrustedCert(cert) && ManageabilityStack.CertificateOperations.IsCaCertificate(cert))
                {
                    Utils.DropDownItem i = new Utils.DropDownItem(GetCertificateString(cert.Subject), 0, cert.Thumbprint);
                    issuerComboBox.Items.Add(i);
                }
            }

            // Select the correct issuer certificate
            issuerComboBox.SelectedIndex = 0;
            if (securityTemplate.TLSCertificateIssuerTP != null)
            {
                foreach (Utils.DropDownItem i in issuerComboBox.Items)
                {
                    if (((string)i.Tag) != null && ((string)i.Tag).CompareTo(securityTemplate.TLSCertificateIssuerTP) == 0)
                    {
                        issuerComboBox.SelectedItem = i;
                        break;
                    }
                }
            }

            orgNameTextBox.Text = securityTemplate.IssueOrganisation;
            countryCodeTextBox.Text = securityTemplate.IssueCountryCode;
            keysize = securityTemplate.IssueKeySize;

            switch (keysize)
            {
                case 1024:
                    keySizeActiveTrackBar.Value = 0;
                    break;
                case 1536:
                    keySizeActiveTrackBar.Value = 1;
                    break;
                case 2048:
                    keySizeActiveTrackBar.Value = 2;
                    break;
            }

            UpdateInfo();
        }

        private void CertificateEditForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            // Close the certificate stores
            CertificateStore.Close();
            CertificateStore = null;
        }

        private string GetCertificateString(string certDataStr)
        {
            string s = "";

            int i = certDataStr.IndexOf("CN=");
            if (i >= 0)
            {
                s = certDataStr.Substring(i + 3);
                i = s.IndexOf(",");
                if (i > 0) s = s.Substring(0, i);
            }

            string s2 = null;
            i = certDataStr.IndexOf("O=");
            if (i >= 0)
            {
                s2 = certDataStr.Substring(i + 2);
                i = s2.IndexOf(",");
                if (i > 0) s2 = s2.Substring(0, i);
                s += ", " + s2;
            }

            return s;
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            // Set the issuer certificate
            securityTemplate.TLSCertificateIssuerTP = (string)(((Utils.DropDownItem)(issuerComboBox.SelectedItem)).Tag);
            securityTemplate.IssueOrganisation = orgNameTextBox.Text;
            securityTemplate.IssueCountryCode = countryCodeTextBox.Text;
            securityTemplate.IssueKeySize = keysize;

            DialogResult = DialogResult.OK;
        }

        private void keySizeActiveTrackBar_Scroll(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void UpdateInfo()
        {
            switch (keySizeActiveTrackBar.Value)
            {
                case 0:
                    keysize = 1024;
                    break;
                case 1:
                    keysize = 1536;
                    break;
                case 2:
                    keysize = 2048;
                    break;
            }
            keySizeLabel.Text = keysize.ToString() + ManageabilityDirectorTool.Properties.Resources.CertEdit_Key_Size;

            orgNameTextBox.Enabled = (issuerComboBox.SelectedIndex != 0);
            countryCodeTextBox.Enabled = (issuerComboBox.SelectedIndex != 0);
            keySizeActiveTrackBar.Enabled = (issuerComboBox.SelectedIndex != 0);
            keySizeLabel.Enabled = (issuerComboBox.SelectedIndex != 0);
        }

        private void issuerComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

    }
}
