// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Net;
using System.Text;
using System.Data;
using System.Drawing;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using ManageabilityStack;

namespace ManageabilityDirectorTool
{
    public class CertificateOperations
    {
        //private static string execdir = new FileInfo(System.Reflection.Assembly.GetEntryAssembly().Location).Directory.ToString() + "\\";

        public static X509Certificate2 LoadPemFile(string filename)
        {
            // Check to see that a private key is in the PEM file.
            FileStream fs;
            try
            {
                fs = new FileStream(filename, FileMode.Open);
            }
            catch (Exception ex) { Utils.FileErrorMessageBox(null, ex); return null; }
            byte[] buf = new byte[(int)fs.Length];
            fs.Read(buf, 0, (int)fs.Length);
            fs.Close();
            string pemfile = UTF8Encoding.UTF8.GetString(buf);
            if (pemfile.Contains("-----BEGIN RSA PRIVATE KEY-----") == true)
            {
                // Convert to PKCS#12
                System.Diagnostics.Process p;
                Directory.SetCurrentDirectory(Application.StartupPath);
                if (File.Exists("openssl\\tempcert.p12")) File.Delete("openssl\\tempcert.p12");
                p = Run("openssl\\openssl", "pkcs12 -export -in \"" + filename + "\" -out openssl\\tempcert.p12 -password pass:PASS");
                while (p.HasExited == false) System.Threading.Thread.Sleep(100);
                X509Certificate2 Certificate = new X509Certificate2("openssl\\tempcert.p12", "PASS", X509KeyStorageFlags.Exportable | X509KeyStorageFlags.MachineKeySet | X509KeyStorageFlags.PersistKeySet);
                File.Delete("openssl\\tempcert.p12");
                return Certificate;
            }
            else
            {
                // Convert to CER
                System.Diagnostics.Process p;
                Directory.SetCurrentDirectory(Application.StartupPath);
                if (File.Exists("openssl\\tempcert.cer")) File.Delete("openssl\\tempcert.cer");
                p = Run("openssl\\openssl", "x509 -outform der -in \"" + filename + "\" -out openssl\\tempcert.cer");
                while (p.HasExited == false) System.Threading.Thread.Sleep(100);
                X509Certificate2 Certificate = new X509Certificate2("openssl\\tempcert.cer");
                File.Delete("openssl\\tempcert.cer");
                return Certificate;
            }
        }

        public static bool ChainAndKeyToPemFile(X509Chain chain, string filename)
        {
            Directory.SetCurrentDirectory(Application.StartupPath);

            if (chain.ChainElements.Count == 0) return false;

            // Save the certificate private key to file
            if (File.Exists("openssl\\tempcert.p12")) File.Delete("openssl\\tempcert.p12");
            byte[] bytes = chain.ChainElements[0].Certificate.Export(X509ContentType.Pkcs12, "PASS");
            FileStream fs = new FileStream("openssl\\tempcert.p12", FileMode.Create);
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            // Convert the certificate to a PEM file
            System.Diagnostics.Process p;
            //p = Run("openssl\\openssl", "pkcs12 -in openssl\\tempcert.p12 -out tempcert.pem -passin pass:PASS -nodes");
            p = Run("openssl\\openssl", "pkcs12 -in openssl\\tempcert.p12 -out \"" + filename + "\" -passin pass:PASS -nodes");
            while (p.HasExited == false) System.Threading.Thread.Sleep(100);
            File.Delete("openssl\\tempcert.p12");

            // Save the rest of the chain to PEM file
            if (chain.ChainElements.Count == 1) return true;
            string AllCerts = "\r\n";
            for (int i = 1; i < chain.ChainElements.Count; i++)
            {
                string b1 = Convert.ToBase64String(chain.ChainElements[i].Certificate.Export(X509ContentType.Cert));
                string b2 = "";
                while (b1.Length > 64)
                {
                    b2 += b1.Substring(0, 64) + "\r\n";
                    b1 = b1.Substring(64);
                }
                b2 += b1 + "\r\n";
                AllCerts += ("-----BEGIN CERTIFICATE-----\r\n" + b2 + "-----END CERTIFICATE-----\r\n\r\n");
            }

            // Save trusted certs
            bytes = UTF8Encoding.UTF8.GetBytes(AllCerts);
            fs = new FileStream(filename, FileMode.Append);
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            return true;
        }

        public static bool ChainToPemFile(X509Chain chain, string filename)
        {
            string AllTrustedCerts = "";
            foreach (X509ChainElement element in chain.ChainElements)
            {
                string b1 = Convert.ToBase64String(element.Certificate.Export(X509ContentType.Cert));
                string b2 = "";
                while (b1.Length > 64)
                {
                    b2 += b1.Substring(0, 64) + "\r\n";
                    b1 = b1.Substring(64);
                }
                b2 += b1 + "\r\n";
                AllTrustedCerts += ("-----BEGIN CERTIFICATE-----\r\n" + b2 + "-----END CERTIFICATE-----\r\n");
            }

            // Save trusted certs
            AllTrustedCerts.Replace("\r\n", "");
            byte[] bytes = UTF8Encoding.UTF8.GetBytes(AllTrustedCerts);
            FileStream fs = new FileStream(filename, FileMode.Create);
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            return true;
        }

        public static string GetCertificateString(string certDataStr)
        {
            return ManageabilityCommanderTool.CertificateOperations.GetCertificateString(certDataStr);
        }

        private static Process Run(string fileName, string args)
        {
            ProcessStartInfo info = new ProcessStartInfo(fileName, args);
            info.CreateNoWindow = true;
            info.UseShellExecute = false;
            Process processChild = Process.Start(info);
            return processChild;
        }

        public static string GetCertThumbprint(X509Certificate2 cert)
        {
            string s1 = cert.Thumbprint;
            string s2 = "";
            while (s1.Length > 0)
            {
                if (s2.Length > 0) s2 += "-";
                s2 += s1.Substring(0, 4);
                s1 = s1.Substring(4);
            }
            return s2;
        }

        public static bool HasKeyUse(X509Certificate2 cert, string keyUseOid)
        {
            foreach (X509Extension ex in cert.Extensions)
            {
                if (ex.Format(false).IndexOf("(" + keyUseOid + ")") >= 0) return true;
            }
            return false;
        }

        public static bool IsAmtRemoteConfig(X509Certificate2 cert)
        {
            return HasKeyUse(cert, "2.16.840.1.113741.1.2.3") == true || cert.Subject.Contains("OU=Intel(R) Client Setup Certificate");
        }

        public static bool HasKeyUsages(X509Certificate2 cert)
        {
            return (cert.Extensions.Count != 0);
        }

    }
}
