// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Net;
using System.Data;
using System.Text;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;

namespace ManageabilityDirectorTool
{
    public partial class ExportRemoteSetupForm : Form
    {
        private MainForm mainform;

        public ExportRemoteSetupForm(MainForm mainform)
        {
            InitializeComponent();

            this.mainform = mainform;
        }

        private void ExportRemoteSetupForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            string localFqdn = ManageabilityStack.Utils.GetLocalFqdn();
            if (localFqdn != null) configServerFqdnTextBox.Text = localFqdn;
            remoteConfigComboBox.SelectedIndex = 0;
            preInstalledCertsComboBox.SelectedIndex = 0;
            redirectionComboBox.SelectedIndex = 0;
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                // Get the list of hashs & names
                ArrayList hashVarList = new ArrayList();
                ArrayList hashVarList2 = new ArrayList();
                
                foreach (X509Certificate2 cert in mainform.CertificateStore.Certificates)
                {
                    if (CertificateOperations.IsAmtRemoteConfig(cert) == true && ManageabilityStack.CertificateOperations.GetTrustedRootCert(cert) != null)
                    {
                        MemoryStream m = new MemoryStream();
                        m.WriteByte(1);
                        X509Certificate2 root = ManageabilityStack.CertificateOperations.GetTrustedRootCert(cert);

                        bool dup = false;
                        foreach (string v in hashVarList2)
                        {
                            if (v.Equals(root.Thumbprint) == true)
                            {
                                dup = true;
                                break;
                            }
                        }

                        if (dup == false)
                        {
                            byte[] hashBuf = ManageabilityStack.Utils.HexToBytes(root.Thumbprint);
                            m.Write(hashBuf, 0, hashBuf.Length);
                            string certname = CertificateOperations.GetCertificateString(root.Subject);
                            while (UTF8Encoding.UTF8.GetByteCount(certname) > 32) certname = certname.Substring(0, certname.Length - 1);
                            byte[] nameBuf = UTF8Encoding.UTF8.GetBytes(certname);
                            m.WriteByte((byte)nameBuf.Length);
                            m.Write(nameBuf, 0, nameBuf.Length);
                            hashVarList.Add(new UsbSetupViewerForm.UsbFileVariable(2, 8, m.ToArray()));
                            hashVarList2.Add(root.Thumbprint);
                        }
                    }
                }

                UsbSetupViewerForm.UsbFileRecord record = new UsbSetupViewerForm.UsbFileRecord(existingPasswordTextBox.Text, mainform.GetZtcPassword, preInstalledCertsComboBox.SelectedIndex - 1, remoteConfigComboBox.SelectedIndex - 1, configServerFqdnTextBox.Text, pkiDnsSuffixTextBox.Text, (UsbSetupViewerForm.UsbFileVariable[])hashVarList.ToArray(typeof(UsbSetupViewerForm.UsbFileVariable)), redirectionComboBox.SelectedIndex);
                
                UsbSetupViewerForm.UsbFileRecord[] records = new UsbSetupViewerForm.UsbFileRecord[250];
                for (int i=0;i<250;i++) records[i] = record;

                UsbSetupViewerForm.UsbFile file = new UsbSetupViewerForm.UsbFile(records, 2 ,0);
                byte[] usbFileBytes = file.GetBytes();
                FileStream f = new FileStream(saveFileDialog.FileName, FileMode.Create);
                f.Write(usbFileBytes, 0, usbFileBytes.Length);
                f.Close();
            }
            DialogResult = DialogResult.OK;
        }

        private void UpdateInfo()
        {
            bool ok = true;
            if (existingPasswordTextBox.Text.Length == 0) ok = false;
            okButton.Enabled = ok;
        }

        private void existingPasswordTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }
    }
}
