// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;

namespace ManageabilityDirectorTool
{
    public partial class FqdnEditForm : Form
    {
        private SecurityTemplate securityTemplate;

        public FqdnEditForm(SecurityTemplate securityTemplate)
        {
            InitializeComponent();

            this.securityTemplate = securityTemplate;
            FqdnList = securityTemplate.TrustedFqdnCN;
        }

        public string[] FqdnList
        {
            set
            {
                if (value != null)
                {
                    fqdnListView.BeginUpdate();
                    fqdnListView.Items.Clear();
                    foreach (string s in value)
                    {
                        fqdnListView.Items.Add(new ListViewItem(s));
                    }
                    fqdnListView.EndUpdate();
                }
            }

            get
            {
                ArrayList lst = new ArrayList();
                foreach (ListViewItem l in fqdnListView.Items)
                {
                    lst.Add(l.SubItems[0].Text);
                }
                return (string[])lst.ToArray(typeof(string));
            }
        }


        private void UpdateInfo()
        {
            addButton.Enabled = (fqdnTextBox.Text.Length > 0);
            removeButton.Enabled = (fqdnListView.SelectedItems.Count > 0);
        }

        private void fqdnTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void fqdnListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (fqdnListView.SelectedItems.Count == 1)
            {
                fqdnTextBox.Text = fqdnListView.SelectedItems[0].Text;
            }
            UpdateInfo();
        }

        private void addButton_Click(object sender, EventArgs e)
        {
            fqdnListView.Items.Add(new ListViewItem(fqdnTextBox.Text));
            fqdnTextBox.Text = "";
        }

        private void removeButton_Click(object sender, EventArgs e)
        {
            if (fqdnListView.SelectedItems.Count != 1) return;
            fqdnListView.Items.RemoveAt(fqdnListView.SelectedIndices[0]);
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            securityTemplate.TrustedFqdnCN = FqdnList;
            DialogResult = DialogResult.OK;
        }

        private void FqdnEditForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
        }

    }
}
