// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;

namespace ManageabilityDirectorTool
{
    public partial class HelloViewerForm : Form
    {
        private MainForm parent;

        public HelloViewerForm(MainForm parent)
        {
            InitializeComponent();
            this.parent = parent;
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void ProvisioningServerForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            UpdateInfo();
        }

        public delegate void EmptyHandler();
        public void UpdateInfo()
        {
            this.Invoke(new EmptyHandler(UpdateInfoEx));
        }

        public void UpdateInfoEx()
        {
            helloListView.BeginUpdate();
            helloListView.Items.Clear();

            foreach (AmtHelloServer.AmtHello h in parent.HelloMsgList)
            {
                string s = "";
                if (h.Pid != null)
                {
                    s = string.Format(Properties.Resources.PasswordId, h.Pid);
                }
                else if (h.CertHash != null)
                {
                    s = string.Format(Properties.Resources.CertificateHashNumber, h.CertHash.Length.ToString());
                }

                ListViewItem l = new ListViewItem(new string[] { h.ReceivedTime.ToString(), h.RemoteEndPoint.ToString(), s });
                l.Tag = h;
                helloListView.Items.Insert(0,l);
            }

            helloListView.EndUpdate();
            removeButton.Enabled = (helloListView.SelectedItems.Count > 0);
        }

        private void ProvisioningServerForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            parent.provisioningServerForm = null;
            parent = null;
        }

        private void helloListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            debugInjectButton.Enabled = removeButton.Enabled = (helloListView.SelectedItems.Count > 0);

            if (helloListView.SelectedItems.Count == 1)
            {
                string str = "";
                AmtHelloServer.AmtHello h = (AmtHelloServer.AmtHello)helloListView.SelectedItems[0].Tag;
                str = string.Format(Properties.Resources.HelloMessageString.Replace("\\r\\n","\r\n"), h.ReceivedTime.ToString(), h.RemoteEndPoint.Address.ToString(), h.NetworkPasswordChanged.ToString(), h.GetVersion().ToString(), h.GetGuid().ToString().ToUpper());
                if (h.Pid != null)
                {
                    str += string.Format(Properties.Resources.PassID.Replace("\\r\\n", "\r\n"), h.Pid).Replace("\0", "\\0");
                }
                if (h.CertHash != null)
                {
                    for (int i = 0; i < h.CertHash.Length; i++)
                    {
                        str += string.Format(Properties.Resources.CertificateHash.Replace("\\r\\n", "\r\n"), i, ManageabilityStack.Utils.BytesToHex(h.CertHash[i]));
                    }
                }

                string s = "";
                foreach (byte b in h.Data) s += b.ToString("X2") + " ";
                str += string.Format(Properties.Resources.RawHelloMessage.Replace("\\r\\n", "\r\n"), h.Data.Length.ToString(), s);
                helloTextBox.Text = str;
            }
            else
            {
                helloTextBox.Clear();
            }
        }

        private void removeButton_Click(object sender, EventArgs e)
        {
            if (helloListView.SelectedItems.Count == 0) return;

            helloTextBox.Clear();
            helloListView.BeginUpdate();
            foreach (ListViewItem l in helloListView.SelectedItems)
            {
                AmtHelloServer.AmtHello h = (AmtHelloServer.AmtHello)l.Tag;
                parent.HelloMsgList.Remove(h);
            }
            helloListView.EndUpdate();

            UpdateInfoEx();
        }

        private void debugInjectButton_Click(object sender, EventArgs e)
        {
            if (helloListView.SelectedItems.Count == 0) return;
            if (parent.helloServer == null)
            {
                MessageBox.Show(parent, Properties.Resources.ProvisioningServerDisabled, Properties.Resources.ProvisioningServerTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            foreach (ListViewItem l in helloListView.SelectedItems)
            {
                parent.helloServer.InjectFakeHello((AmtHelloServer.AmtHello)l.Tag);
            }
        }

        private void helloListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete) removeButton_Click(this, null);
        }

    }
}
