// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityDirectorTool
{
    public partial class KeyGeneratorForm : Form
    {
        private MainForm parent;
        private AmtProvisioningKey key;

        public KeyGeneratorForm(MainForm parent, AmtProvisioningKey key)
        {
            InitializeComponent();
            this.parent = parent;
            this.key = key;
        }

        private void KeyGeneratorForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            passwordControl.Hidden = ((string)Application.UserAppDataRegistry.GetValue("KeyHideAdminPass", Boolean.TrueString)).CompareTo(Boolean.TrueString) == 0;

            if (key == null)
            {
                key = AmtProvisioningKey.GetNextKey(Application.UserAppDataRegistry);
                key.State = AmtProvisioningKey.AmtProvisioningKeyState.Creation;
                keyStrengthTrackBar.Value = (int)Application.UserAppDataRegistry.GetValue("KeyStrength", 6);
                passwordControl.Text = (string)Application.UserAppDataRegistry.GetValue("KeyLastAdminPass", "");

                SetKeyStrength(keyStrengthTrackBar.Value);

                label5.Visible = label9.Visible = label10.Visible = true;
                keyStrengthTrackBar.Visible = true;
            }
            else
            {
                SetKeyStrength(0);
                label5.Visible = label9.Visible = label10.Visible = false;
                keyStrengthTrackBar.Visible = false;
                passwordControl.Text = key.AdminPassword;
            }

            // Populate security templates
            configProfileComboBox.Items.Clear();
            SecurityTemplate[] secTemplates = parent.GetAllSecurityTemplates();
            foreach (SecurityTemplate secTemplate in secTemplates)
            {
                Utils.DropDownItem i = new Utils.DropDownItem(secTemplate.Name, 0, secTemplate);
                configProfileComboBox.Items.Add(i);
                if (key.SecurityTemplate != null && key.SecurityTemplate.CompareTo(secTemplate.Name) == 0) configProfileComboBox.SelectedItem = i;
            }
            if (configProfileComboBox.SelectedIndex < 0 && secTemplates.Length > 0) configProfileComboBox.SelectedIndex = 0;

            UpdateInfo();
        }

        private void UpdateInfo()
        {
            if (key == null) return;

            idTextBox.Text = key.Pid;

            if (key.State == AmtProvisioningKey.AmtProvisioningKeyState.Used && key.CompletedRequests == 1) statusLabel.Text = Properties.Resources.UsedOnce;
            else if (key.State == AmtProvisioningKey.AmtProvisioningKeyState.Used && key.CompletedRequests > 1) statusLabel.Text = string.Format(Properties.Resources.UsedNTimes, key.CompletedRequests);
            else statusLabel.Text = key.State.ToString();

            timeLabel.Text = key.GenTime.ToString();

            bool ok = true;
            if (passwordControl.Text.Length == 0) ok = false;
            if (configProfileComboBox.Items.Count == 0) ok = false;
            okButton.Enabled = ok;
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            if (key.State != AmtProvisioningKey.AmtProvisioningKeyState.Used)
            {
                key.State = AmtProvisioningKey.AmtProvisioningKeyState.Pending;
            }
            key.Pps = keyTextBox.Text;
            key.AdminPassword = passwordControl.Text;
            key.ConfigurationProfile = null; // TODO
            if (configProfileComboBox.SelectedItem != null)
            {
                key.SecurityTemplate = ((SecurityTemplate)(((Utils.DropDownItem)configProfileComboBox.SelectedItem).Tag)).Name;
            }
            else
            {
                key.SecurityTemplate = null;
            }
            parent.provisioningKeyStore.Keys[key.Pid] = key;
            parent.provisioningKeyStore.Flush();
            Application.UserAppDataRegistry.SetValue("KeyHideAdminPass", passwordControl.Hidden.ToString());
            Application.UserAppDataRegistry.SetValue("KeyStrength", (int)keyStrengthTrackBar.Value);
            Application.UserAppDataRegistry.SetValue("KeyLastAdminPass", passwordControl.Text);
            DialogResult = DialogResult.OK;
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
        }

        private void keyStrengthTrackBar_Scroll(object sender, EventArgs e)
        {
            SetKeyStrength(keyStrengthTrackBar.Value);
            UpdateInfo();
        }

        private void SetKeyStrength(int g)
        {
            string nullkey = "0000-0000-0000-0000-0000-0000-0000-0000";
            keyTextBox.Text = nullkey.Substring(0, Math.Min(5 * g, 39)) + key.Pps.Substring(Math.Min(5 * g, 39), 39 - Math.Min(5 * g, 39));
        }

        private void adminHideCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void machineNameTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void domainTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void passwordTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

    }
}
