// =============================================================================

// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Net;
using System.Data;
using System.Text;
using System.Drawing;
using System.Threading;
using System.Collections;
using System.Configuration;
using System.Windows.Forms;
using System.ComponentModel;
using System.Xml.Serialization;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using ManageabilityStack;

namespace ManageabilityDirectorTool
{
    public partial class MainForm : Form
    {
        public TreeNode rootNode;
        private TreeNode provisioningKeysNode;
        private TreeNode zeroTouchNode;
        private TreeNode provisioningServerNode;
        private TreeNode certificatesNode;
        public TreeNode securityTemplatesNode;
        private AmtDiscovery discovery = new AmtDiscovery();
        private bool disposing = false;
        public AmtProvisioningKeyStore provisioningKeyStore;
        public ArrayList HelloMsgList = new ArrayList();
        public AmtHelloServer helloServer;
        public HelloViewerForm provisioningServerForm = null;
        public X509Store CertificateStore = null;
        public X509Store CertificateRootStore = null;
        public string GetZtcPassword { get { return adminPasswordMaskedTextBox.Text; } }
        private int securityProfileGroupBoxHeight = 0;
        private int securitySettingsGroupBoxHeight = 0;
        private string RemoteConfigNoProfileLabel = null;
        private ArrayList HelloRequestPendingList = new ArrayList();
        private Thread HelloRequestHandler = null;
        // For discovery.
        private IPAddress a1 = null;
        private IPAddress a2 = null;

        public MainForm(string[] args)
        {
            InitializeComponent();

            securityProfileGroupBoxHeight = securityProfileGroupBox.Height;
            securitySettingsGroupBoxHeight = securitySettingsGroupBox.Height;
            provisioningKeyStore = new AmtProvisioningKeyStore(Application.UserAppDataRegistry);
            discovery.OnDiscoveryEvent += new AmtDiscovery.AmtDiscoveryEventHandler(AmtDiscoveryEventSink);

            provisioningServerNode = new TreeNode(Properties.Resources.TreeNode_Conf_Server, 4, 5);
            mainTreeView.Nodes.Add(provisioningServerNode);
            certificatesNode = new TreeNode(Properties.Resources.TreeNode_Cert_Mgr, 4, 5);
            provisioningServerNode.Nodes.Add(certificatesNode);
            securityTemplatesNode = new TreeNode(Properties.Resources.TreeNode_Template, 4, 5);
            provisioningServerNode.Nodes.Add(securityTemplatesNode);
            provisioningKeysNode = new TreeNode(Properties.Resources.TreeNode_1_Touch, 4, 5);
            provisioningServerNode.Nodes.Add(provisioningKeysNode);
            zeroTouchNode = new TreeNode(Properties.Resources.TreeNode_0_Touch, 4, 5);
            provisioningServerNode.Nodes.Add(zeroTouchNode);
            provisioningServerNode.Expand();

            rootNode = new TreeNode(Properties.Resources.TreeNode_Network, 4, 5);
            mainTreeView.Nodes.Add(rootNode);

            byte[] systemSettingsBin = (byte[])Application.UserAppDataRegistry.GetValue("SystemsSettings2", null);

            if (systemSettingsBin != null)
            {
                ArrayList settings = null;

                try
                {
                    settings = (ArrayList)Utils.UnserializeBinary(systemSettingsBin);
                }
                catch (Exception) { }

                if (settings == null)
                {
                    Application.UserAppDataRegistry.DeleteSubKey("SystemsSettings2", false);
                    MessageBox.Show(
                        Properties.Resources.MainForm_Reg_ErrStr,
                        Properties.Resources.MainForm_RegSetting,
                        MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }
                else
                {
                    foreach (AmtSerialSystemSettings setting in settings)
                    {
                        AddComputer(setting.GetSystem());
                    }
                }
            }
            rootNode.Expand();

            // Preload some of the classes we will need.
            new Thread(new ThreadStart(AmtSystem.PreLoadModules)).Start();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information, "Manageability Director Tool v" + ManageabilityControls.AboutForm.VersionString);

            mainToolStripStatusLabel.Text = mainToolStripStatusLabel.Text.Replace("0.00", ManageabilityControls.AboutForm.VersionString);
            mainToolStripStatusLabel.Text = Utils.FilterStrEx(mainToolStripStatusLabel.Text);
            this.Text = Utils.FilterStrEx(this.Text);

            adminPasswordMaskedTextBox.Text = (string)Application.UserAppDataRegistry.GetValue("ZtcAdminPassword", "");
            RemoteConfigNoProfileLabel = ztcProfileLabel.Text;
            ztcProfileLabel.Text = (string)Application.UserAppDataRegistry.GetValue("ZtcSecurityProfile", RemoteConfigNoProfileLabel);

            discoveryStartAddrTextBox.Text = (string)Application.UserAppDataRegistry.GetValue("DiscoveryStartAddr", "192.168.1.1");
            discoveryEndAddrTextBox.Text = (string)Application.UserAppDataRegistry.GetValue("DiscoveryEndAddr", "192.168.1.254");
            mainToolTip.Active = displayToolTipsToolStripMenuItem.Checked;
            updateDiscoveryInfo();

            mainTreeView.SelectedNode = rootNode;
            mainTreeView_ArrangeContextMenu();
            mainTreeView_AfterSelect(this, null);

            mainToolTip.Active = ((string)Application.UserAppDataRegistry.GetValue("ToolTips", Boolean.TrueString)).CompareTo(Boolean.TrueString) == 0;
            displayToolTipsToolStripMenuItem.Checked = mainToolTip.Active;

            hideKeysCheckBox.Checked = (((string)Application.UserAppDataRegistry.GetValue("HideProvisioningKeys", bool.FalseString)).CompareTo(bool.TrueString) == 0);

            HelloMsgList = (ArrayList)ManageabilityStack.Utils.UnserializeBinary((byte[])Application.UserAppDataRegistry.GetValue("HelloMessages", null));
            if (HelloMsgList != null)
            {
                try
                {
                    foreach (AmtHelloServer.AmtHello h in HelloMsgList) { }
                }
                catch (Exception) { HelloMsgList = null; }
            }
            if (HelloMsgList == null) HelloMsgList = new ArrayList();

            bool srv = (((string)Application.UserAppDataRegistry.GetValue("ProvisioningServer", bool.TrueString)).CompareTo(bool.TrueString) == 0);
            provServerPortNumericUpDown.Value = (int)Application.UserAppDataRegistry.GetValue("ProvisioningPort", 9971);
            HelloServerState(srv);

            CertificateStore = new X509Store(StoreName.My, StoreLocation.CurrentUser);
            CertificateStore.Open(OpenFlags.ReadWrite);

            CertificateRootStore = new X509Store(StoreName.Root, StoreLocation.CurrentUser);
            CertificateRootStore.Open(OpenFlags.ReadWrite);

            // Load the server log
            byte[] logbin = (byte[])Application.UserAppDataRegistry.GetValue("ServerLog", null);
            if (logbin != null)
            {
                ArrayList log = (ArrayList)ManageabilityStack.Utils.UnserializeBinary(logbin);
                foreach (ListViewItem li in log) {provisioningServerListView.Items.Add(li);}
            }
            clearServerLogButton.Enabled = (provisioningServerListView.Items.Count > 0);

            ServicePointManager.ServerCertificateValidationCallback = new System.Net.Security.RemoteCertificateValidationCallback(RemoteCertificateValidationCallbackSink);

            LoadTemplates();
            if (GetSecurityTemplates(ztcProfileLabel.Text) == null) setZtcProfileButton.BackColor = Color.LightCoral;

            // Check that the time is set on this computer.
            if (DateTime.Now.Year < 2007) AddProvisioningServerLog(Properties.Resources.IncorrectTime);
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            disposing = true;

            CertificateStore.Close();
            CertificateRootStore.Close();

            Application.UserAppDataRegistry.SetValue("ProvisioningServer", (helloServer != null).ToString());
            Application.UserAppDataRegistry.SetValue("ProvisioningPort", (int)provServerPortNumericUpDown.Value);
            HelloServerState(false);

            SaveSettings();

            discovery.Stop();
            Application.UserAppDataRegistry.SetValue("DiscoveryStartAddr", discoveryStartAddrTextBox.Text);
            Application.UserAppDataRegistry.SetValue("DiscoveryEndAddr", discoveryEndAddrTextBox.Text);

            // Lets dispose all of the AmtSystem objects since they include threads that will not clean up.
            ArrayList l = Utils.GetNodeTagsofTypeRec(rootNode, typeof(AmtSystem));
            foreach (AmtSystem computer in l)
            {
                computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
                computer.OnFetchCompleted -= new AmtSystem.FetchHandler(FetchHandler);
                computer.Disconnect();
                computer.Dispose();
            }

            // Save the server log
            ArrayList log = new ArrayList();
            foreach (ListViewItem li in provisioningServerListView.Items) {log.Add(li);}
            Application.UserAppDataRegistry.SetValue("ServerLog", ManageabilityStack.Utils.SerializeBinary(log));

            // Save Hello messages
            Application.UserAppDataRegistry.SetValue("HelloMessages", ManageabilityStack.Utils.SerializeBinary(HelloMsgList));
        }

        private delegate void SafeAddComputerHandler(AmtSystem computer);
        private void SafeAddComputer(AmtSystem computer)
        {
            object[] p = new object[1];
            p[0] = computer;
            this.Invoke(new SafeAddComputerHandler(AddComputer), p);
        }

        private void AddComputer(AmtSystem computer)
        {
            // Limit Commander to 20 computers
            if (Utils.GetNodeTagsofType(rootNode, typeof(AmtSystem)).Count >= 20)
            {
                MessageBox.Show("No more than 20 computers can be added", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            int icon = GetIconIndex(computer);
            TreeNode systemNode = new TreeNode(computer.HostNameIdentity.DisplayName + Properties.Resources.SlashSeperator + computer.Username, icon, icon);
            computer.AutoFetchCache = false;
            computer.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
            computer.OnFetchCompleted += new AmtSystem.FetchHandler(FetchHandler);
            systemNode.Tag = computer;
            Utils.AddTreeNodeSorted(rootNode, systemNode);
            rootNode.Expand();
        }

        private int GetIconIndex(AmtSystem computer)
        {
            int icon = 14;
            if (computer.State == AmtSystemObjState.Connected)
            {
                icon += 2;
                if (computer.Warnings != 0) icon += 1;
            }
            else if (computer.State == AmtSystemObjState.Connecting) icon += 1;
            if (computer.IsMobile) icon += 4;
            return icon;
        }

        public void HelloServerState(bool x)
        {
            if (x)
            {
                helloServer = new AmtHelloServer((ushort)provServerPortNumericUpDown.Value);
                helloServer.StateChangeEvent += new AmtHelloServer.StateChangeHandler(HelloServerStateChangeSink);
                helloServer.HelloEvent += new AmtHelloServer.HelloHandler(HelloServerSink);
                helloServer.Start();
                if (helloServer.IsListening == false)
                {
                    MessageBox.Show(this, string.Format(Properties.Resources.NetworkPortError, helloServer.ListenPort), Properties.Resources.MsgBox_Prov_Serv, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    helloServer.Stop();
                    helloServer.StateChangeEvent -= new AmtHelloServer.StateChangeHandler(HelloServerStateChangeSink);
                    helloServer.HelloEvent -= new AmtHelloServer.HelloHandler(HelloServerSink);
                    helloServer = null;
                }
                else
                {
                    //AddProvisioningServerLog("Server Started.");
                }
            }
            else
            {
                if (helloServer != null)
                {
                    helloServer.Stop();
                    helloServer.StateChangeEvent -= new AmtHelloServer.StateChangeHandler(HelloServerStateChangeSink);
                    helloServer.HelloEvent -= new AmtHelloServer.HelloHandler(HelloServerSink);
                    helloServer = null;
                    //AddProvisioningServerLog("Server Stopped.");
                }
            }

            if (helloServer == null)
            {
                helloMsgToolStripStatusLabel.Text = Properties.Resources.Prov_Stop;
            }
            else
            {
                helloMsgToolStripStatusLabel.Text = Properties.Resources.Prov_Listen + helloServer.ListenPort.ToString();
            }
        }

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void aboutIntelAMTManagerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new ManageabilityControls.AboutForm(Properties.Resources.ProductName, Properties.Resources.ProductSubName).ShowDialog(this);
        }

        private void AmtDiscoveryEventSink(AmtDiscovery amtDiscovery, IPEndPoint ep, AmtDiscovery.AmtDiscoveryEvent e, AmtDiscovery.AmtDiscoveredComputer computer)
        {
            if (this.disposing == true) return;
            object[] p = new object[4];
            p[0] = amtDiscovery;
            p[1] = ep;
            p[2] = e;
            p[3] = computer;
            this.Invoke(new AmtDiscovery.AmtDiscoveryEventHandler(AmtDiscoveryEventSinkEx), p);
        }

        private void AmtDiscoveryEventSinkEx(AmtDiscovery amtDiscovery, IPEndPoint ep, AmtDiscovery.AmtDiscoveryEvent e, AmtDiscovery.AmtDiscoveredComputer computer)
        {
            ListViewItem l;
            if (e == AmtDiscovery.AmtDiscoveryEvent.DiscoveryFinished)
            {
                startDiscoveryButton.Text = Properties.Resources.Discovery_Start;
                discoveryProgressBar.Visible = false;
            }
            else if (e == AmtDiscovery.AmtDiscoveryEvent.LaunchingDiscovery)
            {
                discoveryProgressBar.Visible = true;
            }
            else if (e == AmtDiscovery.AmtDiscoveryEvent.MachineFound ||
                e == AmtDiscovery.AmtDiscoveryEvent.RmcpPong)
            {
                int icon = 16;
                if (computer.IsMobile) icon += 3;
                if (computer.Info.IndexOf("Switchbox") >= 0) icon = 1;
                string hostname = computer.EndPoint.Address.ToString();
                if (resolveNamesToolStripMenuItem.Checked) hostname = computer.Hostname;
                l = new ListViewItem(new string[] { hostname + " - " + computer.Info, computer.Method.ToString(), computer.State.ToString(), computer.State.ToString() }, icon);
                l.Tag = computer;
                lock (discoveryListView)
                {
                    discoveryListView.Items.Add(l);
                }
                if (e == AmtDiscovery.AmtDiscoveryEvent.MachineFound)
                {
                    discoveryProgressBar.Increment(1);
                }
            }
            else if (e == AmtDiscovery.AmtDiscoveryEvent.MachineNotFound)
            {
                discoveryProgressBar.Increment(1);
            }
            else if (e == AmtDiscovery.AmtDiscoveryEvent.RmcpPing)
            {
                discoveryProgressBar.Increment(1);
            }
            else if (e == AmtDiscovery.AmtDiscoveryEvent.StartingTcpSweep)
            {
                discoveryProgressBar.Value = 0;
                discoveryProgressBar.Maximum = (int)AmtDiscovery.GetAddressDiff(a1, a2);
            }
        }

        private bool updateDiscoveryInfo()
        {
            Utils.FilterIpAddress(discoveryStartAddrTextBox);
            Utils.FilterIpAddress(discoveryEndAddrTextBox);

            IPAddress x1 = null;
            IPAddress x2 = null;
            try
            {
                x1 = IPAddress.Parse(discoveryStartAddrTextBox.Text);
                x2 = IPAddress.Parse(discoveryEndAddrTextBox.Text);
            }
            catch (Exception)
            {
                startDiscoveryButton.Enabled = false;
                return false;
            }

            long diff = AmtDiscovery.GetAddressDiff(x1, x2);
            if (diff < 0 || diff > 65536)
            {
                startDiscoveryButton.Enabled = false;
                return false;
            }

            startDiscoveryButton.Enabled = true;
            return true;
        }

        public void mainTreeView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            #region Node - Root
            if (mainTreeView.SelectedNode == null || mainTreeView.SelectedNode == rootNode)
            {
                RightView(ListViewPanels.Discovery);
                if (discovery.IsRunning == true) {startDiscoveryButton.Text = Properties.Resources.Discovery_Stop;} else {startDiscoveryButton.Text = Properties.Resources.Discovery_Start;}
                return;
            }

            if (mainTreeView.SelectedNode == provisioningKeysNode)
            {
                RightView(ListViewPanels.OneTouch);

                provisioningKeysListView.BeginUpdate();
                provisioningKeysListView.Items.Clear();
                if (hideKeysCheckBox.Checked)
                {
                    provisioningKeysListView.Columns[2].Text = Properties.Resources.ListViewTitleSecurityProfile;
                    foreach (AmtProvisioningKey pk in provisioningKeyStore.Keys.Values)
                    {
                        string state = pk.State.ToString();
                        if (pk.CompletedRequests > 1) state += string.Format(" ({0})", pk.CompletedRequests.ToString());
                        ListViewItem l = new ListViewItem(new string[] { pk.Pid.ToString(), state, pk.SecurityTemplate });
                        l.Tag = pk;
                        provisioningKeysListView.Items.Add(l);
                    }
                }
                else
                {
                    provisioningKeysListView.Columns[2].Text = Properties.Resources.ListViewTitleStoredKey;
                    foreach (AmtProvisioningKey pk in provisioningKeyStore.Keys.Values)
                    {
                        string state = pk.State.ToString();
                        if (pk.CompletedRequests > 1) state += string.Format(" ({0})", pk.CompletedRequests.ToString());
                        ListViewItem l = new ListViewItem(new string[] { pk.Pid.ToString(), state, pk.Pps.ToString() });
                        l.Tag = pk;
                        provisioningKeysListView.Items.Add(l);
                    }
                }
                provisioningKeysListView.EndUpdate();
                deleteKeysButton.Enabled = (provisioningKeysListView.SelectedItems.Count > 0);
                openKeyButton.Enabled = (provisioningKeysListView.SelectedItems.Count == 1);

                return;
            }

            if (mainTreeView.SelectedNode == provisioningServerNode)
            {
                RightView(ListViewPanels.ProvisioningServer);

                if (helloServer == null || helloServer.IsListening == false)
                {
                    provServerPortNumericUpDown.Enabled = true;
                    provServerStateLabel.Text = Properties.Resources.provServer_Disabled;
                    provServerButton.Text = Properties.Resources.provServer_Start_Server;
                }
                else
                {
                    provServerPortNumericUpDown.Enabled = false;
                    provServerStateLabel.Text = Properties.Resources.provServer_Active;
                    provServerButton.Text = Properties.Resources.provServer_Stop_Server;
                }

                clearServerLogButton.Enabled = (provisioningServerListView.Items.Count > 0);
            }

            if (mainTreeView.SelectedNode == certificatesNode)
            {
                RightView(ListViewPanels.Certificates);

                certificatesListView.BeginUpdate();
                certificatesListView.Items.Clear();
                foreach (X509Certificate2 cert in CertificateStore.Certificates)
                {
                    string t = Properties.Resources.CertType_Cer;
                    if (cert.Subject.CompareTo(cert.Issuer) == 0) t = Properties.Resources.CertType_Root;
                    if (IsTrusted(cert)) t = Properties.Resources.CertType_Trusted;
                    if (cert.HasPrivateKey) t += Properties.Resources.CertType_Sign;
                    if (cert.HasPrivateKey && ManageabilityStack.CertificateOperations.IsKeyExportable(cert) == false) t += Properties.Resources.CertType_NoExport;

                    int icon = 1;
                    if (ManageabilityStack.CertificateOperations.CheckCertificate(cert) == X509ChainStatusFlags.NoError) icon = 0;
                    if (cert.HasPrivateKey == true && !ManageabilityStack.CertificateOperations.IsKeyExportable(cert)) icon = 2;

                    string issuer = CertificateOperations.GetCertificateString(cert.Issuer);
                    if (cert.Subject.CompareTo(cert.Issuer) == 0) issuer = Properties.Resources.CertType_SelfSign;
                    ListViewItem l = new ListViewItem(new string[] { t, CertificateOperations.GetCertificateString(cert.Subject), issuer }, icon);
                    l.Tag = cert;
                    certificatesListView.Items.Add(l);
                }
                certificatesListView.EndUpdate();
            }

            if (mainTreeView.SelectedNode == securityTemplatesNode)
            {
                RightView(ListViewPanels.SecurityProfiles);
            }

            if (mainTreeView.SelectedNode == zeroTouchNode)
            {
                RightView(ListViewPanels.RemoteConfiguration);

                // Fill the list view with all setup certificates
                remoteConfigCertListView.BeginUpdate();
                remoteConfigCertListView.Items.Clear();
                int exportCertCount = 0;
                foreach (X509Certificate2 cert in CertificateStore.Certificates)
                {
                    if (CertificateOperations.IsAmtRemoteConfig(cert) && ManageabilityStack.CertificateOperations.IsKeyExportable(cert))
                    {
                        int icon = 4;
                        X509Certificate2 rootCert = ManageabilityStack.CertificateOperations.GetTrustedRootCert(cert);
                        string hash = "";
                        if (rootCert != null)
                        {
                            exportCertCount++;
                            hash = CertificateOperations.GetCertThumbprint(rootCert);
                            icon = 1;
                            if (CertificateOperations.GetCertificateString(cert.Subject) == Utils.GetLocalFqdn()) icon = 0; // TODO, Check IP address too
                        }

                        ListViewItem l = new ListViewItem(new string[] { CertificateOperations.GetCertificateString(cert.Subject), hash }, icon);
                        l.Tag = cert;
                        remoteConfigCertListView.Items.Add(l);
                    }
                }
                exportRemoteConfigButton.Enabled = (exportCertCount > 0 && adminPasswordMaskedTextBox.Text.Length > 0);
                remoteConfigCertListView.EndUpdate();

                if (this.ztcProfileLabel.Equals(RemoteConfigNoProfileLabel) == true || GetSecurityTemplates(ztcProfileLabel.Text) == null)
                {
                    setZtcProfileButton.BackColor = Color.LightCoral;
                }
                else
                {
                    setZtcProfileButton.BackColor = exportRemoteConfigButton.BackColor;
                }
            }

            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(SecurityTemplate))
            {
                RightView(ListViewPanels.SecurityProfile);
                SecurityTemplate st = (SecurityTemplate)mainTreeView.SelectedNode.Tag;
                securityTemplateNameLabel.Text = st.Name;
                string fqdns = "";
                if (st.TrustedFqdnCN != null && st.TrustedFqdnCN.Length > 0)
                {
                    foreach (string fqdn in st.TrustedFqdnCN) fqdns += ", " + fqdn;
                    if (fqdns.Length > 2) fqdns = fqdns.Substring(2);
                    trustedFqdnTemplateLabel.Text = fqdns;
                    trustedFqdnEditButton.BackColor = enabledInterfacesEditButton.BackColor;
                }
                else
                {
                    trustedFqdnTemplateLabel.Text = Properties.Resources.Label_NONE;
                    trustedFqdnEditButton.BackColor = Color.LightCoral;
                }

                if (st.EnabledInterfaces == null || (st.EnabledInterfaces.IdeRedirection == false && st.EnabledInterfaces.SerialOverLAN == false && st.EnabledInterfaces.WebUI == false))
                {
                    enabledInterfacesTemplateLabel.Text = Properties.Resources.Label_NONE;
                }
                else
                {
                    string enabledInterfacesStr = "";
                    if (st.EnabledInterfaces.WebUI) enabledInterfacesStr += ", " + Properties.Resources.WebInterface;
                    if (st.RedirectionPort) enabledInterfacesStr += ", " + Properties.Resources.RedirectPort;
                    if (st.EnabledInterfaces.SerialOverLAN) enabledInterfacesStr += ", " + Properties.Resources.SOL;
                    if (st.EnabledInterfaces.IdeRedirection) enabledInterfacesStr += ", " + Properties.Resources.IDER;
                    enabledInterfacesTemplateLabel.Text = enabledInterfacesStr.Substring(2);
                }

                int securityMode = 0;
                if (st.TlsAthenticationLocal == TlsAthenticationType.NoAuth) { tlsAuthTemplateLabel.Text = Properties.Resources.PasswordSecurity; }
                if (st.TlsAthenticationLocal == TlsAthenticationType.ServerAuth && st.TlsAthenticationRemote == TlsAthenticationType.ServerAuth) { tlsAuthTemplateLabel.Text = Properties.Resources.TLSSecurity; securityMode = 1; }
                if (st.TlsAthenticationLocal == TlsAthenticationType.ServerAuth && st.TlsAthenticationRemote == TlsAthenticationType.MutualAuth) { tlsAuthTemplateLabel.Text = Properties.Resources.TLSSecurityConsoleAuthentication; securityMode = 2; }
                if (st.TlsAthenticationLocal == TlsAthenticationType.MutualAuth && st.TlsAthenticationRemote == TlsAthenticationType.ServerAuth) { tlsAuthTemplateLabel.Text = Properties.Resources.TLSSecurityAgentAuthentication; securityMode = 2; }
                if (st.TlsAthenticationLocal == TlsAthenticationType.MutualAuth && st.TlsAthenticationRemote == TlsAthenticationType.MutualAuth) { tlsAuthTemplateLabel.Text = Properties.Resources.TLSSecurityConsoleAgentAuthentication; securityMode = 2; }

                int itemheight = trustedFqdnTemplateLabel.Top - trustedCertTemplateLabel.Top;

                if (securityMode == 0)
                {
                    securityProfileGroupBox.Height = securityProfileGroupBoxHeight - (itemheight * 3);
                    label59.Visible = false;
                    certTemplateLabel.Visible = false;
                    certEditButton.Visible = false;
                }
                else if (securityMode == 1)
                {
                    securityProfileGroupBox.Height = securityProfileGroupBoxHeight - (itemheight * 2);
                    label59.Visible = true;
                    certTemplateLabel.Visible = true;
                    certEditButton.Visible = true;
                    label57.Visible = false;
                    trustedCertTemplateLabel.Visible = false;
                    trustedRootsEditButton.Visible = false;
                }
                else if (securityMode == 2)
                {
                    securityProfileGroupBox.Height = securityProfileGroupBoxHeight;
                    label59.Visible = true;
                    certTemplateLabel.Visible = true;
                    certEditButton.Visible = true;
                    label57.Visible = true;
                    trustedCertTemplateLabel.Visible = true;
                    trustedRootsEditButton.Visible = true;
                }

                if (st.TLSCertificateIssuerTP == null)
                {
                    certTemplateLabel.Text = Properties.Resources.No_Cert_Issued;
                    certEditButton.BackColor = Color.LightCoral;
                }
                else
                {
                    X509Certificate2 cert = GetCertFromThumpprint(st.TLSCertificateIssuerTP);
                    if (cert == null)
                    {
                        certTemplateLabel.Text = Properties.Resources.Temp_Invalid;
                        certEditButton.BackColor = Color.LightCoral;
                    }
                    else
                    {
                        certTemplateLabel.Text = string.Format(Properties.Resources.IssueCertificateUsingRoot, CertificateOperations.GetCertificateString(cert.Subject));
                        certEditButton.BackColor = enabledInterfacesEditButton.BackColor;
                    }
                }

                if (st.TrustedRootCertificatesTP == null || st.TrustedRootCertificatesTP.Length == 0)
                {
                    trustedCertTemplateLabel.Text = Properties.Resources.Label_NONE;
                    trustedRootsEditButton.BackColor = Color.LightCoral;
                }
                else
                {
                    trustedCertTemplateLabel.Text = st.TrustedRootCertificatesTP.Length.ToString() + Properties.Resources.Trust_Cer_Count;
                    trustedRootsEditButton.BackColor = enabledInterfacesEditButton.BackColor;
                }

                if (st.AdminAclPassword == null)
                {
                    adminNameProfileLabel.Text = Properties.Resources.AdminProfileLabel_Text_Unch;
                }
                else
                {
                    adminNameProfileLabel.Text = Properties.Resources.AdminProfileLabel_Text_Name + st.AdminAclEntry;
                }

                string tmpstr = "";
                if (st.UserAclEntries == null || st.UserAclEntries.Length == 0) tmpstr = Properties.Resources.Account_None;
                else if (st.UserAclEntries.Length == 1) tmpstr = Properties.Resources.Account_One;
                else tmpstr = st.UserAclEntries.Length.ToString() + Properties.Resources.Account_Multiple;
                userAccountsProfileLabel.Text = tmpstr;
            }
            #endregion

            #region Node - AmtSystem
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                RightView(ListViewPanels.Connection);

                hostnameTextBox.Text = computer.HostNameIdentity.DisplayName;
                usernameTextBox.Text = computer.Username;
                passwordTextBox.Text = computer.Password;
                rememberPasswordCheckBox.Checked = computer.RememberPassword;
                webLinkLabel.Text = computer.WebURL;

                string EnabledText = Properties.Resources.Node_AMTSys_Enabled;
                string DisabledText = Properties.Resources.Node_AMTSys_Disabled;

                if (computer.State == AmtSystemObjState.Connected)
                {
                    if (computer.UseTls == true)
                    {
                        // Set the padlock icon
                        padlockPictureBox.Visible = true;
                        hostnameTextBox.Width = connectButton.Left - hostnameTextBox.Left - 34;
                        usernameTextBox.Width = connectButton.Left - usernameTextBox.Left - 34;
                    }
                    else
                    {
                        // Clear the padlock icon
                        padlockPictureBox.Visible = false;
                        hostnameTextBox.Width = connectButton.Left - hostnameTextBox.Left - 3;
                        usernameTextBox.Width = connectButton.Left - usernameTextBox.Left - 3;
                    } 
                    
                    connectButton.Text = Properties.Resources.ConnBtn_Disconn;
                    passwordTextBox.Enabled = false;
                    rememberPasswordCheckBox.Enabled = false;
                    webUiToggleButton.Enabled = true;

                    unProvisionButton.Enabled = true;
                    editComputerNameButton.Enabled = true;

                    AmtGeneralInfo.ProvisioningMode provisioningMode = computer.Info.GetProvisioningMode();
                    string provisioningModeStr = Properties.Resources.ProvisionSmallBuisness;
                    if (provisioningMode == AmtGeneralInfo.ProvisioningMode.Enterprise) provisioningModeStr = Properties.Resources.ProvisionEnterprise;
                    provisioningModeLabel.Text = provisioningModeStr;
                    ProvisioningStateType pst = computer.Info.GetProvisioningState();

                    connectionWarningGroupBox.Visible = (computer.Warnings != 0);
                    connectionWarningTextBox.Text = "";
                    if ((computer.Warnings & AmtSystem.AmtConnectWarnings.NameMismatch) != 0 && computer.UseTls == true) connectionWarningTextBox.Text += Properties.Resources.NameMismatchTls + "\r\n";
                    if ((computer.Warnings & AmtSystem.AmtConnectWarnings.NameMismatch) != 0 && computer.UseTls == false) connectionWarningTextBox.Text += Properties.Resources.NameMismatch + "\r\n";
                    if ((computer.Warnings & AmtSystem.AmtConnectWarnings.CertificateExpired) != 0) connectionWarningTextBox.Text += Properties.Resources.CertificateExpired + "\r\n";
                    if ((computer.Warnings & AmtSystem.AmtConnectWarnings.CertificateInvalid) != 0) connectionWarningTextBox.Text += string.Format(Properties.Resources.CertificateInvalid, computer.CertificateError.ToString()) + "\r\n";
                    if ((computer.Warnings & AmtSystem.AmtConnectWarnings.CertificateUntrusted) != 0) connectionWarningTextBox.Text += Properties.Resources.CertificateUntrusted + "\r\n";
                    if ((computer.Warnings & AmtSystem.AmtConnectWarnings.AuthPrivateKeyNotExportable) != 0) connectionWarningTextBox.Text += Properties.Resources.AuthCertificatePrivateKeyNoExport + "\r\n";
                    if ((computer.Warnings & AmtSystem.AmtConnectWarnings.OpenSslNotAvailable) != 0) connectionWarningTextBox.Text += Properties.Resources.OpenSslNotAvailable + "\r\n";

                    provisioningModeStr = "";
                    if (pst == ProvisioningStateType.ProvisioningStateIn) provisioningModeStr = Properties.Resources.ProvMode_In_Setup;
                    if (pst == ProvisioningStateType.ProvisioningStatePost) provisioningModeStr = Properties.Resources.ProvMode_Post_Setup;
                    if (pst == ProvisioningStateType.ProvisioningStatePre) provisioningModeStr = Properties.Resources.ProvMode_Pre_Setup;
                    provisioningStateLabel.Text = provisioningModeStr;

                    AmtSecurityAdmin.ProvisioningMode pst2 = computer.SecurityAdmin.GetProvisioningMode();
                    provisioningModeStr = Properties.Resources.ProvMode_Unknown;
                    if (pst2 == AmtSecurityAdmin.ProvisioningMode.Enterprise) provisioningModeStr = Properties.Resources.ProvisionEnterprise;
                    if (pst2 == AmtSecurityAdmin.ProvisioningMode.SmallBusiness) provisioningModeStr = Properties.Resources.ProvisionSmallBuisness;
                    if (pst2 == AmtSecurityAdmin.ProvisioningMode.Current) provisioningModeStr = Properties.Resources.ProvMode_Current;
                    computerNameDomainLabel.Text = computer.Network.GetHostName();
                    if (computer.Network.GetDomainName() != null) computerNameDomainLabel.Text += "." + computer.Network.GetDomainName();
                    if ((computer.Warnings & AmtSystem.AmtConnectWarnings.NameMismatch) == 0)
                    {
                        editComputerNameButton.BackColor = unProvisionButton.BackColor;
                    }
                    else
                    {
                        editComputerNameButton.BackColor = Color.LightCoral;
                    }
                    iamtVersionLabel.Text = computer.SecurityAdmin.GetCoreVersion();
                    if (computer.Assets.GetBios() != null)
                    {
                        biosVersionLabel.Text = computer.Assets.GetBios().Version;
                    }
                    else
                    {
                        biosVersionLabel.Text = Properties.Resources.BIOSVer_Label_Unknown;
                    }
                    if (computer.SecurityAdmin.WebUiEnabled) { webUiEnabledLabel.Text = EnabledText; } 
                    else { webUiEnabledLabel.Text = DisabledText; }
                }
                else
                {
                    // Computer is disconnected

                    // Clear the padlock icon
                    padlockPictureBox.Visible = false;
                    hostnameTextBox.Width = connectButton.Left - hostnameTextBox.Left - 3;
                    usernameTextBox.Width = connectButton.Left - usernameTextBox.Left - 3;

                    string labelNA = Properties.Resources.Label_NA;
                    if (computer.State == AmtSystemObjState.Connecting) { connectButton.Text = Properties.Resources.ConnBtn_AbortConn; } 
                    else {connectButton.Text = Properties.Resources.ConnBtn_Conn;}

                    passwordTextBox.Enabled = true;
                    rememberPasswordCheckBox.Enabled = true;
                    webUiEnabledLabel.Text = labelNA;
                    webUiEnabledLabel.Enabled = true;
                    webUiToggleButton.Enabled = false;
                    connectionWarningGroupBox.Visible = false;
                    connectionWarningTextBox.Text = "";

                    unProvisionButton.Enabled = false;
                    editComputerNameButton.Enabled = false;

                    computerNameDomainLabel.Text = labelNA;
                    iamtVersionLabel.Text = labelNA;
                    biosVersionLabel.Text = labelNA;
                    provisioningModeLabel.Text = labelNA;
                    provisioningStateLabel.Text = labelNA;
                }

                return;
            }

            if (mainTreeView.SelectedNode.Tag == null && mainTreeView.SelectedNode.Text == Properties.Resources.Zero_Touch_Setting)
            {
                AmtSystem system = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
                RightView(ListViewPanels.RemoteConfigurationSettings);

                if (system.SecurityAdmin.GetZeroTouchConfigurationMode() == true)
                {
                    zeroTouchStatusLabel.Text = Properties.Resources.ZT_Status_Enabled;
                }
                else
                {
                    zeroTouchStatusLabel.Text = Properties.Resources.ZT_Status_Disabled;
                }

                AmtSecurityAdmin.CertHashEntry[] entries = system.SecurityAdmin.GetCertificateHashEntries();
                ztcCertHashListView.BeginUpdate();
                ztcCertHashListView.Items.Clear();
                if (entries != null && entries.Length > 0)
                {
                    foreach (AmtSecurityAdmin.CertHashEntry entry in entries)
                    {
                        string state = Properties.Resources.ZT_Cert_Disabled;
                        if (entry.Entry.Active) state = Properties.Resources.ZT_Cert_Active;
                        string defaultState = "";
                        if (entry.Entry.Default) defaultState += "*";
                        ListViewItem l = new ListViewItem(new string[] { defaultState + entry.Entry.FriendlyName, state, Utils.BytesToHex(entry.Entry.Hash) });
                        l.Tag = entry;
                        ztcCertHashListView.Items.Add(l);
                    }
                }
                ztcCertHashListView.EndUpdate();
            }

            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSecuritySettings))
            {
                AmtSystem system = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
                AmtSecuritySettings profile = system.SecurityAdmin.GetSecurityProfile();
                mainTreeView.SelectedNode.Tag = profile;
                RightView(ListViewPanels.SecuritySettings);

                // Network Access
                if (profile.TrustedFqdnCN == null || profile.TrustedFqdnCN.Length == 0)
                {
                    trustedFqdnLabel.Text = Properties.Resources.Label_NONE;
                }
                else
                {
                    string fqdns = "";
                    foreach (string fqdn in profile.TrustedFqdnCN) {fqdns += ", " + fqdn;}
                    trustedFqdnLabel.Text = fqdns.Substring(2);
                }

                string s = "";
                if (profile.EnabledInterfaces.WebUI) s += ", " + Properties.Resources.WebInterface;
                if (profile.RedirectionPort) s += ", " + Properties.Resources.RedirectPort;
                if (profile.EnabledInterfaces.SerialOverLAN) s += ", " + Properties.Resources.SOL;
                if (profile.EnabledInterfaces.IdeRedirection) s += ", " + Properties.Resources.IDER;
                if (s.Length > 0) spEnabledInterfacesLabel.Text = s.Substring(2); else spEnabledInterfacesLabel.Text = Properties.Resources.Label_NONE;

                // Intel AMT Security
                int securityMode = 0;
                if (profile.TlsAthenticationLocal == TlsAthenticationType.NoAuth) { spTlsAuthLabel.Text = Properties.Resources.PasswordSecurity; }
                if (profile.TlsAthenticationLocal == TlsAthenticationType.ServerAuth && profile.TlsAthenticationRemote == TlsAthenticationType.ServerAuth) { spTlsAuthLabel.Text = Properties.Resources.TLSSecurity; securityMode = 1; }
                if (profile.TlsAthenticationLocal == TlsAthenticationType.ServerAuth && profile.TlsAthenticationRemote == TlsAthenticationType.MutualAuth) { spTlsAuthLabel.Text = Properties.Resources.TLSSecurityConsoleAuthentication; securityMode = 2; }
                if (profile.TlsAthenticationLocal == TlsAthenticationType.MutualAuth && profile.TlsAthenticationRemote == TlsAthenticationType.ServerAuth) { spTlsAuthLabel.Text = Properties.Resources.TLSSecurityAgentAuthentication; securityMode = 2; }
                if (profile.TlsAthenticationLocal == TlsAthenticationType.MutualAuth && profile.TlsAthenticationRemote == TlsAthenticationType.MutualAuth) { spTlsAuthLabel.Text = Properties.Resources.TLSSecurityConsoleAgentAuthentication; securityMode = 2; }

                int itemheight = trustedFqdnLabel.Top - spTrustedCertLabel.Top;

                if (securityMode == 0)
                {
                    securitySettingsGroupBox.Height = securitySettingsGroupBoxHeight - (itemheight * 3);
                    label36.Visible = false;
                    spTlsCertLabel.Visible = false;
                    secProfileViewCertButton.Visible = false;
                }
                else if (securityMode == 1)
                {
                    securitySettingsGroupBox.Height = securitySettingsGroupBoxHeight - (itemheight * 2);
                    label36.Visible = true;
                    spTlsCertLabel.Visible = true;
                    secProfileViewCertButton.Visible = true;
                    label29.Visible = false;
                    spTrustedCertLabel.Visible = false;
                    secProfileViewTrustedCertButton.Visible = false;
                }
                else if (securityMode == 2)
                {
                    securitySettingsGroupBox.Height = securitySettingsGroupBoxHeight;
                    label36.Visible = true;
                    spTlsCertLabel.Visible = true;
                    secProfileViewCertButton.Visible = true;
                    label29.Visible = true;
                    spTrustedCertLabel.Visible = true;
                    secProfileViewTrustedCertButton.Visible = true;
                }

                // PKI Capabilities
                s = "";
                string label_bytes = Properties.Resources.Label_bytes;
                string label_instances = Properties.Resources.Label_instances;

                if (profile.PkiCapabilities != null)
                {
                    foreach (uint l in profile.PkiCapabilities.SupportedKeyLengths) { s += ", " + l.ToString(); }
                    if (s.Length > 2) s = s.Substring(2);
                    spSupportedKeyLengthsLabel.Text = s + label_bytes;
                    spCertChainMaxSizeLabel.Text = profile.PkiCapabilities.CertChainMaxSize.ToString() + label_bytes;
                    spRootCertMaxLabel.Text = profile.PkiCapabilities.RootCertMaxInstances.ToString() + label_instances + profile.PkiCapabilities.RootCertMaxSize + label_bytes;
                    spFqdnSuffixMaxLabel.Text = profile.PkiCapabilities.FqdnSuffixMaxEntries.ToString() + label_instances + profile.PkiCapabilities.FqdnSuffixMaxEntryLength + label_bytes;
                    spCrlStoreSizeLabel.Text = profile.PkiCapabilities.CrlStoreSize.ToString() + label_bytes;
                }
                else
                {
                    string LabelUnsupport = Properties.Resources.Label_Unsupport;
                    spSupportedKeyLengthsLabel.Text = LabelUnsupport;
                    spCertChainMaxSizeLabel.Text = LabelUnsupport;
                    spRootCertMaxLabel.Text = LabelUnsupport;
                    spFqdnSuffixMaxLabel.Text = LabelUnsupport;
                    spCrlStoreSizeLabel.Text = LabelUnsupport;
                }
                

                // Certificates
                if (profile.TLSCertificate == null || profile.TLSCertificate.Length == 0)
                {
                    secProfileViewCertButton.Enabled = false;
                    spTlsCertLabel.Text = Properties.Resources.Label_NONE;
                }
                else
                {
                    secProfileViewCertButton.Enabled = true;
                    spTlsCertLabel.Text = CertificateOperations.GetCertificateString(profile.TLSCertificate[0].Subject);
                }
                if (profile.TrustedRootCertificates == null || profile.TrustedRootCertificates.Length == 0)
                {
                    secProfileViewTrustedCertButton.Enabled = false;
                    spTrustedCertLabel.Text = Properties.Resources.Label_NONE;
                }
                else
                {
                    secProfileViewTrustedCertButton.Enabled = true;
                    spTrustedCertLabel.Text = profile.TrustedRootCertificates.Length.ToString();
                }

                // ACL
                spAdminAclLabel.Text = profile.AdminAclEntry;


                // Power v2
                if (profile.CoreVersionDec.CompareTo((float)(1.9)) > 0 && profile.CoreVersionDec.CompareTo((float)(2.5)) < 0)
                {
                    spActiveStateAcLabel.Text = "S" + profile.ActiveStateAC.ToString();
                    if (profile.WakeOnNetAccessThresholdAC == -1)
                    {
                        spWonAccessThresholdlabel.Text = Properties.Resources.Label_Not_Spec;
                    }
                    else
                    {
                        spWonAccessThresholdlabel.Text = "S" + profile.WakeOnNetAccessThresholdAC.ToString();
                    }
                    if (profile.WakeOnNetAccessSleepTimer == -1)
                    {
                        spWonSleepTimerLabel.Text = Properties.Resources.Label_Not_Spec;
                    }
                    else
                    {
                        spWonSleepTimerLabel.Text = profile.WakeOnNetAccessSleepTimer.ToString();
                    }
                    powerSavingGroupBox2.Visible = false; // TRUE
                    powerSavingGroupBox3.Visible = false;
                }
                
                // Power v3
                if (profile.CoreVersionDec.CompareTo((float)(2.5)) >= 0)
                {
                    spActivePowerPackLabel.Text = profile.ActivePowerPackage.ToString();
                    if (profile.WakeOnNetAccessSleepTimer == -1)
                    {
                        spWonSleepTimerLabel2.Text = Properties.Resources.Label_Not_Spec;
                    }
                    else
                    {
                        spWonSleepTimerLabel2.Text = profile.WakeOnNetAccessSleepTimer.ToString();
                    }
                    powerSavingGroupBox2.Visible = false;
                    powerSavingGroupBox3.Visible = false; // TRUE
                }

            }

            #endregion
        }

        private void startDiscoveryButton_Click(object sender, EventArgs e)
        {
            if (discovery.IsRunning == false)
            {
                if (updateDiscoveryInfo() == false) return;
                if (IPAddress.TryParse(discoveryStartAddrTextBox.Text, out a1) == false) return;
                if (IPAddress.TryParse(discoveryEndAddrTextBox.Text, out a2) == false) return;

                discoveryProgressBar.Maximum = AmtDiscovery.PingCount;
                discoveryProgressBar.Value = 0;
                lock (discoveryListView)
                {
                    discoveryListView.Items.Clear();
                }

                discovery.Start(a1, a2);
                startDiscoveryButton.Text = Properties.Resources.Discovery_Stop;
            }
            else
            {
                discovery.Stop();
                discoveryProgressBar.Visible = false;
                startDiscoveryButton.Text = Properties.Resources.Discovery_Start;
            }
        }

        private void addDiscoveredButton_Click(object sender, EventArgs e)
        {
            if (discoveryListView.SelectedItems.Count != 1) return;
            ListViewItem l = discoveryListView.SelectedItems[0];
            if (l.Tag == null || l.ImageIndex == 1) return;
            AmtDiscovery.AmtDiscoveredComputer discComputer = (AmtDiscovery.AmtDiscoveredComputer)l.Tag;
            string hostname = discComputer.EndPoint.Address.ToString();
            if (resolveNamesToolStripMenuItem.Checked) hostname = discComputer.Hostname;
            ManageabilityControls.AddSystemForm f = new ManageabilityControls.AddSystemForm(hostname);
            if (f.ShowDialog(this) == DialogResult.OK)
            {
                AmtSystem computer = new AmtSystem(f.Hostname, 16992, f.Username, f.Password, f.RememberPassword, true);
                computer.UseTls = (discComputer.State == AmtDiscovery.ProvisionState.TLS);
                computer.AmtRev = discComputer.AmtRev;
                computer.AmtRevMin = discComputer.AmtRevMin;
                AddComputer(computer);
                rootNode.Expand();
                SaveSettings();
            }
        }

        public void SaveSettings()
        {
            ArrayList settings = new ArrayList();
            foreach (TreeNode n in rootNode.Nodes)
            {
                AmtSystem s = (AmtSystem)n.Tag;
                settings.Add(new AmtSerialSystemSettings(s));
            }
            byte[] buf = Utils.SerializeBinary(settings);
            Application.UserAppDataRegistry.SetValue("SystemsSettings2", buf);
        }

        enum ListViewPanels
        {
            Discovery,
            ComputerGroup,
            Connection,
            ProvisioningServer,
            RemoteConfiguration,
            OneTouch,
            Certificates,
            SecuritySettings,
            SecurityProfile,
            SecurityProfiles,
            RemoteConfigurationSettings,
        }

        private void RightView(ListViewPanels v)
        {
            switch (v)
            {
                case ListViewPanels.Discovery: // Discovery
                    if (splitContainer1.Panel2.Controls.Contains(masterDiscoveryPanel)) break;
                    splitContainer1.Panel2.Controls.Add(masterDiscoveryPanel);
                    splitContainer1.Panel2.Controls.RemoveAt(0);
                    break;
                case ListViewPanels.Connection: // Computer
                    if (splitContainer1.Panel2.Controls.Contains(masterConnectPanel)) break;
                    splitContainer1.Panel2.Controls.Add(masterConnectPanel);
                    splitContainer1.Panel2.Controls.RemoveAt(0);
                    break;
                case ListViewPanels.ProvisioningServer: // Provisioning Server
                    if (splitContainer1.Panel2.Controls.Contains(configurationServerPanel)) break;
                    splitContainer1.Panel2.Controls.Add(configurationServerPanel);
                    splitContainer1.Panel2.Controls.RemoveAt(0);
                    break;
                case ListViewPanels.OneTouch: // One Touch Setup
                    if (splitContainer1.Panel2.Controls.Contains(oneTouchPanel)) break;
                    splitContainer1.Panel2.Controls.Add(oneTouchPanel);
                    splitContainer1.Panel2.Controls.RemoveAt(0);
                    break;
                case ListViewPanels.Certificates: // Certificates
                    if (splitContainer1.Panel2.Controls.Contains(certificatesPanel)) break;
                    splitContainer1.Panel2.Controls.Add(certificatesPanel);
                    splitContainer1.Panel2.Controls.RemoveAt(0);
                    break;
                case ListViewPanels.SecuritySettings: // Security Settings
                    if (splitContainer1.Panel2.Controls.Contains(securitySettingsPanel)) break;
                    splitContainer1.Panel2.Controls.Add(securitySettingsPanel);
                    splitContainer1.Panel2.Controls.RemoveAt(0);
                    break;
                case ListViewPanels.SecurityProfile: // Security Profile
                    if (splitContainer1.Panel2.Controls.Contains(securityTemplatePanel)) break;
                    splitContainer1.Panel2.Controls.Add(securityTemplatePanel);
                    splitContainer1.Panel2.Controls.RemoveAt(0);
                    break;
                case ListViewPanels.SecurityProfiles: // Security Profiles
                    if (splitContainer1.Panel2.Controls.Contains(securityTemplatesPanel)) break;
                    splitContainer1.Panel2.Controls.Add(securityTemplatesPanel);
                    splitContainer1.Panel2.Controls.RemoveAt(0);
                    break;
                case ListViewPanels.RemoteConfiguration: // Remote Configuration Setup
                    if (splitContainer1.Panel2.Controls.Contains(zeroTouchPanel)) break;
                    splitContainer1.Panel2.Controls.Add(zeroTouchPanel);
                    splitContainer1.Panel2.Controls.RemoveAt(0);
                    break;
                case ListViewPanels.RemoteConfigurationSettings: // Remote Configuration Settings
                    if (splitContainer1.Panel2.Controls.Contains(zeroTouchSettingsPanel)) break;
                    splitContainer1.Panel2.Controls.Add(zeroTouchSettingsPanel);
                    splitContainer1.Panel2.Controls.RemoveAt(0);
                    break;
            }
            ((Panel)splitContainer1.Panel2.Controls[0]).BorderStyle = BorderStyle.Fixed3D;
        }

        private void discoveryListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            addComputerToolStripMenuItem.Enabled = addDiscoveredButton.Enabled = ((discoveryListView.SelectedItems.Count == 1) && (discoveryListView.SelectedItems[0].Tag != null) && discoveryListView.SelectedItems[0].ImageIndex != 1);
        }

        private void manualAddButton_Click(object sender, EventArgs e)
        {
            ManageabilityControls.AddSystemForm f = new ManageabilityControls.AddSystemForm(false);
            if (f.ShowDialog(this) == DialogResult.OK)
            {
                AmtSystem computer = new AmtSystem(f.Hostname, 16992, f.Username, f.Password, f.RememberPassword, true);
                AddComputer(computer);
                rootNode.Expand();
                SaveSettings();
            }
        }

        private void FetchHandler(AmtSystem computer, object fetch)
        {
            if (this.disposing == true) return;
            object[] p = new object[2];
            p[0] = computer;
            p[1] = fetch;
            Invoke(new AmtSystem.FetchHandler(FetchHandlerEx), p);
        }

        private void FetchHandlerEx(AmtSystem computer, object fetch)
        {
            /* TODO
            try
            {
                if (fetch == computer.Assets) UpdateTreeAssets(computer);
                if (fetch == computer.Storage) UpdateTreeStorage(computer);
                if (fetch == computer.Watchdog) UpdateTreeWatchdogs(computer);
                if (fetch == computer.CircuitBreaker) { UpdateTreePolicies(computer); UpdateTreeFilters(computer); }
            }
            catch (NullReferenceException e)
            {
                string s = e.Message;
                System.Diagnostics.Debugger.Break();
            }
            */

            try
            {
                if (fetch == null && computer.State == AmtSystemObjState.Connected)
                {
                    launchSetTemplate(computer);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        private void SystemStateChangedHandler(AmtSystem system)
        {
            if (this.disposing == true) return;
            this.Invoke(new AmtSystem.ObjectStateHandler(SystemStateChangedHandlerEx), new object[] { system });
        }

        private void SystemStateChangedHandlerEx(AmtSystem computer)
        {
            TreeNode systemNode = Utils.GetNodeWithTagRec(rootNode, computer);
            if (systemNode == null) return;

            mainTreeView.BeginUpdate();

            int icon = GetIconIndex(computer);
            systemNode.ImageIndex = icon;
            systemNode.SelectedImageIndex = icon;

            if (computer.State == AmtSystemObjState.Connected && computer.Tag == null)
            {
                if (systemNode.Nodes.Count == 0)
                {
                    TreeNode securityProfile = new TreeNode(Properties.Resources.TreeNode_Sec_Prof, 4, 5);
                    securityProfile.Tag = computer.SecurityAdmin.GetSecurityProfile();
                    systemNode.Nodes.Add(securityProfile);

                    if (computer.SecurityAdmin.ZeroTouchSupport())
                    {
                        TreeNode zeroTouchNode = new TreeNode(Properties.Resources.Zero_Touch_Setting, 4, 5);
                        systemNode.Nodes.Add(zeroTouchNode);
                    }
                }

                if (computer.Username.CompareTo(computer.SecurityAdmin.GetAdminAclEntry()) != 0)
                {
                    MessageBox.Show(this,
                        Properties.Resources.MsgBox_Admin_Limit, 
                        Properties.Resources.MsgBox_Admin_Limit_Title,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    computer.Disconnect();
                }

                if (computer.AmtRev == 1)
                {
                    MessageBox.Show(this,
                        Properties.Resources.Amt1NotSupported,
                        Properties.Resources.MessageBoxTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    computer.Disconnect();
                }
            }
            else
            {
                systemNode.Nodes.Clear();
            }

            mainTreeView.EndUpdate();

            if (mainTreeView.SelectedNode == systemNode) mainTreeView_AfterSelect(this, null);
        }

        private void connectButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;

                if (computer.State == AmtSystemObjState.Connected || computer.State == AmtSystemObjState.Connecting) { computer.Disconnect(); }
                else
                {
                    // First, update the password & save it if needed.
                    if (computer.Password != passwordTextBox.Text)
                    {
                        computer.Password = passwordTextBox.Text;
                        if (computer.RememberPassword == true) this.SaveSettings();
                    }
                    // Then, connect to the computer.
                    computer.WsManSupport = Properties.Settings.Default.WsmanSupport;
                    computer.Connect();
                }

                if (mainTreeView.SelectedNode == systemNode) mainTreeView_AfterSelect(this, null);
            }
        }

        private void webLinkLabel_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                try
                {
                    System.Diagnostics.Process.Start(computer.WebURL);
                }
                catch (Exception) { }
            }
        }

        private void mainTreeView_MouseDown(object sender, MouseEventArgs e)
        {
            TreeNode node = mainTreeView.GetNodeAt(e.X, e.Y);
            if (node == null)
            {
                treeContextMenuStrip.Items.Clear();
                return;
            }

            mainTreeView.SelectedNode = node;
            mainTreeView_ArrangeContextMenu();

            treeContextMenuStrip.Items.Clear();
            ArrayList temp = new ArrayList();
            temp.AddRange(editToolStripMenuItem.DropDownItems);
            foreach (object o in temp)
            {
                if (o.GetType() == typeof(ToolStripSeparator))
                {
                    ToolStripSeparator mi = (ToolStripSeparator)o;
                    if (mi.Tag != null && (bool)mi.Tag == true)
                    {
                        treeContextMenuStrip.Items.Add(new ToolStripSeparator());
                    }
                }
                else if (o.GetType() == typeof(ToolStripMenuItem))
                {
                    ToolStripMenuItem mi = (ToolStripMenuItem)o;
                    ToolStripMenuItem m = new ToolStripMenuItem();
                    m.Text = mi.Text;
                    if (mi.Tag != null) m.Visible = (bool)mi.Tag;
                    m.Enabled = mi.Enabled;
                    m.Checked = mi.Checked;
                    m.Tag = mi;
                    m.Click += new EventHandler(genericPassthruMenuItem_Click);

                    foreach (ToolStripMenuItem mi2 in mi.DropDownItems)
                    {
                        ToolStripMenuItem m2 = new ToolStripMenuItem();
                        m2.Text = mi2.Text;
                        m2.ToolTipText = mi2.ToolTipText;
                        m2.Enabled = mi2.Enabled;
                        if (mi2.Tag != null) m2.Visible = (bool)mi2.Tag;
                        m2.Checked = mi2.Checked;
                        m2.Tag = mi2;
                        m2.Click += new EventHandler(genericPassthruMenuItem_Click);
                        m.DropDownItems.Add(m2);
                    }

                    treeContextMenuStrip.Items.Add(m);
                }
            }
        }

        private void genericPassthruMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem m1 = (ToolStripMenuItem)sender;
            ToolStripMenuItem m2 = (ToolStripMenuItem)m1.Tag;
            m2.PerformClick();
        }

        private void mainTreeView_ArrangeContextMenu()
        {
            TreeNode node = (TreeNode)mainTreeView.SelectedNode;

            if (node == null)
            {
                foreach (ToolStripMenuItem mi in editToolStripMenuItem.DropDownItems)
                {
                    mi.Visible = false;
                    mi.Tag = false;
                }
                return;
            }

            addAMTSystemToolStripMenuItem.Tag = (node == rootNode);
            removeComputerToolStripMenuItem.Tag = (node.Tag != null && node.Tag.GetType() == typeof(AmtSystem));
            openWebInterfaceToolStripMenuItem.Tag = (node.Tag != null && node.Tag.GetType() == typeof(AmtSystem));
            connectToolStripMenuItem.Tag = (node.Tag != null && (node.Tag.GetType() == typeof(AmtSystem) && (((AmtSystem)node.Tag).State == AmtSystemObjState.Disconnected)));
            disconnectToolStripMenuItem.Tag = (node.Tag != null && (node.Tag.GetType() == typeof(AmtSystem) && (((AmtSystem)node.Tag).State == AmtSystemObjState.Connected)));
            addSecurityTemplateToolStripMenuItem.Tag = (node == securityTemplatesNode);
            removeProfileToolStripMenuItem.Tag = (node.Tag != null && node.Tag.GetType() == typeof(SecurityTemplate));
            renameProfileToolStripMenuItem.Tag = (node.Tag != null && node.Tag.GetType() == typeof(SecurityTemplate));
            setProfileToolStripMenuItem.Tag = (node.Tag != null && node.Tag.GetType() == typeof(SecurityTemplate));
            refreshToolStripMenuItem.Tag = (node.Tag != null && node.Tag.GetType() == typeof(AmtSecuritySettings));

            // Copy Menu
            bool copyMenu = false;
            copyMenu |= (node == rootNode);
            copyMenu |= (node.Tag != null && node.Tag.GetType() == typeof(AmtSystem));
            copyToolStripMenuItem.Tag = copyMenu;

            // Paste Menu
            pasteToolStripMenuItem.Tag = false;
            if (node == rootNode)
            {
                pasteToolStripMenuItem.Tag = true;
                pasteToolStripMenuItem.Enabled =
                    (Utils.UnSerializeFromClipboard(typeof(AmtSerialSystemSettings[])) != null) ||
                    (Utils.UnSerializeFromClipboard(typeof(AmtSerialSystemSettings)) != null);
            }

            // Seperator
            toolStripMenuItem6.Tag = ((bool)copyToolStripMenuItem.Tag || (bool)pasteToolStripMenuItem.Tag);

            foreach (object o in editToolStripMenuItem.DropDownItems)
            {
                if (o.GetType() == typeof(ToolStripMenuItem))
                {
                    ToolStripMenuItem mi = (ToolStripMenuItem)o;
                    mi.Visible = (bool)mi.Tag;
                }
                else if (o.GetType() == typeof(ToolStripSeparator))
                {
                    ToolStripSeparator mi = (ToolStripSeparator)o;
                    mi.Visible = (bool)mi.Tag;
                }
            }
        }

        private void removeSystemToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                computer.Disconnect();
                computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
                computer.OnFetchCompleted -= new AmtSystem.FetchHandler(FetchHandler);
                mainTreeView.Nodes.Remove(mainTreeView.SelectedNode);
                SaveSettings();
            }
        }

        private void showManageabilityStackDebugInfoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AmtDebugForm.ShowForm();
        }

        private void generateKeyButton_Click(object sender, EventArgs e)
        {
            KeyGeneratorForm form = new KeyGeneratorForm(this, null);
            if (form.ShowDialog(this) == DialogResult.OK)
            {
                mainTreeView_AfterSelect(this, null);
            }
        }

        private void hideKeysCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            mainTreeView_AfterSelect(this, null);
            Application.UserAppDataRegistry.SetValue("HideProvisioningKeys", hideKeysCheckBox.Checked.ToString());
        }

        private void provisioningKeysListView_DoubleClick(object sender, EventArgs e)
        {
            if (provisioningKeysListView.SelectedItems.Count != 1) return;
            AmtProvisioningKey key = (AmtProvisioningKey)provisioningKeysListView.SelectedItems[0].Tag;
            new KeyGeneratorForm(this, key).ShowDialog(this);
            mainTreeView_AfterSelect(this, null);
        }

        private void provisioningKeysListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            exportKeysButton.Enabled = deleteKeysButton.Enabled = (provisioningKeysListView.SelectedItems.Count > 0);
            openKeyButton.Enabled = (provisioningKeysListView.SelectedItems.Count == 1);
        }

        private void deleteKeysButton_Click(object sender, EventArgs e)
        {
            if (provisioningKeysListView.SelectedItems.Count == 0) return;

            string s = "";
            if (provisioningKeysListView.SelectedItems.Count > 1) s = Properties.Resources.Label_Plural;

            if (MessageBox.Show(this,
                String.Format(Properties.Resources.MsgBox_Del_Key,provisioningKeysListView.SelectedItems.Count.ToString(),s),
                Properties.Resources.MsgBox_Del_Key_OneTouch, 
                MessageBoxButtons.OKCancel, MessageBoxIcon.Question) == DialogResult.OK)
            {
                foreach (ListViewItem l in provisioningKeysListView.SelectedItems)
                {
                    AmtProvisioningKey key = (AmtProvisioningKey)l.Tag;
                    provisioningKeyStore.Keys.Remove(key.Pid);
                }
                provisioningKeyStore.Flush();
                mainTreeView_AfterSelect(this, null);
            }
        }


        public void HelloServerStateChangeSink(AmtHelloServer sender)
        {
            // NOP
        }

        private delegate void HelloServerSinkhandler(AmtHelloServer sender, AmtHelloServer.AmtHello msg);
        private void HelloServerSink(AmtHelloServer sender, AmtHelloServer.AmtHello msg)
        {
            // Pile this request up on the request stack.
            lock (HelloRequestPendingList)
            {
                HelloRequestPendingList.Add(msg);
                if (HelloRequestHandler == null)
                {
                    HelloRequestHandler = new Thread(new ThreadStart(HelloServerSinkEx));
                    HelloRequestHandler.Start();
                }
            }
        }

        public void HelloServerSinkEx()
        {
            AmtHelloServer.AmtHello msg;

            while (true)
            {
                // Lets not rush anything... Works without, but Intel AMT is slow...
                Thread.Sleep(100);

                // Display a debug message, but don't lock doing it...
                if (HelloRequestPendingList.Count != 0) AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Event, "Getting hello message from pending list.");

                // Get a request from the request list.
                lock (HelloRequestPendingList)
                {
                    if (HelloRequestPendingList.Count == 0)
                    {
                        HelloRequestHandler = null;
                        return;
                    }
                    msg = (AmtHelloServer.AmtHello)HelloRequestPendingList[0];
                    HelloRequestPendingList.RemoveAt(0);
                }

                // Just in case this happens
                if (msg == null) continue;

                // Handle this request
                Directory.SetCurrentDirectory(Application.StartupPath);

                // Check that the time is set on this computer.
                if (DateTime.Now.Year < 2007)
                {
                    AddProvisioningServerLog(Properties.Resources.IncorrectTime);
                    continue;
                }

                HelloMsgList.Add(msg);
                if (msg.CertHash != null)
                {
                    // PKI Hello
                    helloMsgToolStripStatusLabel.Text = Properties.Resources.Server_Rec_Hello + msg.RemoteEndPoint.ToString();
                }
                else
                {
                    // PSK Hello
                    helloMsgToolStripStatusLabel.Text = Properties.Resources.Server_Rec_Hello + msg.RemoteEndPoint.ToString() + Properties.Resources.SlashSeperator + msg.Pid;
                }
                if (provisioningServerForm != null) provisioningServerForm.UpdateInfo();
                Application.UserAppDataRegistry.SetValue("HelloMessages", ManageabilityStack.Utils.SerializeBinary(HelloMsgList));

                if (msg.Pid != null)
                {
                    // One touch provisioning
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Event, "Performing one-touch setup.");

                    // Push a generic configuration into the machine
                    AmtProvisioningKey key = (AmtProvisioningKey)provisioningKeyStore.Keys[msg.Pid];
                    if (key == null)
                    {
                        AddProvisioningServerLog(msg.RemoteEndPoint.Address.ToString() + Properties.Resources.Log_Unex_Hello);
                        continue;
                    }
                    AddProvisioningServerLog(msg.RemoteEndPoint.Address.ToString() + Properties.Resources.Log_Get_Hello + key.Pid);

                    // Set this key as being used for a hello & store it
                    key.HelloRequests++;
                    provisioningKeyStore.Keys[key.Pid] = key;
                    provisioningKeyStore.Flush();

                    // Get the security template
                    SecurityTemplate secTemplate = GetSecurityTemplates(key.SecurityTemplate);
                    if (secTemplate == null)
                    {
                        AddProvisioningServerLog(msg.RemoteEndPoint.Address.ToString() + Properties.Resources.InvalidProfileForPid);
                        continue;
                    }

                    AddProvisioningServerLog(msg.RemoteEndPoint.Address.ToString() + Properties.Resources.Log_Attemp_1_Touch);

                    // Try to get the Reverse DNS name
                    bool resolvedName = false;
                    string computerName = "Setup-" + key.Pid.Replace("-", "");
                    string domainName = "dtk.amt.intel.com";
                    IPHostEntry hostEntry = null;
                    try
                    {
                        hostEntry = System.Net.Dns.GetHostEntry(msg.RemoteEndPoint.Address);
                    }
                    catch (Exception) { }
                    if (hostEntry != null)
                    {
                        // Check that hostEntry is not just an IP address
                        IPAddress addr = IPAddress.None;
                        try
                        {
                            addr = IPAddress.Parse(hostEntry.HostName);
                        }
                        catch (Exception) { }

                        if (addr == IPAddress.None)
                        {
                            int i = hostEntry.HostName.IndexOf(".");
                            if (i > 0)
                            {
                                resolvedName = true;
                                computerName = hostEntry.HostName.Substring(0, i);
                                domainName = hostEntry.HostName.Substring(i + 1);
                            }
                        }
                    }
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Event, "Setup computer name: " + computerName + "." + domainName);

                    // Push the configuration
                    X509Certificate2 tlsRootCert = null;
                    if (secTemplate.TLSCertificateIssuerTP != null) tlsRootCert = GetCertFromThumpprint(secTemplate.TLSCertificateIssuerTP);
                    int returncode = OneTouchPush.PskPushConfig(Application.StartupPath, key, msg.RemoteEndPoint.Address, computerName, domainName, secTemplate, tlsRootCert);
                    if (returncode != 0)
                    {
                        if (returncode == 2)
                        {
                            AddProvisioningServerLog(msg.RemoteEndPoint.Address.ToString() + Properties.Resources.IntelAmtReject);
                        }
                        else
                        {
                            AddProvisioningServerLog(msg.RemoteEndPoint.Address.ToString() + Properties.Resources.Round_1_Err_Code + returncode.ToString());
                        }
                        continue;
                    }

                    // Mark key as used up & store
                    key.State = AmtProvisioningKey.AmtProvisioningKeyState.Used;
                    key.CompletedRequests++;
                    provisioningKeyStore.Keys[key.Pid] = key;
                    provisioningKeyStore.Flush();
                    //if (mainTreeView.SelectedNode == provisioningKeysNode) SafeRefreshUi();
                    SafeRefreshUi();
                    AddProvisioningServerLog(msg.RemoteEndPoint.Address.ToString() + Properties.Resources.Round_1_Successful);

                    // Add the system to the list
                    if (resolvedName == false)
                    {
                        computerName = msg.RemoteEndPoint.Address.ToString();
                    }
                    else
                    {
                        computerName = computerName + "." + domainName;
                    }
                    TreeNode systemNode = new TreeNode(computerName + Properties.Resources.SlashSeperator + "admin", 12, 12);
                    AmtSystem computer = new AmtSystem(computerName, 16992, "admin", key.AdminPassword, true, true); ;
                    computer.SetDnsEntryCache(hostEntry);
                    computer.IPAddress = msg.RemoteEndPoint.Address;
                    SafeAddComputer(computer);
                    SaveSettings();

                    // Add the templates to set
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Event, "Adding templates.");
                    ArrayList setList = new ArrayList();
                    if (secTemplate != null)
                    {
                        setList.Add(secTemplate);
                        if (secTemplate.AdminAclPassword != null) setList.Add(new SecurityTemplateAdminChange(secTemplate.AdminAclEntry, secTemplate.AdminAclPassword));
                    }

                    Thread.Sleep(500); // This is a worker thread, we can do this.

                    // Connect to the computer
                    if (setList.Count > 0) computer.Tag = setList;
                    computer.WsManSupport = Properties.Settings.Default.WsmanSupport;
                    computer.Connect();
                }

                if (msg.CertHash != null)
                {
                    // Remote Configuration (RCG) provisioning
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Event, "Performing remote configuration setup.");

                    // Check if the ZTC password is acceptable
                    if (ManageabilityStack.Utils.CheckPassPhrase(adminPasswordMaskedTextBox.Text) == false)
                    {
                        AddProvisioningServerLog(Properties.Resources.IncorrectSetPassword);
                        continue;
                    }

                    // First, lets find a certificate we can use to authenticate
                    X509Chain authCertChain = null;
                    foreach (byte[] hash in msg.CertHash)
                    {
                        string matchHash = Utils.BytesToHex(hash).ToUpper();
                        foreach (X509Certificate2 cert in CertificateStore.Certificates)
                        {
                            if (CertificateOperations.IsAmtRemoteConfig(cert) && ManageabilityStack.CertificateOperations.IsKeyExportable(cert))
                            {
                                // Try to build a chain and check.
                                X509Chain chain = new X509Chain();
                                chain.ChainPolicy.RevocationMode = X509RevocationMode.NoCheck;
                                if (chain.Build(cert) == true)
                                {
                                    foreach (X509ChainElement e in chain.ChainElements)
                                    {
                                        if (string.Compare(e.Certificate.Thumbprint, matchHash, true) == 0)
                                        {
                                            // Found it
                                            authCertChain = chain;
                                            break;
                                        }
                                    }
                                    if (authCertChain != null) break;
                                }
                            }
                            if (authCertChain != null) break;
                        }
                    }

                    if (authCertChain == null)
                    {
                        AddProvisioningServerLog(msg.RemoteEndPoint.Address.ToString() + Properties.Resources.Hash_Mismatch);
                        continue;
                    }
                    else
                    {
                        AddProvisioningServerLog(msg.RemoteEndPoint.Address.ToString() + Properties.Resources.Log_Attemp_0_Touch);
                    }

                    // Try to get the Reverse DNS name
                    bool resolvedName = false;
                    string computerName = "Setup-PKI";
                    string domainName = "dtk.amt.intel.com";
                    IPHostEntry hostEntry = null;
                    try
                    {
                        hostEntry = System.Net.Dns.GetHostEntry(msg.RemoteEndPoint.Address);
                    }
                    catch (Exception) { }
                    if (hostEntry != null)
                    {
                        // Check that hostEntry is not just an IP address
                        IPAddress addr = IPAddress.None;
                        try
                        {
                            addr = IPAddress.Parse(hostEntry.HostName);
                        }
                        catch (Exception) { }

                        if (addr == IPAddress.None)
                        {
                            int i = hostEntry.HostName.IndexOf(".");
                            if (i > 0)
                            {
                                resolvedName = true;
                                computerName = hostEntry.HostName.Substring(0, i);
                                domainName = hostEntry.HostName.Substring(i + 1);
                            }
                        }
                    }
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Event, "Setup computer name: " + computerName + "." + domainName);

                    // Add the templates to set
                    SecurityTemplate secTemplate = GetSecurityTemplates(ztcProfileLabel.Text);
                    if (secTemplate == null) AddProvisioningServerLog(msg.RemoteEndPoint.Address.ToString() + ": " + Properties.Resources.LogRemoteConfigError);

                    // Push Configuration
                    X509Certificate2 tlsRootCert = null;
                    if (secTemplate.TLSCertificateIssuerTP != null) tlsRootCert = GetCertFromThumpprint(secTemplate.TLSCertificateIssuerTP);
                    int returncode = OneTouchPush.PkiPushConfig(Application.StartupPath, computerName, domainName, msg.RemoteEndPoint.Address, adminPasswordMaskedTextBox.Text, authCertChain, secTemplate, tlsRootCert);
                    if (returncode != 0)
                    {
                        if (returncode == 2)
                        {
                            AddProvisioningServerLog(msg.RemoteEndPoint.Address.ToString() + Properties.Resources.IntelAmtReject);
                        }
                        else
                        {
                            AddProvisioningServerLog(msg.RemoteEndPoint.Address.ToString() + Properties.Resources.Round_1_PKI_Err_Code + returncode.ToString());
                        }
                        continue;
                    }
                    AddProvisioningServerLog(msg.RemoteEndPoint.Address.ToString() + Properties.Resources.Round_1_PKI_Successful);

                    // Add the system to the list
                    if (resolvedName == false)
                    {
                        computerName = msg.RemoteEndPoint.Address.ToString();
                    }
                    else
                    {
                        computerName = computerName + "." + domainName;
                    }
                    TreeNode systemNode = new TreeNode(computerName + Properties.Resources.SlashSeperator + " admin", 12, 12);
                    AmtSystem computer = new AmtSystem(computerName, 16993, "admin", adminPasswordMaskedTextBox.Text, true, true); ;
                    computer.SetDnsEntryCache(hostEntry);
                    computer.IPAddress = msg.RemoteEndPoint.Address;
                    computer.UseTls = false;
                    SafeAddComputer(computer);
                    SaveSettings();

                    // Add the templates to set
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Event, "Adding templates.");
                    ArrayList setList = new ArrayList();
                    if (secTemplate != null)
                    {
                        setList.Add(secTemplate);
                        if (secTemplate.AdminAclPassword != null) setList.Add(new SecurityTemplateAdminChange(secTemplate.AdminAclEntry, secTemplate.AdminAclPassword));
                    }

                    Thread.Sleep(500); // This is a worker thread, we can do this.

                    // Connect to the computer
                    if (setList.Count > 0) computer.Tag = setList;
                    computer.WsManSupport = Properties.Settings.Default.WsmanSupport;
                    computer.Connect();
                }
            }
        }

        private delegate void SafeRefreshUiHandler(object sender, TreeViewEventArgs args);
        private void SafeRefreshUi()
        {
            object[] p = new object[2];
            p[0] = this;
            p[1] = null;
            this.Invoke(new SafeRefreshUiHandler(mainTreeView_AfterSelect), p);
        }

        private void provisioningServerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            provisioningServerForm = new HelloViewerForm(this);
            provisioningServerForm.Show(this);
        }

        private void createCertButton_Click(object sender, EventArgs e)
        {
            ManageabilityControls.CreateCertificateForm form = new ManageabilityControls.CreateCertificateForm(null);
            if (form.ShowDialog(this) == DialogResult.OK)
            {
                if (form.Certificate != null)
                {
                    CertificateStore.Add(form.Certificate);

                    if (form.TrustedRoot() == true)
                    {
                        // Add this certificate to trusted root store.
                        try
                        {
                            CertificateRootStore.Add(form.Certificate);
                        }
                        catch (Exception) { }
                    }

                    mainTreeView_AfterSelect(this, null);
                }
            }
        }

        private void createSignedCertButton_Click(object sender, EventArgs e)
        {
            if (certificatesListView.SelectedItems.Count != 1) return;
            X509Certificate2 rootcert = (X509Certificate2)certificatesListView.SelectedItems[0].Tag;
            if (rootcert.HasPrivateKey == false || !ManageabilityStack.CertificateOperations.IsKeyExportable(rootcert)) return;

            ManageabilityControls.CreateCertificateForm form = new ManageabilityControls.CreateCertificateForm(rootcert);
            if (form.ShowDialog(this) == DialogResult.OK)
            {
                if (form.Certificate != null)
                {
                    CertificateStore.Add(form.Certificate);
                    mainTreeView_AfterSelect(this, null);
                }
            }
        }

        private void certificatesListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            viewCertButton.Enabled = deleteCertButton.Enabled = exportCertButton.Enabled = (certificatesListView.SelectedItems.Count == 1);

            bool rootsign = false;
            if (certificatesListView.SelectedItems.Count == 1)
            {
                X509Certificate2 cert = (X509Certificate2)certificatesListView.SelectedItems[0].Tag;
                trustCertButton.Enabled = (ManageabilityStack.CertificateOperations.IsCaCertificate(cert) && ManageabilityStack.CertificateOperations.IsKeyExportable(cert));
                if (cert.HasPrivateKey == true && trustCertButton.Enabled == true) rootsign = true;
            }
            else
            {
                trustCertButton.Enabled = false;
            }
            createSignedCertButton.Enabled = rootsign;
        }

        private void ImportCert(FileInfo fi)
        {
            X509Certificate2 cert = null;

            if (string.Compare(fi.Extension, ".pem", true) == 0)
            {
                cert = CertificateOperations.LoadPemFile(fi.FullName);
                if (cert == null)
                {
                    MessageBox.Show(this,
                        Properties.Resources.MsgBox_Import_Error,
                        Properties.Resources.MsgBox_Import_Cert,
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }
            else
            {
                try
                {
                    cert = new X509Certificate2(fi.FullName);
                }
                catch (CryptographicException) { }

                if (cert == null)
                {
                    CertificatePasswordForm f = new CertificatePasswordForm();
                    if (f.ShowDialog(this) == DialogResult.OK)
                    {
                        try
                        {
                            cert = new X509Certificate2(fi.FullName, f.Password, X509KeyStorageFlags.Exportable | X509KeyStorageFlags.MachineKeySet | X509KeyStorageFlags.PersistKeySet);
                        }
                        catch (CryptographicException)
                        {
                            MessageBox.Show(this,
                                Properties.Resources.MsgBox_Import_Error,
                                Properties.Resources.MsgBox_Import_Cert,
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                            return;
                        }
                    }
                    else
                    {
                        return;
                    }
                }
            }

            CertificateStore.Add(cert);
            certificatesListView_SelectedIndexChanged(this, null);
            mainTreeView_AfterSelect(this, null);
        }

        private void importCertButton_Click(object sender, EventArgs e)
        {
            if (importCertOpenFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                ImportCert(new FileInfo(importCertOpenFileDialog.FileName));
            }
        }

        private void deleteCertButton_Click(object sender, EventArgs e)
        {
            if (certificatesListView.SelectedItems.Count != 1) return;

            X509Certificate2 cert = (X509Certificate2)certificatesListView.SelectedItems[0].Tag;

            if (IsTrusted(cert))
            {
                try
                {
                    CertificateRootStore.Remove(cert);
                }
                catch (Exception) { }
                if (IsTrusted(cert) == false)
                {
                    CertificateStore.Remove(cert);
                    certificatesListView.SelectedItems.Clear();
                    certificatesListView_SelectedIndexChanged(this, null);
                    mainTreeView_AfterSelect(this, null);
                }
            }
            else if (MessageBox.Show(this, 
                Properties.Resources.MsgBox_Del_Cert,
                Properties.Resources.MsgBox_Del_Cert_Con,
                MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                CertificateStore.Remove(cert);
                certificatesListView.SelectedItems.Clear();
                certificatesListView_SelectedIndexChanged(this, null);
                mainTreeView_AfterSelect(this, null);
            }
        }

        private void exportCertButton_Click(object sender, EventArgs e)
        {
            if (certificatesListView.SelectedItems.Count != 1) return;

            X509Certificate2 cert = (X509Certificate2)certificatesListView.SelectedItems[0].Tag;

            if (cert.HasPrivateKey == true)
            {
                exportCertFileDialog.DefaultExt = "cer";
                exportCertFileDialog.Filter = Properties.Resources.FD_Export_Cert_PK;
            }
            else
            {
                exportCertFileDialog.DefaultExt = "cer";
                exportCertFileDialog.Filter = Properties.Resources.FD_Export_Cert;
            }

            exportCertFileDialog.FileName = Properties.Resources.FD_Export_Cert_Title;
            if (exportCertFileDialog.ShowDialog(this) != DialogResult.OK) return;

            FileInfo fi = new FileInfo(exportCertFileDialog.FileName);
            
            if (string.Compare(fi.Extension, ".cer", true) == 0)
            {
                byte[] bytes = cert.Export(X509ContentType.Cert);
                FileStream fs = new FileStream(exportCertFileDialog.FileName, FileMode.Create);
                fs.Write(bytes, 0, bytes.Length);
                fs.Close();
            }

            if (string.Compare(fi.Extension, ".pfx", true) == 0 || string.Compare(fi.Extension, ".p12", true) == 0)
            {
                CertificatePasswordForm f = new CertificatePasswordForm();
                if (f.ShowDialog(this) == DialogResult.OK)
                {
                    byte[] bytes = cert.Export(X509ContentType.Pkcs12, f.Password);
                    FileStream fs = new FileStream(exportCertFileDialog.FileName, FileMode.Create);
                    fs.Write(bytes, 0, bytes.Length);
                    fs.Close();
                }
            }

            if (string.Compare(fi.Extension, ".pem", true) == 0)
            {
                if (cert.HasPrivateKey == true)
                {
                    ManageabilityStack.CertificateOperations.CertAndKeyToPemFile(cert, exportCertFileDialog.FileName, null);
                }
                else
                {
                    ManageabilityStack.CertificateOperations.CertToPemFile(cert, exportCertFileDialog.FileName);
                }
            }

            if (string.Compare(fi.Extension, ".der", true) == 0)
            {
                ManageabilityStack.CertificateOperations.CertToDerFile(cert, exportCertFileDialog.FileName);
            }
        }

        private void viewCertButton_Click(object sender, EventArgs e)
        {
            if (certificatesListView.SelectedItems.Count != 1) return;
            X509Certificate2 cert = (X509Certificate2)certificatesListView.SelectedItems[0].Tag;
            X509Certificate2UI.DisplayCertificate(cert, Handle);
        }

        private void trustCertButton_Click(object sender, EventArgs e)
        {
            if (certificatesListView.SelectedItems.Count != 1) return;
            X509Certificate2 cert = (X509Certificate2)certificatesListView.SelectedItems[0].Tag;

            if (IsTrusted(cert))
            {
                try
                {
                    CertificateRootStore.Remove(cert);
                }
                catch (Exception) { }
            }
            else
            {
                try
                {
                    CertificateRootStore.Add(cert);
                }
                catch (Exception) { }
            }
            certificatesListView_SelectedIndexChanged(this, null);
            mainTreeView_AfterSelect(this, null);
        }

        private bool IsTrusted(X509Certificate2 cert)
        {
            foreach (X509Certificate2 cert2 in CertificateRootStore.Certificates)
            {
                if (cert2.Equals(cert)) return true;
            }
            return false;
        }

        private void provServerButton_Click(object sender, EventArgs e)
        {
            if (helloServer == null || helloServer.IsListening == false)
            {
                HelloServerState(true);
            }
            else
            {
                HelloServerState(false);
            }
            if (mainTreeView.SelectedNode == provisioningServerNode) mainTreeView_AfterSelect(this, null);
        }

        private delegate void StringHandler(string s);
        public void AddProvisioningServerLog(string message)
        {
            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Event, message);

            object[] p = new object[1];
            p[0] = message;
            this.Invoke(new StringHandler(AddProvisioningServerLogEx),p);
        }

        private void AddProvisioningServerLogEx(string message)
        {
            DateTime n = DateTime.Now;
            ListViewItem l = new ListViewItem(new string[] { n.ToString(), message });
            l.Tag = n;
            provisioningServerListView.Items.Insert(0,l);
            while (provisioningServerListView.Items.Count > 100) provisioningServerListView.Items.RemoveAt(100);
            clearServerLogButton.Enabled = (provisioningServerListView.Items.Count > 0);
        }

        private void clearServerLogButton_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(this, 
                Properties.Resources.MsgBox_Clear_PS_Log, 
                Properties.Resources.MsgBox_Prov_Serv, 
                MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                provisioningServerListView.Items.Clear();
                if (mainTreeView.SelectedNode == provisioningServerNode) mainTreeView_AfterSelect(this, null);
            }
        }

        private void addSecurityTemplateButton_Click(object sender, EventArgs e)
        {
            AskProfileNameForm f = new AskProfileNameForm();
            if (f.ShowDialog(this) == DialogResult.OK)
            {
                TreeNode n = new TreeNode(f.TemplateName, 3, 3);
                n.Tag = new SecurityTemplate(f.TemplateName);
                securityTemplatesNode.Nodes.Add(n);
                securityTemplatesNode.Expand();
                SaveProfiles();

                if (this.ztcProfileLabel.Text.Equals(RemoteConfigNoProfileLabel) == true)
                {
                    this.ztcProfileLabel.Text = f.TemplateName;
                    Application.UserAppDataRegistry.SetValue("ZtcSecurityProfile", ztcProfileLabel.Text);
                    if (mainTreeView.SelectedNode == zeroTouchNode) mainTreeView_AfterSelect(this, null);
                }
            }
        }

        private void SaveProfiles()
        {
            ArrayList securityTemplates = new ArrayList();
            foreach (TreeNode n in securityTemplatesNode.Nodes)
            {
                securityTemplates.Add((SecurityTemplate)n.Tag);
            }
            Application.UserAppDataRegistry.SetValue("SecurityTemplates", Utils.SerializeBinary(securityTemplates));
        }

        private void LoadTemplates()
        {
            try
            {
                byte[] buf = (byte[])Application.UserAppDataRegistry.GetValue("SecurityTemplates", null);
                if (buf != null)
                {
                    ArrayList securityTemplates = (ArrayList)Utils.UnserializeBinary(buf);
                    securityTemplatesNode.Nodes.Clear();
                    if (securityTemplates != null)
                    {
                        foreach (SecurityTemplate st in securityTemplates)
                        {
                            TreeNode n = new TreeNode(st.Name, 3, 3);
                            n.Tag = st;
                            securityTemplatesNode.Nodes.Add(n);
                        }
                    }
                    securityTemplatesNode.Expand();
                }
            }
            catch (Exception) { }
        }

        private void removeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;

            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(SecurityTemplate))
            {
                SecurityTemplate securityTemplate = (SecurityTemplate)mainTreeView.SelectedNode.Tag;
                if (ztcProfileLabel.Text.Equals(securityTemplate.Name) == true)
                {
                    if (MessageBox.Show(this,
                        Properties.Resources.ProfileUsedWarning,
                        Properties.Resources.MsgBox_Remove_Temp_Con,
                        MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.Yes)
                    {
                        TreeNode n = mainTreeView.SelectedNode;
                        n.Parent.Nodes.Remove(n);
                        SaveProfiles();
                        ztcProfileLabel.Text = RemoteConfigNoProfileLabel;
                        if (mainTreeView.SelectedNode == zeroTouchNode) mainTreeView_AfterSelect(this, null);
                    }
                }
                else
                {
                    if (MessageBox.Show(this,
                        Properties.Resources.MsgBox_Remove_Temp,
                        Properties.Resources.MsgBox_Remove_Temp_Con,
                        MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                    {
                        TreeNode n = mainTreeView.SelectedNode;
                        n.Parent.Nodes.Remove(n);
                        SaveProfiles();
                    }
                }
            }
        }

        private void setTemplateToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(SecurityTemplate))
            {
                SecurityTemplate securityTemplate = (SecurityTemplate)mainTreeView.SelectedNode.Tag;

                SetProfileForm f = new SetProfileForm();
                f.SetSecurityTemplate = securityTemplate;
                ArrayList machineList = Utils.GetNodeTagsofTypeRec(rootNode, typeof(AmtSystem));
                f.Machines = machineList;
                f.ShowDialog(this);

                foreach (AmtSystem system in machineList)
                {
                    launchSetTemplate(system);
                }
            }
        }

        private void launchSetTemplate(AmtSystem system)
        {
            launchSetTemplateEx(system);
            //new Thread(new ParameterizedThreadStart(launchSetTemplateEx)).Start(system);
        }

        private void launchSetTemplateEx(object obj)
        {
            AmtSystem system = (AmtSystem)obj;

            try
            {
                if (system.Tag == null) return;
                system.AutoFetchCache = false;

                if (system.State == AmtSystemObjState.Disconnected && system.Tag != null)
                {
                    system.WsManSupport = Properties.Settings.Default.WsmanSupport;
                    system.Connect();
                    return;
                }

                while (system.State == AmtSystemObjState.Connected && system.Tag != null)
                {
                    ArrayList setList = (ArrayList)system.Tag;
                    SecurityTemplateChange o = (SecurityTemplateChange)setList[0];
                    setList.RemoveAt(0);
                    if (setList.Count == 0) system.Tag = null;
                    AmtCallStatus r = o.PerformAction(this, system);

                    if (o.GetType() == typeof(SecurityTemplateReset) && r == AmtCallStatus.SUCCESS)
                    {
                        // This system must be removed
                        rootNode.Nodes.Remove(Utils.GetNodeWithTagRec(rootNode, system));
                        return;
                    }
                }

                system.Disconnect();
                HoldAndConnect(system);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        private void HoldAndConnect(AmtSystem system)
        {
            new Thread(new ParameterizedThreadStart(HoldAndConnectEx)).Start(system);
        }

        private void HoldAndConnectEx(object obj)
        {
            AmtSystem system = (AmtSystem)obj;
            Thread.Sleep(1000);
            system.WsManSupport = Properties.Settings.Default.WsmanSupport;
            system.Connect();
        }

        private void trustedFqdnEditButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(SecurityTemplate))
            {
                SecurityTemplate securityTemplate = (SecurityTemplate)mainTreeView.SelectedNode.Tag;
                if (new FqdnEditForm(securityTemplate).ShowDialog(this) == DialogResult.OK) SaveProfiles();
                mainTreeView_AfterSelect(this, null);
            }
        }

        private void refreshToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSecuritySettings))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
                computer.SecurityAdmin.ClearCache();
                mainTreeView_AfterSelect(this, null);
            }
        }

        private void enabledInterfacesEditButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(SecurityTemplate))
            {
                SecurityTemplate securityTemplate = (SecurityTemplate)mainTreeView.SelectedNode.Tag;
                if (new InterfacesEditForm(securityTemplate).ShowDialog(this) == DialogResult.OK) SaveProfiles();
                mainTreeView_AfterSelect(this, null);
            }
        }

        private void localTlsAuthEditButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(SecurityTemplate))
            {
                SecurityTemplate securityTemplate = (SecurityTemplate)mainTreeView.SelectedNode.Tag;
                if (new TlsAuthForm(securityTemplate).ShowDialog(this) == DialogResult.OK) SaveProfiles();
                mainTreeView_AfterSelect(this, null);
            }
        }

        private void certEditButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(SecurityTemplate))
            {
                SecurityTemplate securityTemplate = (SecurityTemplate)mainTreeView.SelectedNode.Tag;
                if (new CertificateEditForm(securityTemplate).ShowDialog(this) == DialogResult.OK) SaveProfiles();
                mainTreeView_AfterSelect(this, null);
            }
        }

        public X509Certificate2 GetCertFromThumpprint(string thumbprint)
        {
            foreach (X509Certificate2 cert in CertificateStore.Certificates)
            {
                if (string.Compare(cert.Thumbprint, thumbprint, true) == 0) return cert;
            }
            return null;
        }

        private void secProfileViewCertButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode.Tag == null || mainTreeView.SelectedNode.Tag.GetType() != typeof(AmtSecuritySettings)) return;
            AmtSecuritySettings profile = (AmtSecuritySettings)mainTreeView.SelectedNode.Tag;
            new CertificateViewerForm(profile.TLSCertificate).ShowDialog(this);
        }

        private void secProfileViewTrustedCertButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode.Tag == null || mainTreeView.SelectedNode.Tag.GetType() != typeof(AmtSecuritySettings)) return;
            AmtSecuritySettings profile = (AmtSecuritySettings)mainTreeView.SelectedNode.Tag;
            new CertificateViewerForm(profile.TrustedRootCertificates).ShowDialog(this);
        }

        private bool RemoteCertificateValidationCallbackSink(object sender, X509Certificate certificate, X509Chain chain, System.Net.Security.SslPolicyErrors sslPolicyErrors)
        {
            if (sender.GetType() == typeof(System.Net.HttpWebRequest))
            {
                System.Net.HttpWebRequest r = (System.Net.HttpWebRequest)sender;
                X509Certificate2[] certArray = new X509Certificate2[chain.ChainElements.Count];
                for (int i = 0; i < chain.ChainElements.Count; i++)
                {
                    certArray[i] = chain.ChainElements[i].Certificate;
                }
                if (r.ConnectionGroupName != null) AmtSystem.AddTlsCertificateCache(r.ConnectionGroupName, certArray);
            }
            return true; // Accept All Certificates
        }

        public SecurityTemplate[] GetAllSecurityTemplates()
        {
            return (SecurityTemplate[])Utils.GetNodeTagsofTypeRec(securityTemplatesNode, typeof(SecurityTemplate)).ToArray(typeof(SecurityTemplate));
        }

        public SecurityTemplate GetSecurityTemplates(string name)
        {
            SecurityTemplate[] secTemplates = GetAllSecurityTemplates();
            foreach (SecurityTemplate secTemplate in secTemplates)
            {
                if (secTemplate.Name.CompareTo(name) == 0) return secTemplate;
            }
            return null;
        }

        private void adminAccessControlButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(SecurityTemplate))
            {
                SecurityTemplate securityTemplate = (SecurityTemplate)mainTreeView.SelectedNode.Tag;
                AdminAccessControlSetupForm form = new AdminAccessControlSetupForm(securityTemplate);
                if (form.ShowDialog(this) == DialogResult.OK)
                {
                    SaveProfiles();
                    mainTreeView.SelectedNode.Tag = form.securityTemplate;
                    mainTreeView_AfterSelect(this, null);
                }
            }
        }

        private void userAccessControlButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(SecurityTemplate))
            {
                SecurityTemplate securityTemplate = (SecurityTemplate)mainTreeView.SelectedNode.Tag;
                ManageabilityCommanderTool.AccessControlSetupForm form = new ManageabilityCommanderTool.AccessControlSetupForm(securityTemplate.UserAclEntries, null);
                form.AdminName = securityTemplate.AdminAclEntry;
                if (form.ShowDialog(this) == DialogResult.OK)
                {
                    securityTemplate.UserAclEntries = form.aclArray;
                    SaveProfiles();
                    mainTreeView.SelectedNode.Tag = securityTemplate;
                    mainTreeView_AfterSelect(this, null);
                }
            }
        }

        private void trustedRootsEditButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(SecurityTemplate))
            {
                SecurityTemplate securityTemplate = (SecurityTemplate)mainTreeView.SelectedNode.Tag;
                if (new TrustedCertificateEditForm(securityTemplate).ShowDialog(this) == DialogResult.OK) SaveProfiles();
                mainTreeView_AfterSelect(this, null);
            }
        }

        private void editComputerNameButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                new ManageabilityCommanderTool.EditComputerNameForm(computer).ShowDialog(this);
                mainTreeView_AfterSelect(this, null);
            }
        }

        private void renameProfileButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(SecurityTemplate))
            {
                SecurityTemplate t = (SecurityTemplate)mainTreeView.SelectedNode.Tag;
                RenameProfileForm f = new RenameProfileForm();
                f.ProfileName = t.Name;
                if (f.ShowDialog(this) == DialogResult.OK)
                {
                    if (ztcProfileLabel.Text.Equals(t.Name) == true) ztcProfileLabel.Text = f.ProfileName;
                    t.Name = f.ProfileName;
                    mainTreeView.SelectedNode.Text = f.ProfileName;
                    SaveProfiles();
                    if (mainTreeView.SelectedNode == zeroTouchNode) mainTreeView_AfterSelect(this, null);
                }
            }
        }

        private void ztcCertHashListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (ztcCertHashListView.SelectedItems.Count == 0)
            {
                ztcHashToggleButton.Enabled = false;
                removeZtcHashButton.Enabled = false;
            }
            else
            {
                ztcHashToggleButton.Enabled = true;
                removeZtcHashButton.Enabled = true;
            }
        }

        private void ztcHashToggleButton_Click(object sender, EventArgs e)
        {
            if (ztcCertHashListView.SelectedItems.Count == 0) return;
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag == null && mainTreeView.SelectedNode.Text.CompareTo(Properties.Resources.Zero_Touch_Setting) == 0)
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
                AmtSecurityAdmin.CertHashEntry entry = (AmtSecurityAdmin.CertHashEntry)ztcCertHashListView.SelectedItems[0].Tag;
                AmtCallStatus r = computer.SecurityAdmin.EnableCertificateHashEntry(entry, !entry.Entry.Active);
                if (r != AmtCallStatus.SUCCESS)
                {
                    MessageBox.Show(this, 
                        Properties.Resources.MsgBox_ZT_Hash_Err + r.ToString(), 
                        Properties.Resources.MsgBox_ZT_Hash_Err_Title,
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                mainTreeView_AfterSelect(this, null);
            }
        }

        private void addZtcHashButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag == null && mainTreeView.SelectedNode.Text.CompareTo(Properties.Resources.Zero_Touch_Setting) == 0)
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
                if (new AddCertificateHashForm(computer).ShowDialog(this) == DialogResult.OK)
                {
                    mainTreeView_AfterSelect(this, null);
                }
            }
        }

        private void removeZtcHashButton_Click(object sender, EventArgs e)
        {
            if (ztcCertHashListView.SelectedItems.Count == 0) return;
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag == null && mainTreeView.SelectedNode.Text.CompareTo(Properties.Resources.Zero_Touch_Setting) == 0)
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
                AmtSecurityAdmin.CertHashEntry entry = (AmtSecurityAdmin.CertHashEntry)ztcCertHashListView.SelectedItems[0].Tag;
                if (MessageBox.Show(this, 
                    Properties.Resources.MsgBox_Remove_Hash, 
                    Properties.Resources.MsgBox_Remove_Hash_Title, 
                    MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    AmtCallStatus r = computer.SecurityAdmin.DeleteCertificateHashEntry(entry);
                    if (r != AmtCallStatus.SUCCESS)
                    {
                        MessageBox.Show(this,
                            Properties.Resources.MsgBox_Del_Hash_Err + r.ToString(),
                            Properties.Resources.MsgBox_Del_Hash_Err_Title, 
                            MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    mainTreeView_AfterSelect(this, null);
                }
            }
        }

        private void fullUnprovisionButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                if (new UnprovisionForm(this, computer).ShowDialog(this) == DialogResult.OK)
                {
                    ArrayList machineList = Utils.GetNodeTagsofTypeRec(rootNode, typeof(AmtSystem));
                    foreach (AmtSystem system in machineList)
                    {
                        launchSetTemplate(system);
                    }
                    mainTreeView_AfterSelect(this, null);
                }
            }
        }

        private void resolveNamesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            resolveNamesToolStripMenuItem.Checked = !resolveNamesToolStripMenuItem.Checked;

            lock (discoveryListView)
            {
                foreach (ListViewItem l in discoveryListView.Items)
                {
                    AmtDiscovery.AmtDiscoveredComputer discComputer = (AmtDiscovery.AmtDiscoveredComputer)l.Tag;
                    string hostname = discComputer.EndPoint.Address.ToString();
                    if (resolveNamesToolStripMenuItem.Checked) hostname = discComputer.Hostname;
                    l.SubItems[0].Text = hostname + Properties.Resources.DashSeperator + discComputer.Info;
                }
            }
        }

        private void adminHideCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (adminHideCheckBox.Checked == false)
                adminPasswordMaskedTextBox.PasswordChar = '\0';
            else
                adminPasswordMaskedTextBox.PasswordChar = (char) 0x25CF; // Bullet character
        }

        private void adminpasswordMaskedTextBox_TextChanged(object sender, EventArgs e)
        {
            Application.UserAppDataRegistry.SetValue("ZtcAdminPassword", adminPasswordMaskedTextBox.Text);
            exportRemoteConfigButton.Enabled = (remoteConfigCertListView.CheckedItems.Count > 0 && adminPasswordMaskedTextBox.Text.Length > 0);
        }

        private void setZtcProfileButton_Click(object sender, EventArgs e)
        {
            ProfileSelectionForm f = new ProfileSelectionForm(this);
            if (f.ShowDialog(this) == DialogResult.OK)
            {
                ztcProfileLabel.Text = f.SelectedTemplate.Name;
                Application.UserAppDataRegistry.SetValue("ZtcSecurityProfile", ztcProfileLabel.Text);
                if (mainTreeView.SelectedNode == zeroTouchNode) mainTreeView_AfterSelect(this, null);
            }
        }

        private void uSBSetupFileViewerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new UsbSetupViewerForm(this).ShowDialog(this);
        }

        private void exportKeysButton_Click(object sender, EventArgs e)
        {
            if (provisioningKeysListView.SelectedItems.Count == 0) return;

            // Create a list of selected keys
            ArrayList list = new ArrayList();
            foreach (ListViewItem l in provisioningKeysListView.SelectedItems)
            {
                list.Add((AmtProvisioningKey)l.Tag);
            }
            AmtProvisioningKey[] keys = (AmtProvisioningKey[])list.ToArray(typeof(AmtProvisioningKey));

            // Call the dialog box
            new ExportSetupKeysForm(keys).ShowDialog(this);
        }

        private void addressResolutionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new AddressResolveForm().ShowDialog(this);
        }

        private void zeroTouchAuthCertListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            exportRemoteConfigButton.Enabled = (adminPasswordMaskedTextBox.Text.Length > 0);
        }

        private void exportRemoteConfigButton_Click(object sender, EventArgs e)
        {
            new ExportRemoteSetupForm(this).ShowDialog(this);
        }

        private void certificatesListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete) deleteCertButton_Click(this, null);
        }

        private void provisioningKeysListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete) deleteKeysButton_Click(this, null);
        }

        private void ztcCertHashListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete) removeZtcHashButton_Click(this, null);
        }

        private void webUiToggleButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;

                if (computer.State == AmtSystemObjState.Connected)
                {
                    AmtSecurityAdmin.EnabledInterfaces eif = computer.SecurityAdmin.GetEnabledInterfaces();
                    eif.WebUI = !eif.WebUI;
                    computer.SecurityAdmin.SetEnabledInterfaces(eif);
                }

                if (mainTreeView.SelectedNode == systemNode) mainTreeView_AfterSelect(this, null);
            }
        }

        private void saveComputersToolStripMenuItem_Click(object sender, EventArgs e)
        {
            saveFileDialog.Title = Properties.Resources.SaveComputerTitle;
            saveFileDialog.DefaultExt = Properties.Resources.XmlFileExtension;
            saveFileDialog.FileName = Properties.Resources.ComputerDefaultFilename;
            saveFileDialog.Filter = Properties.Resources.XmlFileFilter;
            if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                ArrayList settings = new ArrayList();
                foreach (TreeNode n in rootNode.Nodes)
                {
                    if (n.Tag.GetType() == typeof(AmtSystem))
                    {
                        settings.Add(new AmtSerialSystemSettings((AmtSystem)n.Tag));
                    }
                }

                FileStream file = new FileStream(saveFileDialog.FileName, FileMode.Create);
                byte[] buf = UTF8Encoding.UTF8.GetBytes(Utils.Serialize(settings.ToArray(typeof(AmtSerialSystemSettings)), typeof(AmtSerialSystemSettings[])));
                file.Write(buf, 0, buf.Length);
                file.Close();
            }
        }

        private void loadComputersToolStripMenuItem_Click(object sender, EventArgs e)
        {
            openFileDialog.Title = Properties.Resources.OpenComputerTitle;
            openFileDialog.DefaultExt = Properties.Resources.XmlFileExtension;
            openFileDialog.FileName = Properties.Resources.ComputerDefaultFilename;
            openFileDialog.Filter = Properties.Resources.XmlFileFilter;
            if (openFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                FileStream file = new FileStream(openFileDialog.FileName, FileMode.Open);
                byte[] buf = new byte[file.Length];
                file.Read(buf, 0, (int)file.Length);
                file.Close();
                string text = UTF8Encoding.UTF8.GetString(buf);
                AmtSerialSystemSettings[] settings = (AmtSerialSystemSettings[])Utils.Unserialize(text, typeof(AmtSerialSystemSettings[]));

                foreach (AmtSerialSystemSettings setting in settings)
                {
                    AddComputer(setting.GetSystem());
                }
            }
        }

        private void mainTreeView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                if (mainTreeView.SelectedNode != null && mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
                {
                    if (MessageBox.Show(this, Properties.Resources.RemoveSelectedItems, Properties.Resources.RemoveDialogTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                    {
                        removeSystemToolStripMenuItem_Click(this, null);
                    }
                }
            }
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            TreeNode node = mainTreeView.SelectedNode;

            if (node.Tag != null && node.Tag.GetType() == typeof(AmtSystem))
            {
                // Copy system settings
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                AmtSerialSystemSettings settings = new AmtSerialSystemSettings(computer);
                Utils.SerializeToClipboard(settings, typeof(AmtSerialSystemSettings));
            }
            else if (node == rootNode)
            {
                // Copy system settings
                ArrayList l = new ArrayList();
                foreach (TreeNode n in rootNode.Nodes)
                {
                    if (n.Tag.GetType() == typeof(AmtSystem))
                    {
                        l.Add(new AmtSerialSystemSettings((AmtSystem)(n.Tag)));
                    }
                }
                AmtSerialSystemSettings[] sa = (AmtSerialSystemSettings[])l.ToArray(typeof(AmtSerialSystemSettings));
                Utils.SerializeToClipboard(sa, typeof(AmtSerialSystemSettings[]));
            }
        }

        private void pasteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            TreeNode node = mainTreeView.SelectedNode;

            if (node == rootNode)
            {
                AmtSerialSystemSettings settings = (AmtSerialSystemSettings)Utils.UnSerializeFromClipboard(typeof(AmtSerialSystemSettings));
                if (settings != null)
                {
                    TreeNode systemNode = new TreeNode(settings.Hostname + Properties.Resources.SlashSeperator + settings.Username, 12, 12);
                    AddComputer(settings.GetSystem());
                    return;
                }

                AmtSerialSystemSettings[] settingsArray = (AmtSerialSystemSettings[])Utils.UnSerializeFromClipboard(typeof(AmtSerialSystemSettings[]));
                if (settingsArray != null)
                {
                    foreach (AmtSerialSystemSettings setting in settingsArray)
                    {
                        TreeNode systemNode = new TreeNode(setting.Hostname + Properties.Resources.SlashSeperator + setting.Username, 12, 12);
                        AddComputer(setting.GetSystem());
                    }
                    return;
                }
            }
        }

        private void padlockPictureBox_DoubleClick(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                X509Certificate2[] certs = computer.GetTlsCertificates();
                if (certs != null && certs.Length > 0) X509Certificate2UI.DisplayCertificate(certs[0], Handle);
            }
        }

        private void saveCertificateToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                X509Certificate2[] certs = computer.GetTlsCertificates();
                if (certs != null && certs.Length > 0)
                {
                    saveFileDialog.DefaultExt = Properties.Resources.CertificateFileExtension;
                    saveFileDialog.Filter = Properties.Resources.CertificateFileFilter;
                    saveFileDialog.Title = Properties.Resources.SaveCertificateTitle;
                    if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
                    {
                        byte[] certBin = certs[0].Export(X509ContentType.Cert);
                        FileStream f = new FileStream(saveFileDialog.FileName, FileMode.Create);
                        f.Write(certBin, 0, certBin.Length);
                        f.Close();
                    }
                }
            }
        }

        private void ManageabilityDTKLatestNewsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ManageabilityControls.AboutForm.GotoNews();
        }

        private void ManageabilityDTKDownloadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ManageabilityControls.AboutForm.GotoDownloads();
        }

        private void ManageabilityDTKSupportForumsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ManageabilityControls.AboutForm.GotoForums();
        }

        private void setRemoteConfigButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag == null && mainTreeView.SelectedNode.Text.CompareTo(Properties.Resources.Zero_Touch_Setting) == 0)
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Parent.Tag;
                computer.SecurityAdmin.SetZeroTouchConfigurationMode(!computer.SecurityAdmin.GetZeroTouchConfigurationMode());
                mainTreeView_AfterSelect(this, null);
            }
        }

        private void mainTreeView_DoubleClick(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                if (computer.State == AmtSystemObjState.Connected)
                {
                    computer.Disconnect();
                }
                else
                {
                    computer.WsManSupport = Properties.Settings.Default.WsmanSupport;
                    computer.Connect();
                }
            }
        }

        private void clearAllEntriesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (rootNode.Nodes.Count == 0) return;
            if (MessageBox.Show(this, Properties.Resources.ConfirmRemoveAllComputers, Properties.Resources.ConfirmRemoveAllEntires, MessageBoxButtons.OKCancel, MessageBoxIcon.Asterisk) != DialogResult.OK) return;

            foreach (TreeNode node in rootNode.Nodes)
            {
                if (node.Tag != null && node.Tag.GetType() == typeof(AmtSystem))
                {
                    AmtSystem computer = (AmtSystem)node.Tag;
                    ManageabilityControls.TerminalForm.DisposeRemotingForm(computer);
                    computer.Disconnect();
                    computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
                    computer.OnFetchCompleted -= new AmtSystem.FetchHandler(FetchHandler);
                }
            }
            rootNode.Nodes.Clear();
            SaveSettings();
        }

        private void certificatesListView_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.None;
            if (e.Data.GetDataPresent(DataFormats.FileDrop, false) == false) return;
            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);
            if (files == null || files.Length == 0) return;

            foreach (string file in files)
            {
                FileInfo fi = new FileInfo(files[0]);
                if (string.Compare(fi.Extension, ".pfx", true) != 0 && string.Compare(fi.Extension, ".p12", true) != 0 && string.Compare(fi.Extension, ".cer", true) != 0 && string.Compare(fi.Extension, ".pem", true) != 0) return;
            }
            e.Effect = DragDropEffects.All;
        }

        private void certificatesListView_DragDrop(object sender, DragEventArgs e)
        {
            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);
            if (files == null || files.Length == 0 || mainTreeView.SelectedNode == null) return;
            
            foreach (string file in files)
            {
                FileInfo fi = new FileInfo(file);
                if (string.Compare(fi.Extension, ".pfx", true) == 0 || string.Compare(fi.Extension, ".p12", true) == 0 || string.Compare(fi.Extension, ".cer", true) == 0 || string.Compare(fi.Extension, ".pem", true) == 0) ImportCert(fi);
            }
        }

        private void englishToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void japaneseToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void koreanToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void simplifiedChineseToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

    }
}
