// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Net;
using System.Text;
using System.Diagnostics;
using System.Collections;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;

namespace ManageabilityDirectorTool
{
    class OneTouchPush
    {
        public static int PskPushConfig(string startupPath, AmtProvisioningKey key, IPAddress addr, string computerName, string domainName, SecurityTemplate template, X509Certificate2 tlsRootCert)
        {
            Directory.SetCurrentDirectory(startupPath);

            string execdir = new FileInfo(System.Reflection.Assembly.GetEntryAssembly().Location).Directory.ToString() + "\\";

            // Write the key file
            string cfg = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?><pairs><pair><pid>"+key.Pid+"</pid><pps>"+key.Pps+"</pps></pair></pairs>";
            FileStream fs = new FileStream(execdir + "psk.repository.xml", FileMode.Create);
            byte[] bytes = UTF8Encoding.UTF8.GetBytes(cfg);
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            // Write the config file
            cfg = "";
            cfg += "<?xml version=\"1.0\" encoding=\"UTF-8\"?><config>";
            cfg += "<cfg_username>admin</cfg_username><cfg_password>" + key.AdminPassword + "</cfg_password>";
            cfg += "<host_name>" + computerName + "</host_name><domain_name>" + domainName + "</domain_name>";

            if (tlsRootCert == null)
            {
                // NO TLS
                cfg += "<tls_enable>false</tls_enable><tls_options><local>NoAuthentication</local><remote>NoAuthentication</remote></tls_options>";
            }
            else
            {
                // With TLS
                cfg += "<cert_store><mode>FileCertificate</mode><cert><cert_file>AmtCert.pem</cert_file><key_file>AmtCert.key</key_file></cert>";
                //cfg += "<cert><cert_file>subcacert.pem</cert_file></cert>"; // TODO Sub-CA's
                cfg += "</cert_store>";
                cfg += "<tls_cert><mode>FileCertificate</mode><cert_chain_file>AmtCert.raw</cert_chain_file><key_file>AmtCert.key</key_file></tls_cert><tls_enable>true</tls_enable><tls_options><local>ServerAuthentication</local><remote>ServerAuthentication</remote></tls_options>";
                cfg += "<tls_cert_name>AmtCert.pem</tls_cert_name>";

                // Setup the certificate
                X509Certificate2 newcert = ManageabilityStack.CertificateOperations.IssueSignedCert(tlsRootCert, computerName + "." + domainName, template.IssueOrganisation, template.IssueCountryCode, template.IssueKeySize, ManageabilityStack.CertificateOperations.IssueCertType.TlsServer);

                // Build a certificate chain
                X509Chain chain = new X509Chain();
                chain.ChainPolicy = new X509ChainPolicy();
                chain.ChainPolicy.RevocationMode = X509RevocationMode.NoCheck;
                chain.Build(newcert); // Cert #0 is AMT, Cert #N is Root.
                ArrayList certChainlist = new ArrayList();
                foreach (X509ChainElement certChainElement in chain.ChainElements)
                {
                    certChainlist.Add(certChainElement.Certificate);
                }
                X509Certificate2[] certChain = (X509Certificate2[])certChainlist.ToArray(typeof(X509Certificate2));
                byte[] certChainBin = ManageabilityStack.CertificateOperations.CertificateChainToBytes(certChain);

                // Write public certificates in RAW format
                FileStream AmtCertFs = new FileStream("AmtCert.raw", FileMode.Create);
                AmtCertFs.Write(certChainBin, 0, certChainBin.Length);
                AmtCertFs.Close();

                // Write public certificates in PEM format
                ManageabilityStack.CertificateOperations.CertToPemFile(newcert, "AmtCert.pem"); // TODO Sub-CA's

                // Write private key
                byte[] privateKeyBin = ManageabilityStack.CertificateOperations.GetPkcs10PrivateKeyFromCert(newcert);
                ManageabilityStack.CertificateOperations.PrivateKeyToPemFile(newcert, "AmtCert.key", null);
            }
            
            cfg += "<provisioning_mode>enterprise</provisioning_mode><ping_response>true</ping_response><new_network_username>admin</new_network_username><new_network_password>" + key.AdminPassword + "</new_network_password>";
            cfg += "<set_network_time>true</set_network_time><set_enabled_interfaces><interface>WebUI</interface><interface>SerialOverLAN</interface><interface>IdeRedirection</interface></set_enabled_interfaces>";
            cfg += "</config>";
            fs = new FileStream(execdir + "config.amt.xml", FileMode.Create);
            bytes = UTF8Encoding.UTF8.GetBytes(cfg);
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            // Push a temp config
            int code = Run(execdir + "ModifiedConfigurationServer.exe", "1 " + addr.GetAddressBytes()[0].ToString() + " " + addr.GetAddressBytes()[1].ToString() + " " + addr.GetAddressBytes()[2].ToString() + " " + addr.GetAddressBytes()[3].ToString() + " " + key.Pid.Replace("-", ""));

            // Delete temp files
            if (code == 0)
            {
                File.Delete(execdir + "psk.repository.xml");
                File.Delete(execdir + "config.amt.xml");
                File.Delete(execdir + "RECV.log");
                File.Delete(execdir + "SENT.log");
                File.Delete(execdir + "TEST.log");
                if (File.Exists("AmtCert.raw") == true) File.Delete("AmtCert.raw");
                if (File.Exists("AmtCert.key") == true) File.Delete("AmtCert.key");
                if (File.Exists("AmtCert.pem") == true) File.Delete("AmtCert.pem");
            }
            else
            {
                // Write a test batch file
                cfg = "\"" + execdir + "ModifiedConfigurationServer.exe\" 1 " + addr.GetAddressBytes()[0].ToString() + " " + addr.GetAddressBytes()[1].ToString() + " " + addr.GetAddressBytes()[2].ToString() + " " + addr.GetAddressBytes()[3].ToString() + " " + key.Pid.Replace("-", "") + "\r\npause\r\n";
                fs = new FileStream(execdir + "run-psk.bat", FileMode.Create);
                bytes = UTF8Encoding.UTF8.GetBytes(cfg);
                fs.Write(bytes, 0, bytes.Length);
                fs.Close();
            }

            return code;
        }

        public static int PkiPushConfig(string startupPath, string MachineName, string DomainName, IPAddress addr, string adminPassword, X509Chain certChain, SecurityTemplate template, X509Certificate2 tlsRootCert)
        {
            Directory.SetCurrentDirectory(startupPath);

            string execdir = new FileInfo(System.Reflection.Assembly.GetEntryAssembly().Location).Directory.ToString() + "\\";

            // Save root chain file
            ManageabilityStack.CertificateOperations.CertToPemFile(certChain.ChainElements[certChain.ChainElements.Count - 1].Certificate, execdir + "RootCert.pem");

            // Save certificate file
            CertificateOperations.ChainAndKeyToPemFile(certChain, execdir + "FullChain.pem");

            // Write the config file
            string cfg = "";
            cfg += "<?xml version=\"1.0\" encoding=\"UTF-8\"?><config>";
            cfg += "<cfg_username>admin</cfg_username><cfg_password>" + adminPassword + "</cfg_password>";
            cfg += "<host_name>" + MachineName + "</host_name><domain_name>" + DomainName + "</domain_name>";

            if (tlsRootCert == null)
            {
                // NO TLS
                cfg += "<tls_enable>false</tls_enable><tls_options><local>NoAuthentication</local><remote>NoAuthentication</remote></tls_options>";
            }
            else
            {
                // With TLS
                cfg += "<cert_store><mode>FileCertificate</mode><cert><cert_file>AmtCert.pem</cert_file><key_file>AmtCert.key</key_file></cert>";
                //cfg += "<cert><cert_file>subcacert.pem</cert_file></cert>"; // TODO Sub-CA's
                cfg += "</cert_store>";
                cfg += "<tls_cert><mode>FileCertificate</mode><cert_chain_file>AmtCert.raw</cert_chain_file><key_file>AmtCert.key</key_file></tls_cert><tls_enable>true</tls_enable><tls_options><local>ServerAuthentication</local><remote>ServerAuthentication</remote></tls_options>";
                cfg += "<tls_cert_name>AmtCert.pem</tls_cert_name>";

                // Setup the certificate
                X509Certificate2 newcert = ManageabilityStack.CertificateOperations.IssueSignedCert(tlsRootCert, MachineName + "." + DomainName, template.IssueOrganisation, template.IssueCountryCode, template.IssueKeySize, ManageabilityStack.CertificateOperations.IssueCertType.TlsServer);

                // Build a certificate chain
                X509Chain chain = new X509Chain();
                chain.ChainPolicy = new X509ChainPolicy();
                chain.ChainPolicy.RevocationMode = X509RevocationMode.NoCheck;
                chain.Build(newcert); // Cert #0 is AMT, Cert #N is Root.
                ArrayList certChainlist = new ArrayList();
                foreach (X509ChainElement certChainElement in chain.ChainElements)
                {
                    certChainlist.Add(certChainElement.Certificate);
                }
                X509Certificate2[] tlsCertChain = (X509Certificate2[])certChainlist.ToArray(typeof(X509Certificate2));
                byte[] certChainBin = ManageabilityStack.CertificateOperations.CertificateChainToBytes(tlsCertChain);

                // Write public certificates in RAW format
                FileStream AmtCertFs = new FileStream("AmtCert.raw", FileMode.Create);
                AmtCertFs.Write(certChainBin, 0, certChainBin.Length);
                AmtCertFs.Close();

                // Write public certificates in PEM format
                ManageabilityStack.CertificateOperations.CertToPemFile(newcert, "AmtCert.pem"); // TODO Sub-CA's

                // Write private key
                byte[] privateKeyBin = ManageabilityStack.CertificateOperations.GetPkcs10PrivateKeyFromCert(newcert);
                ManageabilityStack.CertificateOperations.PrivateKeyToPemFile(newcert, "AmtCert.key", null);
            }
            
            cfg += "<provisioning_mode>enterprise</provisioning_mode><ping_response>true</ping_response><new_network_username>admin</new_network_username><new_network_password>" + adminPassword + "</new_network_password>";
            cfg += "<set_network_time>true</set_network_time><set_enabled_interfaces><interface>WebUI</interface><interface>SerialOverLAN</interface><interface>IdeRedirection</interface></set_enabled_interfaces>";
            cfg += "<pki_configuration><full_cert_chain_file>" + execdir + "FullChain.pem</full_cert_chain_file><root_cert_file>" + execdir + "RootCert.pem</root_cert_file><new_mebx_password>" + adminPassword + "</new_mebx_password></pki_configuration>";
            cfg += "</config>";
            FileStream fs = new FileStream(execdir + "config.amt.xml", FileMode.Create);
            byte[] bytes = UTF8Encoding.UTF8.GetBytes(cfg);
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            // Write a test batch file in case things fail.
            cfg = "\"" + execdir + "ModifiedConfigurationServer.exe\" 0 " + addr.GetAddressBytes()[0].ToString() + " " + addr.GetAddressBytes()[1].ToString() + " " + addr.GetAddressBytes()[2].ToString() + " " + addr.GetAddressBytes()[3].ToString() + "\r\npause\r\n";
            fs = new FileStream(execdir + "run-pki.bat", FileMode.Create);
            bytes = UTF8Encoding.UTF8.GetBytes(cfg);
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            // Push a temp config
            int code = Run(execdir + "ModifiedConfigurationServer.exe", "0 " + addr.GetAddressBytes()[0].ToString() + " " + addr.GetAddressBytes()[1].ToString() + " " + addr.GetAddressBytes()[2].ToString() + " " + addr.GetAddressBytes()[3].ToString());

            if (code == 0)
            {
                // Delete temp files
                File.Delete(execdir + "RootCert.pem");
                File.Delete(execdir + "FullChain.pem");
                File.Delete(execdir + "RECV.log");
                File.Delete(execdir + "SENT.log");
                File.Delete(execdir + "TEST.log");
                File.Delete(execdir + "config.amt.xml");
                File.Delete(execdir + "run-pki.bat");
                if (File.Exists("AmtCert.raw") == true) File.Delete("AmtCert.raw");
                if (File.Exists("AmtCert.key") == true) File.Delete("AmtCert.key");
                if (File.Exists("AmtCert.pem") == true) File.Delete("AmtCert.pem");
            }
            
            return code;
        }

        private static int Run(string fileName, string args)
        {
            if (File.Exists(fileName) == false) return -1;
            ProcessStartInfo info = new ProcessStartInfo(fileName, args);
            info.CreateNoWindow = true;
            info.UseShellExecute = false;
            Process processChild = Process.Start(info);
            while (processChild.HasExited == false) System.Threading.Thread.Sleep(100);
            return processChild.ExitCode;
        }

    }
}
