// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Net;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Collections.Generic;

namespace ManageabilityDirectorTool
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main(string[] args)
        {
            WebRequest.DefaultWebProxy = null;

            foreach (string arg in args)
            {
                if (arg.Length > 3 && string.Compare(arg.Substring(0, 3), "-l:", true) == 0) { try { System.Threading.Thread.CurrentThread.CurrentUICulture = new System.Globalization.CultureInfo(arg.Substring(3)); } catch (ArgumentException) { } }
                if (string.Compare(arg, "-debug", true) == 0) ManageabilityStack.AmtDebugForm.StartDebugLog("DebugLog-Director.txt");
                if (arg.Length > 7 && string.Compare(arg.Substring(0, 7), "-proxy:", true) == 0) WebRequest.DefaultWebProxy = new WebProxy(arg.Substring(7));
            }

            if (OneInstanceOnly() == false)
            {
                MessageBox.Show(Properties.Resources.OnceInstanceError, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);

            try
            {
                Application.UserAppDataRegistry.GetValue("SystemsList", "");
            }
            catch (System.Security.SecurityException)
            {
                MessageBox.Show(
                    Properties.Resources.Program_MsgBox_0,
                    Properties.Resources.Program_MsgBox_1,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            Application.ThreadException += new System.Threading.ThreadExceptionEventHandler(ExceptionSink);
            Application.SetUnhandledExceptionMode(UnhandledExceptionMode.CatchException, true);
            AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(UnhandledExceptionEventSink);

            System.Globalization.CultureInfo currentCulture;
            do
            {
                currentCulture = System.Threading.Thread.CurrentThread.CurrentUICulture;
                Application.Run(new MainForm(args));
            }
            while (currentCulture.Equals(System.Threading.Thread.CurrentThread.CurrentUICulture) == false);
        }

        public static void ExceptionSink(object sender, System.Threading.ThreadExceptionEventArgs args)
        {
            ManageabilityStack.AmtDebugForm.GenerateExceptionFile("Exceptions-Director.txt", args.Exception.ToString(), ManageabilityControls.AboutForm.VersionString);
        }

        public static void UnhandledExceptionEventSink(object sender, UnhandledExceptionEventArgs args)
        {
            ManageabilityStack.AmtDebugForm.GenerateExceptionFile("Exceptions-Director.txt", ((Exception)args.ExceptionObject).ToString(), ManageabilityControls.AboutForm.VersionString);
        }

        private static Mutex g_mux = null;
        public static bool OneInstanceOnly()
        {
            try
            {
                g_mux = Mutex.OpenExisting(Application.ProductName);
            }
            catch (WaitHandleCannotBeOpenedException) {}

            if (g_mux == null)
            {
                g_mux = new Mutex(true, Application.ProductName);
                return true;
            }
            return false;
        }

    }
}
