// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using ManageabilityStack;

namespace ManageabilityDirectorTool
{
    public partial class TrustedCertificateEditForm : Form
    {
        private SecurityTemplate securityTemplate;
        private X509Store CertificateStore;
        private X509Store CertificateRootStore = null;

        public TrustedCertificateEditForm(SecurityTemplate securityTemplate)
        {
            this.securityTemplate = securityTemplate;

            InitializeComponent();
        }

        private void TrustedCertificateEditForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            
            // Open the certificate stores
            CertificateStore = new X509Store(StoreName.My, StoreLocation.CurrentUser);
            CertificateStore.Open(OpenFlags.ReadOnly);
            CertificateRootStore = new X509Store(StoreName.Root, StoreLocation.CurrentUser);
            CertificateRootStore.Open(OpenFlags.ReadOnly);

            // Load the list of trusted certificates. Trusted certs must be in both stores.
            foreach (X509Certificate2 cert in CertificateStore.Certificates)
            {
                if (ManageabilityStack.CertificateOperations.IsTrustedCert(cert) && ManageabilityStack.CertificateOperations.IsCaCertificate(cert))
                {
                    ListViewItem l = new ListViewItem(new string[] { GetCertificateString(cert.Subject) });
                    l.Tag = cert;
                    if (securityTemplate.TrustedRootCertificatesTP != null)
                    {
                        bool found = false;
                        foreach (string certTP in securityTemplate.TrustedRootCertificatesTP)
                        {
                            if (cert.Thumbprint == certTP)
                            {
                                found = true;
                                break;
                            }
                        }
                        l.Checked = found;
                    }
                    trustedCertsListView.Items.Add(l);
                }
            }

        }

        private void TrustedCertificateEditForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            // Close the certificate stores
            CertificateStore.Close();
            CertificateStore = null;
            CertificateRootStore.Close();
            CertificateRootStore = null;
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            // Build a list of trusted certificates
            ArrayList certList = new ArrayList();
            foreach (ListViewItem l in trustedCertsListView.Items)
            {
                if (l.Checked == true) certList.Add(((X509Certificate2)l.Tag).Thumbprint);
            }
            securityTemplate.TrustedRootCertificatesTP = (string[])certList.ToArray(typeof(string));

            DialogResult = DialogResult.OK;
        }

        private string GetCertificateString(string certDataStr)
        {
            string s = "";

            int i = certDataStr.IndexOf("CN=");
            if (i >= 0)
            {
                s = certDataStr.Substring(i + 3);
                i = s.IndexOf(",");
                if (i > 0) s = s.Substring(0, i);
            }

            string s2 = null;
            i = certDataStr.IndexOf("O=");
            if (i >= 0)
            {
                s2 = certDataStr.Substring(i + 2);
                i = s2.IndexOf(",");
                if (i > 0) s2 = s2.Substring(0, i);
                s += ", " + s2;
            }

            return s;
        }

    }
}
