// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityDirectorTool
{
    public partial class UnprovisionForm : Form
    {
        private AmtSystem computer;
        private MainForm parent;

        public UnprovisionForm(MainForm parent, AmtSystem computer)
        {
            this.parent = parent;
            this.computer = computer;
            InitializeComponent();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            AmtSecurityAdmin.ProvisioningMode pm = AmtSecurityAdmin.ProvisioningMode.Current;
            if (radioButton2.Checked) pm = AmtSecurityAdmin.ProvisioningMode.SmallBusiness;
            if (radioButton3.Checked) pm = AmtSecurityAdmin.ProvisioningMode.Enterprise;

            if (computer.SecurityAdmin.GetProvisioningMode() == AmtSecurityAdmin.ProvisioningMode.Enterprise && (oneTouchCheckBox.Checked || zeroTouchCheckBox.Checked))
            {
                // Perform some pre-reset settings changes.
                if (computer.SecurityAdmin.GetZeroTouchConfigurationMode() == false)
                {
                    AmtProvisioningKey pk = (AmtProvisioningKey)((Utils.DropDownItem)oneTouchKeyComboBox.SelectedItem).Tag;
                    AmtCallStatus r = computer.SecurityAdmin.SetTLSPSK(UTF8Encoding.UTF8.GetBytes(pk.Pid.Replace("-", "")), UTF8Encoding.UTF8.GetBytes(pk.Pps.Replace("-", "")));
                    if (r != AmtCallStatus.SUCCESS)
                    {
                        MessageBox.Show(this,
                            ManageabilityDirectorTool.Properties.Resources.MsgBox_SetTLSPSK_Err + r.ToString(),
                            ManageabilityDirectorTool.Properties.Resources.MsgBox_SetTLSPSK_Err_Title,
                            MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }
                    r = computer.SecurityAdmin.CommitChanges();
                    if (r != AmtCallStatus.SUCCESS)
                    {
                        MessageBox.Show(this,
                            ManageabilityDirectorTool.Properties.Resources.MsgBox_Commit_Change + r.ToString(),
                            ManageabilityDirectorTool.Properties.Resources.MsgBox_Commit_Change_Title,
                            MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }

                    // Add admin account settings change in pending list along with the reset
                    ArrayList setList = new ArrayList();
                    setList.Add(new SecurityTemplateAdminChange("admin", pk.AdminPassword));
                    setList.Add(new SecurityTemplateReset(fullCheckBox.Checked, pm));
                    computer.Tag = setList;
                    computer.AutoFetchCache = false;
                }
                else
                {
                    // Set PID/PSK to NULL so RCFG can be used.
                    AmtCallStatus r = computer.SecurityAdmin.SetTLSPSK(UTF8Encoding.UTF8.GetBytes("00000000"), UTF8Encoding.UTF8.GetBytes("00000000000000000000000000000000"));
                    if (r != AmtCallStatus.SUCCESS)
                    {
                        MessageBox.Show(this,
                            ManageabilityDirectorTool.Properties.Resources.MsgBox_SetTLSPSK_Err + r.ToString(),
                            ManageabilityDirectorTool.Properties.Resources.MsgBox_SetTLSPSK_Err_Title,
                            MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }
                    r = computer.SecurityAdmin.CommitChanges();
                    if (r != AmtCallStatus.SUCCESS)
                    {
                        MessageBox.Show(this,
                            ManageabilityDirectorTool.Properties.Resources.MsgBox_Commit_Change + r.ToString(),
                            ManageabilityDirectorTool.Properties.Resources.MsgBox_Commit_Change_Title,
                            MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }

                    // Add admin account settings change in pending list along with the reset
                    ArrayList setList = new ArrayList();
                    setList.Add(new SecurityTemplateAdminChange("admin", parent.GetZtcPassword));
                    setList.Add(new SecurityTemplateReset(fullCheckBox.Checked, pm));
                    computer.Tag = setList;
                    computer.AutoFetchCache = false;
                }
            }
            else
            {
                // Add the reset to the pending operations list
                ArrayList setList = new ArrayList();
                setList.Add(new SecurityTemplateReset(fullCheckBox.Checked, pm));
                computer.Tag = setList;
                computer.AutoFetchCache = false;
            }

            DialogResult = DialogResult.OK;
        }

        private void FullUnprovisionForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            
            AmtSecurityAdmin.ProvisioningMode m = computer.SecurityAdmin.GetProvisioningMode();
            string s = "";
            if (m == AmtSecurityAdmin.ProvisioningMode.Enterprise) s = Properties.Resources.ProvisionEnterprise;
            if (m == AmtSecurityAdmin.ProvisioningMode.SmallBusiness) s = Properties.Resources.ProvisionSmallBuisness;
            radioButton1.Text = string.Format(radioButton1.Text, s);

            // Fill the one-touch key combo box
            foreach (AmtProvisioningKey pk in parent.provisioningKeyStore.Keys.Values)
            {
                oneTouchKeyComboBox.Items.Add(new Utils.DropDownItem(pk.Pid + ", " + pk.SecurityTemplate, 0, pk));
            }
            if (parent.provisioningKeyStore.Keys.Count == 0)
            {
                oneTouchCheckBox.Enabled = false;
                oneTouchKeyComboBox.Enabled = false;
            }
            else
            {
                oneTouchKeyComboBox.SelectedIndex = 0;
                oneTouchCheckBox.Checked = true;
            }

            if (computer.SecurityAdmin.GetProvisioningMode() == AmtSecurityAdmin.ProvisioningMode.SmallBusiness)
            {
                // Small Buisness Mode
                fullCheckBox.Checked = true;
                fullCheckBox.Enabled = false;
                zeroTouchGroupBox.Visible = false;
                Height -= zeroTouchGroupBox.Height;
                oneTouchGroupBox.Visible = false;
                Height -= oneTouchGroupBox.Height;
            }
            else
            {
                // Enterprise Mode
                if (computer.SecurityAdmin.GetZeroTouchConfigurationMode() == true)
                {
                    // Computer is in Zero-Touch mode, hide the one-touch setup.
                    oneTouchGroupBox.Visible = false;
                    Height -= oneTouchGroupBox.Height;
                    zeroTouchGroupBox.Top -= oneTouchGroupBox.Height;
                }
                else
                {
                    // Computer is in One-Touch mode, hide the zero-touch setup.
                    zeroTouchGroupBox.Visible = false;
                    Height -= zeroTouchGroupBox.Height;
                }
            }

            UpdateInfo();
        }

        private void UpdateInfo()
        {
            oneTouchKeyComboBox.Enabled = oneTouchCheckBox.Checked;
            radioButton1.Enabled = fullCheckBox.Checked;
            radioButton2.Enabled = fullCheckBox.Checked;
            radioButton3.Enabled = fullCheckBox.Checked;
        }

        private void oneTouchCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void fullCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

    }
}
