// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Data;
using System.Text;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;

namespace ManageabilityDirectorTool
{
    public partial class UsbSetupViewerForm : Form
    {
        private MainForm mainForm;

        public UsbSetupViewerForm(MainForm mainForm)
        {
            this.mainForm = mainForm;
            InitializeComponent();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void openSetupFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog(this) == DialogResult.OK)
            {
                mainToolStripStatusLabel.Text = Properties.Resources.SetupViewerLoading;
                mainListView.Items.Clear();
                Application.DoEvents();
                FileStream fs;
                try
                {
                    fs = new FileStream(openFileDialog1.FileName, FileMode.Open);
                }
                catch (Exception ex) { ManageabilityStack.Utils.FileErrorMessageBox(this, ex); return; }
                UsbFile file = new UsbFile(fs);
                fs.Close();
                mainToolStripStatusLabel.Text = String.Format(Properties.Resources.SetupViewerStatus, file.MajorVersion + "." + file.MinorVersion, file.DataRecordsConsumed, file.DataRecordCount);

                mainListView.BeginUpdate();
                ListViewItem ll = new ListViewItem(new string[] { Properties.Resources.SetupViewerNothing, Properties.Resources.SetupViewerNothing, Properties.Resources.SetupViewerFileHeader });
                ll.Tag = file;
                mainListView.Items.Add(ll);

                foreach (UsbFileRecord r in file.Records)
                {
                    string state = Properties.Resources.SetupViewerKeyReady;
                    if (r.RecordFlags != 1) state = Properties.Resources.SetupViewerKeyUsed;
                    string info = "";
                    foreach (UsbFileVariable v in r.Variables)
                    {
                        if (v.ModuleIdentifier == 2 && v.VariableIdentifier == 8 && v.VariableValue[0] == 1)
                        {
                            byte[] hash = new byte[20];
                            string hashname = UTF8Encoding.UTF8.GetString(v.VariableValue, 22, v.VariableValue.Length - 22);
                            Array.Copy(v.VariableValue, 1, hash, 0, 20);
                            info += ", \"" + hashname + "\":0x" + ManageabilityStack.Utils.BytesToHex(hash);
                        }
                        else if (v.ModuleIdentifier == 2 && v.VariableIdentifier > 4)
                        {
                            info += ", 0x" + ManageabilityStack.Utils.BytesToHex(v.VariableValue);
                        }
                        else
                        {
                            info += ", " + UTF8Encoding.UTF8.GetString(v.VariableValue);
                        }
                    }
                    if (info.Length > 2) info = info.Substring(2);
                    ListViewItem l = new ListViewItem(new string[] { r.RecordNumber.ToString(), state, info });
                    l.Tag = r;
                    mainListView.Items.Add(l);
                }
                mainListView.EndUpdate();
            }
        }

        public class UsbFile
        {
            public byte[] FileTypeUuid;
            public ushort RecordChunkCount;
            public ushort RecordHeaderByteCount;
	        public uint RecordNumber;
	        public byte MajorVersion;
	        public byte MinorVersion;
	        public uint DataRecordCount;
	        public uint DataRecordsConsumed;
            public ushort DataRecordChunkCount;
            public UsbFileRecord[] Records;

            public UsbFile(FileStream stream)
            {
                byte[] data = new byte[512];
                stream.Read(data, 0, 512);

                FileTypeUuid = new byte[16];
                Array.Copy(data, FileTypeUuid, 16);
                RecordChunkCount = BitConverter.ToUInt16(data, 16);
                RecordHeaderByteCount = BitConverter.ToUInt16(data, 18);
                RecordNumber = BitConverter.ToUInt32(data, 20);
                MajorVersion = data[24];
                MinorVersion = data[25];
                DataRecordCount = BitConverter.ToUInt32(data, 28);
                DataRecordsConsumed = BitConverter.ToUInt32(data, 32);
                DataRecordChunkCount = BitConverter.ToUInt16(data, 36);

                ArrayList l = new ArrayList();
                int ptr = 512;
                while (ptr + 512 <= stream.Length)
                {
                    l.Add(new UsbFileRecord(stream));
                    ptr += 512;
                }
                Records = (UsbFileRecord[])l.ToArray(typeof(UsbFileRecord));
            }

            public UsbFile(byte[] data)
            {
                FileTypeUuid = new byte[16];
                Array.Copy(data, FileTypeUuid, 16);
                RecordChunkCount = BitConverter.ToUInt16(data, 16);
                RecordHeaderByteCount = BitConverter.ToUInt16(data, 18);
                RecordNumber = BitConverter.ToUInt32(data, 20);
                MajorVersion = data[24];
                MinorVersion = data[25];
                DataRecordCount = BitConverter.ToUInt32(data, 28);
                DataRecordsConsumed = BitConverter.ToUInt32(data, 32);
                DataRecordChunkCount = BitConverter.ToUInt16(data, 36);

                ArrayList l = new ArrayList();
                int ptr = 512;
                while (ptr + 512 <= data.Length)
                {
                    l.Add(new UsbFileRecord(data, ptr));
                    ptr += 512;
                }
                Records = (UsbFileRecord[])l.ToArray(typeof(UsbFileRecord));
            }

            public UsbFile(UsbFileRecord[] records, byte majorVersion, byte minorVersion)
            {
                RecordChunkCount = 1;
                RecordHeaderByteCount = 0x2E;
	            RecordNumber = 0;
	            MajorVersion = majorVersion;
	            MinorVersion = minorVersion;
                DataRecordCount = (uint)records.Length;
	            DataRecordsConsumed = 0;
                DataRecordChunkCount = 1;
                Records = records;
            }

            public override string ToString()
            {
                string ret = "";
                ret += string.Format(Properties.Resources.SetupViewerFileTypeUuid, ManageabilityStack.Utils.BytesToHex(FileTypeUuid)) + "\r\n";
                ret += string.Format(Properties.Resources.SetupViewerRecordChunkCount, RecordChunkCount.ToString()) + "\r\n";
                ret += string.Format(Properties.Resources.SetupViewerRecordHeaderByteCount, RecordHeaderByteCount.ToString()) + "\r\n";
                ret += string.Format(Properties.Resources.SetupViewerRecordNumber, RecordNumber.ToString()) + "\r\n";
                ret += string.Format(Properties.Resources.SetupViewerFileVersion, MajorVersion.ToString(), MinorVersion.ToString()) + "\r\n";
                ret += string.Format(Properties.Resources.SetupViewerDataRecordCount, DataRecordCount.ToString()) + "\r\n";
                ret += string.Format(Properties.Resources.SetupViewerDataRecordsConsumed, DataRecordsConsumed.ToString()) + "\r\n";
                ret += string.Format(Properties.Resources.SetupViewerDataChunkCount, DataRecordChunkCount.ToString()) + "\r\n";
                return ret;
            }

            public byte[] GetBytes()
            {
                uint dataRecordsConsumed = 0;
                foreach (UsbFileRecord r in Records)
                {
                    if (r.RecordFlags != 1) dataRecordsConsumed++;
                }

                MemoryStream MS = new MemoryStream();
                if (this.MajorVersion == 2)
                {
                    MS.Write(new byte[] { 0x96, 0xB2, 0x81, 0x58, 0xCF, 0x6B, 0x72, 0x4C, 0x8B, 0x91, 0xA1, 0x5E, 0x51, 0x2E, 0x99, 0xC4 }, 0, 16);
                }
                else
                {
                    MS.Write(new byte[] { 0xB5, 0x16, 0xFB, 0x71, 0x87, 0xCB, 0xF9, 0x4A, 0xB4, 0x41, 0xCA, 0x7B, 0x38, 0x35, 0x78, 0xF9 }, 0, 16);
                }
                MS.Write(BitConverter.GetBytes((ushort)RecordChunkCount), 0, 2);
                MS.Write(BitConverter.GetBytes((ushort)RecordHeaderByteCount), 0, 2);
                MS.Write(BitConverter.GetBytes((uint)RecordNumber), 0, 4);
                MS.WriteByte(MajorVersion);
                MS.WriteByte(MinorVersion);
                MS.Write(BitConverter.GetBytes((ushort)0), 0, 2);
                MS.Write(BitConverter.GetBytes((uint)Records.Length), 0, 4);
                MS.Write(BitConverter.GetBytes((uint)dataRecordsConsumed), 0, 4);
                MS.Write(BitConverter.GetBytes((ushort)DataRecordChunkCount), 0, 2);
                MS.Write(BitConverter.GetBytes((ushort)0), 0, 2);
                MS.Write(BitConverter.GetBytes((ushort)1), 0, 2);
                MS.Write(BitConverter.GetBytes((ushort)2), 0, 2);
                while (MS.Length < 512) MS.WriteByte(0);
                uint i = 1;
                foreach (UsbFileRecord r in Records)
                {
                    byte[] b = r.GetBytes(i++);
                    MS.Write(b, 0, b.Length);
                }
                return MS.ToArray();
            }
        }

        public class UsbFileRecord
        {
            public uint RecordTypeIdentifier;
	        public uint RecordFlags;
	        public ushort RecordChunkCount;
	        public ushort RecordHeaderByteCount;
	        public uint RecordNumber;
            public UsbFileVariable[] Variables;

            public UsbFileRecord(FileStream stream)
            {
                byte[] data = new byte[512];
                int offset = 0;
                stream.Read(data, 0, 512);

                RecordTypeIdentifier = BitConverter.ToUInt32(data, offset + 0);
                RecordFlags = BitConverter.ToUInt32(data, offset + 4);
                RecordChunkCount = BitConverter.ToUInt16(data, offset + 16);
                RecordHeaderByteCount = BitConverter.ToUInt16(data, offset + 18);
                RecordNumber = BitConverter.ToUInt32(data, offset + 20);

                ArrayList l = new ArrayList();
                int ptr = offset + 24;
                while (true)
                {
                    UsbFileVariable v = new UsbFileVariable(data, ptr);
                    if (v.ModuleIdentifier == 0) break;
                    l.Add(v);
                    ptr += v.GetTotalSize();
                }

                Variables = (UsbFileVariable[])l.ToArray(typeof(UsbFileVariable));
            }

            public UsbFileRecord(byte[] data, int offset)
            {
                RecordTypeIdentifier = BitConverter.ToUInt32(data, offset + 0);
                RecordFlags = BitConverter.ToUInt32(data, offset + 4);
                RecordChunkCount = BitConverter.ToUInt16(data, offset + 16);
                RecordHeaderByteCount = BitConverter.ToUInt16(data, offset + 18);
                RecordNumber = BitConverter.ToUInt32(data, offset + 20);

                ArrayList l = new ArrayList();
                int ptr = offset + 24;
                while (true)
                {
                    UsbFileVariable v = new UsbFileVariable(data, ptr);
                    if (v.ModuleIdentifier == 0) break;
                    l.Add(v);
                    ptr += v.GetTotalSize();
                }

                Variables = (UsbFileVariable[])l.ToArray(typeof(UsbFileVariable));
            }

            public UsbFileRecord(string pid, string pps, string adminName, string adminPassword)
            {
                RecordTypeIdentifier = 1;
                RecordFlags = 1;
                RecordChunkCount = 1;
                RecordHeaderByteCount = 24;
                RecordNumber = 0;

                Variables = new UsbFileVariable[4];
                Variables[0] = new UsbFileVariable(1, 1, adminName);
                Variables[1] = new UsbFileVariable(1, 2, adminPassword);
                Variables[2] = new UsbFileVariable(2, 1, pid);
                Variables[3] = new UsbFileVariable(2, 2, pps);
            }

            public UsbFileRecord(string adminName, string adminPassword, int preInstalledCerts, int remoteConfig, string configServerFqdn, string pkiDnsSuffix, UsbSetupViewerForm.UsbFileVariable[] hashs, int redirect)
            {
                RecordTypeIdentifier = 1;
                RecordFlags = 1;
                RecordChunkCount = 1;
                RecordHeaderByteCount = 24;
                RecordNumber = 0;

                ArrayList vars = new ArrayList();
                vars.Add(new UsbFileVariable(1, 1, adminName));                                             // Admin Name
                vars.Add(new UsbFileVariable(1, 2, adminPassword));                                         // New Admin Password
                vars.Add(new UsbFileVariable(1, 3, 1));                                                     // AMT Mode
                if (pkiDnsSuffix.Length != 0) vars.Add(new UsbFileVariable(2, 3, pkiDnsSuffix));            // PKI DNS Suffix
                vars.Add(new UsbFileVariable(2, 4, configServerFqdn));                                      // Config Server FQDN
                if (remoteConfig >= 0)      vars.Add(new UsbFileVariable(2, 5, (byte)remoteConfig));        // Remote Configuration
                if (preInstalledCerts >= 0) vars.Add(new UsbFileVariable(2, 6, (byte)preInstalledCerts));   // Pre-Installed Certs
                vars.Add(new UsbFileVariable(2, 7, 2));                                                     // Clear all user certificate hashes
                vars.AddRange(hashs);                                                                       // Trusted certificate hashs
                if (redirect >= 0)
                {
                    if (redirect == 1) vars.Add(new UsbFileVariable(2, 10, 4));
                    else if (redirect == 2) vars.Add(new UsbFileVariable(2, 10, 5));
                    else if (redirect == 3) vars.Add(new UsbFileVariable(2, 10, 6));
                    else if (redirect == 4) vars.Add(new UsbFileVariable(2, 10, 7));
                }
                Variables = (UsbFileVariable[])vars.ToArray(typeof(UsbFileVariable));
            }

            public UsbFileVariable GetVariable(ushort ModuleIdentifier, ushort VariableIdentifier)
            {
                foreach (UsbFileVariable v in Variables)
                {
                    if (v.ModuleIdentifier == ModuleIdentifier && v.VariableIdentifier == VariableIdentifier) return v;
                }
                return null;
            }

            public override string ToString()
            {
                string ret = "";
                ret += string.Format(Properties.Resources.SetupViewerRecordTypeId, RecordTypeIdentifier.ToString()) + "\r\n";
                ret += string.Format(Properties.Resources.SetupViewerRecordFlags, RecordFlags.ToString()) + "\r\n";
                ret += string.Format(Properties.Resources.SetupViewerRecordNumber, RecordNumber.ToString()) + "\r\n";
                foreach (UsbFileVariable v in Variables) ret += v.ToString();
                return ret;
            }

            public byte[] GetBytes(uint recordNumber)
            {
                MemoryStream MS = new MemoryStream();
                MS.Write(BitConverter.GetBytes((uint)RecordTypeIdentifier), 0, 4);
                MS.Write(BitConverter.GetBytes((uint)RecordFlags), 0, 4);
                MS.Write(BitConverter.GetBytes((uint)0), 0, 4);
                MS.Write(BitConverter.GetBytes((uint)0), 0, 4);
                MS.Write(BitConverter.GetBytes((ushort)RecordChunkCount), 0, 2);
                MS.Write(BitConverter.GetBytes((ushort)RecordHeaderByteCount), 0, 2);
                MS.Write(BitConverter.GetBytes((uint)recordNumber), 0, 4);
                foreach (UsbFileVariable v in Variables)
                {
                    byte[] b = v.GetBytes();
                    MS.Write(b, 0, b.Length);
                }
                while (MS.Length < 512) MS.WriteByte(0);
                return MS.ToArray();
            }
        }

        public class UsbFileVariable
        {
            public ushort ModuleIdentifier;
	        public ushort VariableIdentifier;
	        public ushort VariableLength;
            public byte[] VariableValue;

            public UsbFileVariable(byte[] data, int offset)
            {
                ModuleIdentifier = BitConverter.ToUInt16(data, offset + 0);
                VariableIdentifier = BitConverter.ToUInt16(data, offset + 2);
                VariableLength = BitConverter.ToUInt16(data, offset + 4);
                VariableValue = new byte[VariableLength];
                Array.Copy(data, offset + 8, VariableValue, 0, VariableLength);
            }

            public UsbFileVariable(ushort moduleIdentifier, ushort variableIdentifier, string variableValue)
            {
                ModuleIdentifier = moduleIdentifier;
                VariableIdentifier = variableIdentifier;
                VariableValue = UTF8Encoding.UTF8.GetBytes(variableValue);
                VariableLength = (ushort)VariableValue.Length;
            }

            public UsbFileVariable(ushort moduleIdentifier, ushort variableIdentifier, byte variableValue)
            {
                ModuleIdentifier = moduleIdentifier;
                VariableIdentifier = variableIdentifier;
                VariableValue = new byte[1];
                VariableValue[0] = variableValue;
                VariableLength = 1;
            }

            public UsbFileVariable(ushort moduleIdentifier, ushort variableIdentifier, byte[] variableValue)
            {
                ModuleIdentifier = moduleIdentifier;
                VariableIdentifier = variableIdentifier;
                VariableValue = variableValue;
                VariableLength = (ushort)variableValue.Length;
            }

            public int GetTotalSize()
            {
                return 8 + (((VariableLength + 3) / 4) * 4);
            }

            public string GetValueStr()
            {
                return UTF8Encoding.UTF8.GetString(VariableValue);
            }

            public override string ToString()
            {
                if (ModuleIdentifier == 2 && VariableIdentifier == 8 && VariableValue[0] == 1)
                {
                    byte[] hash = new byte[20];
                    string hashname = UTF8Encoding.UTF8.GetString(VariableValue, 22, VariableValue.Length - 22);
                    Array.Copy(VariableValue, 1, hash, 0, 20);
                    return string.Format(Properties.Resources.SetupViewerFileVar1, ModuleIdentifier.ToString(), VariableIdentifier.ToString(), hashname, ManageabilityStack.Utils.BytesToHex(hash)) + "\r\n";
                }
                else if (ModuleIdentifier == 2 && VariableIdentifier > 4 || ModuleIdentifier == 1 && VariableIdentifier > 2)
                {
                    return string.Format(Properties.Resources.SetupViewerFileVar2, ModuleIdentifier.ToString(), VariableIdentifier.ToString(), ManageabilityStack.Utils.BytesToHex(VariableValue)) + "\r\n";
                }
                return string.Format(Properties.Resources.SetupViewerFileVar3, ModuleIdentifier.ToString(), VariableIdentifier.ToString(), UTF8Encoding.UTF8.GetString(VariableValue)) + "\r\n";
            }

            public byte[] GetBytes()
            {
                MemoryStream MS = new MemoryStream();
                MS.Write(BitConverter.GetBytes((ushort)ModuleIdentifier), 0, 2);
                MS.Write(BitConverter.GetBytes((ushort)VariableIdentifier), 0, 2);
                MS.Write(BitConverter.GetBytes((ushort)VariableLength), 0, 2);
                MS.Write(BitConverter.GetBytes((ushort)0), 0, 2);
                MS.Write(VariableValue, 0, VariableValue.Length);
                while (MS.Length % 4 != 0) MS.WriteByte(0);
                return MS.ToArray();
            }
        }

        private void detailedInformationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            splitContainer.Panel2Collapsed = !detailedInformationToolStripMenuItem.Checked;
        }

        private void mainListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateInfo();
            if (mainListView.SelectedItems.Count > 0 && mainListView.SelectedItems[0].Tag != null && mainListView.SelectedItems[0].Tag.GetType() == typeof(UsbFileRecord))
            {
                UsbFileRecord r = (UsbFileRecord)mainListView.SelectedItems[0].Tag;
                mainTextBox.Text = r.ToString();
            }
            else if (mainListView.SelectedItems.Count > 0 && mainListView.SelectedItems[0].Tag != null && mainListView.SelectedItems[0].Tag.GetType() == typeof(UsbFile))
            {
                UsbFile rr = (UsbFile)mainListView.SelectedItems[0].Tag;
                mainTextBox.Text = rr.ToString();
            }
            else
            {
                mainTextBox.Clear();
            }
        }

        private void UpdateInfo()
        {
            saveSelectedKeysToolStripMenuItem.Enabled = importToolStripMenuItem.Enabled = (mainListView.SelectedItems.Count > 0);
        }

        private void UsbSetupViewerForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            mainToolStripStatusLabel.Text = "";
            UpdateInfo();
        }

        private void importToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // First, lets ask about the security template
            if (mainListView.SelectedItems.Count == 0) return;
            AskSecurityTemplateForm f = new AskSecurityTemplateForm(mainForm);
            if (f.ShowDialog(this) != DialogResult.OK) return;
            SecurityTemplate t = f.SelectedTemplate;
            if (t == null) return;

            foreach (ListViewItem l in mainListView.SelectedItems)
            {
                if (l.Tag.GetType() == typeof(UsbFileRecord))
                {
                    UsbFileRecord r = (UsbFileRecord)l.Tag;
                    if (r.GetVariable(2, 1) != null && r.GetVariable(2, 2) != null)
                    {
                        string pwd = r.GetVariable(1, 2).GetValueStr();
                        string pid = r.GetVariable(2, 1).GetValueStr();
                        if (pid.Length != 8) break;
                        pid = pid.Substring(0, 4) + "-" + pid.Substring(4, 4);
                        string pps = r.GetVariable(2, 2).GetValueStr();
                        if (pps.Length != 32) break;
                        pps = pps.Substring(0, 4) + "-" + pps.Substring(4, 4) + "-" + pps.Substring(8, 4) + "-" + pps.Substring(12, 4) + "-" + pps.Substring(16, 4) + "-" + pps.Substring(20, 4) + "-" + pps.Substring(24, 4) + "-" + pps.Substring(28, 4);
                        AmtProvisioningKey.AmtProvisioningKeyState s = AmtProvisioningKey.AmtProvisioningKeyState.Pending;
                        if (r.RecordFlags == 0) s = AmtProvisioningKey.AmtProvisioningKeyState.Used;
                        AmtProvisioningKey key = new AmtProvisioningKey(pid, pps, s, DateTime.Now);
                        if (r.RecordFlags == 0)
                        {
                            key.CompletedRequests = 1;
                            key.HelloRequests = 1;
                        }
                        key.SecurityTemplate = t.Name;
                        key.AdminPassword = pwd;
                        mainForm.provisioningKeyStore.AddKey(key);
                    }
                }
            }
            mainForm.provisioningKeyStore.Flush();
            mainForm.mainTreeView_AfterSelect(this, null);
        }

        private void saveSelectedKeysToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainListView.SelectedItems.Count == 0) return;
            if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                ArrayList list = new ArrayList();
                foreach (ListViewItem l in mainListView.SelectedItems)
                {
                    if (l.Tag.GetType() == typeof(UsbFileRecord))
                    {
                        UsbFileRecord r = (UsbFileRecord)l.Tag;
                        list.Add(r);
                    }
                }

                if (list.Count > 0)
                {
                    UsbFile file = new UsbFile((UsbFileRecord[])list.ToArray(typeof(UsbFileRecord)), 1, 0);
                    byte[] usbFileBytes = file.GetBytes();
                    FileStream f;
                    try
                    {
                        f = new FileStream(saveFileDialog.FileName, FileMode.Create);
                    }
                    catch (Exception ex) { ManageabilityStack.Utils.FileErrorMessageBox(this, ex); return; }
                    f.Write(usbFileBytes, 0, usbFileBytes.Length);
                    f.Close();
                }
            }
        }

        private void mainListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData == (Keys.A | Keys.Control))
            {
                if (mainListView.Items.Count > 0)
                {
                    mainListView.SelectedIndices.Add(0);
                    mainListView.Focus();
                    SendKeys.Send("{HOME}");
                    SendKeys.Send("+{END}");
                }
            }
        }

        private void selectAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            mainListView.Focus();
            SendKeys.Send("{HOME}");
            SendKeys.Send("+{END}");
        }

        private void editToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            int count = 0;
            foreach (ListViewItem l in mainListView.SelectedItems) if (l.Tag.GetType() == typeof(UsbFileRecord)) count++;
            markSelectedAsUnusedToolStripMenuItem.Enabled = markSelectedAsUsedToolStripMenuItem.Enabled = (count > 0);
        }

        private void markSelectedAsUsedToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainListView.SelectedItems.Count == 0) return;
            foreach (ListViewItem l in mainListView.SelectedItems)
            {
                if (l.Tag.GetType() == typeof(UsbFileRecord))
                {
                    UsbFileRecord r = (UsbFileRecord)l.Tag;
                    r.RecordFlags = 0;
                    l.SubItems[1].Text = Properties.Resources.SetupViewerKeyUsed;
                }
            }

            if (mainListView.Items[0].Tag.GetType() == typeof(UsbFile))
            {
                UsbFile file = (UsbFile)mainListView.Items[0].Tag;
                file.DataRecordsConsumed = GetUsedKeyCount();
                mainToolStripStatusLabel.Text = String.Format(Properties.Resources.SetupViewerStatus, file.MajorVersion + "." + file.MinorVersion, file.DataRecordsConsumed, file.DataRecordCount);
            }
        }

        private void markSelectedAsUnusedToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainListView.SelectedItems.Count == 0) return;
            foreach (ListViewItem l in mainListView.SelectedItems)
            {
                if (l.Tag.GetType() == typeof(UsbFileRecord))
                {
                    UsbFileRecord r = (UsbFileRecord)l.Tag;
                    r.RecordFlags = 1;
                    l.SubItems[1].Text = Properties.Resources.SetupViewerKeyReady;
                }
            }

            if (mainListView.Items[0].Tag.GetType() == typeof(UsbFile))
            {
                UsbFile file = (UsbFile)mainListView.Items[0].Tag;
                file.DataRecordsConsumed = file.DataRecordCount - GetUsedKeyCount();
                mainToolStripStatusLabel.Text = String.Format(Properties.Resources.SetupViewerStatus, file.MajorVersion + "." + file.MinorVersion, file.DataRecordsConsumed, file.DataRecordCount);
            }
        }

        private uint GetUsedKeyCount()
        {
            uint count = 0;
            foreach (ListViewItem l in mainListView.Items)
            {
                if (l.Tag.GetType() == typeof(UsbFileRecord))
                {
                    UsbFileRecord r = (UsbFileRecord)l.Tag;
                    if (r.RecordFlags != 1) count++;
                }
            }
            return count;
        }

    }
}
