// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using Microsoft.Win32;
using System.Diagnostics;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityOutpostTool;
using ManageabilityStack;

namespace ManageabilityOutpostServiceControlPanelTool
{
    public partial class MainForm : Form
    {
        private static string Service = "\"Manageability Outpost Tool Service.exe\"";
        private CryptProtectClass cp = new CryptProtectClass(CryptProtectClass.cpStore.USE_MACHINE_STORE);
        private ManageabilityOutpostToolService.OutpostService debugService = null;
        private bool UpdatingUI = false;

        private enum uiStatus
        { 
            Installed = 0,
            Uninstalled,
            Started,
            Stopped,
            Connected,
            Disconnected
        }

        public MainForm()
        {
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            versionLabel.Text = versionLabel.Text.Replace("0.00", ManageabilityControls.AboutForm.VersionString);
            versionLabel.Text = ManageabilityStack.Utils.FilterStrEx(versionLabel.Text);
            this.Text = ManageabilityStack.Utils.FilterStrEx(this.Text);

            ServiceCtlClass.ServiceCtlStatus status = ServiceCtlClass.GetServiceStatus();

            if (ServiceCtlClass.ServiceCtlStatus.Invalid != status)
            {
                ChangeUIStatus(uiStatus.Installed);

                if (ServiceCtlClass.ServiceCtlStatus.Running == status)
                {
                    ChangeUIStatus(uiStatus.Started);

                    if (RegistryCtlClass.OutpostDataRegistry.GetValue
                        ("Connected", bool.FalseString).ToString().CompareTo(bool.TrueString) == 0)
                    {
                        ChangeUIStatus(uiStatus.Connected);
                    }
                }
            }
            else
            { }

            SyncWdFromRegistry();

            autoLaunchServiceCheckBox.Checked = (int)RegistryCtlClass.GetServiceKey().GetValue("Start", 3) == 2;
            autoConnectCheckBox.Checked = RegistryCtlClass.OutpostDataRegistry.GetValue
                ("ConnectAutomatic", bool.FalseString).ToString().CompareTo(bool.TrueString) == 0;
            useSerialAgentCheckBox.Checked = RegistryCtlClass.OutpostDataRegistry.GetValue
                ("SerialAgent", bool.FalseString).ToString().CompareTo(bool.TrueString) == 0;

            amtUsernameTextBox.Text = (string)RegistryCtlClass.OutpostDataRegistry.GetValue("Username", "admin");
            passwordControl.Text = "";

            byte[] bPassword = (byte[])RegistryCtlClass.OutpostDataRegistry.GetValue("PasswordBin", new byte[0],
                        RegistryValueOptions.DoNotExpandEnvironmentNames);
            if (0 != bPassword.Length)
            {
                passwordControl.Text = cp.ByteArrayToStr(cp.Decrypt(bPassword, null));
            }

            UpdatePermissionsListView();
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void installServiceButton_Click(object sender, EventArgs e)
        {
            ServiceCtlClass.ServiceCtlStatus s = ServiceCtlClass.GetServiceStatus();

            ServiceCtlClass.InstallService(Service, true);

            while (ServiceCtlClass.ServiceCtlStatus.Invalid == ServiceCtlClass.GetServiceStatus())
            {
                System.Threading.Thread.Sleep(1000);
            }

            ChangeUIStatus(uiStatus.Installed);
        }

        private void uninstallServiceButton_Click(object sender, EventArgs e)
        {
            ServiceCtlClass.InstallService(Service, false);

            while (ServiceCtlClass.ServiceCtlStatus.Invalid != ServiceCtlClass.GetServiceStatus())
            {
                System.Threading.Thread.Sleep(1000);
            }

            ChangeUIStatus(uiStatus.Uninstalled);
        }

        private void startServiceButton_Click(object sender, EventArgs e)
        {
            ServiceCtlClass.StartService();

            ChangeUIStatus(uiStatus.Started);

            if (RegistryCtlClass.OutpostDataRegistry.GetValue
                        ("Connected", bool.FalseString).ToString().CompareTo(bool.TrueString) == 0)
            {
                ChangeUIStatus(uiStatus.Connected);
            }
        }

        private void stopServiceButton_Click(object sender, EventArgs e)
        {
            ServiceCtlClass.StopService();

            ChangeUIStatus(uiStatus.Stopped);
        }

        private void connectButton_Click(object sender, EventArgs e)
        {
            if (passwordControl.Text.Length == 0) return;

            connectButton.Enabled = false;
            amtUsernameTextBox.Enabled = false;
            passwordControl.Enabled = false;
            passwordControl.Hidden = false;

            byte[] bPassword = cp.Encrypt(cp.StrToByteArray(passwordControl.Text), null);
            RegistryCtlClass.OutpostDataRegistry.SetValue("PasswordBin", bPassword, RegistryValueKind.Binary);
            RegistryCtlClass.OutpostDataRegistry.SetValue("Username", amtUsernameTextBox.Text);

            SendServiceCommand(ServiceCtlClass.CC_AMTLOCAL_CONNECT);

            if (!RegistryCtlClass.WaitForStringValue("Connected", bool.TrueString, 30))
            {
                MessageBox.Show(this, Properties.Resources.LocalAmtConnectionFailed, Properties.Resources.ConnectionErrorTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                ChangeUIStatus(uiStatus.Disconnected);
                return;
            }

            passwordControl.Hidden = true;

            ChangeUIStatus(uiStatus.Connected);
        }

        private void disconnectButton_Click(object sender, EventArgs e)
        {
            SendServiceCommand(ServiceCtlClass.CC_AMTLOCAL_DISCONNECT);

            if (!RegistryCtlClass.WaitForStringValue("Connected", bool.FalseString, -1))
            {
                MessageBox.Show(this, Properties.Resources.LocalAmtDisconnectFailed, Properties.Resources.ConnectionErrorTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            ChangeUIStatus(uiStatus.Disconnected);
        }

        private void ChangeUIStatus(uiStatus status)
        {
            switch (status)
            { 
                case uiStatus.Installed:
                    installServiceButton.Enabled = false;
                    uninstallServiceButton.Enabled = true;

                    installStateLabel.Text = Properties.Resources.StatusInstalled;

                    autoLaunchServiceCheckBox.Checked = true;

                    ServiceStategroupBox.Enabled = true;
                    break;
                case uiStatus.Uninstalled:
                    installServiceButton.Enabled = true;
                    uninstallServiceButton.Enabled = false;

                    installStateLabel.Text = Properties.Resources.StatusNotInstalled;

                    autoLaunchServiceCheckBox.Checked = false;

                    ServiceStategroupBox.Enabled = false;
                    ServiceConfiggroupBox.Enabled = false;
                    break;
                case uiStatus.Started:
                    startServiceButton.Enabled = false;
                    stopServiceButton.Enabled = true;

                    serviceStateLabel.Text = Properties.Resources.StatusRunning;

                    ServiceConfiggroupBox.Enabled = true;
                    break;
                case uiStatus.Stopped:
                    startServiceButton.Enabled = true;
                    stopServiceButton.Enabled = false;

                    serviceStateLabel.Text = Properties.Resources.StatusNotRunning;

                    ServiceConfiggroupBox.Enabled = false;
                    break;
                case uiStatus.Connected:
                    connectButton.Enabled = false;
                    disconnectButton.Enabled = true;

                    amtUsernameTextBox.Enabled = false;
                    passwordControl.Enabled = false;

                    connectStateLabel.Text = Properties.Resources.StatusConnected;
                    break;
                case uiStatus.Disconnected:
                    connectButton.Enabled = true;
                    disconnectButton.Enabled = false;

                    amtUsernameTextBox.Enabled = true;
                    passwordControl.Enabled = true;

                    connectStateLabel.Text = Properties.Resources.StatusNotConnected;
                    break;
            }
        }

        private void autoLaunchServiceCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (ServiceCtlClass.ServiceCtlStatus.Invalid != ServiceCtlClass.GetServiceStatus())
            {
                // Start = 2 mean Auto, 3 mean Mamual, 4 mean Disable
                if (autoLaunchServiceCheckBox.Checked)
                {
                    RegistryCtlClass.GetServiceKey().SetValue("Start", 2, RegistryValueKind.DWord);
                }
                else
                {
                    RegistryCtlClass.GetServiceKey().SetValue("Start", 3, RegistryValueKind.DWord);
                }
            }
        }

        private void autoConnectCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            RegistryCtlClass.OutpostDataRegistry.SetValue("ConnectAutomatic", autoConnectCheckBox.Checked.ToString());
        }

        private void useSerialAgentCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (useSerialAgentCheckBox.Checked)
            {
                SendServiceCommand(ServiceCtlClass.CC_SERIALAGENT_ENABLE);
                if (!RegistryCtlClass.WaitForStringValue("SerialAgent", bool.TrueString, -1))
                {
                    MessageBox.Show(this, Properties.Resources.EnableSerialAgentFailed, Properties.Resources.SerialAgentErrorTitle, 
                        MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    return;
                }
            }
            else
            {
                SendServiceCommand(ServiceCtlClass.CC_SERIALAGENT_DISABLE);
                if (!RegistryCtlClass.WaitForStringValue("SerialAgent", bool.FalseString, -1))
                {
                    MessageBox.Show(this, Properties.Resources.DisableSerialAgentFailed, Properties.Resources.SerialAgentErrorTitle, 
                        MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    return;
                }
            }
        }

        private void UpdatePermissionsListView()
        {
            for (int i = 0; i < (int)SerialAgentPermissionCtl.Permissions.MaxDummy; i++)
            {
                string[] tmp = new string[] { SerialAgentPermissionCtl.PermissionDesc[i], 
                    SerialAgentPermissionCtl.PermissionStatus[SerialAgentPermissionCtl.GetPermission((SerialAgentPermissionCtl.Permissions)i)]};
                serialAgentPermissionsListView.Items.Insert(i, new ListViewItem(tmp));
            }
        }

        private void SyncWdToRegistry()
        {
            if (UpdatingUI == true) return;
            string data = "";
            foreach (ListViewItem l in watchdogListView.Items)
            {
                if (l != null)
                {
                    AmtWatchdogLocal.AmtWatchdog wg = (AmtWatchdogLocal.AmtWatchdog)l.Tag;
                    string active = "0";
                    if (l.Checked == true) active = "1";
                    data += wg.agentID.ToString() + "~" + wg.name + "~" + wg.processCondition + "~" + active + "~";
                }
            }
            RegistryCtlClass.OutpostDataRegistry.SetValue("WatchdogsEx", data);
        }

        private void SyncWdFromRegistry()
        {
            string data = (string)RegistryCtlClass.OutpostDataRegistry.GetValue("WatchdogsEx", "");
            if (data.Length < 3) return;
            string[] datas = data.Split('~');

            watchdogListView.BeginUpdate();
            watchdogListView.Items.Clear();
            UpdatingUI = true;
            for (int i = 0; i < (datas.Length - 1); i += 4)
            {
                try
                {
                    string active = datas[i + 3];
                    string condition = datas[i + 2];
                    string name = datas[i + 1];
                    Guid ids = new Guid(datas[i]);

                    AmtWatchdogLocal.AmtWatchdog wg = new AmtWatchdogLocal.AmtWatchdog(name, ids);
                    if (condition.Length > 0) wg.processCondition = condition;

                    string s = Properties.Resources.None;
                    if (condition.Length > 0) s = condition;
                    ListViewItem lvI = new ListViewItem(new String[] { name, ids.ToString(), s });
                    lvI.Tag = wg;
                    lvI.Checked = active.Equals("1");
                    watchdogListView.Items.Add(lvI);
                }
                catch (Exception) { }
            }
            UpdatingUI = false;
            watchdogListView.EndUpdate();
        }

        private void addWatchdogButton_Click(object sender, EventArgs e)
        {
            AddWatchdogForm f = new AddWatchdogForm();
            if (f.ShowDialog(this) == DialogResult.OK)
            {
                AmtWatchdogLocal.AmtWatchdog wg = new AmtWatchdogLocal.AmtWatchdog(f.AgentName, f.AgentID);
                wg.processCondition = f.AgentProcessCondition;

                string str = Properties.Resources.None;
                if (f.AgentProcessCondition != null)
                {
                    str = f.AgentProcessCondition;
                }
                ListViewItem lvI = new ListViewItem(new String[] { f.AgentName, f.AgentID.ToString(), str});
                lvI.Checked = true;
                lvI.Tag = wg;

                watchdogListView.Items.Add(lvI);
                SyncWdToRegistry();
                SendServiceCommand(ServiceCtlClass.CC_WATCHDOG_ADD);
            }
        }

        private void editWatchdogButton_Click(object sender, EventArgs e)
        {
            if (watchdogListView.SelectedItems.Count != 1) return;
            ListViewItem l = watchdogListView.SelectedItems[0];
            AmtWatchdogLocal.AmtWatchdog wg = (AmtWatchdogLocal.AmtWatchdog)l.Tag;
            AddWatchdogForm f = new AddWatchdogForm(wg);
            if (f.ShowDialog(this) == DialogResult.OK)
            {
                wg.agentID = f.AgentID;
                wg.name = f.AgentName;
                wg.processCondition = f.AgentProcessCondition;
                l.SubItems[0].Text = f.AgentName;
                l.SubItems[1].Text = f.AgentID.ToString();

                string str = Properties.Resources.None;
                if (f.AgentProcessCondition != null)
                {
                    str = f.AgentProcessCondition;
                }
                l.SubItems[2].Text = str;

                l.Tag = wg;
                SyncWdToRegistry();
                SendServiceCommand(ServiceCtlClass.CC_WATCHDOG_EDIT);
            }
        }

        private void removeWatchdogButton_Click(object sender, EventArgs e)
        {
            if (watchdogListView.SelectedItems.Count != 1) return;
            if (MessageBox.Show(this, Properties.Resources.RemoveSelectedWatchdogQ, Properties.Resources.Watchdog, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                ListViewItem l = watchdogListView.SelectedItems[0];
                watchdogListView.Items.Remove(l);
                SyncWdToRegistry();
                SendServiceCommand(ServiceCtlClass.CC_WATCHDOG_REMOVE);
            }
        }

        private void aboutButton_Click(object sender, EventArgs e)
        {
            new ManageabilityControls.AboutForm(ManageabilityOutpostServiceControlPanelTool.Properties.Resources.ProductName, ManageabilityOutpostServiceControlPanelTool.Properties.Resources.ProductSubName).ShowDialog(this);
        }

        private void PermissionListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            denyButton.Enabled = allowButton.Enabled = (serialAgentPermissionsListView.SelectedItems.Count > 0);
        }

        private void allowButton_Click(object sender, EventArgs e)
        {
            foreach (ListViewItem l in serialAgentPermissionsListView.SelectedItems)
            {
                SerialAgentPermissionCtl.SetPermission((SerialAgentPermissionCtl.Permissions)l.Index, 1);
                l.SubItems[1].Text = Properties.Resources.Allow;
            }
            SendServiceCommand(ServiceCtlClass.CC_PERMISSION_CHANGED);
            serialAgentPermissionsListView.SelectedItems.Clear();
        }

        private void denyButton_Click(object sender, EventArgs e)
        {
            foreach (ListViewItem l in serialAgentPermissionsListView.SelectedItems)
            {
                SerialAgentPermissionCtl.SetPermission((SerialAgentPermissionCtl.Permissions)l.Index, 0);
                l.SubItems[1].Text = Properties.Resources.Deny;
            }
            SendServiceCommand(ServiceCtlClass.CC_PERMISSION_CHANGED);
            serialAgentPermissionsListView.SelectedItems.Clear();
        }

        private void watchdogListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            editWatchdogButton.Enabled = (watchdogListView.SelectedItems.Count > 0);
            removeWatchdogButton.Enabled = (watchdogListView.SelectedItems.Count > 0);
        }

        private void startDebugOutpostButton_Click(object sender, EventArgs e)
        {
            if (debugService == null)
            {
                debugService = new ManageabilityOutpostToolService.OutpostService();
                debugService.DebugStart(null);
                startDebugOutpostButton.Text = Properties.Resources.StopDebugOutpost;
            }
            else
            {
                startDebugOutpostButton.Text = Properties.Resources.StartDebugOutpost;
                debugService.DebugStop();
                debugService.DebugShutdown();
                debugService = null;
            }
        }

        public void SendServiceCommand(int command)
        {
            if (debugService != null)
            {
                debugService.DebugCustomCommand(command);
            }
            else
            {
                ServiceCtlClass.SendCommand(command);
            }
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (debugService != null) startDebugOutpostButton_Click(null, null);
        }

        private void watchdogListView_ItemChecked(object sender, ItemCheckedEventArgs e)
        {
            if (UpdatingUI == true) return;
            SyncWdToRegistry();
            SendServiceCommand(ServiceCtlClass.CC_WATCHDOG_EDIT);
        }

        private void watchdogListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete) removeWatchdogButton_Click(this, null);
        }

        private void wgRefreshButton_Click(object sender, EventArgs e)
        {
            SyncWdFromRegistry();
        }

    }
}
