// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Net;
using System.Text;
using System.Threading;
using System.Diagnostics;
using System.Collections;
using System.Web.Services;
using System.Collections.Generic;
using System.Security.Cryptography.X509Certificates;
using ManageabilityStack;

namespace ManageabilityOutpostTool
{
    public class AmtNetworkAccessControl
    {
        private AmtSystem system;
        private EndpointAccessControlService EAC;

        public AmtNetworkAccessControl(AmtSystem system, EndpointAccessControlService service)
        {
            this.system = system;
            EAC = service;
        }

        public void Dispose()
        {
            EAC.Dispose();
            EAC = null;
        }

        public void ClearCache()
        {
            GetEACStatusCached = false;
            GetPostureCache = null;
        }

        public void FetchCache()
        {
            bool enabled;
            GetEACStatus(out enabled);
            byte[] b1, b2;
            GetPosture(out b1, out b2);
        }

        private bool GetEACStatusCache = false;
        private bool GetEACStatusCached = false;
        public AmtCallStatus GetEACStatus(out bool enabled)
        {
            if (GetEACStatusCached == true)
            {
                enabled = GetEACStatusCache;
                return AmtCallStatus.SUCCESS;
            }
            AmtCallStatus r = (AmtCallStatus)EAC.GetEACStatus(out enabled);
            if (r == AmtCallStatus.SUCCESS)
            {
                GetEACStatusCache = enabled;
                GetEACStatusCached = true;
            }
            return r;
        }

        private object[] GetPostureCache = null;
        public AmtCallStatus GetPosture(out byte[] signedPosture, out byte[] postureChangeHash)
        {
            if (GetPostureCache != null)
            {
                signedPosture = (byte[])GetPostureCache[0];
                postureChangeHash = (byte[])GetPostureCache[1];
                return AmtCallStatus.SUCCESS;
            }
            AmtCallStatus r = (AmtCallStatus)EAC.GetPosture(PostureTypeType.PostureAvpTypeFull, out signedPosture, out postureChangeHash);
            if (r == AmtCallStatus.SUCCESS) GetPostureCache = new object[] { signedPosture, postureChangeHash };
            return r;
        }

        private byte[] GetPostureHashCache = null;
        public AmtCallStatus GetPostureHash(out byte[] postureChangeHash)
        {
            if (GetPostureHashCache != null)
            {
                postureChangeHash = GetPostureHashCache;
                return AmtCallStatus.SUCCESS;
            }
            AmtCallStatus r = (AmtCallStatus)EAC.GetPostureHash(PostureTypeType.PostureAvpTypeFull, out postureChangeHash);
            if (r == AmtCallStatus.SUCCESS) GetPostureHashCache = postureChangeHash;
            return r;
        }

    }
}
