// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Net;
using System.Text;
using System.Threading;
using System.Net.Sockets;
using System.Collections;
using System.Diagnostics;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityOutpostTool
{
    public class AmtSerialAgent
    {
        public SerialHighDriver serial;
        private Hashtable tcpList = new Hashtable();
        private Hashtable tcpBufList = new Hashtable();
        private ArrayList tcpHalted = new ArrayList();
        private string uploadFileName = null;
        private Thread flowControlThread;
        private ArrayList commandHistory = new ArrayList();
        private int commandHistoryPtr = 0;
        public string VersionString = "Manageability Outpost Tool";
        public string lastcmd = "";
        public AmtSystem system = null;
        public ChatWindowForm chatForm = null;
        public ArrayList chatStrings = new ArrayList();
        public TextChatClass chatTextApplication = null;
        public TextTopClass chatTextTopClass = null;
        //private RemoteConsole RConsole = null;

        public delegate void ChangedWatchdogStateHandler();
        public event ChangedWatchdogStateHandler ChangedWatchdogState;

        public delegate void SerialOutputHandler(AmtSerialAgent sender, byte[] buf, int offset, int length);
        public event SerialOutputHandler SerialOutputEvent;

        public enum Permission
        {
            Allow,
            Notify,
            Deny
        }

        public AmtSerialAgent(string port, string versionStr)
        {
            VersionString = versionStr;

            serial = new SerialHighDriver(port);

            serial.ProcessCharEvent += new SerialHighDriver.ProcessCharHandler(ProcessCharHandler);
            serial.ProcessDataBlockEvent += new SerialHighDriver.ProcessDataBlockHandler(ProcessDataBlockHandler);
            serial.ProcessDataBlockProgressEvent += new SerialHighDriver.ProcessDataBlockProgressHandler(ProcessDataBlockProgressHandler);
            serial.ProcessEscapeEvent += new SerialHighDriver.ProcessEscapeHandler(ProcessEscapeHandler);
            serial.SerialOutputEvent += new SerialHighDriver.SerialOutputHandler(SerialOutputSink);

            // Send serial intro
            SendIntro();

            flowControlThread = new Thread(new ThreadStart(FlowControlThread));
            flowControlThread.Start();

            chatStrings.Clear();

            commandHistory.Add("");
        }

        public void Dispose()
        {
            if (serial != null)
            {
                serial.ProcessCharEvent -= new SerialHighDriver.ProcessCharHandler(ProcessCharHandler);
                serial.ProcessDataBlockEvent -= new SerialHighDriver.ProcessDataBlockHandler(ProcessDataBlockHandler);
                serial.ProcessDataBlockProgressEvent -= new SerialHighDriver.ProcessDataBlockProgressHandler(ProcessDataBlockProgressHandler);
                serial.ProcessEscapeEvent -= new SerialHighDriver.ProcessEscapeHandler(ProcessEscapeHandler);
                serial.SerialOutputEvent -= new SerialHighDriver.SerialOutputHandler(SerialOutputSink);

                serial.Write("\r\n\r\n *** Manageability Serial Agent Shutdown - " + DateTime.Now.ToString() + " ***\r\n\r\n");
                serial.Dispose();
                serial = null;
            }
        }

        private void SerialOutputSink(SerialHighDriver sender, byte[] buf, int offset, int length)
        {
            if (SerialOutputEvent != null) SerialOutputEvent(this, buf, offset, length);
        }

        private string accumulator = "";
        public void ProcessCharHandler(SerialHighDriver sender, byte c)
        {
            if (chatTextApplication != null)
            {
                chatTextApplication.ProcessCharHandler(c);
                return;
            }

            if (chatTextTopClass != null)
            {
                chatTextTopClass.ProcessCharHandler(c);
                return;
            }

            switch (c)
            {
                case 8: // BACKSPACE
                    if (accumulator.Length > 0)
                    {
                        accumulator = accumulator.Substring(0, accumulator.Length - 1);
                        serial.Write(c);
                    }
                    break;
                case 13: // RETURN
                    ProcessCommand(accumulator.Trim());
                    accumulator = "";
                    break;
                case (byte)'`':
                    SendIntro();
                    break;
                default:
                    if (c >= 32)
                    {
                        serial.Write(c);
                        accumulator += (char)c;
                    }
                    break;
            }
        }

        public void ProcessEscapeHandler(SerialHighDriver sender, byte code, int[] args)
        {
            ProcessEscapeHandler(sender, (SerialHighDriver.EscCode)code, args);
        }

        public void ProcessEscapeHandler(SerialHighDriver sender, SerialHighDriver.EscCode code, int[] args)
        {
            if (chatTextApplication != null)
            {
                chatTextApplication.ProcessEscapeHandler(code, args);
                return;
            }

            if (chatTextTopClass != null)
            {
                chatTextTopClass.ProcessEscapeHandler(code, args);
                return;
            }

            if (code == SerialHighDriver.EscCode.KeyUp)
            {
                if (commandHistoryPtr < (commandHistory.Count-1))
                {
                    commandHistoryPtr++;
                    string s = "";
                    for (int i = 0; i < accumulator.Length; i++) {s += (char)8;}
                    s += commandHistory[commandHistoryPtr];
                    accumulator = (string)commandHistory[commandHistoryPtr];
                    serial.Write(s);
                }
            }
            else if (code == SerialHighDriver.EscCode.KeyDown)
            {
                if (commandHistoryPtr > 0)
                {
                    commandHistoryPtr--;
                    string s = "";
                    for (int i = 0; i < accumulator.Length; i++) { s += (char)8; }
                    s += commandHistory[commandHistoryPtr];
                    accumulator = (string)commandHistory[commandHistoryPtr];
                    serial.Write(s);
                }
            }
        }

        public void ProcessDataBlockProgressHandler(SerialHighDriver sender, string name, int total, int current)
        {

        }

        public void ProcessDataBlockHandler(SerialHighDriver sender, string name, byte[] data)
        {
            if (name.Equals("INFO") == true)
            {
                serial.WriteDataBlock("INFO", UTF8Encoding.UTF8.GetBytes(VersionString + ",TCPR,WMIR,PROC,DEVC,WATG,FILE"));
                return;
            }

            if (name.Equals("CHAT") == true && data.Length > 0)
            {
                string text = UTF8Encoding.UTF8.GetString(data);
                if (data[0] == 'C') AddChatString("R> " + text.Substring(1));
                if (data[0] == 'T') chatForm.RemoteTyping(true);
                if (data[0] == 'N') chatForm.RemoteTyping(false);
                if (data[0] == 'L') ClearChatStrings();
                return;
            }
            
            if (name.Length > 4 && name.Substring(0, 4).CompareTo("WMIQ") == 0)
            {
                int i = name.IndexOf('|');
                if (i > 0)
                {
                    string query = name.Substring(i + 1);
                    ManageabilityControls.ManagementData md = ManageabilityControls.ManagementData.GetWmiQuery(query);
                    if (md == null) md = new ManageabilityControls.ManagementData(query, DateTime.MinValue, new Hashtable[0]);
                    serial.WriteDataBlock("WMIQ", md.ToBinary());
                }
            }

            if (name.Length == 4 && name.Substring(0, 4).CompareTo("WGRQ") == 0)
            {
                if (system != null)
                {
                    AmtWatchdogLocal.AmtWatchdog[] wgs = system.WatchdogLocal.GetWatchdogs();
                    StringBuilder d = new StringBuilder();
                    foreach (AmtWatchdogLocal.AmtWatchdog wg in wgs)
                    {
                        string condition = "";
                        if (wg.processCondition != null) condition = wg.processCondition;
                        string active = "0";
                        if (wg.beat) active = "1";
                        d.AppendFormat("{0}~{1}~{2}~{3}~{4}~", wg.agentID.ToString(), wg.name, condition, active, wg.lastState.ToString());
                    }
                    serial.WriteDataBlock("WGRS", UTF8Encoding.UTF8.GetBytes(d.ToString()));
                }
                else
                {
                    serial.WriteDataBlock("WGRS", UTF8Encoding.UTF8.GetBytes("X"));
                }
            }

            if (name.Length == 4 && name.Substring(0, 4).CompareTo("WGRM") == 0)
            {
                system.WatchdogLocal.RemoveWatchdog(new Guid(data));
                if (ChangedWatchdogState != null) ChangedWatchdogState();
            }

            if (name.Length == 4 && name.Substring(0, 4).CompareTo("WGAD") == 0)
            {
                string str = UTF8Encoding.UTF8.GetString(data);
                if (str.Length < 3) return;
                string[] datas = str.Split('~');
                for (int i = 0; i < (datas.Length - 1); i += 4)
                {
                    try
                    {
                        bool active = datas[i + 3].Equals("1");;
                        string condition = datas[i + 2];
                        if (condition.Length == 0) condition = null;
                        string wgname = datas[i + 1];
                        Guid id = new Guid(datas[i]);
                        AmtWatchdogLocal.AmtWatchdog wg = system.WatchdogLocal.GetWatchdog(id);
                        if (wg == null)
                        {
                            wg = new AmtWatchdogLocal.AmtWatchdog(wgname, id);
                        }
                        wg.beat = active;
                        wg.processCondition = condition;
                        system.WatchdogLocal.AddWatchdog(wg);
                    }
                    catch (Exception) { }
                }
                if (ChangedWatchdogState != null) ChangedWatchdogState();
            }

            if (name.Length == 4 && name.Substring(0, 4).CompareTo("DRIV") == 0)
            {
                if (SerialAgentPermissionCtl.GetPermission(SerialAgentPermissionCtl.Permissions.Filebrowse)
                    == SerialAgentPermissionCtl.SAP_PERMISSION_DENY)
                {
                    serial.WriteDataBlock("DENY", null);
                    return;
                }

                string d = "";

                string[] astrDrives = Directory.GetLogicalDrives();
                foreach (string str in astrDrives)
                {
                    d += str + "|";
                }
                serial.WriteDataBlock("DRIV", UTF8Encoding.UTF8.GetBytes(d));
            }

            if (name.Length == 4 && name.Substring(0, 4).CompareTo("FLLT") == 0)
            {
                if (SerialAgentPermissionCtl.GetPermission(SerialAgentPermissionCtl.Permissions.Filebrowse)
                    == SerialAgentPermissionCtl.SAP_PERMISSION_DENY)
                {
                    serial.WriteDataBlock("DENY", null);
                    return;
                }

                StringBuilder s = new StringBuilder();
                string strPath = UTF8Encoding.UTF8.GetString(data);

                // Get all the files in the current directories
                DirectoryInfo curDir = new DirectoryInfo(strPath);
                FileInfo[] dirFiles;
                try
                {
                    dirFiles = curDir.GetFiles();
                    // Data created and data modified time of the file
                    foreach (FileInfo fileInfo in dirFiles)
                    {
                        string strFileName = fileInfo.Name;
                        s.Append(fileInfo.FullName.ToString() + "|");
                        s.Append(fileInfo.Name.ToString() + "|");
                        s.Append(fileInfo.Length.ToString() + "|");
                        s.Append(fileInfo.CreationTime.ToString() + "|");
                        s.Append(fileInfo.LastAccessTime.ToString() + "|");
                    }

                    // Insert directory into details,get subdirectories for the current directory
                    DirectoryInfo Dir = new DirectoryInfo(strPath);
                    foreach (DirectoryInfo di in Dir.GetDirectories())
                    {
                        s.Append(di.FullName.ToString() + "|");
                        s.Append(di.Name.ToString() + "|");
                        s.Append("-1" + "|");
                        s.Append(di.CreationTime.ToString() + "|");
                        s.Append(di.LastAccessTime.ToString() + "|");
                    }
                }
                catch
                {
                    serial.WriteDataBlock("EROR", null);
                    return;
                }
                serial.WriteDataBlock("FLLT", UTF8Encoding.UTF8.GetBytes(s.ToString()));
            }

            if (name.Length == 4 && name.Substring(0, 4).CompareTo("DEVC") == 0)
            {
                if (SerialAgentPermissionCtl.GetPermission(SerialAgentPermissionCtl.Permissions.Devicemanage)
                    == SerialAgentPermissionCtl.SAP_PERMISSION_DENY)
                {
                    serial.WriteDataBlock("DENY", null);
                    return;
                }

                StringBuilder s = new StringBuilder(Environment.MachineName + "\r");

                Guid[] GuidList = new Guid[128];
                string[] ClassesNameList = new string[128];
                int ClsReqSize = 0;

                if (!DevDriCtlClass.EnumerateClasses(null, 0, ref GuidList, ref ClassesNameList, 128, ref ClsReqSize))
                {
                    return;
                }

                DevDriCtlClass.SP_DEVINFOSTATUS_DATA[] DeviceInfoStatusList = new DevDriCtlClass.SP_DEVINFOSTATUS_DATA[256];

                for (int idc = 0; idc < ClsReqSize; idc++)
                {
                    // We don't want to show LegacyDriver
                    if (ClassesNameList[idc].Equals("LegacyDriver"))
                    {
                        continue;
                    }

                    int DevReqSize = 0;
                    if (!DevDriCtlClass.EnumerateDevices(null, ClassesNameList[idc], DevDriCtlClass.DIGCF_PRESENT,
                        ref DeviceInfoStatusList, 256, ref DevReqSize))
                    {
                        continue;
                    }

                    if (0 != DevReqSize)
                    {
                        string ClassDesc = ClassesNameList[idc];
                        if (!DevDriCtlClass.GetClassDescriptionEx(ref GuidList[idc], ref ClassDesc, null))
                        {
                            continue;
                        }

                        s.Append(DevReqSize.ToString() + "|");
                        s.Append(ClassDesc + "|");
                        s.Append(GuidList[idc].ToString() + "|");
                        s.Append(ClassesNameList[idc] + "|");

                        for (int idd = 0; idd < DevReqSize; idd++)
                        {
                            s.Append(idd.ToString() + "|");
                            s.Append(DeviceInfoStatusList[idd].pulStatus.ToString() + "|");
                            s.Append(DeviceInfoStatusList[idd].pulProblemNumber.ToString() + "|");
                            if (idd != DevReqSize - 1)
                            {
                                s.Append(DeviceInfoStatusList[idd].DeviceName + "|");
                            }
                            else
                            {
                                s.Append(DeviceInfoStatusList[idd].DeviceName + "\r");
                            }
                        }
                    }
                }
                serial.WriteDataBlock("DEVC", UTF8Encoding.UTF8.GetBytes(s.ToString()));
            }

            if (name.Length == 4 && name.Substring(0, 4).CompareTo("HWRS") == 0)
            {
                if (SerialAgentPermissionCtl.GetPermission(SerialAgentPermissionCtl.Permissions.Devicemanage)
                    == SerialAgentPermissionCtl.SAP_PERMISSION_DENY)
                {
                    serial.WriteDataBlock("DENY", null);
                    return;
                }

                DevDriCtlClass.cmdHWRescan();
                serial.WriteDataBlock("UPDT", null);
            }

            if (name.Length == 4 && name.Substring(0, 4).CompareTo("RMVD") == 0)
            {
                if (SerialAgentPermissionCtl.GetPermission(SerialAgentPermissionCtl.Permissions.Devicemanage)
                    == SerialAgentPermissionCtl.SAP_PERMISSION_DENY)
                {
                    serial.WriteDataBlock("DENY", null);
                    return;
                }

                string s = UTF8Encoding.UTF8.GetString(data);
                string[] parm = s.Split('|');
                Guid clsGuid = new Guid();
                clsGuid = (Guid)TypeDescriptor.GetConverter(clsGuid).ConvertFrom(parm[0]);
                int devidx = -1;
                try
                {
                    devidx = int.Parse(parm[1]);
                }
                catch (FormatException) { }
                if (devidx > 0)
                {
                    DevDriCtlClass.DeviceRemove(clsGuid, devidx);
                    serial.WriteDataBlock("UPDT", null);
                }
            }

            if (name.Length == 4 && name.Substring(0, 4).CompareTo("ENAB") == 0)
            {
                if (SerialAgentPermissionCtl.GetPermission(SerialAgentPermissionCtl.Permissions.Devicemanage)
                    == SerialAgentPermissionCtl.SAP_PERMISSION_DENY)
                {
                    serial.WriteDataBlock("DENY", null);
                    return;
                }

                string s = UTF8Encoding.UTF8.GetString(data);
                string[] parm = s.Split('|');
                Guid clsGuid = new Guid();
                clsGuid = (Guid)TypeDescriptor.GetConverter(clsGuid).ConvertFrom(parm[0]);
                int devidx = -1;
                try
                {
                    devidx = int.Parse(parm[1]);
                }
                catch (FormatException) { }
                if (devidx > 0)
                {
                    DevDriCtlClass.DeviceControl(clsGuid, devidx, DevDriCtlClass.DICS_ENABLE);
                    serial.WriteDataBlock("UPDT", null);
                }
            }

            if (name.Length == 4 && name.Substring(0, 4).CompareTo("DSAB") == 0)
            {
                if (SerialAgentPermissionCtl.GetPermission(SerialAgentPermissionCtl.Permissions.Devicemanage)
                    == SerialAgentPermissionCtl.SAP_PERMISSION_DENY)
                {
                    serial.WriteDataBlock("DENY", null);
                    return;
                }

                string s = UTF8Encoding.UTF8.GetString(data);
                string[] parm = s.Split('|');
                Guid clsGuid = new Guid();
                clsGuid = (Guid)TypeDescriptor.GetConverter(clsGuid).ConvertFrom(parm[0]);
                int devidx = -1;
                try
                {
                    devidx = int.Parse(parm[1]);
                }
                catch (FormatException) { }
                if (devidx > 0)
                {
                    DevDriCtlClass.DeviceControl(clsGuid, devidx, DevDriCtlClass.DICS_DISABLE);
                    serial.WriteDataBlock("UPDT", null);
                }
            }

            if (name.Length == 4 && name.Substring(0, 4).CompareTo("PROC") == 0)
            {
                // Get the process table
                Process[] procList = Process.GetProcesses();
                StringBuilder s = new StringBuilder();
                foreach (Process p in procList)
                {
                    try
                    {
                        s.Append(p.Id.ToString() + "|");
                        s.Append(p.ProcessName + "|");
                        s.Append(p.BasePriority.ToString() + "|");
                        s.Append(p.HandleCount.ToString() + "|");
                        s.Append(p.MainWindowTitle + "|");
                        s.Append(p.NonpagedSystemMemorySize64.ToString() + "|");
                        s.Append(p.PagedMemorySize64.ToString() + "|");
                        s.Append(p.PagedSystemMemorySize64.ToString() + "|");
                        s.Append(p.Responding.ToString() + "|");
                        s.Append(p.Threads.Count.ToString() + "|");
                        s.Append(p.WorkingSet64.ToString() + "\r");
                    }
                    catch (Exception) { }
                }
                serial.WriteDataBlock("PROC", UTF8Encoding.UTF8.GetBytes(s.ToString()));
            }

            if (name.Length == 4 && name.Substring(0, 4).CompareTo("KILL") == 0)
            {
                if (SerialAgentPermissionCtl.GetPermission(SerialAgentPermissionCtl.Permissions.Stop)
                    == SerialAgentPermissionCtl.SAP_PERMISSION_DENY)
                {
                    serial.WriteDataBlock("DENY", null);
                    return;
                }

                int id = -1;
                try
                {
                    id = int.Parse(UTF8Encoding.UTF8.GetString(data));
                }
                catch (FormatException) { }
                if (id > 0) {Process.GetProcessById(id).Kill();}
            }

            if (name.Length == 4 && name.Substring(0, 4).CompareTo("EXEC") == 0)
            {
                if (SerialAgentPermissionCtl.GetPermission(SerialAgentPermissionCtl.Permissions.Execute)
                    == SerialAgentPermissionCtl.SAP_PERMISSION_DENY)
                {
                    serial.WriteDataBlock("DENY", null);
                    return;
                }

                string exec = UTF8Encoding.UTF8.GetString(data);
                System.Diagnostics.Process p = null; ;
                try
                {
                    string args = null;
                    if (exec.IndexOf(" ") > 0)
                    {
                        args = exec.Substring(exec.IndexOf(" ") + 1);
                        exec = exec.Substring(0, exec.IndexOf(" "));
                    }
                    p = System.Diagnostics.Process.Start(exec, args);
                }
                catch (System.ComponentModel.Win32Exception) {}
            }

            if (name.Length > 5 && name.Substring(0, 5).CompareTo("FILE|") == 0)
            {
                // Uploaded File
                FileStream f = new FileStream(uploadFileName, FileMode.Create);
                f.Write(data, 0, data.Length);
                f.Close();
            }

            if (name.Length < 4) return;
            if (name.Substring(0, 3).CompareTo("TCP") != 0) return;

            if (name.Substring(0, 5).CompareTo("TCPC|") == 0)
            {
                // Handle a new connection
                string addr = name.Substring(5);
                int i = addr.IndexOf('|');
                string idstr = addr.Substring(i + 1);
                int id;
                try
                {
                    id = int.Parse(idstr);
                }
                catch (FormatException) { return; }
                addr = addr.Substring(0, i);
                i = addr.IndexOf(':');
                string portstr = addr.Substring(i + 1);
                addr = addr.Substring(0, i);

                if (tcpList.ContainsKey(id)) { CloseTcpClient(id); }

                TcpClient client = new TcpClient();
                byte[] buf = new byte[1024];
                client.BeginConnect(addr, int.Parse(portstr), new AsyncCallback(TcpConnectEvent), id);

                tcpList.Add(id, client);
                tcpBufList.Add(id, buf);
            }

            if (name.Substring(0, 5).CompareTo("TCPD|") == 0)
            {
                // Handle Data
                int id = 0;
                try
                {
                    id = int.Parse(name.Substring(5));
                }
                catch (FormatException)
                {
                    return;
                }

                TcpClient client = (TcpClient)tcpList[id];
                if (client != null)
                {
                    if (client.Connected == false)
                    {
                        //int tt = 5; // TODO - GOT DATA AND SOCKET IS NOT CONNECTED ******
                    }
                    else
                    {
                        client.GetStream().Write(data, 0, data.Length);
                    }
                }
            }

            if (name.Substring(0, 5).CompareTo("TCPL|") == 0)
            {
                // Handle Data
                int id = 0;
                try
                {
                    id = int.Parse(name.Substring(5));
                }
                catch (FormatException)
                {
                    return;
                }

                CloseTcpClient(id);
            }
        }

        private void CloseTcpClient(int id)
        {
            TcpClient client = (TcpClient)tcpList[id];
            if (client != null)
            {
                client.Close();
                tcpList.Remove(id);
                tcpBufList.Remove(id);
            }
        }

        private void ProcessPacket(int command, byte[] data)
        {

        }

        private void TcpConnectEvent(IAsyncResult r)
        {
            int id = (int)r.AsyncState;
            TcpClient client = (TcpClient)tcpList[id];
            byte[] buf = (byte[])tcpBufList[id];

            try
            {
                client.EndConnect(r);
            }
            catch (SocketException)
            {
                serial.WriteDataBlock("TCPL|" + id.ToString(), null);
                CloseTcpClient(id);
                return;
            }

            client.ReceiveBufferSize = 20000; // Set a small receive buffer size since serial is slow.
            client.GetStream().BeginRead(buf, 0, 1024, new AsyncCallback(TcpReadEvent), id);
        }

        private void TcpReadEvent(IAsyncResult r)
        {
            int id = (int)r.AsyncState;
            TcpClient client = (TcpClient)tcpList[id];
            if (client == null) return;
            byte[] buf = (byte[])tcpBufList[id];

            int len = 0;
            try
            {
                len = client.GetStream().EndRead(r);
            }
            catch (InvalidOperationException)
            {
                serial.WriteDataBlock("TCPL|" + id.ToString(), null);
                CloseTcpClient(id);
                return;
            }
            catch (IOException)
            {
                serial.WriteDataBlock("TCPL|" + id.ToString(), null);
                CloseTcpClient(id);
                return;
            }

            if (serial != null) serial.WriteDataBlock("TCPD|" + id.ToString(), buf, 0, len);

            if (serial.BytesToWrite > 1024)
            {
                lock (tcpHalted)
                {
                    tcpHalted.Add(id);
                }
            }
            else
            {
                try
                {
                    buf = new byte[1024];
                    tcpBufList[id] = buf;
                    client.GetStream().BeginRead(buf, 0, 1024, new AsyncCallback(TcpReadEvent), id);
                }
                catch (InvalidOperationException)
                {
                    serial.WriteDataBlock("TCPL|" + id.ToString(), null);
                    CloseTcpClient(id);
                }
                catch (IOException)
                {
                    serial.WriteDataBlock("TCPL|" + id.ToString(), null);
                    CloseTcpClient(id);
                }
            }
        }

        private void FlowControlThread()
        {
            try
            {
                while (serial != null)
                {
                    int btw = serial.BytesToWrite;
                    if (btw < 1024 && tcpHalted.Count > 0)
                    {
                        int id;
                        lock (tcpHalted)
                        {
                            id = (int)tcpHalted[0];
                            tcpHalted.RemoveAt(0);
                        }
                        TcpClient client = (TcpClient)tcpList[id];
                        byte[] buf = new byte[1024];
                        tcpBufList[id] = buf;
                        try
                        {
                            client.GetStream().BeginRead(buf, 0, 1024, new AsyncCallback(TcpReadEvent), id);
                        }
                        catch (InvalidOperationException)
                        {
                            serial.WriteDataBlock("TCPL|" + id.ToString(), null);
                            CloseTcpClient(id);
                        }
                        catch (IOException)
                        {
                            serial.WriteDataBlock("TCPL|" + id.ToString(), null);
                            CloseTcpClient(id);
                        }
                    }
                    Thread.Sleep(50);
                }
            }
            catch (Exception) { }
        }

      private int PowerCommand(string cmd, ref bool bForce)
      { 
        int Command = -1;
        bForce = false;
        if (cmd.Length >= 8 && string.Compare(cmd.Substring(0, 8), "SHUTDOWN", true) == 0)
        {
          Command = ShutdownClass.EWX_SHUTDOWN;
        }
        else if (cmd.Length >= 6 && string.Compare(cmd.Substring(0, 6), "LOGOFF", true) == 0)
        {
          Command = ShutdownClass.EWX_LOGOFF;
        }
        else if (cmd.Length >= 6 && string.Compare(cmd.Substring(0, 5), "REBOOT", true) == 0)
        {
          Command = ShutdownClass.EWX_REBOOT;
        }
        else if (cmd.Length >= 5 && string.Compare(cmd.Substring(0, 5), "SLEEP", true) == 0)
        {
          Command = ShutdownClass.SLEEP;
        }
        else if (cmd.Length >= 9 && string.Compare(cmd.Substring(0, 9), "HIBERNATE", true) == 0)
        {
          Command = ShutdownClass.HIBERNATE;
        }

        if (cmd.IndexOf(" /F") > 0) bForce = true;

        return Command;
      }

        private void ProcessCommand(string cmd)
        {
            if (cmd.Length > 0) commandHistory.Insert(1,cmd);
            if (commandHistory.Count > 100) commandHistory.RemoveAt(100);
            commandHistoryPtr = 0;
            string cmd2 = cmd;
            cmd = cmd.ToUpper();
            serial.Write("\r\n");

            int err = 0;
            string msg = "";
            int command = 0;
            bool bForce = false;
            command = PowerCommand(cmd, ref bForce);

            if (cmd.Length == 0)
            {
                // NOP
            }
            else if (cmd.CompareTo("WATCHDOGS") == 0)
            {
                if (system == null)
                {
                    serial.Write("Local Manageability Agent Disconnected.\r\n");
                }
                else
                {
                    AmtWatchdogLocal.AmtWatchdog[] wgs = system.WatchdogLocal.GetWatchdogs();
                    foreach (AmtWatchdogLocal.AmtWatchdog w in wgs)
                    {
                        string s1 = "Disabled";
                        if (w.beat) s1 = "Active";
                        
                        string s2 = "No State";
                        if (w.beat == true)
                        {
                            switch (w.lastState)
                            {
                                case AmtCallStatus.SUCCESS:
                                    s2 = ManageabilityOutpostTool.Properties.Resources.MainForm_CallStatus_OK;
                                    break;
                                case AmtCallStatus.CALL_SKIPPED:
                                    s2 = ManageabilityOutpostTool.Properties.Resources.MainForm_CallStatus_Skipped;
                                    break;
                                case AmtCallStatus.INVALID_HANDLE:
                                    s2 = ManageabilityOutpostTool.Properties.Resources.MainForm_CallStatus_NoExist;
                                    break;
                                default:
                                    s2 = w.lastState.ToString().Trim();
                                    break;
                            }
                        }

                        serial.Write(string.Format("{0},{1},{2}\r\n", w.name, s1, s2));
                    }
                }
            }
            else if (cmd.CompareTo("CMD") == 0)
            {
                // if (RConsole == null) RConsole = new RemoteConsole(); // TODO WORKING ON THIS
            }
            else if (cmd.StartsWith("CHAT"))
            {
                chatTextApplication = new TextChatClass(this, serial);
                return;
            }
            else if (cmd.StartsWith("TOP"))
            {
                chatTextTopClass = new TextTopClass(this, serial);
                return;
            }
            else if (cmd.CompareTo("SENDSAMPLEUTF") == 0)
            {
                serial.Write(ManageabilityOutpostTool.Properties.Resources.SampleUTF + "\r\n");
            }
            else if (cmd.CompareTo("SENDSAMPLECHARS") == 0)
            {
                serial.Write(new byte[] { 177, 177, 177, 178, 178 });
                serial.Write("\r\n");
                serial.Write(new byte[] { 177, 177, 177, 178, 178 });
                serial.Write("\r\n");
                serial.Write(new byte[] { 177, 177, 177, 178, 178 });
                serial.Write("\r\n");
                serial.Write(new byte[] { 177, 177, 177, 178, 178 });
                serial.Write("\r\n");
                serial.Write(new byte[] { 177, 177, 177, 178, 178 });
                serial.Write("\r\n");
                serial.Write(new byte[] { 177, 177, 177, 178, 178 });
                serial.Write("\r\n");
            }
            else if (cmd.CompareTo("CLEAR") == 0 || cmd.CompareTo("CLS") == 0)
            {
                serial.WriteClearScreen();
            }
            else if (cmd.CompareTo("RESTART") == 0)
            {
                SendIntro();
            }
            else if (cmd.CompareTo("HWRESCAN") == 0)
            {
                DevDriCtlClass.cmdHWRescan();
            }
            else if (command != -1) // Power Command
            {
                if (SerialAgentPermissionCtl.GetPermission(SerialAgentPermissionCtl.Permissions.Shutdown)
                    == SerialAgentPermissionCtl.SAP_PERMISSION_DENY)
                {
                    serial.Write("Access Denied.\r\n");
                }
                else
                {
                    err = ShutdownClass.DoExitWin(command, bForce, ref msg);
                    if (err != 0)
                    {
                        serial.Write("The last error code: " + err + "\r\n" + msg + "\r\n");
                        if (!bForce)
                        {
                            serial.Write("Do you want to forcely close all the applications without saving? y/n\r\n");
                        }
                    }
                }
            }
            else if (cmd.CompareTo("Y") == 0)
            {
                command = PowerCommand(lastcmd, ref bForce);
                if (command != 0)
                {
                    err = ShutdownClass.DoExitWin(command, true, ref msg);
                }
            }
            else if (cmd.CompareTo("N") == 0)
            {

            }
            else if (cmd.CompareTo("HELP") == 0)
            {
                StringBuilder sb = new StringBuilder();
                sb.Append("Possible Commands:\r\n");
                sb.Append("  HELP, TIME, EXEC, PS, KILL, CLEAR, CLS, RESTART, TYPE, CHAT, TOP\r\n");
                sb.Append("  HWRESCAN, DOWNLOAD, UPLOAD, WATCHDOGS, POWERSTATUS\r\n");
                sb.Append("  SHUTDOWN, REBOOT, LOGOFF, SLEEP, HIBERNATE\r\n");
                sb.Append("  SENDSAMPLEUTF, SENDSAMPLECHARS\r\n");
                sb.Append("  specify /F to force action, such as LOGOFF /F, SHUTDOWN /F, and so on\r\n");
                serial.Write(sb.ToString());
            }
            else if (cmd.CompareTo("TIME") == 0)
            {
                serial.Write("Current time is: " + DateTime.Now.ToString() + "\r\n");
            }
            else if (cmd.Length > 8 && cmd.Substring(0, 6).CompareTo("UPLOAD") == 0)
            {
                if (SerialAgentPermissionCtl.GetPermission(SerialAgentPermissionCtl.Permissions.Upload)
                    == SerialAgentPermissionCtl.SAP_PERMISSION_DENY)
                {
                    serial.Write("Access Denied.\r\n");
                }
                else
                {
                    uploadFileName = cmd.Substring(7);
                    serial.WriteDataBlock("UPLD|" + uploadFileName, null);
                }
            }
            else if (cmd.Length > 10 && cmd.Substring(0, 8).CompareTo("DOWNLOAD") == 0)
            {
                if (SerialAgentPermissionCtl.GetPermission(SerialAgentPermissionCtl.Permissions.Download)
                    == SerialAgentPermissionCtl.SAP_PERMISSION_DENY)
                {
                    serial.Write("Access Denied.\r\n");
                }
                else
                {
                    FileInfo fi = new FileInfo(cmd.Substring(9));
                    if (fi.Exists == false)
                    {
                        serial.Write("File does not exist\r\n");
                    }
                    else
                    {
                        FileStream f = null;
                        try
                        {
                            f = new FileStream(fi.FullName, FileMode.Open);
                        }
                        catch (Exception ex)
                        {
                            serial.Write(ex.Message + "\r\n");
                        }
                        if (f != null)
                        {
                            serial.Write(string.Format("Sending File: {0} of {1} bytes\r\n", fi.FullName, fi.Length));
                            MemoryStream m = new MemoryStream();
                            byte[] buf = new byte[100];
                            int len = 0;
                            do
                            {
                                len = f.Read(buf, 0, 100);
                                m.Write(buf, 0, len);
                            }
                            while (len != 0);
                            f.Close();
                            serial.WriteDataBlock("FILE|" + fi.FullName, m.ToArray());
                            m.Dispose();
                            m = null;
                        }
                    }
                }
            }
            else if (cmd.CompareTo("PS") == 0)
            {
                Process[] procList = Process.GetProcesses();
                string s = "";
                foreach (Process p in procList)
                {
                    try
                    {
                        s += p.Id.ToString() + " - " + p.ProcessName + "\r\n";
                    }
                    catch (Exception) { }
                }
                serial.Write(s);
            }
            else if (cmd.Length >= 4 && cmd.Substring(0, 2).CompareTo("PS") == 0)
            {
                int pn = -1;
                try
                {
                    pn = int.Parse(cmd.Substring(3));
                }
                catch (FormatException) {}
                if (pn > 0)
                {
                    Process p = null;
                    try
                    {
                        p = Process.GetProcessById(pn);
                    }
                    catch (Exception)
                    {
                        serial.Write("Error Getting Process Info for ID: " + pn + "\r\n");
                        p = null;
                    }
                    if (p != null)
                    {
                        serial.Write(string.Format("Process name: {0}\r\nId: {1}\r\nThread Count: {2}\r\nStart Time: {3}\r\nResponding: {4}\r\n", p.ProcessName, p.Id.ToString(), p.Threads.Count, p.StartTime.ToString(), p.Responding.ToString()));
                    }
                }
                else
                {
                    serial.Write("Invalid process id.\r\n");
                }
            }
            else if (cmd.Length == 11 && cmd.Substring(0, 11).CompareTo("POWERSTATUS") == 0)
            {
                serial.Write(string.Format("PowerLineStatus: {0}\r\n", System.Windows.Forms.SystemInformation.PowerStatus.PowerLineStatus.ToString()));
                serial.Write(string.Format("BatteryChargeStatus: {0}\r\n", System.Windows.Forms.SystemInformation.PowerStatus.BatteryChargeStatus.ToString()));
                if (System.Windows.Forms.SystemInformation.PowerStatus.BatteryChargeStatus != System.Windows.Forms.BatteryChargeStatus.NoSystemBattery)
                {
                    serial.Write(string.Format("BatteryFullLifetime: {0}\r\n", System.Windows.Forms.SystemInformation.PowerStatus.BatteryFullLifetime.ToString()));
                    serial.Write(string.Format("BatteryLifePercent: {0}\r\n", System.Windows.Forms.SystemInformation.PowerStatus.BatteryLifePercent.ToString()));
                    serial.Write(string.Format("BatteryLifeRemaining: {0}\r\n", System.Windows.Forms.SystemInformation.PowerStatus.BatteryLifeRemaining.ToString()));
                }
            }
            else if (cmd.Length > 6 && cmd.Substring(0, 4).CompareTo("TYPE") == 0)
            {
                string filename = cmd.Substring(5);
                if (File.Exists(filename) == false)
                {
                    serial.Write("File not found.\r\n");
                }
                else
                {
                    FileStream f = null;
                    try
                    {
                        f = new FileStream(filename, FileMode.Open);
                    }
                    catch (Exception ex)
                    {
                        serial.Write(ex.Message + "\r\n");
                    }
                    if (f != null)
                    {
                        byte[] buf = new byte[f.Length];
                        f.Read(buf, 0, buf.Length);
                        f.Close();
                        serial.Write(buf);
                        serial.Write("\r\n");
                    }
                }
            }
            else if (cmd.Length > 6 && cmd.Substring(0, 4).CompareTo("KILL") == 0)
            {
                if (SerialAgentPermissionCtl.GetPermission(SerialAgentPermissionCtl.Permissions.Stop)
                    == SerialAgentPermissionCtl.SAP_PERMISSION_DENY)
                {
                    serial.Write("Access Denied.\r\n");
                }
                else
                {
                    int pn = -1;
                    try
                    {
                        pn = int.Parse(cmd.Substring(5));
                    }
                    catch (FormatException) { }
                    if (pn > 0)
                    {
                        try
                        {
                            Process p = Process.GetProcessById(pn);
                            serial.Write(string.Format("Killed Process: {0}\r\n", p.ProcessName));
                            p.Kill();
                        }
                        catch (ArgumentException)
                        {
                            serial.Write("Process not found\r\n");
                        }
                    }
                }
            }
            else if (cmd.Length >= 4 && cmd.Substring(0, 4).CompareTo("EXEC") == 0)
            {
                bool error = false;
                if (SerialAgentPermissionCtl.GetPermission(SerialAgentPermissionCtl.Permissions.Execute)
                    == SerialAgentPermissionCtl.SAP_PERMISSION_DENY)
                {
                    serial.Write("Access Denied.\r\n");
                }
                else if (cmd.Length <= 5)
                {
                    serial.Write("Must sepecify a parameter after EXEC command\"\r\n");
                }
                else
                {
                    string exec = cmd.Substring(5);
                    serial.Write(string.Format("Executing: \"{0}\"\r\n", exec));
                    System.Diagnostics.Process p = null; ;
                    try
                    {
                        string args = null;
                        if (exec.IndexOf(" ") > 0)
                        {
                            args = exec.Substring(exec.IndexOf(" ") + 1);
                            exec = exec.Substring(0, exec.IndexOf(" "));
                        }
                        p = System.Diagnostics.Process.Start(exec, args);
                    }
                    catch (System.ComponentModel.Win32Exception)
                    {
                        serial.Write("File not found\r\n");
                        error = true;
                    }
                    if (error == false)
                    {
                        if (p == null)
                        {
                            serial.Write("Process Created with NULL returned\r\n");
                        }
                        else
                        {
                            serial.Write("Process " + p.Id.ToString() + " Created\r\n");
                        }
                    }
                }
            }
            else
            {
                if (cmd.Length > 0)
                {
                    serial.Write("Got unknown command: \"" + cmd + "\"\r\n");
                }
            }
            serial.Write("> ");
            lastcmd = cmd;
        }

        public void SendIntro()
        {
            accumulator = "";
            serial.Ping();
            serial.WriteEsc((byte)'m', 0);       // Reset attributes
            serial.WriteClearScreen();
            StringBuilder sb = new StringBuilder();
            sb.Append(VersionString).Append(" - Serial Agent\r\n");
            sb.Append("Copyright 2006 - 2008 Intel Corporation\r\n");
            sb.Append("Current Process Handle: " + Process.GetCurrentProcess().Id.ToString() + "\r\n\r\n");
            sb.Append("Type \"HELP\" command at anytime for help, and [`] to clear and restart.\r\n\r\n");
            sb.Append("> ");
            serial.Write(sb.ToString());
        }

        public void AddChatString(string message)
        {
            lock (chatStrings)
            {
                chatStrings.Add(message);
                if (chatStrings.Count > 150) chatStrings.RemoveAt(0);
            }
            if (chatForm != null && chatForm.Visible == false) chatForm.DisplayForm(); else chatForm.UpdateText();
            if (chatTextApplication != null) chatTextApplication.Invalidate();
        }

        public void ClearChatStrings()
        {
            lock (chatStrings)
            {
                chatStrings.Clear();
            }
            if (chatForm != null && chatForm.Visible == false) chatForm.DisplayForm(); else chatForm.UpdateText();
            if (chatTextApplication != null) chatTextApplication.Invalidate();
        }

        public void SetRemoteTyping(bool typing)
        {
            chatForm.RemoteTyping(typing);
        }

        public string GetChatText(int lines)
        {
            if (lines == 0 || chatStrings == null || chatStrings.Count == 0) return "";
            if (lines > chatStrings.Count) lines = chatStrings.Count;
            StringBuilder sb = new StringBuilder();
            lock (chatStrings)
            {
                for (int i = chatStrings.Count - lines; i < chatStrings.Count; i++)
                {
                    sb.Append(chatStrings[i]).Append("\r\n");
                }
            }
            return sb.ToString();
        }

    }
}
