// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;

namespace ManageabilityOutpostTool
{
    public partial class ChatWindowForm : Form
    {
        private Control parentControl = null;
        private SerialHighDriver serial = null;
        private AmtSerialAgent agent = null;
        private bool typing = false;

        public ChatWindowForm(Control parentControl, SerialHighDriver serial, AmtSerialAgent agent)
        {
            this.parentControl = parentControl;
            this.serial = serial;
            this.agent = agent;
            InitializeComponent();
        }

        public void RemoteTyping(bool istyping)
        {
            typingToolStripStatusLabel.Visible = istyping;
        }

        private delegate void NullHandler();
        public void DisplayForm()
        {
            parentControl.Invoke(new NullHandler(DisplayFormEx));
        }

        private void DisplayFormEx()
        {
            if (Visible == false)
            {
                this.Show();
                UpdateText();
            }
        }

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void ChatWindowForm_Load(object sender, EventArgs e)
        {
            UpdateText();
        }

        public void UpdateText()
        {
            parentControl.Invoke(new NullHandler(UpdateTextEx));
        }

        private void UpdateTextEx()
        {
            mainTextBox.Text = agent.GetChatText(200);
            mainTextBox.SelectionStart = mainTextBox.Text.Length;
            mainTextBox.SelectionLength = 0;
            mainTextBox.ScrollToCaret();
        }

        private void sendButton_Click(object sender, EventArgs e)
        {
            if (sendTextBox.Text.Length > 0)
            {
                if (serial != null) serial.WriteDataBlock("CHAT", UTF8Encoding.UTF8.GetBytes("C" + sendTextBox.Text));
                agent.AddChatString("L> " + sendTextBox.Text);
                sendTextBox.Clear();
                serial.WriteDataBlock("CHAT", new byte[] { (byte)'N' });
                if (agent.chatTextApplication != null) agent.chatTextApplication.UserTyping(false);
                typing = false;
            }
        }

        private void sendTextBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter || e.KeyCode == Keys.Return) sendButton_Click(this, null);
        }

        private void sendTextBox_TextChanged(object sender, EventArgs e)
        {
            if (typing == false && sendTextBox.Text.Length > 0 && serial != null)
            {
                serial.WriteDataBlock("CHAT", new byte[] { (byte)'T' });
                if (agent.chatTextApplication != null) agent.chatTextApplication.UserTyping(true);
                typing = true;
            }
            else if (typing == true && sendTextBox.Text.Length == 0 && serial != null)
            {
                serial.WriteDataBlock("CHAT", new byte[] { (byte)'N' });
                if (agent.chatTextApplication != null) agent.chatTextApplication.UserTyping(false);
                typing = false;
            }
        }

        private void ChatWindowForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = true;
            Hide();
            if (agent.chatTextApplication != null) agent.chatTextApplication.Invalidate();
        }

        private void clearToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (serial != null) serial.WriteDataBlock("CHAT", UTF8Encoding.UTF8.GetBytes("L"));
            agent.ClearChatStrings();
            sendTextBox.Clear();
            typing = false;
        }
    }
}
