// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Text;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace ManageabilityOutpostTool
{
    public class CryptProtectClass
    {
        [DllImport("Crypt32.dll", SetLastError = true, CharSet = System.Runtime.InteropServices.CharSet.Auto)]
        private static extern bool CryptProtectData(
                                ref DATA_BLOB pDataIn,
                                String szDataDescr,
                                ref DATA_BLOB pOptionalEntropy,
                                IntPtr pvReserved,
                                ref CRYPTPROTECT_PROMPTSTRUCT pPromptStruct,
                                int dwFlags,
                                ref DATA_BLOB pDataOut);
        [DllImport("Crypt32.dll", SetLastError = true, CharSet = System.Runtime.InteropServices.CharSet.Auto)]
        private static extern bool CryptUnprotectData(
                                ref DATA_BLOB pDataIn,
                                String szDataDescr,
                                ref DATA_BLOB pOptionalEntropy,
                                IntPtr pvReserved,
                                ref CRYPTPROTECT_PROMPTSTRUCT pPromptStruct,
                                int dwFlags,
                                ref DATA_BLOB pDataOut);
        /*[DllImport("kernel32.dll", CharSet = System.Runtime.InteropServices.CharSet.Auto)]
        private unsafe static extern int FormatMessage(
                                int dwFlags,
                                ref IntPtr lpSource,
                                int dwMessageId,
                                int dwLanguageId,
                                ref String lpBuffer,
                                int nSize,
                                IntPtr* Arguments);*/

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
        internal struct DATA_BLOB
        {
            public int cbData;
            public IntPtr pbData;
        }

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
        internal struct CRYPTPROTECT_PROMPTSTRUCT
        {
            public int cbSize;
            public int dwPromptFlags;
            public IntPtr hwndApp;
            public String szPrompt;
        }

        static private IntPtr NullPtr = ((IntPtr)((int)(0)));
        private const int CRYPTPROTECT_UI_FORBIDDEN = 0x1;
        private const int CRYPTPROTECT_LOCAL_MACHINE = 0x4;

        public enum cpStore { USE_MACHINE_STORE = 1, USE_USER_STORE };
        private cpStore cpcstore;
        public CryptProtectClass(cpStore tmpStore)
        {
            cpcstore = tmpStore;
        }

        public byte[] StrToByteArray(string str)
        {
            if (null != str)
            {
                ASCIIEncoding encoding = new ASCIIEncoding();
                return encoding.GetBytes(str);
            }
            else
            {
                return null;
            }
        }

        public string ByteArrayToStr(byte[] barr)
        {
            if (null != barr)
            {
                ASCIIEncoding encoding = new ASCIIEncoding();
                return encoding.GetString(barr);
            }
            else
            {
                return null;
            }
        }

        public byte[] Encrypt(byte[] plainText, byte[] optionalEntropy)
        {
            bool retVal = false;

            DATA_BLOB plainTextBlob = new DATA_BLOB();
            DATA_BLOB cipherTextBlob = new DATA_BLOB();
            DATA_BLOB entropyBlob = new DATA_BLOB();

            CRYPTPROTECT_PROMPTSTRUCT prompt = new CRYPTPROTECT_PROMPTSTRUCT();
            InitPromptstruct(ref prompt);

            int dwFlags;
            try
            {
                try
                {
                    int bytesSize = plainText.Length;
                    plainTextBlob.pbData = Marshal.AllocHGlobal(bytesSize);
                    if (IntPtr.Zero == plainTextBlob.pbData)
                    {
                        //throw new Exception("can not allocate plainText buffer.");
                        return null;
                    }
                    plainTextBlob.cbData = bytesSize;
                    Marshal.Copy(plainText, 0, plainTextBlob.pbData, bytesSize);
                }
                catch (Exception /*ex*/)
                {
                    //throw new Exception("exception during data process. " + ex.Message);
                    return null;
                }
                if (cpStore.USE_MACHINE_STORE == cpcstore)
                {
                    //provide entropy if use local machine storage
                    dwFlags = CRYPTPROTECT_LOCAL_MACHINE | CRYPTPROTECT_UI_FORBIDDEN;
                    //check if entropy is null
                    if (null == optionalEntropy)
                    {
                        //allocate object
                        optionalEntropy = new byte[0];
                    }
                    try
                    {
                        int bytesSize = optionalEntropy.Length;
                        entropyBlob.pbData = Marshal.AllocHGlobal(optionalEntropy.Length);
                        if (IntPtr.Zero == entropyBlob.pbData)
                        {
                            //throw new Exception("can not allocate entropy buffer.");
                            return null;
                        }
                        Marshal.Copy(optionalEntropy, 0, entropyBlob.pbData, bytesSize);
                        entropyBlob.cbData = bytesSize;
                    }
                    catch (Exception /*ex*/)
                    {
                        //throw new Exception("exception during entropy process. " + ex.Message);
                        return null;
                    }
                }
                else
                {
                    //use user storage
                    dwFlags = CRYPTPROTECT_UI_FORBIDDEN;
                }
                retVal = CryptProtectData(ref plainTextBlob, "", ref entropyBlob, IntPtr.Zero, ref prompt, dwFlags, ref cipherTextBlob);
                if (false == retVal)
                {
                    //throw new Exception("encrypt failed. " /*+ GetErrorMessage(Marshal.GetLastWin32Error())*/);
                    return null;
                }
            }
            catch (Exception /*ex*/)
            {
                //throw new Exception("exception during encrypt. " + ex.Message);
                return null;
            }
            byte[] cipherText = new byte[cipherTextBlob.cbData];
            Marshal.Copy(cipherTextBlob.pbData, cipherText, 0, cipherTextBlob.cbData);
            return cipherText;
        }

        public byte[] Decrypt(byte[] cipherText, byte[] optionalEntropy)
        {
            bool retVal = false;

            DATA_BLOB plainTextBlob = new DATA_BLOB();
            DATA_BLOB cipherBlob = new DATA_BLOB();
            DATA_BLOB entropyBlob = new DATA_BLOB();

            CRYPTPROTECT_PROMPTSTRUCT prompt = new CRYPTPROTECT_PROMPTSTRUCT();
            InitPromptstruct(ref prompt);

            int dwFlags;
            try
            {
                try
                {
                    int cipherTextSize = cipherText.Length;
                    cipherBlob.pbData = Marshal.AllocHGlobal(cipherTextSize);
                    if (IntPtr.Zero == cipherBlob.pbData)
                    {
                        //throw new Exception("can not allocate cipherText buffer.");
                        return null;
                    }
                    cipherBlob.cbData = cipherTextSize;
                    Marshal.Copy(cipherText, 0, cipherBlob.pbData, cipherBlob.cbData);
                }
                catch (Exception /*ex*/)
                {
                    //throw new Exception("exception during data process. " + ex.Message);
                    return null;
                }
                if (cpStore.USE_MACHINE_STORE == cpcstore)
                {
                    //provide entropy if use local machine storage
                    dwFlags = CRYPTPROTECT_LOCAL_MACHINE | CRYPTPROTECT_UI_FORBIDDEN;
                    //check if entropy is null
                    if (null == optionalEntropy)
                    {
                        //allocate object
                        optionalEntropy = new byte[0];
                    }
                    try
                    {
                        int bytesSize = optionalEntropy.Length;
                        entropyBlob.pbData = Marshal.AllocHGlobal(bytesSize);
                        if (IntPtr.Zero == entropyBlob.pbData)
                        {
                            //throw new Exception("can not allocate entrpoy buffer.");
                            return null;
                        }
                        entropyBlob.cbData = bytesSize;
                        Marshal.Copy(optionalEntropy, 0, entropyBlob.pbData, bytesSize);
                    }
                    catch (Exception /*ex*/)
                    {
                        //throw new Exception("exception during entropy process. " + ex.Message);
                        return null;
                    }
                }
                else
                {
                    //use user storage
                    dwFlags = CRYPTPROTECT_UI_FORBIDDEN;
                }
                retVal = CryptUnprotectData(ref cipherBlob, null, ref entropyBlob, IntPtr.Zero, ref prompt, dwFlags, ref plainTextBlob);
                if (false == retVal)
                {
                    //throw new Exception("decrypt failed. " /*+ GetErrorMessage(Marshal.GetLastWin32Error())*/);
                    return null;
                }
                //release blob and entropy
                if (IntPtr.Zero != cipherBlob.pbData)
                {
                    Marshal.FreeHGlobal(cipherBlob.pbData);
                }
                if (IntPtr.Zero != entropyBlob.pbData)
                {
                    Marshal.FreeHGlobal(entropyBlob.pbData);
                }
            }
            catch (Exception /*ex*/)
            {
                //throw new Exception("exception during decrypt. " + ex.Message);
                return null;
            }
            byte[] plainText = new byte[plainTextBlob.cbData];
            Marshal.Copy(plainTextBlob.pbData, plainText, 0, plainTextBlob.cbData);
            return plainText;
        }

        private void InitPromptstruct(ref CRYPTPROTECT_PROMPTSTRUCT ps)
        {
            ps.cbSize = Marshal.SizeOf(typeof(CRYPTPROTECT_PROMPTSTRUCT));
            ps.dwPromptFlags = 0;
            ps.hwndApp = NullPtr;
            ps.szPrompt = null;
        }

        /*private unsafe static String GetErrorMessage(int errorCode)
        {
            int FORMAT_MESSAGE_ALLOCATE_BUFFER = 0x00000100;
            int FORMAT_MESSAGE_IGNORE_INSERTS = 0x00000200;
            int FORMAT_MESSAGE_FROM_SYSTEM = 0x00001000;
            int messageSize = 255;
            String lpMsgBuf = "";
            int dwFlags = FORMAT_MESSAGE_ALLOCATE_BUFFER |
            FORMAT_MESSAGE_FROM_SYSTEM |
                          FORMAT_MESSAGE_IGNORE_INSERTS;
            IntPtr ptrlpSource = new IntPtr();
            IntPtr prtArguments = new IntPtr();
            int retVal = FormatMessage(dwFlags, ref ptrlpSource, errorCode, 0,
                                       ref lpMsgBuf, messageSize, &
                                       prtArguments);
            if (0 == retVal)
            {
                throw new Exception("can not set error code message format " +
                                    errorCode + ". ");
            }
            return lpMsgBuf;
        }*/
    }
}
