// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;
using ManageabilityControls;

namespace ManageabilityOutpostTool
{
    public partial class DiagnosticStorageForm : Form
    {
        AmtSystem system;
        bool tls;
        string username;
        string password;

        public DiagnosticStorageForm(AmtSystem system, bool tls, string username, string password)
        {
            this.system = system;
            this.tls = tls;
            this.username = username;
            this.password = password;
            InitializeComponent();
        }

        private void DiagnosticStorageForm_Load(object sender, EventArgs e)
        {
            
        }

        private bool PushDiagnosticData()
        {
            AmtStorageWrapper.AmtStorageStatus r;

            // Generate the management data
            string[] queries = queryListTextBox.Lines;
            ArrayList l = new ArrayList();
            for (int i = 0; i < queries.Length; i++)
            {
                if (queries[i].IndexOf("|") < 0) queries[i] = "|" + queries[i];
                ManagementData m = ManagementData.GetWmiQuery("WMI|" + queries[i]);
                if (m != null) l.Add(m);
            }
            if (l.Count == 0)
            {
                MessageBox.Show(this, Properties.Resources.NoWmiResults, Properties.Resources.Error, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
            byte[] buf = ManagementData.ToBinary((ManagementData[])l.ToArray(typeof(ManagementData)));
            
            // Connect to 3PDS
            LocalStorageInfo store = new LocalStorageInfo(enterpriseTextBox.Text, vendorTextBox.Text, applicationTextBox.Text, new Guid("00000000-0000-0000-0000-000000000001"));
            if (store.Connect(system) == false)
            {
                MessageBox.Show(this, Properties.Resources.ConnectError3pds, Properties.Resources.Error, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            // Get our own application handle
            AmtStorageApplication[] apps = store.GetApplications();
            AmtStorageApplication thisApp = null;
            foreach (AmtStorageApplication app in apps)
            {
                if (app.Enterprise.Equals(enterpriseTextBox.Text) && app.Vendor.Equals(vendorTextBox.Text) && app.Application.Equals(applicationTextBox.Text))
                {
                    thisApp = app;
                    break;
                }
            }
            if (thisApp == null)
            {
                MessageBox.Show(this, Properties.Resources.ConnectError3pds, Properties.Resources.Error, MessageBoxButtons.OK, MessageBoxIcon.Error);
                store.Disconnect();
                return false;
            }

            // Get the block list
            AmtStorageBlock[] blocks = store.Storage.GetBlockList(thisApp.Handle);

            // Remove the old block
            foreach (AmtStorageBlock block in blocks)
            {
                if (block.Name.StartsWith("IntelComDiagV"))
                {
                    r = block.Remove();
                    if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
                    {
                        ShowStorageError("Remove", r);
                        store.Disconnect();
                        return false;
                    }
                }
            }

            // Compute the required size
            int pages = (buf.Length / 4096) + 1;

            // Allocated a new block
            AmtStorageBlock newBlock;
            r = store.Storage.AllocateBlock("IntelComDiagV01", (uint)(pages * 4096), out newBlock);
            if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
            {
                ShowStorageError("AllocateBlock", r);
                store.Disconnect();
                return false;
            }

            // Store data into 3PDS & set visible
            r = newBlock.WriteBlock(buf);
            if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
            {
                ShowStorageError("WriteBlock", r);
                store.Disconnect();
                return false;
            }

            r = newBlock.SetVisibility(false);
            if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
            {
                ShowStorageError("SetVisibility", r);
                store.Disconnect();
                return false;
            }

            // Default default permissions
            AmtStorageWrapper.AmtStoragePermissionsGroup group1, group2;
            r = newBlock.AddPermissionGroup(AmtStorageWrapper.AmtStorageGroupPermission.ReadWrite, "ReadWriteGroup", out group1);
            if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
            {
                ShowStorageError("AddPermissionGroup", r);
                store.Disconnect();
                return false;
            }

            r = newBlock.AddPermissionGroup(AmtStorageWrapper.AmtStorageGroupPermission.ReadOnly, "ReadOnlyGroup", out group2);
            if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
            {
                ShowStorageError("AddPermissionGroup", r);
                store.Disconnect();
                return false;
            }

            r = newBlock.AddPermissionsGroupMembers(group1, new uint[] { 0xFFFFFFF1 });
            if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
            {
                ShowStorageError("AddPermissionsGroupMembers", r);
                store.Disconnect();
                return false;
            }

            store.Disconnect();
            return true;
        }

        private void ShowStorageError(string method, AmtStorageWrapper.AmtStorageStatus r)
        {
            if (r == AmtStorageWrapper.AmtStorageStatus.SUCCESS) return;
            MessageBox.Show(this, string.Format(Properties.Resources.AmtStorageError, method, r.ToString()), Properties.Resources.Error, MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private void UpdateInfo()
        {
            bool ok = true;
            enterpriseTextBox.Text = enterpriseTextBox.Text.Trim();
            vendorTextBox.Text = vendorTextBox.Text.Trim();
            applicationTextBox.Text = applicationTextBox.Text.Trim();

            if (enterpriseTextBox.Text.Length == 0) ok = false;
            if (vendorTextBox.Text.Length == 0) ok = false;
            if (applicationTextBox.Text.Length == 0) ok = false;
            if (queryListTextBox.Lines.Length == 0) ok = false;

            okButton.Enabled = ok;
        }

        private void enterpriseTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void vendorTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void applicationTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void queryListTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void updateButton_Click(object sender, EventArgs e)
        {
            PushDiagnosticData();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            string oktext = okButton.Text;
            okButton.Text = "Wait...";
            okButton.Enabled = false;
            cancelButton.Enabled = false;
            if (PushDiagnosticData() == true)
            {
                DialogResult = DialogResult.OK;
                return;
            }
            cancelButton.Enabled = true;
            okButton.Text = oktext;
            UpdateInfo();
        }

    }
}
