// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Net;
using System.Data;
using System.Text;
using System.Drawing;
using Microsoft.Win32;
using System.IO.Ports;
using System.Threading;
using System.Collections;
using System.Diagnostics;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Security.Cryptography.X509Certificates;
using ManageabilityStack;

namespace ManageabilityOutpostTool
{
    public partial class MainForm : Form
    {
        private AmtSystem system = null;
        private AmtSerialAgent serialAgent = null;
        private ManageabilityControls.TerminalForm terminal = null;
        private string AgentVersionString;
        private bool connected = false;
        private Thread FetchCacheThread = null;
        private bool closing = false;

        public MainForm()
        {
            InitializeComponent();
        }

        public MainForm(string username, string password, bool tls)
        {
            InitializeComponent();
            if (username != null) usernameTextBox.Text = username;
            if (password != null) passwordControl.Text = password;
            Application.UserAppDataRegistry.SetValue("UseTlsFirst", tls.ToString());
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void aboutIntelManageabilityOutpostToolToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new ManageabilityControls.AboutForm(ManageabilityOutpostTool.Properties.Resources.ProductName, ManageabilityOutpostTool.Properties.Resources.ProductSubName).ShowDialog(this);
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            AgentVersionString = mainToolStripStatusLabel.Text.Replace("0.00", ManageabilityControls.AboutForm.VersionString);
            mainToolStripStatusLabel.Text = mainToolStripStatusLabel.Text.Replace("0.00", ManageabilityControls.AboutForm.VersionString).Replace("(R)", "" + (char)0x00AE);
            this.Text = this.Text.Replace("(R)", "" + (char)0x00AE);
            label4.Text = label4.Text.Replace("(R)", "" + (char)0x00AE);
            label14.Text = label14.Text.Replace("(R)", "" + (char)0x00AE);
            label22.Text = label22.Text.Replace("(R)", "" + (char)0x00AE);

            mEInfoToolStripMenuItem.Enabled = ManageabilityStack.HeciWrapper.CheckHeciWrapper();
           
            
            if (usernameTextBox.Text == null || usernameTextBox.Text.Length == 0) usernameTextBox.Text = (string)Application.UserAppDataRegistry.GetValue("Username", "admin");
            if (passwordControl.Text == null || passwordControl.Text.Length == 0) passwordControl.Text = (string)Application.UserAppDataRegistry.GetValue("Password", "");

            updateRedirectLogCheckBox.Checked = Application.UserAppDataRegistry.GetValue("UpdateRedirectLog", bool.TrueString).ToString().CompareTo(bool.TrueString) == 0;
            heartbeatCheckBox.Checked = Application.UserAppDataRegistry.GetValue("Heartbeat", bool.TrueString).ToString().CompareTo(bool.TrueString) == 0;
            string previousAgentPort = (string)Application.UserAppDataRegistry.GetValue("SerialAgentPort", "");

            LoadWatchdogs();
            LoadStorageSettings();

            if (Application.UserAppDataRegistry.GetValue("SerialAgent", bool.FalseString).ToString().CompareTo(bool.TrueString) == 0)
            {
                serialAgentCheckBox_Click(this, null);
            }

            ServicePointManager.ServerCertificateValidationCallback = new System.Net.Security.RemoteCertificateValidationCallback(RemoteCertificateValidationCallbackSink);

            UpdatePermissionsListView();
            SystemEvents.PowerModeChanged += new PowerModeChangedEventHandler(SystemEvents_PowerModeChanged);
            refreshButton_Click(this, null);

            delayedConnectTimer.Enabled = true;
        }

        private void delayedConnectTimer_Tick(object sender, EventArgs e)
        {
            delayedConnectTimer.Enabled = false;
            if (passwordControl.Text.Length > 0) connectButton_Click(this, null);
        }

        private bool DisconnectedLocal = false;
        private bool DisconnectedSerial = false;
        void SystemEvents_PowerModeChanged(object sender, PowerModeChangedEventArgs e)
        {
            switch (e.Mode)
            {
                case PowerModes.Resume:
                    if (system == null && passwordControl.Text.Length > 0 && DisconnectedLocal)
                    {
                        connectButton_Click(this, null); // Connect local
                        DisconnectedLocal = false;
                    }
                    if (serialAgent == null && DisconnectedSerial)
                    {
                        serialAgentCheckBox_Click(this, null); // Connect serial agent
                        DisconnectedSerial = false;
                    }
                    mainToolStripStatusLabel.Text = AgentVersionString.Replace("(R)", "" + (char)0x00AE); ;
                    break;
                case PowerModes.Suspend:
                    if (system != null)
                    {
                        connectButton_Click(this, null); // Disconnect local
                        DisconnectedLocal = true;
                    }
                    if (serialAgent != null)
                    {
                        serialAgentCheckBox_Click(this, null); // Disconnect serial agent
                        DisconnectedSerial = true;
                    }
                    break;
            }
        }

        private void SystemStateChangedHandler(AmtSystem computer)
        {
            this.Invoke(new AmtSystem.ObjectStateHandler(SystemStateChangedHandlerEx), new object[] { computer });
        }

        private void SystemStateChangedHandlerEx(AmtSystem computer)
        {
            switch (computer.State)
            {
                case AmtSystemObjState.Connecting:
                    break;
                case AmtSystemObjState.Disconnected:
                    // Disable the UI
                    watchDogListView.Enabled = false;
                    heartbeatCheckBox.Enabled = false;

                    // Connection failed.
                    if (connected == false && closing == false)
                    {
                        Application.UserAppDataRegistry.SetValue("Password", "");
                        MessageBox.Show(this,
                            ManageabilityOutpostTool.Properties.Resources.MainForm_LocalCon_Fail,
                            ManageabilityOutpostTool.Properties.Resources.MainForm_LocalCon_Err,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Asterisk);
                    }
                    connected = false;

                    if (system != null)
                    {
                        DisconnectStorage();
                        if (serialAgent != null) serialAgent.system = null;

                        if (system.WatchdogLocal != null)
                        {
                            system.WatchdogLocal.WatchdogHeartbeat -= new AmtWatchdogLocal.WatchdogHeartbeatHandler(HeartbeatSink);
                            system.WatchdogLocal.Notification -= new AmtWatchdogLocal.NotificationHandler(NotificationSink);
                            system.WatchdogLocal.Stop();
                        }

                        system.Dispose();
                        system = null;
                    }
                    break;
                case AmtSystemObjState.Connected:
                    connected = true;
                    Application.UserAppDataRegistry.SetValue("Username", usernameTextBox.Text);
                    Application.UserAppDataRegistry.SetValue("Password", passwordControl.Text);
                    Application.UserAppDataRegistry.SetValue("UseTlsFirst", system.UseTls.ToString());
                    if (system.WatchdogLocal != null)
                    {
                        watchDogListView.Enabled = true;
                        heartbeatCheckBox.Enabled = true;

                        if (watchDogListView != null && watchDogListView.Items.Count > 0)
                        {
                            foreach (ListViewItem l in watchDogListView.Items)
                            {
                                AmtWatchdogLocal.AmtWatchdog wg = (AmtWatchdogLocal.AmtWatchdog)l.Tag;
                                system.WatchdogLocal.AddWatchdog(wg);
                            }
                        }
                        system.WatchdogLocal.WatchdogHeartbeat += new AmtWatchdogLocal.WatchdogHeartbeatHandler(HeartbeatSink);
                        system.WatchdogLocal.Notification += new AmtWatchdogLocal.NotificationHandler(NotificationSink);
                        if (heartbeatCheckBox.Checked) system.WatchdogLocal.Start();
                    }
                    if (serialAgent != null) serialAgent.system = system;
                    break;
            }

            if (closing == false)
            {
                refreshButton_Click(this, null);
                refreshRedirectLogButton_Click(this, null);
            }
        }

        private void connectButton_Click(object sender, EventArgs e)
        {
            if (system == null)
            {
                // Connect a new system object
                system = new AmtSystem(Utils.GetLocalConnectFQDN(), 16992, usernameTextBox.Text, passwordControl.Text, false, false);
                system.UseTls = (bool)Application.UserAppDataRegistry.GetValue("UseTlsFirst", bool.FalseString).ToString().Equals(bool.TrueString); ;
                system.LocalConnection = true;
                system.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
                system.OnFetchCompleted += new AmtSystem.FetchHandler(FetchHandler);
                system.Connect();
            }
            else if (system != null && system != null && system.State == AmtSystemObjState.Connected)
            {
                // De-register all watchdogs
                foreach (ListViewItem l in watchDogListView.Items)
                {
                    AmtWatchdogLocal.AmtWatchdog wg = (AmtWatchdogLocal.AmtWatchdog)l.Tag;
                    system.WatchdogLocal.RemoveWatchdog(wg);
                    l.SubItems[1].Text = "";
                    l.SubItems[2].Text = "";
                    l.SubItems[3].Text = "";
                }

                // Disconnect the computer
                system.Disconnect();
            }
        }

        private void FetchHandler(AmtSystem computer, object fetch)
        {
            if (computer == null || fetch == null) return;
            Invoke(new AmtSystem.FetchHandler(FetchHandlerEx), new object[] { computer, fetch });
        }

        private void FetchHandlerEx(AmtSystem computer, object fetch)
        {
            if (fetch == computer.Events) UpdateEventLog();
        }

        private void refreshButton_Click(object sender, EventArgs e)
        {
            connectButton.Enabled = (system == null || system.State != AmtSystemObjState.Connecting);
            if (system == null || system.State == AmtSystemObjState.Disconnected) connectButton.Text = Properties.Resources.MainForm_ConBtn_State_Con;
            else if (system.State == AmtSystemObjState.Connecting) connectButton.Text = Properties.Resources.ConnectingButton;
            else if (system.State == AmtSystemObjState.Connected) connectButton.Text = Properties.Resources.MainForm_ConBtn_State_Discon;

            usernameTextBox.Enabled = (system == null);
            passwordControl.Enabled = (system == null);
            if (system != null) passwordControl.Hidden = true;
            refreshRedirectLogButton.Enabled = (system != null);
            redirectLogTimer.Enabled = (system != null) && updateRedirectLogCheckBox.Checked;

            if (system == null || system.State != AmtSystemObjState.Connected)
            {
                connectionWarningGroupBox.Visible = false;
                storageTreeView.Enabled = false;
                storageInfoTextBox.Clear();
                storageInfoTextBox.Enabled = false;
            }

            if (system != null && system.State == AmtSystemObjState.Connected)
            {
                storageTreeView.Enabled = true;
                storageInfoTextBox.Enabled = true;
            }

            try
            {
                addButton.Enabled = (system != null && system.WatchdogLocal != null);

                if (system != null)
                {
                    if (system.UseTls)
                    {
                        // Display the padlock icon
                        padlockPanel.Visible = true;

                        if (system.Warnings != 0)
                        {
                            connectionWarningGroupBox.Visible = (system.Warnings != 0);
                            connectionWarningTextBox.Text = "";
                            if ((system.Warnings & AmtSystem.AmtConnectWarnings.NameMismatch) != 0) connectionWarningTextBox.Text += Properties.Resources.ConnectionNameDnsnameCertificateMismatch + "\r\n";
                            if ((system.Warnings & AmtSystem.AmtConnectWarnings.CertificateExpired) != 0) connectionWarningTextBox.Text += Properties.Resources.ComputerCertificateExpired + "\r\n";
                            if ((system.Warnings & AmtSystem.AmtConnectWarnings.CertificateInvalid) != 0) connectionWarningTextBox.Text += string.Format(Properties.Resources.ComputerCertificateInvalid + "\r\n", system.CertificateError.ToString());
                            if ((system.Warnings & AmtSystem.AmtConnectWarnings.CertificateUntrusted) != 0) connectionWarningTextBox.Text += Properties.Resources.ComputerCertificateUntrusted + "\r\n";
                            if ((system.Warnings & AmtSystem.AmtConnectWarnings.AuthPrivateKeyNotExportable) != 0) connectionWarningTextBox.Text += Properties.Resources.AuthCertificatePrivateKeyNoExport + "\r\n";
                            if ((system.Warnings & AmtSystem.AmtConnectWarnings.OpenSslNotAvailable) != 0) connectionWarningTextBox.Text += Properties.Resources.OpenSslNotAvailable + "\r\n";
                        }
                    }
                    else
                    {
                        // Hide the padlock icon
                        padlockPanel.Visible = false;
                        connectionWarningGroupBox.Visible = false;
                    }

                    // If it's not hidden yet, hide the password
                    passwordControl.Hidden = true;

                    if (system.Info == null)
                    {
                        // Not general info service
                        for (int i = 0; i < generalInfoListView.Items.Count; i++) generalInfoListView.Items[i].SubItems[1].Text = "-";
                    }
                    else 
                    {
                        string hostName = system.Info.GetHostName();
                        if (hostName != null) generalInfoListView.Items[0].SubItems[1].Text = hostName;
                        generalInfoListView.Items[1].SubItems[1].Text = system.Info.GetCoreVersion();

                        string EnabledStr = ManageabilityOutpostTool.Properties.Resources.MainForm_Tag_Enable;
                        string DisabledStr = ManageabilityOutpostTool.Properties.Resources.MainForm_Tag_Disable;

                        AmtSecurityAdmin.EnabledInterfaces eif = system.Info.GetEnabledInterfaces();
                        if (eif.WebUI) generalInfoListView.Items[5].SubItems[1].Text = EnabledStr; else generalInfoListView.Items[5].SubItems[1].Text = DisabledStr;

                        PasswordModelType pm = system.Info.GetPasswordModel();
                        string[] pmstr = new string[] { Properties.Resources.PasswordModelCoupled, Properties.Resources.PasswordModelSeparate, Properties.Resources.PasswordModelSeparateHash };
                        generalInfoListView.Items[8].SubItems[1].Text = pmstr[(int)pm];

                        bool EnterpriseMode;
                        bool TlsEnabled;
                        bool HwCryptoEnabled;
                        ProvisioningStateType ProvisioningState;
                        bool NetworkInterfaceEnabled;
                        bool SOLEnabled;
                        bool IDEREnabled;
                        bool FWUpdateEnabled;
                        bool LinkIsUp;

                        system.Info.GetSecurityParameters(out EnterpriseMode, out TlsEnabled, out HwCryptoEnabled, out ProvisioningState, out NetworkInterfaceEnabled, out SOLEnabled, out IDEREnabled, out FWUpdateEnabled, out LinkIsUp);

                        if (system.WsMan == null) generalInfoListView.Items[2].SubItems[1].Text = Properties.Resources.SOAPonly;
                        else if (system.WsManOnly == false) generalInfoListView.Items[2].SubItems[1].Text = Properties.Resources.WSMANandSOAP;
                        else generalInfoListView.Items[2].SubItems[1].Text = Properties.Resources.WSMANonly;

                        if (IDEREnabled) generalInfoListView.Items[3].SubItems[1].Text = EnabledStr; else generalInfoListView.Items[3].SubItems[1].Text = DisabledStr;
                        if (SOLEnabled) generalInfoListView.Items[4].SubItems[1].Text = EnabledStr; else generalInfoListView.Items[4].SubItems[1].Text = DisabledStr;
                        if (TlsEnabled) generalInfoListView.Items[11].SubItems[1].Text = EnabledStr; else generalInfoListView.Items[11].SubItems[1].Text = DisabledStr;
                        if (HwCryptoEnabled) generalInfoListView.Items[10].SubItems[1].Text = EnabledStr; else generalInfoListView.Items[10].SubItems[1].Text = DisabledStr;
                        if (NetworkInterfaceEnabled) generalInfoListView.Items[6].SubItems[1].Text = EnabledStr; else generalInfoListView.Items[6].SubItems[1].Text = DisabledStr;
                        if (FWUpdateEnabled) generalInfoListView.Items[12].SubItems[1].Text = EnabledStr; else generalInfoListView.Items[12].SubItems[1].Text = DisabledStr;

                        generalInfoListView.Items[9].SubItems[1].Text = system.Info.GetProvisioningMode().ToString();

                        bool VlanMode;
                        ushort VlanTag;
                        system.Info.GetVlanParameters(out VlanMode, out VlanTag);

                        if (VlanMode)
                        {
                            generalInfoListView.Items[7].SubItems[1].Text = ManageabilityOutpostTool.Properties.Resources.MainForm_Tag_Enable_Ext + VlanTag.ToString();
                        }
                        else
                        {
                            generalInfoListView.Items[7].SubItems[1].Text = DisabledStr;
                        }

                        if (system.CoreVersion < 2.5 || system.NetworkAccessControl == null)
                        {
                            generalInfoListView.Items[13].SubItems[1].Text = Properties.Resources.Unsupported;
                            generalInfoListView.Items[14].SubItems[1].Text = Properties.Resources.Unsupported;
                            generalInfoListView.Items[15].SubItems[1].Text = Properties.Resources.Unsupported;
                        }
                        else
                        {
                            system.NetworkAccessControl.ClearCache();

                            bool eacEnabled;
                            system.NetworkAccessControl.GetEACStatus(out eacEnabled);
                            if (eacEnabled) generalInfoListView.Items[13].SubItems[1].Text = EnabledStr; else generalInfoListView.Items[13].SubItems[1].Text = DisabledStr;

                            if (eacEnabled == true)
                            {
                                byte[] h1, h2;
                                system.NetworkAccessControl.GetPosture(out h1, out h2);
                                string hs1 = Properties.Resources.Null;
                                if (h1 != null && h1.Length > 0) hs1 = "0x" + Utils.BytesToHex(h1);
                                string hs2 = Properties.Resources.Null;
                                if (h2 != null && h2.Length > 0) hs2 = "0x" + Utils.BytesToHex(h2);
                                generalInfoListView.Items[14].SubItems[1].Text = hs1 + ", " + hs2;

                                byte[] hash;
                                system.NetworkAccessControl.GetPostureHash(out hash);
                                if (hash == null) generalInfoListView.Items[15].SubItems[1].Text = Properties.Resources.Null; else generalInfoListView.Items[15].SubItems[1].Text = "0x" + Utils.BytesToHex(hash);
                            }
                            else
                            {
                                generalInfoListView.Items[14].SubItems[1].Text = DisabledStr;
                                generalInfoListView.Items[16].SubItems[1].Text = DisabledStr;
                            }
                        }
                    }
                }
                else
                {
                    // Hide the padlock icon
                    padlockPanel.Visible = false;
                    for (int i = 0; i < generalInfoListView.Items.Count; i++) generalInfoListView.Items[i].SubItems[1].Text = "-";
                }

                UpdateEventLog();
            }
            catch (Exception)
            {
                if (system == null) connectButton_Click(this, null);
            }
        }

        private void addButton_Click(object sender, EventArgs e)
        {
            if (system == null || system.WatchdogLocal == null) return;
            AddWatchdogForm f = new AddWatchdogForm();
            if (f.ShowDialog(this) == DialogResult.OK)
            {
                AmtWatchdogLocal.AmtWatchdog wg = new AmtWatchdogLocal.AmtWatchdog(f.AgentName, f.AgentID);
                wg.processCondition = f.AgentProcessCondition;
                string s = "";
                if (wg.processCondition != null) s = String.Format(" ({0})", wg.processCondition );
                ListViewItem l = new ListViewItem(new String[] { wg.ToString() + s, "", "", "" });
                l.Tag = wg;
                system.WatchdogLocal.AddWatchdog(wg);
                watchDogListView.Items.Add(l);
                SaveWatchdogs();
            }
        }

        private void SaveWatchdogs()
        {
            string data = "";
            foreach (ListViewItem l in watchDogListView.Items)
            {
                if (l != null)
                {
                    AmtWatchdogLocal.AmtWatchdog wg = (AmtWatchdogLocal.AmtWatchdog)l.Tag;
                    string condition = "";
                    if (wg.processCondition != null) condition = wg.processCondition;
                    string active = "0";
                    if (l.Checked == true) active = "1";
                    data += wg.agentID.ToString() + "~" + wg.name + "~" + condition + "~" + active + "~";
                }
            }
            Application.UserAppDataRegistry.SetValue("WatchdogsEx", data);
        }

        private void LoadWatchdogs()
        {
            string data = (string)Application.UserAppDataRegistry.GetValue("WatchdogsEx", "");
            if (data.Length < 3) return;
            string[] datas = data.Split('~');
            for (int i = 0; i < (datas.Length-1); i+=4)
            {
                try
                {
                    string active = datas[i + 3];
                    string condition = datas[i + 2];
                    string name = datas[i + 1];
                    string ids = datas[i];
                    string s = "";
                    Guid id = new Guid(ids);
                    AmtWatchdogLocal.AmtWatchdog wg = new AmtWatchdogLocal.AmtWatchdog(name, id);
                    wg.beat = active.Equals("1");
                    if (condition.Length > 0) wg.processCondition = condition;
                    if (wg.processCondition != null) s = String.Format(" ({0})" ,wg.processCondition );
                    ListViewItem l = new ListViewItem(new String[] { wg.ToString() + s, "", "", "" });
                    l.Checked = wg.beat;
                    l.Tag = wg;
                    watchDogListView.Items.Add(l);
                }
                catch (Exception) { }
            }
        }

        private void watchDogListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            removeButton.Enabled = (watchDogListView.SelectedItems.Count != 0);
            editButton.Enabled = (watchDogListView.SelectedItems.Count != 0);
        }

        private void removeButton_Click(object sender, EventArgs e)
        {
            if (system == null || system.WatchdogLocal == null) return;
            if (watchDogListView.SelectedItems.Count != 1) return;
            ListViewItem l = watchDogListView.SelectedItems[0];
            AmtWatchdogLocal.AmtWatchdog wg = (AmtWatchdogLocal.AmtWatchdog)l.Tag;
            system.WatchdogLocal.RemoveWatchdog(wg);
            l.Tag = null;
            watchDogListView.Items.Remove(l);
            SaveWatchdogs();
        }

        private void refreshRedirectLogButton_Click(object sender, EventArgs e)
        {
            redirectLogListView.BeginUpdate();
            redirectLogListView.Items.Clear();
            if (system == null || system.Info == null)
            {
                redirectLogListView.Enabled = false;
                refreshRedirectLogButton.Enabled = false;
                updateRedirectLogCheckBox.Enabled = false;
                return;
            }
            else
            {
                redirectLogListView.Enabled = true;
                refreshRedirectLogButton.Enabled = true;
                updateRedirectLogCheckBox.Enabled = true;
            }

            try
            {
                AmtRedirectEvent[] log = system.Info.GetIderSessionLog();
                if (log != null & log.Length != 0)
                {
                    ListViewGroup currentGroup = null;
                    string currentGroupStr = "";

                    foreach (AmtRedirectEvent entry in log)
                    {
                        IPAddress addr = entry.ConsoleAddress.Address;
                        DateTime time = entry.TimeStamp;

                        if (currentGroupStr.Equals(time.ToLongDateString()) == false)
                        {
                            currentGroup = new ListViewGroup(time.ToLongDateString());
                            redirectLogListView.Groups.Add(currentGroup);
                            currentGroupStr = time.ToLongDateString();
                        }
                        ListViewItem l = new ListViewItem(new String[] { time.ToLongTimeString(), addr.ToString() + ":" + entry.ConsoleAddress.Port }, 0);
                        l.Group = currentGroup;
                        redirectLogListView.Items.Insert(0, l);
                    }
                }
            }
            catch (Exception)
            {
                connectButton_Click(this, null);
            }

            redirectLogListView.EndUpdate();
        }

        private void updateRedirectLogCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            redirectLogTimer.Enabled = (system != null) && updateRedirectLogCheckBox.Checked;
            Application.UserAppDataRegistry.SetValue("UpdateRedirectLog", updateRedirectLogCheckBox.Checked.ToString());
        }

        private void heartbeatCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            Application.UserAppDataRegistry.SetValue("Heartbeat", heartbeatCheckBox.Checked.ToString());
            if (system != null && system.WatchdogLocal != null)
            {
                if (heartbeatCheckBox.Checked) { system.WatchdogLocal.Start(); } else { system.WatchdogLocal.Stop(); }
            }
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            closing = true;

            if (system != null)
            {
                system.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
                system.OnFetchCompleted -= new AmtSystem.FetchHandler(FetchHandler);
            }

            if (system != null && system.State == AmtSystemObjState.Connected)
            {
                if (system.WatchdogLocal != null)
                {
                    // Stop events and heartbeats
                    system.WatchdogLocal.WatchdogHeartbeat -= new AmtWatchdogLocal.WatchdogHeartbeatHandler(HeartbeatSink);
                    system.WatchdogLocal.Notification -= new AmtWatchdogLocal.NotificationHandler(NotificationSink);
                    system.WatchdogLocal.Stop();

                    // De-register all watchdogs
                    foreach (ListViewItem l in watchDogListView.Items)
                    {
                        AmtWatchdogLocal.AmtWatchdog wg = (AmtWatchdogLocal.AmtWatchdog)l.Tag;
                        system.WatchdogLocal.RemoveWatchdog(wg);
                        l.SubItems[1].Text = "";
                        l.SubItems[2].Text = "";
                        l.SubItems[3].Text = "";
                    }
                }

                // Disconnect the computer
                system.Disconnect();
            }

            SystemEvents.PowerModeChanged -= new PowerModeChangedEventHandler(SystemEvents_PowerModeChanged);
            
            if (system != null)
            {
                if (system.WatchdogLocal != null)
                {
                    system.WatchdogLocal.WatchdogHeartbeat -= new AmtWatchdogLocal.WatchdogHeartbeatHandler(HeartbeatSink);
                    system.WatchdogLocal.Notification -= new AmtWatchdogLocal.NotificationHandler(NotificationSink);
                }
                system.Dispose();
                system = null;
            }
            if (serialAgent != null)
            {
                serialAgent.Dispose();
                serialAgent = null;
            }
        }

        private void NotificationSink(AmtSystem system, int notificationCode)
        {
            this.Invoke(new AmtWatchdogLocal.NotificationHandler(NotificationSinkEx), new object[] { system, notificationCode });
        }

        private void NotificationSinkEx(AmtSystem system, int notificationCode)
        {
            Application.DoEvents();

            string msg = null;
            switch (notificationCode)
            {
                case 100:
                    mainToolStripStatusLabel.Text = Properties.Resources.ActionGoingToSleep;
                    Application.DoEvents();
                    ShutdownClass.DoExitWin(ShutdownClass.SLEEP, false, ref msg); // Suggest goto sleep
                    break;
                case 101:
                    mainToolStripStatusLabel.Text = Properties.Resources.ActionShutdown;
                    Application.DoEvents();
                    ShutdownClass.DoExitWin(ShutdownClass.EWX_SHUTDOWN, false, ref msg); // Suggest shutdown
                    break;
                case 102:
                    mainToolStripStatusLabel.Text = Properties.Resources.ActionLoggingOff;
                    Application.DoEvents();
                    ShutdownClass.DoExitWin(ShutdownClass.EWX_LOGOFF, false, ref msg); // Suggest logoff
                    break;
                case 103:
                    mainToolStripStatusLabel.Text = Properties.Resources.ActionRestarting;
                    Application.DoEvents();
                    ShutdownClass.DoExitWin(ShutdownClass.EWX_REBOOT, false, ref msg); // Suggest reboot
                    break;
            }
        }

        private void HeartbeatSink(AmtSystem system, AmtWatchdogLocal localAgent, AmtWatchdogLocal.AmtWatchdog watchdog, uint sequenceNumber, AmtCallStatus result)
        {
            this.Invoke(new AmtWatchdogLocal.WatchdogHeartbeatHandler(HeartbeatSinkEx), new object[] {system, localAgent, watchdog, sequenceNumber, result });
        }

        private void HeartbeatSinkEx(AmtSystem system, AmtWatchdogLocal localAgent, AmtWatchdogLocal.AmtWatchdog watchdog, uint sequenceNumber, AmtCallStatus result)
        {
            foreach (ListViewItem l in watchDogListView.Items)
            {
                if (l.Tag == watchdog)
                {
                    l.SubItems[1].Text = watchdog.sessionSequenceNumber.ToString();
                    l.SubItems[2].Text = watchdog.agentHeartbeatTime.ToString();
                    switch (result)
                    {
                        case AmtCallStatus.SUCCESS:
                            l.SubItems[3].Text = ManageabilityOutpostTool.Properties.Resources.MainForm_CallStatus_OK;
                            break;
                        case AmtCallStatus.CALL_SKIPPED:
                            l.SubItems[3].Text = ManageabilityOutpostTool.Properties.Resources.MainForm_CallStatus_Skipped;
                            break;
                        case AmtCallStatus.INVALID_HANDLE:
                            l.SubItems[3].Text = ManageabilityOutpostTool.Properties.Resources.MainForm_CallStatus_NoExist;
                            break;
                        default:
                            l.SubItems[3].Text = ManageabilityOutpostTool.Properties.Resources.MainForm_CallStatus_Code + result.ToString();
                            break;
                    }
                }
            }
        }

        private void editButton_Click(object sender, EventArgs e)
        {
            if (system == null || system.WatchdogLocal == null) return;
            if (watchDogListView.SelectedItems.Count != 1) return;
            ListViewItem l = watchDogListView.SelectedItems[0];
            AmtWatchdogLocal.AmtWatchdog wg = (AmtWatchdogLocal.AmtWatchdog)l.Tag;
            AddWatchdogForm f = new AddWatchdogForm(wg);
            if (f.ShowDialog(this) == DialogResult.OK)
            {
                system.WatchdogLocal.RemoveWatchdog(wg);
                wg.agentID = f.AgentID;
                wg.name = f.AgentName;
                wg.processCondition = f.AgentProcessCondition;
                system.WatchdogLocal.AddWatchdog(wg);
                l.SubItems[0].Text = f.AgentName;
                l.Tag = wg;
                SaveWatchdogs();
            }
        }

        private void watchDogListView_ItemChecked(object sender, ItemCheckedEventArgs e)
        {
            foreach (ListViewItem l in watchDogListView.Items)
            {
                if (l != null)
                {
                    AmtWatchdogLocal.AmtWatchdog wg = (AmtWatchdogLocal.AmtWatchdog)l.Tag;
                    wg.beat = l.Checked;
                }
            }
            SaveWatchdogs();
        }

        private bool RemoteCertificateValidationCallbackSink(object sender, X509Certificate certificate, X509Chain chain, System.Net.Security.SslPolicyErrors sslPolicyErrors)
        {
            if (sender.GetType() == typeof(System.Net.HttpWebRequest))
            {
                System.Net.HttpWebRequest r = (System.Net.HttpWebRequest)sender;
                X509Certificate2[] certArray = new X509Certificate2[chain.ChainElements.Count];
                for (int i = 0; i < chain.ChainElements.Count; i++)
                {
                    certArray[i] = chain.ChainElements[i].Certificate;
                }
                if (r.ConnectionGroupName != null) AmtSystem.AddTlsCertificateCache(r.ConnectionGroupName, certArray);
            }
            return true; // Accept All Certificates
        }

        private void certificateManagerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new ManageabilityStack.CertificateManagerForm().ShowDialog(this);
        }


        /// <summary>
        /// This class is used to add items to DropDown combo boxes and still be able to reference the item quickly.
        /// </summary>
        public class DropDownItem
        {
            public string Text = null;
            public object Tag = null;
            public int Handle = 0;
            public override string ToString() { return Text; }

            public DropDownItem()
            {
            }

            public DropDownItem(string Text, int Handle, object Tag)
            {
                this.Text = Text;
                this.Handle = Handle;
                this.Tag = Tag;
            }
        }

        private void ChangedWatchdogStateSink()
        {
            Invoke(new AmtSerialAgent.ChangedWatchdogStateHandler(ChangedWatchdogStateSinkEx));
        }

        private void ChangedWatchdogStateSinkEx()
        {
            watchDogListView.BeginUpdate();
            watchDogListView.Items.Clear();
            AmtWatchdogLocal.AmtWatchdog[] wgs = system.WatchdogLocal.GetWatchdogs();

            foreach (AmtWatchdogLocal.AmtWatchdog wg in wgs)
            {
                string s = "";
                if (wg.processCondition != null) s = String.Format(" ({0})", wg.processCondition);
                ListViewItem l = new ListViewItem(new String[] { wg.ToString() + s, "", "", "" });
                l.Tag = wg;
                l.Checked = wg.beat;
                system.WatchdogLocal.AddWatchdog(wg);
                watchDogListView.Items.Add(l);
                HeartbeatSinkEx(system, system.WatchdogLocal, wg, 0, wg.lastState);
            }

            watchDogListView.EndUpdate();
        }

        public void SerialOutputSink(AmtSerialAgent sender, byte[] buf, int offset, int length)
        {
            if (terminal != null && terminal.TerminalDisposed == false)
            {
                terminal.ProcessChar(buf, offset, length);
            }
        }

        private void serialAgentCheckBox_Click(object sender, EventArgs e)
        {
            if (serialAgent == null)
            {
                string SOLPortName = DevDriCtlClass.GetSOLPortName();
                if (SOLPortName != null)
                {
                    try
                    {
                        serialAgent = new AmtSerialAgent(SOLPortName, AgentVersionString);
                        serialAgent.ChangedWatchdogState += new AmtSerialAgent.ChangedWatchdogStateHandler(ChangedWatchdogStateSink);
                        serialAgent.SerialOutputEvent += new AmtSerialAgent.SerialOutputHandler(SerialOutputSink);
                        serialAgent.system = system;
                        serialAgent.chatForm = new ChatWindowForm(this, serialAgent.serial, serialAgent);
                        Application.UserAppDataRegistry.SetValue("SerialAgent", bool.TrueString);
                        terminalButton.Enabled = true;
                    }
                    catch (UnauthorizedAccessException)
                    {
                        MessageBox.Show(this, Properties.Resources.SolPortInUse, Properties.Resources.SerialAgentTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
                else
                {
                    MessageBox.Show(this, Properties.Resources.SolPortNotFound, Properties.Resources.SerialAgentTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
            {
                terminalButton.Enabled = false;
                if (terminal != null)
                {
                    terminal.UserInputEvent -= new ManageabilityControls.TerminalForm.TerminalInputHandler(TerminalUserInputSink);
                    terminal.Close();
                    terminal.Dispose();
                    terminal = null;
                }

                serialAgent.ChangedWatchdogState -= new AmtSerialAgent.ChangedWatchdogStateHandler(ChangedWatchdogStateSink);
                serialAgent.SerialOutputEvent -= new AmtSerialAgent.SerialOutputHandler(SerialOutputSink);
                serialAgent.system = null;
                serialAgent.Dispose();
                serialAgent = null;
                Application.UserAppDataRegistry.SetValue("SerialAgent", bool.FalseString);
            }

            serialAgentCheckBox.Checked = (serialAgent != null);
        }

        private void UpdatePermissionsListView()
        {
            for (int i = 0; i < (int)SerialAgentPermissionCtl.Permissions.MaxDummy; i++)
            {
                string[] tmp = new string[] { SerialAgentPermissionCtl.PermissionDesc[i], 
                    SerialAgentPermissionCtl.PermissionStatus[SerialAgentPermissionCtl.GetPermission((SerialAgentPermissionCtl.Permissions)i)]};
                serialAgentPermissionsListView.Items.Insert(i, new ListViewItem(tmp));
            }
        }

        private void allowButton_Click(object sender, EventArgs e)
        {
            foreach (ListViewItem l in serialAgentPermissionsListView.SelectedItems)
            {
                SerialAgentPermissionCtl.SetPermission((SerialAgentPermissionCtl.Permissions)l.Index, 1);
                l.SubItems[1].Text = Properties.Resources.Allow;
            }
            ServiceCtlClass.SendCommand(ServiceCtlClass.CC_PERMISSION_CHANGED);
            serialAgentPermissionsListView.SelectedItems.Clear();
        }

        private void denyButton_Click(object sender, EventArgs e)
        {
            foreach (ListViewItem l in serialAgentPermissionsListView.SelectedItems)
            {
                SerialAgentPermissionCtl.SetPermission((SerialAgentPermissionCtl.Permissions)l.Index, 0);
                l.SubItems[1].Text = Properties.Resources.Deny;
            }
            ServiceCtlClass.SendCommand(ServiceCtlClass.CC_PERMISSION_CHANGED);
            serialAgentPermissionsListView.SelectedItems.Clear();
        }

        private void ChangeLanguage(string lang)
        {
            if (System.Threading.Thread.CurrentThread.CurrentUICulture == new System.Globalization.CultureInfo(lang)) return;
            if (MessageBox.Show(this, Properties.Resources.LanguagesChanging, Properties.Resources.LanguagesTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                System.Threading.Thread.CurrentThread.CurrentUICulture = new System.Globalization.CultureInfo(lang);
                Close();
            }
        }

        private void englishToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ChangeLanguage("en");
        }

        private void simplifiedChineseToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ChangeLanguage("zh-CHS");
        }

        private void japaneseToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ChangeLanguage("ja-JP");
        }

        private void frenchToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ChangeLanguage("fr");
        }

        private void koreanToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ChangeLanguage("ko");
        }

        private void addAppButton_Click(object sender, EventArgs e)
        {
            AddStorageApplicationForm form = new AddStorageApplicationForm();
            if (form.ShowDialog(this) == DialogResult.OK)
            {
                LocalStorageInfo store = new LocalStorageInfo(form.Enterprise, form.Vendor, form.AppName, form.UUID);
                TreeNode n = new TreeNode(string.Format(Properties.Resources.StorageNameConcat, form.Enterprise, form.Vendor, form.AppName), 4, 5);
                n.Tag = store;
                
                if (system != null)
                {
                    if (store.Connect(system) == true)
                    {
                        string str = Properties.Resources.Disconnected;
                        if (store.Connected) str = Properties.Resources.Connected;
                        n.Text = string.Format(Properties.Resources.StorageStateAndNameConcat, str, form.Enterprise, form.Vendor, form.AppName);
                    }
                }
                
                storageTreeView.Nodes.Add(n);
                SaveStorageSettings();
                showStorageBlocks();
            }
        }

        private void hideStorageBlocks()
        {
            foreach (TreeNode n in storageTreeView.Nodes) n.Nodes.Clear();
        }

        private void showStorageBlocks()
        {
            storageTreeView.BeginUpdate();
            hideStorageBlocks();
            ArrayList list = new ArrayList();
            foreach (TreeNode n in storageTreeView.Nodes)
            {
                if (n.Tag.GetType() == typeof(LocalStorageInfo)) list.Add(n);
            }
            foreach (TreeNode n in list)
            {
                LocalStorageInfo s = (LocalStorageInfo)n.Tag;
                string constr = Properties.Resources.Disconnected;
                if (s.Connected) constr = Properties.Resources.Connected;
                n.Text = constr + ": " + s.Enterprise + ", " + s.Vendor + ", " + s.AppName;
                if (s.Storage != null && s.Storage.Connected == true)
                {
                    AmtStorageApplication[] apps = s.GetApplications();
                    if (apps != null)
                    {
                        foreach (AmtStorageApplication app in apps)
                        {
                            string self = "";
                            if (app.Handle == s.Storage.SessionHandle) self = " " + Properties.Resources.StorageBlockOwner;
                            TreeNode na = new TreeNode(app.Vendor + ", " + app.Application + self, 4, 5);
                            na.Tag = app;
                            n.Nodes.Add(na);

                            AmtStorageBlock[] blocks = s.Storage.GetBlockList(app.Handle);
                            if (blocks != null)
                            {
                                foreach (AmtStorageBlock block in blocks)
                                {
                                    //string chunk = "chunk";
                                    //if (block.Size > 4096) chunk = "chunks";
                                    string visible = "";
                                    if (block.Hidden) visible = " " + Properties.Resources.StorageBlockHidden;

                                    /*
                                    // Get permission groups (Debug)
                                    string groups = ", No Permission Groups";
                                    AmtStorageWrapper.AmtStoragePermissionsGroup[] pgroups;
                                    block.GetPermissionsGroups(out pgroups);
                                    if (pgroups.Length == 1) groups = ", 1 Permission Group";
                                    if (pgroups.Length > 1) groups = ", " + pgroups.Length + " Permission Groups";
                                    */

                                    TreeNode nn = new TreeNode(block.Name + string.Format(" - {0}k{1}", block.Size / 1024, visible), 1, 1);
                                    nn.Tag = block;
                                    na.Nodes.Add(nn);
                                }
                            }
                        }
                    }
                }
            }
            storageTreeView.ExpandAll();
            storageTreeView.EndUpdate();
            storageTreeView_AfterSelect(this, null);
        }

        private void storageRemoveButton_Click(object sender, EventArgs e)
        {
            TreeNode selectedNode = storageTreeView.SelectedNode;
            if (selectedNode == null) return;
            
            if (selectedNode.Tag.GetType() == typeof(LocalStorageInfo))
            {
                if (MessageBox.Show(this, Properties.Resources.RemoteApplicationPrompt, Properties.Resources.RemoveApplicationTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    LocalStorageInfo s = (LocalStorageInfo)selectedNode.Tag;
                    storageTreeView.SelectedNode.Remove();
                    SaveStorageSettings();
                    showStorageBlocks();
                }
            }
            if (selectedNode.Tag.GetType() == typeof(AmtStorageApplication))
            {
                AmtStorageApplication app = (AmtStorageApplication)selectedNode.Tag;
                LocalStorageInfo s = (LocalStorageInfo)selectedNode.Parent.Tag;
                // TODO (Not sure what to do here).
            }
            else if (selectedNode.Tag.GetType() == typeof(AmtStorageBlock))
            {
                if (MessageBox.Show(this, Properties.Resources.DeallocateStorageBlockPrompt, Properties.Resources.RemoveBlockTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    AmtStorageBlock block = (AmtStorageBlock)selectedNode.Tag;
                    AmtStorageWrapper.AmtStorageStatus r = block.Remove();
                    if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
                    {
                        MessageBox.Show(this, string.Format(Properties.Resources.DeallocateStorageBlockError, r.ToString()), Properties.Resources.RemoveBlockTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    else
                    {
                        showStorageBlocks();
                    }
                }
            }

        }

        private void LoadStorageSettings()
        {
            storageTreeView.Nodes.Clear();
            byte[] buf = (byte[])Application.UserAppDataRegistry.GetValue("StorageSettings", null);
            if (buf != null)
            {
                storageTreeView.BeginUpdate();
                try
                {
                    ArrayList lst = (ArrayList)UnserializeBinary(buf);
                    foreach (LocalStorageInfo s in lst)
                    {
                        TreeNode n = new TreeNode(Properties.Resources.Disconnected + ": " + s.Enterprise + ", " + s.Vendor + ", " + s.AppName, 4, 5);
                        n.Tag = s;
                        storageTreeView.Nodes.Add(n);
                    }
                }
                catch (Exception) { }
                storageTreeView.EndUpdate();
            }
        }

        private void SaveStorageSettings()
        {
            ArrayList lst = new ArrayList();
            foreach (TreeNode n in storageTreeView.Nodes)
            {
                if (n.Tag.GetType() == typeof(LocalStorageInfo)) lst.Add(n.Tag);
            }
            Application.UserAppDataRegistry.SetValue("StorageSettings", SerializeBinary(lst));
        }

        public static byte[] SerializeBinary(object o)
        {
            System.Runtime.Serialization.Formatters.Binary.BinaryFormatter serializer = new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter();
            MemoryStream m = new MemoryStream();
            serializer.Serialize(m, o);
            return m.ToArray();
        }

        public static object UnserializeBinary(byte[] buf)
        {
            try
            {
                MemoryStream m = new MemoryStream();
                m.Write(buf, 0, buf.Length);
                m.Seek(0, SeekOrigin.Begin);
                System.Runtime.Serialization.Formatters.Binary.BinaryFormatter serializer = new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter();
                return serializer.Deserialize(m);
            }
            catch (Exception) { return null; }
        }

        private void storageListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                storageRemoveButton_Click(this, null);
                e.Handled = true;
            }
        }

        private void ConnectStorage()
        {
            foreach (TreeNode n in storageTreeView.Nodes)
            {
                if (n.Tag.GetType() == typeof(LocalStorageInfo))
                {
                    LocalStorageInfo s = (LocalStorageInfo)n.Tag;
                    if (s.Connect(system) == true)
                    {
                        string constr = Properties.Resources.Disconnected;
                        if (s.Connected) constr = Properties.Resources.Connected;
                        n.Text = string.Format(Properties.Resources.StorageStateAndNameConcat, constr, s.Enterprise, s.Vendor, s.AppName);
                    }
                }
            }
            showStorageBlocks();
        }

        private void DisconnectStorage()
        {
            hideStorageBlocks();
            foreach (TreeNode n in storageTreeView.Nodes)
            {
                if (n.Tag.GetType() == typeof(LocalStorageInfo))
                {
                    LocalStorageInfo s = (LocalStorageInfo)n.Tag;
                    s.Disconnect();
                    string constr = Properties.Resources.Disconnected;
                    if (s.Connected) constr = Properties.Resources.Connected;
                    n.Text = string.Format(Properties.Resources.StorageStateAndNameConcat, constr, s.Enterprise, s.Vendor, s.AppName);
                }
            }
        }

        private void terminalButton_Click(object sender, EventArgs e)
        {
            if (terminal != null && terminal.TerminalDisposed == false)
            {
                terminal.Activate();
                return;
            }
            if (terminal != null && terminal.TerminalDisposed == true)
            {
                terminal.UserInputEvent -= new ManageabilityControls.TerminalForm.TerminalInputHandler(TerminalUserInputSink);
                terminal.Dispose();
                terminal = null;
            }
            if (serialAgent == null) return;
            terminal = new ManageabilityControls.TerminalForm(Properties.Resources.LocalTerminalTitle);
            terminal.UserInputEvent += new ManageabilityControls.TerminalForm.TerminalInputHandler(TerminalUserInputSink);
            terminal.Show(this);
            serialAgent.ProcessCharHandler(null, (byte)'`');
        }

        private void TerminalUserInputSink(ManageabilityControls.TerminalForm form, byte[] buf)
        {
            if (serialAgent != null) foreach (byte b in buf) ProcessCharEx(b);
            terminal.InvalidateTerminal();
        }

        private int termstate = 0;
        private int[] escNumber = new int[32];
        private int escNumberPtr = 0;
        private byte[] packetAccumulator = null;
        private int packetAccumulatorPtr = 0;
        private void ProcessCharEx(byte b)
        {
            switch (termstate)
            {
                case 0: // Normal Term State
                    switch (b)
                    {
                        case 27: // ESC
                            termstate = 1;
                            break;
                        default:
                            // Process a single char
                            serialAgent.ProcessCharHandler(null, b);
                            break;
                    }
                    break;
                case 1:
                    switch (b)
                    {
                        case 91:
                            escNumberPtr = 0;
                            Array.Clear(escNumber, 0, 32);
                            termstate = 2;
                            break;
                        default:
                            termstate = 0;
                            break;
                    }
                    break;
                case 2:
                    if (b >= '0' && b <= '9')
                    {
                        // This is a number
                        escNumber[escNumberPtr] = (escNumber[escNumberPtr] * 10) + (b - '0');
                        break;
                    }
                    else if (b == ';')
                    {
                        // New number
                        escNumberPtr++;
                        break;
                    }
                    else if (b == '@')
                    {
                        // New Packet
                        if (escNumber[1] == 0)
                        {
                            // Empty Command
                            termstate = 0;
                        }
                        else
                        {
                            packetAccumulatorPtr = 0;
                            packetAccumulator = new byte[escNumber[1]];
                            termstate = 3;
                        }
                        break;
                    }
                    else
                    {
                        // Process Escape Sequence
                        int[] vals = new int[escNumberPtr + 1];
                        Array.Copy(escNumber, vals, escNumberPtr + 1);
                        serialAgent.ProcessEscapeHandler(null, b, vals);
                        termstate = 0;
                    }
                    break;
                case 3: // Packet Accumulator
                    packetAccumulator[packetAccumulatorPtr] = b;
                    if (packetAccumulatorPtr == packetAccumulator.Length - 1)
                    {
                        packetAccumulator = null;
                        termstate = 0;
                    }
                    else
                    {
                        // More to come
                        packetAccumulatorPtr++;
                    }
                    break;
            }
        }

        private void serialAgentPermissionsListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            denyButton.Enabled = allowButton.Enabled = (serialAgentPermissionsListView.SelectedItems.Count > 0);
        }

        private void addBlockButton_Click(object sender, EventArgs e)
        {
            if (storageTreeView.SelectedNode == null) return;
            if (storageTreeView.SelectedNode.Tag.GetType() != typeof(AmtStorageApplication)) return;
            AmtStorageApplication app = (AmtStorageApplication)storageTreeView.SelectedNode.Tag;
            LocalStorageInfo si = (LocalStorageInfo)storageTreeView.SelectedNode.Parent.Tag;
            if (new BlockAllocateForm(si.Storage).ShowDialog(this) == DialogResult.OK) showStorageBlocks();
        }

        private void storageListView_DoubleClick(object sender, EventArgs e)
        {
            if (storageTreeView.SelectedNode == null) return;
            if (storageTreeView.SelectedNode.Tag.GetType() == typeof(AmtStorageBlock))
            {
                AmtStorageBlock block = (AmtStorageBlock)storageTreeView.SelectedNode.Tag;
                new ManageabilityControls.StoragePermissionsEditForm(null, null, block).ShowDialog(this);
            }
        }

        private void permissionsButton_Click(object sender, EventArgs e)
        {
            if (storageTreeView.SelectedNode == null) return;
            if (storageTreeView.SelectedNode.Tag.GetType() == typeof(AmtStorageBlock))
            {
                LocalStorageInfo store = (LocalStorageInfo)storageTreeView.SelectedNode.Parent.Parent.Tag;
                AmtStorageApplication app = (AmtStorageApplication)storageTreeView.SelectedNode.Parent.Tag;
                AmtStorageBlock block = (AmtStorageBlock)storageTreeView.SelectedNode.Tag;
                new ManageabilityControls.StoragePermissionsEditForm(store, app, block).ShowDialog(this);
            }
        }

        private void storageTreeView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            storageToolStripMenuItem.Enabled = (system != null);
            
            if (storageTreeView.SelectedNode == null)
            {
                storageInfoTextBox.Text = "";
                toolStripMenuItem10.Visible = true;
                toolStripMenuItem12.Visible = true;
                connectToolStripMenuItem.Visible = toolStripMenuItem3.Visible = false;
                disconnectToolStripMenuItem.Visible = toolStripMenuItem4.Visible = false;
                setVisibilityToolStripMenuItem.Visible = toolStripMenuItem6.Visible = false;
                allocateBlockToolStripMenuItem.Visible = toolStripMenuItem5.Visible = false;
                viewBlockDataToolStripMenuItem.Visible = viewBlockToolStripMenuItem.Visible = false;
                removeToolStripMenuItem.Visible = toolStripMenuItem9.Visible = false;
                permissionsToolStripMenuItem.Visible = permissionsToolStripMenuItem1.Visible = false;
                toolStripMenuItem2.Visible = toolStripMenuItem11.Visible = false;
            }
            else if (storageTreeView.SelectedNode.Tag.GetType() == typeof(LocalStorageInfo))
            {
                LocalStorageInfo storage = (LocalStorageInfo)storageTreeView.SelectedNode.Tag;
                storageInfoTextBox.Text = string.Format(Properties.Resources.StorageInfo.Replace("\\r\\n","\r\n"), storage.Enterprise, storage.Vendor, storage.AppName, storage.UUID.ToString());
                if (storage.Connected)
                {
                    storageInfoTextBox.Text += Properties.Resources.Connected;
                }
                else
                {
                    if (storage.ConnectError != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
                    {
                        storageInfoTextBox.Text += string.Format(Properties.Resources.ConnectionError.Replace("\\r\\n", "\r\n"), storage.ConnectError);
                    }
                    else
                    {
                        storageInfoTextBox.Text += Properties.Resources.Disconnected.Replace("\\r\\n", "\r\n");
                    }
                }

                toolStripMenuItem10.Visible = true;
                toolStripMenuItem12.Visible = true;
                connectToolStripMenuItem.Visible = toolStripMenuItem3.Visible = (storage.Connected == false);
                disconnectToolStripMenuItem.Visible = toolStripMenuItem4.Visible = (storage.Connected == true);
                setVisibilityToolStripMenuItem.Visible = toolStripMenuItem6.Visible = false;
                allocateBlockToolStripMenuItem.Visible = toolStripMenuItem5.Visible = false;
                viewBlockDataToolStripMenuItem.Visible = viewBlockToolStripMenuItem.Visible = false;
                removeToolStripMenuItem.Visible = toolStripMenuItem9.Visible = true;
                permissionsToolStripMenuItem.Visible = permissionsToolStripMenuItem1.Visible = false;
                toolStripMenuItem2.Visible = toolStripMenuItem11.Visible = true;
            }
            else if (storageTreeView.SelectedNode.Tag.GetType() == typeof(AmtStorageApplication))
            {
                AmtStorageApplication app = (AmtStorageApplication)storageTreeView.SelectedNode.Tag;
                LocalStorageInfo storage = (LocalStorageInfo)storageTreeView.SelectedNode.Parent.Tag;
                bool self = (app.Handle == storage.Storage.SessionHandle);

                storageInfoTextBox.Text = string.Format(Properties.Resources.ApplicationInfo.Replace("\\r\\n", "\r\n"), app.Enterprise, app.Vendor, app.Application, app.Handle, self);
                toolStripMenuItem10.Visible = false;
                connectToolStripMenuItem.Visible = toolStripMenuItem3.Visible = false;
                disconnectToolStripMenuItem.Visible = toolStripMenuItem4.Visible = false;
                setVisibilityToolStripMenuItem.Visible = toolStripMenuItem6.Visible = false;
                allocateBlockToolStripMenuItem.Visible = toolStripMenuItem5.Visible = toolStripMenuItem12.Visible = self;
                viewBlockDataToolStripMenuItem.Visible = viewBlockToolStripMenuItem.Visible = false;
                removeToolStripMenuItem.Visible = toolStripMenuItem9.Visible = false;
                permissionsToolStripMenuItem.Visible = permissionsToolStripMenuItem1.Visible = false;
                toolStripMenuItem2.Visible = true;
                toolStripMenuItem11.Visible = self;
            }
            else if (storageTreeView.SelectedNode.Tag.GetType() == typeof(AmtStorageBlock))
            {
                AmtStorageBlock block = (AmtStorageBlock)storageTreeView.SelectedNode.Tag;
                AmtStorageApplication app = (AmtStorageApplication)storageTreeView.SelectedNode.Parent.Tag;
                LocalStorageInfo storage = (LocalStorageInfo)storageTreeView.SelectedNode.Parent.Parent.Tag;
                bool owner = (app.Handle == storage.Storage.SessionHandle);

                storageInfoTextBox.Text = string.Format(Properties.Resources.BlockInfo.Replace("\\r\\n", "\r\n"), block.Name, block.Size, block.Handle, block.Hidden, owner);
                toolStripMenuItem10.Visible = false;
                toolStripMenuItem12.Visible = true;
                connectToolStripMenuItem.Visible = toolStripMenuItem3.Visible = false;
                disconnectToolStripMenuItem.Visible = toolStripMenuItem4.Visible = false;
                setVisibilityToolStripMenuItem.Visible = toolStripMenuItem6.Visible = true;
                setVisibilityToolStripMenuItem.Enabled = toolStripMenuItem6.Enabled = owner;
                allocateBlockToolStripMenuItem.Visible = toolStripMenuItem5.Visible = false;
                viewBlockDataToolStripMenuItem.Visible = viewBlockToolStripMenuItem.Visible = true;
                removeToolStripMenuItem.Visible = toolStripMenuItem9.Visible = true;
                removeToolStripMenuItem.Enabled = toolStripMenuItem9.Enabled = owner;
                permissionsToolStripMenuItem.Visible = permissionsToolStripMenuItem1.Visible = true;
                permissionsToolStripMenuItem.Enabled = permissionsToolStripMenuItem1.Enabled = owner;
                toolStripMenuItem2.Visible = toolStripMenuItem11.Visible = true;
            }
        }

        private void visibleToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (storageTreeView.SelectedNode == null || storageTreeView.SelectedNode.Tag.GetType() != typeof(AmtStorageBlock)) return;

            AmtStorageBlock block = (AmtStorageBlock)storageTreeView.SelectedNode.Tag;
            AmtStorageWrapper.AmtStorageStatus r = block.SetVisibility(false);
            if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
            {
                MessageBox.Show(this, string.Format(Properties.Resources.SetVisibilityError, Boolean.TrueString, r.ToString()), Properties.Resources.BlockVisibilityTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else showStorageBlocks();
        }

        private void hiddenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (storageTreeView.SelectedNode == null || storageTreeView.SelectedNode.Tag.GetType() != typeof(AmtStorageBlock)) return;

            AmtStorageBlock block = (AmtStorageBlock)storageTreeView.SelectedNode.Tag;
            AmtStorageWrapper.AmtStorageStatus r = block.SetVisibility(true);
            if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
            {
                MessageBox.Show(this, string.Format(Properties.Resources.SetVisibilityError, Boolean.FalseString, r.ToString()), Properties.Resources.BlockVisibilityTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else showStorageBlocks();
        }

        private void storageTreeView_MouseDown(object sender, MouseEventArgs e)
        {
            TreeNode n = storageTreeView.GetNodeAt(e.X, e.Y);
            storageTreeView.SelectedNode = n;
            if (n == null) storageTreeView_AfterSelect(this, null);
        }

        private void connectToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (storageTreeView.SelectedNode == null || storageTreeView.SelectedNode.Tag.GetType() != typeof(LocalStorageInfo)) return;
            LocalStorageInfo storage = (LocalStorageInfo)storageTreeView.SelectedNode.Tag;

            if (storage.Connect(system) == true)
            {
                showStorageBlocks();
            }

            storageTreeView_AfterSelect(this, null);
        }

        private void disconnectToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (storageTreeView.SelectedNode == null || storageTreeView.SelectedNode.Tag.GetType() != typeof(LocalStorageInfo)) return;
            LocalStorageInfo storage = (LocalStorageInfo)storageTreeView.SelectedNode.Tag;

            storage.Disconnect();
            showStorageBlocks();
        }

        private void viewBlockToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (storageTreeView.SelectedNode == null || storageTreeView.SelectedNode.Tag.GetType() != typeof(AmtStorageBlock)) return;
            AmtStorageBlock block = (AmtStorageBlock)storageTreeView.SelectedNode.Tag;
            AmtStorageApplication app = (AmtStorageApplication)storageTreeView.SelectedNode.Parent.Tag;
            new ManageabilityControls.StorageViewerForm(app, block).Show(this);
        }

        private void storageTreeView_DoubleClick(object sender, EventArgs e)
        {
            if (storageTreeView.SelectedNode != null && storageTreeView.SelectedNode.Tag.GetType() == typeof(AmtStorageBlock))
            {
                AmtStorageBlock block = (AmtStorageBlock)storageTreeView.SelectedNode.Tag;
                AmtStorageApplication app = (AmtStorageApplication)storageTreeView.SelectedNode.Parent.Tag;
                new ManageabilityControls.StorageViewerForm(app, block).Show(this);
            }
        }

        private void refreshToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            foreach (TreeNode node in storageTreeView.Nodes)
            {
                LocalStorageInfo storage = (LocalStorageInfo)node.Tag;
                if (storage.Connected == true)
                {
                    storage.Disconnect();
                    storage.Connect(system);
                }
            }
            showStorageBlocks();
        }

        private void mainTabControl_SelectedIndexChanged(object sender, EventArgs e)
        {
            storageToolStripMenuItem.Visible = (mainTabControl.SelectedIndex == 5);
            if (mainTabControl.SelectedIndex == 5) storageTreeView_AfterSelect(this, null);
        }

        private void padlockPictureBox_DoubleClick(object sender, EventArgs e)
        {
            // Display the TLS certificate
            if (system.GetTlsCertificates() != null && system.GetTlsCertificates().Length > 0) X509Certificate2UI.DisplayCertificate(system.GetTlsCertificates()[0], Handle);
        }

        private void saveCertificateToolStripMenuItem_Click(object sender, EventArgs e)
        {
            X509Certificate2[] certs = system.GetTlsCertificates();
            if (certs != null && certs.Length > 0)
            {
                saveFileDialog.DefaultExt = Properties.Resources.SaveCertificateFileExtension;
                saveFileDialog.Filter = Properties.Resources.SaveCertificateFileFilter;
                saveFileDialog.Title = Properties.Resources.SaveCertificateTitle;
                if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
                {
                    byte[] certBin = certs[0].Export(X509ContentType.Cert);
                    FileStream f = new FileStream(saveFileDialog.FileName, FileMode.Create);
                    f.Write(certBin, 0, certBin.Length);
                    f.Close();
                }
            }
        }

        private void ManageabilityDTKLatestNewsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ManageabilityControls.AboutForm.GotoNews();
        }

        private void ManageabilityDTKDownloadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ManageabilityControls.AboutForm.GotoDownloads();
        }

        private void ManageabilityDTKSupportForumsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ManageabilityControls.AboutForm.GotoForums();
        }

        private void windowsManagementToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new ManageabilityControls.DiagnosticViewerForm().ShowDialog(this);
        }

        private void diagnosticStorageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new DiagnosticStorageForm(system, system.UseTls, usernameTextBox.Text, passwordControl.Text).ShowDialog(this);
        }

        private void storageTreeView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete) storageRemoveButton_Click(this, null);
        }

        private void toolStripMenuItem13_Click(object sender, EventArgs e)
        {
            ManageabilityStack.AmtDebugForm.ShowForm();
        }

        private void fetchEventsEx(object param)
        {
            system.Events.FetchCache();
            FetchHandler(system, system.Events);
        }

        private void refreshLogButton_Click(object sender, EventArgs e)
        {
            if (system == null || system.Events == null) return;
            if (FetchCacheThread != null) return;
            system.Events.ClearCache();
            UpdateEventLog();
            new Thread(new ParameterizedThreadStart(fetchEventsEx)).Start(null);
        }

        private void UpdateEventLog()
        {
            if (system == null || system.State != AmtSystemObjState.Connected)
            {
                logListView.Items.Clear();
                logListView.Groups.Clear();
                logListView.Items.Add(new ListViewItem(Properties.Resources.NotConnected, 3));
                return;
            }

            if (system.Events == null)
            {
                logListView.Items.Clear();
                logListView.Groups.Clear();
                logListView.Items.Add(new ListViewItem(Properties.Resources.NotSupported, 3));
                return;
            }

            if (system.Events.IsCached() == false)
            {
                logListView.Items.Clear();
                logListView.Groups.Clear();
                logListView.Items.Add(new ListViewItem(Properties.Resources.Loading, 3));
                return;
            }

            AmtEvent[] log = system.Events.GetEventLog();

            logListView.BeginUpdate();
            logListView.Groups.Clear();
            logListView.Items.Clear();

            int select = -1;
            if (logListView.SelectedIndices.Count == 1)
            {
                select = logListView.SelectedIndices[0];
            }

            if (log == null)
            {
                logListView.EndUpdate();
            }
            else
            {
                string dateStr = "";
                ListViewGroup currentGroup = null;
                foreach (AmtEvent entry in log)
                {
                    if (entry.TimeStamp.ToLongDateString().CompareTo(dateStr) != 0)
                    {
                        dateStr = entry.TimeStamp.ToLongDateString();
                        currentGroup = new ListViewGroup(dateStr);
                        logListView.Groups.Add(currentGroup);
                    }
                    int severity = 1;
                    if (entry.EventSeverity < 5) severity = 0;
                    if (entry.EventSeverity > 10) severity = 2;
                    ListViewItem l = new ListViewItem(new String[] { entry.TimeStamp.ToLongTimeString() + " " + entry.EntityTypeStr, entry.EventDetailStr }, severity);
                    l.Group = currentGroup;
                    l.Tag = entry;
                    logListView.Items.Add(l);
                }
                if (select != -1) logListView.SelectedIndices.Add(select);
                logListView.EndUpdate();
            }

        }

        private void watchDogListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete) removeButton_Click(this, null);
        }

        private void mEInfoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new ManageabilityControls.MeInfoForm().ShowDialog(this);
        }

    }
}
