// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Text;
using Microsoft.Win32;
using System.Collections.Generic;
//using System.Runtime.InteropServices;

namespace ManageabilityOutpostTool
{
    public class RegistryCtlClass
    {
        public static RegistryKey OutpostDataRegistry = Registry.LocalMachine.CreateSubKey(GetSubKeyPath());

        // Builds a key path based on the CompanyName, ProductName, and ProductVersion attributes in 
        // the AssemblyInfo file. But can not get these information in class library now.
        private static string GetSubKeyPath()
        {
            return "Software\\Intel\\Manageability Outpost Tool Core\\1.0.0.0";
            //return "Software\\" + .CompanyName + "\\" + .ProductName + "\\" + .ProductVersion;
        }

        public static RegistryKey GetServiceKey()
        { 
            return Registry.LocalMachine.CreateSubKey("SYSTEM\\CurrentControlSet\\Services\\Manageability Outpost Tool Agent Service");
        }

        /*[DllImport("advapi32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        static extern Int32 RegOpenKeyEx(
            IntPtr hKey, string lpSubKey, Int32 ulOptions, Int32 samDesired, ref IntPtr phkResult);

        [DllImport("advapi32.dll", SetLastError = true)]
        static extern Int32 RegNotifyChangeKeyValue(IntPtr hKey, bool bWatchSubtree,
           Int32 dwNotifyFilter, IntPtr hEvent, bool fAsynchronous);

        [DllImport("advapi32.dll", SetLastError = true)]
        static extern Int32 RegCloseKey(IntPtr hKey);*/

        public static bool WaitForStringValue(string name, string value, int timeout)
        {
            bool result = false;

            if (timeout < 0) timeout = 6;

            for (int i = 0; i < timeout; i++)
            {
                System.Threading.Thread.Sleep(250);
                try
                {
                    if (OutpostDataRegistry.GetValue(name).ToString().CompareTo(value) == 0)
                    {
                        result = true;
                        break;
                    }
                }
                catch (NullReferenceException)
                {}
                System.Threading.Thread.Sleep(750);
            }

            return result;
        }
    }

    public class SerialAgentPermissionCtl
    {
        public const byte SAP_PERMISSION_DENY = (0x00);
        public const byte SAP_PERMISSION_ALLOW = (0x01);

        public static string[] PermissionStatus = new string[]
            {
                Properties.Resources.Deny,
                Properties.Resources.Allow
            };

        public static string[] PermissionDesc = new string[]
            {
                Properties.Resources.PermissionConsoleManagementCommands,
                Properties.Resources.PermissionDownloadFileManagementConsole,
                Properties.Resources.PermissionUploadFileManagementConsole,
                Properties.Resources.PermissionExecuteApplications,
                Properties.Resources.PermissionStopApplications,
                Properties.Resources.PermissionShutdownComputerCommands,
                Properties.Resources.PermissionRemoteDeviceDriverManagement,
                Properties.Resources.PermissionRemoteFileSystemManagement
            };

        private static byte[] PermissionData = GetFromRegistry();

        public enum Permissions
        { 
            Command = 0,
            Download,
            Upload,
            Execute,
            Stop,
            Shutdown,
            Devicemanage,
            Filebrowse,
            MaxDummy
        }

        private static byte[] GetFromRegistry()
        {
            return (byte[])RegistryCtlClass.OutpostDataRegistry.GetValue("PermissionData", new byte[(int)Permissions.MaxDummy] { 1, 1, 1, 1, 1, 1, 1, 1 }, RegistryValueOptions.DoNotExpandEnvironmentNames);
        }

        public static void SetPermission(Permissions prms, byte status)
        {
            PermissionData[(int)prms] = status;
            RegistryCtlClass.OutpostDataRegistry.SetValue("PermissionData", PermissionData, RegistryValueKind.Binary);
        }

        public static void SyncFromRegistry()
        {
            PermissionData = (byte[])RegistryCtlClass.OutpostDataRegistry.GetValue("PermissionData", new byte[(int)Permissions.MaxDummy],
                RegistryValueOptions.DoNotExpandEnvironmentNames);
        }

        public static byte GetPermission(Permissions prms)
        {
            return PermissionData[(int)prms];
        }
    }
}
