// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Text;
using System.Diagnostics;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace ManageabilityOutpostTool
{
    class RemoteConsole : NativeWindow, IDisposable 
    {
        private const uint STD_INPUT_HANDLE = 0xfffffff6;
        private const uint STD_OUTPUT_HANDLE = 0xfffffff5;
        private const uint STD_ERROR_HANDLE = 0xfffffff4;
        private const uint ATTACH_PARENT_PROCESS = 0xffffffff;
 
        public struct PROCESS_INFORMATION
        {
            public IntPtr hProcess;
            public IntPtr hThread;
            public uint dwProcessId;
            public uint dwThreadId;
        }

        public struct STARTUPINFO
        {
            public uint cb;
            public string lpReserved;
            public string lpDesktop;
            public string lpTitle;
            public uint dwX;
            public uint dwY;
            public uint dwXSize;
            public uint dwYSize;
            public uint dwXCountChars;
            public uint dwYCountChars;
            public uint dwFillAttribute;
            public uint dwFlags;
            public short wShowWindow;
            public short cbReserved2;
            public IntPtr lpReserved2;
            public IntPtr hStdInput;
            public IntPtr hStdOutput;
            public IntPtr hStdError;
        }

        /*
        public struct SECURITY_ATTRIBUTES
        {
            public int length;
            public IntPtr lpSecurityDescriptor;
            public bool bInheritHandle;
        }
        */
        
        [DllImport("kernel32.dll")]
        public static extern IntPtr GetConsoleWindow();
        [DllImport("kernel32.dll")]
        public static extern bool AttachConsole(uint dwProcessId);
        [DllImport("kernel32.dll")]
        public static extern bool AllocConsole();
        [DllImport("kernel32.dll")]
        public static extern bool FreeConsole();
        [DllImport("kernel32.dll")]
        public static extern int GetStdHandle(uint nStdHandle);
        [DllImport("kernel32.dll")]
        public static extern bool WriteConsole(int hConsoleOutput, string lpBuffer, int nNumberOfCharsToWrite, ref int lpNumberOfCharsWritten, int lpReserved);
        [DllImport("kernel32.dll")]
        public static extern bool ReadConsole(int hConsoleInput, StringBuilder lpBuffer, int nNumberOfCharsToRead, ref int lpNumberOfCharsRead, int lpReserved);
        [DllImport("kernel32.dll")]
        static extern bool CreateProcess(string lpApplicationName, string lpCommandLine, IntPtr lpProcessAttributes, IntPtr lpThreadAttributes, bool bInheritHandles, uint dwCreationFlags, IntPtr lpEnvironment, string lpCurrentDirectory, ref STARTUPINFO lpStartupInfo,out PROCESS_INFORMATION lpProcessInformation);

        private int stdin;
        private int stdout;

        public RemoteConsole()
        {
            IntPtr consolePtr = GetConsoleWindow();
            if (consolePtr == IntPtr.Zero) AllocConsole();
            consolePtr = GetConsoleWindow();
            stdin = GetStdHandle(STD_INPUT_HANDLE);
            stdout = GetStdHandle(STD_OUTPUT_HANDLE);
            this.AssignHandle(consolePtr);

            /*
            System.Diagnostics.ProcessStartInfo x = new System.Diagnostics.ProcessStartInfo("cmd");
            x.UseShellExecute = false;
            x.RedirectStandardError = true;
            x.RedirectStandardInput = true;
            x.RedirectStandardOutput = true;
            System.Diagnostics.Process.Start(x);
            */

            STARTUPINFO si = new STARTUPINFO();
            si.cb = (uint)Marshal.SizeOf(typeof(STARTUPINFO));
            si.dwFlags = 1;

            PROCESS_INFORMATION pi = new PROCESS_INFORMATION();
            string path = Environment.GetFolderPath(Environment.SpecialFolder.System) + "\\cmd.exe";
            CreateProcess(path, null, IntPtr.Zero, IntPtr.Zero, true, 0x200, IntPtr.Zero, null, ref si, out pi);
        }

        public void WriteLine(string s)
        {
            int len = 0;
            WriteConsole(stdout, s + "\r\n", s.Length + 2, ref len, 0);
        }

        public void Write(string s)
        {
            int len = 0;
            WriteConsole(stdout, s, s.Length, ref len, 0);
        }

        public string ReadLine()
        {
            int len = 0;
            StringBuilder sb = new StringBuilder();
            ReadConsole(stdin, sb, 256, ref len, 0);
            return sb.ToString(0, sb.Length - 2);
        }

        public void Dispose()
        {
            FreeConsole();
        }

        protected override void WndProc(ref Message m)
        {
            const int WM_MOUSEMOVE = 0x0200;

            /*
            switch (dwEvent)
            {
                case EVENT_CONSOLE_CARET:
                    break;
                case EVENT_CONSOLE_END_APPLICATION:
                    break;
                case EVENT_CONSOLE_LAYOUT:
                    break;
                case EVENT_CONSOLE_UPDATE_REGION:
                    break;
                case EVENT_CONSOLE_START_APPLICATION:
                    break;
                case EVENT_CONSOLE_UPDATE_SCROLL:
                    break;
                case EVENT_CONSOLE_UPDATE_SIMPLE:
                    break;
                default:
                    break;
            }
            */


            switch (m.Msg)
            {
                case WM_MOUSEMOVE:
                    // Do what you need to do.
                    break;
            }

            base.WndProc(ref m);
        }
    }

}
