// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Text;
using System.ServiceProcess;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace ManageabilityOutpostTool
{
    public class ServiceCtlClass
    {
        const int MAX_PATH = 256;
        public static string GetNetFrameworkDirectory()
        {
            StringBuilder buf = new StringBuilder(
                MAX_PATH, MAX_PATH);
            int cch = MAX_PATH;
            int hr = GetCORSystemDirectory(
                buf, MAX_PATH, ref cch);
            if (hr < 0) Marshal.ThrowExceptionForHR(hr);
            return buf.ToString();
        }

        [DllImport("mscoree.dll",CharSet = CharSet.Unicode,ExactSpelling = true)]
        public static extern int GetCORSystemDirectory(
                StringBuilder buf,
                int cchBuf,
                ref int cchRequired);


        // The command ID parameter should be an integer value between 128 and 256. 
        // Values less than 128 are reserved by the operating system.
        public const int CC_STARTTYPE_MANUAL = (0x00000081);
        public const int CC_STARTTYPE_AUTOMATIC = (0x00000082);
        public const int CC_STARTTYPE_DISABLE = (0x00000083);

        public const int CC_AMTLOCAL_CONNECT = (0x00000084);
        public const int CC_AMTLOCAL_DISCONNECT = (0x00000085);

        public const int CC_SERIALAGENT_ENABLE = (0x00000086);
        public const int CC_SERIALAGENT_DISABLE = (0x00000087);

        public const int CC_WATCHDOG_ADD = (0x00000088);
        public const int CC_WATCHDOG_EDIT = (0x00000089);
        public const int CC_WATCHDOG_REMOVE = (0x0000008A);

        public const int CC_PERMISSION_CHANGED = (0x0000008B);

        public enum ServiceCtlStatus
        {
            Invalid = 0,
            Stopped = 1,
            StartPending = 2,
            StopPending = 3,
            Running = 4,
            ContinuePending = 5,
            PausePending = 6,
            Paused = 7,
        }

        public static void InstallService(string serviceApp, bool bInstall)
        {
            //string winpath = Environment.GetEnvironmentVariable("windir");
            //string path = System.IO.Path.GetDirectoryName(System.Windows.Forms.Application.ExecutablePath);
            //Process.Start(winpath + @"\Microsoft.NET\Framework\v1.0.3705\Installutil.exe",path + "\\MyService.exe");

            string installUtil = GetNetFrameworkDirectory() + "InstallUtil.exe";

            System.Diagnostics.Process p = new System.Diagnostics.Process();
            //set the process file name
            p.StartInfo.FileName = "cmd.exe";
            //close the shell
            p.StartInfo.UseShellExecute = false;
            //redirect the standard input, output, err
            p.StartInfo.RedirectStandardInput = true;
            p.StartInfo.RedirectStandardOutput = true;
            p.StartInfo.RedirectStandardError = true;
            //do not create the cmd window
            p.StartInfo.CreateNoWindow = true;

            bool ret = p.Start();
            if (bInstall)
            {
                p.StandardInput.WriteLine(installUtil + " " + serviceApp);
            }
            else
            {
                p.StandardInput.WriteLine(installUtil + " /u " + serviceApp);
            }

            p.StandardInput.WriteLine("exit");

            p.Close();
        }
        
        private static ServiceController OutpostServiceCtlr = new ServiceController("Manageability Outpost Tool Agent Service");

        public static ServiceCtlStatus GetServiceStatus()
        {
            ServiceCtlStatus status = ServiceCtlStatus.Invalid;

            if (OutpostServiceCtlr == null)
            {
                return ServiceCtlStatus.Invalid;
            }

            OutpostServiceCtlr.Refresh();

            try
            {
                switch (OutpostServiceCtlr.Status)
                { 
                    case ServiceControllerStatus.ContinuePending:
                        status = ServiceCtlStatus.ContinuePending;
                        break;
                    case ServiceControllerStatus.Paused:
                        status = ServiceCtlStatus.Paused;
                        break;
                    case ServiceControllerStatus.PausePending:
                        status = ServiceCtlStatus.PausePending;
                        break;
                    case ServiceControllerStatus.Running:
                        status = ServiceCtlStatus.Running;
                        break;
                    case ServiceControllerStatus.StartPending:
                        status = ServiceCtlStatus.StartPending;
                        break;
                    case ServiceControllerStatus.Stopped:
                        status = ServiceCtlStatus.Stopped;
                        break;
                    case ServiceControllerStatus.StopPending:
                        status = ServiceCtlStatus.StopPending;
                        break;
                    default:
                        status = ServiceCtlStatus.Invalid;
                        break;
                }
            }
            catch (InvalidOperationException)
            {
                return ServiceCtlStatus.Invalid;
            }

            return status;
        }

        public static void StartService()
        {
            if (ServiceControllerStatus.Running == OutpostServiceCtlr.Status)
            {
                return;
            }

            OutpostServiceCtlr.Start();
            OutpostServiceCtlr.WaitForStatus(ServiceControllerStatus.Running);
        }

        public static bool StopService()
        {
            if (!OutpostServiceCtlr.CanStop)
            {
                return false;
            }
            
            if (ServiceControllerStatus.Stopped == OutpostServiceCtlr.Status)
            {
                return true;
            }

            OutpostServiceCtlr.Stop();
            OutpostServiceCtlr.WaitForStatus(ServiceControllerStatus.Stopped);

            return true;
        }

        public static void SendCommand(int command)
        {
            try
            {
                OutpostServiceCtlr.ExecuteCommand(command);
            }
            catch (InvalidOperationException)
            { }
        }
    }
}
