// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Text;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace ManageabilityOutpostTool
{
    public class ShutdownClass
    {
        [StructLayout(LayoutKind.Sequential, Pack = 1)]
        internal struct TokPriv1Luid
        {
            public int Count;
            public long Luid;
            public int Attr;
        }

        [DllImport("kernel32.dll", ExactSpelling = true)]
        internal static extern IntPtr GetCurrentProcess();

        [DllImport("advapi32.dll", ExactSpelling = true)]
        internal static extern bool OpenProcessToken(IntPtr h, int acc, ref IntPtr phtok);

        [DllImport("advapi32.dll")]
        internal static extern bool LookupPrivilegeValue(string host, string name, ref long pluid);

        [DllImport("advapi32.dll", ExactSpelling = true)]
        internal static extern bool AdjustTokenPrivileges(IntPtr htok, bool disall,
        ref TokPriv1Luid newst, int len, IntPtr prev, IntPtr relen);

        [DllImport("user32.dll", ExactSpelling = false, SetLastError = true)]
        internal static extern bool ExitWindowsEx(int flg, int rea);

        [DllImport("advapi32.dll", ExactSpelling = false, SetLastError = true)]
        internal static extern bool InitiateSystemShutdown(
          string lpMachineName,
          string lpMessage,
          int dwTimeout,
          bool bForceAppsClosed,
          bool bRebootAfterShutdown
          );

        [DllImport("kernel32.dll", ExactSpelling = true)]
        internal static extern bool SetProcessShutdownParameters(
          int dwLevel,
          int dwFlags
        );

        [DllImport("kernel32.dll", ExactSpelling = true)]
        internal static extern int GetLastError();

        [DllImport("kernel32.dll", ExactSpelling = false)]
        internal static extern int FormatMessage(
          int dwFlags,
          IntPtr lpSource,
          int dwMessageId,
          int dwLanguageId,
          ref string lpBuffer,
          int nSize,
          IntPtr Arguments
        );

        [DllImport("kernel32.dll", ExactSpelling = false, SetLastError = true)]
        internal static extern bool SetSystemPowerState(
          bool fSuspend,
          bool fForce
        );

        [DllImport("user32.dll", ExactSpelling = false)]
        internal static extern IntPtr FindWindow(string lpClassName, string lpWindowName);

        [DllImport("user32.dll", ExactSpelling = false)]
        internal static extern IntPtr GetWindow(IntPtr hWnd, uint uCmd);

        [DllImport("user32.dll", ExactSpelling = false)]
        internal static extern long GetWindowLong(IntPtr hWnd, int nIndex);

        public const int SLEEP = 3;
        public const int HIBERNATE = 5;

        internal const int FORMAT_MESSAGE_ALLOCATE_BUFFER = 0x00000100;
        internal const int FORMAT_MESSAGE_IGNORE_INSERTS = 0x00000200;
        internal const int FORMAT_MESSAGE_FROM_STRING = 0x00000400;
        internal const int FORMAT_MESSAGE_FROM_HMODULE = 0x00000800;
        internal const int FORMAT_MESSAGE_FROM_SYSTEM = 0x00001000;
        internal const int LANG_NEUTRAL = 0x00;
        internal const int SUBLANG_DEFAULT = 0x01;

        internal const int ERROR_MACHINE_LOCKED = 1271;
        internal const int ERROR_NOT_READY = 21;
        internal const int ERROR_SHUTDOWN_IN_PROGRESS = 1115;
        internal const int ERROR_NOT_LOGGED_ON = 1245;
        internal const int ERROR_IO_PENDING = 997;

        public static int MAKELANGID(int p, int s)
        {
          return ((((int)(s)) << 10) | (int)(p));
        }

        internal const int SE_PRIVILEGE_ENABLED = 0x00000002;
        internal const int TOKEN_QUERY = 0x00000008;
        internal const int TOKEN_ADJUST_PRIVILEGES = 0x00000020;
        internal const string SE_SHUTDOWN_NAME = "SeShutdownPrivilege";
        public const int EWX_LOGOFF = 0x00000000;
        public const int EWX_SHUTDOWN = 0x00000001;
        public const int EWX_REBOOT = 0x00000002;
        public const int EWX_FORCE = 0x00000004;
        public const int EWX_POWEROFF = 0x00000008;
        public const int EWX_FORCEIFHUNG = 0x00000010;

        internal const int SHTDN_REASON_FLAG_USER_DEFINED = 0x40000000;
        internal const int SHTDN_REASON_FLAG_PLANNED = unchecked((int)0x80000000);

        // Microsoft major reasons.
        internal const int SHTDN_REASON_MAJOR_OTHER = 0x00000000;
        internal const int SHTDN_REASON_MAJOR_NONE = 0x00000000;

        // Microsoft minor reasons.
        internal const int SHTDN_REASON_MINOR_OTHER = 0x00000000;
        internal const int SHTDN_REASON_MINOR_NONE = 0x000000ff;
        
        internal const uint GW_HWNDLAST = 1;
        internal const uint GW_HWNDPREV = 3;
        internal const int GWL_STYLE = -16;

        public static bool HasSaveMsgBox()  
        {
          IntPtr topWnd = FindWindow ("#32770", null);
          long lStyle = 0;
          bool bRet = false;
          //
          // Get top level window
          //
          topWnd = GetWindow (topWnd, GW_HWNDLAST);
          while (topWnd != null) {
            lStyle = GetWindowLong(topWnd, GWL_STYLE);
            if((lStyle&0x94c800C4) == 0x94c800C4) {
              bRet = true;
              break;
            }
            topWnd = GetWindow (topWnd, GW_HWNDPREV);
          }

          return bRet;
        }

        public static int DoExitWin(int flg, bool bForce, ref string errmsg)
        {
            bool ok;
            TokPriv1Luid tp;
            IntPtr hproc = GetCurrentProcess();
            IntPtr htok = IntPtr.Zero;
            ok = OpenProcessToken(hproc, TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, ref htok);
            tp.Count = 1;
            tp.Luid = 0;
            tp.Attr = SE_PRIVILEGE_ENABLED;
            ok = LookupPrivilegeValue(null, SE_SHUTDOWN_NAME, ref tp.Luid);
            ok = AdjustTokenPrivileges(htok, false, ref tp, 0, IntPtr.Zero, IntPtr.Zero);

            //set the process in the last shutdown range
            SetProcessShutdownParameters(0, 0);

            bool bReboot = (flg == EWX_REBOOT) ? true : false;
            int err = 0;
            int waittime = 0; //sec
            string msg = String.Format(Properties.Resources.SystemShutdownWarning, waittime);

            switch (flg)
            {
              case HIBERNATE:
                SetSystemPowerState(false, bForce);
                break;
              case SLEEP:
                SetSystemPowerState(true, bForce);
                break;
              case EWX_LOGOFF:
                if (bForce)  
                {                
                  ok = ExitWindowsEx(EWX_LOGOFF | EWX_FORCE, SHTDN_REASON_FLAG_PLANNED);
                }
                else
	              {
                  ok = ExitWindowsEx(EWX_LOGOFF, SHTDN_REASON_FLAG_PLANNED);
	              }
                break;
              case EWX_SHUTDOWN:
              case EWX_REBOOT:
                InitiateSystemShutdown(null,
                  msg,
                  waittime,
                  bForce,
                  bReboot
                );                
                break;
              default:
                break;
            }

            err = GetLastError();
            if (err != 0)
            {
              int fmt = ShutdownClass.FORMAT_MESSAGE_ALLOCATE_BUFFER |
                  ShutdownClass.FORMAT_MESSAGE_FROM_SYSTEM |
                  ShutdownClass.FORMAT_MESSAGE_IGNORE_INSERTS;

              ShutdownClass.FormatMessage(
                  fmt,
                  (IntPtr)null,
                  err,
                  ShutdownClass.MAKELANGID(ShutdownClass.LANG_NEUTRAL, ShutdownClass.SUBLANG_DEFAULT), // Default language
                  ref errmsg,
                  0,
                  (IntPtr)null);

              if (err == ERROR_IO_PENDING)
              {
                err = 0;
              }
            }

            return err;
        }
    }
}


