// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Text;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityOutpostTool
{
    public class TextChatClass
    {
        private SerialHighDriver serial;
        private AmtSerialAgent agent;
        private string accumulator = "";
        private bool typing = false;

        public TextChatClass(AmtSerialAgent agent, SerialHighDriver serial)
        {
            this.agent = agent;
            this.serial = serial;
            Invalidate();
        }

        public void ProcessCharHandler(byte c)
        {
            switch (c)
            {
                case 8: // BACKSPACE
                    if (accumulator.Length > 0)
                    {
                        accumulator = accumulator.Substring(0, accumulator.Length - 1);
                        if (accumulator.Length == 0) agent.SetRemoteTyping(false);
                        serial.Write(c);
                    }
                    break;
                case 13: // RETURN
                    if (accumulator.Length > 0)
                    {
                        agent.AddChatString("R> " + accumulator);
                        accumulator = "";
                        agent.SetRemoteTyping(false);
                        Invalidate();
                    }
                    break;
                case (byte)'`':
                    Invalidate();
                    break;
                case (byte)2:
                    if (agent.chatForm != null)
                    {
                        if (agent.chatForm.Visible) agent.chatForm.Hide(); else agent.chatForm.Show();
                    }
                    Invalidate();
                    break;
                case (byte)3:
                    Exit();
                    break;
                case (byte)4:
                    agent.ClearChatStrings();
                    agent.SetRemoteTyping(false);
                    Invalidate();
                    break;
                default:
                    if (c >= 32 && accumulator.Length < 76)
                    {
                        serial.Write(c);
                        if (accumulator.Length == 0) agent.SetRemoteTyping(true);
                        accumulator += (char)c;
                    }
                    break;
            }
        }

        public void ProcessEscapeHandler(SerialHighDriver.EscCode code, int[] args)
        {
        }

        public void Invalidate()
        {
            string[] chatlines = agent.GetChatText(21).Replace("\r\n","\r").Split('\r');
            int i = chatlines.Length;
            serial.WriteCursorMove(1, 1);
            serial.WriteEsc((byte)'m', 37, 44);  // Color (White / Blue)
            serial.WriteEsc((byte)'K');          // Clear line
            serial.Write(" Remote Chat            [^B] Toggle Chat   [^C] Exit   [^D] Clear   [`] Refresh\r\n");
            serial.WriteEsc((byte)'m', 0);       // Reset attributes
            bool yellow = false;
            for (int j = 0; j < i; j++)
            {
                if (chatlines[j].StartsWith("R> ") && yellow == false) { serial.WriteEsc((byte)'m', 33); yellow = true; }
                if (!(chatlines[j].StartsWith("R> ")) && yellow == true) { serial.WriteEsc((byte)'m', 0); yellow = false; }
                serial.Write(chatlines[j]);
                serial.WriteEsc((byte)'K');      // Clear line
                serial.Write("\r\n");
            }
            for (int j = i; j < 22; j++)
            {
                serial.WriteEsc((byte)'K');      // Clear line
                serial.Write("\r\n");
            }
            serial.WriteEsc((byte)'m', 37, 44);  // Color (White / Blue)
            serial.WriteEsc((byte)'K');          // Clear line
            if (agent.chatForm.Visible == false) serial.Write(" User chat hidden"); else serial.Write(" User chat visible");
            if (typing) serial.Write(", User is typing...");
            serial.Write("\r\n");
            serial.WriteEsc((byte)'m', 0);       // Reset attributes
            serial.Write("> " + accumulator);
            serial.WriteEsc((byte)'K');          // Clear line
        }

        public void Exit()
        {
            agent.chatTextApplication = null;
            agent.SendIntro();
        }

        public void UserTyping(bool typing)
        {
            if (this.typing != typing)
            {
                this.typing = typing;
                Invalidate();
            }
        }
    }
}
