// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Text;
using System.Threading;
using System.Diagnostics;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityOutpostTool
{
    public class TextTopClass
    {
        private SerialHighDriver serial;
        private AmtSerialAgent agent;
        private Thread timer;
        private int page = 0;
        private object mainlock = new object();
        private bool skip = false;

        public TextTopClass(AmtSerialAgent agent, SerialHighDriver serial)
        {
            this.agent = agent;
            this.serial = serial;
            Invalidate();
            timer = new Thread(new ThreadStart(TimerLoop));
            timer.Start();
        }

        private void TimerLoop()
        {
            try
            {
                while (timer != null && serial != null)
                {
                    Thread.Sleep(5000);
                    if (timer == null || serial == null) return;
                    if (skip == false) Invalidate();
                    skip = false;
                }
            }
            catch (Exception) {}
        }

        public void ProcessCharHandler(byte c)
        {
            switch (c)
            {
                case (byte)'`':
                    Invalidate();
                    break;
                case (byte)3:
                    Exit();
                    break;
                case (byte)'a':
                    if (page > 0) { page--; Invalidate(); }
                    break;
                case (byte)'z':
                    page++;
                    Invalidate();
                    break;
            }
        }

        public void ProcessEscapeHandler(SerialHighDriver.EscCode code, int[] args)
        {
            if (code == SerialHighDriver.EscCode.KeyUp)
            {
                if (page > 0) { page--; Invalidate(); }
            }
            else if (code == SerialHighDriver.EscCode.KeyDown)
            {
                page++;
                Invalidate();
            }
        }

        public void Invalidate()
        {
            lock (mainlock)
            {
                // Get the process table
                Process[] procList = Process.GetProcesses();
                int totalpages = procList.Length / 21;
                page = Math.Min(totalpages, page);

                serial.WriteCursorMove(1, 1);
                serial.WriteEsc((byte)'m', 37, 44);  // Color (White / Blue)
                serial.WriteEsc((byte)'K');          // Clear line
                serial.Write(string.Format(" Remote Process Monitor, {0} Processes, Page {1} of {2}\r\n", procList.Length, page + 1, totalpages + 1));
                serial.WriteEsc((byte)'K');          // Clear line
                serial.Write(" | ID  | Thrd | Ram    | Name\r\n");
                serial.WriteEsc((byte)'m', 0);       // Reset attributes

                for (int i = (page * 21); i < (page * 21) + 22; i++)
                {
                    Process p = null;
                    if (procList.Length > i)
                    {
                        try
                        {
                            p = procList[i];
                            serial.Write(string.Format("{0, 7} {1, 6} {2, 7}k {3, -30}", p.Id, p.Threads.Count, (p.WorkingSet64 / 1024).ToString(), p.ProcessName));
                        }
                        catch (Exception) { }
                    }
                    serial.WriteEsc((byte)'K');      // Clear line
                    serial.Write("\r\n");
                }

                serial.WriteEsc((byte)'m', 37, 44);  // Color (White / Blue)
                serial.WriteEsc((byte)'K');          // Clear line
                serial.Write("                                                        [^C] Exit   [`] Refresh");
                skip = true;
            }
        }

        public void Exit()
        {
            timer.Interrupt();
            timer = null;
            agent.chatTextTopClass = null;
            agent.SendIntro();
        }

    }
}
