// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.Net;
using System.Text;

namespace ManageabilityStack
{
    [Serializable]
    public class AmtAlert
    {
        public AmtEvent amtEvent;
        public int TrapVersion = -1;
        public string Community = null;
        public IPAddress TrapSource = null;
        public uint TimeStamp = 0;
        public Guid PlatformGuid = new Guid();
        public ushort SequenceNumber = 0;
        public uint TimeStamp2 = 0;
        public ushort UtcOffset = 0;
        public object Tag;
        public DateTime ReceivedTime;

        public static AmtAlert DecodeAlert(byte[] buf, int off, int len)
        {
            BerSEQUENCE s = BerSEQUENCE.DecodeBER(buf, off, len);
            AmtAlert alert = new AmtAlert();

            alert.ReceivedTime = DateTime.Now;

            // Get into the trap
            if (s.sequence.Count != 1 || s.sequence[0].GetType() != typeof(BerSEQUENCE)) return null;
            s = (BerSEQUENCE)s.sequence[0];
            if (s.sequence.Count != 3) return null;
            
            // Get Trap Version
            if ((s.sequence[0].GetType() != typeof(int)) || ((int)s.sequence[0]) != 0) return null;
            alert.TrapVersion = (int)s.sequence[0];

            // Get Trap Community
            if ((s.sequence[1].GetType() != typeof(BerOCTETS))) return null;
            alert.Community = ((BerOCTETS)s.sequence[1]).GetString();

            // Get into the enterprise trap
            if (s.sequence[2].GetType() != typeof(BerSEQUENCE)) return null;
            s = (BerSEQUENCE)s.sequence[2];
            if (s.sequence.Count != 6) return null;

            // Get the trap source
            if ((s.sequence[1].GetType() != typeof(IPAddress))) return null;
            alert.TrapSource = (IPAddress)s.sequence[1];

            // Get the "specific trap"
            if ((s.sequence[3].GetType() != typeof(int))) return null;
            uint specificTrap = (uint)((int)s.sequence[3]);

            // Get data from the "specific trap"
            EventLogRecordType evt = new EventLogRecordType();
            evt.EventSensorType = (byte)((specificTrap & 0x00FF0000) >> 16);
            evt.EventType = (byte)((specificTrap & 0x0000FF00) >> 8);
            evt.EventOffset = (byte)(specificTrap & 0x000000FF);

            // Get the "time stamp"
            if ((s.sequence[4].GetType() != typeof(uint))) return null;
            alert.TimeStamp = (uint)s.sequence[4];

            // Get into the data
            if (s.sequence[5].GetType() != typeof(BerSEQUENCE)) return null;
            s = (BerSEQUENCE)s.sequence[5];
            if (s.sequence.Count != 1) return null;
            if (s.sequence[0].GetType() != typeof(BerSEQUENCE)) return null;
            s = (BerSEQUENCE)s.sequence[0];
            if (s.sequence.Count != 2) return null;
            if ((s.sequence[1].GetType() != typeof(BerOCTETS))) return null;
            byte[] data = ((BerOCTETS)s.sequence[1]).octets;

            // Decode Data
            if (data.Length < 47 && data.Length > 110) return null;

            // Get the GUID
            byte[] g = new byte[16];
            Array.Copy(data,0,g,0,16);
            alert.PlatformGuid = new Guid(g);

            // Get the sequence number
            alert.SequenceNumber = BitConverter.ToUInt16(data, 16);

            // Get the time stamp
            //long timestamp = (long)BitConverter.ToUInt32(data, 18);

            // Get the UTC offset
            alert.UtcOffset = BitConverter.ToUInt16(data, 22);

            // Get more data
            byte TrapSource = data[24];
            evt.EventSourceType = data[25];
            evt.EventSeverity = data[26];
            evt.DeviceAddress = data[27];
            evt.SensorNumber = data[28];
            evt.Entity = data[29];
            evt.EntityInstance = data[30];
            evt.EventData = new Byte[8];
            Array.Copy(data, 31, evt.EventData, 0, 8);

            byte LanguageCode = data[39];
            uint ManufacturerID = BitConverter.ToUInt32(data, 40);
            ushort SystemID = BitConverter.ToUInt16(data, 44);

            alert.amtEvent = new AmtEvent(evt);
            alert.amtEvent.TimeStamp = DateTime.Now;

            return alert;
        }
    }

    public class BerSEQUENCE
    {
        public ArrayList sequence = null;

        public BerSEQUENCE(ArrayList sequence)
        {
            this.sequence = sequence;
        }

        public override string ToString()
        {
            string s = "{";
            foreach (object o in sequence)
            {
                if (s.Length != 1) s += ", ";
                s += o.ToString();
            }
            s += "}";
            return s;
        }

        public string GetDebugString()
        {
            return GetDebugString("");
        }

        public string GetDebugString(string ident)
        {
            string s = ident + "SEQUENCE:\r\n";
            s += ident + "{\r\n";
            foreach (object o in sequence)
            {
                if (o.GetType() == typeof(BerSEQUENCE))
                {
                    s += ((BerSEQUENCE)o).GetDebugString(ident + "   ");
                }
                if (o.GetType() == typeof(BerOCTETS))
                {
                    BerOCTETS oc = (BerOCTETS)o;
                    s += ident + "   OCTETS:\r\n";
                    s += ident + "   {\r\n";
                    s += ident + "      " + oc.ToString() + "\r\n";
                    s += ident + "   }\r\n";
                }
                if (o.GetType() == typeof(BerOID))
                {
                    BerOID oid = (BerOID)o;
                    s += ident + "   OID: " + oid.ToString() + "\r\n";
                }
                if (o.GetType() == typeof(int))
                {
                    s += ident + "   INTEGER: " + ((int)o).ToString() + "\r\n";
                }
                if (o.GetType() == typeof(uint))
                {
                    s += ident + "   UINTEGER: " + ((uint)o).ToString() + "\r\n";
                }
                if (o.GetType() == typeof(IPAddress))
                {
                    s += ident + "   IPADDRESS: " + ((IPAddress)o).ToString() + "\r\n";
                }
            }
            s += ident + "}\r\n";
            return s;
        }

        public static BerSEQUENCE DecodeBER(byte[] buffer, int off, int len)
        {
            int ptr = off;
            ArrayList list = new ArrayList();

            while (ptr < off + len)
            {
                byte tag = buffer[ptr];
                int size = buffer[ptr + 1];
                if (size == 0x81)
                {
                    // Length is over next byte
                    size = (int)buffer[ptr + 2];
                    ptr += 1;
                }
                else if (size == 0x82)
                {
                    // Length is over two next bytes
                    size = (int)((((int)buffer[ptr + 2]) << (int)8) + (int)(buffer[ptr + 3]));
                    ptr += 2;
                }
                switch (tag)
                {
                    case 0x02: // Integer
                        int i = 0;
                        int v = 0;
                        while (i < size)
                        {
                            v = v << 8;
                            v += buffer[ptr + 2 + i];
                            i++;
                        }
                        list.Add(v);
                        break;
                    case 0x04: // Octet String
                        byte[] b = new byte[size];
                        Array.Copy(buffer, ptr + 2, b, 0, size);
                        list.Add(new BerOCTETS(b));
                        break;
                    case 0x40: // IP Address
                        byte[] b1 = new byte[size];
                        Array.Copy(buffer, ptr + 2, b1, 0, size);
                        list.Add(new IPAddress(b1));
                        break;
                    case 0x43: // Time laps (???)
                        if (size != 0)
                        {
                            byte[] b2 = new byte[Math.Min(size, (byte)4)];
                            Array.Copy(buffer, ptr + 2, b2, 0, size);
                            uint timestamp = (uint)BitConverter.ToInt32(b2, 0);
                            list.Add(timestamp);
                        }
                        else
                        {
                            list.Add((uint)0);
                        }
                        break;
                    case 0x06: // OID
                        ArrayList oid = new ArrayList();
                        int oidval = buffer[ptr + 2];
                        oid.Add(oidval / 40);
                        oid.Add(oidval % 40);
                        for (int j = 1; j < size; j++)
                        {
                            oidval = buffer[ptr + 2 + j];
                            if ((oidval & 0x80) != 0)
                            {
                                oidval = ((oidval & 0x7F) << 7) + buffer[ptr + 3 + j];
                                oid.Add(oidval);
                                j++;
                            }
                            else
                            {
                                oid.Add(oidval);
                            }
                        }
                        list.Add(new BerOID((int[])(oid.ToArray(typeof(int)))));
                        break;
                    case 0x30: // Sequence
                    case 0xA4: // Application Sequence
                        list.Add(DecodeBER(buffer, ptr + 2, size));
                        break;
                }
                ptr += (2 + size);
            }

            return new BerSEQUENCE(list);
        }

    }

    public class BerOID
    {
        public int[] oid = null;

        public BerOID(int[] oid)
        {
            this.oid = oid;
        }

        public override string ToString()
        {
            string s = "\"";
            for (int i = 0; i < oid.Length; i++)
            {
                if (s.Length != 1) s += ".";
                s += oid[i].ToString();
            }
            s += "\"";
            return s;
        }
    }

    public class BerOCTETS
    {
        public byte[] octets = null;
        public bool text = false;

        public BerOCTETS(byte[] octets)
        {
            this.octets = octets;
            text = true;
            foreach (byte b in octets)
            {
                if (b == 00)
                {
                    text = false;
                    break;
                }
            }
        }

        public BerOCTETS(byte[] octets, bool text)
        {
            this.octets = octets;
            this.text = text;
        }

        public string GetString()
        {
            return UTF8Encoding.UTF8.GetString(octets, 0, octets.Length);
        }

        public override string ToString()
        {
            if (text)
            {
                return "\"" + UTF8Encoding.UTF8.GetString(octets, 0, octets.Length) + "\"";
            }
            else
            {
                string s = "\"";
                for (int i = 0; i < octets.Length; i++)
                {
                    if (s.Length != 1) s += ",";
                    s += octets[i].ToString("X2");
                }
                s += "\"";
                return s;
            }
        }
    }

}
