// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.Net;
using System.Net.NetworkInformation;
using System.Net.Sockets;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;

namespace ManageabilityStack
{
    public class AmtDiscovery2
    {

        [DllImport("iphlpapi.dll", CharSet = CharSet.Ansi)]
        public static extern int GetAdaptersInfo(IntPtr pAdapterInfo, ref Int64 pBufOutLen);

        
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
        public struct IP_ADDRESS_STRING
        {
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 16)]
            public string Address;
        }

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
        public struct IP_ADDR_STRING
        {
            public IntPtr Next;
            public IP_ADDRESS_STRING IpAddress;
            public IP_ADDRESS_STRING IpMask;
            public Int32 Context;
        }

        const int MAX_ADAPTER_NAME_LENGTH = 256;
        const int MAX_ADAPTER_DESCRIPTION_LENGTH = 128;
        const int MAX_ADAPTER_ADDRESS_LENGTH = 8;

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
        public struct IP_ADAPTER_INFO
        {
            public IntPtr Next;
            public Int32 ComboIndex;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = MAX_ADAPTER_NAME_LENGTH + 4)]
            public string AdapterName;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = MAX_ADAPTER_DESCRIPTION_LENGTH + 4)]
            public string AdapterDescription;
            public UInt32 AddressLength;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = MAX_ADAPTER_ADDRESS_LENGTH)]
            public byte[] Address;
            public Int32 Index;
            public UInt32 Type;
            public UInt32 DhcpEnabled;
            public IntPtr CurrentIpAddress;
            public IP_ADDR_STRING IpAddressList;
            public IP_ADDR_STRING GatewayList;
            public IP_ADDR_STRING DhcpServer;
            public bool HaveWins;
            public IP_ADDR_STRING PrimaryWinsServer;
            public IP_ADDR_STRING SecondaryWinsServer;
            public Int32 LeaseObtained;
            public Int32 LeaseExpires;
        }


        public IPAddress start;
        public IPAddress end;
        private IPAddress current;
        private IPAddress local;
        private ushort port;
        private Hashtable socketTable = new Hashtable();
        private Thread timerThread;
        private int timeout = 4000;
        private string[] ipCurrentSubnet;
        private string[] CurrSubnetMask;

        [DllImport("iphlpapi.dll", ExactSpelling = true)]
        private static extern int SendARP(int DestIP, int SrcIP, byte[] pMacAddr, ref int PhyAddrLen);

        public enum AmtDiscoveryEvent
        {
            LaunchingDiscovery = 1,
            MachineFound = 2,
            MachineNotFound = 3,
            DiscoveryFinished = 4
        }

        private struct DiscoverySocketInfo
        {
            public Socket s;
            public IPEndPoint ep;
            public IAsyncResult async;
            public DateTime t;
            public bool stopped;
        }

        public class AmtDiscoveredComputer
        {
            public string Hostname;
            public IPEndPoint EndPoint;
            public string Info;
            public bool UseTls;
            public IPHostEntry Entry;
            public uint AmtRev = 0;
            public uint AmtRevMin = 0;
            public string IpAddress = null;
            public AmtDiscoveredComputer(string hostname, IPEndPoint endPoint, string info, bool useTls, IPHostEntry entry, uint amtRev, uint amtRevMin,string strIpAddress)
            {
                Hostname = hostname;
                EndPoint = endPoint;
                Info = info;
                UseTls = useTls;
                Entry = entry;
                AmtRev = amtRev;
                AmtRevMin = amtRevMin;
                IpAddress = strIpAddress;
            }

            /// <summary>
            /// Returns true if this is a mobile platform
            /// </summary>
            public bool IsMobile
            {
                get
                {
                    return (AmtRev == 2 && AmtRevMin >= 5);
                }
            }
        }

        public delegate void AmtDiscoveryEventHandler(AmtDiscovery2 amtDiscovery, IPEndPoint ep, AmtDiscoveryEvent e, AmtDiscoveredComputer computer);
        public event AmtDiscoveryEventHandler OnDiscoveryEvent;

        public AmtDiscovery2()
        {
        }

        public bool IsRunning
        {
            get
            {
                return timerThread != null;
            }
        }

        // Get the next IP address - Works with IPv4 & IPv6
        private IPAddress GetNextAddress(IPAddress addr)
        {
            byte[] ip = addr.GetAddressBytes();
            ip[ip.Length - 1]++;
            bool carry = false;
            carry = (ip[ip.Length - 1] == 0);
            for (int i = ip.Length - 1; i > 0; i--)
            {
                if (carry)
                {
                    ip[i - 1]++;
                    carry = (ip[i - 1] == 0);
                }
            }
            return new IPAddress(ip);
        }

        public static bool IsSameAddress(IPAddress a1, IPAddress a2)
        {
            byte[] b1 = a1.GetAddressBytes();
            byte[] b2 = a2.GetAddressBytes();
            if (b1.Length != b2.Length) return false;
            for (int i = 0; i < b1.Length; i++) if (b1[i] != b2[i]) return false;
            return true;
        }

        public static long GetAddressDiff(IPAddress a1, IPAddress a2)
        {
            byte[] b1 = a1.GetAddressBytes();
            byte[] b2 = a2.GetAddressBytes();
            if (b1.Length != b2.Length) return -1;
            int carry = 0;
            long total = 0;
            int l = b1.Length - 1;
            for (int i = l; i >= 0; i--)
            {
                int d = b2[i] - carry - b1[i];
                if (d < 0)
                {
                    carry = 1;
                    total += (d << ((l - i) * 8));
                }
                else
                {
                    carry = 0;
                    total += (d << ((l - i) * 8));
                }
            }
            return total + 1;
        }

        public void Dispose()
        {
            Stop();
        }
        public long CalcNoOfMachines()
        {
            //Thread ConnThread;

            bool WiredFound = false;
            bool WireLess = false;
            string strTemp = null;
            byte[] startIP = new byte[4];
            byte[] endIP = new byte[4];
            string[] strTemp1 =null;

            // check for wired network adapter
            foreach (NetworkInterface ni in NetworkInterface.GetAllNetworkInterfaces())
            {
                
                if( (ni.Description.ToLower().Contains("wireless") == false) && (ni.NetworkInterfaceType == NetworkInterfaceType.Ethernet ) &&
                    (ni.OperationalStatus == OperationalStatus.Up) //&&  (ni.Supports(NetworkInterfaceComponent.IPv6) == false) 
                    )
                
                {
                    foreach (UnicastIPAddressInformation uipi in ni.GetIPProperties().UnicastAddresses)
                    {
                        
                        local = uipi.Address;
                        if (local.IsIPv6LinkLocal == false) // check for ipv4 packet type
                        {
                            
                            strTemp = local.ToString();

                            CurrSubnetMask = uipi.IPv4Mask.ToString().Split('.');
                            strTemp1 = ipCurrentSubnet = strTemp.Split('.');
                            WiredFound = true;
                            
                            break;
                        }
                    }
                    if (WiredFound)
                        break;
                }
                

            }
            // if wired is not found check for wireless network adapter
            if (WiredFound == false)
            {
                foreach (NetworkInterface ni in NetworkInterface.GetAllNetworkInterfaces())
                {
                    
                    if ((ni.Description.ToLower().Contains("wireless") == true) && (ni.NetworkInterfaceType == NetworkInterfaceType.Ethernet || ni.NetworkInterfaceType == NetworkInterfaceType.Wireless80211) &&
                        (ni.OperationalStatus == OperationalStatus.Up)  //&& (ni.Supports(NetworkInterfaceComponent.IPv6) == false)                   
                        )
                    {
                        foreach (UnicastIPAddressInformation uipi in ni.GetIPProperties().UnicastAddresses)
                        {
                            local = uipi.Address;
                            if (local.IsIPv6LinkLocal == false) // check for ipv4 packet type
                            {
                                strTemp = local.ToString();

                                CurrSubnetMask = uipi.IPv4Mask.ToString().Split('.');
                                strTemp1 = ipCurrentSubnet = strTemp.Split('.');
                                WireLess = true;
                                break;
                            }

                            
                        }
                        if (WireLess)
                            break;
                    }


                }
            }
            if (WireLess != true && WiredFound != true)
                return -1;
            
            for (int iCount = 0; iCount <= 3; iCount++)
            {
                int iData = (System.Convert.ToByte(CurrSubnetMask[iCount]) & System.Convert.ToByte(ipCurrentSubnet[iCount]));
                strTemp1[iCount] = System.Convert.ToString(iData);
                startIP[iCount] = endIP[iCount] = System.Convert.ToByte(ipCurrentSubnet[iCount]);
                if (System.Convert.ToByte(strTemp1[iCount]) == 0)
                {
                    startIP[iCount] = 1;
                    endIP[iCount] = 254;
                }
                else if (System.Convert.ToByte(CurrSubnetMask[iCount]) != 255)
                {
                    startIP[iCount] = (Byte)(System.Convert.ToByte(strTemp1[iCount]) + 1);
                    endIP[iCount] = 254;
                }
            }
            start = new IPAddress(startIP);
            end = new IPAddress(endIP);


            /* startIP[0] = 10; 
             startIP[1] = 100;
             startIP[2] = 135;
             startIP[3] = 164;

             endIP[0] = 10;
             endIP[1] = 100;
             endIP[2] = 135;
             endIP[3] = 165;*/

            start = new IPAddress(startIP);
            end = new IPAddress(endIP);
            return GetAddressDiff(start, end);
        }

        public void Start(IPAddress start, IPAddress end, ushort port)
        {
            //Thread ConnThread;
            string myHost = System.Net.Dns.GetHostName();
            string strTemp = System.Net.Dns.GetHostEntry(myHost).AddressList[0].ToString();

            ipCurrentSubnet = strTemp.Split('.');
            if (timerThread != null) return;

            this.start = start;
            this.end = end;
            this.port = port;
            current = start;

            const int ERROR_BUFFER_OVERFLOW = 111;
            long structSize = Marshal.SizeOf(typeof(IP_ADAPTER_INFO));
            IntPtr pArray = Marshal.AllocHGlobal(new IntPtr(structSize));

            int ret = GetAdaptersInfo(pArray, ref structSize);

            if (ret == ERROR_BUFFER_OVERFLOW)
            {
                // Buffer was too small, reallocate the correct size for the buffer.
                pArray = Marshal.ReAllocHGlobal(pArray, new IntPtr(structSize));

                ret = GetAdaptersInfo(pArray, ref structSize);
            }

            // Call Succeeded
            IntPtr pEntry = pArray;

            // Retrieve the adapter info from the memory address
            IP_ADAPTER_INFO entry = (IP_ADAPTER_INFO)Marshal.PtrToStructure(pEntry, typeof(IP_ADAPTER_INFO));

            CurrSubnetMask = entry.IpAddressList.IpMask.Address.ToString().Split('.');


            LaunchOneSocket(new IPEndPoint(current, port));

            timerThread = new Thread(new ParameterizedThreadStart(TimerThread));
            timerThread.Start(this);


            for (int i = 0; i < 50; i++)
            {
                LaunchNextSocket();
            }

        }

        public void Start( ushort port)
        {
            
            this.port = port;
            current = start;

            if (timerThread != null) return;
            timerThread = new Thread(new ParameterizedThreadStart(TimerThread));
            timerThread.Start(this);

            for (int i = 0; i < 5; i++)
            {
                LaunchNextSocket();
            }
        }

        private void TimerThread(object x)
        {
            try
            {
                ArrayList l = new ArrayList();
                DateTime n;
                while (true)
                {
                    Thread.Sleep(500);
                    n = DateTime.Now;
                    lock (socketTable)
                    {
                        foreach (Socket s in socketTable.Keys)
                        {
                            DiscoverySocketInfo inf = (DiscoverySocketInfo)socketTable[s];
                            int c = inf.t.CompareTo(n);
                            if (c < 0)
                            {
                                // Break the socket
                                try
                                {
                                    s.Close();
                                }
                                catch (SocketException) { }
                                inf.stopped = true;
                                l.Add(inf);
                            }
                        }
                        
                    }
                    
                    l.Clear();
                    if (socketTable.Count == 0 && IsSameAddress(current, end))
                    {
                        timerThread = null;
                        if (OnDiscoveryEvent != null) OnDiscoveryEvent(this, null, AmtDiscoveryEvent.DiscoveryFinished, null);
                        break;
                    }
                }
            }
            catch (ThreadInterruptedException) { }
            //timerThread = null;
        }

        private void LaunchOneSocket(IPEndPoint ep)
        {
            string[] strTemp = ep.Address.ToString().Split('.');

            if (OnDiscoveryEvent != null) OnDiscoveryEvent(this, null, AmtDiscoveryEvent.LaunchingDiscovery, null);

             
            // If available in the current subnet, send ARP to resolve the MAC.  
            if ((System.Convert.ToByte(CurrSubnetMask[0]) & System.Convert.ToByte(ipCurrentSubnet[0])) == (System.Convert.ToByte(strTemp[0]) & System.Convert.ToByte(CurrSubnetMask[0])) &&
                (System.Convert.ToByte(CurrSubnetMask[1]) & System.Convert.ToByte(ipCurrentSubnet[1])) == (System.Convert.ToByte(strTemp[1]) & System.Convert.ToByte(CurrSubnetMask[1])) &&
                (System.Convert.ToByte(CurrSubnetMask[2]) & System.Convert.ToByte(ipCurrentSubnet[2])) == (System.Convert.ToByte(strTemp[2]) & System.Convert.ToByte(CurrSubnetMask[2])))
            {
                byte[] ab = new byte[6];
                int len = ab.Length;
                int dest = BitConverter.ToInt32(IPAddress.Parse(ep.Address.ToString()).GetAddressBytes(), 0);
                int r = SendARP(dest, 0, ab, ref len);

                if (r != 0)
                {
                    if (OnDiscoveryEvent != null) OnDiscoveryEvent(this, null, AmtDiscoveryEvent.MachineNotFound, null);
                    LaunchNextSocket();
                    return;
                }
            }
            Socket s = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            s.Ttl = 2;
            s.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.DontLinger, true);
            s.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);

            DiscoverySocketInfo inf = new DiscoverySocketInfo();
            inf.ep = ep;
            inf.s = s;
            inf.t = DateTime.Now.AddMilliseconds(timeout);

            lock (socketTable)
            {
                socketTable.Add(s, inf);
            }

            inf.async = s.BeginConnect(ep, new AsyncCallback(SocketConnectCallback), s);

        }

        private void SocketConnectCallback(IAsyncResult result)
        {

            bool success = true;
            DiscoverySocketInfo inf;
            Socket s = (Socket)result.AsyncState;

            lock (socketTable)
            {
                if (socketTable[s] == null) return;
                inf = (DiscoverySocketInfo)socketTable[s];
                socketTable.Remove(s);

                try
                {
                    s.EndConnect(result);
                }
                catch (SocketException)
                {
                    success = false;
                }
                catch (ObjectDisposedException)
                {
                    success = false;
                }
            }
            if (OnDiscoveryEvent != null)
            {
                if (success == true)
                {
                    uint AmtRev = 0;
                    uint AmtRevMin = 0;


                    IPEndPoint ip = null;  

                    // Try a manual TCP Client GET request
                    string str = null;
                    TcpClient client = new TcpClient();
                    try
                    {
                        ip = (IPEndPoint)s.RemoteEndPoint;
                        client.Connect(ip);
                    }
                    
                    // Try a manual TCP Client GET request
                    
                    catch (Exception)
                    {
                        client = null;
                        //MessageBox.Show(ex.Message + "\n\n" + ex.GetType().ToString() + "\n" + ex.StackTrace, "Exception");
                    }

                    if (client != null)
                    {
                        NetworkStream ns = client.GetStream();
                        
                        DateTime failTime = DateTime.Now.AddSeconds(5);
                        byte[] b = UTF8Encoding.UTF8.GetBytes("HEAD / HTTP/1.1\r\nHost: AmtDiscovery\r\n\r\n");
                        ns.Write(b, 0, b.Length);
                        byte[] buf = new byte[2000];
                        int ptr = 0;
                        string head = "";
                        try
                        {
                            while (DateTime.Now.CompareTo(failTime) < 0 && head.IndexOf("\r\n\r\n") < 0)
                            {
                                Thread.Sleep(500);
                                ptr = ns.Read(buf, ptr, 2000 - ptr);
                                head = UTF8Encoding.UTF8.GetString(buf, 0, ptr);
                            }
                        }
                        catch (Exception) { }
                        client.Close();
                        client = null;

                        if (head.IndexOf("\r\n\r\n") > 0)
                        {
                            int i = head.IndexOf("\r\nServer: ");
                            if (i >= 0)
                            {
                                str = head.Substring(i + 10);
                                i = str.IndexOf("\r\n");
                                str = str.Substring(0, i);
                            }
                            if (str == null)
                            {
                                str = "Unknown Network Node";
                            }
                            else if (str.Equals("Intel(R) Active Management Technology") == true)
                            {
                                str = "Intel(R) Active Management Technology 1.0";
                                AmtRev = 1;
                                AmtRevMin = 0;
                            }
                            else if (str.StartsWith("Intel(R) Active Management Technology 2.") == true)
                            {
                                AmtRev = 2;
                                try
                                {
                                    AmtRevMin = uint.Parse(str.Substring(40, 1));
                                }
                                catch (Exception) { }
                            }
                            else if (str.StartsWith("Intel(R) Active Management Technology 3.") == true)
                            {
                                AmtRev = 3;
                                try
                                {
                                    AmtRevMin = uint.Parse(str.Substring(40, 1));
                                }
                                catch (Exception) { }
                            }
                            else if (str.StartsWith("Intel(R) Active Management Technology 4.") == true)
                            {
                                AmtRev = 4;
                                try
                                {
                                    AmtRevMin = uint.Parse(str.Substring(40, 1));
                                }
                                catch (Exception) { }
                            }
                        }
                    }
                    else
                    {
                        str = "Active refusal of TCP connection.";
                        LaunchNextSocket();
                        return;
                    }

                    bool tls = false;
                    if (str == null)
                    {
                        // Just for kicks, lets try to connect to the TLS port and see what happens.
                        client = new TcpClient();
                        try
                        {
                            client.Connect(ip.Address, 16993);
                            tls = true;
                            client.Close();
                        }
                        catch (Exception) { }
                        if (tls) str = "Intel(R) AMT with TLS or not configured";
                        client = null;
                    }

                    if (str == null)
                    {
                        str = ManageabilityStack.Properties.Resources.Discovery_DiscoveryEvent;
                    }
                    else
                    {
                        str = Utils.FilterStr(str);
                    }

                    // Try to get the Reverse DNS name
                    String hostname = inf.ep.Address.ToString();
                    IPHostEntry hostEntry = null;
                    
                    try
                    {
                        hostEntry = System.Net.Dns.GetHostEntry(inf.ep.Address);
                       
                    }
                    catch (Exception) { }
                    if (hostEntry != null && (hostEntry.HostName != inf.ep.Address.ToString()))
                    {
                        hostname = hostEntry.HostName; ///+"-" + inf.ep.Address.ToString();
                        OnDiscoveryEvent(this, inf.ep, AmtDiscoveryEvent.MachineFound, new AmtDiscoveredComputer(hostname, inf.ep, str, tls, hostEntry, AmtRev, AmtRevMin, inf.ep.Address.ToString()));
                    }
                    else
                        OnDiscoveryEvent(this, inf.ep, AmtDiscoveryEvent.MachineFound, new AmtDiscoveredComputer(hostname, inf.ep, str, tls, hostEntry, AmtRev, AmtRevMin, null));
                }
                else // if (success == false)
                {
                    // Just for kicks, lets try to connect to the TLS port and see what happens.
                    string str = null;
                    bool tls = false;
                    TcpClient client = new TcpClient();
                    try
                    {
                        IPEndPoint ip = (IPEndPoint)inf.ep;
                        client.Connect(ip.Address, 16993);
                        tls = true;
                        client.Close();
                    }
                    catch (Exception) { }
                    if (tls)
                    {
                        str = "Intel(R) AMT with TLS configured";
                        client = null;

                        if (str == null)
                        {
                            str = ManageabilityStack.Properties.Resources.Discovery_DiscoveryEvent;
                        }
                        else
                        {
                            str = Utils.FilterStr(str);
                        }

                        // Try to get the Reverse DNS name
                        string hostname = inf.ep.Address.ToString();
                        IPHostEntry hostEntry = null;
                        try
                        {
                            hostEntry = System.Net.Dns.GetHostEntry(inf.ep.Address);
                        }
                        catch (Exception) { }
                        if (hostEntry != null && (hostEntry.HostName != inf.ep.Address.ToString()))
                        {
                            hostname = hostEntry.HostName; ///+"-" + inf.ep.Address.ToString();
                            OnDiscoveryEvent(this, inf.ep, AmtDiscoveryEvent.MachineFound, new AmtDiscoveredComputer(hostname, inf.ep, str, tls, hostEntry, 0, 0,inf.ep.Address.ToString()));
                        }
                        else
                            OnDiscoveryEvent(this, inf.ep, AmtDiscoveryEvent.MachineFound, new AmtDiscoveredComputer(hostname, inf.ep, str, tls, hostEntry, 0, 0, null));
                    }
                    else
                    {
                        OnDiscoveryEvent(this, inf.ep, AmtDiscoveryEvent.MachineNotFound, null);
                    }
                }
            }
            LaunchNextSocket();
        }

        private void LaunchNextSocket()
        {
            if (current == null) return;

            lock (socketTable)
            {
                if (socketTable.Count > 5)
                    return;
            }
            if (!IsSameAddress(current, end))
            {
                current = GetNextAddress(current);
                if (local != current)
                 LaunchOneSocket(new IPEndPoint(current, port)) ;
            }

        }

        public void Stop()
        {

            try
            {
                if (timerThread != null) { timerThread.Interrupt(); timerThread.Abort(); }
                timerThread = null;
            }
            catch (Exception)
            {
                //MessageBox.Show(ex.Message + "\n\n" + ex.GetType().ToString() + "\n" + ex.StackTrace, "Exception");
            }
            lock (socketTable)
            {
                if (socketTable.Count > 0)
                {
                    //   Socket[] table = (Socket[])socketTable.Keys;
                    foreach (Socket s in socketTable.Keys)
                    {
                        DiscoverySocketInfo inf = (DiscoverySocketInfo)socketTable[s];
                        // Break the socket
                        try
                        {
                            inf.stopped = true;
                            s.Close();
                        }
                        catch (SocketException) { }
                    }
                    socketTable.Clear();
                }
            }
        }
    }
}


