// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.Net;

namespace ManageabilityStack
{
    [Serializable]
    public class AmtMpsServer
    {
        [Serializable]
        public class AmtMpsServerSerialize
        {
            public string networkName;
            public ForceIPOption forceOption;
            public IPAddress ipAddress;
            public ushort HttpPort;
            public ushort SocksPort;
            public ArrayList Computers;
        }

        #region Private Members

        public NetworkIdentity _NetworkNameIdentity;
        public ushort HttpPort;
        public ushort SocksPort;
        public ArrayList Computers = new ArrayList();

        #endregion

        #region Constructors

        public AmtMpsServer()
        {
        }

        public AmtMpsServer(string name, ushort httpProxyPort, ushort socksProxyPort)
        {
            this.NetworkNameIdentity = new NetworkIdentity(name);
            this.HttpPort = httpProxyPort;
            this.SocksPort = socksProxyPort;
        }

        public AmtMpsServer(NetworkIdentity netNameId, ushort httpProxyPort, ushort socksProxyPort)
        {
            this.NetworkNameIdentity = netNameId;
            this.HttpPort = httpProxyPort;
            this.SocksPort = socksProxyPort;
        }

        public AmtMpsServer(string name, ushort httpProxyPort, ushort socksProxyPort, ForceIPOption forceIPOption, IPAddress ipAddress)
        {
            this.NetworkNameIdentity = new NetworkIdentity(name, forceIPOption, ipAddress);
            this.HttpPort = httpProxyPort;
            this.SocksPort = socksProxyPort;
        }

        #endregion

        #region Public Properties

        // Intel AMT Connection information
        /// <summary>
        /// Name of the network to which the Intel(R) AMT system is connected.
        /// </summary>
        public NetworkIdentity NetworkNameIdentity
        {
            get
            {
                return _NetworkNameIdentity;
            }
            set
            {
                _NetworkNameIdentity = value;
            }
        }

        #endregion

        #region Public Methods

        #region Serialization

        public string Serialize()
        {
            AmtMpsServerSerialize s = new AmtMpsServerSerialize();
            s.networkName = NetworkNameIdentity.DisplayName;
            s.forceOption = NetworkNameIdentity.ForceIPOption;
            s.ipAddress = NetworkNameIdentity.IPAddress;
            s.HttpPort = HttpPort;
            s.SocksPort = SocksPort;
            s.Computers = new ArrayList();
            foreach (AmtSystem computer in Computers) s.Computers.Add(new AmtSerialSystemSettings(computer));
            return Utils.BytesToHex(Utils.SerializeBinary(s));
        }

        public static AmtMpsServer UnSerialize(string str)
        {
            AmtMpsServerSerialize s = (AmtMpsServerSerialize)Utils.UnserializeBinary(Utils.HexToBytes(str));
            if (s == null) return null;
            AmtMpsServer m = new AmtMpsServer(s.networkName, s.HttpPort, s.SocksPort, s.forceOption, s.ipAddress);
            m.Computers = new ArrayList();
            foreach (AmtSerialSystemSettings ss in s.Computers) m.Computers.Add(ss.GetSystem());
            return m;
        }

        #endregion

        public void AddComputer(AmtSystem computer)
        {
            computer.HttpProxy = new WebProxy(NetworkNameIdentity.ToWebURL(HttpPort));
            computer.SocksProxy = new AmtRedirectorWrapper.SOCKSProxy();
            computer.SocksProxy.server = NetworkNameIdentity.DefaultName;
            computer.SocksProxy.port = SocksPort;
            Computers.Add(computer);
        }

        public void SetProxy(AmtSystem computer)
        {
            computer.HttpProxy = new WebProxy(NetworkNameIdentity.DefaultName, HttpPort);
            computer.SocksProxy = new AmtRedirectorWrapper.SOCKSProxy();
            computer.SocksProxy.server = NetworkNameIdentity.DefaultName;
            computer.SocksProxy.port = SocksPort;
        }

        public bool RemoveComputer(AmtSystem computer)
        {
            if (!Computers.Contains(computer)) return false;
            Computers.Remove(computer);
            return true;
        }

        #endregion
    }
}
