// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Diagnostics;

namespace ManageabilityStack
{
    public class AmtNetworkAccessControlAdmin
    {
        private AmtSystem system;
        private EndpointAccessControlAdminService EACA;

        public AmtNetworkAccessControlAdmin(AmtSystem system, EndpointAccessControlAdminService eaca)
        {
            this.system = system;
            EACA = eaca;
        }

        public void Dispose()
        {
            if (EACA != null) EACA.Dispose();
            EACA = null;
        }

        public void ClearCache()
        {
            GetEACStatusCached = false;
        }

        public void FetchCache()
        {
            try
            {
                bool e;
                GetEACStatus(out e);
                uint signCertificateHandle;
                bool signCertificateHandleSpecified;
                GetPosture(out signCertificateHandle, out signCertificateHandleSpecified);
            }
            catch (Exception e)
            {
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Error,
                    "Failure to retrieve the NAC Administration data.",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
            }
        }

        private bool GetEACStatusCache = false;
        private bool GetEACStatusCached = false;
        public AmtCallStatus GetEACStatus(out bool enabled)
        {
            if (GetEACStatusCached == true)
            {
                enabled = GetEACStatusCache;
                return AmtCallStatus.SUCCESS;
            }
            AmtCallStatus r = (AmtCallStatus)EACA.GetEACStatus(out enabled);
            if (r == AmtCallStatus.SUCCESS)
            {
                GetEACStatusCache = enabled;
                GetEACStatusCached = true;
            }
            return r;
        }

        public AmtCallStatus EnableEAC(bool enabled)
        {
            AmtCallStatus r = (AmtCallStatus)EACA.EnableEAC(enabled);
            if (r == AmtCallStatus.SUCCESS) GetEACStatusCached = false;
            return r;
        }

        public AmtCallStatus UpdatePostureState(PostureUpdateType postureUpdate)
        {
            AmtCallStatus r = (AmtCallStatus)EACA.UpdatePostureState(PostureUpdateType.PostureUpdateBootCounters);
            return r;
        }

        private object[] GetPostureCache = null;
        public AmtCallStatus GetPosture(out uint signCertificateHandle, out bool signCertificateHandleSpecified)
        {
            if (GetPostureCache != null)
            {
                signCertificateHandle = (uint)GetPostureCache[0];
                signCertificateHandleSpecified = (bool)GetPostureCache[1];
                return AmtCallStatus.SUCCESS;
            }
            AmtCallStatus r = (AmtCallStatus)EACA.GetPostureSigner(out signCertificateHandle, out signCertificateHandleSpecified);
            if (r == AmtCallStatus.SUCCESS) GetPostureCache = new object[] { signCertificateHandle, signCertificateHandleSpecified };
            return r;
        }

        public AmtCallStatus SetPostureSigner(uint signCertificateHandle, bool signCertificateHandleSpecified)
        {
            AmtCallStatus r = (AmtCallStatus)EACA.SetPostureSigner(signCertificateHandle, signCertificateHandleSpecified);
            if (r == AmtCallStatus.SUCCESS) GetPostureCache = null;
            return r;
        }

    }

}
