// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Net;
using System.Runtime.InteropServices;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace ManageabilityStack
{
    public class AmtRedirectorWrapper
    {
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_Init(IntPtr versionInfo, [MarshalAs(UnmanagedType.LPStr)] string iniFilePath);
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_Close();
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_GetErrorStringLen(IMRResult res, out int len);
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_GetErrorString(IMRResult res, IntPtr errorString);
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_SetCertificateInfo([MarshalAs(UnmanagedType.LPStr)] string root_cert, [MarshalAs(UnmanagedType.LPStr)] string private_cert, [MarshalAs(UnmanagedType.LPStr)] string cert_pass);
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_AddClient(IMRClientType type, [MarshalAs(UnmanagedType.LPStr)] string ip, byte[] guid, out uint clientId);
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_RemoveClient(uint clientId);
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_RemoveAllClients();
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_GetAllClients(IntPtr client_list, out int list_size);
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_GetClientInfo(uint clientId, IntPtr info);
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_SOLOpenTCPSession(uint clientId, byte[] loginparams, IntPtr touts, IntPtr loopback);
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_SOLOpenTCPSessionEx(uint clientId, byte[] paramsEx, IntPtr touts, IntPtr loopback);
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_SOLCloseSession(uint clientId);
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_SOLSendText(uint clientId, byte[] data, int len);
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_SOLReceiveText(uint clientId, IntPtr data, ref int len);
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_IDEROpenTCPSession(uint clientId, byte[] sessionparams, IntPtr touts, [MarshalAs(UnmanagedType.LPStr)] string drive0, [MarshalAs(UnmanagedType.LPStr)] string drive1);
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_IDEROpenTCPSessionEx(uint clientId, byte[] paramsEx, IntPtr touts, [MarshalAs(UnmanagedType.LPStr)] string drive0, [MarshalAs(UnmanagedType.LPStr)] string drive1);
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_IDERCloseSession(uint clientId);
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_IDERClientFeatureSupported(uint clientId, IntPtr supportedfeatures);
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_IDERGetDeviceState(uint clientId, IntPtr state);
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_IDERSetDeviceState(uint clientId, byte[] IDERDeviceCmd, IntPtr IDERDeviceResult);
        [DllImport("imrsdk.dll")]
        static extern IMRResult IMR_IDERGetSessionStatistics(uint clientId, IntPtr IDERStatistics);

        /* List of all imrsdk.dll functions
          1 *IMR_AddClient
          2 *IMR_Close
          3 *IMR_GetAllClients
          4 *IMR_GetClientInfo
          5 *IMR_GetErrorString
          6 *IMR_GetErrorStringLen
          7 IMR_IDERClientFeatureSupported
          8 IMR_IDERCloseSession
          9 IMR_IDERGetDeviceState
         10 IMR_IDERGetSessionStatistics
         11 IMR_IDEROpenTCPSession
         12 IMR_IDEROpenTCPSessionEx
         13 IMR_IDERSetDeviceState
         14 *IMR_Init
         15 *IMR_RemoveAllClients
         16 *IMR_RemoveClient
         17 *IMR_SOLCloseSession
         18 *IMR_SOLOpenTCPSession
         19 *IMR_SOLOpenTCPSessionEx
         20 *IMR_SOLReceiveText
         21 *IMR_SOLSendText
         22 *IMR_SetCertificateInfo
        */

        public static bool CheckRedirectionWrapper()
        {
            System.IO.Directory.SetCurrentDirectory(System.Windows.Forms.Application.StartupPath);
            return File.Exists("imrsdk.dll");
        }

        private static int libRefCounter = 0;
        private bool redirect = false;
        public bool RedirectActive { get { return redirect; } }
        private X509Certificate2 Certificate = null;
        public float VersionInfo = 0;
        private bool disposed = false;
        private IntPtr GlobalReceiveBuffer = IntPtr.Zero;
        private const int GlobalReceiveBufferlen = 1024;

        [StructLayout(LayoutKind.Sequential)]
        private struct ClientInfo
        {
            public IMRClientType type;
            public IPAddress ip;
            public Guid guid;
        };

        private enum IMRClientType
        {
            CLI_TCP = 1,
            CLI_TLS = 2,
        };

        public enum IMRResult
        {
            IMR_RES_OK,
            IMR_RES_ERROR,
            IMR_RES_INVALID_PARAMETER,
            IMR_RES_NOT_INITIALIZED,
            IMR_RES_ALREADY_INITIALIZED,
            IMR_RES_MEMALLOC_FAILED,
            IMR_RES_UNSUPPORTED,
            IMR_RES_CLIENT_NOT_FOUND,
            IMR_RES_DUPLICATE_CLIENT,
            IMR_RES_CLIENT_NOT_ACTIVE,
            IMR_RES_CLIENT_ACTIVE,
            IMR_RES_SESSION_ALREADY_OPEN,
            IMR_RES_SESSION_CLOSED,
            IMR_RES_SOCKET_ERROR,
            IMR_RES_UNKNOWN_PROTOCOL,
            IMR_RES_PROTOCOL_ALREADY_REGISTERED,
            IMR_RES_PENDING,
            IMR_RES_UNEXPECTED_PACKET,
            IMR_RES_TIMEOUT,
            IMR_RES_CORRUPT_PACKET,
            IMR_RES_OS_ERROR,
            IMR_RES_IDER_VERSION_NOT_SUPPORTED,
            IMR_RES_IDER_COMMAND_RUNNING,
            IMR_RES_STORAGE_FAILURE,
            IMR_RES_UNKNOWN,
            IMR_RES_AUTH_FAILED,
            IMR_RES_CLIENT_TYPE_UNKNOWN,
            IMR_RES_CLIENT_BUSY,
            IMR_RES_CLIENT_UNSUPPORTED,
            IMR_RES_CLIENT_ERROR,
            IMR_RES_NOT_ENOUGH_SPACE,
            IMR_RES_SESSION_LOOPBACK,
            IMR_RES_TLS_CONNECTION_FAILED,
            IMR_RES_SOCKS_ERROR,
            IMR_RES_SOCKS_AUTH_FAIL,
            IMR_RES_OOB_SOL_DISABLED,
            IMR_RES_AUDIT_LOG_FULL,
            IMR_RES_CLIENT_INVALID_OPTION,
            IMR_RES_USER_CONSENT_REQUIRED,
            IMR_RES_COUNT,   // must be last entry
        };

        [Serializable]
        public class SOCKSProxy
        {
            public string server;
            public ushort port;
            public string user;
            public string password;
        }

        private enum PROXY_TYPE
        {
            PROXY_NO_PROXY,
            PROXY_SOCKS_V5
        };

        private string ip;
        private string username;
        private string password;
        private string domain; 
        private uint clientId = uint.MaxValue;
        private Guid guid;
        private bool SOL = false;
        private Thread listenerThread = null;
        private bool UseTls;
        private static object GlobalLock = new Object();
        private SOCKSProxy proxy;

        private X509Certificate2[] serverCerts = null;
        public X509Certificate2[] ServerCerts
        {
            get { return this.serverCerts; }
            set { this.serverCerts = value; }
        }

        public delegate void SerialDataHandler(AmtRedirectorWrapper wrapper, byte[] data, int len);
        public event SerialDataHandler SerialData;

        public AmtRedirectorWrapper(string ip, string username, string password, bool tls, X509Certificate2 cert)
        {
            this.ip = ip;
            this.username = username;
            this.password = password;
            this.domain = null; 

            if (username.Contains("\\"))
            {
                //split domain/username 
                this.domain = username.Substring(0, username.IndexOf("\\"));
                this.username = username.Substring(username.IndexOf("\\") + 1);
            }

            this.Certificate = cert;
            guid = Guid.NewGuid();
            UseTls = tls;
            this.proxy = null;
            Initialize();
            AddClient();
        }

        public AmtRedirectorWrapper(string ip, string username, string password, bool tls, X509Certificate2 cert, SOCKSProxy proxy)
        {
            this.ip = ip;
            this.username = username;
            this.password = password;
            this.domain = null; 

            if (username.Contains("\\"))
            {
                //split domain/username 
                this.domain = username.Substring(0, username.IndexOf("\\"));
                this.username = username.Substring(username.IndexOf("\\") + 1);
            }

            this.Certificate = cert;
            guid = Guid.NewGuid();
            UseTls = tls;
            this.proxy = proxy;
            Initialize();
            AddClient();
        }

        private IMRResult AddClient()
        {
            IMRResult r;
            lock (GlobalLock)
            {
                IMRClientType clientType = IMRClientType.CLI_TCP;
                if (UseTls) clientType = IMRClientType.CLI_TLS;
                r = IMR_AddClient(clientType, ip, guid.ToByteArray(), out clientId);
                if (r != IMRResult.IMR_RES_OK)
                {
                    if (r == IMRResult.IMR_RES_DUPLICATE_CLIENT)
                    {
                        uint[] clientlist = ExGetClientList();
                        foreach (uint id in clientlist)
                        {
                            ClientInfo c = ExGetClientInfo(id);
                            if (c.ip == new IPAddress((long)id))
                            {
                                clientId = id;
                                break;
                            }
                        }
                        return IMRResult.IMR_RES_OK;
                    }
                    else
                    {
                        // AddClient failed.
                        clientId = uint.MaxValue;
                        return r;
                    }
                }
            }
            return r;

            /*
            if (clientId == uint.MaxValue)
            {
                //MessageBox.Show(r.ToString(), ManageabilityStack.Properties.Resources.RedirectorWrapper_MsgBox_1, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return IMRResult.
            }
            */

            /*
            if (File.Exists("Trusted Root Certificates.pem"))
            {
                // TLS support initialized here
                if (cert != null)
                {
                    // Convert the certificate to PEM
                    CertificateOperations.CertAndKeyToPemFile(cert, "PrivateCert.pem", null);
                    r = IMR_SetCertificateInfo("Trusted Root Certificates.pem", Application.StartupPath + "\\PrivateCert.pem", null);
                }
                else
                {
                    r = IMR_SetCertificateInfo("Trusted Root Certificates.pem", null, null);
                }
            }
            */

            /*
            if (r != IMRResult.IMR_RES_OK)
            {
                MessageBox.Show(ManageabilityStack.Properties.Resources.RedirectorWrapper_MsgBox_2_1 + r.ToString(), ManageabilityStack.Properties.Resources.RedirectorWrapper_MsgBox_2_2, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            */
        }

        private void Initialize()
        {
            IMRResult r;
            lock (GlobalLock)
            {
                if (libRefCounter++ == 0)
                {
                    // This is the first instance of this DLL, so, lets initialize it.
                    if (File.Exists(Application.StartupPath + "\\imrsdk.ini") == false)
                    {
                        byte[] buf = UTF8Encoding.UTF8.GetBytes("[COMMON]\r\nDebug_Level=1\r\nStorage_Enabled=0\r\n");
                        FileStream f = new FileStream(Application.StartupPath + "\\imrsdk.ini", FileMode.Create);
                        f.Write(buf, 0, buf.Length);
                        f.Close();
                    }

                    IntPtr str = Marshal.AllocHGlobal(4);
                    r = IMR_Init(str, Application.StartupPath + "\\imrsdk.ini");
                    short v1 = Marshal.ReadInt16(str, 0);
                    short v2 = Marshal.ReadInt16(str, 2);
                    VersionInfo = (((float)v2) / 10) + v1;
                    Marshal.FreeHGlobal(str);

                    // Lets go ahead and remove all the clients, there should be none anyway, but since
                    // this DLL keeps a client list on file, it's possible to have clients already setup.
                    uint[] clientlist = ExGetClientList();
                    if (clientlist != null && clientlist.Length != 0)
                    {
                        foreach (uint id in clientlist)
                        {
                            IMR_RemoveClient(id);
                        }
                    }
                }
            }
        }

        public IMRResult StartSerialRedirect()
        {
            if (clientId == uint.MaxValue) return IMRResult.IMR_RES_UNKNOWN;
            if (libRefCounter == 0) Initialize();

            IMRResult r = IMRResult.IMR_RES_OK;
            if (!File.Exists("Trusted Root Certificates.pem"))
            {
                // Attempt to create the trusted root certificates file.
                CertificateOperations.AllTrustedRootsToPemFile("Trusted Root Certificates.pem");
            }
            if (this.ServerCerts != null)
            {
                // Use the given server certificates (from the client Intel(R) 
                // AMT system to generate the certificate chain for SOL.
                // NOTE: This is a work around for when Sub-CA certs are not 
                // available locally to the management system (where Commander 
                // is running).
                CertificateOperations.AllRootsToPemFile("Trusted Root Certificates.pem", this.ServerCerts);
            }

            if (File.Exists("Trusted Root Certificates.pem"))
            {
                // TLS support initialized here
                if (Certificate != null)
                {
                    // Convert the certificate to PEM
                    CertificateOperations.CertAndKeyChainToPemFile(Certificate, "PrivateCert.pem", null);
                    r = IMR_SetCertificateInfo("Trusted Root Certificates.pem", Application.StartupPath + "\\PrivateCert.pem", null);
                }
                else
                {
                    r = IMR_SetCertificateInfo("Trusted Root Certificates.pem", null, null);
                }
            }

            if (r != IMRResult.IMR_RES_OK)
            {
                //MessageBox.Show(ManageabilityStack.Properties.Resources.RedirectorWrapper_MsgBox_2_1 + r.ToString(), ManageabilityStack.Properties.Resources.RedirectorWrapper_MsgBox_2_2, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return r;
            }

            lock (GlobalLock)
            {

                r = ExSOLOpenTCPSessionEx(clientId, username, password, domain, proxy);

            }
            if (r != IMRResult.IMR_RES_OK)
            {
                return r;
                //MessageBox.Show(ManageabilityStack.Properties.Resources.RedirectorWrapper_MsgBox_3_1 + r.ToString(), ManageabilityStack.Properties.Resources.RedirectorWrapper_MsgBox_3_2, MessageBoxButtons.OK, MessageBoxIcon.Error);
                //return false;
            }
            SOL = true;

            if (GlobalReceiveBuffer == IntPtr.Zero) GlobalReceiveBuffer = Marshal.AllocHGlobal(GlobalReceiveBufferlen);
            listenerThread = new Thread(new ThreadStart(SerialLoop));
            listenerThread.Start();

            return IMRResult.IMR_RES_OK;
        }

        public void StopSerialRedirect()
        {
            if (SOL == false) return;
            SOL = false;
            if (listenerThread != null) while (listenerThread.IsAlive) Thread.Sleep(10);
            new Thread(new ThreadStart(StopSerialRedirectEx)).Start();
        }

        private void StopSerialRedirectEx()
        {
            lock (GlobalLock)
            {
                listenerThread = null;
                IMR_SOLCloseSession(clientId);
            }
        }

        public void Write(string text)
        {
            if (clientId == uint.MaxValue) return;
            byte[] buffer = UTF8Encoding.UTF8.GetBytes(text);
            IMR_SOLSendText(clientId, buffer, buffer.Length);
        }

        public void Write(byte[] buffer)
        {
            if (clientId == uint.MaxValue) return;
            IMR_SOLSendText(clientId, buffer, buffer.Length);
        }

        private void SerialLoop()
        {
            try
            {
                byte[] recv;
                IMRResult r;
                while (SOL == true)
                {
                    r = ExSOLReceiveText(GlobalReceiveBuffer, clientId, out recv);
                    if (r != IMRResult.IMR_RES_OK)
                    {
                        // Disconnection
                        if (r == IMRResult.IMR_RES_SESSION_CLOSED)
                        {
                            SOL = false;
                            listenerThread = null;
                            if (SerialData != null) SerialData(this, null, 0);
                        }
                        else
                        {
                            SOL = false;
                            listenerThread = null;
                            IMR_SOLCloseSession(clientId);
                            if (SerialData != null) SerialData(this, null, 0);
                        }
                        return;
                    }
                    else if (recv != null && SerialData != null)
                    {
                        // Received data
                        SerialData(this, recv, recv.Length);
                    }
                    else
                    {
                        // Received nothing, just wait
                        if (SOL == false) return;
                        Thread.Sleep(50);
                    }
                }
            }
            catch (ThreadInterruptedException) { }
            if (GlobalReceiveBuffer != IntPtr.Zero)
            {
                Marshal.FreeHGlobal(GlobalReceiveBuffer);
                GlobalReceiveBuffer = IntPtr.Zero;
            }
        }

        public IMRResult StartRedirect(string drive0, string drive1, RedirectState state)
        {
            if (clientId == uint.MaxValue) return IMRResult.IMR_RES_UNKNOWN;
            if (libRefCounter == 0) Initialize();

            IMRResult rr = IMRResult.IMR_RES_OK;
            if (!File.Exists("Trusted Root Certificates.pem"))
            {
                // Attempt to create the trusted root certificates file.
                CertificateOperations.AllTrustedRootsToPemFile("Trusted Root Certificates.pem");
            }
            if (this.ServerCerts != null)
            {
                // Use the given server certificates (from the client Intel(R) 
                // AMT system to generate the certificate chain for IDER.
                // NOTE: This is a work around for when Sub-CA certs are not 
                // available locally to the management system (where Commander 
                // is running).
                CertificateOperations.AllRootsToPemFile("Trusted Root Certificates.pem", this.ServerCerts);
            }

            if (File.Exists("Trusted Root Certificates.pem"))
            {
                // TLS support initialized here
                if (Certificate != null)
                {
                    // Convert the certificate to PEM
                    CertificateOperations.CertAndKeyChainToPemFile(Certificate, "PrivateCert.pem", null);
                    rr = IMR_SetCertificateInfo("Trusted Root Certificates.pem", Application.StartupPath + "\\PrivateCert.pem", null);
                }
                else
                {
                    rr = IMR_SetCertificateInfo("Trusted Root Certificates.pem", null, null);
                }
            }

            if (rr != IMRResult.IMR_RES_OK)
            {
                return rr;
                //MessageBox.Show(ManageabilityStack.Properties.Resources.RedirectorWrapper_MsgBox_2_1 + rr.ToString(), ManageabilityStack.Properties.Resources.RedirectorWrapper_MsgBox_2_2, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            rr = ExIDEROpenTCPSessionEx(clientId, username, password, domain, drive0, drive1, proxy);

            
            if (rr == IMRResult.IMR_RES_OK)
            {
                redirect = true;
                lock (GlobalLock)
                {
                    ExIDERSetDeviceState(clientId, RedirectOperations.IDER_ENABLE, state);
                }
            }
            return rr;
        }

        public IMRResult StopRedirect()
        {
            if (clientId == uint.MaxValue) return IMRResult.IMR_RES_UNKNOWN;
            ExIDERSetDeviceState(clientId, RedirectOperations.IDER_DISABLE, RedirectState.IDER_SET_IMMEDIATELY);
            IMRResult r;
            lock (GlobalLock)
            {
                r = ExIDERCloseSession(clientId);
            }
            if (r == IMRResult.IMR_RES_OK) { redirect = false; }
            return r;
        }

        public void Dispose()
        {
            if (disposed == true) return;
            disposed = true;
            SOL = false;
            if (redirect == true) StopRedirect();
            if (listenerThread != null) listenerThread.Interrupt();
            listenerThread = null;
            if (clientId != uint.MaxValue) IMR_RemoveClient(clientId);
            clientId = uint.MaxValue;
            if (--libRefCounter == 0) IMR_Close();
        }

        public void Reset()
        {
            lock (GlobalLock)
            {
                IMRResult r;
                r = IMR_RemoveClient(clientId);
                redirect = false;
                r = IMR_AddClient(IMRClientType.CLI_TCP, ip, guid.ToByteArray(), out clientId);
            }
        }

        private static string ExGetErrorString(IMRResult error)
        {
            int len = 0;
            IMR_GetErrorStringLen(error, out len);
            IntPtr str = Marshal.AllocHGlobal(len);
            IMR_GetErrorString(error, str);
            string errstr = Marshal.PtrToStringAnsi(str, len - 1);
            Marshal.FreeHGlobal(str);
            return errstr;
        }

        private static uint[] ExGetClientList()
        {
            IMRResult r;
            int list_size;
            r = IMR_GetAllClients(IntPtr.Zero, out list_size);
            if (r != IMRResult.IMR_RES_OK) return null;
            IntPtr ulongarray = Marshal.AllocHGlobal(4 * list_size);
            r = IMR_GetAllClients(ulongarray, out list_size);
            if (r != IMRResult.IMR_RES_OK)
            {
                Marshal.FreeHGlobal(ulongarray);
                return null;
            }
            uint[] arr = new uint[list_size];
            for (int i = 0; i < list_size; i++)
            {
                arr[i] = (uint)Marshal.ReadInt32(ulongarray, i * 4);
            }
            Marshal.FreeHGlobal(ulongarray);
            return arr;
        }

        private static ClientInfo ExGetClientInfo(uint clientId)
        {
            lock (GlobalLock)
            {
                IMRResult r;
                IntPtr info = Marshal.AllocHGlobal(148);
                r = IMR_GetClientInfo(clientId, info);
                ClientInfo ci = new ClientInfo();
                ci.type = (IMRClientType)Marshal.ReadInt32(info, 0);
                string ipstr = Marshal.PtrToStringAnsi(info, 132);
                ipstr = ipstr.Substring(4);
                ipstr = ipstr.Substring(0, ipstr.IndexOf("\0"));
                IPAddress[] addrs = System.Net.Dns.GetHostAddresses(ipstr);
                if (addrs == null || addrs.Length == 0)
                {
                    Marshal.FreeHGlobal(info);
                    return new ClientInfo();
                }
                ci.ip = addrs[0];

                byte[] x = new byte[16];
                for (int i = 0; i < 16; i++)
                {
                    x[i] = Marshal.ReadByte(info, i + 132);
                }
                ci.guid = new Guid(x);

                Marshal.FreeHGlobal(info);
                return ci;
            }
        }

        private static IMRResult ExSOLOpenTCPSession(uint clientId, string username, string password)
        {
            byte[] login = new byte[256];
            UTF8Encoding.UTF8.GetBytes(username, 0, username.Length, login, 0);
            UTF8Encoding.UTF8.GetBytes(password, 0, password.Length, login, 128);

            bool sol_timeout = (((string)Application.UserAppDataRegistry.GetValue("SOL-Timeout", Boolean.FalseString)).CompareTo(Boolean.TrueString) == 0);
            IntPtr data = IntPtr.Zero;
            if (sol_timeout)
            {
                ushort sol1 = ushort.Parse((string)Application.UserAppDataRegistry.GetValue("SOL-Timeout-1", "0"));
                ushort sol2 = ushort.Parse((string)Application.UserAppDataRegistry.GetValue("SOL-Timeout-2", "100"));
                ushort sol3 = ushort.Parse((string)Application.UserAppDataRegistry.GetValue("SOL-Timeout-3", "5000"));
                ushort sol4 = ushort.Parse((string)Application.UserAppDataRegistry.GetValue("SOL-Timeout-4", "100"));
                ushort sol5 = ushort.Parse((string)Application.UserAppDataRegistry.GetValue("SOL-Timeout-5", "10000"));
                data = Marshal.AllocHGlobal(10);
                Marshal.WriteInt16(data, 0, (short)sol1); // tx_over_timeout (0)
                Marshal.WriteInt16(data, 2, (short)sol2); // tx_buf_timeout (100)
                Marshal.WriteInt16(data, 4, (short)sol3); // hb_interval (5000)
                Marshal.WriteInt16(data, 6, (short)sol4); // fifo_rx_flush_timeout (100)
                Marshal.WriteInt16(data, 8, (short)sol5); // rx_timeout (10000)
            }
            else
            {
                // Passing NULL as data should not, but is causes SOL disconnects, so we always pass explicit values.
                data = Marshal.AllocHGlobal(10);
                Marshal.WriteInt16(data, 0, (short)0);     // tx_over_timeout (0)
                Marshal.WriteInt16(data, 2, (short)100);   // tx_buf_timeout (100)
                Marshal.WriteInt16(data, 4, (short)5000);  // hb_interval (5000)
                Marshal.WriteInt16(data, 6, (short)100);   // fifo_rx_flush_timeout (100)
                Marshal.WriteInt16(data, 8, (short)10000); // rx_timeout (10000)
            }
            IMRResult r = IMR_SOLOpenTCPSession(clientId, login, data, IntPtr.Zero);
            if (data != IntPtr.Zero) Marshal.FreeHGlobal(data);

            return r;
        }

        private static IMRResult ExSOLOpenTCPSessionEx(uint clientId, string username, string password, string domain, SOCKSProxy proxy)
        {
            byte[] paramsEx = CreateTCPSessionParamsEx(username, password, domain, proxy);

            bool sol_timeout = (((string)Application.UserAppDataRegistry.GetValue("SOL-Timeout", Boolean.FalseString)).CompareTo(Boolean.TrueString) == 0);
            IntPtr data = IntPtr.Zero;
            if (sol_timeout)
            {
                ushort sol1 = ushort.Parse((string)Application.UserAppDataRegistry.GetValue("SOL-Timeout-1", "0"));
                ushort sol2 = ushort.Parse((string)Application.UserAppDataRegistry.GetValue("SOL-Timeout-2", "100"));
                ushort sol3 = ushort.Parse((string)Application.UserAppDataRegistry.GetValue("SOL-Timeout-3", "5000"));
                ushort sol4 = ushort.Parse((string)Application.UserAppDataRegistry.GetValue("SOL-Timeout-4", "100"));
                ushort sol5 = ushort.Parse((string)Application.UserAppDataRegistry.GetValue("SOL-Timeout-5", "10000"));
                data = Marshal.AllocHGlobal(10);
                Marshal.WriteInt16(data, 0, (short)sol1); // tx_over_timeout (0)
                Marshal.WriteInt16(data, 2, (short)sol2); // tx_buf_timeout (100)
                Marshal.WriteInt16(data, 4, (short)sol3); // hb_interval (5000)
                Marshal.WriteInt16(data, 6, (short)sol4); // fifo_rx_flush_timeout (100)
                Marshal.WriteInt16(data, 8, (short)sol5); // rx_timeout (10000)
            }
            else
            {
                // Passing NULL as data should not, but is causes SOL disconnects, so we always pass explicit values.
                data = Marshal.AllocHGlobal(10);
                Marshal.WriteInt16(data, 0, (short)0);     // tx_over_timeout (0)
                Marshal.WriteInt16(data, 2, (short)100);   // tx_buf_timeout (100)
                Marshal.WriteInt16(data, 4, (short)5000);  // hb_interval (5000)
                Marshal.WriteInt16(data, 6, (short)100);   // fifo_rx_flush_timeout (100)
                Marshal.WriteInt16(data, 8, (short)10000); // rx_timeout (10000)
            }
            IMRResult r = IMR_SOLOpenTCPSessionEx(clientId, paramsEx, data, IntPtr.Zero);
            if (data != IntPtr.Zero) Marshal.FreeHGlobal(data);

            return r;
        }

        private static bool ExSOLCloseSession(uint clientId)
        {
            return (IMR_SOLCloseSession(clientId) == IMRResult.IMR_RES_OK);
        }

        private static IMRResult ExSOLReceiveText(IntPtr GlobalReceiveBuffer, uint clientId, out byte[] buf)
        {
            IMRResult r;
            if (GlobalReceiveBuffer == IntPtr.Zero)
            {
                buf = null;
                return IMRResult.IMR_RES_NOT_INITIALIZED;
            }
            int len = GlobalReceiveBufferlen;
            r = IMR_SOLReceiveText(clientId, GlobalReceiveBuffer, ref len);
            if (r != IMRResult.IMR_RES_OK || len == 0)
            {
                buf = null;
                return r;
            }
            buf = new byte[len];
            Marshal.Copy(GlobalReceiveBuffer, buf, 0, len);
            return r;
        }

        private static IMRResult ExIDEROpenTCPSession(uint clientId, string username, string password, string drive0, string drive1)
        {
            byte[] login = new byte[256];
            UTF8Encoding.UTF8.GetBytes(username, 0, username.Length, login, 0);
            UTF8Encoding.UTF8.GetBytes(password, 0, password.Length, login, 128);

            bool ide_timeout = (((string)Application.UserAppDataRegistry.GetValue("IDER-Timeout", Boolean.FalseString)).CompareTo(Boolean.TrueString) == 0);
            IntPtr data = IntPtr.Zero;
            if (ide_timeout)
            {
                ushort ider1 = ushort.Parse((string)Application.UserAppDataRegistry.GetValue("IDER-Timeout-1", "10000"));
                ushort ider2 = ushort.Parse((string)Application.UserAppDataRegistry.GetValue("IDER-Timeout-2", "0"));
                ushort ider3 = ushort.Parse((string)Application.UserAppDataRegistry.GetValue("IDER-Timeout-3", "5000"));
                data = Marshal.AllocHGlobal(6);
                Marshal.WriteInt16(data, 0, (short)ider1); // rx_timeout (10000)
                Marshal.WriteInt16(data, 2, (short)ider2); // tx_timeout (0);
                Marshal.WriteInt16(data, 4, (short)ider3); // hb_timeout (5000)
            }
            else
            {
                data = Marshal.AllocHGlobal(6);
                Marshal.WriteInt16(data, 0, (short)10000); // rx_timeout (10000)
                Marshal.WriteInt16(data, 2, (short)0);     // tx_timeout (0);
                Marshal.WriteInt16(data, 4, (short)5000);  // hb_timeout (5000)
            }
            IMRResult r = IMR_IDEROpenTCPSession(clientId, login, data, drive0, drive1);
            if (data != IntPtr.Zero) Marshal.FreeHGlobal(data);
            return r;
        }

        private static IMRResult ExIDEROpenTCPSessionEx(uint clientId, string username, string password, string domain, string drive0, string drive1, SOCKSProxy proxy)
        {
            byte[] paramEx = CreateTCPSessionParamsEx(username, password, domain, proxy);

            bool ide_timeout = (((string)Application.UserAppDataRegistry.GetValue("IDER-Timeout", Boolean.FalseString)).CompareTo(Boolean.TrueString) == 0);
            IntPtr data = IntPtr.Zero;
            if (ide_timeout)
            {
                ushort ider1 = ushort.Parse((string)Application.UserAppDataRegistry.GetValue("IDER-Timeout-1", "10000"));
                ushort ider2 = ushort.Parse((string)Application.UserAppDataRegistry.GetValue("IDER-Timeout-2", "0"));
                ushort ider3 = ushort.Parse((string)Application.UserAppDataRegistry.GetValue("IDER-Timeout-3", "5000"));
                data = Marshal.AllocHGlobal(6);
                Marshal.WriteInt16(data, 0, (short)ider1); // rx_timeout (10000)
                Marshal.WriteInt16(data, 2, (short)ider2); // tx_timeout (0);
                Marshal.WriteInt16(data, 4, (short)ider3); // hb_timeout (5000)
            }
            else
            {
                data = Marshal.AllocHGlobal(6);
                Marshal.WriteInt16(data, 0, (short)10000); // rx_timeout (10000)
                Marshal.WriteInt16(data, 2, (short)0);     // tx_timeout (0);
                Marshal.WriteInt16(data, 4, (short)5000);  // hb_timeout (5000)
            }
            IMRResult r = IMR_IDEROpenTCPSessionEx(clientId, paramEx, data, drive0, drive1);
            if (data != IntPtr.Zero) Marshal.FreeHGlobal(data);
            return r;
        }

        private static IMRResult ExIDERCloseSession(uint clientId)
        {
            return IMR_IDERCloseSession(clientId);
        }

        private enum RedirectOperations
        {
            IDER_ENABLE,
            IDER_DISABLE,
            IDER_NOP
        };

        public enum RedirectState
        {
            IDER_SET_ONRESET,
            IDER_SET_GRACEFULLY,
            IDER_SET_IMMEDIATELY,
        };

        /*
        private static bool ExIDERClientFeatureSupported(uint clientid, IntPtr supportedfeatures)
        {
            return false;
        }

        private static bool ExIDERGetDeviceState(uint clientId, IntPtr state)
        {
            return false;
        }
        */

        private static bool ExIDERSetDeviceState(uint clientId, RedirectOperations operation, RedirectState state)
        {
            MemoryStream MS = new MemoryStream();
            MS.Write(BitConverter.GetBytes((int)operation), 0, 4);
            MS.Write(BitConverter.GetBytes((int)state), 0, 4);
            MS.Write(BitConverter.GetBytes((int)RedirectOperations.IDER_NOP), 0, 4);
            MS.Write(BitConverter.GetBytes((int)RedirectState.IDER_SET_ONRESET), 0, 4);
            byte[] op = MS.ToArray();

            IntPtr resultbin = Marshal.AllocHGlobal(8);
            IMRResult r = IMR_IDERSetDeviceState(clientId, op, resultbin);
            int ret = Marshal.ReadInt32(resultbin, 0);
            Marshal.FreeHGlobal(resultbin);
            return (r == IMRResult.IMR_RES_OK && ret == 1);
        }

        public class AmtIdeStats
        {
            public bool error_state;
            public bool data_transfer;
            public ushort num_reopen;
            public uint num_error;
            public uint num_reset;
            public uint last_cmd_length;
            public uint data_sent;
            public uint data_received;
            public uint packets_sent;
            public uint packets_received;
        };

        public AmtIdeStats GetRedirectorStats()
        {
            AmtIdeStats stats = ExIDERGetSessionStatistics(clientId);
            if (stats == null && redirect == true)
            {
                redirect = false;
            }
            if (stats != null && redirect == true)
            {
                redirect = true;
            }
            return stats;
        }

        private static AmtIdeStats ExIDERGetSessionStatistics(uint clientId)
        {
            IntPtr statsbin = Marshal.AllocHGlobal(40);
            IMRResult r = IMR_IDERGetSessionStatistics(clientId, statsbin);
            if (r != IMRResult.IMR_RES_OK)
            {
                Marshal.FreeHGlobal(statsbin);
                return null;
            }

            AmtIdeStats stats = new AmtIdeStats();
            stats.error_state = Marshal.ReadInt32(statsbin, 0) == 0;
            stats.data_transfer = Marshal.ReadInt32(statsbin, 4) == 0;
            stats.num_reopen = (ushort)Marshal.ReadInt32(statsbin, 8);
            stats.num_error = (ushort)Marshal.ReadInt32(statsbin, 12);
            stats.num_reset = (ushort)Marshal.ReadInt32(statsbin, 16);
            stats.last_cmd_length = (uint)Marshal.ReadInt32(statsbin, 20);
            stats.data_sent = (uint)Marshal.ReadInt32(statsbin, 24);
            stats.data_received = (uint)Marshal.ReadInt32(statsbin, 28);
            stats.packets_sent = (uint)Marshal.ReadInt32(statsbin, 32);
            stats.packets_received = (uint)Marshal.ReadInt32(statsbin, 36);

            Marshal.FreeHGlobal(statsbin);

            return stats;
        }

        private static byte[] CreateTCPSessionParamsEx(string username, string password, string domain, SOCKSProxy proxy)
        {
            //typedef struct {
            //  int version;                    //must be 3
            //  char user_name[MAX_WIN_UN_LEN];
            //  char user_pswd[MAX_PSWD_LEN];
            //  char user_domain[MAX_WIN_DN_LEN];
            //  ProxySettings proxy_settings
            //} TCPSessionParamsEx2;

            // 
            // typedef struct {
            //   PROXY_TYPE type;               // enum PROXY_NO_PROXY, PROXY_SOCKS_V5
            //   char server[MAX_IP_LEN];       // MAX_IP_LEN = 128
            //   int port;
            //   char user[MAX_NAME_LEN];       // MAX_NAME_LEN = 128
            //   char password[MAX_PSWD_LEN];   // MAZ_PSWD_LEN = 128
            // } ProxyType;
            const int version = 3;
            const int MAX_WIN_UN_LEN = 257; 
            const int MAX_WIN_DN_LEN = 254; 
            const int MAX_NAME_LEN = 128;
            const int MAX_PSWD_LEN = 128;
            const int MAX_IP_LEN = 128;
            byte[] buffer = null;

            MemoryStream MS = new MemoryStream(4 + MAX_WIN_UN_LEN + MAX_PSWD_LEN + MAX_WIN_DN_LEN + 4 + MAX_IP_LEN + 4 + MAX_NAME_LEN + MAX_PSWD_LEN);

            // TCPSessionParamsEx
            MS.Write(BitConverter.GetBytes(version), 0, 4);

            buffer = new byte[MAX_WIN_UN_LEN];
            UTF8Encoding.UTF8.GetBytes(username, 0, Math.Min(username.Length, MAX_WIN_UN_LEN), buffer, 0);
            MS.Write(buffer, 0, buffer.Length);

            buffer = new byte[MAX_PSWD_LEN];
            UTF8Encoding.UTF8.GetBytes(password, 0, Math.Min(password.Length, MAX_NAME_LEN), buffer, 0);
            MS.Write(buffer, 0, buffer.Length);

            if (domain != null)
            {
                buffer = new byte[MAX_WIN_DN_LEN];
                UTF8Encoding.UTF8.GetBytes(domain, 0, Math.Min(domain.Length, MAX_WIN_DN_LEN), buffer, 0);
                MS.Write(buffer, 0, buffer.Length);
            }
            else
            {
                buffer = new byte[MAX_WIN_DN_LEN];
                MS.Write(buffer, 0, buffer.Length);
            }

            // ProxyType
            if (proxy == null)
            {
                MS.Write(BitConverter.GetBytes((uint)PROXY_TYPE.PROXY_NO_PROXY), 0, 4);

                // Fill the remaining fields of the ProxyType struct with zeroes
                buffer = new byte[4 + MAX_NAME_LEN + MAX_PSWD_LEN];
                MS.Write(buffer, 0, buffer.Length);

            }
            else
            {
                MS.Write(BitConverter.GetBytes((uint)PROXY_TYPE.PROXY_SOCKS_V5), 0, 4);

                buffer = new byte[MAX_IP_LEN];
                UTF8Encoding.UTF8.GetBytes(proxy.server, 0, Math.Min(proxy.server.Length, MAX_IP_LEN), buffer, 0);
                MS.Write(buffer, 0, buffer.Length);

                MS.Write(BitConverter.GetBytes((uint)proxy.port), 0, 4);

                if (proxy.user != null)
                {
                    buffer = new byte[MAX_NAME_LEN];
                    UTF8Encoding.UTF8.GetBytes(proxy.user, 0, Math.Min(proxy.user.Length, MAX_NAME_LEN), buffer, 0);
                    MS.Write(buffer, 0, buffer.Length);

                    buffer = new byte[MAX_PSWD_LEN];
                    UTF8Encoding.UTF8.GetBytes(proxy.password, 0, Math.Min(proxy.password.Length, MAX_NAME_LEN), buffer, 0);
                    MS.Write(buffer, 0, buffer.Length);
                }
                else
                {
                    // Fill the proxy name and password fields with zeroes
                    buffer = new byte[MAX_NAME_LEN + MAX_PSWD_LEN];
                    MS.Write(buffer, 0, buffer.Length);
                }

            }

            return MS.ToArray();
        }

    }
}
