// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System.Security.Cryptography.X509Certificates;

namespace ManageabilityStack
{
    public class AmtSecuritySettings
    {
        public string CoreVersion;
        public float CoreVersionDec;
        public AmtSecurityAdmin.EnabledInterfaces EnabledInterfaces;
        public AmtSecurityAdmin.AmtPowerPackage[] PowerPackages;
        public AmtSecurityAdmin.AmtPowerPackage ActivePowerPackage;
        public AmtSecurityAdmin.ProvisioningMode ProvisioningMode;
        public CrlType[] Crls;
        public string[] TrustedFqdnCN;
        public PkiCapsType PkiCapabilities;
        public string DigestRealm;
        public X509Certificate2[] TLSCertificate;
        public X509Certificate2[] TrustedRootCertificates;
        public TlsAthenticationType TlsAthenticationLocal;
        public TlsAthenticationType TlsAthenticationRemote;
        public KerberosOptionsType KerberosOptions;
        public string AdminAclEntry;
        public AmtSecurityAdmin.AmtUserAcl[] UserAclEntries;
        public int ActiveStateAC;
        public int WakeOnNetAccessThresholdAC;
        public int WakeOnNetAccessSleepTimer;
        public bool RedirectionPort;

        public AmtSecuritySettings(AmtSecurityAdmin secAdmin, AmtRedirect redirect)
        {
            AmtCallStatus r;
            CoreVersion = secAdmin.GetCoreVersion();

            // Parse the version
            CoreVersionDec = 0;
            string[] vers = CoreVersion.Split('.');
            if (vers.Length > 0)
            {
                int v;
                if (int.TryParse(vers[0], out v) == true) CoreVersionDec = v;
            }
            if (vers.Length > 1)
            {
                int v;
                if (int.TryParse(vers[1], out v) == true) CoreVersionDec += ((float)v / (float)(10.0));
            }

            EnabledInterfaces = secAdmin.GetEnabledInterfaces();
            PowerPackages = secAdmin.GetPowerPackages();
            ActivePowerPackage = secAdmin.GetActivePowerPackage();
            ProvisioningMode = secAdmin.GetProvisioningMode();
            secAdmin.GetCRL(out Crls);
            TrustedFqdnCN = secAdmin.GetTrustedFqdnCN();
            PkiCapabilities = secAdmin.GetPkiCapabilities();
            DigestRealm = secAdmin.GetDigestRealm();
            TLSCertificate = secAdmin.GetTLSCertificate();
            TrustedRootCertificates = secAdmin.GetTrustedRootCertificates();
            secAdmin.GetTlsOptions(out TlsAthenticationLocal, out TlsAthenticationRemote);
            secAdmin.GetKerberosOptions(out KerberosOptions);
            AdminAclEntry = secAdmin.GetAdminAclEntry();

            AmtSecurityAdmin.AmtUserAcl[] UserAclEntries;
            r = secAdmin.GetUserAclEntries(out UserAclEntries);

            secAdmin.GetPowerSavingOptions(out ActiveStateAC, out WakeOnNetAccessThresholdAC, out WakeOnNetAccessSleepTimer);
            RedirectionPort = redirect.GetRedirectionListenerState();
        }
    }
}
