// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.Diagnostics;
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.WSManagement;
using Intel.Manageability.Exceptions;

namespace ManageabilityStack
{
    public class AmtStorage
    {
        private AmtSystem system;
        private StorageAdministrationService storageAdministrationService;
        //private string username;
        //private string password;
        //private bool useTls;
        private Hashtable WrapperTable = new Hashtable();
        private AmtStorageWrapper SingleWrapper = null;

        /* List of all native WSDL functions
        
        storageAdministrationService.AdminGetRegisteredApplications;
        storageAdministrationService.AdminGetApplicationAttributes;
        storageAdministrationService.AdminRemoveApplication;

        storageAdministrationService.GetGlobalStorageAttributes;
        storageAdministrationService.SetGlobalStorageAttributes;

        storageAdministrationService.AddStorageEaclEntry;
        storageAdministrationService.EnumerateStorageEaclEntries
        storageAdministrationService.GetStorageEaclEntry;
        storageAdministrationService.RemoveStorageEaclEntry;

        storageAdministrationService.AddStorageFpaclEntry;
        storageAdministrationService.EnumerateStorageAllocEntries
        storageAdministrationService.GetStorageAllocEntry;
        storageAdministrationService.UpdateStorageFpaclEntry;
        storageAdministrationService.RemoveStorageFpaclEntry;
        */

        #region WSMAN_3PDS_Admin_Methods
        
        private AmtCallStatus WSMAN_GetGlobalStorageAttributes(out uint TotalStorage, out uint TotalAllocatedStorage, out uint MaxPartnerStorage, out uint TotalPartnerAllocatedStorage, out uint MaxNonPartnerStorage, out ushort MaxFpaclEntries, out ushort MaxAslEntries, out ushort MaxEaclEntries, out ushort MaxGroupsPerBlock, out ushort MaxMembersPerGroup, out uint MaxNonPartnerTotalAllocationSize)
        {
            TotalStorage = 0;
            TotalAllocatedStorage = 0;
            MaxPartnerStorage = 0;
            TotalPartnerAllocatedStorage = 0;
            MaxNonPartnerStorage = 0;
            MaxFpaclEntries = 0;
            MaxAslEntries = 0;
            MaxEaclEntries = 0;
            MaxGroupsPerBlock = 0;
            MaxMembersPerGroup = 0;
            MaxNonPartnerTotalAllocationSize = 0;

            if (system == null || system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageAdministrationService amtWsmanService = new AMT_ThirdPartyDataStorageAdministrationService(system.WsMan);
                return (AmtCallStatus)amtWsmanService.GetGlobalStorageAttributes(out TotalStorage, out TotalAllocatedStorage,
                    out MaxPartnerStorage, out TotalPartnerAllocatedStorage, out MaxNonPartnerStorage, out MaxFpaclEntries,
                    out MaxAslEntries, out MaxEaclEntries, out MaxGroupsPerBlock, out MaxMembersPerGroup, out MaxNonPartnerTotalAllocationSize);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_GetGlobalStorageAttributes WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_GetGlobalStorageAttributes CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_GetGlobalStorageAttributes",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }
        
        private AmtCallStatus WSMAN_SetGlobalStorageAttributes(uint MaxPartnerStorage, ushort MaxNonPartnerTotalAllocationSize)
        {
            if (system == null || system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageAdministrationService amtWsmanService = new AMT_ThirdPartyDataStorageAdministrationService(system.WsMan);
                return (AmtCallStatus)amtWsmanService.SetGlobalStorageAttributes(MaxPartnerStorage, MaxNonPartnerTotalAllocationSize);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_SetGlobalStorageAttributes WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_SetGlobalStorageAttributes CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_SetGlobalStorageAttributes",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }
        
        private AmtCallStatus WSMAN_AdminGetRegisteredApplications(out uint[] ApplicationHandles)
        {
            ApplicationHandles = null;

            if (system == null || system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageAdministrationService amtWsmanService = new AMT_ThirdPartyDataStorageAdministrationService(system.WsMan);
                return (AmtCallStatus)amtWsmanService.AdminGetRegisteredApplications(out ApplicationHandles);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_AdminGetRegisteredApplications WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_AdminGetRegisteredApplications CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_AdminGetRegisteredApplications",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }
        
        private AmtCallStatus WSMAN_AdminGetApplicationAttributes(uint Handle, out uint AttrType, out byte[] UUID, out string VendorName, out string ApplicationName, out string EnterpriseName, out uint CurrentAllocationSize, out bool ActiveSession, out bool Partner)
        {
            UUID = null;
            AttrType = 0;
            VendorName = null;
            ApplicationName = null;
            EnterpriseName = null;
            CurrentAllocationSize = 0;
            ActiveSession = false;
            Partner = false;

            if (system == null || system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageAdministrationService amtWsmanService = new AMT_ThirdPartyDataStorageAdministrationService(system.WsMan);
                return (AmtCallStatus)amtWsmanService.AdminGetApplicationAttributes(Handle, out AttrType, out UUID, out VendorName,
                    out ApplicationName, out EnterpriseName, out CurrentAllocationSize, out ActiveSession, out Partner);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_AdminGetApplicationAttributes WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_AdminGetApplicationAttributes CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_AdminGetApplicationAttributes",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }

        private AmtCallStatus WSMAN_AdminRemoveApplication(uint Handle)
        {
            if (system == null || system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageAdministrationService amtWsmanService = new AMT_ThirdPartyDataStorageAdministrationService(system.WsMan);
                return (AmtCallStatus)amtWsmanService.AdminRemoveApplication(Handle);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_AdminRemoveApplication WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_AdminRemoveApplication CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_AdminRemoveApplication",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }

        private AmtCallStatus WSMAN_AddStorageEaclEntry(string EnterpriseName, out uint Handle)
        {
            Handle = 0;

            if (system == null || system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageAdministrationService amtWsmanService = new AMT_ThirdPartyDataStorageAdministrationService(system.WsMan);
                return (AmtCallStatus)amtWsmanService.AddStorageEaclEntry(EnterpriseName, out Handle);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_AddStorageEaclEntry WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_AddStorageEaclEntry CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_AddStorageEaclEntry",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }

        private AmtCallStatus WSMAN_EnumerateStorageEaclEntries(out uint[] Handles)
        {
            Handles = null;

            if (system == null || system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageAdministrationService amtWsmanService = new AMT_ThirdPartyDataStorageAdministrationService(system.WsMan);
                return (AmtCallStatus)amtWsmanService.EnumerateStorageEaclEntries(out Handles);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_EnumerateStorageEaclEntries WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_EnumerateStorageEaclEntries CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_EnumerateStorageEaclEntries",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }

        private AmtCallStatus WSMAN_GetStorageEaclEntry(uint Handle, out string EnterpriseName)
        {
            EnterpriseName = null;

            if (system == null || system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageAdministrationService amtWsmanService = new AMT_ThirdPartyDataStorageAdministrationService(system.WsMan);
                return (AmtCallStatus)amtWsmanService.GetStorageEaclEntry(Handle, out EnterpriseName);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_GetStorageEaclEntry WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_GetStorageEaclEntry CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_GetStorageEaclEntry",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }

        private AmtCallStatus WSMAN_RemoveStorageEaclEntry(uint Handle)
        {
            if (system == null || system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageAdministrationService amtWsmanService = new AMT_ThirdPartyDataStorageAdministrationService(system.WsMan);
                return (AmtCallStatus)amtWsmanService.RemoveStorageEaclEntry(Handle);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_RemoveStorageEaclEntry WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_RemoveStorageEaclEntry CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_RemoveStorageEaclEntry",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }

        private AmtCallStatus WSMAN_AddStorageFpaclEntry(uint AttrType, string ApplicationName, string VendorName, bool IsPartner, uint TotalAllocationSize, out uint Handle)
        {
            Handle = 0;

            if (system == null || system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageAdministrationService amtWsmanService = new AMT_ThirdPartyDataStorageAdministrationService(system.WsMan);
                return (AmtCallStatus)amtWsmanService.AddStorageFpaclEntry(AttrType, ApplicationName, VendorName, IsPartner, TotalAllocationSize, out Handle);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_AddStorageFpaclEntry WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_AddStorageFpaclEntry CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_AddStorageFpaclEntry",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }

        private AmtCallStatus WSMAN_EnumerateStorageAllocEntries(out uint[] Handles)
        {
            Handles = null;

            if (system == null || system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageAdministrationService amtWsmanService = new AMT_ThirdPartyDataStorageAdministrationService(system.WsMan);
                return (AmtCallStatus)amtWsmanService.EnumerateStorageAllocEntries(out Handles);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_EnumerateStorageAllocEntries WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_EnumerateStorageAllocEntries CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_EnumerateStorageAllocEntries",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }

        private AmtCallStatus WSMAN_GetStorageAllocEntry(uint Handle, out uint AttrType, out string ApplicationName, out string VendorName, out bool IsPartner, out uint TotalAllocationSize)
        {
            AttrType = 0;
            ApplicationName = null;
            VendorName = null;
            IsPartner = false;
            TotalAllocationSize = 0;

            if (system == null || system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageAdministrationService amtWsmanService = new AMT_ThirdPartyDataStorageAdministrationService(system.WsMan);
                return (AmtCallStatus)amtWsmanService.GetStorageAllocEntry(Handle, out AttrType, out ApplicationName, out VendorName, out IsPartner, out TotalAllocationSize);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_GetStorageAllocEntry WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_GetStorageAllocEntry CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_GetStorageAllocEntry",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }

        private AmtCallStatus WSMAN_UpdateStorageFpaclEntry(uint Handle, uint NewAllocationSize)
        {
            if (system == null || system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageAdministrationService amtWsmanService = new AMT_ThirdPartyDataStorageAdministrationService(system.WsMan);
                return (AmtCallStatus)amtWsmanService.UpdateStorageFpaclEntry(Handle, NewAllocationSize);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_UpdateStorageFpaclEntry WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_UpdateStorageFpaclEntry CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_UpdateStorageFpaclEntry",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }

        private AmtCallStatus WSMAN_RemoveStorageFpaclEntry(uint Handle)
        {
            if (system == null || system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageAdministrationService amtWsmanService = new AMT_ThirdPartyDataStorageAdministrationService(system.WsMan);
                return (AmtCallStatus)amtWsmanService.RemoveStorageFpaclEntry(Handle);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_RemoveStorageFpaclEntry WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_RemoveStorageFpaclEntry CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorage.WSMAN_RemoveStorageFpaclEntry",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }

        #endregion // WSMAN_3PDS_Admin_Methods

        public AmtStorage(AmtSystem system, StorageAdministrationService storageAdministrationService, StorageService storageService)
        {
            this.system = system;
            this.storageAdministrationService = storageAdministrationService;
        }
        public void Dispose()
        {
            foreach (AmtStorageWrapper w in WrapperTable.Values) w.Dispose();
            storageAdministrationService = null;
        }

        public void ClearCache()
        {
            StorageAttributesCache = null;
            RegisteredApplicationsCache = null;
            EnterpriseEntriesCache = null;
            StorageAllocationsCache = null;
        }

        public void ClearAttributesCache()
        {
            StorageAttributesCache = null;
        }

        public void FetchCache()
        {
            try
            {
                GetStorageAttributes();
                GetRegisteredApplications();
                GetEnterpriseEntries();
                GetStorageAllocations();
            }
            catch (Exception e)
            {
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Error,
                    "Failure to retrieve the Storage data.",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
            }
        }

        private Hashtable AmtApplicationCache = new Hashtable();
        private AmtApplication[] RegisteredApplicationsCache = null;
        public AmtApplication[] GetRegisteredApplications()
        {
            if (RegisteredApplicationsCache != null) return RegisteredApplicationsCache;

            if (system.LimitedFeatures == true)
            {
                if (SingleWrapper == null)
                {
                    ManageabilityStack.AmtStorageWrapper.AmtStorageStatus rs;
                    rs = GetStorageWrapper("Intel", "Intel", "Agent Foundations", new Guid("31313131-3131-3131-3131-313131313131"), out SingleWrapper);
                    if (rs != AmtStorageWrapper.AmtStorageStatus.SUCCESS) return null;
                }
                AmtStorageApplication[] apps2;
                if (SingleWrapper.GetRegisteredApplications(out apps2) == AmtStorageWrapper.AmtStorageStatus.SUCCESS)
                {
                    ArrayList l = new ArrayList();
                    foreach (AmtStorageApplication app in apps2)
                    {
                        StorageApplicationAttributesType appAttributes2 = new StorageApplicationAttributesType();
                        appAttributes2.ApplicationAttr = new StorageSnrpApplicationAttributeType();
                        appAttributes2.AttrType = 0;
                        if (SingleWrapper.GetHandle() == app.Handle)
                        {
                            // 1111111111111111
                            appAttributes2.ApplicationAttr.UUID = new Guid("31313131-3131-3131-3131-313131313131").ToByteArray();
                        }
                        else
                        {
                            // 0000000011111111
                            appAttributes2.ApplicationAttr.UUID = new Guid("00000000-0000-0000-3131-313131313131").ToByteArray();
                        }
                        appAttributes2.ApplicationAttr.VendorName = app.Vendor;
                        appAttributes2.ApplicationAttr.ApplicationName = app.Application;
                        appAttributes2.ApplicationAttr.EnterpriseName = app.Enterprise;
                        appAttributes2.CurrentAllocationSize = 192 * 1024;
                        appAttributes2.ActiveSession = false;
                        appAttributes2.Partner = true;
                        l.Add(new AmtApplication(app.Handle, appAttributes2));
                    }
                    RegisteredApplicationsCache = (AmtApplication[])l.ToArray(typeof(AmtApplication));
                }
                else
                {
                    RegisteredApplicationsCache = new AmtApplication[0];
                }

                return RegisteredApplicationsCache;
            }

            uint totalCount = 0;
            uint handlesCount = 0;
            uint[] applicationHandles;
            AmtCallStatus r;
            AmtApplication[] apps;

            if(system.WsMan != null)
            {
                // USE WSMAN
                r = WSMAN_AdminGetRegisteredApplications(out applicationHandles);
                if(applicationHandles == null)
                {
                    handlesCount = 0;
                }
                else
                {
                    handlesCount = (uint)applicationHandles.Length;
                }
            }
            else
            {    
                // USE EOI
                r = (AmtCallStatus)storageAdministrationService.AdminGetRegisteredApplications(1, out totalCount, out handlesCount, out applicationHandles);
            }
            if (handlesCount == 0) { RegisteredApplicationsCache = new AmtApplication[0]; return null; }

            Hashtable AmtApplicationCache2 = new Hashtable();
            StorageApplicationAttributesType appAttributes;
            apps = new AmtApplication[applicationHandles.Length];
            for (int i = 0; i < applicationHandles.Length; i++)
            {
                AmtApplication c = (AmtApplication)AmtApplicationCache[applicationHandles[i]];
                if (c != null)
                {
                    // Grab this application from the cache
                    apps[i] = c;
                    AmtApplicationCache2.Add(applicationHandles[i],c);
                }
                else
                {
                    // Get the application
                    if(system.WsMan != null)
                    {
                        // USE WSMAN
                        uint attrType = 0;
                        byte[] uuid = null;
                        string vendorName = null;
                        string applicationName = null;
                        string enterpriseName = null;
                        uint currentAllocationSize = 0;
                        bool activeSession = false;
                        bool partner = false;
                        
                        r = WSMAN_AdminGetApplicationAttributes(applicationHandles[i], 
                            out attrType, out uuid, out vendorName, out applicationName, 
                            out enterpriseName, out currentAllocationSize, out activeSession, 
                            out partner);

                        appAttributes = new StorageApplicationAttributesType();
                        appAttributes.ApplicationAttr = new StorageSnrpApplicationAttributeType();
                        
                        appAttributes.AttrType = attrType;
                        appAttributes.ApplicationAttr.UUID = uuid;
                        appAttributes.ApplicationAttr.VendorName = vendorName;
                        appAttributes.ApplicationAttr.ApplicationName = applicationName;
                        appAttributes.ApplicationAttr.EnterpriseName = enterpriseName;
                        appAttributes.CurrentAllocationSize = currentAllocationSize;
                        appAttributes.ActiveSession = activeSession;
                        appAttributes.Partner = partner;
                    }
                    else
                    {
                        // USE EOI
                        r = (AmtCallStatus)storageAdministrationService.AdminGetApplicationAttributes(applicationHandles[i], out appAttributes);
                    }
                    apps[i] = new AmtApplication(applicationHandles[i], appAttributes);
                    AmtApplicationCache2.Add(applicationHandles[i], apps[i]);
                }
            }

            AmtApplicationCache = AmtApplicationCache2;
            RegisteredApplicationsCache = apps;
            return RegisteredApplicationsCache;
        }
        
        public AmtCallStatus RemoveApplication(AmtApplication application)
        {
            if (system.LimitedFeatures == true) return AmtCallStatus.NOT_PERMITTED;

            RegisteredApplicationsCache = null;
            StorageAttributesCache = null;
            if(system.WsMan != null)
            {
                // USE WSMAN
                return WSMAN_AdminRemoveApplication(application.Handle);
            }
            else
            {
                // USE EOI
                return (AmtCallStatus)storageAdministrationService.AdminRemoveApplication(application.Handle);
            }
        }

        public AmtApplication LocateApplication(string enterprise, string vendor, string application)
        {
            AmtApplication[] apps = GetRegisteredApplications();
            if (apps == null) return null;
            foreach (AmtApplication app in apps)
            {
                if (app.EnterpriseName.CompareTo(enterprise) == 0 && app.VendorName.CompareTo(vendor) == 0 && app.ApplicationName.CompareTo(application) == 0) return app;
            }
            return null;
        }

        public AmtCallStatus SetStorageAttributes(uint MaxPartnerStorage, ushort MaxNonPartnerTotalAllocationSize)
        {
            if (system.LimitedFeatures == true) return AmtCallStatus.NOT_PERMITTED;

            StorageAttributesCache = null;
            if(system.WsMan != null)
            {
                // USE WSMAN
                return WSMAN_SetGlobalStorageAttributes(MaxPartnerStorage, MaxNonPartnerTotalAllocationSize);
            }
            else
            {
                // USE EOI
                return (AmtCallStatus)storageAdministrationService.SetGlobalStorageAttributes(MaxPartnerStorage, MaxPartnerStorage != 0, MaxNonPartnerTotalAllocationSize, MaxNonPartnerTotalAllocationSize != 0);
            }
        }

        private GlobalStorageAttributesType StorageAttributesCache = null;
        public GlobalStorageAttributesType GetStorageAttributes()
        {
            if (StorageAttributesCache != null) return StorageAttributesCache;

            GlobalStorageAttributesType attribs;
            if (system.LimitedFeatures == true)
            {
                attribs = new GlobalStorageAttributesType();
                attribs.TotalStorage = 192 * 1024;
                attribs.TotalAllocatedStorage = 0;
                attribs.MaxPartnerStorage = 192 * 1024;
                attribs.TotalPartnerAllocatedStorage = 192 * 1024;
                attribs.MaxNonPartnerStorage = 0;
                attribs.MaxFpaclEntries = 1;
                attribs.MaxAslEntries = 1;
                attribs.MaxEaclEntries = 1;
                attribs.MaxGroupsPerBlock = 2;
                attribs.MaxMembersPerGroup = 2;
                attribs.MaxNonPartnerTotalAllocationSize = 0;
                StorageAttributesCache = attribs;
                return StorageAttributesCache;
            }

            if(system.WsMan != null)
            {
                // USE WSMAN
                uint totalStorage = 0;
                uint totalAllocatedStorage = 0;
                uint maxPartnerStorage = 0;
                uint totalPartnerAllocatedStorage = 0;
                uint maxNonPartnerStorage = 0;
                ushort maxFpaclEntries = 0;
                ushort maxAslEntries = 0;
                ushort maxEaclEntries = 0;
                ushort maxGroupsPerBlock = 0;
                ushort maxMembersPerGroup = 0;
                uint maxNonPartnerTotalAllocationSize = 0;
                
                AmtCallStatus r = WSMAN_GetGlobalStorageAttributes(out totalStorage, 
                    out totalAllocatedStorage, out maxPartnerStorage, out totalPartnerAllocatedStorage, 
                    out maxNonPartnerStorage, out maxFpaclEntries, out maxAslEntries, out maxEaclEntries, 
                    out maxGroupsPerBlock, out maxMembersPerGroup, out maxNonPartnerTotalAllocationSize);
            
                attribs = new GlobalStorageAttributesType();
                attribs.TotalStorage = totalStorage;
                attribs.TotalAllocatedStorage = totalAllocatedStorage;
                attribs.MaxPartnerStorage = maxPartnerStorage;
                attribs.TotalPartnerAllocatedStorage = totalPartnerAllocatedStorage;
                attribs.MaxNonPartnerStorage = maxNonPartnerStorage;
                attribs.MaxFpaclEntries = maxFpaclEntries;
                attribs.MaxAslEntries = maxAslEntries;
                attribs.MaxEaclEntries = maxEaclEntries;
                attribs.MaxGroupsPerBlock = maxGroupsPerBlock;
                attribs.MaxMembersPerGroup = maxMembersPerGroup;
                attribs.MaxNonPartnerTotalAllocationSize = maxNonPartnerTotalAllocationSize;
            }
            else
            {
                // USE EOI
                AmtCallStatus r = (AmtCallStatus)storageAdministrationService.GetGlobalStorageAttributes(out attribs);
            }
            StorageAttributesCache = attribs;
            return StorageAttributesCache;
        }

        public AmtEnterprise AddEnterpriseEntry(string EnterpriseName)
        {
            if (system.LimitedFeatures == true) return null;

            EnterpriseEntriesCache = null;
            StorageEaclEntryType entry = new StorageEaclEntryType();
            entry.EnterpriseName = EnterpriseName;
            uint handle;
            if(system.WsMan != null)
            {
                // USE WSMAN
                AmtCallStatus r = WSMAN_AddStorageEaclEntry(EnterpriseName, out handle);
            }
            else
            {
                // USE EOI
                AmtCallStatus r = (AmtCallStatus)storageAdministrationService.AddStorageEaclEntry(entry, out handle);
            }
            AmtEnterprise enterprise = new AmtEnterprise();
            enterprise.Handle = handle;
            enterprise.Name = EnterpriseName;
            return enterprise;
        }
        public AmtCallStatus RemoveEnterpriseEntry(AmtEnterprise enterprise)
        {
            if (system.LimitedFeatures == true) return AmtCallStatus.NOT_PERMITTED;

            EnterpriseEntriesCache = null;
            if(system.WsMan != null)
            {
                // USE WSMAN
                return WSMAN_RemoveStorageEaclEntry(enterprise.Handle);
            }
            else
            {
                // USE EOI
                return (AmtCallStatus)storageAdministrationService.RemoveStorageEaclEntry(enterprise.Handle);
            }
        }

        private Hashtable enterpriseCache = new Hashtable();
        private AmtEnterprise[] EnterpriseEntriesCache = null;
        public AmtEnterprise[] GetEnterpriseEntries()
        {
            if (EnterpriseEntriesCache != null) return EnterpriseEntriesCache;

            if (system.LimitedFeatures == true)
            {
                AmtEnterprise e = new AmtEnterprise();
                e.Name = "Intel";
                e.Handle = 0;
                EnterpriseEntriesCache = new AmtEnterprise[1];
                EnterpriseEntriesCache[0] = e;
                return EnterpriseEntriesCache;
            }

            ushort totalCount = 0;
            ushort handlesCount = 0;
            uint[] handles = null;
            AmtEnterprise[] enterprises;
            AmtCallStatus r;

            if(system.WsMan != null)
            {
                // USE WSMAN
                r = WSMAN_EnumerateStorageEaclEntries(out handles);
                if(handles == null)
                {
                    handlesCount = 0;
                }
                else
                {
                    handlesCount = (ushort)handles.Length;
                }
            }
            else
            {
                // USE EOI
                r = (AmtCallStatus)storageAdministrationService.EnumerateStorageEaclEntries(1, out totalCount, out handlesCount, out handles);
            }
            if (handlesCount == 0)
            {
                EnterpriseEntriesCache = new AmtEnterprise[0];
                return EnterpriseEntriesCache;
            }
            enterprises = new AmtEnterprise[handlesCount]; // TODO : Handle total != handle count

            Hashtable enterpriseCache2 = new Hashtable();
            int i = 0;
            foreach (uint handle in handles)
            {
                AmtEnterprise c = (AmtEnterprise)enterpriseCache[handle];
                if (c != null)
                {
                    // Get from the cache
                    enterprises[i] = c;
                    enterpriseCache2.Add(handle, c);
                }
                else
                {
                    // Get the enterprise
                    StorageEaclEntryType entry;
                    
                    if(system.WsMan != null)
                    {
                        // USE WSMAN
                        string enterpriseName = null;
                        r = WSMAN_GetStorageEaclEntry(handle, out enterpriseName);
                        
                        entry = new StorageEaclEntryType();
                        entry.EnterpriseName = enterpriseName;
                    }
                    else
                    {
                        // USE EOI
                        r = (AmtCallStatus)storageAdministrationService.GetStorageEaclEntry(handle, out entry);
                    }
                    enterprises[i] = new AmtEnterprise();
                    enterprises[i].Name = entry.EnterpriseName;
                    enterprises[i].Handle = handle;
                    enterpriseCache2.Add(handle, enterprises[i]);
                }
                i++;
            }

            enterpriseCache = enterpriseCache2;
            EnterpriseEntriesCache = enterprises;
            return EnterpriseEntriesCache;
        }

        private Hashtable storageAllocationsCache = new Hashtable();
        private AmtStorageAlloc[] StorageAllocationsCache = null;
        public AmtStorageAlloc[] GetStorageAllocations()
        {
            if (StorageAllocationsCache != null) return StorageAllocationsCache;

            if (system.LimitedFeatures == true)
            {
                StorageAllocationsCache = new AmtStorageAlloc[1];
                StorageAllocationsCache[0] = new AmtStorageAlloc();
                StorageAllocationsCache[0].Attribute = 0;
                StorageAllocationsCache[0].SnrpEntryApplicationName = "Agent Foundations";
                StorageAllocationsCache[0].SnrpEntryVendorName = "Intel";
                StorageAllocationsCache[0].TotalAllocationSize = 192 * 1024;
                StorageAllocationsCache[0].IsPartner = true;
                return StorageAllocationsCache;
            }

            ushort totalCount = 0;
            ushort handlesCount = 0;
            uint[] handles;
            AmtCallStatus r;
            
            if(system.WsMan != null)
            {
                // USE WSMAN
                r = WSMAN_EnumerateStorageAllocEntries(out handles);
                if(handles == null)
                {
                    handlesCount = 0;
                }
                else
                {
                    handlesCount = (ushort)handles.Length;
                }
            }
            else
            {
                // USE EOI 
                r = (AmtCallStatus)storageAdministrationService.EnumerateStorageAllocEntries(1, out totalCount, out handlesCount, out handles);
            }
            
            if (handlesCount == 0)
            {
                StorageAllocationsCache = new AmtStorageAlloc[0];
                return StorageAllocationsCache;
            }
            AmtStorageAlloc[] storagelist = new AmtStorageAlloc[handlesCount]; // TODO

            Hashtable storageAllocationsCache2 = new Hashtable();
            int i = 0;
            foreach (uint handle in handles)
            {
                AmtStorageAlloc c = (AmtStorageAlloc)storageAllocationsCache[handle];

                if (c != null)
                {
                    storagelist[i] = c;
                    storageAllocationsCache2[handle] = c;
                }
                else
                {
                    StorageAllocEntryType entry;
                    if(system.WsMan != null)
                    {
                        // USE WSMAN
                        uint attrType = 0;
                        string applicationName = null;
                        string vendorName = null;
                        bool isPartner = false;
                        uint totalAllocationSize = 0;
                        
                        r = WSMAN_GetStorageAllocEntry(handle, out attrType, out applicationName, 
                            out vendorName, out isPartner, out totalAllocationSize);
                    
                        entry = new StorageAllocEntryType();
                        entry.StorageAllocSnrpEntry = new StorageAllocSnrpEntryType();
                        entry.AttrType = attrType;
                        entry.StorageAllocSnrpEntry.ApplicationName = applicationName;
                        entry.StorageAllocSnrpEntry.VendorName = vendorName;
                        entry.IsPartner = isPartner;
                        entry.TotalAllocationSize = totalAllocationSize;
                    }
                    else
                    {
                        // USE EOI
                        r = (AmtCallStatus)storageAdministrationService.GetStorageAllocEntry(handle, out entry);
                    }
                    storagelist[i] = new AmtStorageAlloc(handle, entry);
                    storageAllocationsCache2[handle] = storagelist[i];
                }
                i++;
            }

            storageAllocationsCache = storageAllocationsCache2;
            StorageAllocationsCache = storagelist;
            return StorageAllocationsCache;
        }

        public AmtStorageAlloc AddStorageFpaclEntry(string VendorName, string AppName, uint AllocSize)
        {
            if (system.LimitedFeatures == true) return null;

            StorageAllocationsCache = null;
            AmtStorageAlloc entry = new AmtStorageAlloc();
            entry.IsPartner = true;
            entry.SnrpEntryVendorName = VendorName;
            entry.SnrpEntryApplicationName = AppName;
            entry.TotalAllocationSize = AllocSize;
            entry.Attribute = 1;
            uint handle;
            if(system.WsMan != null)
            {
                // USE WSMAN
                AmtCallStatus r = WSMAN_AddStorageFpaclEntry(entry.Attribute, 
                    entry.SnrpEntryApplicationName, entry.SnrpEntryVendorName, entry.IsPartner, 
                    entry.TotalAllocationSize, out handle);
            }
            else
            {
                // USE EOI
                AmtCallStatus r = (AmtCallStatus)storageAdministrationService.AddStorageFpaclEntry(entry.storageEntryType, out handle);
            }
            entry.Handle = handle;
            return entry;
        }

        public AmtStorageAlloc UpdateStorageFpaclEntry(AmtStorageAlloc entry, uint newSize)
        {
            if (system.LimitedFeatures == true) return null;

            StorageAllocationsCache = null;
            if(system.WsMan != null)
            {
                // USE WSMAN
                AmtCallStatus r = WSMAN_UpdateStorageFpaclEntry(entry.Handle, newSize);
            }
            else
            {
                // USE EOI
                AmtCallStatus r = (AmtCallStatus)storageAdministrationService.UpdateStorageFpaclEntry(entry.Handle, newSize);
            }
            entry.TotalAllocationSize = newSize;
            storageAllocationsCache[entry.Handle] = entry;
            return entry;
        }

        public AmtCallStatus RemoveStorageFpaclEntry(AmtStorageAlloc entry)
        {
            if (system.LimitedFeatures == true) return AmtCallStatus.NOT_PERMITTED;

            StorageAllocationsCache = null;
            if(system.WsMan != null)
            {
                // USE WSMAN
                return WSMAN_RemoveStorageFpaclEntry(entry.Handle);
            }
            else
            {
                // USE EOI
                return (AmtCallStatus)storageAdministrationService.RemoveStorageFpaclEntry(entry.Handle);
            }
        }

        private AmtStorageWrapper GetStorageWrapper(AmtApplication app)
        {
            AmtStorageWrapper wrapper = (AmtStorageWrapper)WrapperTable[app.Handle];
            if (wrapper != null) return wrapper;

            string targetUrl = storageAdministrationService.Url.Replace("/StorageAdministrationService", "/StorageService");
            if (system.LimitedFeatures == true)
            {
                // This uses an emulated StorageAdmin
                wrapper = new AmtStorageWrapper(system, app.EnterpriseName, app.VendorName, app.ApplicationName, app.UUID);
            }
            else
            {
                // This is the real storage admin, we will log in as console
                wrapper = new AmtStorageWrapper(system, app.EnterpriseName, app.VendorName, app.ApplicationName, Guid.Empty);
            }
            
            ManageabilityStack.AmtStorageWrapper.AmtStorageStatus r = wrapper.Connect();
            if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
            {
                ManageabilityStack.AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Event, "Failed to connect to 3PDS: " + r.ToString());
                return null;
            }

            WrapperTable.Add(app.Handle, wrapper);
            return wrapper;
        }

        private ManageabilityStack.AmtStorageWrapper.AmtStorageStatus GetStorageWrapper(string enterprise, string vendor, string application, Guid UUID, out AmtStorageWrapper wrapper)
        {
            // First check to see if this wrapper already exists
            foreach (AmtStorageWrapper w in WrapperTable.Values)
            {
                if (w.ApplicationGuid.Equals(UUID) && w.AppName.Equals(application) && w.VendorName.Equals(vendor) && w.EnterpriseName.Equals(enterprise))
                {
                    wrapper = w;
                    return AmtStorageWrapper.AmtStorageStatus.SUCCESS;
                }
            }

            string targetUrl = storageAdministrationService.Url.Replace("/StorageAdministrationService", "/StorageService");
            wrapper = new AmtStorageWrapper(system, enterprise, vendor, application, UUID);

            ManageabilityStack.AmtStorageWrapper.AmtStorageStatus r = wrapper.Connect();
            if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
            {
                ManageabilityStack.AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Event, "Failed to connect to 3PDS: " + r.ToString());
                return r;
            }

            WrapperTable.Add(wrapper.SessionHandle, wrapper);
            return r;
        }

        public AmtStorageBlock[] GetStorageBlocks(AmtApplication app)
        {
            AmtStorageWrapper wrapper;
            if (SingleWrapper != null) wrapper = SingleWrapper; else wrapper = GetStorageWrapper(app);
            if (wrapper == null) return null;
            //if (system.CastlePeak == true) app.Handle = wrapper.GetHandle();
            return wrapper.GetBlockList(app.Handle);
        }

        public AmtStorageWrapper.AmtStorageStatus AllocateBlock(AmtApplication app, string name, uint size, out AmtStorageBlock block)
        {
            block = null;
            AmtStorageWrapper wrapper = GetStorageWrapper(app);
            if (wrapper == null) return AmtStorageWrapper.AmtStorageStatus.INTERNAL_ERROR;
            RegisteredApplicationsCache = null;
            StorageAttributesCache = null;
            return wrapper.AllocateBlock(name, size, out block);
        }

        public AmtStorageWrapper.AmtStorageStatus AllocateBlock(string enterprise, string vendor, string application, Guid UUID, string name, uint size, out AmtStorageBlock block)
        {
            AmtStorageWrapper.AmtStorageStatus r;
            AmtApplication app = LocateApplication(enterprise, vendor, application);
            block = null;
            if (app != null) return AllocateBlock(app, name, size, out block);
            AmtStorageWrapper wrapper;
            ManageabilityStack.AmtStorageWrapper.AmtStorageStatus rs = GetStorageWrapper(enterprise, vendor, application, UUID, out wrapper);
            if (rs != AmtStorageWrapper.AmtStorageStatus.SUCCESS) return rs;
            RegisteredApplicationsCache = null;
            StorageAttributesCache = null;
            r = wrapper.AllocateBlock(name, size, out block);
            return r;
        }

    }

    public class AmtEnterprise
    {
        public string Name;
        public uint Handle;

        public override string ToString() {return Name;}
    }

    public class AmtApplication
    {
        private StorageApplicationAttributesType appAttributes;
        private Guid uuid;
        public uint Handle;
        public object Tag;

        public string EnterpriseName { get { return appAttributes.ApplicationAttr.EnterpriseName; } }
        public string VendorName { get { return appAttributes.ApplicationAttr.VendorName; } }
        public string ApplicationName { get { return appAttributes.ApplicationAttr.ApplicationName; } }
        public Guid UUID { get { return uuid; } }

        public bool PartnerApplication { get { return appAttributes.Partner; } }
        public uint AttributeType { get { return appAttributes.AttrType; } }
        public bool ActiveSession { get { return appAttributes.ActiveSession; } }

        public AmtApplication(uint handle, StorageApplicationAttributesType appAttributes)
        {
            this.Handle = handle;
            this.appAttributes = appAttributes;
            uuid = new Guid(appAttributes.ApplicationAttr.UUID);
        }

        public override string ToString() { return ApplicationName; }
    }

    public class AmtStorageAlloc
    {
        public StorageAllocEntryType storageEntryType;
        public uint Handle;

        public bool IsPartner { get { return storageEntryType.IsPartner; } set { storageEntryType.IsPartner = value; } }
        public string SnrpEntryApplicationName { get { return storageEntryType.StorageAllocSnrpEntry.ApplicationName; } set { storageEntryType.StorageAllocSnrpEntry.ApplicationName = value; }}
        public string SnrpEntryVendorName { get { return storageEntryType.StorageAllocSnrpEntry.VendorName; } set { storageEntryType.StorageAllocSnrpEntry.VendorName = value; }}
        public uint TotalAllocationSize { get { return storageEntryType.TotalAllocationSize; } set { storageEntryType.TotalAllocationSize = value; }}
        public uint Attribute { get { return storageEntryType.AttrType; } set { storageEntryType.AttrType = value; } }

        public AmtStorageAlloc()
        {
            storageEntryType = new StorageAllocEntryType();
            storageEntryType.StorageAllocSnrpEntry = new StorageAllocSnrpEntryType();
        }

        public AmtStorageAlloc(uint handle, StorageAllocEntryType storageEntryType)
        {
            this.Handle = handle;
            this.storageEntryType = storageEntryType;
        }

        public override string ToString() { return SnrpEntryApplicationName; }
    }

}
