// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Text;
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.Exceptions;
using Intel.Manageability.WSManagement;

namespace ManageabilityStack
{
    /// <summary>
    /// Summary description for AmtStorageWrapper.
    /// </summary>
    public class AmtStorageWrapper
    {
        #region DLL Imports
        [DllImport("StorageWrapper.dll")]
        static extern int Storage_Test();
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_Initialize(
                    ref UIntPtr LibMajorVersion,
                    ref UIntPtr LibMinorVersion,
                    out UIntPtr LibBuildNumber
                    );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_Uninitialize();
        [DllImport("StorageWrapper.dll")]
        public static extern AmtStorageStatus Storage_InitializeCOMinThread();
        [DllImport("StorageWrapper.dll")]
        public static extern AmtStorageStatus Storage_UninitializeCOMinThread();
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_RegisterApplicationEx(
            out IntPtr SessionHandle,
            [MarshalAs(UnmanagedType.LPWStr)] string Username,
            [MarshalAs(UnmanagedType.LPWStr)] string Password,
            [MarshalAs(UnmanagedType.LPStr)] string TargetUrl,
            IntPtr MachineUUID,
            IntPtr VendorName,
            IntPtr AppName,
            IntPtr EnterpriseName,
            IntPtr AuthInfo
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_GetRegisteredApplications(
            IntPtr SessionHandle,
            UInt32 StartIndex,
            out UIntPtr TotalRegisteredAppCount,
            ref uint AppHandleCount,
            IntPtr AppHandles       // [<AppHandleCount>]
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_GetCurrentApplicationHandle(
            IntPtr SessionHandle,
            out IntPtr AppHandle
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_GetApplicationAttributes(
            IntPtr SessionHandle,
            uint ApplicationBeingRequested,
            IntPtr ApplicationAttributes
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_UnregisterApplication(
            IntPtr SessionHandle
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_RemoveApplication(
            IntPtr SessionHandle
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_GetBytesAvailable(
            IntPtr SessionHandle,
            out IntPtr BytesAvailable
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_AllocateBlock(
            IntPtr SessionHandle,
            UInt32 BytesRequested,
            Boolean BlockHidden,
            IntPtr BlockName,
            out IntPtr BlockHandle
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_GetAllocatedBlocks(
            IntPtr SessionHandle,
            UInt32 BlockOwnerApplication,
            UInt32 StartIndex,
            out UIntPtr TotalAllocatedBlockCount,
            ref UIntPtr BlockHandleCount,
            IntPtr BlockHandles      // [<BlockHandleCount>]
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_GetBlockAttributes(
            IntPtr SessionHandle,
            UInt32 BlockHandle,
            ref IntPtr BlockAttributes
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_DeallocateBlock(
            IntPtr SessionHandle,
            UInt32 BlockHandle
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_WriteBlock(
            IntPtr SessionHandle,
            UInt32 BlockHandle,
            UInt32 ByteOffset,
            UInt32 ByteCount,
            out UIntPtr BytesWritten,
            byte[] Data
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_ReadBlock(
            IntPtr SessionHandle,
            UInt32 BlockHandle,
            UInt32 ByteOffset,
            UInt32 ByteCount,
            out UIntPtr BytesRead,
            IntPtr Data
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_LockBlock(
            IntPtr SessionHandle,
            UInt32 BlockHandle
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_UnlockBlock(
            IntPtr SessionHandle,
            UInt32 BlockHandle
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_SetBlockName(
            IntPtr SessionHandle,
            UInt32 BlockHandle,
            [MarshalAs(UnmanagedType.LPStr)] string BlockName
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_SetBlockVisibility(
            IntPtr SessionHandle,
            UInt32 BlockHandle,
            Boolean BlockHidden
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_AddPermissionsGroup(
            IntPtr SessionHandle,
            UInt32 BlockHandle,
            UInt32 GroupPermissions,
            IntPtr GroupName,
            out IntPtr GroupHandle
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_GetPermissionsGroups(
            IntPtr SessionHandle,
            UInt32 BlockHandle,
            UInt32 StartIndex,
            out IntPtr TotalGroupCount,
            ref IntPtr GroupHandleCount,
            ref	IntPtr GroupHandles  // [<HandleCount>]
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_GetPermissionsGroupAttributes(
            IntPtr SessionHandle,
            UInt32 BlockHandle,
            UInt32 GroupHandle,
            ref IntPtr GroupAttributes
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_RemovePermissionsGroup(
            IntPtr SessionHandle,
            UInt32 BlockHandle,
            UInt32 GroupHandle
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_SetPermissionsGroupName(
            IntPtr SessionHandle,
            UInt32 BlockHandle,
            UInt32 GroupHandle,
            [MarshalAs(UnmanagedType.LPStr)] string GroupName
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_SetPermissionsGroupPermissions(
            IntPtr SessionHandle,
            UInt32 BlockHandle,
            UInt32 GroupHandle,
            IntPtr Permissions
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_AddPermissionsGroupMembers(
            IntPtr SessionHandle,
            UInt32 BlockHandle,
            UInt32 GroupHandle,
            UInt32 MemberHandleCount,
            UInt32[] MemberHandles      // [<MemberHandleCount>]
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_GetPermissionsGroupMembers(
            IntPtr SessionHandle,
            UInt32 BlockHandle,
            UInt32 GroupHandle,
            UInt32 StartIndex,
            out UIntPtr TotalMemberCount,
            ref uint MemberHandleCount,
            IntPtr MemberHandles      // [<MemberHandleCount>]
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_RemovePermissionsGroupMembers(
            IntPtr SessionHandle,
            UInt32 BlockHandle,
            UInt32 GroupHandle,
            UInt32 MemberHandleCount,
            UInt32[] MemberHandles      // [<MemberHandleCount>]
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_GetTimeoutValues(
            IntPtr SessionHandle,
            out UIntPtr RegistrationTimeout,
            out UIntPtr LockTimeout
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_GetHostUUID(
            out IntPtr Uuid
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_GetAPIVersion(
            [MarshalAs(UnmanagedType.LPWStr)] string Username,
            [MarshalAs(UnmanagedType.LPWStr)] string Password,
            string TargetUrl,
            out IntPtr Version
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_GetAPIVersionEx(
            IntPtr Version,
            [MarshalAs(UnmanagedType.LPWStr)] string Username,
            [MarshalAs(UnmanagedType.LPWStr)] string Password,
            [MarshalAs(UnmanagedType.LPStr)] string TargetUrl,
            IntPtr AuthInfo
            );
        [DllImport("StorageWrapper.dll")]
        static extern AmtStorageStatus Storage_GetBlockWriteEraseLimit(
            IntPtr SessionHandle,
            UInt32 BlockHandle,
            out UIntPtr WriteEraseLimit
            );
        [DllImport("StorageWrapper.dll")]
        static extern void Storage_GetLastNetworkError(
            IntPtr SessionHandle,
            out UIntPtr NetworkError
            );
        #endregion
        
        #region WSMAN_3PDS_Methods

        private AmtStorageStatus WSMAN_GetMTU(out uint Mtu)
        {
            Mtu = 0;

            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.GetMTU(out Mtu);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetMTU WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetMTU CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetMTU",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        /// <summary>
        /// Convert a hex array to a string value
        /// </summary>
        /// <param name="inByteArray">a byte array to convert</param>
        /// <returns>the input byte array converted to a string</returns>
        private static string ConvertByteArrayToHexString(byte[] inByteArray)
        {
            if (inByteArray == null)
                throw new ArgumentNullException("inByteArray");

            var returnString = string.Empty;

            foreach (var curByte in inByteArray)
            {
                if (curByte == 0)
                {
                    returnString += "00";
                }
                else if (curByte < 16)
                {
                    returnString = returnString + string.Format("0{0:x}", curByte);
                }
                else
                {
                    returnString = returnString + string.Format("{0:x}", curByte);
                }
            }

            return returnString;
        }

        private AmtStorageStatus WSMAN_RegisterApplication(byte[][] CallerUUID, string VendorName, string ApplicationName, string EnterpriseName, out string SessionHandle)
        {
            SessionHandle = null;

            string UUID = ConvertByteArrayToHexString(CallerUUID[0]);

            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.RegisterApplication(UUID, VendorName, ApplicationName, EnterpriseName, out SessionHandle);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_RegisterApplication WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_RegisterApplication CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_RegisterApplication",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }
        
        private AmtStorageStatus WSMAN_UnregisterApplication(string SessionHandle)
        {
            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.UnregisterApplication(SessionHandle);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_UnregisterApplication WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_UnregisterApplication CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_UnregisterApplication",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }
        
        private AmtStorageStatus WSMAN_GetRegisteredApplications(string SessionHandle, out uint[] AppHandles)
        {
            AppHandles = null;
            
            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.GetRegisteredApplications(SessionHandle, out AppHandles);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetRegisteredApplications WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetRegisteredApplications CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetRegisteredApplications",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }
        
        private AmtStorageStatus WSMAN_GetCurrentApplicationHandle(string SessionHandle, out uint ApplicationHandle)
        {
            ApplicationHandle = 0;

            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.GetCurrentApplicationHandle(SessionHandle, out ApplicationHandle);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetCurrentApplicationHandle WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetCurrentApplicationHandle CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetCurrentApplicationHandle",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_GetApplicationAttributes(string SessionHandle, uint ApplicationBeingRequested, out string VendorName, out string ApplicationName)
        {
            VendorName = null;
            ApplicationName = null;

            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.GetApplicationAttributes(SessionHandle, ApplicationBeingRequested, out VendorName, out ApplicationName);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetApplicationAttributes WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetApplicationAttributes CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetApplicationAttributes",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_RemoveApplication(string SessionHandle)
        {
            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.RemoveApplication(SessionHandle);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_RemoveApplication WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_RemoveApplication CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_RemoveApplication",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_GetBytesAvailable(string SessionHandle, out uint BytesAvailable)
        {
            BytesAvailable = 0;

            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.GetBytesAvailable(SessionHandle, out BytesAvailable);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetBytesAvailable WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetBytesAvailable CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetBytesAvailable",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_AllocateBlock(string SessionHandle, uint BytesRequested, bool BlockHidden, string BlockName, out uint BlockHandle)
        {
            BlockHandle = 0;

            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.AllocateBlock(SessionHandle, BytesRequested, BlockHidden, BlockName, out BlockHandle);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_AllocateBlock WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_AllocateBlock CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_AllocateBlock",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_DeallocateBlock(string SessionHandle, uint BlockHandle)
        {
            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.DeallocateBlock(SessionHandle, BlockHandle);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_DeallocateBlock WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_DeallocateBlock CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_DeallocateBlock",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_RemovePermissionsGroup(string SessionHandle, uint BlockHandle, uint GroupHandle)
        {
            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.RemovePermissionsGroup(SessionHandle, BlockHandle, GroupHandle);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_RemovePermissionsGroup WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_RemovePermissionsGroup CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_RemovePermissionsGroup",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_GetPermissionsGroupMembers(string SessionHandle, uint BlockHandle, uint GroupHandle, out uint[] MemberHandles)
        {
            MemberHandles = null;
            
            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.GetPermissionsGroupMembers(SessionHandle, BlockHandle, GroupHandle, out MemberHandles);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetPermissionsGroupMembers WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetPermissionsGroupMembers CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetPermissionsGroupMembers",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_GetAllocatedBlocks(string SessionHandle, uint BlockOwnerApplication, out uint[] BlockHandles)
        {
            BlockHandles = null;

            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.GetAllocatedBlocks(SessionHandle, BlockOwnerApplication, out BlockHandles);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetAllocatedBlocks WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetAllocatedBlocks CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetAllocatedBlocks",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_RemovePermissionsGroupMembers(string SessionHandle, uint BlockHandle, uint GroupHandle, uint[] MemberHandles)
        {
            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.RemovePermissionsGroupMembers(SessionHandle, BlockHandle, GroupHandle, MemberHandles);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_RemovePermissionsGroupMembers WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_RemovePermissionsGroupMembers CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_RemovePermissionsGroupMembers",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_GetBlockAttributes(string SessionHandle, uint BlockHandle, out uint BlockSize, out bool BlockHidden, out string BlockName)
        {
            BlockSize = 0;
            BlockHidden = false;
            BlockName = null;

            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.GetBlockAttributes(SessionHandle, BlockHandle, out BlockSize, out BlockHidden, out BlockName);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetBlockAttributes WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetBlockAttributes CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetBlockAttributes",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_AddPermissionsGroupMembers(string SessionHandle, uint BlockHandle, uint GroupHandle, uint[] MemberHandles)
        {
            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.AddPermissionsGroupMembers(SessionHandle, BlockHandle, GroupHandle, MemberHandles);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_AddPermissionsGroupMembers WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_AddPermissionsGroupMembers CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_AddPermissionsGroupMembers",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_UnlockBlock(string SessionHandle, uint BlockHandle)
        {
            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.UnlockBlock(SessionHandle, BlockHandle);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_UnlockBlock WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_UnlockBlock CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_UnlockBlock",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_SetPermissionsGroupName(string SessionHandle, uint BlockHandle, uint GroupHandle, string GroupName)
        {
            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.SetPermissionsGroupName(SessionHandle, BlockHandle, GroupHandle, GroupName);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_SetPermissionsGroupName WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_SetPermissionsGroupName CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_SetPermissionsGroupName",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_GetTimeoutValues(out uint RegistrationTimeout, out uint LockTimeout)
        {
            RegistrationTimeout = 0;
            LockTimeout = 0;

            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.GetTimeoutValues(out RegistrationTimeout, out LockTimeout);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetTimeoutValues WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetTimeoutValues CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetTimeoutValues",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_ReadBlock(string SessionHandle, uint BlockHandle, uint ByteOffset, uint ByteCount, out byte[] Data)
        {
            Data = null;

            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.ReadBlock(SessionHandle, BlockHandle, ByteOffset, ByteCount, out Data);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_ReadBlock WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_ReadBlock CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_ReadBlock",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_WriteBlock(string SessionHandle, uint BlockHandle, uint ByteOffset, byte[] Data, out uint BytesWritten)
        {
            BytesWritten = 0;

            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.WriteBlock(SessionHandle, BlockHandle, ByteOffset, Data, out BytesWritten);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_WriteBlock WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_WriteBlock CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_WriteBlock",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_SetPermissionsGroupPermissions(string SessionHandle, uint BlockHandle, uint GroupHandle, uint Permissions)
        {
            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.SetPermissionsGroupPermissions(SessionHandle, BlockHandle, GroupHandle, Permissions);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_SetPermissionsGroupPermissions WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_SetPermissionsGroupPermissions CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_SetPermissionsGroupPermissions",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_SetBlockVisibility(string SessionHandle, uint BlockHandle, bool BlockHidden)
        {
            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.SetBlockVisibility(SessionHandle, BlockHandle, BlockHidden);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_SetBlockVisibility WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_SetBlockVisibility CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_SetBlockVisibility",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_GetPermissionsGroupAttributes(string SessionHandle, uint BlockHandle, uint GroupHandle, out string Name, out uint Permissions)
        {
            Name = null;
            Permissions = 0;

            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.GetPermissionsGroupAttributes(SessionHandle, BlockHandle, GroupHandle, out Name, out Permissions);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetPermissionsGroupAttributes WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetPermissionsGroupAttributes CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetPermissionsGroupAttributes",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_GetPermissionsGroups(string SessionHandle, uint BlockHandle, out uint[] GroupHandles)
        {
            GroupHandles = null;

            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.GetPermissionsGroups(SessionHandle, BlockHandle, out GroupHandles);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetPermissionsGroups WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetPermissionsGroups CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetPermissionsGroups",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_SetBlockName(string SessionHandle, uint BlockHandle, string BlockName)
        {
            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.SetBlockName(SessionHandle, BlockHandle, BlockName);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_SetBlockName WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_SetBlockName CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_SetBlockName",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_AddPermissionsGroup(string SessionHandle, uint BlockHandle, uint GroupPermissions, string GroupName, out uint GroupHandle)
        {
            GroupHandle = 0;

            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.AddPermissionsGroup(SessionHandle, BlockHandle, GroupPermissions, GroupName, out GroupHandle);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_AddPermissionsGroup WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_AddPermissionsGroup CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_AddPermissionsGroup",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_LockBlock(string SessionHandle, uint BlockHandle)
        {
            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.LockBlock(SessionHandle, BlockHandle);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_LockBlock WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_LockBlock CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_LockBlock",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        private AmtStorageStatus WSMAN_GetWriteEraseLimit(out uint WriteEraseLimit)
        {
            WriteEraseLimit = 0;
            
            if (WsMan == null) return AmtStorageStatus.NO_WSMAN;

            try
            {
                AMT_ThirdPartyDataStorageService service = new AMT_ThirdPartyDataStorageService(WsMan);
                return (AmtStorageStatus)service.GetWriteEraseLimit(out WriteEraseLimit);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetWriteEraseLimit WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetWriteEraseLimit CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtStorageWrapper.WSMAN_GetWriteEraseLimit",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        #endregion // WSMAN_3PDS_Methods   

        public static bool CheckStorageWrapper()
        {
            System.IO.Directory.SetCurrentDirectory(System.Windows.Forms.Application.StartupPath);
            return System.IO.File.Exists("StorageWrapper.dll");
        }

        public static UIntPtr LibMajorVersion = new UIntPtr((uint)2);
        public static UIntPtr LibMinorVersion = new UIntPtr((uint)0);
        public static UIntPtr LibBuildNumber = new UIntPtr((uint)1);

        private const uint VARIABLE_ARRAY = 200;
        private const uint MAX_ENUM_HANDLES = 20;
        private const uint AmtStorageStatus_INVALID_HANDLE = 0xFFFFFFFF;
        private const uint ISVS_GROUP_PERMISSIONS_READ_ONLY = 1;
        private const uint ISVS_GROUP_PERMISSIONS_READ_WRITE = 2;
        private const uint ISVS_APPLICATION_NAME_FILTER = 0xFFFFFFF0;
        private const uint ISVS_VENDOR_NAME_FILTER = 0xFFFFFFF1;

        public enum AmtStorageGroupPermission
        {
            ReadOnly = 1,
            ReadWrite = 2,
        }

        #region Private Structures
        [StructLayout(LayoutKind.Sequential)]
        private struct ISVS_VERSION
        {
            byte MajorNumber;
            byte MinorNumber;
        };

        // Certificate data for mutual auth
        // and authentication scheme info
        [StructLayout(LayoutKind.Sequential)]
        public struct CERT_CONTEXT
        {
            public uint dwCertEncodingType;
            public IntPtr pbCertEncoded;
            public int cbCertEncoded;
            public IntPtr pCertInfo;
            public IntPtr hCertStore;
        }

        [StructLayout(LayoutKind.Sequential)]
        private struct SESSION_AUTHENTICATION_INFO
        {
            string CertificateName;
            IntPtr Certificate;
            bool Kerberos;
        };

        private byte[] ISVS_APPLICATION_ID = new byte[12];
        private byte[] ISVS_HMAC_SHA_1 = new byte[20];

        [StructLayout(LayoutKind.Sequential)]
        public struct PTSDK_BLOCK_ATTRIBUTES
        {
            public UInt32 BlockSize;
            public UInt32 BlockHidden;
            public IntPtr BlockName;

        }

        [StructLayout(LayoutKind.Sequential)]
        public struct PTSDK_UNICODE_STRING
        {
            UInt16 Length;
            UInt16 MaximumLength;
            IntPtr Buffer;
        };

        [StructLayout(LayoutKind.Sequential)]
        public class SESSION_HANDLE
        {
            SESSION_HANDLE Next;
            byte[] AppID;
            UInt32 AppHandle;
            UInt32 MessageNumber;
            UInt32 MaxMtu;
            byte[] IcvSikTx;
            byte[] IcvSikRx;

            UInt32 LastNetworkError;

            // Time (seconds) expiration check
            UInt32 TimeMark;
            UInt32 TimeRegTimeout;
            UInt32 LockTimeout;

            // SOAP destination Uri
            IntPtr TargetUrl;
            IntPtr Username;
            IntPtr Password; // encrypted in Windows
            ulong PassLen; // length of password for encrypt/decrypt

            byte IsvsIfMinorVersion;
            IntPtr AuthInfo;
        };
        #endregion

        public string EnterpriseName;
        public string VendorName;
        public string AppName;
        public Guid ApplicationGuid = Guid.Empty;

        private string TargetUrl;
        private string UserName;
        private string Password;
        private uint appHandle;
        private bool connected = false;
        public  bool Connected { get { return connected; } }
        private IntPtr sh;
        private static int InitCount = 0;
        private byte MajorVersion;
        private byte MinorVersion;
        private uint RegistrationTimeout;
        private uint LockTimeout;
        
        // WSMAN support
        private DotNetWSManClient WsMan;
        private string sessionHandleWSMAN;
        private uint MTU;

        public uint SessionHandle { get { return appHandle; } }

        public enum AmtStorageStatus
        {
            NO_WSMAN = -1,
            SUCCESS = 0,
            INTERNAL_ERROR,
            NOT_READY,
            INVALID_PT_MODE,
            INVALID_MESSAGE_LENGTH,
            TABLE_FINGERPRINT_NOT_AVAILABLE,
            INTEGRITY_CHECK_FAILED,
            UNSUPPORTED_ISVS_VERSION,
            APPLICATION_NOT_REGISTERED,
            INVALID_REGISTRATION_DATA,
            APPLICATION_DOES_NOT_EXIST,
            NOT_ENOUGH_STORAGE,
            INVALID_NAME,
            BLOCK_DOES_NOT_EXIST,
            INVALID_BYTE_OFFSET,
            INVALID_BYTE_COUNT,
            NOT_PERMITTED,
            NOT_OWNER,
            BLOCK_LOCKED_BY_OTHER,
            BLOCK_NOT_LOCKED,
            INVALID_GROUP_PERMISSIONS,
            GROUP_DOES_NOT_EXIST,
            INVALID_MEMBER_COUNT,
            MAX_LIMIT_REACHED,
            INVALID_AUTH_TYPE,
            AUTHENTICATION_FAILED,
            INVALID_DHCP_MODE,
            INVALID_IP_ADDRESS,
            INVALID_DOMAIN_NAME,
            NOT_USED1,
            REQUEST_UNEXPECTED,
            INVALID_TABLE_TYPE,
            INVALID_PROVISIONING_MODE,
            UNSUPPORTED_OBJECT,
            INVALID_TIME,
            INVALID_INDEX,
            INVALID_PARAMETER,
            NOT_USED2,
            FLASH_WRITE_LIMIT_EXCEEDED,
            NETWORK_IF_ERROR_BASE = 0x0800,
            SDK_DEFINED_ERROR_BASE = 0x1000,
            SDK_INTERNAL_ERROR = 0x1000,
            SDK_NOT_INITIALIZED,
            SDK_LIB_VERSION_UNSUPPORTED,
            SDK_INVALID_PARAM,
            SDK_RESOURCES,
            SDK_HARDWARE_ACCESS_ERROR,
            SDK_REQUESTOR_NOT_REGISTERED,
            SDK_NETWORK_ERROR,
            SDK_PARAM_BUFFER_TOO_SHORT,
            SDK_COM_NOT_INITIALIZED_IN_THREAD,
            SDK_URL_REQUIRED
        }

        /// <summary>
        /// Log into 3PDS using an existing WSMAN object
        /// </summary>
        /// <param name="WsMan"></param>
        /// <param name="EnterpriseName"></param>
        /// <param name="VendorName"></param>
        /// <param name="AppName"></param>
        /// <param name="UUID"></param>
        public AmtStorageWrapper(DotNetWSManClient WsMan, string EnterpriseName, string VendorName, string AppName, Guid UUID)
        {
            this.WsMan = WsMan;
            this.EnterpriseName = EnterpriseName;
            this.VendorName = VendorName;
            this.AppName = AppName;
            this.ApplicationGuid = UUID;
        }

        /// <summary>
        /// Log into 3PDS using SOAP
        /// </summary>
        /// <param name="WsMan"></param>
        /// <param name="EnterpriseName"></param>
        /// <param name="VendorName"></param>
        /// <param name="AppName"></param>
        /// <param name="UUID"></param>
        /// <param name="TargetUrl"></param>
        /// <param name="UserName"></param>
        /// <param name="Password"></param>
        public AmtStorageWrapper(string EnterpriseName, string VendorName, string AppName, Guid UUID, string TargetUrl, string UserName, string Password)
        {
            this.EnterpriseName = EnterpriseName;
            this.VendorName = VendorName;
            this.AppName = AppName;
            this.ApplicationGuid = UUID;
            this.TargetUrl = TargetUrl;
            this.UserName = UserName;
            this.Password = Password;
        }

        /// <summary>
        /// Long into 3PDS using an existing Intel AMT System object
        /// </summary>
        /// <param name="system"></param>
        /// <param name="EnterpriseName"></param>
        /// <param name="VendorName"></param>
        /// <param name="AppName"></param>
        /// <param name="UUID"></param>
        /// <param name="TargetUrl"></param>
        /// <param name="UserName"></param>
        /// <param name="Password"></param>
        public AmtStorageWrapper(AmtSystem system, string EnterpriseName, string VendorName, string AppName, Guid UUID)
        {
            this.WsMan = system.WsMan;
            this.EnterpriseName = EnterpriseName;
            this.VendorName = VendorName;
            this.AppName = AppName;
            this.ApplicationGuid = UUID;

            if (system.UseTls == false)
                TargetUrl = system.HostNameIdentity.ToWebHttpURL(16992) + "/StorageService";
            else
                TargetUrl = system.HostNameIdentity.ToWebHttpsURL(16993) + "/StorageService";

            this.UserName = system.Username;
            this.Password = system.Password;
        }

        public uint GetHandle()
        {
            return appHandle;
        }

        public AmtStorageStatus Connect()
        {
            AmtStorageStatus status = AmtStorageStatus.SUCCESS;
                
            if(WsMan != null)
            {
                // USE WSMAN
                // Open a session using WSMAN
                byte[][] appGuid = {ApplicationGuid.ToByteArray()};
                status = WSMAN_RegisterApplication(appGuid, VendorName, AppName, EnterpriseName, out sessionHandleWSMAN);
                if (status != AmtStorageStatus.SUCCESS)
                {
                    sessionHandleWSMAN = null;
                    return status;
                }
            }
            else
            {
                // USE StorageWrapper
                if (++InitCount == 1)
                {
                    // Initilize LIB
                    status = Storage_Initialize(ref LibMajorVersion, ref LibMinorVersion, out LibBuildNumber);
                    if (status != AmtStorageStatus.SUCCESS)
                    {
                        InitCount = 0;
                        return status;
                    }

                    // Initialize thread
                    status = Storage_InitializeCOMinThread();
                    if (status != AmtStorageStatus.SUCCESS)
                    {
                        InitCount = 0;
                        return status;
                    }

                    // Get version
                    IntPtr version = Marshal.AllocHGlobal(16); //IntPtr.Zero;
                    status = Storage_GetAPIVersionEx(version, UserName, Password, TargetUrl, IntPtr.Zero);
                    if (status != AmtStorageStatus.SUCCESS)
                    {
                        InitCount = 0;
                        return status;
                    }
                    MajorVersion = Marshal.ReadByte(version, 0);
                    MinorVersion = Marshal.ReadByte(version, 1);
                    Marshal.FreeHGlobal(version);

                    if (MajorVersion != 1 && MinorVersion != 3)
                    {
                        InitCount = 0;
                        return AmtStorageStatus.UNSUPPORTED_OBJECT;
                    }
                }

                // Convert Enterprisename, Vendorname, Appname to PTU string 
                IntPtr enterprise = CreatePTUString(EnterpriseName);
                IntPtr vendor = CreatePTUString(VendorName);
                IntPtr app = CreatePTUString(AppName);

                // Use a specified GUID
                IntPtr guidPtr = Marshal.AllocHGlobal(16);
                byte[] buf = ApplicationGuid.ToByteArray();
                for (int i = 0; i < 16; i++) Marshal.WriteByte(guidPtr, i, buf[i]);

                // Open a session using StorageWrapper
                sh = IntPtr.Zero;
                status = Storage_RegisterApplicationEx(out sh, UserName, Password, TargetUrl, guidPtr, vendor, app, enterprise, IntPtr.Zero);
                
                FreePTUString(vendor);
                FreePTUString(app);
                FreePTUString(enterprise);
                Marshal.FreeHGlobal(guidPtr);

                if (status != AmtStorageStatus.SUCCESS) return status;
                
                // Set WSMAN Session Handle to null so WSMAN is not used
                sessionHandleWSMAN = null;
            }
            
            // Get our application handle
            if(WsMan != null)
            {
                // USE WSMAN
                status = WSMAN_GetCurrentApplicationHandle(sessionHandleWSMAN, out appHandle);
                if (status != AmtStorageStatus.SUCCESS) return status;
            }
            else
            {           
                // USE StorageWrapper
                IntPtr appHandlePtr = IntPtr.Zero;
                status = Storage_GetCurrentApplicationHandle(sh, out appHandlePtr);
                if (status != AmtStorageStatus.SUCCESS) return status;
                appHandle = (UInt32)appHandlePtr.ToInt32();
            }
            connected = true;

            // Get timeouts
            if(WsMan != null)
            {
                // USE WSMAN
                WSMAN_GetTimeoutValues(out RegistrationTimeout, out LockTimeout);                
            }
            else
            {
                // USE StorageWrapper
                UIntPtr registrationTimeoutPtr = UIntPtr.Zero;
                UIntPtr lockTimeoutPtr = UIntPtr.Zero;
                Storage_GetTimeoutValues(sh, out registrationTimeoutPtr, out lockTimeoutPtr);
                RegistrationTimeout = registrationTimeoutPtr.ToUInt32();
                LockTimeout = lockTimeoutPtr.ToUInt32();
            }
            
            // Get MTU (WSMAN only)
            if(WsMan != null)
            {
                status = WSMAN_GetMTU(out MTU);
                if (status != AmtStorageStatus.SUCCESS) return status;
            }

            return status;
        }

        public AmtStorageStatus WriteBlock(uint handle, byte[] data)
        {
            AmtStorageStatus r = AmtStorageStatus.SUCCESS;

            // Lock block
            if (sessionHandleWSMAN != null)
            {
                // USE WSMAN
                r = WSMAN_LockBlock(sessionHandleWSMAN, handle);
            }
            else
            {
                // USE StorageWrapper
                r = Storage_LockBlock(sh, handle);
            }
            if (r != AmtStorageStatus.SUCCESS) return r;

            // Write block
            if(WsMan != null)
            {
                // USE WSMAN
                uint offset = 0;
                uint remaining = (uint)data.Length;

                while (remaining > 0)
                {
                    uint writeSize = Math.Min(remaining, MTU);
                    byte[] writeUnit = new byte[writeSize];
                    Array.Copy(data, offset, writeUnit, 0, writeSize);

                    r = WSMAN_WriteBlock(sessionHandleWSMAN, handle, offset, writeUnit, out writeSize);
                    if (r == AmtStorageStatus.SUCCESS)
                    {
                        // Calculate remaining size and new offset
                        remaining -= writeSize;
                        offset += writeSize;
                    }
                    else
                    {
                        // Failure writing block
                        break;
                    }
                }
            }
            else
            {
                // USE StorageWraper
                UIntPtr bytesWritten = UIntPtr.Zero;
                UInt32 offset = 0;
                r = Storage_WriteBlock(sh, handle, offset, (uint)data.Length, out bytesWritten, data);
            }

            // Unlock block
            if (sessionHandleWSMAN != null)
            {
                // USE WSMAN
                r = WSMAN_UnlockBlock(sessionHandleWSMAN, handle);
            }
            else
            {
                // USE StorageWrapper
                r = Storage_UnlockBlock(sh, handle);
            }

            return r;
        }
        public byte[] ReadBlock(uint handle, uint size)
        {
            byte[] data = null;
            AmtStorageStatus r;

            // Lock block
            if(WsMan != null)
            {
                // USE WSMAN
                r = WSMAN_LockBlock(sessionHandleWSMAN, handle);
                if (r != AmtStorageStatus.SUCCESS) return null;
            }
            else
            {
                // USE StorageWrapper 
                r = Storage_LockBlock(sh, handle);
                if (r != AmtStorageStatus.SUCCESS) return null;
            }

            // Read block
            if(WsMan != null)
            {
                // USE WSMAN
                uint offset = 0;
                byte[] readUnit = null;
                uint readSize = 0;
                data = new byte[size];
                
                while(size > 0)
                {
                    readSize = Math.Min(size, MTU);
                     
                    AmtStorageStatus status = WSMAN_ReadBlock(sessionHandleWSMAN, handle, offset, readSize, out readUnit);
                    if(status == AmtStorageStatus.SUCCESS)
                    {
                        // Append readBlock to data
                        readSize = (uint)readUnit.Length;
                        Array.Copy(readUnit, 0, data, offset, readSize);
                        
                        // Calculate remaining size and new offset
                        size -= readSize;
                        offset += readSize;
                    }
                    else
                    {
                        // Failure reading block
                        data = null;
                        break;
                    }
                }       
            }
            else
            {
                // USE StorageWrapper
                UIntPtr bytesRead = UIntPtr.Zero;
                UInt32 offset = 0;
                IntPtr Data = Marshal.AllocHGlobal((int)size);
                
                AmtStorageStatus status = Storage_ReadBlock(sh, handle, offset, size, out bytesRead, Data);
                if (status == AmtStorageStatus.SUCCESS)
                {
                    uint count = bytesRead.ToUInt32();

                    // Check size
                    if (count == size)
                    {
                        // Copy data
                        data = new byte[size];
                        for (int i = 0; i < count; i++) data[i] = Marshal.ReadByte(Data, i);
                    }
                }
                Marshal.FreeHGlobal(Data);
            }

            // Unlock block
            if (sessionHandleWSMAN != null)
            {
                // USE WSMAN
                if (WSMAN_UnlockBlock(sessionHandleWSMAN, handle) != AmtStorageStatus.SUCCESS) data = null;
            }
            else
            {
                // USE StorageWrapper 
                if (Storage_UnlockBlock(sh, handle) != AmtStorageStatus.SUCCESS) data = null;
            }

            return data;
        }

        public void Dispose()
        {
            if (connected)
            {
                if(WsMan != null)
                {
                    // USE WSMAN
                    WSMAN_UnregisterApplication(sessionHandleWSMAN);
                }
                else
                {
                    // USE StorageWrapper
                    Storage_UnregisterApplication(sh);

                    if (--InitCount == 0)
                    {
                        Storage_UninitializeCOMinThread();
                        Storage_Uninitialize();
                    }
                }
            }

            connected = false;
        }

        private void ClearHGlobal(IntPtr bufferPtr, int size)
        {
            for (int i = 0; i < size; i++) Marshal.WriteByte(bufferPtr, i, 0);
        }

        private IntPtr AllocatePGroupAttribStruct(int size)
        {
            // allocate wide char string buffer
            IntPtr bufferPtr = Marshal.AllocHGlobal((size * 2) + 2);
            ClearHGlobal(bufferPtr, (size * 2) + 2);
            // allocate attribute struct ptr
            IntPtr attribPtr = Marshal.AllocHGlobal(12);
            ClearHGlobal(attribPtr, 12);
            // buffer legnth
            Marshal.WriteInt16(attribPtr, 0, 0);
            // buffer max length
            Marshal.WriteInt16(attribPtr, 2, (short)(size * 2));
            // buffer ptr
            Marshal.WriteIntPtr(attribPtr, 4, bufferPtr);

            return attribPtr;
        }
        private void FreePGroupAttribStruct(IntPtr ptr)
        {
            IntPtr buf = Marshal.ReadIntPtr(ptr, 4);
            Marshal.FreeHGlobal(buf);
            Marshal.FreeHGlobal(ptr);
        }

        private IntPtr AllocateAppAttribStruct(int size)
        {
            // allocate attribute struct ptr
            IntPtr attribPtr = Marshal.AllocHGlobal(16);

            // allocate wide char string buffer, vendor
            IntPtr ebufferPtr = Marshal.AllocHGlobal((size * 2) + 2);
            ClearHGlobal(ebufferPtr, (size * 2) + 2);
            Marshal.WriteInt16(attribPtr, 0, 0);
            Marshal.WriteInt16(attribPtr, 2, (short)(size));
            Marshal.WriteIntPtr(attribPtr, 4, ebufferPtr);

            // allocate wide char string buffer, application
            IntPtr vbufferPtr = Marshal.AllocHGlobal((size * 2) + 2);
            ClearHGlobal(ebufferPtr, (size * 2) + 2);
            Marshal.WriteInt16(attribPtr, 8, 0);
            Marshal.WriteInt16(attribPtr, 10, (short)(size));
            Marshal.WriteIntPtr(attribPtr, 12, vbufferPtr);

            return attribPtr;
        }
        private void FreeBlockAppStruct(IntPtr ptr)
        {
            Marshal.FreeHGlobal(Marshal.ReadIntPtr(ptr, 4));
            Marshal.FreeHGlobal(Marshal.ReadIntPtr(ptr, 12));
            Marshal.FreeHGlobal(ptr);
        }

        private IntPtr AllocateBlockAttribStruct(int size)
        {
            // allocate wide char string buffer
            IntPtr bufferPtr = Marshal.AllocHGlobal((size * 2) + 2);
            // allocate attribute struct ptr
            IntPtr attribPtr = Marshal.AllocHGlobal(16);
            // buffer legnth
            Marshal.WriteInt16(attribPtr, 8, 0);
            // buffer max length
            Marshal.WriteInt16(attribPtr, 10, (short)(size));
            // buffer ptr
            Marshal.WriteIntPtr(attribPtr, 12, bufferPtr);

            return attribPtr;
        }
        private void FreeBlockAttribStruct(IntPtr ptr)
        {
            IntPtr buf = Marshal.ReadIntPtr(ptr, 12);
            Marshal.FreeHGlobal(buf);
            Marshal.FreeHGlobal(ptr);
        }
        private IntPtr CreatePTUString(string strParam)
        {
            byte[] data = new UnicodeEncoding().GetBytes(strParam);
            short strLen = (short)data.Length;

            IntPtr bufferPtr = Marshal.AllocHGlobal(strLen + 2);
            Marshal.Copy(data, 0, bufferPtr, strLen);
            Marshal.WriteInt16(data, strLen, 0);

            IntPtr strPtr = Marshal.AllocHGlobal(12);
            Marshal.WriteInt16(strPtr, 0, strLen);
            Marshal.WriteInt16(strPtr, 2, (short)(strLen + 2));
            Marshal.WriteIntPtr(strPtr, 4, bufferPtr); //Marshal.StringToHGlobalUni(strParam));

            return strPtr;
        }
        private void FreePTUString(IntPtr ptr)
        {
            IntPtr buf = Marshal.ReadIntPtr(ptr, 4);
            Marshal.FreeHGlobal(buf);
            Marshal.FreeHGlobal(ptr);
        }

        public AmtStorageStatus GetRegisteredApplications(out AmtStorageApplication[] apps)
        {
            AmtStorageStatus r = AmtStorageStatus.SUCCESS;
            ArrayList lst = new ArrayList();
            apps = null;

            if (sessionHandleWSMAN != null)
            {
                // USE WSMAN
                uint[] appHandles = null;
                r = WSMAN_GetRegisteredApplications(sessionHandleWSMAN, out appHandles);
                if(r != AmtStorageStatus.SUCCESS) return r;
                
                foreach (uint handle in appHandles)
                {
                    string ven = null;
                    string app = null;
                    r = WSMAN_GetApplicationAttributes(sessionHandleWSMAN, handle, out ven, out app);
                    lst.Add(new AmtStorageApplication(handle, this.EnterpriseName, ven, app));
                }
            }
            else
            {
                // USE StorageWrapper            
                UIntPtr TotalRegisteredAppCount;
                uint AppHandleCount = 100;
                IntPtr AppHandlesPtr = Marshal.AllocHGlobal(100 * 4);
                r = Storage_GetRegisteredApplications(sh, 1, out TotalRegisteredAppCount, ref AppHandleCount, AppHandlesPtr);
                if (r != AmtStorageStatus.SUCCESS) return r;

                IntPtr AppAttribs = AllocateAppAttribStruct(64);
                for (uint i = 0; i < AppHandleCount; i++)
                {
                    uint handle = (uint)Marshal.ReadInt32(AppHandlesPtr, (int)(i * 4));
                    r = Storage_GetApplicationAttributes(sh, handle, AppAttribs);

                    int len = Marshal.ReadInt16(AppAttribs, 0);
                    int ptr = Marshal.ReadInt32(AppAttribs, 4) + len;
                    Marshal.WriteInt16(new IntPtr(ptr), 0);
                    string ven = Marshal.PtrToStringUni(Marshal.ReadIntPtr(AppAttribs, 4));

                    len = Marshal.ReadInt16(AppAttribs, 8);
                    ptr = Marshal.ReadInt32(AppAttribs, 12) + len;
                    Marshal.WriteInt16(new IntPtr(ptr), 0);
                    string app = Marshal.PtrToStringUni(Marshal.ReadIntPtr(AppAttribs, 12));

                    lst.Add(new AmtStorageApplication(handle, this.EnterpriseName, ven, app));
                }
                FreeBlockAppStruct(AppAttribs);
                Marshal.FreeHGlobal(AppHandlesPtr);
            }

            apps = (AmtStorageApplication[])lst.ToArray(typeof(AmtStorageApplication));
            return r;
        }

        public AmtStorageStatus AllocateBlock(string name, uint size, out AmtStorageBlock block)
        {
            block = null;
            if (name == null) return AmtStorageStatus.INTERNAL_ERROR;
            AmtStorageStatus s = AmtStorageStatus.SUCCESS;
            if (WsMan != null)
            {
                // USE WSMAN
                uint blockHandle = 0;
                s = WSMAN_AllocateBlock(sessionHandleWSMAN, size, false, name, out blockHandle);
                if (s != AmtStorageStatus.SUCCESS) return s;
                block = new AmtStorageBlock(this, name, blockHandle, size, false);
            }
            else
            {
                // USE StorageWrapper
                IntPtr blockHandle = IntPtr.Zero;
                IntPtr BlockNamePtr = CreatePTUString(name);
                s = Storage_AllocateBlock(sh, size, false, BlockNamePtr, out blockHandle);
                if (s != AmtStorageStatus.SUCCESS) return s;
                block = new AmtStorageBlock(this, name, (uint)blockHandle.ToInt32(), size, false);
            }
            return s;
        }
        public AmtStorageStatus RemoveBlock(AmtStorageBlock block)
        {
            if(WsMan != null)
            {
                // USE WSMAN
                return WSMAN_DeallocateBlock(sessionHandleWSMAN, block.Handle);
            }
            else
            {
                // USE StorageWrapper
                return Storage_DeallocateBlock(sh, block.Handle);
            }            
        }

        public AmtStorageBlock[] GetOwnBlockList()
        {
            return GetBlockList(appHandle);
        }

        public AmtStorageBlock[] GetBlockList(uint applicationHandle)
        {
            AmtStorageBlock[] blocklist = null; 
            if(WsMan != null)
            {
                // USE WSMAN
                uint[] handles = null;
                AmtStorageStatus status = WSMAN_GetAllocatedBlocks(sessionHandleWSMAN, applicationHandle, out handles);
                if(status != AmtStorageStatus.SUCCESS)
                {
                    return null;
                }
                
                // Match StorageWrapper behavior.  If an application has zero 
                // blocks, return an array of zero size (not null).
                int count = 0;
                if(handles != null)
                {
                    count = handles.Length;
                }

                blocklist = new AmtStorageBlock[count];
                for(uint i = 0; i < count; i++)
                {
                    uint size;
                    bool hidden;
                    string blockName;
                    status = WSMAN_GetBlockAttributes(sessionHandleWSMAN, handles[i], out size, out hidden, out blockName);
                    if(status != AmtStorageStatus.SUCCESS)
                    {
                        continue;
                    }
                    blocklist[i] = new AmtStorageBlock(this, blockName, handles[i], size, hidden);
                }
            }
            else
            {
                // USE StorageWrapper               
                AmtStorageStatus status;

                // Initialize with invalid handle
                UInt32 startIndex = 1;
                UIntPtr totalCount = new UIntPtr((UInt32)MAX_ENUM_HANDLES);
                UIntPtr handleCount = new UIntPtr((UInt32)totalCount);
                IntPtr handles = Marshal.AllocHGlobal((int)MAX_ENUM_HANDLES * sizeof(UInt32));

                if (Storage_GetAllocatedBlocks(sh, applicationHandle, startIndex, out totalCount, ref handleCount, handles) != AmtStorageStatus.SUCCESS) return null;

                // Get handle count
                int count = (int)handleCount.ToUInt32();

                blocklist = new AmtStorageBlock[count];

                // Search for block name
                UInt32 curHandle;
                for (int i = 0; i < count; i++)
                {
                    curHandle = (UInt32)Marshal.ReadInt32(handles, i * 4);

                    // Allocate a block attribute struct with a wide string buffer
                    IntPtr attrib = AllocateBlockAttribStruct(64);
                    status = Storage_GetBlockAttributes(sh, curHandle, ref attrib);

                    if (status != AmtStorageStatus.SUCCESS)
                    {
                        FreeBlockAttribStruct(attrib);
                        continue;
                    }

                    IntPtr bufferPtr = Marshal.ReadIntPtr(attrib, 12);
                    short len = Marshal.ReadInt16(attrib, 8);
                    string blockName = Marshal.PtrToStringUni(bufferPtr, len / 2);
                    UInt32 size = (UInt32)Marshal.ReadInt32(attrib, 0);
                    bool hidden = Marshal.ReadInt32(attrib, 4) == 1;
                    blocklist[i] = new AmtStorageBlock(this, blockName, curHandle, size, hidden);
                    FreeBlockAttribStruct(attrib);
                }

                Marshal.FreeHGlobal(handles);
            }
            return blocklist;
        }

        public AmtStorageStatus LockBlock(uint blockHandle)
        {
            if(WsMan != null)
            {
                // USE WSMAN
                return WSMAN_LockBlock(sessionHandleWSMAN, blockHandle);
            }
            else
            {
                // USE StorageWrapper
                return Storage_LockBlock(sh, blockHandle);
            }
        }
        public AmtStorageStatus UnlockBlock(uint blockHandle)
        {
            if (sessionHandleWSMAN != null)
            {
                // USE WSMAN
                return WSMAN_UnlockBlock(sessionHandleWSMAN, blockHandle);
            }
            else
            {
                // USE StorageWrapper
                return Storage_UnlockBlock(sh, blockHandle);
            }
        }
        public AmtStorageStatus SetBlockName(uint blockHandle, string blockName)
        {
            if (sessionHandleWSMAN != null)
            {
                // USE WSMAN
                return WSMAN_SetBlockName(sessionHandleWSMAN, blockHandle, blockName);
            }
            else
            {
                // USE StorageWrapper
                return Storage_SetBlockName(sh, blockHandle, blockName);
            }
        }
        public AmtStorageStatus SetBlockVisibility(uint blockHandle, bool hidden)
        {
            if (sessionHandleWSMAN != null)
            {
                // USE WSMAN
                return WSMAN_SetBlockVisibility(sessionHandleWSMAN, blockHandle, hidden);
            }
            else
            {
                // USE StorageWrapper
                return Storage_SetBlockVisibility(sh, blockHandle, hidden);
            }
        }

        public AmtStorageStatus AddPermissionGroup(uint blockHandle, AmtStorageGroupPermission groupPermissions, string groupName, out AmtStorageWrapper.AmtStoragePermissionsGroup groupHandle)
        {
            AmtStorageStatus r = AmtStorageStatus.SUCCESS;
            groupHandle = null;
            if(WsMan != null)
            {
                // USE WSMAN
                uint newGroupHandle = 0;
                r = WSMAN_AddPermissionsGroup(sessionHandleWSMAN, blockHandle, (uint)groupPermissions, groupName, out newGroupHandle);
                groupHandle = new AmtStoragePermissionsGroup(newGroupHandle, groupName, groupPermissions);
            }
            else
            {
                // USE StorageWrapper
                IntPtr groupNamePtr = CreatePTUString(groupName);
                IntPtr groupHandlePtr = IntPtr.Zero;
                r = Storage_AddPermissionsGroup(sh, blockHandle, (uint)groupPermissions, groupNamePtr, out groupHandlePtr);
                FreePTUString(groupNamePtr);
                groupHandle = new AmtStoragePermissionsGroup((uint)(groupHandlePtr.ToInt32()), groupName, groupPermissions);
            }
            return r;
        }

        public class AmtStoragePermissionsGroup
        {
            public uint Handle;
            public string Name;
            public AmtStorageGroupPermission Permission;

            public AmtStoragePermissionsGroup(uint handle, string name, AmtStorageGroupPermission permission)
            {
                Handle = handle;
                Name = name;
                Permission = permission;
            }
        }

        public AmtStorageStatus GetPermissionsGroups(uint blockHandle, out AmtStoragePermissionsGroup[] groups)
        {
            AmtStorageStatus r = AmtStorageStatus.SUCCESS;
            groups = null;
            ArrayList groupHandlesList = new ArrayList();
            
            if(WsMan != null)
            {
                // USE WSMAN
                uint[] groupHandles = null;
                r = WSMAN_GetPermissionsGroups(sessionHandleWSMAN, blockHandle, out groupHandles);
                if (r != AmtStorageStatus.SUCCESS) return r;
                foreach(uint handle in groupHandles)
                {
                    groupHandlesList.Add(handle);
                }
            }
            else
            {
                // USE StorageWrapper
                IntPtr totalGroupCount = IntPtr.Zero;
                IntPtr groupHandleCount = Marshal.AllocHGlobal(4);
                Marshal.WriteInt32(groupHandleCount, 10);
                IntPtr groupHandles = Marshal.AllocHGlobal(4 * 10);

                // Get the list of group handles
                r = Storage_GetPermissionsGroups(sh, blockHandle, 1, out totalGroupCount, ref groupHandleCount, ref groupHandles);
                if (r != AmtStorageStatus.SUCCESS) return r;
                for (int i = 0; i < groupHandleCount.ToInt32(); i++)
                {
                    groupHandlesList.Add((uint)(Marshal.ReadInt32(groupHandles, i * 4)));
                }
                Marshal.FreeHGlobal(groupHandleCount);
                Marshal.FreeHGlobal(groupHandles);
            }

            // Get information on each group
            ArrayList lst = new ArrayList();
            foreach (uint groupHandle in groupHandlesList)
            {
                if(WsMan != null)
                {
                    // USE WSMAN
                    string groupName = null;
                    uint groupPermission = 0;
                    r = WSMAN_GetPermissionsGroupAttributes(sessionHandleWSMAN, blockHandle, groupHandle, out groupName, out groupPermission);
                    if (r != AmtStorageStatus.SUCCESS) return r;
                    
                    lst.Add(new AmtStoragePermissionsGroup(groupHandle, groupName, (AmtStorageGroupPermission)groupPermission));
                }
                else
                {
                    // USE StorageWrapper
                    IntPtr groupPerAtrib = AllocatePGroupAttribStruct(64);

                    // Get group atrributes
                    r = Storage_GetPermissionsGroupAttributes(sh, blockHandle, groupHandle, ref groupPerAtrib);
                    if (r != AmtStorageStatus.SUCCESS)
                    {
                        FreePGroupAttribStruct(groupPerAtrib);
                        return r;
                    }

                    // Get group handle & name
                    string groupName = Marshal.PtrToStringUni(Marshal.ReadIntPtr(groupPerAtrib, 4));
                    AmtStorageGroupPermission groupPermission = (AmtStorageGroupPermission)Marshal.ReadInt32(groupPerAtrib, 8);

                    lst.Add(new AmtStoragePermissionsGroup(groupHandle, groupName, groupPermission));

                    FreePGroupAttribStruct(groupPerAtrib);
                }
            }
            if (r != AmtStorageStatus.SUCCESS) return r;
            groups = (AmtStoragePermissionsGroup[])lst.ToArray(typeof(AmtStoragePermissionsGroup));

            return r;
        }

        public AmtStorageStatus RemovePermissionGroup(uint blockHandle, AmtStorageWrapper.AmtStoragePermissionsGroup group)
        {
            if(WsMan != null)
            {
                // USE WSMAN
                return WSMAN_RemovePermissionsGroup(sessionHandleWSMAN, blockHandle, group.Handle);
            }
            else
            {
                // USE StorageWrapper
                return Storage_RemovePermissionsGroup(sh, blockHandle, group.Handle);
            }
        }

        public AmtStorageStatus RemoveAllPermissionGroups(uint blockHandle)
        {
            AmtStorageStatus status = AmtStorageStatus.SUCCESS;
            if(WsMan != null)
            {
                // USE WSMAN
                uint[] groupHandles = null;
                status = WSMAN_GetPermissionsGroups(sessionHandleWSMAN, blockHandle, out groupHandles);
                if(status != AmtStorageStatus.SUCCESS) return status;
                
                foreach(uint handle in groupHandles)
                {
                    status = WSMAN_RemovePermissionsGroup(sessionHandleWSMAN, blockHandle, handle);
                    if (status != AmtStorageStatus.SUCCESS) break;
                }
            }
            else
            {
                // USE StorageWrapper           
                IntPtr TotalgroupCount;
                IntPtr GroupHandleCount = new IntPtr((Int32)16);
                IntPtr GroupHandles = Marshal.AllocHGlobal(16);
                UInt32 startIndex = 1;

                status = Storage_GetPermissionsGroups(sh, blockHandle, startIndex, out TotalgroupCount, ref GroupHandleCount, ref GroupHandles);
                if (status != AmtStorageStatus.SUCCESS)
                {
                    Marshal.FreeHGlobal(GroupHandles);
                    return status;
                }

                int groupCount = GroupHandleCount.ToInt32();
                for (int i = 0; i < groupCount; i++)
                {
                    status = Storage_RemovePermissionsGroup(sh, blockHandle, (UInt32)Marshal.ReadInt32(GroupHandles, i));
                    if (status != AmtStorageStatus.SUCCESS) break;
                }
                Marshal.FreeHGlobal(GroupHandles);
            }

            return status;
        }

        public AmtStorageStatus AddPermissionsGroupMembers(uint blockHandle, AmtStoragePermissionsGroup group, uint[] members)
        {
            if(WsMan != null)
            {
                // USE WSMAN
                return WSMAN_AddPermissionsGroupMembers(sessionHandleWSMAN, blockHandle, group.Handle, members);
            }
            else
            {
                // USE StorageWrapper
                return Storage_AddPermissionsGroupMembers(sh, blockHandle, group.Handle, (uint)(members.Length), members);
            }
        }

        public AmtStorageStatus GetPermissionsGroupMembers(uint blockHandle, AmtStoragePermissionsGroup group, out uint[] members)
        {
            members = null;
            AmtStorageStatus r = AmtStorageStatus.SUCCESS;
            
            if(WsMan != null)
            {
                // USE WSMAN
                r = WSMAN_GetPermissionsGroupMembers(sessionHandleWSMAN, blockHandle, group.Handle, out members);
                
                // StorageWrapper path returns an empty array instead of null,
                // so mimic that behavior if there are no group members.
                if(members == null)
                {
                    members = new uint[] {};
                }
            }
            else
            {
                // USE StorageWrapper
                UIntPtr TotalMemberCount = UIntPtr.Zero;
                uint MemberHandleCount = 8;
                IntPtr MemberHandles = Marshal.AllocHGlobal(8 * 4);
                this.ClearHGlobal(MemberHandles, 8 * 4);
                r = Storage_GetPermissionsGroupMembers(sh, blockHandle, group.Handle, 1, out TotalMemberCount, ref MemberHandleCount, MemberHandles);
                if (r != AmtStorageStatus.SUCCESS)
                {
                    Marshal.FreeHGlobal(MemberHandles);
                    return r;
                }
                ArrayList lst = new ArrayList();
                for (int i = 0; i < MemberHandleCount; i++)
                {
                    lst.Add((uint)(Marshal.ReadInt32(MemberHandles, i * 4)));
                }
                members = (uint[])lst.ToArray(typeof(uint));
            }

            return r;
        }

        public AmtStorageStatus RemovePermissionsGroupMembers(uint blockHandle, AmtStoragePermissionsGroup group, uint[] members)
        {
            if(WsMan != null)
            {
                // USE WSMAN
                return WSMAN_RemovePermissionsGroupMembers(sessionHandleWSMAN, blockHandle, group.Handle, members);
            }
            else
            {
                // USE StorageWrapper
                return Storage_RemovePermissionsGroupMembers(sh, blockHandle, group.Handle, (uint)(members.Length), members);
            }
        }

        public AmtStorageStatus GetBlockWriteEraseLimit(uint blockHandle, out uint writeEraseLimit)
        {
            AmtStorageStatus r = AmtStorageStatus.SUCCESS;
            writeEraseLimit = 0;
            
            if(WsMan != null)
            {
                // USE WSMAN
                r = WSMAN_GetWriteEraseLimit(out writeEraseLimit);
                if (r != AmtStorageStatus.SUCCESS) return r;
            }
            else
            {
                // USE StorageWrapper
                UIntPtr WriteEraseLimit;
                r = Storage_GetBlockWriteEraseLimit(sh, blockHandle, out WriteEraseLimit);
                if (r != AmtStorageStatus.SUCCESS) return r;
                writeEraseLimit = WriteEraseLimit.ToUInt32();
            }
            return r;
        }

        private Guid HostUUIDCache = Guid.Empty;
        private bool HostUUIDCacheSet = false;
        public AmtStorageStatus GetHostUUID(out Guid uuid)
        {
            if (HostUUIDCacheSet == true)
            {
                uuid = HostUUIDCache;
                return AmtStorageStatus.SUCCESS;
            }

            AmtStorageStatus r = AmtStorageStatus.SUCCESS;
            uuid = Guid.Empty;

            if(WsMan != null)
            {
                // USE WSMAN
                // No WSMAN equivalent to Storage_GetHostUUID
            }
            else
            {
                // USE StorageWrapper
                IntPtr UuidPtr = Marshal.AllocHGlobal(16);
                r = Storage_GetHostUUID(out UuidPtr);
                if (r != AmtStorageStatus.SUCCESS)
                {
                    Marshal.FreeHGlobal(UuidPtr);
                    return r;
                }
                byte[] buf = new byte[16];
                for (int i = 0; i < 16; i++) buf[i] = Marshal.ReadByte(UuidPtr, i);
                Marshal.FreeHGlobal(UuidPtr);
                uuid = new Guid(buf);
                HostUUIDCache = uuid;
                HostUUIDCacheSet = true;
            }
            return r;
        }

    }

    public class AmtStorageApplication
    {
        public string Enterprise;
        public string Vendor;
        public string Application;
        public uint Handle;

        public AmtStorageApplication(uint handle, string enterprise, string vendor, string application)
        {
            Handle = handle;
            Enterprise = enterprise;
            Vendor = vendor;
            Application = application;
        }
    }

    public class AmtStorageBlock
    {
        public AmtStorageWrapper Storage;
        public string Name;
        public uint Handle;
        public uint Size;
        public bool Hidden;
        private byte[] DataCache = null;

        public AmtStorageBlock(AmtStorageWrapper storageWrapper, string name, uint handle, uint size, bool hidden)
        {
            this.Storage = storageWrapper;
            this.Name = name;
            this.Handle = handle;
            this.Size = size;
            this.Hidden = hidden;
        }

        public AmtStorageWrapper.AmtStorageStatus Remove() { return Storage.RemoveBlock(this); }
        public void ClearCache() { DataCache = null; }
        public bool IsCached() { return (DataCache != null); }

        public byte[] ReadBlock()
        {
            if (DataCache != null) return DataCache;
            DataCache = Storage.ReadBlock(Handle, Size);
            if (DataCache == null) DataCache = new byte[0];
            return DataCache;
        }
        
        public AmtStorageWrapper.AmtStorageStatus WriteBlock(byte[] data)
        {
            AmtStorageWrapper.AmtStorageStatus r = Storage.WriteBlock(Handle, data);
            if (r == AmtStorageWrapper.AmtStorageStatus.SUCCESS) DataCache = data;
            return r;
        }

        private AmtStorageWrapper.AmtStorageStatus LockBlock() { return Storage.LockBlock(Handle); }
        private AmtStorageWrapper.AmtStorageStatus UnlockBlock() { return Storage.UnlockBlock(Handle); }
        public AmtStorageWrapper.AmtStorageStatus SetName(string blockName)
        {
            AmtStorageWrapper.AmtStorageStatus r = Storage.SetBlockName(Handle, blockName);
            if (r == AmtStorageWrapper.AmtStorageStatus.SUCCESS) Name = blockName;
            return r;
        }
        public AmtStorageWrapper.AmtStorageStatus SetVisibility(bool hidden)
        {
            AmtStorageWrapper.AmtStorageStatus r = Storage.SetBlockVisibility(Handle, hidden);
            if (r == AmtStorageWrapper.AmtStorageStatus.SUCCESS) Hidden = hidden;
            return r;
        }
        public AmtStorageWrapper.AmtStorageStatus AddPermissionGroup(AmtStorageWrapper.AmtStorageGroupPermission groupPermissions, string groupName, out AmtStorageWrapper.AmtStoragePermissionsGroup group) { return Storage.AddPermissionGroup(Handle, groupPermissions, groupName, out group); }
        public AmtStorageWrapper.AmtStorageStatus GetPermissionsGroups(out AmtStorageWrapper.AmtStoragePermissionsGroup[] groups) { return Storage.GetPermissionsGroups(Handle, out groups); }
        public AmtStorageWrapper.AmtStorageStatus RemoveAllPermissionGroups() { return Storage.RemoveAllPermissionGroups(Handle); }
        public AmtStorageWrapper.AmtStorageStatus RemovePermissionGroup(AmtStorageWrapper.AmtStoragePermissionsGroup group) { return Storage.RemovePermissionGroup(Handle, group); }
        public AmtStorageWrapper.AmtStorageStatus GetBlockWriteEraseLimit(out uint writeEraceLimit) { return Storage.GetBlockWriteEraseLimit(Handle, out writeEraceLimit); }
        public AmtStorageWrapper.AmtStorageStatus AddPermissionsGroupMembers(AmtStorageWrapper.AmtStoragePermissionsGroup group, uint[] members) { return Storage.AddPermissionsGroupMembers(Handle, group, members); }
        public AmtStorageWrapper.AmtStorageStatus GetPermissionsGroupMembers(AmtStorageWrapper.AmtStoragePermissionsGroup group, out uint[] members) { return Storage.GetPermissionsGroupMembers(Handle, group, out members); }
        public AmtStorageWrapper.AmtStorageStatus RemovePermissionsGroupMembers(AmtStorageWrapper.AmtStoragePermissionsGroup group, uint[] members) { return Storage.RemovePermissionsGroupMembers(Handle, group, members);  }
    }

    [Serializable]
    public class LocalStorageInfo
    {
        public string Enterprise;
        public string Vendor;
        public string AppName;
        public Guid UUID = Guid.Empty;

        [NonSerialized]
        public bool Connected = false;
        
        [NonSerialized]
        public AmtStorageWrapper Storage = null;

        [NonSerialized]
        public AmtStorageWrapper.AmtStorageStatus ConnectError = AmtStorageWrapper.AmtStorageStatus.SUCCESS;

        public LocalStorageInfo()
        {
        }

        public LocalStorageInfo(string enterprise, string vendor, string appName, Guid uuid)
        {
            Enterprise = enterprise;
            Vendor = vendor;
            AppName = appName;
            UUID = uuid;
        }

        /// <summary>
        /// Connect to 3PDS using SOAP
        /// </summary>
        /// <param name="useTls"></param>
        /// <param name="username"></param>
        /// <param name="password"></param>
        /// <param name="auth"></param>
        /// <returns></returns>
        public bool Connect(bool useTls, string username, string password)
        {
            if (Storage != null) return true;

            string targetUrl = null;
            if (useTls == false)
            {
                targetUrl = "http://" + Utils.GetLocalFqdn() + ":16992/StorageService";
            }
            else
            {
                targetUrl = "https://" + Utils.GetLocalFqdn() + ":16993/StorageService";
            }

            Storage = new AmtStorageWrapper(Enterprise, Vendor, AppName, UUID, targetUrl, username, password);
            ConnectError = Storage.Connect();
            if (ConnectError != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
            {
                Storage = null;
                return false;
            }

            Connected = true;
            return true;
        }

        /// <summary>
        /// Connect to 3PDS using WSMAN
        /// </summary>
        /// <param name="useTls"></param>
        /// <param name="username"></param>
        /// <param name="password"></param>
        /// <param name="auth"></param>
        /// <returns></returns>
        public bool Connect(DotNetWSManClient WsMan)
        {
            if (Storage != null) return true;
            Storage = new AmtStorageWrapper(WsMan, Enterprise, Vendor, AppName, UUID);
            ConnectError = Storage.Connect();
            if (ConnectError != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
            {
                Storage = null;
                return false;
            }

            Connected = true;
            return true;
        }

        /// <summary>
        /// Connect to 3PDS using an Intel AMT System object
        /// </summary>
        /// <param name="useTls"></param>
        /// <param name="username"></param>
        /// <param name="password"></param>
        /// <param name="auth"></param>
        /// <returns></returns>
        public bool Connect(AmtSystem system)
        {
            if (Storage != null) return true;
            Storage = new AmtStorageWrapper(system, Enterprise, Vendor, AppName, UUID);
            ConnectError = Storage.Connect();
            if (ConnectError != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
            {
                Storage = null;
                return false;
            }

            Connected = true;
            return true;
        }

        public void Disconnect()
        {
            Connected = false;
            if (Storage == null) return;
            Storage.Dispose();
            Storage = null;
        }

        public AmtStorageApplication[] GetApplications()
        {
            AmtStorageApplication[] apps = null;
            Storage.GetRegisteredApplications(out apps);
            return apps;
        }
    }

}
