// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================

namespace ManageabilityStack
{
    public class AmtWakeOnEvent
    {
        private WoxService service;
        private AmtSystem system;
        public bool IsCached() { return true; }

        public enum WoxCallStatus
        {
            STATUS_SUCCESS = 					        0x0,
            STATUS_INTERNAL_ERROR = 				    0x1,
            STATUS_NOT_CONNECTED =				        0x2001,
            STATUS_INVALID_PARAMETER	=			    0x2002,
            STATUS_NOT_SUPPORTED =				        0x2003,	
            STATUS_BUFFER_SIZE_TOO_SMALL =			    0x2004,	
            STATUS_FAILURE =					        0x2005,
            STATUS_LOW_RESOURCES =				        0x2006,	
            STATUS_HARDWARE_ACCESS_ERROR = 		        0x2007,	
            STATUS_INVALID_INTERFACE_HANDLE =		    0x2008,	
            STATUS_INVALID_AGENT_HANDLE = 			    0x2009,	
            STATUS_INVALID_GEN_ID =				        0x200A,	
            STATUS_INVALID_FILTER_ID = 			        0x200B,	
            STATUS_INVALID_DOC_ID =				        0x200C,	
            STATUS_PORT_IN_USE = 				        0x200D,	
            STATUS_DOC_IN_USE =				            0x200E,	
            STATUS_INVALID_AGENT_TYPE = 			    0x200F,	
            STATUS_INVALID_TRANSPORT_TYPE =		        0x2010,	
            STATUS_INVALID_ADDRESS =				    0x2011,	
            STATUS_INVALID_DOC	=				        0x2012,	
            STATUS_INVALID_RESP_OPTION =			    0x2013,	
            STATUS_INVALID_ACTION =				        0x2014,	
            STATUS_INVALID_START_MODE =			        0x2015,	
            STATUS_INVALID_STOP_MODE	=			    0x2016,	
            STATUS_MAX_NUM_OF_AGENT_REACHED =		    0x2017,	
            STATUS_MAX_NUM_OF_GEN_REACHED =		        0x2018,	
            STATUS_MAX_NUM_OF_FILTER_REACHED =		    0x2019,	
            STATUS_MAX_NUM_OF_DOC_REACHED =		        0x201A,	
            STATUS_TOO_MANY_REMOTE_ADDR_QUALIFIER =	    0x201B,	
            STATUS_RETRANSMISSION_INTERVAL_TOO_SHORT =	0x201C,	
            STATUS_DOC_TOO_LONG =				        0x201D,
            STATUS_STRING_TOO_LONG = 			        0x201E,
            STATUS_AGENT_RUNNING =				        0x201F,
            STATUS_RX_BUFFER_EMPTY =				    0x2020,
            STATUS_UPNP_INVALID_DEVICE_ID =		        0x2021,
            STATUS_SIP_REGISTER_MSG_TOO_LONG =		    0x2022,
            STATUS_NO_PACKET_SENT =				        0x2023,
            STATUS_MAX_NUM_OF_APPS_REACHED =            0x2024,
            STATUS_MAX_NUM_OF_CREDENTIALS_REACHED =	    0x2025,
            STATUS_INVALID_STATEFUL_FILTER_STATE =	    0x2026,
            STATUS_METHOD_FILTER_ALREADY_EXISTS =	    0x2027,  
            STATUS_INVALID_CRED_ID = 			        0x2028,	
            STATUS_AGENT_NOT_STARTED = 			        0x2029,	
            AGENT_NET_BINDING_FAILURE =             	0x202A,  
            GEN_UNKNOWN_ERROR =                     	0x202B,  
            GEN_DNS_ERROR =                          	0x202C,  
            GEN_TRANSPORT_ERROR =                    	0x202D,  
            GEN_SIP_REGISTER_TIMEOUT =              	0x202E,  
            GEN_SIP_REGISTER_REJECT =              	    0x202F,  
            GEN_SIP_REGISTER_UNKNOWN_CHALLENGE =     	0x2030,  
            GEN_SIP_REGISTER_MSG_TOO_LONG =          	0x2031,  
            GEN_SIP_REGISTER_RETRANSMISSION_INTERVAL_TOO_SHORT = 0x2032, 
            GEN_SIP_REGISTER_STUNKA_FLOW_LOST =      	0x2033,
            GEN_STUN_FAILURE =                       	0x2034,  
            GEN_SIP_UNHANDLED_RESPONSE =            	0x2035,  
            GEN_SIP_CHALLENGE_COUNT_EXCEEDED = 		    0x2036,
            GEN_SIP_CHALLENGE_CREATE_FAILED =		    0x2037,
            FILTER_UNKNOWN_ERROR =                   	0x2038,  
            FILTER_PACKET_DROPPED =                  	0x2039,  
            FILTER_TRANSPORT_ERROR =                	0x203A,  
            FILTER_TIMEOUT =                         	0x203B,
            STATUS_SHARED_PORT_ERROR =                	0x203C,  
            EXCEEDED_PORT_LIMIT =                     	0x203D,
            AGENT_IP_ADDRESS_CHANGED =			        0x203E  
        }

        private uint AppHandle = uint.MaxValue;

        public AmtWakeOnEvent(WoxService service, AmtSystem system)
        {
            if (service != null) this.service = service;
            this.system = system;
        }

        public void Dispose()
        {
            service = null;
        }

        public void ClearCache()
        {
        }

        public void FetchCache()
        {
        }

        private VersionType VersionCache = null;
        public WoxCallStatus GetVersion(out VersionType version)
        {
            if (VersionCache != null) {version = VersionCache; return WoxCallStatus.STATUS_SUCCESS; }
            WoxCallStatus r = (WoxCallStatus)service.GetVersion(out VersionCache);
            version = VersionCache;
            return r;
        }

        public WoxCallStatus Connect(uint hostPartitionID, ushort defaultPort)
        {
            if (AppHandle != uint.MaxValue) return WoxCallStatus.STATUS_FAILURE;
            WoxCallStatus r = (WoxCallStatus)service.Connect(hostPartitionID, defaultPort, defaultPort != 0, out AppHandle);
            return r;
        }

        public WoxCallStatus Disconnect()
        {
            if (AppHandle == uint.MaxValue) return WoxCallStatus.STATUS_FAILURE;
            WoxCallStatus r = (WoxCallStatus)service.Disconnect(AppHandle);
            AppHandle = uint.MaxValue;
            return r;
        }

        private CapabilityType CapabilityCache = null;
        public WoxCallStatus GetCapability(CapabilityType capability)
        {
            if (CapabilityCache != null) { capability = CapabilityCache; return WoxCallStatus.STATUS_SUCCESS; }
            WoxCallStatus r = (WoxCallStatus)service.GetCapability(AppHandle, out CapabilityCache);
            capability = CapabilityCache;
            return r;
        }

        public WoxCallStatus GetNetAgentCapability(uint AgentType, out AgentCapabilityType CapabilityBuffer)
        {
            WoxCallStatus r = (WoxCallStatus)service.GetNetAgentCapability(AppHandle, AgentType, out CapabilityBuffer);
            return r;
        }

        public WoxCallStatus AddDocument(DocumentBufferType document, out uint documentid)
        {
            WoxCallStatus r = (WoxCallStatus)service.AddDocument(AppHandle, document, out documentid);
            return r;
        }

        public WoxCallStatus RemoveDocument(uint documentid)
        {
            WoxCallStatus r = (WoxCallStatus)service.RemoveDocument(AppHandle, documentid);
            return r;
        }

        public WoxCallStatus AddNetAgent(ushort LocalPort, uint AgentType, uint StopMode, out uint AgentHandle)
        {
            WoxCallStatus r = (WoxCallStatus)service.AddNetAgent(AppHandle, LocalPort, AgentType, StopMode, out AgentHandle);
            return r;
        }

        public WoxCallStatus AddNetAgent(uint AgentHandle, uint StartMode)
        {
            WoxCallStatus r = (WoxCallStatus)service.StartAgent(AgentHandle, StartMode);
            return r;
        }

        public WoxCallStatus StopAgent(uint AgentHandle)
        {
            WoxCallStatus r = (WoxCallStatus)service.StopAgent(AgentHandle);
            return r;
        }

        public WoxCallStatus RemoveAgent(uint AgentHandle)
        {
            WoxCallStatus r = (WoxCallStatus)service.RemoveAgent(AppHandle, AgentHandle);
            return r;
        }

        public WoxCallStatus AgentAddStunGenerator(NetAddressType DestAddress, StunInfoType StunInfo, uint Action, out uint GeneratorId)
        {
            WoxCallStatus r = (WoxCallStatus)service.AgentAddStunGenerator(AppHandle, DestAddress, StunInfo, Action, out GeneratorId);
            return r;
        }

        public WoxCallStatus NetAgentRemoveGenerator(uint GeneratorId)
        {
            WoxCallStatus r = (WoxCallStatus)service.NetAgentRemoveGenerator(AppHandle, GeneratorId);
            return r;
        }

        public WoxCallStatus NetAgentRemoveFilter(uint FilterId)
        {
            WoxCallStatus r = (WoxCallStatus)service.NetAgentRemoveFilter(AppHandle, FilterId);
            return r;
        }

        public WoxCallStatus AgentGetStatus(out AgentStatusType AgentStatus)
        {
            WoxCallStatus r = (WoxCallStatus)service.AgentGetStatus(AppHandle, out AgentStatus);
            return r;
        }

        public WoxCallStatus NetAgentGetFilterStatus(uint FilterId, out FilterStatusType FilterStatus)
        {
            WoxCallStatus r = (WoxCallStatus)service.NetAgentGetFilterStatus(AppHandle, FilterId, out FilterStatus);
            return r;
        }

        public WoxCallStatus NetAgentGetGeneratorStatus(uint GeneratorId, out GeneratorStatusType GeneratorStatus)
        {
            WoxCallStatus r = (WoxCallStatus)service.NetAgentGetGeneratorStatus(AppHandle, GeneratorId, out GeneratorStatus);
            return r;
        }

        public WoxCallStatus NetAgentGetNextBufferedPacket(out byte[] Buffer, out PacketInfoType RxPacketInfo, out uint FilterId, out uint TimePassed, out uint NumUnReadPacket)
        {
            WoxCallStatus r = (WoxCallStatus)service.NetAgentGetNextBufferedPacket(AppHandle, out Buffer, out RxPacketInfo, out FilterId, out TimePassed, out NumUnReadPacket);
            return r;
        }

        public WoxCallStatus NetAgentGetLastGeneratedPacket(uint GeneratorId, out byte[] Buffer, out PacketInfoType TxPacketInfo, out uint TimePassed)
        {
            WoxCallStatus r = (WoxCallStatus)service.NetAgentGetLastGeneratedPacket(AppHandle, GeneratorId, out Buffer, out TxPacketInfo, out TimePassed);
            return r;
        }

        public WoxCallStatus SipAgentAddCredential(CredentialDataType Credential, out uint CredentialId)
        {
            WoxCallStatus r = (WoxCallStatus)service.SipAgentAddCredential(AppHandle, Credential, out CredentialId);
            return r;
        }

        public WoxCallStatus SipAgentRemoveCredential(uint CredentialId)
        {
            WoxCallStatus r = (WoxCallStatus)service.SipAgentRemoveCredential(AppHandle, CredentialId);
            return r;
        }

        public WoxCallStatus SipAgentSetUnsupportedMethodResponse(SipResponseInfoType RespInfo)
        {
            WoxCallStatus r = (WoxCallStatus)service.SipAgentSetUnsupportedMethodResponse(AppHandle, RespInfo);
            return r;
        }

        public WoxCallStatus SipAgentAddFilter(uint Type, uint Action, SipResponseInfoType RespInfo, SipResponseInfoType StatefulProvisionalRespInfo, string StringtoMatch, out uint FilterId)
        {
            WoxCallStatus r = (WoxCallStatus)service.SipAgentAddFilter(AppHandle, Type, Action, RespInfo, StatefulProvisionalRespInfo, StringtoMatch, out FilterId);
            return r;
        }

        public WoxCallStatus SipStatefulFilterTerminateTransaction(uint FilterId, SipResponseDocumentType FinalRespDoc, uint SipRespOpt)
        {
            WoxCallStatus r = (WoxCallStatus)service.SipStatefulFilterTerminateTransaction(AppHandle, FilterId, FinalRespDoc, SipRespOpt);
            return r;
        }

        public WoxCallStatus SipStatefulFilterGetStatus(uint FilterId, out uint TransactionStatus)
        {
            WoxCallStatus r = (WoxCallStatus)service.SipStatefulFilterGetStatus(AppHandle, FilterId, out TransactionStatus);
            return r;
        }

        public WoxCallStatus SipAgentAddRegisterGenerator(NetAddressType DestAddress, SipRegisterInfoType RegisterInfo, bool PersistentConnection, StunInfoType StunInfo, uint NATKeepAliveInterval, uint Action, out uint GeneratorId)
        {
            WoxCallStatus r = (WoxCallStatus)service.SipAgentAddRegisterGenerator(AppHandle, DestAddress, RegisterInfo, PersistentConnection, StunInfo, NATKeepAliveInterval, Action, out GeneratorId);
            return r;
        }

        public WoxCallStatus UpnpAddRootDevice(UpnpRootDeviceType RootDevice, ushort AdditionalPort, out uint DeviceId)
        {
            WoxCallStatus r = (WoxCallStatus)service.UpnpAddRootDevice(AppHandle, RootDevice, AdditionalPort, out DeviceId);
            return r;
        }

        public WoxCallStatus UpnpRemoveRootDevice(uint DeviceId)
        {
            WoxCallStatus r = (WoxCallStatus)service.UpnpRemoveRootDevice(AppHandle, DeviceId);
            return r;
        }

    }
}
