// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.IO;
using System.Net;
using System.Net.Sockets;

namespace ManageabilityStack
{
    public class HeciBridgeClass
    {
        private ushort port;
        private TcpListener tcpListener = null;
        private ArrayList tcpClients = new ArrayList();
        public HeciWrapper Heci;

        public delegate void HeciMessageHandler(HeciBridgeClass bridge, TcpClient client, byte[] buf, int buflen);
        public event HeciMessageHandler HeciMessageEvent;

        public HeciBridgeClass(HeciWrapper heci, ushort port)
        {
            this.Heci = heci;
            this.port = port;
        }

        public bool Start()
        {
            if (tcpListener != null) return true;

            tcpListener = new TcpListener(IPAddress.Any, port);
            try
            {
                tcpListener.Start();
                tcpListener.BeginAcceptTcpClient(new AsyncCallback(ListenerEvent), this);
            }
            catch (SocketException)
            {
                tcpListener = null;
                return false;
            }
            return true;
        }

        public void Stop()
        {
            if (tcpListener != null)
            {
                tcpListener.Stop();
                tcpListener = null;
            }

            foreach (TcpClient client in tcpClients) { client.Close(); }
            tcpClients.Clear();
        }

        private class TcpClientState
        {
            public TcpClient client;
            public byte[] buf;
            public int read;

            public TcpClientState(TcpClient client, byte[] buf)
            {
                this.client = client;
                this.buf = buf;
                this.read = 0;
            }
        }

        private void ListenerEvent(IAsyncResult r)
        {
            if (tcpListener == null) return;

            TcpClient client;
            try
            {
                client = tcpListener.EndAcceptTcpClient(r);
            }
            catch (ObjectDisposedException) { return; }

            byte[] buf = new byte[30000];
            client.GetStream().BeginRead(buf, 0, 30000, new AsyncCallback(ReadEvent), new TcpClientState(client, buf));

            tcpListener.BeginAcceptTcpClient(new AsyncCallback(ListenerEvent), this);
        }

        private void ReadEvent(IAsyncResult r)
        {
            TcpClientState state = (TcpClientState)r.AsyncState;
            if (state == null) return;
            TcpClient client = state.client;

            int len;
            try
            {
                len = client.GetStream().EndRead(r);
            }
            catch (InvalidOperationException)
            {
                client.Close();
                return;
            }
            catch (IOException)
            {
                client.Close();
                return;
            }
            if (len == 0)
            {
                client.Close();
                return;
            }

            state.read += len;

            // Check that we got the entire HECI command
            int msglen = HeciWrapper.GetHeciRequestMessageLength(state.buf);
            if (state.read >= msglen)
            {
                byte[] b = new byte[msglen];
                Array.Copy(state.buf, b, msglen);

                byte[] outbuf = Heci.ExecuteCommand(b, 10000);
                if (outbuf == null)
                {
                    client.Close();
                }
                try
                {
                    client.GetStream().Write(outbuf, 0, outbuf.Length);
                }
                catch (Exception)
                {
                    client.Close();
                }

                if (HeciMessageEvent != null) HeciMessageEvent(this, client, b, msglen);
            }

            try
            {
                client.GetStream().BeginRead(state.buf, state.read, 30000 - state.read, new AsyncCallback(ReadEvent), state);
            }
            catch (ObjectDisposedException) { client.Close(); }
            catch (InvalidOperationException) { client.Close(); }
            catch (IOException) { client.Close(); }
        }

    }
}
