// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.IO;
using System.IO.Compression;
using System.Net;
using System.Net.NetworkInformation;
using System.Text;
using System.Windows.Forms;
using System.Xml.Serialization;

namespace ManageabilityStack
{
    public class Utils
    {
        private static Hashtable PCIDevicesTable = null;
        public static void LoadPciDeviceList(FileInfo file)
        {
            if (file.Exists == true)
            {
                FileStream f;
                try
                {
                    f = new FileStream(file.FullName, FileMode.Open);
                }
                catch (Exception ex) { Utils.FileErrorMessageBox(null, ex); return; }
                byte[] buf = new byte[(int)f.Length];
                f.Read(buf, 0, (int)f.Length);
                f.Close();
                string filestr = UTF8Encoding.UTF8.GetString(buf);
                string[] strs = filestr.Split('\n');
                PCIDevicesTable = new Hashtable();
                Hashtable vendor = null;
                foreach (string str in strs)
                {
                    if (str.Length < 5 || str[0] == ';') continue;
                    if (str[0] == '\t')
                    {
                        if (vendor == null) continue;
                        string[] x = str.Split('\t');
                        if (x.Length == 3)
                        {
                            try
                            {
                                string xx = x[1].Trim();
                                int i = xx.IndexOf(" ");
                                if (i >= 0) xx = xx.Substring(0, i);
                                if (xx.EndsWith("h")) xx = xx.Substring(0, xx.Length - 1);
                                if (IsHex(xx) == true)
                                {
                                    int deviceid = Convert.ToInt32(xx, 16);
                                    if (deviceid != 0 && vendor.ContainsKey(deviceid) == false) vendor.Add(deviceid, FilterStrEx(x[2]));
                                }
                            }
                            catch (Exception) { }
                        }
                    }
                    else
                    {
                        string[] x = str.Split('\t');
                        if (x.Length == 2)
                        {
                            int vendorid = Convert.ToInt32(x[0], 16);
                            if (vendorid == 0)
                            {
                                vendor = null;
                            }
                            else
                            {
                                vendor = new Hashtable();
                                vendor[0] = FilterStrEx(x[1]);
                                PCIDevicesTable.Add(vendorid, vendor);
                            }
                        }
                    }
                }
            }
        }

        private static bool MonoDetected = false;
        private static bool MonoActive = false;
        public static bool IsMono()
        {
            if (MonoDetected) return MonoActive;
            MonoActive = (Type.GetType("Mono.Runtime") != null);
            MonoDetected = true;
            return MonoActive;
        }
        
        public static void ErrorMessageBox(IWin32Window parent, string call, AmtCallStatus error)
        {
            MessageBox.Show(parent, string.Format(Properties.Resources.CallError, call, error.ToString()), Properties.Resources.CallErrorTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        public static void ErrorMessageBox(IWin32Window parent, string call, string error)
        {
            MessageBox.Show(parent, string.Format(Properties.Resources.CallError, call, error), Properties.Resources.CallErrorTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        public static void FileErrorMessageBox(IWin32Window parent, Exception ex)
        {
            MessageBox.Show(parent, ex.Message, Properties.Resources.FileCallErrorTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        public static bool GetPciDevice(int vendorid, int deviceid, out string vendor, out string device)
        {
            vendor = string.Format(Properties.Resources.UnknownPciVendor, vendorid.ToString());
            device = string.Format(Properties.Resources.UnknownPciDevice, deviceid.ToString());
            if (PCIDevicesTable == null) return false;
            Hashtable t = (Hashtable)PCIDevicesTable[vendorid];
            if (t == null) return false;
            vendor = (string)t[0];
            if (t.Contains(deviceid) == true)
            {
                device = (string)t[deviceid];
                return true;
            }
            return false;
        }

        public static void AddTreeNodeSorted(TreeNode root, TreeNode node)
        {
            int i = 0;
            foreach (TreeNode n in root.Nodes)
            {
                if (n.Text.CompareTo(node.Text) > 0)
                {
                    root.Nodes.Insert(i, node);
                    return;
                }
                i++;
            }
            root.Nodes.Add(node);
        }

        public static string Serialize(object o, Type t)
        {
            XmlSerializer serializer = new XmlSerializer(t);
            MemoryStream m = new MemoryStream();
            serializer.Serialize(m, o);
            return UTF8Encoding.UTF8.GetString(m.ToArray());
        }

        public static object Unserialize(string s, Type t)
        {
            try
            {
                MemoryStream m = new MemoryStream();
                byte[] buf = UTF8Encoding.UTF8.GetBytes(s);
                m.Write(buf, 0, buf.Length);
                m.Seek(0, SeekOrigin.Begin);
                XmlSerializer serializer = new XmlSerializer(t);
                return serializer.Deserialize(m);
            }
            catch (Exception) {return null;}
        }

        public static byte[] SerializeBinary(object o)
        {
            System.Runtime.Serialization.Formatters.Binary.BinaryFormatter serializer = new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter();
            MemoryStream m = new MemoryStream();
            serializer.Serialize(m, o);
            return m.ToArray();
        }

        public static object UnserializeBinary(byte[] buf)
        {
            try
            {
                MemoryStream m = new MemoryStream();
                m.Write(buf, 0, buf.Length);
                m.Seek(0, SeekOrigin.Begin);
                System.Runtime.Serialization.Formatters.Binary.BinaryFormatter serializer = new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter();
                return serializer.Deserialize(m);
            }
            catch (Exception) { return null; }
        }

        public static void SerializeToClipboard(object o, Type t)
        {
            XmlSerializer serializer = new XmlSerializer(t);
            MemoryStream m = new MemoryStream();
            serializer.Serialize(m, o);
            Clipboard.SetDataObject(UTF8Encoding.UTF8.GetString(m.ToArray()), true);
        }

        public static object UnSerializeFromClipboard(Type t)
        {
            try
            {
                IDataObject d = Clipboard.GetDataObject();
                string s = (string)d.GetData(typeof(string));
                if (t == typeof(string)) return s;
                if (s.StartsWith("<?xml version=\"") == false) return null;
                MemoryStream m = new MemoryStream();
                byte[] buf = UTF8Encoding.UTF8.GetBytes(s);
                m.Write(buf,0,buf.Length);
                m.Seek(0, SeekOrigin.Begin);
                XmlSerializer serializer = new XmlSerializer(t);
                return serializer.Deserialize(m);
            }
            catch (Exception) {return null;}
        }

        public static ListViewItem GetItemWithTag(ListView listview, object tag)
        {
            foreach (ListViewItem l in listview.Items) {if (l.Tag.Equals(tag)) return l;}
            return null;
        }

        public static void CheckAllItems(ListView listview)
        {
            foreach (ListViewItem l in listview.Items) { l.Checked = true; }
        }

        public static void RemoveAllCheckedItems(ListView listview)
        {
            ArrayList a = new ArrayList();
            foreach (ListViewItem l in listview.Items) {if (l.Checked == true) a.Add(l);}
            foreach (ListViewItem l in a) {listview.Items.Remove(l);}
        }

        public static ArrayList GetNodeTagsofType(TreeNode node, Type tagType)
        {
            ArrayList list = new ArrayList();
            foreach (TreeNode n in node.Nodes)
            {
                if (n.Tag != null && n.Tag.GetType() == tagType)
                {
                    list.Add(n.Tag);
                }
            }
            return list;
        }

        public static byte[] Base32ToBytes(string Base32)
        {
            // 4-30-2008 RKFOOTE
            string input = Base32.Replace("-", "");
            char c;
            int bits = 0;
            int currentByte = 0;
            byte[] output = new byte[(int)Math.Ceiling((((decimal)input.Length * 5) / 8))];
            int working = 0;
            int offset = 0;
            int[] leftShift = { 11, 10, 9, 8, 7, 6, 5, 4, 3, 2 };
            for (int i = 0; i < input.Length; i++)
            {
                c = char.Parse(input.Substring(i, 1));
                if (c >= 'A' && c <= 'Z')
                {
                    bits = c - 65;
                }
                else if (c >= '2' && c <= '7')
                {
                    bits = c - 24;
                }
                working |= bits << leftShift[offset];
                offset += 5;
                if (offset >= 8)
                {
                    output[currentByte] = (byte)((working & 0x0000FF00) >> 8);
                    working &= 0x000000FF;
                    working <<= 8;
                    offset -= 8;
                    currentByte++;
                }
            }
            return output;
        }

        public static string BytesToBase32(byte[] buf)
        {
            // 4-23-2008 RKFOOTE
            // See RFC-4648 for details
            // Adds "-" every 4 characters
            // Does not add "=" processing character

            StringBuilder s = new StringBuilder();
            int CurrentBit = 0;
            int CurrentByte = 0;
            int data = 0;

            char[] lookup = { 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '2', '3', '4', '5', '6', '7' };
            int[] bitmask = { 0xF800, 0x7C00, 0x3E00, 0x1F00, 0x0F80, 0x07C0, 0x03E0, 0x01F0, 0x0F80 };
            int[] rightShift = { 11, 10, 9, 8, 7, 6, 5, 4, 3, 2 };

            int charCount = 0;
            string test;
            while (CurrentByte < buf.Length)
            {
                data = 0;
                data = (buf[CurrentByte] << 8);
                if (CurrentByte + 1 <= buf.GetUpperBound(0))
                {
                    data += buf[CurrentByte + 1];
                }
                data &= bitmask[CurrentBit];
                data = data >> rightShift[CurrentBit];

                s.Append(lookup[data]);
                test = s.ToString();


                CurrentBit += 5;
                if (CurrentBit >= 8)
                {
                    CurrentBit -= 8;
                    CurrentByte++;
                }

                charCount++;
                if (charCount % 4 == 0 && CurrentByte < buf.Length)
                {
                    s.Append("-");
                }
            }
            return s.ToString();
        }

        public static string BytesToHexReadable(byte[] buf, int spacing)
        {
            if (spacing == 0) return BytesToHex(buf);
            if (buf == null) return "";
            StringBuilder s = new StringBuilder(buf.Length * 3);
            int i = 0;
            int offset = 0;
            string tempAscii = "";

            s.Append("[").Append(offset.ToString("X08")).Append("] ");
            foreach (byte b in buf)
            {
                s.Append(b.ToString("X02")).Append(" ");
                tempAscii += char.IsControl((char)b) ? '.' : (char)b;

                i++;
                if ((i % spacing) == 0)
                {
                    s.Append("  [").Append(tempAscii).Append("]");
                    s.Append("\r\n");
                    offset += spacing;
                    tempAscii = "";
                    if (i != buf.Length)
                    {
                        s.Append("[").Append(offset.ToString("X08")).Append("] ");
                    }
                }
            }

            if ((i % spacing) != 0)
            {
                // Align last item.
                s.Append(' ', (spacing - (i % spacing)) * 3);
                s.Append("  [").Append(tempAscii).Append("]");
                s.Append("\r\n");
                offset += spacing;
            }

            return s.ToString();
        }

        public static string GetKiloLongString(ulong m)
        {
            if (m < 2048) return string.Format(Properties.Resources.NumBytes, m);
            m = m / 1024;
            if (m < 2048) return string.Format(Properties.Resources.NumKBytes, m);
            m = m / 1024;
            if (m < 2048) return string.Format(Properties.Resources.NumMBytes, m);
            m = m / 1024;
            if (m < 2048) return string.Format(Properties.Resources.NumGBytes, m);
            m = m / 1024;
            return string.Format(Properties.Resources.NumTBytes, m);
        }

        public static string GetKiloShortString(ulong m)
        {
            if (m < 2048) return string.Format(Properties.Resources.NumB, m);
            m = m / 1024;
            if (m < 2048) return string.Format(Properties.Resources.NumK, m);
            m = m / 1024;
            if (m < 2048) return string.Format(Properties.Resources.NumM, m);
            m = m / 1024;
            if (m < 2048) return string.Format(Properties.Resources.NumG, m);
            m = m / 1024;
            return string.Format(Properties.Resources.NumT, m);
        }

        public static TreeNode[] GetAllNodes(TreeNode rootnode)
        {
            ArrayList l = new ArrayList();
            l.Add(rootnode);
            if (rootnode.Nodes.Count > 0) l.AddRange(GetAllNodesEx(rootnode));
            return (TreeNode[])l.ToArray(typeof(TreeNode));
        }

        private static TreeNode[] GetAllNodesEx(TreeNode node)
        {
            ArrayList l = new ArrayList();
            foreach (TreeNode n in node.Nodes)
            {
                l.Add(n);
                if (n.Nodes.Count > 0) l.AddRange(GetAllNodesEx(n));
            }
            return (TreeNode[])l.ToArray(typeof(TreeNode));
        }

        public static ArrayList GetNodeTagsofTypeRec(TreeNode node, Type tagType)
        {
            ArrayList l = new ArrayList();
            GetNodeTagsofTypeRecEx(ref l, node, tagType);
            return l;
        }

        private static void GetNodeTagsofTypeRecEx(ref ArrayList list, TreeNode node, Type tagType)
        {
            foreach (TreeNode n in node.Nodes)
            {
                if (n.Tag != null && n.Tag.GetType() == tagType)
                {
                    list.Add(n.Tag);
                }
                else
                {
                    GetNodeTagsofTypeRecEx(ref list, n, tagType);
                }
            }
        }

        public static string FilterStrEx(string s)
        {
            if (s == null) return null;
            if (s.IndexOf('\0') >= 0) s = s.Substring(0, s.IndexOf('\0'));
            s = s.Replace("(R)", "" + (char)0x00AE).Replace("(C)", "" + (char)0x00A9).Trim();
            return s;
        }

        public static string FilterStr(string s)
        {
            if (s == null) return null;
            if (s.IndexOf('\0') >= 0) s = s.Substring(0, s.IndexOf('\0'));
            s = s.Replace("(R)", "" + (char)0x00AE).Replace("(C)", "" + (char)0x00A9).Trim();
            if (s.Length == 0) s = Properties.Resources.NotSpecified;
            return s;
        }

        public static void FilterAmtName(TextBox textbox)
        {
            int ss = textbox.SelectionStart;
            int sl = textbox.SelectionLength;
            textbox.Text = Utils.FilterAmtName(textbox.Text);
            textbox.SelectionStart = ss;
            textbox.SelectionLength = sl;
        }

        public static string FilterAmtName(string name)
        {
            name = name.Replace(" ", "").Replace(":", "").Replace(",", "").Replace("<", "").Replace(">", "").Replace("&", "").Replace("\"", "").Trim(); ;
            string o = "";
            foreach (char c in name)
            {
                if (c > 32 && c < 127) o += c;
            }
            if (o.Length > 16) o = o.Substring(0, 16);
            return o;
        }

        public static string FilterWirelessProfileName(string name)
        {
            name = name.Replace("\\", "").Replace("/", "").Replace(":", "").Replace("*", "").Replace("?", "").Replace("<", "").Replace(">", "").Replace("|", "").Replace("\"", "");
            if (name.Length > 32) name = name.Substring(0, 32);
            return name;
        }

        public static string FilterNumber(string number)
        {
            string o = "";
            foreach (char c in number)
            {
                if (c >= '0' && c <= '9') o += c;
            }
            if (o.Length > 5) o = o.Substring(0, 5);
            return o;
        }

        public static string FilterIpAddress(string number)
        {
            string o = "";
            foreach (char c in number)
            {
                if ((c >= '0' && c <= '9') || c == '.' || c == ':') o += c;
            }
            if (o.Length > 30) o = o.Substring(0, 30);
            return o;
        }

        public static void FilterIpAddress(TextBox textbox)
        {
            int ss = textbox.SelectionStart;
            int sl = textbox.SelectionLength;
            if (ss < 0 || sl < 0) return; // Do nothing
            textbox.Text = Utils.FilterIpAddress(textbox.Text);
            textbox.SelectionStart = ss;
            textbox.SelectionLength = sl;
        }

        public static TreeNode GetNodeWithTag(TreeNode root, object tag)
        {
            foreach (TreeNode n in root.Nodes) { if (n.Tag == tag) return n; }
            return null;
        }

        public static TreeNode GetNodeWithTagRec(TreeNode root, object tag)
        {
            if (root.Tag == tag) return root;
            foreach (TreeNode n in root.Nodes)
            {
                if (n.Tag == tag) return n;
                if (n.Nodes.Count > 0)
                {
                    TreeNode n2 = GetNodeWithTagRec(n, tag);
                    if (n2 != null) return n2;
                }
            }
            return null;
        }

        public static void CheckAllNodes(TreeNode root)
        {
            foreach (TreeNode n in root.Nodes) { n.Checked = true; }
        }

        public static void CheckAllNodesRec(TreeNode root)
        {
            foreach (TreeNode n in root.Nodes) { n.Checked = true; CheckAllNodesRec(n); }
        }

        public static void DeleteCheckedNodes(TreeNode root)
        {
            ArrayList l = new ArrayList();
            foreach (TreeNode n in root.Nodes) { if (n.Checked == true) l.Add(n); }
            foreach (TreeNode n in l) { root.Nodes.Remove(n); }
        }

        public static void DeleteCheckedNodesRec(TreeNode root)
        {
            ArrayList l = new ArrayList();
            foreach (TreeNode n in root.Nodes) { if (n.Checked == true) l.Add(n); }
            foreach (TreeNode n in l) { root.Nodes.Remove(n); }
            foreach (TreeNode n in root.Nodes) DeleteCheckedNodesRec(n);
        }

        public static string BytesToHex(byte[] buf)
        {
            if (buf == null) return "";
            StringBuilder s = new StringBuilder(buf.Length * 3);
            foreach (byte b in buf) s.Append(b.ToString("X02"));
            return s.ToString();
        }

        public static string BytesToHex(byte[] buf, int spacing)
        {
            if (spacing == 0) return BytesToHex(buf);
            if (buf == null) return "";
            StringBuilder s = new StringBuilder(buf.Length * 3);
            int i = 0;
            foreach (byte b in buf)
            {
                s.Append(b.ToString("X02"));
                i++;
                if ((i % spacing) == 0) s.Append(" ");
            }
            return s.ToString();
        }

        public static bool IsHex(string hex)
        {
            hex = hex.ToUpper();
            foreach (char c in hex) if (!((c >= '0' && c <= '9') || (c >= 'A' && c <= 'F'))) return false;
            return true;
        }

        // 0000-0000-0000-0000-0000-0000-0000-0000-0000-0000
        public static byte[] HexToBytes(string hex)
        {
            hex = hex.ToUpper();
            bool firstbyte = true;
            byte v = 0;
            ArrayList output = new ArrayList();
            foreach (char c in hex)
            {
                if (c >= '0' && c <= '9') v += (byte)(c - '0');
                else if (c >= 'A' && c <= 'F') v += (byte)((c - 'A') + 10);
                else continue;

                if (firstbyte)
                {
                    v = (byte)(v * 16);
                    firstbyte = false;
                }
                else
                {
                    output.Add(v);
                    v = 0;
                    firstbyte = true;
                }
            }
            return (byte[])output.ToArray(typeof(byte));
        }

        public static string BytesToHex(byte[] buf, int offset, int len)
        {
            StringBuilder s = new StringBuilder(len / 2);
            int x = 0;
            for (int i = offset; i < offset + len; i++)
            {
                if ((x != 0) && (x % 4 == 0)) s.Append(" ");
                s.Append(buf[i].ToString("X02"));
                x++;
            }
            return s.ToString();
        }

        public static int CountHexChars(string hex)
        {
            hex = hex.ToUpper();
            int cnt = 0;
            foreach (char c in hex)
            {
                if (c >= '0' && c <= '9') cnt++;
                else if (c >= 'A' && c <= 'F') cnt++;
            }
            return cnt;
        }

        public static void FilterHexStr(TextBox textbox)
        {
            int ss = textbox.SelectionStart;
            int sl = textbox.SelectionLength;
            textbox.Text = Utils.FilterHexStr(textbox.Text);
            textbox.SelectionStart = ss;
            textbox.SelectionLength = sl;
        }

        public static string FilterHexStr(string hex)
        {
            hex = hex.ToUpper();
            StringBuilder output = new StringBuilder();
            foreach (char c in hex)
            {
                if (c >= '0' && c <= '9') output.Append(c);
                else if (c >= 'A' && c <= 'F') output.Append(c);
                else if (c == '-' || c == ' ') output.Append(c);
            }
            return output.ToString();
        }

        public static uint GetIpAddr(IPAddress addr)
        {
            byte[] buf = addr.GetAddressBytes();
            return (uint)(buf[3] + (buf[2] << 8) + (buf[1] << 16) + (buf[0] << 24));
        }

        public static bool CheckPassPhrase(string passPhrase)
        {
            if (passPhrase.Length < 8 || passPhrase.Length > 63) return false;

            bool ascii = true;
            bool numeric = false;
            bool nonalpha = false;
            bool lowercase = false;
            bool uppercase = false;
            foreach (char c in passPhrase)
            {
                if (c < 32 || c > 126 || c == ':' || c == ',' || c == '\"') ascii = false;
                if (c >= '0' && c <= '9') numeric = true;
                if ((c > 32 && c < 48) || (c > 57 && c < 65) || (c > 90 && c < 95) || (c == 96) || (c > 122 && c < 127)) nonalpha = true;
                if (c >= 'a' && c <= 'z') lowercase = true;
                if (c >= 'A' && c <= 'Z') uppercase = true;
            }
            return ((ascii == true) && (numeric == true) && (nonalpha == true) && (lowercase == true) && (uppercase == true));
        }

        private static string GetLocalFqdnCache = null;
        public static string GetLocalFqdn()
        {
            if (GetLocalFqdnCache != null) return GetLocalFqdnCache;
            
            IPHostEntry entry = null;
            try
            {
                entry = System.Net.Dns.GetHostEntry("");
            }
            catch (Exception) { }
            if (entry != null && entry.HostName.IndexOf(".") > 0)
            {
                // This computer is probably part of a domain.
                GetLocalFqdnCache = entry.HostName;
                return GetLocalFqdnCache;
            }

            try
            {
                // Find a network adapter with a domain name
                NetworkInterface[] netinterfaces = NetworkInterface.GetAllNetworkInterfaces();
                if (netinterfaces.Length == 0) return null;
                foreach (NetworkInterface n in netinterfaces)
                {
                    string suffix = n.GetIPProperties().DnsSuffix;
                    if (suffix != null && suffix.Length > 0)
                    {
                        GetLocalFqdnCache = System.Net.Dns.GetHostName() + "." + suffix;
                        return GetLocalFqdnCache;
                    }
                }
            }
            catch (Exception) { }
            return "localhost";
        }

        private static string GetLocalConnectFqdnCache = null;
        public static string GetLocalConnectFQDN()
        {
            if (GetLocalConnectFqdnCache != null) return GetLocalConnectFqdnCache;
            string host = Utils.GetLocalFqdn();
            IPAddress[] addr = null;
            try
            {
                addr = System.Net.Dns.GetHostAddresses(host);
            }
            catch (Exception) { }
            if (addr == null) host = "localhost";
            GetLocalConnectFqdnCache = host;
            return GetLocalConnectFqdnCache;
        }

        public static byte[] Compress(byte[] buffer)
        {
            MemoryStream ms = new MemoryStream();
            GZipStream gzip = new GZipStream(ms, CompressionMode.Compress, true);
            gzip.Write(buffer, 0, buffer.Length);
            gzip.Close();
            ms.Position = 0;

            MemoryStream outStream = new MemoryStream();

            byte[] compressed = new byte[ms.Length];
            ms.Read(compressed, 0, compressed.Length);

            byte[] gzBuffer = new byte[compressed.Length + 4];
            Buffer.BlockCopy(compressed, 0, gzBuffer, 4, compressed.Length);
            Buffer.BlockCopy(BitConverter.GetBytes(buffer.Length), 0, gzBuffer, 0, 4);
            return gzBuffer;
        }

        public static byte[] Decompress(byte[] gzBuffer)
        {
            MemoryStream ms = new MemoryStream();
            int msgLength = BitConverter.ToInt32(gzBuffer, 0);
            if (msgLength > 5000000) return null; // Don't process anything larger than 5Meg
            ms.Write(gzBuffer, 4, gzBuffer.Length - 4);

            byte[] buffer = new byte[msgLength];

            ms.Position = 0;
            GZipStream gzip = new GZipStream(ms, CompressionMode.Decompress);
            gzip.Read(buffer, 0, buffer.Length);

            return buffer;
        }

        public static int FindInByteArray(byte[] Haystack, byte[] Needle)
        {
            int lFoundPosition = -1;
            int lMayHaveFoundIt = -1;
            int lMiniCounter = 0;

            for (int lCounter = 0; lCounter < Haystack.Length; lCounter++)
            {
                if (Haystack[lCounter] == Needle[lMiniCounter])
                {
                    if (lMiniCounter == 0) lMayHaveFoundIt = lCounter;
                    if (lMiniCounter == Needle.Length - 1) return lMayHaveFoundIt;
                    lMiniCounter++;
                }
                else
                {
                    lMayHaveFoundIt = -1;
                    lMiniCounter = 0;
                }
            }

            return lFoundPosition;
        }

        public static byte[] ReplaceInByteArray(byte[] original, int ptr, int len, byte[] replacement)
        {
            MemoryStream m = new MemoryStream();
            m.Write(original, 0, ptr);
            if (replacement != null) m.Write(replacement, 0, replacement.Length);
            m.Write(original, ptr + len, original.Length - (ptr + len));
            return m.ToArray();
        }

        public static byte[] ReplaceAllInByteArray(byte[] original, byte[] target, byte[] replacement)
        {
            int i = FindInByteArray(original, target);
            while (i >= 0)
            {
                original = ReplaceInByteArray(original, i, target.Length, replacement);
                i = FindInByteArray(original, target);
            }
            return original;
        }

        public static DateTime ClockToDate(uint time)
        {
            DateTime returnDt = new DateTime(1970, 1, 1).AddSeconds(time);
            return returnDt;
        }


        public static uint DateToClock(DateTime time)
        {
            TimeSpan ts = time.Subtract(new DateTime(1970, 1, 1));
            return (uint)ts.TotalSeconds;
        }


        public static string[] PowerStatesLong = new String[]
        {
           ManageabilityStack.Properties.Resources.Utils_PowerStateLong_0,
           ManageabilityStack.Properties.Resources.Utils_PowerStateLong_1,
           ManageabilityStack.Properties.Resources.Utils_PowerStateLong_2,
           ManageabilityStack.Properties.Resources.Utils_PowerStateLong_3,
           ManageabilityStack.Properties.Resources.Utils_PowerStateLong_4,
           ManageabilityStack.Properties.Resources.Utils_PowerStateLong_5,
           ManageabilityStack.Properties.Resources.Utils_PowerStateLong_6,
           ManageabilityStack.Properties.Resources.Utils_PowerStateLong_7,
           ManageabilityStack.Properties.Resources.Utils_PowerStateLong_8,
           ManageabilityStack.Properties.Resources.Utils_PowerStateLong_9,
           ManageabilityStack.Properties.Resources.Utils_PowerStateLong_10,
           ManageabilityStack.Properties.Resources.Utils_PowerStateLong_11,
           ManageabilityStack.Properties.Resources.Utils_PowerStateLong_12,
           ManageabilityStack.Properties.Resources.Utils_PowerStateLong_13,
           ManageabilityStack.Properties.Resources.Utils_PowerStateLong_14,
           ManageabilityStack.Properties.Resources.Utils_PowerStateLong_15,
           ManageabilityStack.Properties.Resources.Utils_PowerStateLong_16
 
            /*
            "Full power (S0)",
            "Sleeping with full context (S1)",
            "Sleeping without context (S2)",
            "Sleeping with memory context only (S3)",
            "Hibernating (S4)",
            "Soft off (S5)",
            "Hibernating / Soft Off (S4 or S5)",
            "Mechanical off",
            "Sleep state (S1,S2 or S3)",
            "Sleep state (S1,S2,S3 or S4)",
            "Softoff override",
            "Powered on (Legacy on)",
            "Powered off (Legacy off)",
            "Unknown Power State (Code 13)",
            "Unknown Power State (Code 14)",
            "Unknown Power State (Code 15)",
            "Unable to obtain power state",*/
        };

        public static string[] PowerStatesShort = new String[]
        {
            "S0",
            "S1",
            "S2",
            "S3",
            "S4",
            "S5",
            "S4/S5",
            "OFF",
            "S1/S3",
            "S1/S4",
            "OFF",
            "ON",
            "OFF",
            "#13",
            "#14",
            "#15",
            "N/A",
        };

        /// <summary>
        /// This class is used to add items to DropDown combo boxes and still be able to reference the item quickly.
        /// </summary>
        public class DropDownItem
        {
            public string Text = null;
            public object Tag = null;
            public int Handle = 0;
            public override string ToString() { return Text; }

            public DropDownItem()
            {
            }

            public DropDownItem(string Text, int Handle, object Tag)
            {
                this.Text = Text;
                this.Handle = Handle;
                this.Tag = Tag;
            }
        }
    }
}
