// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Net;
using System.Text;
using System.Net.Sockets;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Security.Cryptography.X509Certificates;
using ManageabilityControls;
using ManageabilityStack;

namespace ManageabilityTerminalTool
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main(string[] args)
        {
            if ((Type.GetType("Mono.Runtime") != null))
            {
                MessageBox.Show(Properties.Resources.NotCompatibleWithMono, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            // We will not be using any web proxy.
            WebRequest.DefaultWebProxy = null;

            string one_hostname = null;
            string one_username = null;
            string one_password = null;
            string one_title = null;
            string one_script = null;
            bool one_tls = false;
            bool Simplified = false;
            bool Silent = false;

            foreach (string arg in args)
            {
                if (arg.Length > 3 && string.Compare(arg.Substring(0, 3), "-l:", true) == 0) { try { System.Threading.Thread.CurrentThread.CurrentUICulture = new System.Globalization.CultureInfo(arg.Substring(3)); } catch (ArgumentException) { } }
                if (arg.Length > 3 && string.Compare(arg.Substring(0, 3), "-h:", true) == 0) one_hostname = arg.Substring(3);
                if (arg.Length > 3 && string.Compare(arg.Substring(0, 3), "-u:", true) == 0) one_username = arg.Substring(3);
                if (arg.Length > 3 && string.Compare(arg.Substring(0, 3), "-p:", true) == 0) one_password = arg.Substring(3);
                if (arg.Length > 3 && string.Compare(arg.Substring(0, 3), "-t:", true) == 0) one_title = arg.Substring(3);
                if (arg.Length > 3 && string.Compare(arg.Substring(0, 3), "-s:", true) == 0) one_script = arg.Substring(3);
                if (string.Compare(arg, "-simple", true) == 0) Simplified = true;
                if (string.Compare(arg, "-silent", true) == 0) Silent = true;
                if (string.Compare(arg, "-tls", true) == 0) one_tls = true;
                if (string.Compare(arg, "-debug", true) == 0) ManageabilityStack.AmtDebugForm.StartDebugLog("DebugLog-Terminal.txt");
                if (arg.Length > 7 && string.Compare(arg.Substring(0, 7), "-proxy:", true) == 0) WebRequest.DefaultWebProxy = new WebProxy(arg.Substring(7));
            }

            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);

            try
            {
                Application.UserAppDataRegistry.GetValue("SystemsList", "");
            }
            catch (System.Security.SecurityException)
            {
                MessageBox.Show(
                    Properties.Resources.Program_MsgBox_0,
                    Properties.Resources.OpenAmtTerminalTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            if (one_hostname == null || one_username == null || one_password == null)
            {
                one_hostname = null;
                one_username = null;
                one_password = null;
                one_title = null;
                one_tls = false;
            }

            Application.ThreadException += new System.Threading.ThreadExceptionEventHandler(ExceptionSink);
            Application.SetUnhandledExceptionMode(UnhandledExceptionMode.CatchException, true);
            AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(UnhandledExceptionEventSink);
            ServicePointManager.ServerCertificateValidationCallback = new System.Net.Security.RemoteCertificateValidationCallback(RemoteCertificateValidationCallbackSink);

            // Call the terminal in Intel AMT mode.
            if (one_hostname != null)
            {
                // In case TLS is used, generate/update the trusted roots file.
                if (ManageabilityStack.CertificateOperations.AllTrustedRootsToPemFile("Trusted Root Certificates.pem") == false)
                {
                    return;
                }

                ushort port = 16992;
                if (one_tls) port = 16993;
                TerminalForm term = new TerminalForm(one_hostname, port, one_username, one_password, one_title);
                term.EnableOutpostFeatures = false;
                term.Simplified = Simplified;
                term.AutoStartScript = one_script;
                if (one_script != null) term.Silent = Silent;
                Application.Run(term);
                return;
            }

            // Call the terminal in TCP mode
            ushort v;
            if ((args.Length == 2 || args.Length == 3) && ushort.TryParse(args[1], out v))
            {
                IPAddress[] ips;
                try
                {
                    if (string.Compare(args[0], "localhost", true) == 0)
                    {
                        ips = new IPAddress[] { IPAddress.Loopback };
                    }
                    else
                    {
                        ips = Dns.GetHostAddresses(args[0]);
                    }
                }
                catch (Exception)
                {
                    // DNS lookup failed.
                    MessageBox.Show(Properties.Resources.DnsLookupFailedExitingTerminal, Properties.Resources.OpenAmtTerminalTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                string title = ips[0].ToString();
                if (args.Length == 3) title = args[2];
                TcpClient tcpClient = new TcpClient();
                try
                {
                    tcpClient.Connect("127.0.0.1", v);
                }
                catch (Exception)
                {
                    // Connection Failed.
                    MessageBox.Show(Properties.Resources.ConnectionFailedExitingTerminal, Properties.Resources.OpenAmtTerminalTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                NetworkStream netStream = tcpClient.GetStream();
                TerminalForm term = new TerminalForm(netStream, title);
                term.EnableOutpostFeatures = false;
                if (one_script != null) term.Silent = Silent;

                try
                {
                    Application.Run(term);
                }
                catch (Exception ex)
                {
                    ExceptionSink(null, new System.Threading.ThreadExceptionEventArgs(ex));
                }
            }
            else
            {
                try
                {
                    Application.Run(new MainForm(args));
                }
                catch (Exception ex)
                {
                    ExceptionSink(null, new System.Threading.ThreadExceptionEventArgs(ex));
                }
            }
        }

        public static void ExceptionSink(object sender, System.Threading.ThreadExceptionEventArgs args)
        {
            ManageabilityStack.AmtDebugForm.GenerateExceptionFile("Exceptions-IAmtTerm.txt", args.Exception.ToString(), ManageabilityControls.AboutForm.VersionString);
        }

        public static void UnhandledExceptionEventSink(object sender, UnhandledExceptionEventArgs args)
        {
            ManageabilityStack.AmtDebugForm.GenerateExceptionFile("Exceptions-IAmtTerm.txt", ((Exception)args.ExceptionObject).ToString(), ManageabilityControls.AboutForm.VersionString);
        }

        private static bool RemoteCertificateValidationCallbackSink(object sender, X509Certificate certificate, X509Chain chain, System.Net.Security.SslPolicyErrors sslPolicyErrors)
        {
            if (sender.GetType() == typeof(System.Net.HttpWebRequest))
            {
                System.Net.HttpWebRequest r = (System.Net.HttpWebRequest)sender;
                X509Certificate2[] certArray = new X509Certificate2[chain.ChainElements.Count];
                for (int i = 0; i < chain.ChainElements.Count; i++)
                {
                    certArray[i] = chain.ChainElements[i].Certificate;
                }
                if (r.ConnectionGroupName != null) AmtSystem.AddTlsCertificateCache(r.ConnectionGroupName, certArray);
            }
            return true; // Accept All Certificates
        }

    }
}
