// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
//
// StorageWrapper.h

#pragma once

#include "iamt_api.h"

#if defined STORAGEWRAPPER_EXPORTS
#define STORAGE_API __declspec(dllexport)
#else
#define STORAGE_API __declspec(dllimport)
#endif

#if defined __cplusplus
extern "C" {
#endif // __cplusplus

STORAGE_API PT_STATUS __stdcall Storage_Initialize(
    INOUT UINT32 *LibMajorVersion, 
    INOUT UINT32 *LibMinorVersion, 
    OUT UINT32 *LibBuildNumber) ; 

STORAGE_API PT_STATUS __stdcall Storage_Uninitialize() ; 

STORAGE_API PT_STATUS __stdcall Storage_InitializeCOMinThread() ; 

STORAGE_API PT_STATUS __stdcall Storage_UninitializeCOMinThread() ; 

STORAGE_API PT_STATUS __stdcall Storage_RegisterApplicationEx(
    OUT SESSION_HANDLE             *SessionHandle,
    IN  wchar_t                    *Username    OPTIONAL,
    IN  wchar_t                    *Password    OPTIONAL,
    IN  CHAR                       *TargetUrl   ,
    IN  PT_UUID                    MachineUUID  OPTIONAL,
    IN  PTSDK_UNICODE_STRING       *VendorName,
    IN  PTSDK_UNICODE_STRING       *AppName,
    IN  PTSDK_UNICODE_STRING       *EnterpriseName,
    IN  SESSION_AUTHENTICATION_INFO          *AuthInfo   OPTIONAL
    ) ; 

STORAGE_API PT_STATUS __stdcall Storage_GetRegisteredApplications(
    IN    SESSION_HANDLE           SessionHandle,
    IN    UINT32                   StartIndex,
    OUT   UINT32                   *TotalRegisteredAppCount,
    INOUT UINT32                   *AppHandleCount,
    OUT   ISVS_APPLICATION_HANDLE  AppHandles[]        // [<AppHandleCount>]
) ; 


STORAGE_API PT_STATUS __stdcall Storage_GetCurrentApplicationHandle (
    IN  SESSION_HANDLE             SessionHandle,
    OUT	ISVS_APPLICATION_HANDLE    *AppHandle
    ) ; 

STORAGE_API PT_STATUS __stdcall Storage_GetApplicationAttributes(
    IN    SESSION_HANDLE                    SessionHandle,
    IN    ISVS_APPLICATION_HANDLE           ApplicationBeingRequested,
    INOUT PTSDK_APPLICATION_ATTRIBUTES      *ApplicationAttributes
    ) ;

STORAGE_API PT_STATUS __stdcall Storage_UnregisterApplication(
    IN SESSION_HANDLE       SessionHandle
    ) ;

STORAGE_API PT_STATUS __stdcall Storage_RemoveApplication(
    SESSION_HANDLE          SessionHandle
    ) ;

STORAGE_API PT_STATUS __stdcall Storage_GetBytesAvailable(
    IN  SESSION_HANDLE      SessionHandle,
    OUT UINT32              *BytesAvailable
    ) ;

STORAGE_API PT_STATUS __stdcall Storage_AllocateBlock(
    IN  SESSION_HANDLE          SessionHandle,
    IN  UINT32                  BytesRequested,
    IN  PT_BOOLEAN              BlockHidden,
    IN  PTSDK_UNICODE_STRING    *BlockName,
    OUT ISVS_BLOCK_HANDLE       *BlockHandle
    ) ;

STORAGE_API PT_STATUS __stdcall Storage_GetAllocatedBlocks(
    IN    SESSION_HANDLE            SessionHandle,
    IN    ISVS_APPLICATION_HANDLE   BlockOwnerApplication,
    IN    UINT32                    StartIndex,
    OUT   UINT32                    *TotalAllocatedBlockCount,
    INOUT UINT32                    *BlockHandleCount,
    OUT   ISVS_BLOCK_HANDLE         BlockHandles[]      // [<BlockHandleCount>]
) ;

STORAGE_API PT_STATUS __stdcall Storage_GetBlockAttributes(
    IN    SESSION_HANDLE            SessionHandle,
    IN    ISVS_BLOCK_HANDLE         BlockHandle,
    INOUT PTSDK_BLOCK_ATTRIBUTES    **BlockAttributes
    ) ;

STORAGE_API PT_STATUS __stdcall Storage_DeallocateBlock(
    IN  SESSION_HANDLE          SessionHandle,
    IN  ISVS_BLOCK_HANDLE       BlockHandle
    ) ;

STORAGE_API PT_STATUS __stdcall Storage_WriteBlock(
    IN  SESSION_HANDLE          SessionHandle,
    IN  ISVS_BLOCK_HANDLE       BlockHandle,
    IN  UINT32                  ByteOffset,
    IN  UINT32                  ByteCount,
    OUT UINT32                  *BytesWritten,
    IN  UINT8                   Data[]
) ;

STORAGE_API PT_STATUS __stdcall Storage_ReadBlock(
                                        IN    SESSION_HANDLE        SessionHandle,
                                        IN    ISVS_BLOCK_HANDLE     BlockHandle,
                                        IN    UINT32                ByteOffset,
                                        IN    UINT32                ByteCount,
                                        OUT   UINT32                *BytesRead,
                                        OUT   UINT8                 Data[]
) ;

STORAGE_API PT_STATUS __stdcall Storage_LockBlock(
                                        IN SESSION_HANDLE           SessionHandle,
                                        IN ISVS_BLOCK_HANDLE        BlockHandle
                                        ) ;

STORAGE_API PT_STATUS __stdcall Storage_UnlockBlock(
    IN SESSION_HANDLE           SessionHandle,
    IN ISVS_BLOCK_HANDLE        BlockHandle
    ) ;

STORAGE_API PT_STATUS __stdcall Storage_SetBlockName(
    IN SESSION_HANDLE           SessionHandle,
    IN ISVS_BLOCK_HANDLE        BlockHandle,
    IN PTSDK_UNICODE_STRING     *BlockName
    ) ;

STORAGE_API PT_STATUS __stdcall Storage_SetBlockVisibility(
    IN SESSION_HANDLE           SessionHandle,
    IN ISVS_BLOCK_HANDLE        BlockHandle,
    IN PT_BOOLEAN               BlockHidden
    ) ;

STORAGE_API PT_STATUS __stdcall Storage_AddPermissionsGroup(
    IN  SESSION_HANDLE          SessionHandle,
    IN  ISVS_BLOCK_HANDLE       BlockHandle,
    IN  ISVS_GROUP_PERMISSIONS  GroupPermissions,
    IN  PTSDK_UNICODE_STRING    *GroupName,
    OUT ISVS_GROUP_HANDLE       *GroupHandle
    ) ;

STORAGE_API PT_STATUS __stdcall Storage_GetPermissionsGroups(
    IN    SESSION_HANDLE        SessionHandle,
    IN    ISVS_BLOCK_HANDLE     BlockHandle,
    IN    UINT32                StartIndex,
    OUT   UINT32                *TotalGroupCount,
    INOUT UINT32                *GroupHandleCount,
    OUT   ISVS_GROUP_HANDLE     **GroupHandles  // [<HandleCount>]
    ) ;

STORAGE_API PT_STATUS __stdcall Storage_GetPermissionsGroupAttributes(
    IN    SESSION_HANDLE                        SessionHandle,
    IN    ISVS_BLOCK_HANDLE                     BlockHandle,
    IN    ISVS_GROUP_HANDLE                     GroupHandle,
    INOUT PTSDK_PERMISSIONS_GROUP_ATTRIBUTES    **GroupAttributes
    ) ;

STORAGE_API PT_STATUS __stdcall Storage_RemovePermissionsGroup(
    IN SESSION_HANDLE          SessionHandle,
    IN ISVS_BLOCK_HANDLE       BlockHandle,
    IN ISVS_GROUP_HANDLE       GroupHandle
    ) ;

STORAGE_API PT_STATUS __stdcall Storage_SetPermissionsGroupName(
    IN SESSION_HANDLE           SessionHandle,
    IN ISVS_BLOCK_HANDLE        BlockHandle,
    IN ISVS_GROUP_HANDLE        GroupHandle,
    IN PTSDK_UNICODE_STRING     *GroupName
    ) ;

STORAGE_API PT_STATUS __stdcall Storage_SetPermissionsGroupPermissions(
    IN SESSION_HANDLE           SessionHandle,
    IN ISVS_BLOCK_HANDLE        BlockHandle,
    IN ISVS_GROUP_HANDLE        GroupHandle,
    IN ISVS_GROUP_PERMISSIONS   Permissions
    ) ;

STORAGE_API PT_STATUS __stdcall Storage_AddPermissionsGroupMembers(
    IN SESSION_HANDLE           SessionHandle,
    IN ISVS_BLOCK_HANDLE        BlockHandle,
    IN ISVS_GROUP_HANDLE        GroupHandle,
    IN UINT32                   MemberHandleCount,
    IN ISVS_APPLICATION_HANDLE  MemberHandles[]      // [<MemberHandleCount>]
) ;

STORAGE_API PT_STATUS __stdcall Storage_GetPermissionsGroupMembers(
    IN    SESSION_HANDLE            SessionHandle,
    IN    ISVS_BLOCK_HANDLE         BlockHandle,
    IN    ISVS_GROUP_HANDLE         GroupHandle,
    IN    UINT32                    StartIndex,
    OUT   UINT32                    *TotalMemberCount,
    INOUT UINT32                    *MemberHandleCount,
    OUT   ISVS_APPLICATION_HANDLE   MemberHandles[]      // [<MemberHandleCount>]
) ;

STORAGE_API PT_STATUS __stdcall Storage_RemovePermissionsGroupMembers(
    IN SESSION_HANDLE           SessionHandle,
    IN ISVS_BLOCK_HANDLE        BlockHandle,
    IN ISVS_GROUP_HANDLE        GroupHandle,
    IN UINT32                   MemberHandleCount,
    IN ISVS_APPLICATION_HANDLE  MemberHandles[]      // [<MemberHandleCount>]
) ;

STORAGE_API PT_STATUS __stdcall Storage_GetTimeoutValues(
    IN  SESSION_HANDLE          SessionHandle,
    OUT UINT32                  *RegistrationTimeout,
    OUT UINT32                  *LockTimeout
    ) ;

STORAGE_API PT_STATUS __stdcall Storage_GetHostUUID(
    OUT PT_UUID         Uuid
    ) ;

STORAGE_API PT_STATUS __stdcall Storage_GetAPIVersion(
    IN  wchar_t             *Username   OPTIONAL,
    IN  wchar_t             *Password   OPTIONAL,
    IN  CHAR                *TargetUrl,
    OUT ISVS_VERSION        *Version
    ) ;

STORAGE_API PT_STATUS __stdcall Storage_GetAPIVersionEx(
    OUT ISVS_VERSION    *Version,
    IN  wchar_t             *Username   OPTIONAL,
    IN  wchar_t             *Password   OPTIONAL,
    IN  CHAR                *TargetUrl,
    IN  SESSION_AUTHENTICATION_INFO   *AuthInfo   OPTIONAL
    ) ;

STORAGE_API PT_STATUS __stdcall Storage_GetBlockWriteEraseLimit(
    IN SESSION_HANDLE       SessionHandle,
    IN ISVS_BLOCK_HANDLE    BlockHandle,
    OUT UINT32              *WriteEraseLimit
    ) ;

STORAGE_API VOID __stdcall Storage_GetLastNetworkError(
    IN  SESSION_HANDLE  SessionHandle  OPTIONAL,
    OUT VOID            *NetworkError
    ) ;

#if defined __cplusplus
} ;
// End of extern "C"
#endif // __cplusplus


// [EoF]

